@@ -44,7 +44,7 @@
 #include <stdio.h>
 #include <errno.h>
 
-#if !defined(__GLIBC__) && !defined(__FreeBSD__)
+#ifndef __GLIBC__
 /* BSD 4.3 errno.h does not declare errno */
 extern int errno;
 extern int sys_nerr;
