@@ -48,9 +48,21 @@ find_next_and_look()
     return nextunit;
 }
 
+static UnitVector *selvec;
+
+UnitVector *
+get_selected_units(Side *side)
+{
+    if (selvec == NULL)
+      selvec = make_unit_vector(2);
+    clear_unit_vector(selvec);
+    if (in_play(curunit))
+      selvec = add_unit_to_vector(selvec, curunit, 0);
+    return selvec;
+}
+
 void
-do_add_terrain(side)
-Side *side;
+do_add_terrain(Side *side)
 {
     int u, t, dir;
 
@@ -96,19 +108,6 @@ do_attack(Side *side)
     }
 }
 
-/* Set the unit to automatic control.  */
-
-void
-do_auto(Side *side)
-{
-    REQUIRE_UNIT();
-    if (curunit->plan) {
-	net_set_unit_ai_control(dside, curunit, !curunit->plan->aicontrol, FALSE);
-	/* a hack */
-	curunit->plan->waitingfortasks = !curunit->plan->aicontrol;
-    }
-}
-
 void
 do_build(Side *side)
 {
@@ -168,100 +167,45 @@ impl_build(Side *side, Unit *unit, int u2, int n)
     net_push_build_task(unit, u2, n, 0, 0);
 }
 
-void
-do_clear_plan(Side *side)
-{
-    REQUIRE_UNIT();
-    if (curunit->plan) {
-	net_set_unit_plan_type(dside, curunit, PLAN_NONE);
-    }
-}
-
 /* (should reindent) */
 void
-do_collect(side)
-Side *side;
+do_collect(Side *side)
 {
-  int mtocollect;
-  char *arg, *rest;
+    int mtocollect;
+    char *arg, *rest;
 
-  REQUIRE_UNIT();
-  mtocollect = NONMTYPE;
-  if (nummtypes == 0) {
-    cmd_error(side, "No materials to collect");
-    return;
-  }
-  if (!empty_string(cmdargstr)) {
-    rest = get_next_arg(cmdargstr, tmpbuf, &arg);
-    mtocollect = mtype_from_name(arg);
-  } else {
-    cmd_error(side, "No material name given");
-    return;
-  }
-  if (!is_material_type(mtocollect)) {
-    cmd_error(side, "`%s' is not a recognized material name", arg);
-    return;
-  }
-  if (curunit->plan)
-    net_set_collect_task(curunit, mtocollect, curunit->x, curunit->y);
+    REQUIRE_UNIT();
+    mtocollect = NONMTYPE;
+    if (nummtypes == 0) {
+	cmd_error(side, "No materials to collect");
+	return;
+    }
+    if (!empty_string(cmdargstr)) {
+	rest = get_next_arg(cmdargstr, tmpbuf, &arg);
+	mtocollect = mtype_from_name(arg);
+    } else {
+	cmd_error(side, "No material name given");
+	return;
+    }
+    if (!is_material_type(mtocollect)) {
+	cmd_error(side, "`%s' is not a recognized material name", arg);
+	return;
+    }
+    if (curunit->plan)
+      net_set_collect_task(curunit, mtocollect, curunit->x, curunit->y);
 }
 
 void
-do_copying(side)
-Side *side;
+do_copying(Side *side)
 {
     cur_help_node = copying_help_node;
     do_help(side);
 }
 
-void
-do_delay(side)
-Side *side;
-{
-    Unit *oldcurunit = curunit, *nextmover;
-
-    nextmover = find_next_and_look();
-
-    if (nextmover == NULL) {
-	cmd_error(side, "no units???");
-    } else if (curunit == oldcurunit) {
-	notify(dside, "No other units to move!");
-    }
-}
-
-void
-do_detach(side)
-Side *side;
-{
-    Unit *unit = curunit;
-
-    REQUIRE_UNIT();
-    if (!completed(unit)) {
-	cmd_error(side, "cannot detach from an incomplete unit");
-	return;
-    }
-    if (valid(check_transfer_part_action(unit, unit, unit->hp / 2, NULL))) {
-	net_prep_transfer_part_action(unit, unit, unit->hp / 2, NULL);
-    } else {
-	cmd_error(side, "cannot detach right now");
-    }
-}
-
-void
-do_detonate(side)
-Side *side;
-{
-    Unit *unit = curunit;
-
-    REQUIRE_UNIT();
-    net_prep_detonate_action(unit, unit, unit->x, unit->y, unit->z);
-}
-
 /* Supposedly you could only get to these by typing the full command names. */
 
 void
-do_dir(side)
-Side *side;
+do_dir(Side *side)
 {
     int ndirs, dir1, dir2, modif;
 
@@ -275,66 +219,15 @@ Side *side;
 }
 
 void
-do_dir_multiple(side)
-Side *side;
+do_dir_multiple(Side *side)
 {
     cmd_error(side, "use the single-character commands instead");
 }
 
-/* Get rid of a unit. */
-
-void
-do_disband(side)
-Side *side;
-{
-    int rslt;
-
-    REQUIRE_UNIT();
-    rslt = net_disband_unit(dside, curunit);
-    if (!rslt) {
-	cmd_error(side, "You can't get rid of %s!",
-		  unit_handle(dside, curunit));
-    }
-}
-
-void
-do_disembark(side)
-Side *side;
-{
-    Unit *transport;
-
-    REQUIRE_UNIT();
-    transport = curunit->transport;
-    if (transport == NULL) {
-	cmd_error(side, "Not in a transport");
-	return;
-    }
-    if (!in_play(transport)) {
-	cmd_error(side, "Transport is nonsensical?");
-	return;
-    }
-    /* Try moving into the transport's transport, if there is one. */
-    if (transport->transport != NULL
-        && can_occupy(curunit, transport->transport)) {
-	net_prep_enter_action(curunit, curunit, transport->transport);
-	/* (should be able to set up task if can't do action immedly) */
-	return;
-    }
-    /* Try moving into the open in the cell. */
-    if (can_occupy_cell(curunit, curunit->x, curunit->y)
-	|| can_occupy_conn(curunit, curunit->x, curunit->y, curunit->z)) {
-	net_prep_move_action(curunit, curunit, curunit->x, curunit->y, curunit->z);
-	/* (should be able to set up task if can't do action immedly) */
-	return;
-    }
-    cmd_error(side, "Can't disembark here!");
-}
-
 /* Determine how far away another point is.  */
 
 void
-do_distance(side)
-Side *side;
+do_distance(Side *side)
 {
     int x, y;
 
@@ -343,35 +236,8 @@ Side *side;
     }
 }
 
-/* What about trying to embark a unit on itself or on its previous transp? */
-
 void
-do_embark(side)
-Side *side;
-{
-    Unit *transport;
-
-    REQUIRE_UNIT();
-    /* (should only ask if multiple transport possibilities) */
-    if (ask_unit("Which unit to board?", &transport)) {
-	if (!in_play(transport)) {
-	    cmd_error(side, "This transport is nonsensical?");
-	    return;
-	}
-	if (!can_occupy(curunit, transport)) {
-	    cmd_error(side, "Can't occupy chosen transport");
-	    return;
-	}
-	net_prep_enter_action(curunit, curunit, transport);
-	/* (should be able to set up task if can't do action immedly) */
-    } else {
-	xbeep();
-    }
-}
-
-void
-do_escape(side)
-Side *side;
+do_escape(Side *side)
 {
     cmd_error(side, "No curses version of this command.");
 }
@@ -379,36 +245,33 @@ Side *side;
 /* Command to fire at a specified unit or location. */
 
 void
-do_fire(side)
-Side *side;
+do_fire(Side *side)
 {
     int x, y;
     Unit *unit2;
+    UnitView *uview;
 
     REQUIRE_UNIT();
     sprintf(spbuf, "Fire %s at where?", unit_handle(dside, curunit));
     /* (should have some sort of range feedback) */
     if (ask_position(spbuf, &x, &y)) {
-	if (units_visible(dside, x, y)) {
-	    for_all_stack(x, y, unit2) {
-		if (unit2->side != curunit->side) {
-		    net_prep_fire_at_action(curunit, curunit, unit2, -1);
-		    return;
-		}
+	for_all_view_stack(dside, x, y, uview) {
+	    unit2 = view_unit(uview);
+	    /* (should avoid shooting at allies too) */
+	    if (unit2 != NULL
+		&& unit2->side != curunit->side
+		&& uview->date == g_turn()) {
+		net_prep_fire_at_action(curunit, curunit, unit2, -1);
+		return;
 	    }
 	}
-	/* (should say that nothing is visible and verify firing) */
-	if (1) {
-	    net_prep_fire_into_action(curunit, curunit, x, y, 0, -1);
-	} else {
-	    xbeep();
-	}
+	notify(dside, "No target unit seen, firing blindly");
+	net_prep_fire_into_action(curunit, curunit, x, y, 0, -1);
     }
 }
 
 void
-do_fire_into(side)
-Side *side;
+do_fire_into(Side *side)
 {
     int x, y;
 
@@ -421,8 +284,7 @@ Side *side;
 }
 
 void
-do_flash(side)
-Side *side;
+do_flash(Side *side)
 {
     cmd_error(side, "No curses version of this command.");
 }
@@ -430,8 +292,7 @@ Side *side;
 /* Toggle the action following flag. */
 
 void
-do_follow_action(side)
-Side *side;
+do_follow_action(Side *side)
 {
     follow_action = !follow_action;
     if (follow_action) {
@@ -441,48 +302,10 @@ Side *side;
     }
 }
 
-/* Give supplies to a transport.  The argument tells how many to give. */
-
-void
-do_give(side)
-Side *side;
-{
-    int n = prefixarg, something = FALSE, u, m, r, gift, actual;
-    Unit *mainunit;
-
-    REQUIRE_UNIT();
-    u = curunit->type;
-    mainunit = curunit->transport;
-    if (mainunit != NULL) {
-	spbuf[0] = '\0';
-	m = mainunit->type;
-	for_all_material_types(r) {
-	    gift = (n < 0 ? (um_storage_x(m, r) - mainunit->supply[r]) : n);
-	    if (gift > 0) {
-		something = TRUE;
-		/* Be stingy if we're low */
-		if (2 * curunit->supply[r] < um_storage_x(u, r))
-		    gift = max(1, gift/2);
-		actual = transfer_supply(curunit, mainunit, r, gift);
-		sprintf(tmpbuf, " %d %s", actual, m_type_name(r));
-		strcat(spbuf, tmpbuf);
-	    }
-	}
-	if (something) {
-	    notify(dside, "%s gave%s.", unit_handle(dside, curunit), spbuf);
-	} else {
-	    notify(dside, "%s gave nothing.", unit_handle(dside, curunit));
-	}
-    } else {
-	cmd_error(side, "Can't transfer supplies here!");
-    }
-}
-
 /* Give a unit to another side or "to" independence. */
 
 void
-do_give_unit(side)
-Side *side;
+do_give_unit(Side *side)
 {
     REQUIRE_UNIT();
 #ifdef DESIGNERS
@@ -501,8 +324,7 @@ Side *side;
 /* Bring up help info. */
 
 void
-do_help(side)
-Side *side;
+do_help(Side *side)
 {
     /* Switch to help mode, saving current mode first. */
     prevmode = mode;
@@ -511,23 +333,10 @@ Side *side;
     refresh();
 }
 
-/* Tell the unit to sit around for a given number of turns. */
-
-void
-do_idle(side)
-Side *side;
-{
-    REQUIRE_UNIT();
-    if (prefixarg < 0)
-      prefixarg = 9999;
-    net_set_sentry_task(curunit, prefixarg);
-}
-
 /* Set the display of various kinds of data. */
 
 void
-do_map(side)
-Side *side;
+do_map(Side *side)
 {
     int value;
     char *str, *str2, tmpbuf[BUFSIZE];
@@ -561,6 +370,8 @@ Side *side;
 		drawnames = value;
 	    } else if (strcmp(str2, "people") == 0 || strcmp(str2, "p") == 0) {
 		drawpeople = value;
+	    } else if (strcmp(str2, "cover") == 0 || strcmp(str2, "c") == 0) {
+		draw_cover = value;
 	    } else if (strcmp(str2, "one") == 0 || strcmp(str2, "1") == 0) {
 		use_both_chars = !value;
 	    } else if (strcmp(str2, "two") == 0 || strcmp(str2, "2") == 0) {
@@ -629,8 +440,7 @@ Side *side;
 }
 
 static void
-resize_map(n)
-int n;
+resize_map(int n)
 {
     /* Resize the left-hand-side windows. */
     mw += n;
@@ -650,8 +460,7 @@ int n;
 /* Send a short message to another side. */
 
 void
-do_message(side)
-Side *side;
+do_message(Side *side)
 {
     char *msg;
     Side *side2;
@@ -680,8 +489,7 @@ Side *side;
 /* Set unit to move to a given location.  Designers do a teleport. */
 
 void
-do_move_to(side)
-Side *side;
+do_move_to(Side *side)
 {
     int x, y;
 
@@ -702,8 +510,7 @@ Side *side;
 /* Command to name or rename the current unit or a given side. */
 
 void
-do_name(side)
-Side *side;
+do_name(Side *side)
 {
     char *newname;
 
@@ -716,15 +523,13 @@ Side *side;
 }
 
 void
-do_new_map(side)
-Side *side;
+do_new_map(Side *side)
 {
     cmd_error(side, "No curses version of this command.");
 }
 
 void
-do_occupant(side)
-Side *side;
+do_occupant(Side *side)
 {
     Unit *nextocc;
 
@@ -738,14 +543,13 @@ Side *side;
 }
 
 void
-do_orders_popup(side)
-Side *side;
+do_orders_popup(Side *side)
 {
+    cmd_error(side, "No curses version of this command.");
 }
 
 void
-do_other(side)
-Side *side;
+do_other(Side *side)
 {
     char *cmd;
 
@@ -762,15 +566,13 @@ Side *side;
 }
 
 void
-do_print_view(side)
-Side *side;
+do_print_view(Side *side)
 {
     dump_text_view(dside, use_both_chars);
 }
 
 void
-do_produce(side)
-Side *side;
+do_produce(Side *side)
 {
     int m, n;
     Unit *unit = curunit;
@@ -794,8 +596,7 @@ Side *side;
 /* Command to get out of a game, one way or another. */
 
 void
-do_quit(side)
-Side *side;
+do_quit(Side *side)
 {
     if (endofgame || !dside->ingame) {
 	exit_cconq();
@@ -834,8 +635,7 @@ Side *side;
    possible, and redraw everything. */
 
 void
-do_recenter(side)
-Side *side;
+do_recenter(Side *side)
 {
     set_view_focus(mvp, curx, cury);
     center_on_focus(mvp);
@@ -847,15 +647,13 @@ Side *side;
 /* Redraw everything using the same code as when windows need a redraw. */
 
 void
-do_refresh(side)
-Side *side;
+do_refresh(Side *side)
 {
     redraw();
 }
 
 void
-do_remove_terrain(side)
-Side *side;
+do_remove_terrain(Side *side)
 {
     int t, dir;
 
@@ -877,23 +675,13 @@ Side *side;
 }
 
 void
-do_repair(side)
-Side *side;
+do_repair(Side *side)
 {
-  cmd_error(side, "repair command not implemented");
+    cmd_error(side, "repair command not implemented");
 }
 
 void
-do_reserve(side)
-Side *side;
-{
-    REQUIRE_UNIT();
-    net_set_unit_asleep(dside, curunit, TRUE, TRUE);
-}
-
-void
-do_resign(side)
-Side *side;
+do_resign(Side *side)
 {
     Side *side2;
 
@@ -914,23 +702,12 @@ Side *side;
     }
 }
 
-/* Set unit to return to a good resupply place. */
-
-void
-do_return(side)
-Side *side;
-{
-    REQUIRE_UNIT();
-    net_set_resupply_task(curunit, NONMTYPE);
-}
-
 /* Stuff game state into a file.  By default, it goes into the current
    directory.  If building a scenario, we can specify just which parts
    of the game state are to be written. */
 
 void
-do_save(side)
-Side *side;
+do_save(Side *side)
 {
     char *rawcontents;
     Module *module;
@@ -979,8 +756,7 @@ Side *side;
 }
 
 void
-do_set_formation(side)
-Side *side;
+do_set_formation(Side *side)
 {
     Unit *leader;
 
@@ -1000,30 +776,19 @@ Side *side;
 }
 
 void
-do_set_view_angle(side)
-Side *side;
+do_set_view_angle(Side *side)
 {
     cmd_error(side, "No curses version of this command.");
 }
 
 void
-do_side_closeup(side)
-Side *side;
+do_side_closeup(Side *side)
 {
     cmd_error(side, "No curses version of this command.");
 }
 
 void
-do_sleep(side)
-Side *side;
-{
-    REQUIRE_UNIT();
-    net_set_unit_asleep(dside, curunit, TRUE, TRUE);
-}
-
-void
-do_standing_orders(side)
-Side *side;
+do_standing_orders(Side *side)
 {
     int rslt;
 
@@ -1038,8 +803,7 @@ Side *side;
 /* Command to toggle between interaction modes. */
 
 void
-do_survey(side)
-Side *side;
+do_survey(Side *side)
 {
     if (mode == MOVE) {
 	lastactor = curunit;
@@ -1056,41 +820,14 @@ Side *side;
     refresh();
 }
 
-/* Take supplies from transport. */
-
 void
-do_take(side)
-Side *side;
-{
-    int m, rslt;
-    short amts[MAXMTYPES];
-
-    REQUIRE_UNIT();
-    rslt = take_supplies(curunit, NULL, amts);
-    if (rslt) {
-	spbuf[0] = '\0';
-	for_all_material_types(m) {
-	    if (amts[m] > 0) {
-		sprintf(tmpbuf, " %d %s", amts[m], m_type_name(m));
-		strcat(spbuf, tmpbuf);
-	    }
-	}
-	notify(dside, "%s got%s.", unit_handle(dside, curunit), spbuf);
-    } else {
-	notify(dside, "%s got nothing.", unit_handle(dside, curunit));
-    }
-}
-
-void
-do_unit_closeup(side)
-Side *side;
+do_unit_closeup(Side *side)
 {
     cmd_error(side, "No curses version of this command.");
 }
 
 void
-do_up(side)
-Side *side;
+do_up(Side *side)
 {
     cmd_error(side, "No curses version of this command.");
 }
@@ -1098,59 +835,33 @@ Side *side;
 /* Display the program version. */
 
 void
-do_version(side)
-Side *side;
+do_version(Side *side)
 {
     notify(dside, "Curses Xconq version %s", version_string());
     notify(dside, "(c) %s", copyright_string());
 }
 
-/* Wake *everything* (that's ours) within the given radius.  Two commands
-   actually; "top-level" units (not in a transport) vs all units. */
-
-/* Wake top-level units. */
-
 void
-do_wake(side)
-Side *side;
-{
-    net_wake_area(dside, curx, cury, prefixarg, FALSE);
-}
-
-/* Wake all units found. */
-
-void
-do_wake_all(side)
-Side *side;
-{
-    net_wake_area(dside, curx, cury, prefixarg, TRUE);
-}
-
-void
-do_warranty(side)
-Side *side;
+do_warranty(Side *side)
 {
     cur_help_node = warranty_help_node;
     do_help(side);
 }
 
 void
-do_world_map(side)
-Side *side;
+do_world_map(Side *side)
 {
     cmd_error(side, "No curses version of this command.");
 }
 
 void
-do_zoom_in(side)
-Side *side;
+do_zoom_in(Side *side)
 {
     cmd_error(side, "No curses version of this command.");
 }
 
 void
-do_zoom_out(side)
-Side *side;
+do_zoom_out(Side *side)
 {
     cmd_error(side, "No curses version of this command.");
 }
@@ -1158,8 +869,7 @@ Side *side;
 #ifdef DESIGNERS
 
 void
-do_design(side)
-Side *side;
+do_design(Side *side)
 {
     if (!dside->designer) {
 	net_become_designer(dside);
@@ -1173,15 +883,13 @@ Side *side;
 #ifdef DEBUGGING
 
 void
-do_profile(side)
-Side *side;
+do_profile(Side *side)
 {
     cmd_error(side, "No curses version of this command.");
 }
 
 void
-do_trace(side)
-Side *side;
+do_trace(Side *side)
 {
     cmd_error(side, "No curses version of this command.");
 }
