@@ -1,5 +1,5 @@
 /* The main program of the curses interface to Xconq.
-   Copyright (C) 1986-1989, 1991-1997 Stanley T. Shebs.
+   Copyright (C) 1986-1989, 1991-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -17,15 +17,16 @@ any later version.  See the file COPYING.  */
 #endif /* THINK_C */
 
 #ifdef __MWERKS__
+#ifndef MAC
 #define MAC
-/* This is to get the command line reader in Metrowerks on the Mac. */
-#include <console.h>
+#endif
 #include <sioux.h>
 #endif /* __MWERKS__ */
 
-extern int hosting;
+extern int autofinish_start;
+extern int autofinish_count;
 
-static void describe_help PARAMS ((int arg, char *key, TextBuffer *buf));
+static void describe_help(int arg, char *key, TextBuffer *buf);
 
 #ifdef MAC /* temporary */
 char **current_cursor;
@@ -107,8 +108,6 @@ HelpNode *cur_help_node;
 
 HelpNode *help_help_node;
 
-HelpNode *commands_help_node;
-
 HelpNode *topics_help_node;
 
 struct ccwin *datewin;
@@ -177,6 +176,8 @@ int drawnames;
 
 int drawpeople;
 
+int draw_cover;
+
 enum listsides listsides;
 int test;
 int value;
@@ -187,12 +188,6 @@ int sortorder;
 
 int active;
 
-/* Countdown for when multiple turns autofinishing. */
-
-int autofinish_start;
-
-int autofinish_count;
-
 enum movie_type {
   movie_null,
   movie_miss,
@@ -218,9 +213,7 @@ struct a_movie *movies;
 /* The main program just invokes everything else. */
 
 int
-main(argc, argv)
-int argc;
-char *argv[];
+main(int argc, char *argv[])
 {
 #ifdef THINK_C
     /* This is how Think C picks up a command line. */
@@ -253,17 +246,14 @@ char *argv[];
 #endif /* DEBUGGING */
     if (option_game_to_join != NULL) {
 	int rslt;
- 	extern int hosting;
-	extern char *connection_method_name;
-	extern int connection_method;
 
 	/* (should parse variant specs, then comment that they must be ignored) */
 	/* (should accept player specs) */
 	/* Go through once more and complain about anything not used. */
 	parse_command_line(argc, argv, leftover_options);
 	hosting = FALSE;
-	open_remote_connection(option_game_to_join);
-	if (connection_method > 0) {
+	rslt = open_remote_connection(option_game_to_join, hosting);
+	if (rslt > 0) {
 	    rslt = send_join("xxx");
 	    if (rslt) {
 		master_rid = 1;
@@ -274,10 +264,7 @@ char *argv[];
 		    }
 		}
 	    } else {
-		close_remote_connection();
-		/* Use of connection_method this way is weak; means
-		   having to reselect everything up reconnection. */
-		connection_method = 0;
+		close_remote_connection(0);
 	    }
 	}
     } else {
@@ -294,7 +281,7 @@ char *argv[];
 	make_trial_assignments();
 	if (option_game_to_host != NULL) {
 	    hosting = TRUE;
-	    open_remote_connection(option_game_to_host);
+	    open_remote_connection(option_game_to_host, hosting);
 	}
 	calculate_globals();
 	run_synth_methods();
@@ -335,7 +322,7 @@ please somebody write this code
    straight. */
 
 Player *
-add_default_player()
+add_default_player(void)
 {
     Player *player = add_player();
 
@@ -344,11 +331,16 @@ add_default_player()
     return player;
 }
 
+void
+make_default_player_spec(void)
+{
+    default_player_spec = "term";
+}
+
 /* Set up the basic user interface for a side. */
 
 void
-init_ui(side)
-Side *side;
+init_ui(Side *side)
 {
     if (side_wants_display(side)) {
 	if (dside == NULL) {
@@ -371,7 +363,7 @@ Side *side;
    recognition later. */
 
 void
-init_display()
+init_display(void)
 {
     int x, y, i;
 
@@ -414,7 +406,7 @@ init_display()
 
     lw = COLS - mw - 1;
 
-    sh = numsides + 1;
+    sh = numtotsides + 1;
     /* The height of the list window is what's left over after making
        space for the side list and status lines. */
     lh = LINES - sh - 1 - 1;
@@ -453,10 +445,8 @@ init_display()
 }
 
 void
-init_interaction()
+init_interaction(void)
 {
-    HelpNode *long_commands_help_node;
-
     curunit = NULL;
     find_next_and_look();
     if (curunit) {
@@ -470,11 +460,11 @@ init_interaction()
     if (cur_help_node == NULL) {
 	help_help_node =
 	  add_help_node("help", describe_help, 0, first_help_node);
+	key_commands_help_node =
+	  add_help_node("commands", describe_key_commands, 0, first_help_node);
 	long_commands_help_node =
 	  add_help_node("long commands", describe_long_commands, 0,
-			first_help_node);
-	commands_help_node =
-	  add_help_node("commands", describe_commands, 0, first_help_node);
+			key_commands_help_node);
 	topics_help_node =
 	  add_help_node("topics", describe_topics, 0, first_help_node);
 	cur_help_node = topics_help_node;
@@ -486,8 +476,7 @@ init_interaction()
 /* Windows in curses are just simple bounding boxes. */
 
 struct ccwin *
-create_window(x, y, w, h)
-int x, y, w, h;
+create_window(int x, int y, int w, int h)
 {
     struct ccwin *newwin;
 
@@ -510,7 +499,7 @@ int x, y, w, h;
    of modal thing (perhaps aborts modal dialog automatically, then runs?) */
 
 int
-wait_for_char()
+wait_for_char(void)
 {
     show_cursor();
     /* Only take 7-bit chars. */
@@ -519,8 +508,7 @@ wait_for_char()
 }
 
 void
-maybe_handle_input(tmout)
-int tmout;
+maybe_handle_input(int tmout)
 {
     Unit *unit;
 
@@ -528,8 +516,7 @@ int tmout;
 	if (mode == MOVE) {
 	    if (curunit != NULL
 	    	&& in_play(curunit)
-		&& (curunit->act
-		&& curunit->act->acp > 0)  /* should be "above min"? */
+		&& has_acp_left(curunit)
 	    	&& (curunit->plan
 		    && !curunit->plan->asleep
 		    && !curunit->plan->reserve
@@ -564,7 +551,7 @@ please somebody write this code
 }
 
 void
-interpret_input()
+interpret_input(void)
 {
     int dir;
 
@@ -596,7 +583,8 @@ interpret_input()
 	      prefixarg = 10;
 	    do_dir_2(dir, prefixarg);
 	} else {
-	    execute_command();
+	    dside->prefixarg = prefixarg;
+	    execute_command(dside, inpch);
 	}
 	/* Reset the arg so we don't get confused next time around */
 	prefixarg = -1;
@@ -606,10 +594,11 @@ interpret_input()
 /* Interpret a direction character. */
 
 void
-do_dir_2(dir, n)
-int dir, n;
+do_dir_2(int dir, int n)
 {
     int nx, ny, rslt;
+    HistEventType reason;
+    char failbuf[BUFSIZE];
 
     if (mode == SURVEY) {
 	if (point_in_dir_n(curx, cury, dir, n, &nx, &ny)) {
@@ -626,9 +615,13 @@ int dir, n;
 	net_set_move_dir_task(curunit, dir, n);
     } else {
 	if (point_in_dir(curx, cury, dir, &nx, &ny)) {
-	    rslt = advance_into_cell(dside, curunit, nx, ny, unit_at(nx, ny));
-	    if (!rslt)
-	      xbeep();
+	    rslt = advance_into_cell(dside, curunit, nx, ny, unit_at(nx, ny),
+				     &reason);
+	    if (!rslt) {
+		advance_failure_desc(failbuf, curunit, reason);
+		notify(dside, "%s", failbuf);
+		xbeep();
+	    }
 	} else {
 	    xbeep();
 	}
@@ -636,15 +629,13 @@ int dir, n;
 }
 
 int
-auto_attack_on_move(unit, unit2)
-Unit *unit, *unit2;
+auto_attack_on_move(Unit *unit, Unit *unit2)
 {
     return TRUE;
 }
 
 void
-move_survey(nx, ny)
-int nx, ny;
+move_survey(int nx, int ny)
 {
     if (inside_area(nx, ny)) {
 	make_current_at(nx, ny);
@@ -657,8 +648,7 @@ int nx, ny;
 /* Ensure that given location is visible. */
 
 void
-put_on_screen(x, y)
-int x, y;
+put_on_screen(int x, int y)
 {
     if (!in_middle(x, y)) {
 	set_view_focus(mvp, x, y);
@@ -673,9 +663,7 @@ int x, y;
 /* Prompt for a yes/no answer with a settable default. */
 
 int
-ask_bool(question, dflt)
-char *question;
-int dflt;
+ask_bool(char *question, int dflt)
 {
     char ch;
 
@@ -697,9 +685,7 @@ int dflt;
    vector of unit types, so as to be able to map back when done. */
 
 int
-ask_unit_type(prompt, possibles)
-char *prompt;
-short *possibles;
+ask_unit_type(char *prompt, short *possibles)
 {
     char ch;
     int numtypes = 0, u;
@@ -708,8 +694,8 @@ short *possibles;
 	bvec[u] = 0;
 	if (possibles == NULL || possibles[u]) {
 	    bvec[u] = 1;
-	    ustr[numtypes] = utype_name_n(u, 1)[0];
 	    uvec[numtypes] = u;
+	    ustr[numtypes] = unitchars[u];
 	    numtypes++;
 	}
     }
@@ -745,9 +731,7 @@ short *possibles;
    when done. */
 
 int
-ask_terrain_type(prompt, possibles)
-char *prompt;
-short *possibles;
+ask_terrain_type(char *prompt, short *possibles)
 {
     char ch;
     int numtypes = 0, t, type;
@@ -756,9 +740,8 @@ short *possibles;
 	bvec[t] = 0;
 	if (possibles == NULL || possibles[t]) {
 	    bvec[t] = 1;
-	    ustr[numtypes]
-	      = (t_char(t) ? t_char(t)[0] : t_type_name(t)[0]);
 	    uvec[numtypes] = t;
+	    ustr[numtypes] = terrchars[t];
 	    numtypes++;
 	}
     }
@@ -789,9 +772,7 @@ short *possibles;
 /* Ask for a direction. */
 
 int
-ask_direction(prompt, dirp)
-char *prompt;
-int *dirp;
+ask_direction(char *prompt, int *dirp)
 {
     char ch;
 
@@ -827,9 +808,7 @@ int *dirp;
    '.' designates the final position. */
 
 int
-ask_position(prompt, xp, yp)
-char *prompt;
-int *xp, *yp;
+ask_position(char *prompt, int *xp, int *yp)
 {
     char ch;
     int dir, nx, ny;
@@ -875,7 +854,7 @@ int *xp, *yp;
 /* Save away the currently selected position. */
 
 void
-save_cur()
+save_cur(void)
 {
     tmpcurx = curx;  tmpcury = cury;
     tmpcurunit = curunit;
@@ -884,7 +863,7 @@ save_cur()
 /* Restore the saved "cur" slots. */
 
 void
-restore_cur()
+restore_cur(void)
 {
     curx = tmpcurx;  cury = tmpcury;
     curunit = tmpcurunit;
@@ -894,8 +873,7 @@ restore_cur()
    cursor is displayed. */
 
 int
-ask_string(prompt, dflt, strp)
-char *prompt, *dflt, **strp;
+ask_string(char *prompt, char *dflt, char **strp)
 {
     char ch;
     int done = FALSE, rslt = FALSE;
@@ -955,9 +933,7 @@ char *prompt, *dflt, **strp;
 /* Ask for a side. */
 
 Side *
-ask_side(prompt, dflt)
-char *prompt;
-Side *dflt;
+ask_side(char *prompt, Side *dflt)
 {
     char ch;
     Side *rslt = dflt;
@@ -987,13 +963,10 @@ Side *dflt;
 }
 
 int
-ask_unit(prompt, unitp)
-char *prompt;
-Unit **unitp;
+ask_unit(char *prompt, Unit **unitp)
 {
     char ch;
     int ndirs, dir, nx, ny, rslt = FALSE;
-    Unit *nextocc;
 
     *unitp = NULL;
     prevmode = mode;
@@ -1046,8 +1019,7 @@ Unit **unitp;
 /* Given a unit, make it be the one that we are "looking at". */
 
 void
-make_current(unit)
-Unit *unit;
+make_current(Unit *unit)
 {
     curunit = unit;
     if (in_play(unit)) {
@@ -1063,8 +1035,7 @@ Unit *unit;
    the current unit. */
 
 void
-make_current_at(x, y)
-int x, y;
+make_current_at(int x, int y)
 {
     Unit *unit;
 
@@ -1080,11 +1051,10 @@ int x, y;
     put_on_screen(curx, cury);
 }
 
-/* (should add 'P' for printing/saving to file (which?)) */
 /* (should add individual screen scrolling also) */
 
 void
-interpret_help()
+interpret_help(void)
 {
     HelpNode *prevhelpnode = cur_help_node;
     extern int first_visible_help_pos, last_visible_help_pos;
@@ -1125,10 +1095,7 @@ interpret_help()
 }
 
 static void
-describe_help(arg, key, buf)
-int arg;
-char *key;
-TextBuffer *buf;
+describe_help(int arg, char *key, TextBuffer *buf)
 {
     tbcat(buf, "' ' to page down through a node\n");
     tbcat(buf, "'n' to go to next help node\n");
@@ -1142,15 +1109,14 @@ TextBuffer *buf;
 /* Reading is usually pretty fast, so don't do anything here. */
 
 void
-announce_read_progress()
+announce_read_progress(void)
 {
 }
 
 /* Announce the start of a time-consuming computation. */
 
 void
-announce_lengthy_process(msg)
-char *msg;
+announce_lengthy_process(char *msg)
 {
     n_seconds_elapsed(0);
     announcemsg = copy_string(msg);
@@ -1163,8 +1129,7 @@ char *msg;
 }
 
 void
-announce_progress(percentdone)
-int percentdone;
+announce_progress(int percentdone)
 {
     if (n_seconds_elapsed(2)) {
 	printf(" %d%%,", percentdone);
@@ -1174,7 +1139,7 @@ int percentdone;
 }
 
 void
-finish_lengthy_process()
+finish_lengthy_process(void)
 {
     if (announced) {
 	printf(" done.\n");
@@ -1185,18 +1150,15 @@ finish_lengthy_process()
 /* This should be called before any sort of normal exit. */
 
 void
-exit_cconq()
+exit_cconq(void)
 {
     close_displays();
-    /* Leave any commentary on the console. */
-    printf("\n%s\n", exit_commentary(dside));
     printf("\n%s\n", get_scores(dside));
     exit(0);
 }
 
 void
-flush_display_buffers(side)
-Side *side;
+flush_display_buffers(Side *side)
 {
     if (active_display(side)) {
 	refresh();
@@ -1206,8 +1168,7 @@ Side *side;
 /* A predicate that tests whether our display can validly be written to. */
 
 int
-active_display(side)
-Side *side;
+active_display(Side *side)
 {
     return (side == dside
 	    && dside != NULL
@@ -1215,17 +1176,21 @@ Side *side;
 	    && active);
 }
 
+void
+update_area_display(Side *side)
+{
+  if (active_display(side))
+    refresh();
+}
+
 /* Draw an individual detailed cell, as a row of one, on all maps. */
 
 void
-update_cell_display(side, x, y, rightnow)
-Side *side;
-int x, y;
-int rightnow;
+update_cell_display(Side *side, int x, int y, int flags)
 {
     if (active_display(side)) {
 	draw_row(x, y, 1);
-	if (rightnow)
+	if (flags & UPDATE_ALWAYS)
 	  refresh();
     }
 }
@@ -1233,24 +1198,25 @@ int rightnow;
 /* Curses updates efficiently, so don't need special routine. */
 
 void
-update_side_display(side, side2, rightnow)
-Side *side, *side2;
-int rightnow;
+update_side_display(Side *side, Side *side2, int rightnow)
 {
     if (active_display(side)) {
 	show_side_list();
 	if (rightnow)
 	  refresh();
     }
+    if (numatypes > 0
+	&& g_side_can_research()
+	&& dside->research_topic == NOADVANCE) {
+	      /* Should eventually pop up a side research dialog instead. */
+	      auto_pick_side_research(dside);
+    }
 }
 
 /* Just change the part of the unit list relevant to the given unit. */
 
 void
-update_unit_display(side, unit, rightnow)
-Side *side;
-Unit *unit;
-int rightnow;
+update_unit_display(Side *side, Unit *unit, int rightnow)
 {
     if (active_display(side) && unit != NULL) {
 	if (curunit == unit) {
@@ -1265,20 +1231,13 @@ int rightnow;
 }
 
 void
-update_unit_acp_display(side, unit, rightnow)
-Side *side;
-Unit *unit;
-int rightnow;
+update_unit_acp_display(Side *side, Unit *unit, int rightnow)
 {
-    if (active_display(side)) {
-	/* (should impl?) */
-    }
+    update_unit_display(side, unit, rightnow);
 }
 
 void
-update_turn_display(side, rightnow)
-Side *side;
-int rightnow;
+update_turn_display(Side *side, int rightnow)
 {
     if (active_display(side)) {
 	show_game_date();
@@ -1291,9 +1250,7 @@ int rightnow;
 }
 
 void
-update_action_display(side, rightnow)
-Side *side;
-int rightnow;
+update_action_display(Side *side, int rightnow)
 {
     if (active_display(side)) {
 	show_side_list();
@@ -1303,10 +1260,7 @@ int rightnow;
 }
 
 void
-update_action_result_display(side, unit, rslt, rightnow)
-Side *side;
-Unit *unit;
-int rslt, rightnow;
+update_action_result_display(Side *side, Unit *unit, int rslt, int rightnow)
 {
     Action *action;
 
@@ -1322,19 +1276,14 @@ int rslt, rightnow;
 }
 
 void
-update_event_display(side, hevt, rightnow)
-Side *side;
-HistEvent *hevt;
-int rightnow;
+update_event_display(Side *side, HistEvent *hevt, int rightnow)
 {
     /* (should add to some sort of history display) */
 }
 
 void
-update_fire_at_display(side, unit, unit2, m, rightnow)
-Side *side;
-Unit *unit, *unit2;
-int m, rightnow;
+update_fire_at_display(Side *side, Unit *unit, Unit *unit2, int m,
+		       int rightnow)
 {
     /* (should flash at target) */
 }
@@ -1342,10 +1291,8 @@ int m, rightnow;
 /* This is for animation of fire-into actions. */
 
 void
-update_fire_into_display(side, unit, x, y, z, m, rightnow)
-Side *side;
-Unit *unit;
-int x, y, z, m, rightnow;
+update_fire_into_display(Side *side, Unit *unit, int x, int y, int z, int m,
+			 int rightnow)
 {
     /* (should flash at target) */
 }
@@ -1353,9 +1300,7 @@ int x, y, z, m, rightnow;
 /* Updates to clock need to be sure that display changes immediately. */
 
 void
-update_clock_display(side, rightnow)
-Side *side;
-int rightnow;
+update_clock_display(Side *side, int rightnow)
 {
     if (active_display(side)) {
 	show_clock();
@@ -1366,10 +1311,7 @@ int rightnow;
 }
 
 void
-update_message_display(side, sender, str, rightnow)
-Side *side, *sender;
-char *str;
-int rightnow;
+update_message_display(Side *side, Side *sender, char *str, int rightnow)
 {
     if (active_display(side)) {
 	notify(side, "%s", str);
@@ -1379,22 +1321,18 @@ int rightnow;
 }
 
 void
-update_all_progress_displays(str, s)
-char *str;
-int s;
+update_all_progress_displays(char *str, int s)
 {
 }
 
 void
-update_everything()
+update_everything(void)
 {
     redraw();
 }
 
 void
-action_point(side, x, y)
-Side *side;
-int x, y;
+action_point(Side *side, int x, int y)
 {
     if (!active_display(dside))
       return;
@@ -1405,7 +1343,7 @@ int x, y;
 }
 
 void
-close_displays()
+close_displays(void)
 {
     if (!active_display(dside))
       return;
@@ -1419,8 +1357,7 @@ close_displays()
 /* An init error needs to have the command re-run. */
 
 void
-low_init_error(str)
-char *str;
+low_init_error(char *str)
 {
     fprintf(stderr, "Error: %s.\n", str);
     fflush(stderr);
@@ -1429,8 +1366,7 @@ char *str;
 /* A warning just gets displayed, no other action is taken. */
 
 void
-low_init_warning(str)
-char *str;
+low_init_warning(char *str)
 {
     fprintf(stdout, "Warning: %s.\n", str);
     fflush(stdout);
@@ -1439,8 +1375,7 @@ char *str;
 /* A run error is fatal. */
 
 void
-low_run_error(str)
-char *str;
+low_run_error(char *str)
 {
     close_displays();
     fprintf(stderr, "Error: %s.\n", str);
@@ -1456,8 +1391,7 @@ char *str;
    usually a problem with Xconq or a game design. */
 
 void
-low_run_warning(str)
-char *str;
+low_run_warning(char *str)
 {
     if (active_display(dside)) {
 	notify(dside, "Warning: %s; continue? ", str);
@@ -1468,30 +1402,23 @@ char *str;
 }
 
 void
-print_form(form)
-Obj *form;
+print_form(Obj *form)
 {
     print_form_and_value(stdout, form);
 }
 
 void
-end_printing_forms()
+end_printing_forms(void)
 {
 }
 
 /* Even a curses interface can do simple "movies". */
 
 int
-#ifdef __STDC__
 schedule_movie(Side *side, char *movie, ...)
-#else
-schedule_movie(side, movie, a1, a2, a3, a4, a5)
-Side *side;
-char *movie;
-int a1, a2, a3, a4, a5;
-#endif
 {
     int i, itype;
+    va_list ap;
 
     if (numscheduled >= 10)
       return FALSE;
@@ -1513,29 +1440,18 @@ int a1, a2, a3, a4, a5;
     else if (strcmp(movie, "nuke") == 0)
       itype = movie_nuke;
     movies[numscheduled].itype = itype;
-#ifdef ANSI_PROTOTYPES
-    {
-	va_list ap;
 
-	va_start(ap, movie);
-	for (i = 0; i < 5; ++i)
-	  movies[numscheduled].args[i] = va_arg(ap, int);
-	va_end(ap);
-    }
-#else
-    movies[numscheduled].args[0] = a1;
-    movies[numscheduled].args[1] = a2;
-    movies[numscheduled].args[2] = a3;
-    movies[numscheduled].args[3] = a4;
-    movies[numscheduled].args[4] = a5;
-#endif
+    va_start(ap, movie);
+    for (i = 0; i < 5; ++i)
+      movies[numscheduled].args[i] = va_arg(ap, int);
+    va_end(ap);
+
     ++numscheduled;
     return TRUE;
 }
 
 void
-play_movies(sidemask)
-SideMask sidemask;
+play_movies(SideMask sidemask)
 {
     int j, unitid, blasttype;
     Unit *unit;
@@ -1568,41 +1484,7 @@ SideMask sidemask;
 }
 
 void
-set_you_player(id)
-int id;
-{
-}
-
-void
-add_remote_player(name)
-char *name;
-{
-}
-
-int
-launch_game()
-{
-    /* Trial side/player assignment, will create the default player if
-       nececessary. */
-    make_trial_assignments();
-    /* A last-minute patch to ensure reasonable area dimensions. */
-    if (area.width == 0 || area.height == 0)
-      set_area_shape(DEFAULTWIDTH, DEFAULTHEIGHT, TRUE);
-    /* Synthesize the remainder of the initial game setup. */
-    calculate_globals();
-    run_synth_methods();
-    final_init();
-    assign_players_to_sides();
-    /* Init the display now. */
-    init_display();
-    /* Do first set of turn calcs. */
-    run_game(0);
-    return TRUE;
-}
-
-void
-make_generic_image_data(imf)
-ImageFamily *imf;
+make_generic_image_data(ImageFamily *imf)
 {
 }
 
@@ -1616,3 +1498,19 @@ serial_port_dialog()
 }
 
 #endif /* MAC */
+
+void
+unit_research_dialog(Unit *unit)
+{
+    auto_pick_unit_research(unit);
+}
+
+void
+add_remote_locally(int rid, char *str)
+{
+}
+
+void
+send_chat(int rid, char *str)
+{
+}
