@@ -7,7 +7,11 @@ it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
+#ifdef MAC
+#include "curses.h"
+#else
 #include <curses.h>
+#endif
 
 #include "imf.h"
 #include "ui.h"
@@ -98,6 +102,7 @@ extern int drawterrain;
 extern int drawunits;
 extern int drawnames;
 extern int drawpeople;
+extern int draw_cover;
 extern enum listsides listsides;
 extern int test;
 extern int value;
