@@ -1,6 +1,5 @@
 /* Map graphics for the curses interface to Xconq.
-   Copyright (C) 1986, 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995
-   Stanley T. Shebs.
+   Copyright (C) 1986-1989, 1991-1996, 1998-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -8,23 +7,21 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern void others_here_desc PARAMS ((char *buf, Unit *unit));
-extern void occupants_desc PARAMS ((char *buf, Unit *unit));
 #include "cconq.h"
 
-static int xform PARAMS ((int x, int y, int *sxp, int *syp));
-static void draw_divider PARAMS ((void));
-static void draw_terrain_row PARAMS ((int x, int y, int len));
-static void draw_units PARAMS ((int x, int y));
-static void draw_people PARAMS ((int x, int y));
-static void draw_legend PARAMS ((int x, int y));
-static void draw_unit_details PARAMS ((Unit *unit));
-static void describe_cell PARAMS ((int x, int y, int tview, char *filler,
-				  char *buf));
-static void organize_list_contents PARAMS ((void));
-static void add_unit_to_list PARAMS ((Unit *unit));
-static void draw_unit_list_entry PARAMS ((int line));
-static void draw_type_list_entry PARAMS ((int line));
+static int xform(int x, int y, int *sxp, int *syp);
+static void draw_divider(void);
+static void draw_terrain_row(int x, int y, int len);
+static void draw_units(int x, int y);
+static void draw_people(int x, int y);
+static void draw_legend(int x, int y);
+static void draw_unit_details(Unit *unit);
+static void describe_cell(int x, int y, int tview, char *filler,
+				  char *buf);
+static void organize_list_contents(void);
+static void add_unit_to_list(Unit *unit);
+static void draw_unit_list_entry(int line);
+static void draw_type_list_entry(int line);
 
 char *dashbuffer;
 
@@ -42,7 +39,7 @@ int use_standout = FALSE;
 
 int drawlinear = TRUE;
 
-char linear_char = '#';
+char linear_char = '\\';
 
 char bord_char = '\0';
 
@@ -65,7 +62,7 @@ int last_visible_help_pos;
    input. */
 
 void
-redraw()
+redraw(void)
 {
     if (active_display(dside)) {
 	switch (mode) {
@@ -96,7 +93,7 @@ redraw()
 }
 
 static void
-draw_divider()
+draw_divider(void)
 {
     int i;
 
@@ -111,8 +108,7 @@ draw_divider()
    limit of 5x5 for the map window. (I think) */
 
 int
-in_middle(x, y)
-int x, y;
+in_middle(int x, int y)
 {
     int sx, sy;
 
@@ -122,15 +118,14 @@ int x, y;
 }
 
 static int
-xform(x, y, sxp, syp)
-int x, y, *sxp, *syp;
+xform(int x, int y, int *sxp, int *syp)
 {
     xform_cell(mvp, x, y, sxp, syp);
     return TRUE;
 }
 
 void
-set_scroll()
+set_scroll(void)
 {
     int sx, sy, hexadj = hexagon_adjust(mvp);
 
@@ -145,7 +140,7 @@ set_scroll()
 }
 
 void
-set_map_viewport()
+set_map_viewport(void)
 {
     /* Compute the size of the viewport. */
     vw = min(area.width, mw / 2 + 1);
@@ -162,7 +157,7 @@ set_map_viewport()
 /* Display a map and all of its paraphernalia. */
 
 void
-show_map()
+show_map(void)
 {
     int y1, y2, y, x1, x2;
     int halfheight = area.height / 2;
@@ -232,8 +227,7 @@ show_map()
 }
 
 void
-draw_row(x0, y0, len)
-int x0, y0, len;
+draw_row(int x0, int y0, int len)
 {
     int x;
 
@@ -271,8 +265,7 @@ int x0, y0, len;
 /* Draw a single row of just terrain. */
 
 static void
-draw_terrain_row(x0, y0, len)
-int x0, y0, len;
+draw_terrain_row(int x0, int y0, int len)
 {
     char ch, ch2;
     int x, sx, sy, t, t2;
@@ -296,6 +289,9 @@ int x0, y0, len;
 		    }
 		}
 	    }
+	    /* Don't draw the second char if showing view coverage. */
+	    if (draw_cover && !dside->show_all && cover(dside, x, y0) <= 0)
+	      ch2 = ' ';
 	}
 	if (cur_at(mapwin, sx, sy))
 	  addch(ch);
@@ -308,32 +304,21 @@ int x0, y0, len;
 /* Draw a single unit char pair as appropriate. */
 
 static void
-draw_units(x, y)
-int x, y;
+draw_units(int x, int y)
 {
-    int draw = FALSE;
-    int xw, sx, sy, uview, u, s;
-    Unit *unit;
+    int draw = FALSE, xw, sx, sy, u, s;
+    UnitView *uview;
 
     xw = wrapx(x);    
-    if (units_visible(dside, xw, y)) {
-	if (in_play(curunit) && curunit->x == xw && curunit->y == y) {
-	    unit = curunit;
-	} else {
-	    unit = unit_at(xw, y);
-	}
-	if (unit != NULL) {
-	    u = unit->type;
-	    s = side_number(unit->side);
-	    draw = TRUE;
-	}
-    } else {
-	uview = unit_view(dside, xw, y);
-	if (uview != EMPTY) {
-	    u = vtype(uview);
-	    s = vside(uview);
-	    draw = TRUE;
-	}
+    /* Always make the curunit take precedence. */
+    if (in_play(curunit) && curunit->x == xw && curunit->y == y) {
+	u = curunit->type;
+	s = curunit->side->id;
+	draw = TRUE;
+    } else if ((uview = unit_view_at(dside, x, y)) != NULL) {
+	u = view_type(uview);
+	s = view_side(uview)->id;
+	draw = TRUE;
     }
     if (draw) {
 	xform(x, y, &sx, &sy);
@@ -354,8 +339,7 @@ int x, y;
 /* Indicate what kind of people are living in the given cell. */
 
 static void
-draw_people(x, y)
-int x, y;
+draw_people(int x, int y)
 {
     int pop, sx, sy;
 
@@ -376,16 +360,16 @@ int x, y;
    feature's "center" */
 
 static void
-draw_legend(x, y)
-int x, y;
+draw_legend(int x, int y)
 {
     int sx, sy, pixlen;
     char legend[100], *str;
-    int uview = unit_view(dside, x, y);
     Feature *feature;
     Unit *unit;
+    UnitView *uview;
 
-    if (uview == EMPTY)
+    uview = unit_view_at(dside, x, y);
+    if (uview == NULL)
       return;
     /* Draw a unit's name or number. */
     /* (This will only do top unit in hex, what about others?) */
@@ -393,7 +377,7 @@ int x, y;
 	&& (unit = unit_at(x, y)) != NULL) {
 	/* still has a bug if types happen to match - need to use view date
 	   instead? */
-	if (vtype(uview) != unit->type)
+	if (view_type(uview) != unit->type)
 	  return;
 	if (unit->name != NULL) {
 	    strcpy(legend, unit->name);
@@ -421,9 +405,7 @@ int x, y;
 /* (should change text[12] to an array) */
 
 void
-low_notify(side, str)
-Side *side;
-char *str;
+low_notify(Side *side, char *str)
 {
     char *lastblank, extrabuf[BUFSIZE];
 
@@ -449,7 +431,7 @@ char *str;
 }
 
 void
-show_toplines()
+show_toplines(void)
 {
     clear_window(toplineswin);
     draw_text(toplineswin, 0, 0, text1);
@@ -457,7 +439,7 @@ show_toplines()
 }
 
 void
-clear_toplines()
+clear_toplines(void)
 {
     text1[0] = '\0';
     text2[0] = '\0';
@@ -465,14 +447,16 @@ clear_toplines()
 }
 
 /* Display all the details of the currently-selected unit/cell. */
+/* (should use generic code) */
 
 void
-show_closeup()
+show_closeup(void)
 {
-    int u, t, uview, tview;
+    int u, t, tview;
     char *filler = "Empty ";
     Unit *unit = NULL;
     Side *side2;
+    UnitView *uview;
 
     clear_window(closeupwin);
     if (inside_area(curx, cury)) {
@@ -481,23 +465,24 @@ show_closeup()
 	} else {
 	    tview = terrain_view(dside, curx, cury);
 	}
-	if (units_visible(dside, curx, cury)) {
+	if (dside->show_all || in_play(curunit)) {
 	    unit = (in_play(curunit) ? curunit : unit_at(curx, cury));
 	    if (in_play(unit)) {
-		/* If there is a unit there, we can at least see basic info. */
+		/* If there is a unit there, we can at least see basic
+                   info. */
 		sprintf(tmpbuf, "%s", unit_handle(dside, unit));
 		draw_text(closeupwin, 0, 0, tmpbuf);
-		if (unit->side == dside) {
-		    draw_unit_details(unit);
-		}
+		/* Show more about one of our own units. */
+		if (unit->side == dside)
+		  draw_unit_details(unit);
 		filler = "In ";
 	    }
 	} else {
-	    uview = unit_view(dside, curx, cury);
-	    if (uview != EMPTY) {
+	    uview = unit_view_at(dside, curx, cury);
+	    if (uview != NULL) {
 		filler = "In ";
-		u = vtype(uview);
-		side2 = side_n(vside(uview));
+		u = view_type(uview);
+		side2 = view_side(uview);
 		sprintf(tmpbuf, "%s %s",
 		    side_adjective(side2), u_type_name(u));
 		draw_text(closeupwin, 0, 0, tmpbuf);
@@ -507,7 +492,8 @@ show_closeup()
 	if (unit != NULL && unit->transport != NULL) {
 	    t = terrain_at(unit->x, unit->y);
 	    sprintf(tmpbuf, "In %s (%s at %d,%d)",
-		    short_unit_handle(unit->transport), t_type_name(t), unit->x, unit->y);
+		    short_unit_handle(unit->transport), t_type_name(t),
+		    unit->x, unit->y);
 	} else {
 	    describe_cell(curx, cury, tview, filler, tmpbuf);
 	}
@@ -519,9 +505,7 @@ show_closeup()
 }
 
 static void
-describe_cell(x, y, tview, filler, buf)
-int x, y, tview;
-char *filler, *buf;
+describe_cell(int x, int y, int tview, char *filler, char *buf)
 {
     int t;
     char *featname;
@@ -552,8 +536,7 @@ char *filler, *buf;
 /* Describe the state of the given unit, in maximal detail. */
 
 static void
-draw_unit_details(unit)
-Unit *unit;
+draw_unit_details(Unit *unit)
 {
     int t;
 
@@ -601,7 +584,7 @@ Unit *unit;
 	plan_desc(spbuf, unit);
 	draw_text(closeupwin, 0, 4, spbuf);
 	for (task = unit->plan->tasks; task != NULL; task = task->next) {
-	    task_desc(spbuf, unit->side, task);
+	    task_desc(spbuf, unit->side, unit, task);
 	    draw_text(closeupwin, 0, row++, spbuf);
 	}
     }
@@ -610,7 +593,7 @@ Unit *unit;
 /* Basic routine that displays the list of sides. */
 
 void
-show_side_list()
+show_side_list(void)
 {
     char ismoving, progress[20], *dpyname;
     int sy = 0, totacp;
@@ -621,6 +604,7 @@ show_side_list()
     clear_window(sideswin);
     for_all_sides(side2) {
     	ismoving = ' ';
+	/* (should use generic test) */
 	if ((g_use_side_priority()
 	     ? (curpriority == side2->priority)
 	     : (!side2->finishedturn)))
@@ -650,11 +634,11 @@ show_side_list()
 	    }
 	}
 	dpyname = "";
-	if (side2->player->displayname)
+	if (side2->player && side2->player->displayname)
 	  dpyname = side2->player->displayname;
 	sprintf(spbuf, "%d%c %s %s (%s)",
 		side_number(side2), ismoving, progress,
-		side_name(side2), dpyname);
+		short_side_title(side2), dpyname);
 	draw_text(sideswin, 0, sy, spbuf);
 	sy += 1;
     }
@@ -672,7 +656,7 @@ show_side_list()
 /* Display the date. */
 
 void
-show_game_date()
+show_game_date(void)
 {
     clear_window(datewin);
     /* First line of the game state. */
@@ -687,7 +671,7 @@ show_game_date()
 /* (should add scrolling interaction) */
 
 static void
-organize_list_contents()
+organize_list_contents(void)
 {
     Side *side2;
     Unit *unit;
@@ -708,18 +692,12 @@ organize_list_contents()
 	    }
 	}
     }
-    for_all_side_units(indepside, unit) {
-	if (side_sees_image(dside, unit)) {
-	    add_unit_to_list(unit);
-	}
-    }
     /* Now sort the list according to its keys. */
     sort_unit_vector(listvector);
 }
 
 static void
-add_unit_to_list(unit)
-Unit *unit;
+add_unit_to_list(Unit *unit)
 {
     if (alive(unit)) {
 	add_unit_to_vector(listvector, unit, FALSE);
@@ -729,7 +707,7 @@ Unit *unit;
 }
 
 void
-show_list()
+show_list(void)
 {
     int i = 0, line = 1;
     char *maincat = "xxxxx", *filter = "?yyyy?";
@@ -817,19 +795,19 @@ show_list()
 }
 
 void
-cycle_list_type()
+cycle_list_type(void)
 {
     listtype = (listtype + 1) % 2;
 }
 
 void
-cycle_list_filter()
+cycle_list_filter(void)
 {
     listsides = (listsides + 1) % 3;
 }
 
 void
-cycle_list_order()
+cycle_list_order(void)
 {
     tmpsortkeys[0] = (tmpsortkeys[0] + 1) % numsortkeytypes;
     sort_unit_vector(listvector);
@@ -844,8 +822,7 @@ int firsttypevisible = 0;
 int lasttypevisible = MAXUTYPES;
 
 static void
-draw_type_list_entry(line)
-int line;
+draw_type_list_entry(int line)
 {
     int u, num;
 
@@ -863,12 +840,10 @@ int line;
 	sprintf(tmpbuf, "    ");
     }
     strcat(spbuf, tmpbuf);
-    /* Our units under construction (left-justified) */
+    /* Our units under construction. */
     num = num_units_incomplete(dside, u);
     if (num > 0) {
-	sprintf(tmpbuf, "(%2d)", num);
-	if (num > 9)
-	  strcat(tmpbuf, " ");
+	sprintf(tmpbuf, "(%d)", num);
     } else {
 	sprintf(tmpbuf, "    ");
     }
@@ -891,10 +866,9 @@ int line;
 }
 
 static void
-draw_unit_list_entry(line)
-int line;
+draw_unit_list_entry(int line)
 {
-    Unit *unit = listvector->units[line].unit;
+    Unit *unit = unit_in_vector(listvector, line);
     char tmpbuf[BUFSIZE];
 
     if (unit == NULL)
@@ -906,7 +880,7 @@ int line;
 	name_or_number(unit, tmpbuf);
 	sprintf(spbuf, "%c%d %-16s ",
 		unitchars[unit->type], side_number(unit->side), tmpbuf);
-	if (unit->act && unit->act->acp > 0) {
+	if (has_acp_left(unit)) {
 	    tprintf(spbuf, "%3d", unit->act->acp);
 	}
 	if (unit->plan && unit->plan->asleep) {
@@ -926,7 +900,7 @@ int line;
 }
 
 void
-show_clock()
+show_clock(void)
 {
 #if 0
     int time = 0;
@@ -941,8 +915,7 @@ show_clock()
 /* General window clearing. */
 
 void
-clear_window(win)
-struct ccwin *win;
+clear_window(struct ccwin *win)
 {
     int i;
 
@@ -969,8 +942,7 @@ struct ccwin *win;
 /* Draw a large blot over the area. */
 
 void
-draw_blast(x, y, r)
-int x, y, r;
+draw_blast(int x, int y, int r)
 {
     int sx, sy;
 
@@ -993,10 +965,7 @@ int x, y, r;
 /* Drawing text is easy, but we do need to do clipping manually. */
 
 int
-draw_text(win, x, y, str)
-struct ccwin *win;
-int x, y;
-char *str;
+draw_text(struct ccwin *win, int x, int y, char *str)
 {
     int i, slen, linestart = 0;
 
@@ -1026,9 +995,9 @@ char *str;
 /* Make a beep by writing ^G. */
 
 void
-xbeep()
+xbeep(void)
 {
-#ifdef THINK_C /* the libcurses library already includes a beep function */
+#ifdef MAC /* the libcurses library already includes a beep function */
     beep();
 #else
     putchar('\007');
@@ -1039,7 +1008,7 @@ xbeep()
    calcs simpler) */
 
 void
-show_help()
+show_help(void)
 {
     int i, slen, x = 0, y = 0, more = FALSE;
     char *str;
@@ -1097,7 +1066,7 @@ show_help()
 /* Put the terminal's cursor at an appropriate place. */
 
 void
-show_cursor()
+show_cursor(void)
 {
     int sx, sy;
 
@@ -1152,9 +1121,7 @@ show_cursor()
 /* Position the cursor, being careful to test for sensibility. */
 
 int
-cur_at(win, x, y)
-struct ccwin *win;
-int x, y;
+cur_at(struct ccwin *win, int x, int y)
 {
     int sx, sy;
 
