@@ -6,16 +6,36 @@ it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
+/* The commands in this file are the same for all interfaces.  They
+   typically consist of direct operations on units, or complicated
+   command-line-type commands. */
+
 #include "conq.h"
 #include "kpublic.h"
 #include "imf.h"
 #include "ui.h"
+
 int autofinish_start;
 int autofinish_count;
 
-extern void notify_relationships(Side *side);
-
+static void notify_relationships(Side *side);
 static int parse_advance_spec(char *str);
+static int do_one_auto(Side *side, Unit *unit);
+static int do_one_clear_plan(Side *side, Unit *unit);
+static int do_one_delay(Side *side, Unit *unit);
+static int do_one_detach(Side *side, Unit *unit);
+static int do_one_detonate(Side *side, Unit *unit);
+static int do_one_disband(Side *side, Unit *unit);
+static int do_one_disembark(Side *side, Unit *unit);
+static int do_one_embark(Side *side, Unit *unit);
+static int do_one_give(Side *side, Unit *unit);
+static int do_one_idle(Side *side, Unit *unit);
+static int do_one_reserve(Side *side, Unit *unit);
+static int do_one_return(Side *side, Unit *unit);
+static int do_one_sleep(Side *side, Unit *unit);
+static int do_one_take(Side *side, Unit *unit);
+static int do_one_wake(Side *side, Unit *unit);
+static int do_one_wake_all(Side *side, Unit *unit);
 
 /* The command table is an array of all the commands. */
 
@@ -40,10 +60,19 @@ CmdTab commands[] = {
   { 0, NULL, /* NULL, */ NULL, NULL }
 };
 
-char *cmdargstr;
+/* This is the actual key typed, for use if several keyboard commands
+   share a single function. */
 
 char tmpkey;
 
+/* If the command was typed as a command-line sort of thing, then this
+   global will point to whatever came after the name of the
+   command. */
+
+char *cmdargstr;
+
+/* Help nodes that list all the commands, both single-char and long. */
+
 HelpNode *key_commands_help_node;
 
 HelpNode *long_commands_help_node;
@@ -195,6 +224,36 @@ describe_long_commands(int arg, char *key, TextBuffer *buf)
     }
 }
 
+static int
+apply_to_all_selected_units(Side *side, int (*fn)(Side *side, Unit *unit))
+{
+    int i, rslt, numcould = 0, numfail = 0;
+    Unit *unit;
+    UnitVector *selvec;
+
+    if (fn == NULL)
+      /* (should be a fatal error?) */
+      return 0;
+    /* Scan all the selected units. Note that we assume that the list
+       (unit vector) doesn't change while we're scanning, which means
+       that we should be checking each unit, in case a command caused
+       later units in the list to disappear (for instance a
+       detonation). */
+    selvec = get_selected_units(side);
+    for (i = 0; i < selvec->numunits; ++i) {
+	unit = unit_in_vector(selvec, i);
+	if (in_play(unit) && side_controls_unit(side, unit)) {
+	    ++numcould;
+	    rslt = (*fn)(side, unit);
+	    if (!rslt)
+	      ++numfail;
+	}
+    }
+    /* Return true if everything went OK, false if the function failed
+       on some units. */
+    return ((numcould > 0) ? (numfail == 0) : TRUE);
+}
+
 /* Generic command functions.  These are the command functions that
    behave identically for all interfaces. */
 
@@ -232,43 +291,37 @@ do_agreement_draft(Side *side)
 }
 
 void
-do_agreement_drafter(side)
-Side *side;
+do_agreement_drafter(Side *side)
 {
     DURING_GAME_ONLY(side);
 }
 
 void
-do_agreement_propose(side)
-Side *side;
+do_agreement_propose(Side *side)
 {
     DURING_GAME_ONLY(side);
 }
 
 void
-do_agreement_proposer(side)
-Side *side;
+do_agreement_proposer(Side *side)
 {
     DURING_GAME_ONLY(side);
 }
 
 void
-do_agreement_sign(side)
-Side *side;
+do_agreement_sign(Side *side)
 {
     DURING_GAME_ONLY(side);
 }
 
 void
-do_agreement_signer(side)
-Side *side;
+do_agreement_signer(Side *side)
 {
     DURING_GAME_ONLY(side);
 }
 
 void
-do_agreement_term(side)
-Side *side;
+do_agreement_term(Side *side)
 {
     int agid, agterm;
     char *arg, *rest, *term;
@@ -304,41 +357,78 @@ do_ai_side(Side *side)
     char *arg, *rest, *aitypename = NULL;
 
     DURING_GAME_ONLY(side);
+    /* Look at the optional command argument, extract options and/or
+       AI type name from it. */
     if (!empty_string(cmdargstr)) {
 	rest = get_next_arg(cmdargstr, tmpbuf, &arg);
-	if (strcmp(arg, "+") == 0) {
-	    /* Note that this flag is used by AIs only, so don't need
-	       to synchronize kernels here. */
+	if (strcmp(arg, "help") == 0
+	    || strcmp(arg, "?") == 0) {
+	    aitypename = next_ai_type_name(NULL);
+	    while (aitypename != NULL) {
+		notify(side, " %s - %s",
+		       aitypename, ai_type_help(find_ai_type(aitypename)));
+		aitypename = next_ai_type_name(aitypename);
+	    }
+	    return;
+	} else if (strcmp(arg, "+") == 0) {
+	    /* Note that this flag is used by local AIs only, so don't
+	       need to synchronize kernels here. */
 	    side->ai_may_resign = TRUE;
 	    notify(side, "AI may decide to draw or resign.");
-	    rest = get_next_arg(rest, tmpbuf, &arg);
+	    return;
 	} else if (strcmp(arg, "-") == 0) {
 	    side->ai_may_resign = FALSE;
 	    notify(side, "AI may only recommend whether to draw or resign.");
-	    rest = get_next_arg(rest, tmpbuf, &arg);
+	    return;
 	}
 	aitypename = arg;
     }
     if (empty_string(aitypename)) {
-    	/* Toggle AI between mplayer, iplayer and NULL. */
+    	/* Toggle AI between mplayer and NULL. */
+	/* (should toggle between "preferred AI" for the side) */
     	if (side->player->aitypename == NULL) {
-	    net_set_side_ai(side, "mplayer");
-	} else if (strcmp(side->player->aitypename, "mplayer") == 0) {
-	    net_set_side_ai(side, "iplayer");
+	    aitypename = "mplayer";
 	} else {
-	    net_set_side_ai(side, NULL);
+	    aitypename = NULL;
     	}
     } else if (strcmp(aitypename, "none") == 0) {
 	/* Special case to handle command "AI None" in Side menu. */
-	net_set_side_ai(side, NULL);
-    } else {
-	net_set_side_ai(side, aitypename);
+	aitypename = NULL;
     }
-    if (!empty_string(side->player->aitypename)) {
-	notify(side, "AI %s is running this side.", side->player->aitypename);
+    /* Feed back to user.  Note that we say "want to have" because we
+       haven't gotten feedback from remote programs yet. */
+    if (!empty_string(aitypename)) {
+	notify(side, "Want to have AI %s running this side.",
+	       aitypename);
     } else {
-	notify(side, "No AI is running this side.");
+	notify(side, "Want to have no AI running this side.");
     }
+    net_set_side_ai(side, aitypename);
+}
+
+void
+do_auto(Side *side)
+{
+    DURING_GAME_ONLY(side);
+    apply_to_all_selected_units(side, do_one_auto);
+}
+
+static int
+do_one_auto(Side *side, Unit *unit)
+{
+    int newval;
+
+    if (unit->plan == NULL)
+      return FALSE;
+    if (side->prefixarg < 0)
+      newval = !unit->plan->aicontrol;
+    else if (side->prefixarg == 0)
+      newval = FALSE;
+    else
+      newval = TRUE;
+    net_set_unit_ai_control(side, unit, newval, FALSE);
+    net_force_replan(side, unit, FALSE);
+    return TRUE;
 }
 
 void
@@ -389,6 +479,152 @@ do_c_rate(Side *side)
     notify(side, "%s conversion rates: %s", m_type_name(m), tbuf);
 }
 
+void
+do_clear_plan(Side *side)
+{
+    DURING_GAME_ONLY(side);
+    apply_to_all_selected_units(side, do_one_clear_plan);
+}
+
+static int
+do_one_clear_plan(Side *side, Unit *unit)
+{
+    if (unit->plan == NULL)
+      return FALSE;
+    net_set_unit_plan_type(side, unit, PLAN_NONE);
+    net_clear_task_agenda(side, unit);
+    return TRUE;
+}
+
+void
+do_delay(Side *side)
+{
+    DURING_GAME_ONLY(side);
+    apply_to_all_selected_units(side, do_one_delay);
+}
+
+static int
+do_one_delay(Side *side, Unit *unit)
+{
+    if (unit->plan == NULL)
+      return FALSE;
+    net_delay_unit(unit, TRUE);
+    return TRUE;
+#if 0 /* add this case from tkcmd.c? */
+	unit = find_next_awake_mover(side, map->curunit);
+	if (unit != map->curunit) {
+	    set_current_unit(map, unit);
+	} else {
+	    cmd_error(side, "No next awake mover found.");
+	}
+#endif
+}
+
+void
+do_detach(Side *side)
+{
+    DURING_GAME_ONLY(side);
+    apply_to_all_selected_units(side, do_one_detach);
+}
+
+static int
+do_one_detach(Side *side, Unit *unit)
+{
+    int rslt;
+
+    if (!completed(unit)) {
+	cmd_error(side, "%s is incomplete; cannot detach",
+		  unit_handle(side, unit));
+	return FALSE;
+    }
+    rslt = check_transfer_part_action(unit, unit, unit->hp / 2, NULL);
+    if (valid(rslt)) {
+	net_prep_transfer_part_action(unit, unit, unit->hp / 2, NULL);
+    } else {
+	notify(side, "can't detach for some reason?");
+    }
+    return TRUE;
+}
+
+void
+do_detonate(Side *side)
+{
+    DURING_GAME_ONLY(side);
+    /* (should add Mac version's choice of where to detonate?) */
+    apply_to_all_selected_units(side, do_one_detonate);
+}
+
+static int
+do_one_detonate(Side *side, Unit *unit)
+{
+    int rslt;
+
+    rslt = check_detonate_action(unit, unit, unit->x, unit->y, unit->z);
+    if (valid(rslt)) {
+	net_prep_detonate_action(unit, unit, unit->x, unit->y, unit->z);
+	return TRUE;
+    } else {
+	notify(side, "can't detonate for some reason?");
+	return FALSE;
+    }
+}
+
+/* Get rid of a unit. */
+
+void
+do_disband(Side *side)
+{
+    DURING_GAME_ONLY(side);
+    apply_to_all_selected_units(side, do_one_disband);
+}
+
+static int
+do_one_disband(Side *side, Unit *unit)
+{
+    /* (should have more feedback?) */
+    net_disband_unit(side, unit);
+    return TRUE;
+}
+
+void
+do_disembark(Side *side)
+{
+    DURING_GAME_ONLY(side);
+    apply_to_all_selected_units(side, do_one_disembark);
+}
+
+static int
+do_one_disembark(Side *side, Unit *unit)
+{
+    Unit *transport;
+
+    transport = unit->transport;
+    if (transport == NULL) {
+	cmd_error(side, "Not in a transport");
+	return FALSE;
+    }
+    if (!in_play(transport)) {
+	cmd_error(side, "Transport is nonsensical?");
+	return FALSE;
+    }
+    /* Try moving into the transport's transport, if there is one. */
+    if (transport->transport != NULL
+        && can_occupy(unit, transport->transport)) {
+	net_prep_enter_action(unit, unit, transport->transport);
+	/* (should be able to set up task if can't do action immedly) */
+	return TRUE;
+    }
+    /* Try moving into the open in the cell. */
+    if (can_occupy_cell(unit, unit->x, unit->y)
+	|| can_occupy_conn(unit, unit->x, unit->y, unit->z)) {
+	net_prep_move_action(unit, unit, unit->x, unit->y, unit->z);
+	/* (should be able to set up task if can't do action immedly) */
+	return TRUE;
+    }
+    cmd_error(side, "Can't disembark here!");
+    return FALSE;
+}
+
 void
 do_distrust(Side *side)
 {
@@ -472,8 +708,7 @@ do_doctrine(Side *side)
 }
 
 void
-do_down(side)
-Side *side;
+do_down(Side *side)
 {
     cmd_error(side, "not yet implemented");
 }
@@ -487,6 +722,27 @@ do_draw_willingness(Side *side)
     net_set_willing_to_draw(side, (side->prefixarg ? 1 : 0));
 }
 
+void
+do_embark(Side *side)
+{
+    DURING_GAME_ONLY(side);
+    apply_to_all_selected_units(side, do_one_embark);
+}
+
+static int
+do_one_embark(Side *side, Unit *unit)
+{
+    Unit *transport;
+
+    transport = embarkation_unit(unit);
+    if (transport != NULL) {
+	net_prep_enter_action(unit, unit, transport);
+	return TRUE;
+    }
+    cmd_error(side, "Nothing for %s to enter!", unit_handle(side, unit));
+    return FALSE;
+}
+
 /* Command to end the side's activity for the turn. */
 
 void
@@ -509,22 +765,79 @@ do_force_global_replan(Side *side)
     }
 }
 
+void
+do_give(Side *side)
+{
+    DURING_GAME_ONLY(side);
+    if (nummtypes == 0) {
+	cmd_error(side, "No materials in this game!");
+	return;
+    }
+    apply_to_all_selected_units(side, do_one_give);
+}
+
+static int
+do_one_give(Side *side, Unit *unit)
+{
+    short m, amts[MAXMTYPES], rslts[MAXMTYPES];
+    Unit *unit2;
+
+    for_all_material_types(m)
+      amts[m] = side->prefixarg;
+    unit2 = give_supplies(unit, amts, rslts);
+    report_give(side, unit, unit2, rslts);
+    return TRUE;
+}
+
+/* Tell the unit to sit around for a given number of turns. */
+
+void
+do_idle(Side *side)
+{
+    DURING_GAME_ONLY(side);
+    apply_to_all_selected_units(side, do_one_idle);
+}
+
+static int
+do_one_idle(Side *side, Unit *unit)
+{
+    if (side->prefixarg < 0)
+      side->prefixarg = 9999;
+    net_set_sentry_task(unit, side->prefixarg);
+    return TRUE;
+}
+
+/* Release a controlled side to act independently. */
+
 void
 do_release(Side *side)
 {
     Side *side2;
 
     DURING_GAME_ONLY(side);
-    if (!empty_string(cmdargstr)) {
-	side2 = parse_side_spec(cmdargstr);
-	if (side2 != NULL && side2 != side) {
-	    net_set_controlled_by(side2, side, FALSE);
-	    return;
-	}
+    if (empty_string(cmdargstr)) {
+	cmd_error(side, "No side given to release!");
+	return;
     }
-    cmd_error(side, "?");
+    side2 = parse_side_spec(cmdargstr);
+    if (side2 == NULL) {
+	cmd_error(side, "Can't get a side from \"%s\"!", cmdargstr);
+	return;
+    }
+    if (side2 == side) {
+	cmd_error(side, "Can't release ourselves!?");
+	return;
+    }
+    if (side2->controlled_by != side) {
+	cmd_error(side, "You don't control %s!", short_side_title(side2));
+	return;
+    }
+    /* Modify the controllee's bit, not the controller's. */
+    net_set_controlled_by(side2, side, FALSE);
 }
 
+/* Set a side's current research topic. */
+
 void
 do_research(Side *side)
 {
@@ -577,7 +890,7 @@ do_research(Side *side)
 	/* (should be a separate function perhaps) */
 	strcpy(abuf, "Next advances: ");
 	for_all_advance_types(a) {
-	    if (has_advance_to_research(side, a) && !has_advance(side, a)) {
+	    if (side_can_research(side, a)) {
 		if (i > 0)
 		  strcat(abuf, ", ");
 		strcat(abuf, a_type_name(a));
@@ -612,6 +925,44 @@ parse_advance_spec(char *str)
     return rslt;
 }
 
+/* Make the selected units sleep just for the remainder of the current
+   turn. */
+
+void
+do_reserve(Side *side)
+{
+    DURING_GAME_ONLY(side);
+    apply_to_all_selected_units(side, do_one_reserve);
+}
+
+static int
+do_one_reserve(Side *side, Unit *unit)
+{
+    net_set_unit_reserve(side, unit, TRUE, TRUE /* is this right??? */);
+    return TRUE;
+}
+
+/* Set up tasks to resupply the selected units. */
+
+/* (should warn if task is likely to fail) */
+
+void
+do_return(Side *side)
+{
+    DURING_GAME_ONLY(side);
+    apply_to_all_selected_units(side, do_one_return);
+}
+
+static int
+do_one_return(Side *side, Unit *unit)
+{
+    /* (should doublecheck range and error out if no chance) */
+    net_set_resupply_task(unit, NONMTYPE);
+    return TRUE;
+}
+
+/* Auto-finish turns, for the given number of turns. */
+
 void
 do_run(Side *side)
 {
@@ -633,9 +984,10 @@ do_run(Side *side)
     }
 }
 
+/* Set the rate at which AIs move their units. */
+
 void
-do_set_rate(side)
-Side *side;
+do_set_rate(Side *side)
 {
     int slow, fast;
     char *reststr;
@@ -648,19 +1000,76 @@ Side *side;
     }
 }
 
+/* Put a unit to sleep, which means it is inactive each turn until it
+   is woken up, either by some alarm going off (such as low supply) or
+   by explicit player action. */
+
+void
+do_sleep(Side *side)
+{
+    DURING_GAME_ONLY(side);
+    apply_to_all_selected_units(side, do_one_sleep);
+}
+
+static int
+do_one_sleep(Side *side, Unit *unit)
+{
+    net_set_unit_asleep(side, unit, TRUE, TRUE /* is this right??? */);
+    return TRUE;
+}
+
+/* Submit to (become controlled by) the specified side. */
+ 
 void
 do_submit(Side *side)
 {
     Side *side2;
 
     DURING_GAME_ONLY(side);
-    if (!empty_string(cmdargstr)) {
-	side2 = parse_side_spec(cmdargstr);
-	if (side2 != NULL && side2 != side) {
-	    net_set_controlled_by(side, side2, TRUE);
-	    return;
-	} 
+    if (side->controlled_by != NULL) {
+	cmd_error(side, "You are already controlled by %s!",
+		  short_side_title(side->controlled_by));
     }
+    if (empty_string(cmdargstr)) {
+	cmd_error(side, "No side given to submit to!");
+	return;
+    }
+    side2 = parse_side_spec(cmdargstr);
+    if (side2 == NULL) {
+	cmd_error(side, "Can't get a side from \"%s\"!", cmdargstr);
+	return;
+    }
+    if (side2 == side) {
+	cmd_error(side, "Can't submit to ourselves!?");
+	return;
+    }
+    net_set_controlled_by(side, side2, TRUE);
+    /* (should draw emblem inverted underneath controller's emblem) */
+}
+
+/* Take supplies from transport. */
+
+void
+do_take(Side *side)
+{
+    DURING_GAME_ONLY(side);
+    if (nummtypes == 0) {
+	cmd_error(side, "No materials in this game!");
+	return;
+    }
+    apply_to_all_selected_units(side, do_one_take);
+}
+
+static int
+do_one_take(Side *side, Unit *unit)
+{
+    short m, amts[MAXMTYPES], rslts[MAXMTYPES], needed;
+
+    for_all_material_types(m)
+      amts[m] = side->prefixarg;
+    needed = take_supplies(unit, amts, rslts);
+    report_take(side, unit, needed, rslts);
+    return TRUE;
 }
 
 void
@@ -685,6 +1094,42 @@ do_trust(Side *side)
     net_set_trust(side, side2, 1);
 }
 
+void
+do_wake(Side *side)
+{
+    DURING_GAME_ONLY(side);
+    apply_to_all_selected_units(side, do_one_wake);
+}
+
+static int
+do_one_wake(Side *side, Unit *unit)
+{
+    net_wake_unit(side, unit, FALSE);
+    /* If an argument was given, apply to all "top-level" units within
+       the radius specified by the argument. */
+    if (side->prefixarg >= 0)
+      net_wake_area(side, unit->x, unit->y, side->prefixarg, FALSE);
+    return 1;
+}
+
+void
+do_wake_all(Side *side)
+{
+    DURING_GAME_ONLY(side);
+    apply_to_all_selected_units(side, do_one_wake_all);
+}
+
+static int
+do_one_wake_all(Side *side, Unit *unit)
+{
+    net_wake_unit(side, unit, TRUE);
+    /* If an argument was given, apply to all units and occs within
+       the radius specified by the argument. */
+    if (side->prefixarg >= 0)
+      net_wake_area(side, unit->x, unit->y, side->prefixarg, TRUE);
+    return 1;
+}
+
 void
 do_warning_log(Side *side)
 {
