@@ -14,9 +14,10 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 extern short initially_no_ai;
+#include "kpublic.h"
 #include "cmdline.h"
-extern void make_default_player_spec(void);
 
+static void add_a_raw_player_spec(char *specstr);
 static void version_info(void);
 static void general_usage_info(void);
 static void game_usage_info(void);
@@ -24,13 +25,16 @@ static void player_usage_info(void);
 static void parse_world_option(char *str);
 static void parse_realtime_option(char *subopt, char *arg);
 static void parse_variant(char *str);
-static void add_a_raw_player_spec(char *specstr);
+static int find_variant_from_name(Module *module, char *name);
 
 /* The startup-settable options. */
 
-static int option_totalgametime;
-static int option_perturntime;
-static int option_persidetime;
+static int option_width;
+static int option_height;
+static int option_circumference;
+static int option_total_game_time;
+static int option_per_side_time;
+static int option_per_turn_time;
 static int option_add_default_player;
 
 char *option_game_to_host = NULL;
@@ -41,14 +45,9 @@ int option_num_to_wait_for = 0;
 
 static char *default_ai_type = ",mplayer";
 
-/* This says whether the command line arguments actually chose a
-   specific game to play (as opposed to setting random flags). */
-
-static int game_chosen = FALSE;
-
 /* Use this to record the options used to set up a game, so it can be
    reported to users. */
-/* (should flush? only useful for multi-X-display case) */
+/* (should flush? better to display in intelligible form) */
 
 char *args_used;
 
@@ -63,23 +62,6 @@ char *raw_default_player_spec;
 
 char *default_player_spec;
 
-/* The list of modules to be loaded in addition to the main one. */
-
-struct module_spec {
-  Module *module;
-  struct module_spec *next;
-} *extra_modules, *last_extra_module;
-
-/* The lists of GDL forms to interpret before and after loading. */
-
-struct gdl_spec {
-  char *spec;
-  struct gdl_spec *next;
-} *pre_gdl_specs, *last_pre_gdl_spec, *post_gdl_specs, *last_post_gdl_spec;
-
-static void add_a_gdl_spec(char *specstr, struct gdl_spec **first,
-			   struct gdl_spec **last);
-
 /* The list of accumulated variant choices. */
 
 static Obj *variant_settings;
@@ -193,7 +175,7 @@ parse_command_line(int argc, char *argv[], int spec)
 		if (strlen(arg) > 2) {
 		    aispec = arg + 2;
 		    if (*aispec != ',') {
-			sprintf(tmpspec, ",mplayer%s", aispec);
+			sprintf(tmpspec, "%s%s", default_ai_type, aispec);
 			aispec = tmpspec;
 		    }
 		} else {
@@ -215,7 +197,7 @@ parse_command_line(int argc, char *argv[], int spec)
 		n = atoi(argv[i+1]);
 		option_num_to_wait_for += n;
 		while (n-- > 0)
-		  add_a_raw_player_spec("x@");
+		  add_a_raw_player_spec("?@");
 	    } else if (strcmp(arg, "-help") == 0) {
 		GENERAL_OPTION;
 		help_wanted = TRUE;
@@ -224,12 +206,10 @@ parse_command_line(int argc, char *argv[], int spec)
 		REQUIRE_ONE_ARG;
 		GENERAL_OPTION;
 		option_game_to_host = copy_string(argv[i+1]);
-		fprintf(stderr, "-host is currently broken, use interactive setup for network games\n");
 	    } else if (strcmp(arg, "-join") == 0) {
 		REQUIRE_ONE_ARG;
 		GENERAL_OPTION;
 		option_game_to_join = copy_string(argv[i+1]);
-		fprintf(stderr, "-join is currently broken, use interactive setup for network games\n");
 	    } else if (strcmp(arg, "-L") == 0) {
 		REQUIRE_ONE_ARG;
 		GENERAL_OPTION;
@@ -244,14 +224,6 @@ parse_command_line(int argc, char *argv[], int spec)
 	    } else if (strcmp(arg, "-noai") == 0) {
 		PLAYER_OPTION;
 		initially_no_ai = TRUE;
-	    } else if (strcmp(arg, "-pre") == 0) {
-		REQUIRE_ONE_ARG;
-		GENERAL_OPTION;
-		add_a_gdl_spec(argv[i+1], &pre_gdl_specs, &last_pre_gdl_spec);
-	    } else if (strcmp(arg, "-post") == 0) {
-		REQUIRE_ONE_ARG;
-		GENERAL_OPTION;
-		add_a_gdl_spec(argv[i+1], &post_gdl_specs, &last_post_gdl_spec);
 	    } else if (strcmp(arg, "-r") == 0) {
 		PLAYER_OPTION;
 		option_add_default_player = FALSE;
@@ -370,26 +342,15 @@ void
 add_a_module(char *name, char *filename)
 {
     Module *module;
-    struct module_spec *extra;
 
     module = get_game_module(name);
     if (module == NULL)
       exit(1);  /* bad error if happens */
     if (filename)
       module->filename = copy_string(filename);
-    if (mainmodule == NULL) {
-	mainmodule = module;
-    } else {
-	extra = (struct module_spec *) xmalloc(sizeof(struct module_spec));
-	extra->module = module;
-	/* Add the module at the end of the list. */
-	if (extra_modules == NULL)
-	  extra_modules = extra;
-	else
-	  last_extra_module->next = extra;
-	last_extra_module = extra;
-    }
-    game_chosen = TRUE;
+    if (mainmodule != NULL)
+      printf("Warning: game module already specified, will replace");
+    mainmodule = module;
 }
 
 /* Glue an unprocessed player spec onto the list of other specs found
@@ -409,21 +370,6 @@ add_a_raw_player_spec(char *specstr)
     last_raw_player_spec = spec;
 }
 
-static void
-add_a_gdl_spec(char *specstr, struct gdl_spec **firstp,
-	       struct gdl_spec **lastp)
-{
-    struct gdl_spec *spec =
-      (struct gdl_spec *) xmalloc (sizeof(struct gdl_spec));
-
-    spec->spec = copy_string(specstr);
-    if (*firstp == NULL)
-      *firstp = spec;
-    else
-      (*lastp)->next = spec;
-    *lastp = spec;
-}
-
 static void
 version_info(void)
 {
@@ -451,8 +397,8 @@ general_usage_info(void)
     printf(" (not available)");
 #endif
     printf("\n");
-    /* We don't display the debugging options -D*, -pre, -post, and
-       -R, on the theory that only hackers should try to use them. */
+    /* We don't display the debugging options -D* and -R, on the
+       theory that only hackers should try to use them. */
     /* Group the long options together, but don't identify them
        specially. */
     printf("    --help\t\tdisplay this help info\n");
@@ -599,46 +545,33 @@ player_usage_info(void)
 }
 
 /* Given a string representing world dimensions, extract various
-   components and compose a variant setting. */
+   components. */
 
 static void
 parse_world_option(char *str)
 {
-    int width, height, circumference;
     char *str2;
-    Obj *varval;
 
-    width = atoi(str);
+    option_width = atoi(str);
+    option_height = option_width;
     str2 = strchr(str, 'x');
-    if (str2 != NULL) {
-	height = atoi(str2 + 1);
-    } else {
-	height = width;
-    }
-    varval = lispnil;
+    if (str2 != NULL)
+      option_height = atoi(str2 + 1);
     /* Add a world circumference setting if present. */
     str2 = strchr(str, 'W');
-    if (str2 != NULL) {
-	circumference = atoi(str2 + 1);
-	if (circumference > 0)
-	  varval = cons(new_number(circumference), varval);
-	else
-	  fprintf(stderr, "Requested circumference is bad, ignoring\n");
-    }
-    varval = cons(new_number(width), cons(new_number(height), varval));
-    /* Glue onto the list of variant_settings. */
-    push_key_cdr_binding(&variant_settings, K_WORLD_SIZE, varval);
+    if (str2 != NULL)
+      option_circumference = atoi(str2 + 1);
 }
 
 static void
 parse_realtime_option(char *subopt, char *arg)
 {
     if (strcmp(subopt, "-tgame") == 0) {
-	option_totalgametime = 60 * atoi(arg);
+	option_total_game_time = 60 * atoi(arg);
     } else if (strcmp(subopt, "-tside") == 0) {
-	option_persidetime = 60 * atoi(arg);
+	option_per_side_time = 60 * atoi(arg);
     } else if (strcmp(subopt, "-tturn") == 0) {
-	option_perturntime = 60 * atoi(arg);
+	option_per_turn_time = 60 * atoi(arg);
     } else {
 	fprintf(stderr, "Realtime option not one of -tgame, -tside, -tturn, ignoring\n");
     }
@@ -673,33 +606,20 @@ parse_variant(char *str)
     }
 }
 
-/* Load all the game modules that were asked for on cmd line. */
+/* Load the game module asked for on the cmd line, or else the
+   default. */
 
 void
 load_all_modules(void)
 {
-    struct module_spec *extra;
-    struct gdl_spec *gdlspec;
-    Obj *gdlform;
-
-    for (gdlspec = pre_gdl_specs; gdlspec != NULL; gdlspec = gdlspec->next) {
-	gdlform = read_form_from_string(gdlspec->spec, NULL, NULL, NULL);
-	interp_form(NULL, gdlform);
-    }
     if (mainmodule != NULL) {
 	load_game_module(mainmodule, TRUE);
-	for (extra = extra_modules; extra != NULL; extra = extra->next) {
-	    load_game_module(extra->module, TRUE);
-	}
     } else {
-	/* If we've got absolutely no files to load, the standard game is
-	   the one to go for.  It will direct the remainder of random gen. */
+	/* If we've got absolutely no files to load, the standard game
+	   is the one to go for.  It will direct the remainder of
+	   random generation. */
 	load_default_game();
     }
-    for (gdlspec = post_gdl_specs; gdlspec != NULL; gdlspec = gdlspec->next) {
-	gdlform = read_form_from_string(gdlspec->spec, NULL, NULL, NULL);
-	interp_form(NULL, gdlform);
-    }
 }
 
 /* Set module variants from command line options. */
@@ -707,25 +627,65 @@ load_all_modules(void)
 void
 set_variants_from_options(void)
 {
-    Obj *varval;
+    int which;
+    char *varname;
+    Obj *varrest, *varset;
 
     /* Only the host of a networked game can set variants. */
     if (option_game_to_join != NULL) {
-	if (variant_settings != lispnil)
+	if (variant_settings != lispnil
+	    || option_width > 0)
 	  fprintf(stderr, "Not the host, ignoring variant settings\n");
 	return;
     }
-    /* Build and add a real-time variant if any times were supplied. */
-    if (option_totalgametime != 0
-	|| option_persidetime != 0
-	|| option_perturntime != 0) {
-	varval = cons(new_number(option_totalgametime),
-		      cons(new_number(option_persidetime),
-			   cons(new_number(option_perturntime), lispnil)));
-	/* Glue onto the list of variant_settings. */
-	push_key_cdr_binding(&variant_settings, K_REAL_TIME, varval);
+    if (option_width > 0) {
+	which = find_variant_from_name(mainmodule, keyword_name(K_WORLD_SIZE));
+	if (which >= 0)
+	  net_set_variant_value(which, option_width, option_height,
+				option_circumference);
+	else
+	  fprintf(stderr, "World size variant not available, -M ignored\n");
     }
-    do_module_variants(mainmodule, variant_settings);
+    /* Set the real-time variant if any times were supplied. */
+    if (option_total_game_time != 0
+	|| option_per_side_time != 0
+	|| option_per_turn_time != 0) {
+	which = find_variant_from_name(mainmodule, keyword_name(K_REAL_TIME));
+	if (which >= 0)
+	  net_set_variant_value(which, option_total_game_time,
+				option_per_side_time, option_per_turn_time);
+	else
+	  fprintf(stderr, "Real time variants not available, ignored\n");
+    }
+    for_all_list(variant_settings, varrest) {
+	varset = car(varrest);
+	varname = c_string(car(varset));
+	which = find_variant_from_name(mainmodule, varname);
+	if (which >= 0)
+	  net_set_variant_value(which, c_number(cadr(varset)), 0, 0);
+	else
+	  fprintf(stderr, "No variant `%s' known, ignored\n", varname);
+    }
+    do_module_variants(mainmodule, lispnil);
+}
+
+/* Given a string, return the index of the variant for the module, or
+   -1 to indicate that it is not the name of a valid variant. */
+
+static int
+find_variant_from_name(Module *module, char *name)
+{
+    int i;
+    Variant *var;
+
+    if (module->variants == NULL)
+      return -1;
+    for (i = 0; module->variants[i].id != lispnil; ++i) {
+	var = &(module->variants[i]);
+	if (strcmp(name, c_string(var->id)) == 0)
+	  return i;
+    }
+    return -1;
 }
 
 /* Set player characteristics from command-line options. */
@@ -747,13 +707,14 @@ set_players_from_options(void)
        game of some sort, and merge instead of adding new players. */
     mergespecs = (numplayers > 0);
     /* Add a player for the indepside if none found. */
-    if (find_player(0) == NULL)
+    if (indepside_needed() && find_player(0) == NULL)
       player = add_player();
     /* If we're merging specs, skip over the indepside player. */
     if (mergespecs)
       player = playerlist->next;
     /* Add the default player. */
-    if (raw_player_specs == NULL || option_add_default_player) {
+    if (option_game_to_host == NULL
+	&& (raw_player_specs == NULL || option_add_default_player)) {
 	if (!mergespecs)
 	  player = add_default_player();
 	parse_player_spec(player, raw_default_player_spec);
@@ -782,6 +743,12 @@ set_players_from_options(void)
 	    player->aitypename = NULL;
 	}
     }
+    /* Transmit the official list of players to everybody else. */
+    if (my_rid > 0 && my_rid == master_rid) {
+	for_all_players(player) {
+	    net_update_player(player);
+	}
+    }
 }
 
 /* This is not, strictly speaking, part of command line processing,
