@@ -1,5 +1,5 @@
 /* Command-line parsing definitions for Xconq.
-   Copyright (C) 1993, 1994, 1995, 1999 Stanley T. Shebs.
+   Copyright (C) 1993, 1994, 1995, 1999, 2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -17,11 +17,13 @@ enum parsestage {
 /* Global variables that command-line options tweak. */
 
 extern int checkpointinterval;
-extern int allbedesigners;
 extern int option_popup_new_game_dialog;
 extern char *option_game_to_host;
 extern char *option_game_to_join;
 extern int option_num_to_wait_for;
+extern char *default_player_spec;
+extern char *raw_default_player_spec;
+extern char *args_used;
 
 extern void init_options(void);
 extern void parse_command_line(int argc, char **argv, int spec);
@@ -30,3 +32,5 @@ extern void load_all_modules(void);
 extern void set_variants_from_options(void);
 extern void set_players_from_options(void);
 extern void print_instructions(void);
+
+extern void make_default_player_spec(void);
