@@ -8,7 +8,6 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 #include "kernel.h"
-#include "kpublic.h"
 
 static int one_attack(Unit *atker, Unit *defender);
 static void fire_on_unit(Unit *atker, Unit *other);
@@ -16,6 +15,8 @@ static void attack_unit(Unit *atker, Unit *other);
 static void maybe_hit_unit(Unit *atker, Unit *other, int fire, int fallsoff);
 static void hit_unit(Unit *unit, int hit, Unit *atker);
 static void model_1_attack(Unit *unit, int x, int y);
+static int real_attack_value(Unit *unit);
+static int real_defense_value(Unit *unit);
 
 static void reckon_damage(int fire);
 static void reckon_damage_here(int x, int y);
@@ -29,6 +30,12 @@ static void capture_unit_2(Unit *unit, Unit *pris, Side *prevside);
 static void capture_occupant(Unit *unit, Unit *pris, Unit *occ, Side *newside);
 static void detonate_on_cell(int x, int y);
 static void hit_unit_with_detonation(Unit *unit, int hit, Unit *atker);
+static int found_blocking_elevation(int u, int ux, int uy, int uz, int u2, int u2x, int u2y, int u2z);
+static Unit *mobile_enemy_threat(Unit *unit, int range);
+static int enough_ammo(Unit *unit, Unit *other);
+
+static void damage_terrain(int u, int x, int y);
+static int damaged_terrain_type(int t);
 
 /* We can't declare all the action functions as static because some of them
    are in other files, but don't let them be visible to all files. */
@@ -68,16 +75,6 @@ prep_attack_action(Unit *unit, Unit *unit2, Unit *defender, int n)
     return TRUE;
 }
 
-int
-net_prep_attack_action(Unit *unit, Unit *unit2, Unit *defender, int n)
-{
-    int rslt;
-
-    rslt = prep_attack_action(unit, unit2, defender, n);
-    broadcast_next_action(unit);
-    return rslt;
-}
-
 int
 do_attack_action(Unit *unit, Unit *unit2, Unit *defender, int n)
 {
@@ -165,7 +162,7 @@ check_attack_action(Unit *unit, Unit *unit2, Unit *defender, int n)
     }
     /* (should prorate ammo needs by intensity of attack) */
     if (!enough_ammo(unit2, defender))
-      return A_ANY_ERROR;
+      return A_ANY_NO_AMMO;
     /* Allow attacks even if zero damage, this amounts to "harassment". */
     return A_ANY_OK;
 }
@@ -189,16 +186,6 @@ prep_overrun_action(Unit *unit, Unit *unit2, int x, int y, int z, int n)
     return TRUE;
 }
 
-int
-net_prep_overrun_action(Unit *unit, Unit *unit2, int x, int y, int z, int n)
-{
-    int rslt;
-
-    rslt = prep_overrun_action(unit, unit2, x, y, z, n);
-    broadcast_next_action(unit);
-    return rslt;
-}
-
 int
 do_overrun_action(Unit *unit, Unit *unit2, int x, int y, int z, int n)
 {
@@ -254,7 +241,7 @@ do_overrun_action(Unit *unit, Unit *unit2, int x, int y, int z, int n)
 		  use_up_acp(unit, uu_acp_to_attack(u2, u3));
 		acpused += uu_acp_to_attack(u2, u3);
 		/* The target of an attack has to take time to defend
-	       itself. */
+		   itself. */
 		if (!gotonext && alive(defender)) {
 		    use_up_acp(defender, uu_acp_to_defend(u, u3));
 		}
@@ -349,7 +336,7 @@ check_overrun_action(Unit *unit, Unit *unit2, int x, int y, int z, int n)
 	    /* (should test if units here can be attacked en masse) */
 	    /* (should prorate ammo needs by intensity of overrun) */
 	    if (!enough_ammo(unit2, defender))
-	      return A_ANY_NO_MATERIAL;
+	      return A_ANY_NO_AMMO;
 	}
 	break;
       case 1:
@@ -487,8 +474,7 @@ model_1_attack(Unit *unit, int x, int y)
 	if (unit_trusts_unit(unit, unit2))
 	  continue;
 	/* Identify the best defender. */
-	u2 = unit2->type;
-	def = u_defend(u2);
+	def = real_defense_value(unit2);
 	if (def > maxdef) {
 	    maxdef = def;
 	    defender = unit2;
@@ -499,7 +485,9 @@ model_1_attack(Unit *unit, int x, int y)
       return;
     /* For cities, find a defender. */
     if (u_advanced(defender->type)) {
-	maxdef = -1;
+        /* We don't want defenseless occs (facilities) to count as defenders,
+         * so initialize maxdefs to 0 instead of -1. */
+	maxdef = 0;
 	defocc = NULL;
 	for_all_occupants(defender, occ) {
 	    u2 = occ->type;
@@ -519,8 +507,8 @@ model_1_attack(Unit *unit, int x, int y)
     d = defender->type;
     limit = 100;
     while (alive(unit) && alive(defender) && limit-- > 0) {
-	att = u_attack(u);
-	def = u_defend(d);
+	att = real_attack_value(unit);
+	def = real_defense_value(defender);
 	winround = probability((att * 100) / (att + def));
 	if (winround) {
 	    dmg = roll_dice(uu_damage(u, d));
@@ -575,6 +563,118 @@ model_1_attack(Unit *unit, int x, int y)
     }
 }
 
+/* The real attack value of a unit after adjustments for combat exp, 
+terrain effects, occupants etc. */
+
+int
+real_attack_value(Unit *unit)
+{
+	Unit *unit2;
+	int t, u, att, effect;
+	
+	u = unit->type;
+	att = u_attack(u);
+	/* First factor in combat experience. */
+	if (u_cxp_max(u) > 0 && unit->cxp > 0) {
+		effect = u_full_cxp_affects_attack(u);
+		if (effect != 100) {
+			att *= (100 + (effect - 100) * unit->cxp / u_cxp_max(u)) / 100;
+		}
+	}
+	/* Then factor in terrain effects. */
+	t = terrain_at(unit->x, unit->y);
+	effect = ut_terrain_affects_attack(u, t);
+	if (effect != 100) {
+		att *= effect / 100;
+	}
+	/* Then factor in effect of transport. */
+	if (unit->transport) {
+		if (is_active(unit->transport)) {
+			effect = uu_transport_affects_attack(unit->transport->type, u);			
+			if (effect != 100) {
+				att *= effect / 100;
+			}
+		}
+	}	
+	/* Then factor in effect of occupants. */
+	for_all_occupants(unit, unit2) {
+		if (is_active(unit2)) {
+			effect = uu_occ_affects_attack(unit2->type, u);			
+			if (effect != 100) {
+				att *= effect / 100;
+			}
+		}
+	}	
+	/* Then factor in effect of friendly units in the same cell. */
+	for_all_stack_with_occs(unit->x, unit->y, unit2) {
+		if (is_active(unit2)
+		    && trusted_side(unit2->side, unit->side)) {
+			effect = uu_neighbour_affects_attack(unit2->type, u);			
+			if (effect != 100) {
+				att *= effect / 100;
+			}
+		}
+	}
+	return att;
+}
+
+
+/* The real defense value of a unit after adjustments for combat exp, 
+terrain effects, occupants etc. */
+
+int
+real_defense_value(Unit *unit)
+{
+	Unit *unit2;
+	int t, u, def, effect;
+	
+	u = unit->type;
+	def = u_defend(u);
+	/* First factor in combat experience. */
+	if (u_cxp_max(u) > 0 && unit->cxp > 0) {
+		effect = u_full_cxp_affects_defense(u);
+		if (effect != 100) {
+			def *= (100 + (effect - 100) * unit->cxp / u_cxp_max(u)) / 100;
+		}
+	}
+	/* Then factor in terrain effects. */
+	t = terrain_at(unit->x, unit->y);
+	effect = ut_terrain_affects_defense(u, t);
+	if (effect != 100) {
+		def *= effect / 100;
+	}
+	/* Then factor in effect of transport. */
+	if (unit->transport) {
+		if (is_active(unit->transport)) {
+			effect = uu_transport_affects_defense(unit->transport->type, u);			
+			if (effect != 100) {
+				def *= effect / 100;
+			}
+		}
+	}	
+	/* Then factor in effect of occupants. */
+	for_all_occupants(unit, unit2) {
+		if (is_active(unit2)) {
+			effect = uu_occ_affects_defense(unit2->type, u);			
+			if (effect != 100) {
+				def *= effect / 100;
+			}
+		}
+	}	
+	/* Then factor in effect of friendly units in the same cell. */
+	for_all_stack_with_occs(unit->x, unit->y, unit2) {
+		if (is_active(unit2)
+		    && trusted_side(unit2->side, unit->side)) {
+			effect = uu_neighbour_affects_defense(unit2->type, u);			
+			if (effect != 100) {
+				def *= effect / 100;
+			}
+		}
+	}
+	return def;
+}
+
+
 /* Fire-at action. */
 
 /* Shooting at a given unit. */
@@ -591,16 +691,6 @@ prep_fire_at_action(Unit *unit, Unit *unit2, Unit *unit3, int m)
     return TRUE;
 }
 
-int
-net_prep_fire_at_action(Unit *unit, Unit *unit2, Unit *unit3, int m)
-{
-    int rslt;
-
-    rslt = prep_fire_at_action(unit, unit2, unit3, m);
-    broadcast_next_action(unit);
-    return rslt;
-}
-
 int
 do_fire_at_action(Unit *unit, Unit *unit2, Unit *unit3, int m)
 {
@@ -613,17 +703,17 @@ do_fire_at_action(Unit *unit, Unit *unit2, Unit *unit3, int m)
     /* Make up the list of sides that will see the fire. */
     sidemask = NOSIDES;
     for_all_sides(side) {
-    /* Let all sides that can see the attacker's cell see it. */
+	/* Let all sides that can see the attacker's cell see it. */
 	if (units_visible(side, unit2->x, unit2->y))
-		sidemask = add_side_to_set(side, sidemask);
-    /* Let all sides that can see the unit3's cell see it. */
+	  sidemask = add_side_to_set(side, sidemask);
+	/* Let all sides that can see the unit3's cell see it. */
 	if (units_visible(side, unit3->x, unit3->y))
-		sidemask = add_side_to_set(side, sidemask);
+	  sidemask = add_side_to_set(side, sidemask);
     }
     /* Show the fire. */
     for_all_sides(side) {
 	if (side_in_set(side, sidemask))
-		update_fire_at_display(side, unit, unit3, m, TRUE);
+	  update_fire_at_display(side, unit, unit3, m, TRUE);
     }
     ox = unit3->x;  oy = unit3->y;  oz = unit3->z;
     amain = unit;  omain = unit3;
@@ -707,8 +797,9 @@ check_fire_at_action(Unit *unit, Unit *unit2, Unit *unit3, int m)
     if ((uu_fire_hit(u2, u3) != -1 ? uu_fire_hit(u2, u3) : uu_hit(u2, u3)) == 0)
       return A_ANY_ERROR;
     /* Check intervening elevations. */
-    if (found_blocking_elevation(u2, ux, uy, uz, u3, unit3->x, unit3->y, unit3->z))
-      return A_ANY_ERROR;
+    if (found_blocking_elevation(u2, ux, uy, uz, u3,
+				 unit3->x, unit3->y, unit3->z))
+      return A_FIRE_BLOCKED;
     /* We have to have a minimum level of supply to be able to attack. */
     for_all_material_types(m2) {
 	if (unit2->supply[m2] < um_to_fire(u2, m2))
@@ -717,10 +808,10 @@ check_fire_at_action(Unit *unit, Unit *unit2, Unit *unit3, int m)
     /* Check for enough of right kind of ammo. */
     if (is_material_type(m)) {
     	if (unit->supply[m] == 0)
-    	  return A_ANY_NO_MATERIAL;
+    	  return A_ANY_NO_AMMO;
     } else {
     	if (!enough_ammo(unit2, unit3))
-    	  return A_ANY_NO_MATERIAL;
+    	  return A_ANY_NO_AMMO;
     }
     return A_ANY_OK;
 }
@@ -743,16 +834,6 @@ prep_fire_into_action(Unit *unit, Unit *unit2, int x, int y, int z, int m)
     return TRUE;
 }
 
-int
-net_prep_fire_into_action(Unit *unit, Unit *unit2, int x, int y, int z, int m)
-{
-    int rslt;
-
-    rslt = prep_fire_into_action(unit, unit2, x, y, z, m);
-    broadcast_next_action(unit);
-    return rslt;
-}
-
 /* One can always shoot, if the cell is visible, but there might not
    not be anything to hit!  No counterattacks when shooting, and the
    results might not be visible to the shooter. */
@@ -768,12 +849,12 @@ do_fire_into_action(Unit *unit, Unit *unit2, int x, int y, int z, int m)
     /* Make up the list of sides that will see the fire. */
     sidemask = NOSIDES;
     for_all_sides(side) {
-    /* Let all sides that can see the attacker's cell see it. */
+	/* Let all sides that can see the attacker's cell see it. */
 	if (units_visible(side, unit2->x, unit2->y))
-		sidemask = add_side_to_set(side, sidemask);
-    /* Let all sides that can see the fired-into cell see it. */
+	  sidemask = add_side_to_set(side, sidemask);
+	/* Let all sides that can see the fired-into cell see it. */
 	if (units_visible(side, x, y))
-		sidemask = add_side_to_set(side, sidemask);
+	  sidemask = add_side_to_set(side, sidemask);
     }
     /* Show the fire. */
     for_all_sides(side) {
@@ -846,7 +927,7 @@ check_fire_into_action(Unit *unit, Unit *unit2, int x, int y, int z, int m)
       return A_ANY_TOO_NEAR;
     /* Check intervening elevations and terrain. */
     if (found_blocking_elevation(u2, u2x, u2y, u2z, NONUTYPE, x, y, z))
-      return A_ANY_ERROR;
+      return A_FIRE_BLOCKED;
     /* We have to have a minimum level of supply to be able to attack. */
     for_all_material_types(m2) {
 	if (unit2->supply[m2] < um_to_fire(u2, m2))
@@ -855,7 +936,7 @@ check_fire_into_action(Unit *unit, Unit *unit2, int x, int y, int z, int m)
     /* Check for enough of right kind of ammo. */
     if (is_material_type(m)) {
     	if (unit->supply[m] == 0)
-    	  return A_ANY_NO_MATERIAL;
+    	  return A_ANY_NO_AMMO;
     } else {
     	/* should just assume amount is appropriate? */
     }
@@ -996,7 +1077,7 @@ static void
 maybe_hit_unit(Unit *atker, Unit *other, int fire, int fallsoff)
 {
     int chance, t, hit = 0, a = atker->type, o = other->type;
-    int cxpeffect, cxpmax, effect, prot;
+    int teffect, cxpeffect, cxpmax, effect, prot;
     int dist, disthit, rangedelta, hitdelta, rangeamt;
     int dmgspec;
     char *hitmovietype;
@@ -1009,7 +1090,8 @@ maybe_hit_unit(Unit *atker, Unit *other, int fire, int fallsoff)
       chance = uu_fire_hit(a, o);
     else
       chance = uu_hit(a, o);
-    /* Combat experience tends to raise the hit chance, so do that first. */
+    /* Combat experience tends to raise the hit chance, so do that
+       first, reduces roundoff weirdnesses later. */
     cxpmax = u_cxp_max(a);
     if (cxpmax > 0 && atker->cxp > 0) {
 	cxpeffect = uu_hit_cxp(a, o);
@@ -1021,10 +1103,18 @@ maybe_hit_unit(Unit *atker, Unit *other, int fire, int fallsoff)
     /* (should modify due to cxp of defender too) */
     /* Account for terrain effects. */
     t = terrain_at(atker->x, atker->y);
-    chance = (chance * ut_attack_terrain_effect(a, t)) / 100;
+    if (fire && ut_fire_attack_terrain_effect(a, t) != -1)
+      teffect = ut_fire_attack_terrain_effect(a, t);
+    else
+      teffect = ut_attack_terrain_effect(a, t);
+    chance = (chance * teffect) / 100;
     t = terrain_at(other->x, other->y);
-    chance = (chance * ut_defend_terrain_effect(o, t)) / 100;
-    /* Account for protective units nearby. */
+    if (fire && ut_fire_defend_terrain_effect(o, t) != -1)
+      teffect = ut_fire_defend_terrain_effect(o, t);
+    else
+      teffect = ut_defend_terrain_effect(o, t);
+    chance = (chance * teffect) / 100;
+    /* Account for protection by occupants. */
     for_all_occupants(other, occ) {
 	if (in_play(occ) && completed(occ)) {
 	    prot = uu_protection(occ->type, o);
@@ -1032,6 +1122,7 @@ maybe_hit_unit(Unit *atker, Unit *other, int fire, int fallsoff)
 	      chance = (chance * prot) / 100;
 	}
     }
+    /* Account for protection by neighbours. */
     for_all_stack(other->x, other->y, unit2) {
 	if (unit2 != other
 	    && in_play(unit2)
@@ -1170,6 +1261,7 @@ static void
 hit_unit(Unit *unit, int hit, Unit *atker)
 {
     int u = unit->type, u2, hpmin, dmgspec, tpdmg, tp;
+    Side *aside;
 
     /* Some units might detonate automatically upon being hit. */
     if (hit > 0
@@ -1177,17 +1269,17 @@ hit_unit(Unit *unit, int hit, Unit *atker)
         && probability(uu_detonate_on_hit(u, atker->type))
         && !was_detonated(unit)) {
     	detonate_unit(unit, unit->x, unit->y, unit->z);
-    	/* If the detonating unit still exists, then continue
-    	   on to normal damage computation. */
+    	/* If the detonating unit still exists, then continue on to
+    	   normal damage computation. */
     }
     if (hit > 0) {
 	/* Record the loss of hp. */
 	unit->hp2 -= hit;
 	if (atker != NULL) {
-	    /* Attacker might not be able to do any more damage.  Note that the
-	       positioning of this code is such that all the usual side effects
-	       of combat happen, but the victim doesn't get any more worse off
-	       than it is already. */
+	    /* Attacker might not be able to do any more damage.  Note
+	       that the positioning of this code is such that all the
+	       usual side effects of combat happen, but the victim
+	       doesn't get any more worse off than it is already. */
 	    hpmin = uu_hp_min(atker->type, u);
 	    if (hpmin > 0 && hpmin > unit->hp2)
 	      unit->hp2 = hpmin;
@@ -1209,13 +1301,15 @@ hit_unit(Unit *unit, int hit, Unit *atker)
     }
     /* Maybe record for statistical analysis. */
     /* (this is only useful if code always goes through here - is that true?) */
-    if (atker != NULL && atker->side != NULL) {
-	if (atker->side->atkstats[atker->type] == NULL)
-	  atker->side->atkstats[atker->type] = (long *) xmalloc(numutypes * sizeof(long));
-	if (atker->side->hitstats[atker->type] == NULL)
-	  atker->side->hitstats[atker->type] = (long *) xmalloc(numutypes * sizeof(long));
-	++((atker->side->atkstats[atker->type])[u]);
-	(atker->side->hitstats[atker->type])[u] += hit;
+    if (atker != NULL) {
+	aside = atker->side;
+	u2 = atker->type;
+	if (aside->atkstats[u2] == NULL)
+	  aside->atkstats[u2] = (long *) xmalloc(numutypes * sizeof(long));
+	if (aside->hitstats[u2] == NULL)
+	  aside->hitstats[u2] = (long *) xmalloc(numutypes * sizeof(long));
+	++((aside->atkstats[u2])[u]);
+	(aside->hitstats[u2])[u] += hit;
     }
     /* Some units may detonate automatically just before dying. */
     if (hit > 0
@@ -1274,11 +1368,18 @@ reckon_damage_around(int x, int y, int r)
     }
 }
 
+/* Given the units involved in combat, decide how to report it.  The
+   strings like "destroy" and "miss" are symbolic, don't normally
+   appear in the final output. */
+
 static void
 report_damage(Unit *unit, Unit *atker, Unit *mainunit)
 {
     Unit *occ;
 
+    /* Don't report supposed actions by dead units. */
+    if (!alive(atker))
+      return;
     if (unit->hp2 <= 0) {
 	if (unit == mainunit) {
 	    report_combat(atker, unit, "destroy");
@@ -1303,6 +1404,9 @@ report_damage(Unit *unit, Unit *atker, Unit *mainunit)
     }
 }
 
+/* Return true if the given unit or one of its occupants has been
+   damaged. */
+
 static int
 will_report_damage(Unit *unit)
 {
@@ -1332,8 +1436,10 @@ damage_unit(Unit *unit, enum damage_reasons dmgreason, Unit *agent)
 	damage_unit(occ, dmgreason, agent);
     }
     /* If no damage was recorded, just return. */
-    if (unit->hp2 == unit->hp)
-      return;
+    if (unit->hp2 == unit->hp) {
+	set_was_detonated(unit, FALSE);
+	return;
+    }
     /* If unit is to die, do the consequences. */
     if (unit->hp2 <= 0) {
 	if (u_wrecked_type(unit->type) == NONUTYPE) {
@@ -1381,6 +1487,8 @@ damage_unit(Unit *unit, enum damage_reasons dmgreason, Unit *agent)
     update_unit_display(unit->side, unit, TRUE);
 }
 
+/* Occupants may be able to avoid the fate of their transport. */
+
 void
 rescue_occupants(Unit *unit)
 {
@@ -1453,6 +1561,8 @@ retreat_unit(Unit *unit, Unit *atker)
     return FALSE;
 }
 
+/* Try to beat a retreat in the given direction. */
+
 static int
 retreat_in_dir(Unit *unit, int dir)
 {
@@ -1500,16 +1610,6 @@ prep_capture_action(Unit *unit, Unit *unit2, Unit *unit3)
     return TRUE;
 }
 
-int
-net_prep_capture_action(Unit *unit, Unit *unit2, Unit *unit3)
-{
-    int rslt;
-
-    rslt = prep_capture_action(unit, unit2, unit3);
-    broadcast_next_action(unit);
-    return rslt;
-}
-
 /* Execute a capture action. */
 
 int
@@ -1871,16 +1971,6 @@ prep_detonate_action(Unit *unit, Unit *unit2, int x, int y, int z)
     return TRUE;
 }
 
-int
-net_prep_detonate_action(Unit *unit, Unit *unit2, int x, int y, int z)
-{
-    int rslt;
-
-    rslt = prep_detonate_action(unit, unit2, x, y, z);
-    broadcast_next_action(unit);
-    return rslt;
-}
-
 int
 do_detonate_action(Unit *unit, Unit *unit2, int x, int y, int z)
 {
@@ -2231,10 +2321,17 @@ type_can_capture(int u)
 	
     for_all_unit_types(u2) {
 	/* Capture can happen by both direct action and as result of an
-	   attack. */
-	if ((uu_acp_to_attack(u, u2) > 0 || uu_acp_to_capture(u, u2) > 0)
-	    && (uu_capture(u, u2) > 0 || uu_indep_capture(u, u2) > 0))
-	  return TRUE;
+	   attack in combat model 0. */
+	if (g_combat_model() == 0) {
+		if ((uu_acp_to_attack(u, u2) > 0 || uu_acp_to_capture(u, u2) > 0)
+		    && (uu_capture(u, u2) > 0 || uu_indep_capture(u, u2) > 0))
+		  return TRUE;
+	/* Capture can only happen as result of an attack in combat model 1. */
+	} else if (g_combat_model() == 1) {
+		if (uu_acp_to_attack(u, u2) > 0
+		    && (uu_capture(u, u2) > 0 || uu_indep_capture(u, u2) > 0))
+		  return TRUE;
+	}
     }
     return FALSE;
 }
@@ -2258,6 +2355,9 @@ type_can_capture_directly(int u)
 {
     int u2;
 
+    /* In combat model 1, we can't capture units directly. */
+    if (g_combat_model() == 1)
+      return FALSE;
     for_all_unit_types(u2) {
 	if (uu_acp_to_capture(u, u2) > 0
 	    && (uu_capture(u, u2) > 0 
@@ -2304,9 +2404,9 @@ defended_by_occupants(Unit *unit)
 	}
     }
     if (enough_to_garrison(unit, defenders))
-        return TRUE;
+      return TRUE;
     else 
-        return FALSE;
+      return FALSE;
 }
 
 /* True if first type can both occupy and protect second type. */
@@ -2314,17 +2414,30 @@ defended_by_occupants(Unit *unit)
 int
 occ_can_defend_transport(int o, int t)
 {
-    /* Never true if transport cannot carry occupant. */
-    if (!could_carry(t, o))
-      return FALSE;
-    /* Specific protection of transport by its occupant. */
-    /* Note: zero protection is 100, full protection is 0! */
-    if (uu_protection(o, t) < 100)
-      return TRUE;
-    /* Cellwide protection of all units by occupant. */
-    if (uu_cellwide_protection_for(o, t) < 100)
-      return TRUE;
-    return FALSE;	
+	/* Never true if transport cannot carry occupant. */
+	if (!could_carry(t, o))
+	    return FALSE;
+	if (g_combat_model() == 0) {
+		/* Specific protection of transport by its occupant.
+		Note: zero protection is 100, full protection is 0! */
+		if (uu_protection(o, t) < 100)
+		    return TRUE;
+		/* Cellwide protection of all units by occupant. */
+		if (uu_cellwide_protection_for(o, t) < 100)
+		    return TRUE;
+	} else if (g_combat_model() == 1) {
+		/* Advanced units are protected by any kind of occupant
+		that has a positive defense value. */
+		 if (u_advanced(t) && u_defend(o) > 0) {
+			return TRUE;
+		/* Non-advanced units are protected by this table. */ 
+		} else if (!u_advanced(t)
+			    /*	Note: More than 100 is protection. */
+			     && uu_occ_affects_defense(o, t) > 100) {
+			return TRUE;
+		}
+	}
+	return FALSE;	
 }
 
 /* True if the specified number of defenders is sufficient to defend
@@ -2359,25 +2472,25 @@ mobile_enemy_threat(Unit *unit, int range)
 	
     /* Go through all cells within the specified range. */
     for_all_cells_within_range(unit->x, unit->y, range, x, y) {
-		if (!inside_area(x, y))
-		  continue;
-		if (!terrain_visible(unit->side, x, y))
-		  continue;
-		/* Important to count also occupants here. */
-		for_all_stack_with_occs(x, y, unit2) {
-		    /* Only count visible mobile enemy units that actually 
-		       threaten us. */
-		    if (is_active(unit2)
-			/* The NULL side rarely attacks anybody :-) */
-			&& unit2->side
-			&& mobile(unit2->type)
-			&& side_sees_image(unit->side, unit2)
-			&& !trusted_side(unit->side, unit2->side)
-			&& (could_hit(unit2->type, unit->type)
-			    || capture_chance(unit2->type, unit->type, unit->side) > 0)) {
-			return unit2;
-		    }
-		}
+	if (!inside_area(x, y))
+	  continue;
+	if (!terrain_visible(unit->side, x, y))
+	  continue;
+	/* Important to count also occupants here. */
+	for_all_stack_with_occs(x, y, unit2) {
+	    /* Only count visible mobile enemy units that actually
+	       threaten us. */
+	    if (is_active(unit2)
+		/* The NULL side rarely attacks anybody :-) */
+		&& unit2->side
+		&& mobile(unit2->type)
+		&& side_sees_image(unit->side, unit2)
+		&& !trusted_side(unit->side, unit2->side)
+		&& (could_hit(unit2->type, unit->type)
+		    || capture_chance(unit2->type, unit->type, unit->side) > 0)) {
+		return unit2;
+	    }
+	}
     }
     return NULL;
 }
