@@ -1,5 +1,5 @@
 /* Configuration-related definitions for Xconq.
-   Copyright (C) 1987-1989, 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,18 +20,9 @@ any later version.  See the file COPYING.  */
 #include <ctype.h>
 #include <string.h>
 #include <time.h>
-
-#if defined(__STDC__) || defined(MPW_C)
 #include <stddef.h>
 #include <stdarg.h>
 #include <stdlib.h>
-#endif
-
-#if defined(__STDC__) || defined(MPW_C)
-#ifndef ANSI_PROTOTYPES
-#define ANSI_PROTOTYPES
-#endif
-#endif
 
 /* Default names for special files. */
 
@@ -132,14 +123,9 @@ any later version.  See the file COPYING.  */
 #define MAXATYPES 254
 #endif
 
-/* Some parts of the code rarely need the full magnitude of the MAX*TYPE
-   values, so these values are used to cut down average allocation.  They
-   can be adjusted to favor the kinds of game designs that you usually run. */
+/* Maximum numbers of types of AIs. */
 
-#define INITMAXUTYPES 20
-#define INITMAXMTYPES  5
-#define INITMAXTTYPES 15
-#define INITMAXATYPES 50
+#define MAXAITYPES 10
 
 /* All names, phrases, and messages must be able to fit in statically
    allocated buffers of this size. */
@@ -154,11 +140,6 @@ any later version.  See the file COPYING.  */
 
 #define RUDE
 
-/* This sets a limit on how many previous views will be retained.
-   Each increment adds 4*area bytes of allocation. */
-
-#define MAXVIEWHISTORY 0
-
 /* If defined, then an editing mode is available.  This is also a
    powerful way to cheat, which is why it can be disabled. */
 
@@ -167,8 +148,3 @@ any later version.  See the file COPYING.  */
 /* If defined, some (actually a lot) debugging code will be included. */
 
 #define DEBUGGING
-
-/* The number of AI types defined.  Increment for each new AI type
-   that is compiled in. */
-
-#define numaitypes 3
