@@ -340,14 +340,6 @@ extern VarDefn vardefns[];
 
 #define for_all_advance_types(v)   for (v = 0; v < numatypes; ++v)
 
-#define for_all_possible_unit_types(v)      for (v = 0; v < MAXUTYPES; ++v)
-
-#define for_all_possible_material_types(v)  for (v = 0; v < MAXMTYPES; ++v)
-
-#define for_all_possible_terrain_types(v)   for (v = 0; v < MAXTTYPES; ++v)
-
-#define for_all_possible_advance_types(v)   for (v = 0; v < MAXATYPES; ++v)
-
 /* Iterate over border and connection types by using a cached array
    linking each type to the next. */
 
