@@ -87,6 +87,20 @@ get_gif(FileImage *fimg)
 	    return rslt;
 	}
     }
+
+#ifdef MAC
+
+    /* We must not use unix pathnames on the mac. */
+    sprintf(fallback, "::images:%s", fimg->name);
+    /* Adding library paths here gives the wrong result. */ 
+    if ((fp = open_file(fallback, "rb")) != NULL) {
+	rslt = get_gif_from_file(fimg, fp);
+	fclose(fp);
+	return rslt;
+    }
+
+#else
+
     /* If we didn't find in the list, try adding a "../images" to see
        if the image is in a sibling dir. */
     sprintf(fallback, "../images/%s", fimg->name);
@@ -98,6 +112,9 @@ get_gif(FileImage *fimg)
 	    return rslt;
 	}
     }
+
+#endif
+
     return FALSE;
 }
 
@@ -133,7 +150,7 @@ get_gif_from_file(FileImage *fimg, FILE *fp)
     fimg->numcolors = numcolors;
     /* If we're going to add black to the palette, bump up the number of
        colors now, so the right amount of space gets allocated. */
-    if (fimg->numtransparent > 0 && blacki < 0)
+    if (!use_clip_mask && fimg->numtransparent > 0 && blacki < 0)
       ++(fimg->numcolors);
     fimg->palette = (int *) xmalloc(4 * fimg->numcolors * sizeof(int));
     for (i = 0; i < numcolors; ++i) {
@@ -142,16 +159,18 @@ get_gif_from_file(FileImage *fimg, FILE *fp)
 	  fimg->palette[4*i+j] = rawpal[4*i+j];
     }
     /* Maybe add black to the palette. */
-    if (fimg->numtransparent > 0 && blacki < 0) {
-	blacki = numcolors;
-	fimg->palette[4 * blacki] = blacki;
-	fimg->palette[4 * blacki + 1] = 0;
-	fimg->palette[4 * blacki + 2] = 0;
-	fimg->palette[4 * blacki + 3] = 0;
-    }
-    if (fimg->numtransparent > 0) {
-	/* Record blackness as another one of the transparent colors. */
-	fimg->transparent[fimg->numtransparent++] = blacki;
+    if (!use_clip_mask) {
+	if (fimg->numtransparent > 0 && blacki < 0) {
+	    blacki = numcolors;
+	    fimg->palette[4 * blacki] = blacki;
+	    fimg->palette[4 * blacki + 1] = 0;
+	    fimg->palette[4 * blacki + 2] = 0;
+	    fimg->palette[4 * blacki + 3] = 0;
+	}
+	if (fimg->numtransparent > 0) {
+	    /* Record blackness as another one of the transparent colors. */
+	    fimg->transparent[fimg->numtransparent++] = blacki;
+	}
     }
     return TRUE;
 }
