@@ -24,8 +24,6 @@ int any_enter_indep(int u);
 #define obstack_chunk_alloc xmalloc
 #define obstack_chunk_free free
 
-extern int error_is_bug;
-
 static void describe_help_system(int arg, char *key, TextBuffer *buf);
 static void describe_instructions(int arg, char *key, TextBuffer *buf);
 static void describe_synth_run(TextBuffer *buf, int methkey);
@@ -212,7 +210,6 @@ add_help_node(char *key, void (*fn)(int t, char *key, TextBuffer *buf),
     HelpNode *node, *nextnode;
 
     if (empty_string(key)) {
-	error_is_bug = TRUE;
 	run_error("empty help key");
     }
     node = create_help_node();
@@ -727,7 +724,7 @@ describe_utype(int u, char *key, TextBuffer *buf)
     if (u_possible_sides(u) != lispnil) {
 	tbcat(buf, "Possible sides in this game: ");
 	first = TRUE;
-	for_all_sides_plus_indep(side) {
+	for_all_sides(side) {
 	    if (type_allowed_on_side(u, side)) {
 		if (first)
 		  first = FALSE;
@@ -1307,9 +1304,9 @@ describe_utype(int u, char *key, TextBuffer *buf)
 	else
 	  tbcat(buf, "Not always seen even if terrain has been seen.\n");
 	/* (should only say if can be an occupant) */
-	if (uu_table_row_not_default(u, uu_occ_can_see, 1)) {
-	    tbcat(buf, "Can see while an occupant: ");
-	    uu_table_row_desc(buf, u, uu_occ_can_see, tb_bool_desc, "in");
+	if (uu_table_row_not_default(u, uu_occ_vision, 100)) {
+	    tbcat(buf, "Vision effect when occupying: ");
+	    uu_table_row_desc(buf, u, uu_occ_vision, NULL, "in");
 	    tbcat(buf, ".\n");
 	}
 	/* (should only say if unit can have occupants) */
@@ -2440,41 +2437,43 @@ histogram_compare(CONST void *h1, CONST void *h2)
     }
 }
 
+static struct histo *u_histogram;
+
 static void
-u_property_desc(buf, fn, formatter)
-TextBuffer *buf;
-int (*fn)(int i);
-void (*formatter)(TextBuffer *buf, int val);
+u_property_desc(TextBuffer *buf, int (*fn)(int i),
+		void (*formatter)(TextBuffer *buf, int val))
 {
     int val, u, val2, constant = TRUE, found;
     int i, numentries, first;
-    struct histo histogram[MAXUTYPES];
 
     if (formatter == NULL)
       formatter = tb_value_desc;
+    if (u_histogram == NULL)
+      u_histogram =
+	(struct histo *) xmalloc(numutypes * sizeof(struct histo));
     /* Compute a histogram of all the values for the given property. */
     numentries = 0;
     val = (*fn)(0);
-    histogram[numentries].val = val;
-    histogram[numentries].num = 1;
+    u_histogram[numentries].val = val;
+    u_histogram[numentries].num = 1;
     ++numentries;
     for_all_unit_types(u) {
 	val2 = (*fn)(u);
 	if (val2 == val) {
-	    ++(histogram[0].num);
+	    ++(u_histogram[0].num);
 	} else {
 	    constant = FALSE;
 	    found = FALSE;
 	    for (i = 1; i < numentries; ++i) {
-		if (val2 == histogram[i].val) {
-		    ++(histogram[i].num);
+		if (val2 == u_histogram[i].val) {
+		    ++(u_histogram[i].num);
 		    found = TRUE;
 		    break;
 		}
 	    }
 	    if (!found) {
-		histogram[numentries].val = val2;
-		histogram[numentries].num = 1;
+		u_histogram[numentries].val = val2;
+		u_histogram[numentries].num = 1;
 		++numentries;
 	    }
 	}
@@ -2486,11 +2485,11 @@ void (*formatter)(TextBuffer *buf, int val);
     	return;
     }
     /* Not a constant row; sort the histogram and compose a description. */
-    qsort(histogram, numentries, sizeof(struct histo), histogram_compare);
+    qsort(u_histogram, numentries, sizeof(struct histo), histogram_compare);
     /* Show a "by default" clause if at least half of the entries are all
        the same. */
-    if (histogram[0].num * 2 >= numutypes) {
-	(*formatter)(buf, histogram[0].val);
+    if (u_histogram[0].num * 2 >= numutypes) {
+	(*formatter)(buf, u_histogram[0].val);
 	tbcat(buf, " by default");
     	i = 1;
     } else {
@@ -2499,11 +2498,11 @@ void (*formatter)(TextBuffer *buf, int val);
     for (; i < numentries; ++i) {
 	if (i > 0)
 	  tbcat(buf, ", ");
-	(*formatter)(buf, histogram[i].val);
+	(*formatter)(buf, u_histogram[i].val);
 	tbcat(buf, " for ");
 	first = TRUE;
 	for_all_unit_types(u) {
-	    if ((*fn)(u) == histogram[i].val) {
+	    if ((*fn)(u) == u_histogram[i].val) {
 		if (!first)
 		  /* For this and similar situations, we need a space
 		     in addition to the comma, so interface elements
@@ -2518,41 +2517,43 @@ void (*formatter)(TextBuffer *buf, int val);
     }
 }
 
+static struct histo *t_histogram;
+
 static void
-t_property_desc(buf, fn, formatter)
-TextBuffer *buf;
-int (*fn)(int i);
-void (*formatter)(TextBuffer *buf, int val);
+t_property_desc(TextBuffer *buf, int (*fn)(int i),
+		void (*formatter)(TextBuffer *buf, int val))
 {
     int val, t, val2, constant = TRUE, found;
     int i, numentries, first;
-    struct histo histogram[MAXUTYPES];
 
     if (formatter == NULL)
       formatter = tb_value_desc;
+    if (t_histogram == NULL)
+      t_histogram =
+	(struct histo *) xmalloc(numttypes * sizeof(struct histo));
     /* Compute a histogram of all the values for the given property. */
     numentries = 0;
     val = (*fn)(0);
-    histogram[numentries].val = val;
-    histogram[numentries].num = 1;
+    t_histogram[numentries].val = val;
+    t_histogram[numentries].num = 1;
     ++numentries;
     for_all_terrain_types(t) {
 	val2 = (*fn)(t);
 	if (val2 == val) {
-	    ++(histogram[0].num);
+	    ++(t_histogram[0].num);
 	} else {
 	    constant = FALSE;
 	    found = FALSE;
 	    for (i = 1; i < numentries; ++i) {
-		if (val2 == histogram[i].val) {
-		    ++(histogram[i].num);
+		if (val2 == t_histogram[i].val) {
+		    ++(t_histogram[i].num);
 		    found = TRUE;
 		    break;
 		}
 	    }
 	    if (!found) {
-		histogram[numentries].val = val2;
-		histogram[numentries].num = 1;
+		t_histogram[numentries].val = val2;
+		t_histogram[numentries].num = 1;
 		++numentries;
 	    }
 	}
@@ -2564,11 +2565,11 @@ void (*formatter)(TextBuffer *buf, int val);
     	return;
     }
     /* Not a constant row; sort the histogram and compose a description. */
-    qsort(histogram, numentries, sizeof(struct histo), histogram_compare);
+    qsort(t_histogram, numentries, sizeof(struct histo), histogram_compare);
     /* Show a "by default" clause if at least half of the entries are all
        the same. */
-    if (histogram[0].num * 2 >= numttypes) {
-	(*formatter)(buf, histogram[0].val);
+    if (t_histogram[0].num * 2 >= numttypes) {
+	(*formatter)(buf, t_histogram[0].val);
 	tbcat(buf, " by default");
     	i = 1;
     } else {
@@ -2577,11 +2578,11 @@ void (*formatter)(TextBuffer *buf, int val);
     for (; i < numentries; ++i) {
 	if (i > 0)
 	  tbcat(buf, ", ");
-	(*formatter)(buf, histogram[i].val);
+	(*formatter)(buf, t_histogram[i].val);
 	tbcat(buf, " for ");
 	first = TRUE;
 	for_all_terrain_types(t) {
-	    if ((*fn)(t) == histogram[i].val) {
+	    if ((*fn)(t) == t_histogram[i].val) {
 		if (!first)
 		  tbcat(buf, ", ");
 		else
@@ -2596,67 +2597,58 @@ void (*formatter)(TextBuffer *buf, int val);
    unit types wrt a given table. */
 
 static void
-uu_table_row_desc(buf, u, fn, formatter, connect)
-TextBuffer *buf;
-int u;
-int (*fn)(int i, int j);
-void (*formatter)(TextBuffer *buf, int val);
-char *connect;
+uu_table_row_desc(TextBuffer *buf, int u, int (*fn)(int i, int j),
+		  void (*formatter)(TextBuffer *buf, int val), char *connect)
 {
     uu_table_rowcol_desc(buf, u, fn, formatter, connect, 0);
 }
 
 static void
-uu_table_column_desc(buf, u, fn, formatter, connect)
-TextBuffer *buf;
-int u;
-int (*fn)(int i, int j);
-void (*formatter)(TextBuffer *buf, int val);
-char *connect;
+uu_table_column_desc(TextBuffer *buf, int u, int (*fn)(int i, int j),
+		     void (*formatter)(TextBuffer *buf, int val),
+		     char *connect)
 {
     uu_table_rowcol_desc(buf, u, fn, formatter, connect, 1);
 }
 
 static void
-uu_table_rowcol_desc(buf, u, fn, formatter, connect, rowcol)
-TextBuffer *buf;
-int u;
-int (*fn)(int i, int j);
-void (*formatter)(TextBuffer *buf, int val);
-char *connect;
-int rowcol;
+uu_table_rowcol_desc(TextBuffer *buf, int u, int (*fn)(int i, int j),
+		     void (*formatter)(TextBuffer *buf, int val),
+		     char *connect, int rowcol)
 {
     int val, val2, u2, constant = TRUE, found;
     int i, numentries, first;
-    struct histo histogram[MAXUTYPES];
 
     if (formatter == NULL)
       formatter = tb_value_desc;
     if (empty_string(connect))
       connect = "for";
+    if (u_histogram == NULL)
+      u_histogram =
+	(struct histo *) xmalloc(numutypes * sizeof(struct histo));
     val = (rowcol ? (*fn)(0, u) : (*fn)(u, 0));
     /* Compute a histogram of all the values in the row of the table. */
     numentries = 0;
-    histogram[numentries].val = val;
-    histogram[numentries].num = 1;
+    u_histogram[numentries].val = val;
+    u_histogram[numentries].num = 1;
     ++numentries;
     for_all_unit_types(u2) {
 	val2 = (rowcol ? (*fn)(u2, u) : (*fn)(u, u2));
 	if (val2 == val) {
-	    ++(histogram[0].num);
+	    ++(u_histogram[0].num);
 	} else {
 	    constant = FALSE;
 	    found = FALSE;
 	    for (i = 1; i < numentries; ++i) {
-		if (val2 == histogram[i].val) {
-		    ++(histogram[i].num);
+		if (val2 == u_histogram[i].val) {
+		    ++(u_histogram[i].num);
 		    found = TRUE;
 		    break;
 		}
 	    }
 	    if (!found) {
-		histogram[numentries].val = val2;
-		histogram[numentries].num = 1;
+		u_histogram[numentries].val = val2;
+		u_histogram[numentries].num = 1;
 		++numentries;
 	    }
 	}
@@ -2668,11 +2660,11 @@ int rowcol;
     	return;
     }
     /* Not a constant row; sort the histogram and compose a description. */
-    qsort(histogram, numentries, sizeof(struct histo), histogram_compare);
+    qsort(u_histogram, numentries, sizeof(struct histo), histogram_compare);
     /* Show a "by default" clause if at least half of the entries are all
        the same. */
-    if (histogram[0].num * 2 >= numutypes) {
-	(*formatter)(buf, histogram[0].val);
+    if (u_histogram[0].num * 2 >= numutypes) {
+	(*formatter)(buf, u_histogram[0].val);
 	tbcat(buf, " by default");
     	i = 1;
     } else {
@@ -2681,12 +2673,12 @@ int rowcol;
     for (; i < numentries; ++i) {
 	if (i > 0)
 	  tbcat(buf, ", ");
-	(*formatter)(buf, histogram[i].val);
+	(*formatter)(buf, u_histogram[i].val);
 	tbprintf(buf, " %s ", connect);
 	first = TRUE;
 	for_all_unit_types(u2) {
 	    val2 = (rowcol ? (*fn)(u2, u) : (*fn)(u, u2));
-	    if (val2 == histogram[i].val) {
+	    if (val2 == u_histogram[i].val) {
 		if (!first)
 		  tbcat(buf, ", ");
 		else
@@ -2701,43 +2693,41 @@ int rowcol;
    terrain types wrt a given table. */
 
 static void
-ut_table_row_desc(buf, u, fn, formatter, connect)
-TextBuffer *buf;
-int u;
-int (*fn)(int i, int j);
-void (*formatter)(TextBuffer *buf, int val);
-char *connect;
+ut_table_row_desc(TextBuffer *buf, int u, int (*fn)(int i, int j),
+		  void (*formatter)(TextBuffer *buf, int val), char *connect)
 {
     int val = (*fn)(u, 0), val2, t, constant = TRUE, found;
     int i, numentries, first;
-    struct histo histogram[MAXUTYPES];
 
     if (formatter == NULL)
       formatter = tb_value_desc;
     if (empty_string(connect))
       connect = "for";
+    if (u_histogram == NULL)
+      u_histogram =
+	(struct histo *) xmalloc(numutypes * sizeof(struct histo));
     /* Compute a histogram of all the values in the row of the table. */
     numentries = 0;
-    histogram[numentries].val = val;
-    histogram[numentries].num = 1;
+    u_histogram[numentries].val = val;
+    u_histogram[numentries].num = 1;
     ++numentries;
     for_all_terrain_types(t) {
 	val2 = (*fn)(u, t);
 	if (val2 == val) {
-	    ++(histogram[0].num);
+	    ++(u_histogram[0].num);
 	} else {
 	    constant = FALSE;
 	    found = FALSE;
 	    for (i = 1; i < numentries; ++i) {
-		if (val2 == histogram[i].val) {
-		    ++(histogram[i].num);
+		if (val2 == u_histogram[i].val) {
+		    ++(u_histogram[i].num);
 		    found = TRUE;
 		    break;
 		}
 	    }
 	    if (!found) {
-		histogram[numentries].val = val2;
-		histogram[numentries].num = 1;
+		u_histogram[numentries].val = val2;
+		u_histogram[numentries].num = 1;
 		++numentries;
 	    }
 	}
@@ -2749,11 +2739,11 @@ char *connect;
     	return;
     }
     /* Not a constant row; sort the histogram and compose a description. */
-    qsort(histogram, numentries, sizeof(struct histo), histogram_compare);
+    qsort(u_histogram, numentries, sizeof(struct histo), histogram_compare);
     /* Show a "by default" clause if at least half of the entries are all
        the same. */
-    if (histogram[0].num * 2 >= numttypes) {
-	(*formatter)(buf, histogram[0].val);
+    if (u_histogram[0].num * 2 >= numttypes) {
+	(*formatter)(buf, u_histogram[0].val);
 	tbcat(buf, " by default");
     	i = 1;
     } else {
@@ -2762,11 +2752,11 @@ char *connect;
     for (; i < numentries; ++i) {
 	if (i > 0)
 	  tbcat(buf, ", ");
-	(*formatter)(buf, histogram[i].val);
+	(*formatter)(buf, u_histogram[i].val);
 	tbprintf(buf, " %s ", connect);
 	first = TRUE;
 	for_all_terrain_types(t) {
-	    if ((*fn)(u, t) == histogram[i].val) {
+	    if ((*fn)(u, t) == u_histogram[i].val) {
 		if (!first)
 		  tbcat(buf, ", ");
 		else
@@ -2778,41 +2768,40 @@ char *connect;
 }
 
 static void
-um_table_row_desc(buf, u, fn, formatter)
-TextBuffer *buf;
-int u;
-int (*fn)(int i, int j);
-void (*formatter)(TextBuffer *buf, int val);
+um_table_row_desc(TextBuffer *buf, int u, int (*fn)(int i, int j),
+		  void (*formatter)(TextBuffer *buf, int val))
 {
     int val = (*fn)(u, 0), val2, m, constant = TRUE, found;
     int i, numentries, first;
-    struct histo histogram[MAXUTYPES];
     char *connect = "vs";
 
     if (formatter == NULL)
       formatter = tb_value_desc;
+    if (u_histogram == NULL)
+      u_histogram =
+	(struct histo *) xmalloc(numutypes * sizeof(struct histo));
     /* Compute a histogram of all the values in the row of the table. */
     numentries = 0;
-    histogram[numentries].val = val;
-    histogram[numentries].num = 1;
+    u_histogram[numentries].val = val;
+    u_histogram[numentries].num = 1;
     ++numentries;
     for_all_material_types(m) {
 	val2 = (*fn)(u, m);
 	if (val2 == val) {
-	    ++(histogram[0].num);
+	    ++(u_histogram[0].num);
 	} else {
 	    constant = FALSE;
 	    found = FALSE;
 	    for (i = 1; i < numentries; ++i) {
-		if (val2 == histogram[i].val) {
-		    ++(histogram[i].num);
+		if (val2 == u_histogram[i].val) {
+		    ++(u_histogram[i].num);
 		    found = TRUE;
 		    break;
 		}
 	    }
 	    if (!found) {
-		histogram[numentries].val = val2;
-		histogram[numentries].num = 1;
+		u_histogram[numentries].val = val2;
+		u_histogram[numentries].num = 1;
 		++numentries;
 	    }
 	}
@@ -2824,11 +2813,11 @@ void (*formatter)(TextBuffer *buf, int val);
     	return;
     }
     /* Not a constant row; sort the histogram and compose a description. */
-    qsort(histogram, numentries, sizeof(struct histo), histogram_compare);
+    qsort(u_histogram, numentries, sizeof(struct histo), histogram_compare);
     /* Show a "by default" clause if at least half of the entries are all
        the same. */
-    if (histogram[0].num * 2 >= nummtypes) {
-	(*formatter)(buf, histogram[0].val);
+    if (u_histogram[0].num * 2 >= nummtypes) {
+	(*formatter)(buf, u_histogram[0].val);
 	tbcat(buf, " by default");
     	i = 1;
     } else {
@@ -2837,11 +2826,11 @@ void (*formatter)(TextBuffer *buf, int val);
     for (; i < numentries; ++i) {
 	if (i > 0)
 	  tbcat(buf, ", ");
-	(*formatter)(buf, histogram[i].val);
+	(*formatter)(buf, u_histogram[i].val);
 	tbprintf(buf, " %s ", connect);
 	first = TRUE;
 	for_all_material_types(m) {
-	    if ((*fn)(u, m) == histogram[i].val) {
+	    if ((*fn)(u, m) == u_histogram[i].val) {
 		if (!first)
 		  tbcat(buf, ", ");
 		else
@@ -2853,40 +2842,39 @@ void (*formatter)(TextBuffer *buf, int val);
 }
 
 static void
-tt_table_row_desc(buf, t0, fn, formatter)
-TextBuffer *buf;
-int t0;
-int (*fn)(int i, int j);
-void (*formatter)(TextBuffer *buf, int val);
+tt_table_row_desc(TextBuffer *buf, int t0, int (*fn)(int i, int j),
+		  void (*formatter)(TextBuffer *buf, int val))
 {
     int val = (*fn)(t0, 0), val2, t, constant = TRUE, found;
     int i, numentries, first;
-    struct histo histogram[MAXUTYPES];
 
     if (formatter == NULL)
       formatter = tb_value_desc;
+    if (t_histogram == NULL)
+      t_histogram =
+	(struct histo *) xmalloc(numttypes * sizeof(struct histo));
     /* Compute a histogram of all the values in the row of the table. */
     numentries = 0;
-    histogram[numentries].val = val;
-    histogram[numentries].num = 1;
+    t_histogram[numentries].val = val;
+    t_histogram[numentries].num = 1;
     ++numentries;
     for_all_terrain_types(t) {
 	val2 = (*fn)(t0, t);
 	if (val2 == val) {
-	    ++(histogram[0].num);
+	    ++(t_histogram[0].num);
 	} else {
 	    constant = FALSE;
 	    found = FALSE;
 	    for (i = 1; i < numentries; ++i) {
-		if (val2 == histogram[i].val) {
-		    ++(histogram[i].num);
+		if (val2 == t_histogram[i].val) {
+		    ++(t_histogram[i].num);
 		    found = TRUE;
 		    break;
 		}
 	    }
 	    if (!found) {
-		histogram[numentries].val = val2;
-		histogram[numentries].num = 1;
+		t_histogram[numentries].val = val2;
+		t_histogram[numentries].num = 1;
 		++numentries;
 	    }
 	}
@@ -2898,11 +2886,11 @@ void (*formatter)(TextBuffer *buf, int val);
     	return;
     }
     /* Not a constant row; sort the histogram and compose a description. */
-    qsort(histogram, numentries, sizeof(struct histo), histogram_compare);
+    qsort(t_histogram, numentries, sizeof(struct histo), histogram_compare);
     /* Show a "by default" clause if at least half of the entries are all
        the same. */
-    if (histogram[0].num * 2 >= numttypes) {
-	(*formatter)(buf, histogram[0].val);
+    if (t_histogram[0].num * 2 >= numttypes) {
+	(*formatter)(buf, t_histogram[0].val);
 	tbcat(buf, " by default");
     	i = 1;
     } else {
@@ -2911,11 +2899,11 @@ void (*formatter)(TextBuffer *buf, int val);
     for (; i < numentries; ++i) {
 	if (i > 0)
 	  tbcat(buf, ", ");
-	(*formatter)(buf, histogram[i].val);
+	(*formatter)(buf, t_histogram[i].val);
 	tbcat(buf, " vs ");
 	first = TRUE;
 	for_all_terrain_types(t) {
-	    if ((*fn)(t0, t) == histogram[i].val) {
+	    if ((*fn)(t0, t) == t_histogram[i].val) {
 		if (!first)
 		  tbcat(buf, ", ");
 		else
@@ -3003,61 +2991,56 @@ void (*formatter)(TextBuffer *buf, int val);
 #endif
 
 static void
-aa_table_row_desc(buf, a0, fn, formatter)
-TextBuffer *buf;
-int a0;
-int (*fn)(int i, int j);
-void (*formatter)(TextBuffer *buf, int val);
+aa_table_row_desc(TextBuffer *buf, int a0, int (*fn)(int i, int j),
+		  void (*formatter)(TextBuffer *buf, int val))
 {
     aa_table_rowcol_desc(buf, a0, fn, formatter, 0);
 }
 
 static void
-aa_table_column_desc(buf, a1, fn, formatter)
-TextBuffer *buf;
-int a1;
-int (*fn)(int i, int j);
-void (*formatter)(TextBuffer *buf, int val);
+aa_table_column_desc(TextBuffer *buf, int a1, int (*fn)(int i, int j),
+		  void (*formatter)(TextBuffer *buf, int val))
 {
     aa_table_rowcol_desc(buf, a1, fn, formatter, 1);
 }
 
+static struct histo *a_histogram;
+
 static void
-aa_table_rowcol_desc(buf, a1, fn, formatter, rowcol)
-TextBuffer *buf;
-int a1;
-int (*fn)(int i, int j);
-void (*formatter)(TextBuffer *buf, int val);
+aa_table_rowcol_desc(TextBuffer *buf, int a1, int (*fn)(int i, int j),
+		     void (*formatter)(TextBuffer *buf, int val), int rowcol)
 {
     int val = (*fn)(0, a1), val2, a, constant = TRUE, found;
     int i, numentries, first;
-    struct histo histogram[MAXATYPES];
 
     if (formatter == NULL)
       formatter = tb_value_desc;
+    if (a_histogram == NULL)
+      a_histogram =
+	(struct histo *) xmalloc(numatypes * sizeof(struct histo));
     val = (rowcol ? (*fn)(0, a1) : (*fn)(a1, 0));
     /* Compute a histogram of all the values in the row of the table. */
     numentries = 0;
-    histogram[numentries].val = val;
-    histogram[numentries].num = 1;
+    a_histogram[numentries].val = val;
+    a_histogram[numentries].num = 1;
     ++numentries;
     for_all_advance_types(a) {
 	val2 = (rowcol ? (*fn)(a, a1) : (*fn)(a1, a));
 	if (val2 == val) {
-	    ++(histogram[0].num);
+	    ++(a_histogram[0].num);
 	} else {
 	    constant = FALSE;
 	    found = FALSE;
 	    for (i = 1; i < numentries; ++i) {
-		if (val2 == histogram[i].val) {
-		    ++(histogram[i].num);
+		if (val2 == a_histogram[i].val) {
+		    ++(a_histogram[i].num);
 		    found = TRUE;
 		    break;
 		}
 	    }
 	    if (!found) {
-		histogram[numentries].val = val2;
-		histogram[numentries].num = 1;
+		a_histogram[numentries].val = val2;
+		a_histogram[numentries].num = 1;
 		++numentries;
 	    }
 	}
@@ -3069,11 +3052,11 @@ void (*formatter)(TextBuffer *buf, int val);
     	return;
     }
     /* Not a constant column; sort the histogram and compose a description. */
-    qsort(histogram, numentries, sizeof(struct histo), histogram_compare);
+    qsort(a_histogram, numentries, sizeof(struct histo), histogram_compare);
     /* Show a "by default" clause if at least half of the entries are all
        the same. */
-    if (histogram[0].num * 2 >= numttypes) {
-	(*formatter)(buf, histogram[0].val);
+    if (a_histogram[0].num * 2 >= numttypes) {
+	(*formatter)(buf, a_histogram[0].val);
 	tbcat(buf, " by default");
     	i = 1;
     } else {
@@ -3082,12 +3065,12 @@ void (*formatter)(TextBuffer *buf, int val);
     for (; i < numentries; ++i) {
 	if (i > 0)
 	  tbcat(buf, ", ");
-	(*formatter)(buf, histogram[i].val);
+	(*formatter)(buf, a_histogram[i].val);
 	tbcat(buf, " vs ");
 	first = TRUE;
 	for_all_advance_types(a) {
 	    val2 = (rowcol ? (*fn)(a, a1) : (*fn)(a1, a));
-	    if (val2 == histogram[i].val) {
+	    if (val2 == a_histogram[i].val) {
 		if (!first)
 		  tbcat(buf, ", ");
 		else
