@@ -7,13 +7,6 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern int find_event_type(Obj *sym);
-extern int update_total_hist_lines(Side *side);
-extern int build_hist_contents(Side *side, int n, HistEvent **histcontents,
-			       int numvishistlines);
-
-int pattern_matches_event(Obj *pattern, HistEvent *hevt);
-void event_desc_from_list(Side *side, Obj *lis, HistEvent *hevt, char *buf);
 
 static void notify_event(Side *side, HistEvent *hevt);
 static void play_event_movies(Side *side, HistEvent *hevt);
