@@ -1,5 +1,5 @@
 /* Definitions for the historical record.
-   Copyright (C) 1992, 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1992-1995, 1998-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@ typedef struct a_histevent {
     SideMask observers;		/* which sides know about this event */
     struct a_histevent *next;	/* link to next in list */
     struct a_histevent *prev;	/* link to previous in list */
-    short data[4];		/* data describing event */
+    int data[4];		/* data describing event */
 } HistEvent;
 
 /* This is a snapshot of key bits of a unit's state at a particular
@@ -38,7 +38,7 @@ typedef struct a_histevent {
 
 typedef struct a_pastunit {
     short type;			/* type */
-    short id;			/* truly unique id number */
+    int id;			/* truly unique id number */
     char *name;			/* the name, if given */
     int number;			/* semi-unique number */
     short x, y, z;		/* position of unit in world */
@@ -76,20 +76,25 @@ extern HistEvent *history;
 
 extern PastUnit *past_unit_list;
 
-extern void init_history PARAMS ((void));
-extern void start_history PARAMS ((void));
-extern HistEvent *create_historical_event PARAMS ((HistEventType type));
-extern HistEvent *record_event PARAMS ((HistEventType type, SideMask observers, ...));
-extern void record_unit_death PARAMS ((Unit *unit, HistEventType reason));
-extern void record_unit_name_change PARAMS ((Unit *unit, char *newname));
-extern void record_unit_side_change PARAMS ((Unit *unit, Side *newside, HistEventType reason, Unit *agent));
-extern void count_gain PARAMS ((Side *side, int u, enum gain_reasons reason));
-extern void count_loss PARAMS ((Side *side, int u, enum loss_reasons reason));
-extern void end_history PARAMS ((void));
-extern HistEvent *get_nth_history_line PARAMS ((Side *side, int n, HistEvent **nextevt));
-extern PastUnit *create_past_unit PARAMS ((int type, int id));
-extern PastUnit *find_past_unit PARAMS ((int n));
-extern char *past_unit_desig PARAMS ((PastUnit *pastunit));
-extern PastUnit *change_unit_to_past_unit PARAMS ((Unit *unit));
-extern void dump_statistics PARAMS ((void));
+extern void init_history(void);
+extern void start_history(void);
+extern HistEvent *create_historical_event(HistEventType type);
+extern HistEvent *record_event(HistEventType type, SideMask observers, ...);
+extern void record_unit_death(Unit *unit, HistEventType reason);
+extern void record_unit_name_change(Unit *unit, char *newname);
+extern void record_unit_side_change(Unit *unit, Side *newside,
+				    HistEventType reason, Unit *agent);
+extern void count_gain(Side *side, int u, enum gain_reasons reason);
+extern void count_loss(Side *side, int u, enum loss_reasons reason);
+extern void end_history(void);
+extern HistEvent *get_nth_history_line(Side *side, int n, HistEvent **nextevt);
+extern PastUnit *create_past_unit(int type, int id);
+extern PastUnit *find_past_unit(int n);
+extern char *past_unit_desig(PastUnit *pastunit);
+extern PastUnit *change_unit_to_past_unit(Unit *unit);
+extern void dump_statistics(void);
 
+extern int update_total_hist_lines(Side *side);
+extern int build_hist_contents(Side *side, int n, HistEvent **histcontents, int numvis);
+extern int total_gain(Side *side, int u);
+extern int total_loss(Side *side, int u);
