@@ -13,17 +13,9 @@ any later version.  See the file COPYING.  */
 #include "misc.h"
 #include "lisp.h"
 #include "imf.h"
-extern Image *get_subimg(ImageFamily *imf, int w, int h);
-extern Image *largest_image(ImageFamily *imf);
-extern int image_pixel_at(Image *img, int imtype, int x, int y);
-extern void set_image_pixel_at(Image *img, int imtype, int x, int y, int val);
-extern void make_image_from_file_image(ImageFamily *imf, Image *img,
-				       Image *subimg, int subi);
 #include "module.h"
 #include "system.h"
 
-extern void syntax_error(Obj *x, char *msg);
-
 /* RGB above this value should be considered white. */
 
 #define WHITE_THRESHOLD (65535 - 256)
@@ -34,9 +26,7 @@ extern void syntax_error(Obj *x, char *msg);
 
 /* (should remove these fixed limits someday) */
 
-#define MAXIMAGEFAMILIES 1000
-
-#define MAXIMAGEPALETTES 100
+#define MAXIMAGEFAMILIES 1200
 
 enum {
     K_MONO_,
@@ -46,6 +36,10 @@ enum {
     K_OTHER_
 };
 
+static Image *get_subimg(ImageFamily *imf, int w, int h);
+static Image *largest_image(ImageFamily *imf);
+static int image_pixel_at(Image *img, int imtype, int x, int y);
+static void set_image_pixel_at(Image *img, int imtype, int x, int y, int val);
 static ImageFamily *new_imf(char *name);
 static Image *add_shrunken_image(ImageFamily *imf);
 static Image *add_magnified_image(ImageFamily *imf);
@@ -67,12 +61,6 @@ ImageFamily **images;
 
 int numimages = 0;
 
-/* This is the array and count of named palettes. */
-
-ImagePalette **palettes;
-
-int numpalettes = 0;
-
 /* Head of the linked list of image files. */
 
 ImageFile *image_files;
@@ -87,6 +75,20 @@ ImageFamily *(*imf_interp_hook)(ImageFamily *imf, Image *img, int force);
 
 short write_synthetic_also;
 
+/* There are two ways to use image families.  The "old" way is to use
+   the image families' masks and other bitmaps as clip masks.  tcl/tk
+   for Windows doesn't support clip masks however, so the "new" way is
+   to do a GXand with the mask, carving out a mask-shaped black hole,
+   then do a GXor with the image proper.  Note that the part of the
+   image that is not included in the mask must be converted to black
+   if the ORing is to have the right effect.  X doesn't guarantee that
+   ANDing and ORing will work in general though; we just know that the
+   X emulation in tcl/tk for Windows can be used in this way. */
+
+/* This should be TRUE everywhere except on Windows. */
+
+int use_clip_mask = TRUE;
+
 /* Create and return an image family. */
 
 static ImageFamily *
@@ -307,8 +309,6 @@ interp_imf_form(Obj *form, void (*imf_callback)(ImageFamily *imf, int loadnow))
 	imf = interp_imf(form);
 	if (imf_callback != NULL && imf != NULL)
 	  (*imf_callback)(imf, TRUE);
-    } else if (match_keyword(head, K_PALETTE)) {
-	interp_palette(form);
     } else {
 	/* Ignore any non-image forms, we might be reading from a 
 	   normal game design. */
@@ -462,7 +462,7 @@ interp_imf_contents(ImageFamily *imf, Obj *clauses)
 void
 interp_image(ImageFamily *imf, Obj *size, Obj *parts)
 {
-    int w, h, imtype, emx, emy, emw, emh, subi;
+    int w, h, imtype, emx, emy, emw, emh, numsubs, subi;
     char *name;
     Image *img, *subimg;
     Obj *head, *rest, *typ, *prop, *proptype, *datalist;
@@ -485,6 +485,7 @@ interp_image(ImageFamily *imf, Obj *size, Obj *parts)
       img->isborder = TRUE;
     if (match_keyword(car(cddr(size)), K_TRANSITION))
       img->istransition = TRUE;
+    numsubs = 0;
     for_all_list(parts, rest) {
 	head = car(rest);
 	typ = car(head);
@@ -522,7 +523,7 @@ interp_image(ImageFamily *imf, Obj *size, Obj *parts)
 	    img->notes = cadr(head);
 	    syntax_error(head, "extra junk after image notes property");
 	} else if (match_keyword(typ, K_X)) {
-	    img->numsubimages = c_number(cadr(head));
+	    numsubs = c_number(cadr(head));
 	    if (cddr(head) != lispnil) {
 		img->subx = c_number(caddr(head));
 		img->suby = c_number(car(cdddr(head)));
@@ -585,41 +586,27 @@ interp_image(ImageFamily *imf, Obj *size, Obj *parts)
 	}
     }
     /* Allocate space for any subimages that might be needed. */
-    /* (should simplify?) */
+    /* First set some standard numbers of subimages. */
     if (img->isborder) {
-	img->numsubimages = 16;
-	img->subimages =
-	  (Image **) xmalloc(img->numsubimages * sizeof(Image *));
-	for (subi = 0; subi < img->numsubimages; ++subi) {
-	    subimg = get_subimg(imf, img->w, img->h);
-	    img->subimages[subi] = subimg;
-	}
+	numsubs = 16;
     } else if (img->isconnection) {
-	img->numsubimages = 64;
-	img->subimages =
-	  (Image **) xmalloc(img->numsubimages * sizeof(Image *));
-	for (subi = 0; subi < img->numsubimages; ++subi) {
-	    subimg = get_subimg(imf, img->w, img->h);
-	    img->subimages[subi] = subimg;
-	}
+	numsubs = 64;
     } else if (img->istransition) {
-	/* Leave the main image mostly alone, iterate over the subimages. */
-	img->numsubimages = 4 * 4;
-	img->subimages =
-	  (Image **) xmalloc(img->numsubimages * sizeof(Image *));
-	for (subi = 0; subi < img->numsubimages; ++subi) {
+	numsubs = 4 * 4;
+    }
+    /* Deal with possible weird situations. */
+    if (img->numsubimages > 0 && numsubs != img->numsubimages) {
+	run_warning("Going from %d to %d subimages in %dx%d image of \"%s\"",
+		    img->numsubimages, numsubs, w, h, imf->name);
+	img->subimages = NULL;
+    }
+    img->numsubimages = numsubs;
+    if (img->subimages == NULL) {
+	img->subimages = (Image **) xmalloc(numsubs * sizeof(Image *));
+	for (subi = 0; subi < numsubs; ++subi) {
 	    subimg = get_subimg(imf, img->w, img->h);
 	    img->subimages[subi] = subimg;
 	}
-    } else {
-	if (img->numsubimages > 0) {
-	    img->subimages =
-	      (Image **) xmalloc(img->numsubimages * sizeof(Image *));
-	    for (subi = 0; subi < img->numsubimages; ++subi) {
-		subimg = get_subimg(imf, img->w, img->h);
-		img->subimages[subi] = subimg;
-	    }
-	}
     }
 }
 
@@ -768,7 +755,7 @@ add_shrunken_image(ImageFamily *imf)
     img = smallest_image(imf);
     /* Don't try to shrink tiles or already-shrunken images or very
        small images. */
-    if (img->istile || img->synthetic || img->w <= 8 || img->h <= 8)
+    if (img->istile /*|| img->synthetic*/ || img->w <= 8 || img->h <= 8)
       return NULL;
     img2 = get_img(imf, img->w / 2, img->h / 2);
     /* Chances are that any embedded subimage will become unrecognizable,
@@ -989,84 +976,6 @@ interp_bytes(Obj *datalist, int numbytes, char *destaddr, int jump)
     }
 }
 
-ImagePalette *
-interp_palette(Obj *form)
-{
-    Obj *elts;
-    ImagePalette *imp;
-
-    if (stringp(cadr(form))) {
-	imp = get_imp(c_string(cadr(form)));
-	elts = cddr(form);
-	if (consp(car(elts))
-	    && symbolp(car(car(elts)))) {
-	    if (match_keyword(car(car(elts)), K_NOTES)) {
-		imp->notes = cadr(car(elts));
-	    } else {
-	    }
-	    elts = cdr(elts);
-	}
-	return imp;
-    }
-    return NULL;
-}
-
-ImagePalette *
-new_image_palette(char *name)
-{
-    ImagePalette *imp;
-
-    imp = (ImagePalette *) xmalloc(sizeof(ImagePalette));
-    imp->name = name;
-    imp->notes = lispnil;
-    imp->palette = lispnil;
-    return imp;
-}
-
-char *
-canonical_palette_name(char *str)
-{
-    return str;
-}
-
-/* Given a name, find or create an image palette with that name. */
-
-ImagePalette *
-get_imp(char *name)
-{
-    ImagePalette *imp = NULL;
-
-    if (name == NULL)
-      return NULL;
-    if (palettes == NULL)
-      palettes =
-	(ImagePalette **) xmalloc(MAXIMAGEPALETTES * sizeof(ImagePalette *));
-    if ((imp = find_imp(name)) == NULL) {
-	if (numpalettes >= MAXIMAGEPALETTES)
-	  return NULL;
-	imp = new_image_palette(canonical_palette_name(name));
-	if (imp != NULL) {
-	    palettes[numpalettes++] = imp;
-	}
-    }
-    return imp;
-}
-
-/* Find the image palette of the given name, if it exists. */
-
-ImagePalette *
-find_imp(char *name)
-{
-    int i;
-
-    for (i = 0; i < numpalettes; ++i) {
-	if (strcmp(name, palettes[i]->name) == 0)
-	  return palettes[i];
-    }
-    return NULL;
-}
-
-
 /* Try to find the best of multiple images for the given bounding box.
    Don't return anything that won't fit in min space. */
 
@@ -1238,23 +1147,6 @@ sort_all_images(void)
     qsort(&(images[0]), numimages, sizeof(ImageFamily *), image_name_compare);
 }
 
-/* The comparison function for the palette list just does "strcmp" order
-   and *requires* that all image palettes be named and named uniquely. */
-
-static int
-palette_name_compare(CONST void *imp1, CONST void *imp2)
-{
-    return strcmp((*((ImagePalette **) imp1))->name,
-		  (*((ImagePalette **) imp2))->name);
-}
-
-void
-sort_all_palettes(void)
-{
-    qsort(&(palettes[0]), numpalettes, sizeof(ImagePalette *),
-	  palette_name_compare);
-}
-
 /* Check Lisp-format and binary-format data for consistency. */
 
 void
@@ -1345,7 +1237,7 @@ write_imf_dir(char *filename, ImageFamily **images, int num)
 void
 write_imf(FILE *fp, ImageFamily *imf)
 {
-    Obj *palent;
+    Obj *palent, *posdata;
     Image *img;
     
     if (fp == NULL || imf == NULL)
@@ -1428,10 +1320,15 @@ write_imf(FILE *fp, ImageFamily *imf)
 	    } else if (img->filedata != lispnil) {
 		fprintf(fp, " (%s ", keyword_name(K_FILE));
 		fprintf(fp, " \"%s\"", c_string(car(img->filedata)));
-		if (cdr(img->filedata) != lispnil)
-		  fprintf(fp, " %d %d",
-			  c_number(cadr(img->filedata)),
-			  c_number(caddr(img->filedata)));
+		posdata = cdr(img->filedata);
+		if (posdata != lispnil) {
+		    if (symbolp(car(posdata))) {
+			fprintf(fp, " %s", c_string(car(posdata)));
+			posdata = cdr(posdata);
+		    }
+		    fprintf(fp, " %d %d",
+			    c_number(car(posdata)), c_number(cadr(posdata)));
+		}
 		fprintf(fp, ")");
 	    } else if ((img->colrdata != lispnil || img->rawcolrdata)
 			&& !color_matches_mono(img)) {
@@ -1665,9 +1562,7 @@ write_palette_contents(FILE *fp, Obj *palette, int *rawpalette, int numcolors)
     if (len > 2)
       fprintf(fp, "\n  ");
     fprintf(fp, " (%s", keyword_name(K_PALETTE));
-    if (symbolp(palette)) {
-	fprintf(fp, " %s", c_string(palette));
-    } else if (palette != lispnil) {
+    if (palette != lispnil) {
 	for_all_list(palette, restpal) {
 	    parse_lisp_palette_entry(car(restpal), &col, &red, &grn, &blu);
 	    if (len > 2)
@@ -1707,26 +1602,6 @@ write_color(FILE *fp, int n, int r, int g, int b)
       fprintf(fp, ")");
 }
 
-void
-write_imp(FILE *fp, ImagePalette *imp)
-{
-    if (imp == NULL)
-      return;
-    if (imp->name == NULL) {
-	fprintf(fp, "; garbage palette?\n");
-	return;
-    }
-    fprintf(fp, "(%s \"%s\"",
-	    keyword_name(K_PALETTE), imp->name);
-    if (imp->notes != lispnil) {
-	fprintf(fp, "\n  (%s ", keyword_name(K_NOTES));
-	fprintlisp(fp, imp->notes);
-	fprintf(fp, ")\n ");
-    }
-    write_palette_contents(fp, imp->palette, NULL, 0);
-    fprintf(fp, ")\n");
-}
-
 /* Given rgb components, return names of standard colors if the match
    is close. */
 
@@ -1828,8 +1703,9 @@ void
 make_image_from_file_image(ImageFamily *imf, Image *img, Image *subimg,
 			   int subi)
 {
-    int hch, pad = 2;
+    int hch, pad = 2, stdlayout = FALSE;
     int xoffset, yoffset, xoff, yoff;
+    Obj *posdata;
     FileImage *fimg;
 
     if (img->filedata != lispnil) {
@@ -1845,9 +1721,19 @@ make_image_from_file_image(ImageFamily *imf, Image *img, Image *subimg,
 	return;
     }
     xoffset = yoffset = 0;
-    if (cdr(img->filedata) != lispnil) {
-	xoffset = c_number(cadr(img->filedata));
-	yoffset = c_number(caddr(img->filedata));
+    stdlayout = FALSE;
+    posdata = cdr(img->filedata);
+    if (posdata != lispnil) {
+	if (match_keyword(car(posdata), K_STD)) {
+	    stdlayout = TRUE;
+	    posdata = cdr(posdata);
+	}
+	xoffset = c_number(car(posdata));
+	yoffset = c_number(cadr(posdata));
+	if (stdlayout) {
+	    xoffset = (xoffset * (img->w + 2)) + 2;
+	    yoffset = (yoffset * (img->h + 2)) + 2;
+	}
     }
     /* All subimages share color data. */
     if (subimg == img ||
