@@ -18,18 +18,23 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 #include "kernel.h"
-#include "ai.h"
 #include "version.h"
 
 static void cache_possible_types(void);
 static void kill_excess_units(void);
 static void trial_assign_player(int i, Player *player);
-static void init_view_2(Side *side, int terrainset);
 static void init_view_cell(int x, int y);
 static void maybe_init_view_cell(int x, int y);
 static int adj_seen_terrain(int x, int y, Side *side);
 static void init_ranged_views(void);
-static int indepside_needed(void);
+static void init_side_advantage(Side *side);
+static int make_random_date(int calls, int runs);
+static int make_weather(int calls, int runs);
+static void configure_sides(void);
+static void init_all_views(void);
+static int make_initial_materials(int calls, int runs);
+static void init_side_balance(void);
+static void check_consistency(void);
 
 /* This is true after the game is totally synthesized, but players may
    not all be in the game yet. */
@@ -222,6 +227,56 @@ set_g_synth_methods_default(void)
     set_g_synth_methods(synthlist);
 }
 
+/* Game setup happens in several stages. */
+
+void
+start_game_load_stage(void)
+{
+    current_stage = game_load_stage;
+}
+
+/* Advance to variant setting. */
+
+void
+start_variant_setup_stage(void)
+{
+    current_stage = variant_setup_stage;
+    /* Advance everybody else to this stage. */
+    if (my_rid > 0 && my_rid == master_rid)
+      broadcast_start_variant_setup();
+}
+
+void
+start_player_pre_setup_stage(void)
+{
+    do_module_variants(mainmodule, lispnil);
+    /* Stop accepting variant settings. */
+    current_stage = player_pre_setup_stage;
+    if (my_rid > 0 && my_rid == master_rid)
+      broadcast_variants_chosen();
+}
+
+void
+start_player_setup_stage(void)
+{
+    make_trial_assignments();
+    /* Start accepting player settings. */
+    current_stage = player_setup_stage;
+    /* Advance everybody else to this stage. */
+    if (my_rid > 0 && my_rid == master_rid)
+      broadcast_start_player_setup();
+}
+
+void
+start_game_ready_stage(void)
+{
+    /* Stop accepting player settings. */
+    current_stage = game_ready_stage;
+    /* Advance everybody else to this stage. */
+    if (my_rid > 0 && my_rid == master_rid)
+      broadcast_players_assigned();
+}
+
 /* Run a doublecheck on plausibility of game parameters.  Additional
    checks are performed elsewhere as needed, for instance during
    random generation.  Serious mistakes exit now, since they can cause
@@ -279,6 +334,11 @@ check_game_validity(void)
     /* Check that all the unit names and chars are distinct. */
     for_all_unit_types(u1) {
 	for_all_unit_types(u2) {
+
+#if 0	    /* It may be desirable to have several unit types with the same
+	    name, e.g. to make it possible to use different unit images for
+	    different sides. */
+
 	    /* Only do "upper triangle" of utype x utype matrix. */
 	    if (u1 < u2) {
 		if (strcmp(u_type_name(u1), u_type_name(u2)) == 0) {
@@ -288,6 +348,8 @@ check_game_validity(void)
 		    /* This is bad but not disastrous, so don't fail. */
 		}
 	    }
+#endif
+
 	    if (uu_zoc_range(u1, u2) > max_zoc_range) {
 		max_zoc_range = uu_zoc_range(u1, u2);
 	    }
@@ -318,7 +380,7 @@ check_game_validity(void)
 	    failed = TRUE;
 	}
 	/* should be part of general bounds check */
-	if (u_cp(u1) <= 0) {
+	if (u_cp(u1) < 0) {
 	    init_error("unit type %d has nonpositive cp", u1);
 	    failed = TRUE;
 	}
@@ -375,8 +437,10 @@ check_game_validity(void)
 void
 calculate_globals(void)
 {
+    Side *side;
+
     /* The game is now completely defined; no further user-specified
-       changes can occur. */
+       changes (such as variants or player choices) can occur. */
     gamedefined = TRUE;
     calculate_world_globals();
     /* This needs to be precalculated instead of as-needed, since
@@ -394,7 +458,13 @@ calculate_globals(void)
 		  max(range, max_detonate_on_approach_range);
 	    }
 	}
-    }	
+    }
+    /* Set up the cache of what each side can build.  We do it here
+       because if it's too early, type_allowed_on_side will get filled
+       in with values that don't reflect side props such as its class. */
+    for_all_sides(side) {
+	update_canbuild_vector(side);    	 
+    }
 }
 
 /* Clean up all the objects and cross-references. NOTE: this may be
@@ -711,7 +781,7 @@ make_trial_assignments(void)
 
 /* Returns true if indepside already has units or may get units made. */
 
-static int
+int
 indepside_needed(void)
 {
     Unit *unit;
@@ -868,13 +938,22 @@ set_player_advantage(int n, int newadv)
 }
 
 void
-rename_side_for_player(int n)
+rename_side_for_player(int n, int which)
 {
+    Obj *entry;
     Side *side;
 
     side = assignments[n].side;
     side->name = side->noun = side->pluralnoun = side->adjective = NULL;
-    make_up_side_name(side);
+    if (which >= 0) {
+	entry = elt(g_side_lib(), which);
+	/* Take away the optional weighting number. */
+	if (numberp(car(entry)))
+	  entry = cdr(entry);
+	fill_in_side(side, entry, FALSE);
+    } else {
+	make_up_side_name(side);
+    }
 }
 
 /* Given the name of an AI type, assign it to the player at the given
@@ -1105,7 +1184,7 @@ final_init(void)
     Side *side;
 
     /* Fill in any empty doctrines. */
-    for_all_sides_plus_indep(side) {
+    for_all_sides(side) {
 	init_doctrine(side);
     }
     /* Make sure each side has a self-unit if it needs one. */
@@ -1141,6 +1220,10 @@ final_init(void)
     Dprintf("One side is %d bytes.\n", sizeof(Side));
     Dprintf("One unit is %d bytes, one plan is %d bytes.\n",
 	    sizeof(Unit), sizeof(Plan));
+    Dprintf("One task is %d bytes.\n", sizeof(Task));
+    Dprintf("One unit view is %d bytes.\n", sizeof(UnitView));
+    Dprintf("One hist event is %d bytes, one past unit is %d bytes.\n",
+	    sizeof(HistEvent), sizeof(PastUnit));
     if (Debug)
       report_malloc();
 #endif /* DEBUGGING */
@@ -1182,19 +1265,34 @@ init_all_views(void)
     int x, y, i = 0, todo = max(1, numtotsides * area.numcells);
     int terrainset;
     Side *side;
+    extern int suppress_see_wakeup;
 
+    suppress_see_wakeup = TRUE;
     /* Set up the basic view structures for all sides first. */
     for_all_sides(side) {
-	if (g_see_all()) {
-	    side->see_all = TRUE;
-	} else {
-	    side->see_all = FALSE;
+	side->see_all = g_see_all();
+	if (!side->see_all) {
 	    terrainset = init_view(side);
-	    init_view_2(side, terrainset);
+	    calc_coverage(side);
+	    tmpside = side;
+	    if (g_terrain_seen()) {
+		for_all_cells(x, y) {
+		    init_view_cell(x, y);
+		}
+	    } else {
+		for_all_cells(x, y) {
+		    if (!terrainset)
+		      set_terrain_view(side, x, y, UNSEEN);
+		    /* (should do already-seen down below?) */
+		    if (terrain_view(side, x, y) != UNSEEN)
+		      init_view_cell(x, y);
+		}
+	    }
 	}
 	/* We're not normally allowed to choose whether to display all
            or not. */
 	side->may_set_show_all = FALSE;
+	/* Display appearance normally matches viewing abilities. */
 	side->show_all = side->see_all;
 	/* Can't think of any other place to put this... */
 	calc_start_xy(side);
@@ -1202,8 +1300,9 @@ init_all_views(void)
     /* Possibly nothing more to do here. */
     if (g_see_all())
       return;
+    /* Now we go to computing the actual view of the units in each
+       cell, as well as the view coverage count (really?). */
     announce_lengthy_process("Computing current view at each location");
-    /* (coverage is also done at this point) */
     for_all_sides(side) {
 	for_all_cells(x, y) {
 	    ++i;
@@ -1214,31 +1313,11 @@ init_all_views(void)
     }
     finish_lengthy_process();
     init_ranged_views();
+    suppress_see_wakeup = FALSE;
 }
 
-static void
-init_view_2(Side *side, int terrainset)
-{
-    int x, y;
-
-    calc_coverage(side);
-    tmpside = side;
-    if (g_terrain_seen()) {
-	for_all_cells(x, y) {
-	    init_view_cell(x, y);
-	}
-    } else {
-	for_all_cells(x, y) {
-	    if (!terrainset)
-	      set_terrain_view(side, x, y, UNSEEN);
-	    set_unit_view(side, x, y, EMPTY);
-	    /* View date is 0, which is what we want. */
-	}
-    }
-}
-
-/* This is a helper for the following routine.  The helper is applied to
-   each cell, decides what is visible in that cell. */
+/* This is a helper routine.  The helper is applied to each cell,
+   decides what is visible in that cell. */
 
 /* This only works from already_seen, does not account for coverage. */
 
@@ -1265,12 +1344,13 @@ init_view_cell(int x, int y)
 		set_cover(tmpside, x, y, 1);
 	    	return;
 	    }
-	    chance = already_seen_chance(tmpside, unit);
-	    if (!tmpside->unit_view_restored && probability(chance)) {
-	    	see_exact(tmpside, x, y);
-	    	/* This view might be overwritten by a view of another
-	    	   might-be-seen unit at this location, so don't
-	    	   return just yet. */
+	    /* Only check already-seen if we didn't read in a set of
+	       unit views already. */
+	    if (!tmpside->unit_view_restored) {
+		chance = already_seen_chance(tmpside, unit);
+		if (probability(chance)) {
+		    add_unit_view(tmpside, unit);
+		}
 	    }
 	}
     }
@@ -1279,6 +1359,9 @@ init_view_cell(int x, int y)
       set_wind_view(tmpside, x, y, raw_wind_at(x, y));
 }
 
+/* Get the already-seen chance from either the side, if defined, or
+   from the unit type. */
+
 int
 already_seen_chance(Side *side, Unit *unit)
 {
@@ -1447,6 +1530,8 @@ init_supply(Unit *unit)
     }
 }
 
+#if 0		/* Unused. */
+
 /* Quicky test needed in a couple places. */
 
 int
@@ -1463,6 +1548,8 @@ saved_game(void)
     }
 }
 
+#endif
+
 /* Count all the initial units in each side's balance sheet, but only
    if all the balance sheets are completely blank. */
 
