@@ -121,6 +121,34 @@ strmgetc(Strm *strm)
 	  ch = *((strm->ptr.sp)++);
     } else {
 	ch = getc(strm->ptr.fp);
+
+/* Handle some non-english characters whose coding differ between unix and
+mac. This is required for the Swedish name generator to return non-garbage
+on the mac. Note 1: The signs that look like < and , are not the normal < and ,.
+Note 2: Typecasting ch to char is necessary here, but it must remain an int to 
+handle EOF correctly on unix. */
+
+#ifdef MAC
+
+	    if ((char) ch == '')
+	        ch = '';
+	    if ((char) ch == '')
+	        ch = '';
+	    if ((char) ch == '')
+	        ch = '';
+	    if ((char) ch == '')
+	        ch = '';
+	    if ((char) ch == '')
+	        ch = '';
+	    if ((char) ch == '')
+	        ch = '';
+	    if ((char) ch == '')
+	        ch = '';
+	    if ((char) ch == '')
+	        ch = '';
+
+#endif
+
     }
     if (ch != EOF) {
 	++(strm->numread);
@@ -629,36 +657,31 @@ cdr(Obj *x)
 }
 
 Obj *
-cadr(x)
-Obj *x;
+cadr(Obj *x)
 {
     return car(cdr(x));
 }
 
 Obj *
-cddr(x)
-Obj *x;
+cddr(Obj *x)
 {
     return cdr(cdr(x));
 }
 
 Obj *
-caddr(x)
-Obj *x;
+caddr(Obj *x)
 {
     return car(cdr(cdr(x)));
 }
 
 Obj *
-cdddr(x)
-Obj *x;
+cdddr(Obj *x)
 {
     return cdr(cdr(cdr(x)));
 }
 
 void
-set_cdr(x, v)
-Obj *x, *v;
+set_cdr(Obj *x, Obj *v)
 {
     if (x->type == CONS) {
 	x->v.cons.cdr = v;
@@ -670,8 +693,7 @@ Obj *x, *v;
 /* Return the string out of both strings and symbols. */
 
 char *
-c_string(x)
-Obj *x;
+c_string(Obj *x)
 {
     if (x->type == STRING) {
 	return x->v.str;
@@ -687,8 +709,7 @@ Obj *x;
 /* Return the actual number in a number object. */
 
 int
-c_number(x)
-Obj *x;
+c_number(Obj *x)
 {
     if (x->type == NUMBER
 	|| x->type == UTYPE
@@ -704,8 +725,7 @@ Obj *x;
 }
 
 Obj *
-intern_symbol(str)
-char *str;
+intern_symbol(char *str)
 {
     int n;
     Symentry *se;
@@ -736,8 +756,7 @@ char *str;
 /* Given a string, try to find a symbol entry with that as its name. */
 
 static Symentry *
-lookup_string(str)
-char *str;
+lookup_string(char *str)
 {
     Symentry *se;
 
@@ -751,7 +770,11 @@ char *str;
 static int
 hash_name(char *str)
 {
-    return str[0];
+    int rslt = 0;
+
+    while (*str != '\0')
+      rslt ^= *str++;
+    return (ABS(rslt) & 0xff);
 }
 
 Obj *
@@ -1190,11 +1213,11 @@ escaped_string(char *str)
 }
 
 #ifdef DEBUGGING
-/* For calling from debuggers, at least that those that support output to stderr. */
+/* For calling from debuggers, at least that those that support output
+   to stderr. */
 
 void
-dlisp(x)
-Obj *x;
+dlisp(Obj *x)
 {
     fprintlisp(stderr, x);
     fprintf(stderr, "\n");
@@ -1202,9 +1225,7 @@ Obj *x;
 #endif /* DEBUGGING */
 
 void
-print_form_and_value(fp, form)
-FILE *fp;
-Obj *form;
+print_form_and_value(FILE *fp, Obj *form)
 {
     fprintlisp(fp, form);
     if (symbolp(form)) {
@@ -1219,8 +1240,7 @@ Obj *form;
 }
 
 Obj *
-append_two_lists(x1, x2)
-Obj *x1, *x2;
+append_two_lists(Obj *x1, Obj *x2)
 {
     if (!listp(x1))
       x1 = cons(x1, lispnil);
@@ -1236,8 +1256,7 @@ Obj *x1, *x2;
 }
 
 Obj *
-append_lists(lis)
-Obj *lis;
+append_lists(Obj *lis)
 {
     if (lis == lispnil) {
 	return lispnil;
@@ -1251,8 +1270,7 @@ Obj *lis;
 /* Remove all occurrences of a single object from a given list. */
 
 Obj *
-remove_from_list(elt, lis)
-Obj *elt, *lis;
+remove_from_list(Obj *elt, Obj *lis)
 {
     Obj *tmp;
 
