@@ -16,7 +16,10 @@ any later version.  See the file COPYING.  */
 #include "lisp.h"
 #include "module.h"
 #include "system.h"
+
+#if 0  /* Unused. */
 void mac_abort(void);
+#endif
 
 /* Following are from tp.h, but tp.h is too high-level to include. */
 
@@ -31,15 +34,16 @@ extern void close_file_port(void);
 extern void low_file_send(int id, char *buf);
 extern int low_file_receive(int *id, char *buf, int maxchars, int timeout);
 
-extern int serial_port_dialog(void);
+/* Private functions. */
 
 static void init_serial_port(int port);
 static void close_serial_port(void);
-
 static void open_appletalk(void);
-extern int low_appletalk_receive(int *id, char *buf, int maxchars, int timout);
+static int low_appletalk_receive(int *id, char *buf, int maxchars, int timout);
 static void low_appletalk_send(int id, char *buf);
 static void close_appletalk(void);
+static void low_serial_send(int id, char *buf);
+static int low_serial_readchars(char *buf, int maxchars, int timeout);
 
 #ifdef THINK_C
 #include <MacHeaders>
@@ -109,9 +113,6 @@ extern CursHandle sendcursor;
 extern CursHandle receivecursor;
 extern CursHandle current_cursor;
 
-static void low_serial_send(int id, char *buf);
-static int low_serial_readchars(char *buf, int maxchars, int timeout);
-
 int connection_method;
 
 /* The HFS volume that the program started with. */
@@ -409,9 +410,7 @@ open_scorefile_for_reading(char *name)
 	
     GetVol(NULL, &curvrefnum);
     SetVol(NULL, initialvrefnum);
-    /* Now try to open the file. */
-    sprintf(buf, ":Scores:%s", name);
-    fp = open_file(buf, "r");
+    fp = open_file(name, "r");
     SetVol(NULL, curvrefnum);
     return fp;
 }
@@ -507,6 +506,8 @@ record_ms()
     last_ticks_for_ms = TickCount();
 }
 
+#if 0		/* Not called from anywhere. */
+
 /* Instead of coredumping, which is not a normal Mac facility, we
    drop into Macsbug.  If we then "g" from Macsbug, the program will
    exit cleanly. */
@@ -529,6 +530,8 @@ mac_abort ()
     exit(1);
 }
 
+#endif
+
 int
 open_remote_connection(char *methodname, int willhost)
 {
