@@ -1,5 +1,5 @@
 /* Random definitions used everywhere in Xconq.
-   Copyright (C) 1987-1989, 1991-1997 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1997, 1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -13,27 +13,10 @@ any later version.  See the file COPYING.  */
 #define FALSE (0)
 #endif
 
-/* This is how we do optional prototypes and const decls. */
-
-#ifdef MSDOS
-#define PARAMS(ARGS) ARGS
-#define CONST const
-#endif
-
-#ifndef PARAMS
-#ifdef ANSI_PROTOTYPES
-#define PARAMS(ARGS) ARGS
-#else
-#define PARAMS(ARGS) ()
-#endif /* ANSI_PROTOTYPES */
-#endif /* PARAMS */
+/* This is how we do optional const decls. */
 
 #ifndef CONST
-#ifdef __STDC__
 #define CONST const
-#else
-#define CONST
-#endif /* __STDC__ */
 #endif /* CONST */
 
 #ifndef ABS
@@ -132,32 +115,43 @@ typedef struct a_library_path {
 extern LibraryPath *xconq_libs;
 extern LibraryPath *last_user_xconq_lib;
 
-extern char *getenv();
+extern void init_xrandom(int seed);
+extern int xrandom(int m);
+extern int probability(int prob);
+extern int roll_dice(int n);
+extern int multiply_dice(int dice, int mult);
+extern int prob_fraction(int n);
 
-extern void init_xrandom PARAMS ((int seed));
-extern int xrandom PARAMS ((int m));
-extern int probability PARAMS ((int prob));
-extern int roll_dice PARAMS ((int n));
-extern int multiply_dice PARAMS ((int dice, int mult));
-extern int prob_fraction PARAMS ((int n));
+extern char *xmalloc(int amt);
+extern void report_malloc(void);
+extern void tprintf(char *buf, char *str, ...);
+extern void tnprintf(char *buf, int n, char *str, ...);
+extern int select_by_weight(int *arr, int numvals);
+extern char *copy_string(char *str);
+extern char *pad_blanks(char *str, int n);
+extern int iindex(int ch, char *str);
+extern long idifftime(time_t t1, time_t t0);
+extern void case_panic(char *str, int var);
+extern int isqrt(int i);
+extern void init_debug_to_stdout(void);
+extern void update_debugging(void);
+extern void toggle_debugging(int *flagp);
+extern void debug_printf(char *str, ...);
+extern void debugm_printf(char *str, ...);
+extern void debugg_printf(char *str, ...);
 
-extern char *xmalloc PARAMS ((int amt));
-extern void report_malloc PARAMS ((void));
-extern void tprintf PARAMS ((char *buf, char *str, ...));
-extern void tnprintf PARAMS ((char *buf, int n, char *str, ...));
-extern char *copy_string PARAMS ((char *str));
-extern char *pad_blanks PARAMS ((char *str, int n));
-extern int iindex PARAMS ((int ch, char *str));
-extern long idifftime PARAMS ((time_t t1, time_t t0));
-extern void case_panic PARAMS ((char *str, int var));
-extern int isqrt PARAMS ((int i));
-extern void init_debug_to_stdout PARAMS ((void));
-extern void update_debugging PARAMS ((void));
-extern void toggle_debugging PARAMS ((int *flagp));
-extern void debug_printf PARAMS ((char *str, ...));
-extern void debugm_printf PARAMS ((char *str, ...));
-extern void debugg_printf PARAMS ((char *str, ...));
+extern void prealloc_debug(void);
 
-extern void prealloc_debug PARAMS ((void));
+extern void record_activity_start(char *type, int detail);
+extern void record_activity_end(char *type, int detail);
+extern void dump_activity_trace(void);
 
-extern void vtprintf PARAMS ((char *buf, char *str, va_list ap));
+extern void vtprintf(char *buf, char *str, va_list ap);
+
+extern void log_warning(char *typ, char *str);
+
+/* Needed for emergency game savers. */
+extern int write_entire_game_state(char *fname);
+
+/* New wrapper for fopen defined in mac.c and unix.c. */
+extern FILE *open_file(char *filename, char *mode);
