@@ -20,6 +20,10 @@ Obj *lastnamer;
 static int total_side_weights;
 static int total_color_weights;
 
+static char *name_from_grammar(Obj *grammar);
+static void gen_name(Obj *nonterm, Obj *rules, int depth, char *rslt);
+static void gen_from_rule(Obj *rule, Obj *rules, int depth, char *rslt);
+
 void
 init_namers(void)
 {
@@ -320,21 +324,25 @@ name_from_grammar(Obj *grammar)
 void
 gen_name(Obj *nonterm, Obj *rules, int depth, char *rslt)
 {
-    Obj *rest;
+    Obj *rest, *rule;
 
+    /* Look for the first rule whose head matches the nonterminal. */
     for_all_list(rules, rest) {
-	if (equal(nonterm, car(car(rest)))) {
-	    gen_from_rule(cadr(car(rest)), rules, depth, rslt);
+	rule = car(rest);
+	if (equal(nonterm, car(rule))) {
+	    gen_from_rule(cadr(rule), rules, depth, rslt);
 	    return;
 	}
     }
+    /* If we didn't find the symbol mentioned as a nonterminal, see if
+       it's a namer itself, and run it if so. */
     if (symbolp(nonterm)
 	&& boundp(nonterm)
 	&& pointerp(symbol_value(nonterm))) {
 	strcat(rslt, run_namer(symbol_value(nonterm)));
     } else {
 	/* Assume that the purported nonterm symbol is actually a
-           terminal. */
+           terminal and just concatenate it. */
 	strcat(rslt, c_string(nonterm));
     }
 }
@@ -345,66 +353,105 @@ gen_name(Obj *nonterm, Obj *rules, int depth, char *rslt)
 void
 gen_from_rule(Obj *rule, Obj *rules, int depth, char *rslt)
 {
-    int total, oldlen;
-    Obj *rest, *subrule;
+    int total, oldlen, maxtries = 100, tries, found;
+    char buf[BUFSIZE];
+    char *rejstr;
+    Obj *rest, *subrule, *rejrest;
     
     if (depth >= maxdepth)
       return;
     switch (rule->type) {
-      case NUMBER:
       case NIL:
-      case UTYPE:
-      case MTYPE:
-      case TTYPE:
-      case ATYPE:
-      case POINTER:
-      case EOFOBJ:
-	break;  /* ignore for now.. (but should warn?) */
-      case SYMBOL:
-	gen_name(rule, rules, depth, rslt);
+	/* Assume an empty rule body to be a deliberate no-op. */
 	break;
       case STRING:
-	/* Strings are terminals, just append them directly. */
+	/* Strings are always terminals, just append them directly. */
 	strcat(rslt, c_string(rule));
 	break;
+      case SYMBOL:
+	/* Assume that a symbol might be a nonterminal, and recurse. */
+	gen_name(rule, rules, depth, rslt);
+	break;
       case CONS:
         if (symbolp(car(rule))) {
 	    switch (keyword_code(c_string(car(rule)))) {
 	      case K_OR:
 		/* Make a weighted selection from the alternatives. */
 		total = 0;
-		subrule = choose_from_weighted_list(rule, &total, TRUE);
+		subrule = choose_from_weighted_list(cdr(rule), &total, TRUE);
 		if (subrule != lispnil)
 		  gen_from_rule(subrule, rules, depth + 1, rslt);
 		break;
-	      case K_ANY:
-		/* uniform selection */
-		strcat(rslt, c_string(elt(cdr(rule),
-					  xrandom(length(cdr(rule))))));
-		break;
 	      case K_CAPITALIZE:
+		/* Run the rule, then capitalize the result. */
+		if (cdr(rule) == lispnil) {
+		    run_warning("`capitalize' form missing argument");
+		    break;
+		}
 		oldlen = strlen(rslt);
-		gen_name(cadr(rule), rules, depth + 1, rslt);
+		gen_from_rule(cadr(rule), rules, depth + 1, rslt);
 		if (islower(rslt[oldlen]))
 		  rslt[oldlen] = uppercase(rslt[oldlen]);
 		break;
+	      case K_REJECT:
+		/* Run the rule, then retry if the result includes any
+		   substrings that we want to exclude. */
+		if (cdr(rule) == lispnil) {
+		    run_warning("`reject' form missing argument");
+		    break;
+		}
+		for (tries = 0; tries < maxtries; ++tries) {
+		    /* Only look at the just-generated part of the result. */
+		    oldlen = strlen(rslt);
+		    /* Generate a name to test. */
+		    gen_from_rule(cadr(rule), rules, depth + 1, rslt);
+		    found = FALSE;
+		    for_all_list(cddr(rule), rejrest) {
+			if (symbolp(car(rejrest)) || stringp(car(rejrest))) {
+			    rejstr = c_string(car(rejrest));
+			    if (!empty_string(rejstr)
+				&& strstr(rslt + oldlen, rejstr)) {
+				found = TRUE;
+				break;
+			    }
+			}
+		    }
+		    if (found) {
+			if (tries >= maxtries - 1) {
+			    sprintlisp(buf, rule, BUFSIZE);
+			    run_warning("Grammar rule `%s' still got rejects"
+					" after %d tries, accepting last try",
+					buf, maxtries - 1);
+			    break;
+			}
+			/* "Erase" result and start over. */
+			rslt[oldlen] = '\0';
+		    } else {
+			/* Name looks good, we're done here. */
+			break;
+		    }
+		}
+		break;
 	      default:
 		/* Assume anything else to be a nested subsequence. */
 		for_all_list(rule, rest)
 		  gen_from_rule(car(rest), rules, depth + 1, rslt);
+		break;
 	    }
 	} else if (stringp(car(rule))) {
 	    /* Assume a nested subsequence. */
 	    for_all_list(rule, rest)
 	      gen_from_rule(car(rest), rules, depth + 1, rslt);
 	} else {
-	    char buf[BUFSIZE];
 	    sprintlisp(buf, rule, BUFSIZE);
-	    run_warning("Grammar rule `%s' does not begin with a symbol", buf);
+	    run_warning("Grammar rule `%s' does not begin"
+			" with a symbol or string",
+			buf);
 	}
 	break;
       default:
-	case_panic("lisp type in grammar", rule->type);
+	sprintlisp(buf, rule, BUFSIZE);
+	run_warning("`%s' is not a valid grammar rule", buf);
 	break;
     }
 }
