@@ -136,6 +136,10 @@ make_roads(int calls, int runs)
     return TRUE;
 }
 
+/* Test whether we expect that units on different sides can see each
+   other.  This has special cases so this works during game setup,
+   when the usual unit view layers may not be accurate yet. */
+
 static int
 unit_sees_other_unit(Unit *unit1, Unit *unit2)
 {
@@ -147,9 +151,8 @@ unit_sees_other_unit(Unit *unit1, Unit *unit2)
        a road going to it) */
     if (indep(unit1))
       return TRUE;
-    /* This works sometimes, but views aren't always completely set up yet. */
-    if (side->unitview != NULL
-	&& unit_view(side, unit2->x, unit2->y) != EMPTY)
+    /* Run a normal unit view test to see if we see anything. */
+    if (unit_view_at(side, unit2->x, unit2->y) != NULL)
       return TRUE;
     /* If the unit view hasn't been set up yet, but we anticipate that the
        unit in question will be visible. */
