@@ -1,5 +1,5 @@
 /* Terrain generation for Xconq.
-   Copyright (C) 1986-1989, 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1986-1989, 1991-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -11,37 +11,39 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 #include "kernel.h"
 
-static void make_blobs PARAMS ((short *layer, int numblobs, int blobradius, int blobalt));
-static void limit_layer PARAMS ((short *layer, int hi, int lo));
-static void smooth_layer PARAMS ((short *layer, int times));
-static void percentile PARAMS ((short *layer, int *percentiles));
-static void compose_area PARAMS ((void));
-static int terrain_from_percentiles PARAMS ((int x, int y));
-static void make_earthlike_fractal_terrain PARAMS ((int calls, int runs));
-static void compose_earthlike_area PARAMS ((void));
-static int earthlike_terrain_from_percentiles PARAMS ((int x, int y));
-static void dig_maze_path PARAMS ((int x1, int y1, int dir1));
-static void dig_maze_path_between PARAMS ((int x1, int y1, int x2, int y2));
-static int test_maze_segment PARAMS ((int x, int y, int dir));
-static int sort_maze_segments PARAMS ((int x, int y, int *dirchoices,
-				       int numchoices));
-static int lay_maze_segment PARAMS ((int x, int y, int dir, int choice,
-				     int numchoices));
-static int num_open_adj PARAMS ((int x, int y));
-static int random_solid_terrain PARAMS ((void));
-static int random_room_terrain PARAMS ((void));
-static int random_passage_terrain PARAMS ((void));
-static int high_point PARAMS ((int x, int y));
-static int water_point PARAMS ((int x, int y));
-static int bay_point PARAMS ((int x, int y));
-static void set_room_interior PARAMS ((int x, int y));
-static void name_highest_peaks PARAMS ((Obj *parms));
-static void name_lakes PARAMS ((Obj *parms));
-static char *name_feature_at PARAMS ((int x, int y, char *typename));
-static char *name_feature_at_using PARAMS ((Obj *namerlist, int x, int y, char *typename));
-static void fix_adjacent_terrain PARAMS ((void));
-static void flatten_liquid_terrain PARAMS ((void));
-static void set_edge_values PARAMS ((int x, int y, int t));
+static void make_blobs(short *layer, int numblobs, int blobradius,
+		       int blobalt);
+static void limit_layer(short *layer, int hi, int lo);
+static void smooth_layer(short *layer, int times);
+static void percentile(short *layer, int *percentiles);
+static void compose_area(void);
+static int terrain_from_percentiles(int x, int y);
+static void make_earthlike_fractal_terrain(int calls, int runs);
+static void compose_earthlike_area(void);
+static int earthlike_terrain_from_percentiles(int x, int y);
+static void dig_maze_path(int x1, int y1, int dir1);
+static void dig_maze_path_between(int x1, int y1, int x2, int y2);
+static int test_maze_segment(int x, int y, int dir);
+static int sort_maze_segments(int x, int y, int *dirchoices, int numchoices);
+static int lay_maze_segment(int x, int y, int dir, int choice, int numchoices);
+static int num_open_adj(int x, int y);
+static int random_solid_terrain(void);
+static int random_room_terrain(void);
+static int random_passage_terrain(void);
+static void set_room_interior(int x, int y);
+static void name_highest_peaks(char *specificname, Obj *parms);
+static int high_point(int x, int y);
+static void name_islands(char *specificname, Obj *parms);
+static void name_lakes(char *specificname, Obj *parms);
+static void name_bays(char *specificname, Obj *parms);
+static int bay_point(int x, int y);
+static char *name_feature_at(int x, int y, char *typename);
+static char *name_feature_at_using(Obj *namerlist, int x, int y,
+				   char *typename);
+static void fix_adjacent_terrain(void);
+static void flatten_liquid_terrain(void);
+static void set_edge_values(int x, int y, int t);
+static void set_edge_elevation(int x, int y);
 
 /* Fractal terrain generation. */
 
@@ -76,9 +78,10 @@ static int numholes;
 
 /* Should add a cheap erosion simulator. */
 
+static void add_random_changes(void);
+
 int
-make_fractal_terrain(calls, runs)
-int calls, runs;
+make_fractal_terrain(int calls, int runs)
 {
     int actualcells, altnumblobs, altblobradius, altblobalt;
     int wetnumblobs, wetblobradius, wetblobalt;
@@ -133,6 +136,7 @@ int calls, runs;
     /* Put it all together. */
     partdone += stepsize;
     compose_area();
+    add_random_changes();
     fix_adjacent_terrain();
     add_edge_terrain();
     flatten_liquid_terrain();
@@ -150,9 +154,7 @@ int calls, runs;
 }
 
 static void
-make_blobs(layer, numblobs, blobradius, blobalt)
-short *layer;
-int numblobs, blobradius, blobalt;
+make_blobs(short *layer, int numblobs, int blobradius, int blobalt)
 {
     int x0, y0, x1, y1, x2, y2, updown, x, y, xw;
     int maxdz, i, dz, oz;
@@ -211,9 +213,7 @@ int numblobs, blobradius, blobalt;
 /* Ensure that area values stay within given range. */
 
 static void
-limit_layer(layer, hi, lo)
-short *layer;
-int hi, lo;
+limit_layer(short *layer, int hi, int lo)
 {
     int x, y, m;
     
@@ -226,9 +226,7 @@ int hi, lo;
 /* Average each cell with its neighbors, using tmp3 as scratch layer. */
 
 static void
-smooth_layer(layer, times)
-short *layer;
-int times;
+smooth_layer(short *layer, int times)
 {
     int i, x, y, nx, px, dir, x1, y1, ndirs, sum;
 
@@ -280,9 +278,7 @@ int times;
    value, and save them all away.  */
 
 static void
-percentile(layer, percentiles)
-short *layer;
-int *percentiles;
+percentile(short *layer, int *percentiles)
 {
     int i, x, y, total;
     
@@ -310,7 +306,7 @@ int *percentiles;
 /* Final creation and output of the area. */
 
 static void
-compose_area()
+compose_area(void)
 {
     int x, y, t, t2, t3;
     int elev, elevrange[MAXTTYPES], elevlo[MAXTTYPES], elevhi[MAXTTYPES];
@@ -399,8 +395,7 @@ compose_area()
    there. */
 
 static int
-terrain_from_percentiles(x, y)
-int x, y;
+terrain_from_percentiles(int x, int y)
 {
     int t, rawalt = tmp1_at(x, y), rawwet = tmp2_at(x, y);
 
@@ -422,11 +417,10 @@ int x, y;
    random elevations. */
 
 int
-make_random_terrain(calls, runs)
-int calls, runs;
+make_random_terrain(int calls, int runs)
 {
     int t, x, y, cellsum, cellsumtable[MAXTTYPES];
-    int i, numcells, n, elevrange, occur, dir;
+    int i, numcells, n, elevrange, elev, occur, dir;
 
     if (terrain_defined())
       return FALSE;
@@ -456,7 +450,8 @@ int calls, runs;
 	set_terrain_at(x, y, t);
 	if (elevations_defined() && !world_is_flat()) {
 	    elevrange = t_elev_max(t) - t_elev_min(t) + 1;
-	    set_elev_at(x, y, xrandom(elevrange) + t_elev_min(t));
+	    elev = xrandom(elevrange) + t_elev_min(t);
+	    set_elev_at(x, y, elev);
 	}
     }
     /* Make sure the edge of the area has something in it. */
@@ -537,7 +532,7 @@ int
 make_earthlike_terrain(calls, runs)
 int calls, runs;
 {
-    int t, x, y, elevrange;
+    int t, elevrange;
     int seafrac, desertfrac, forestfrac;
 
     if (terrain_defined())
@@ -701,8 +696,8 @@ int calls, runs;
 static void
 compose_earthlike_area()
 {
-    int x, y, t, t2, t3;
-    int elev, elevrange[MAXTTYPES], elevlo[MAXTTYPES], elevhi[MAXTTYPES];
+    int x, y, t;
+    int elev, elevrange[MAXTTYPES];
 
     Dprintf("Assigning terrain types to cells...\n");
     /* Make the terrain layer itself. */
@@ -1052,6 +1047,33 @@ random_passage_terrain()
     return (xrandom(numttypes));
 }
 
+static void
+add_random_changes(void)
+{
+    int t1, t2, anychanges = FALSE, x, y, dens;
+
+    /* Might not be anything to do, scan the table. */
+    for_all_terrain_types(t1) {
+	for_all_terrain_types(t2) {
+	    if (tt_density(t1, t2) > 0) {
+		anychanges = TRUE;
+		break;
+	    }
+	}
+    }
+    if (!anychanges)
+      return;
+    for_all_cells(x, y) {
+	for_all_terrain_types(t2) {
+	    dens = tt_density(terrain_at(x, y), t2);
+	    if (dens > 0 && dens > xrandom(10000)) {
+		set_terrain_at(x, y, t2);
+		break;
+	    }
+	}
+    }
+}
+
 /* This method adds some randomly named geographical features. */
 
 /* (This needs to interact properly with convex region finder eventually) */
@@ -1060,8 +1082,8 @@ int
 name_geographical_features(calls, runs)
 int calls, runs;
 {
-    char *classname;
-    Obj *rest;
+    char *classname, *specificname;
+    Obj *rest, *head, *parms;
 
     /* If we got features from file or somewhere, don't overwrite them. */
     if (features_defined())
@@ -1077,18 +1099,33 @@ int calls, runs;
     /* Set up the basic layer of data. */
     init_features();
     /* Scan through list to see what's being requested. */
-    for (rest = g_feature_types(); rest != lispnil; rest = cdr(rest)) {
-	if (consp(car(rest)) && stringp(car(car(rest)))) {
-	    classname = c_string(car(car(rest)));
-	    if (strcmp(classname, "peak") == 0) {
-		name_highest_peaks(cdr(car(rest)));
-	    } else if (strcmp(classname, "lake") == 0) {
-		name_lakes(cdr(car(rest)));
-	    } else {
-		run_warning("Don't know to identify \"%s\" features", classname);
-	    }
+    for_all_list(g_feature_types(), rest) {
+	head = car(rest);
+	if (symbolp(head)) {
+	    classname = c_string(head);
+	    parms = lispnil;
+	} else if (consp(head) && symbolp(car(head))) {
+	    classname = c_string(car(head));
+	    parms = cdr(head);
 	} else {
-	    run_warning("Clause not recognized");
+	    run_warning("Feature type clause not recognized");
+	    continue;
+	}
+	specificname = classname;
+	if (parms != lispnil && symbolp(car(parms))) {
+	    specificname = c_string(car(parms));
+	    parms = cdr(parms);
+	}
+	if (strcmp(classname, "peak") == 0) {
+	    name_highest_peaks(specificname, parms);
+	} else if (strcmp(classname, "island") == 0) {
+	    name_islands(specificname, parms);
+	} else if (strcmp(classname, "lake") == 0) {
+	    name_lakes(specificname, parms);
+	} else if (strcmp(classname, "bay") == 0) {
+	    name_bays(specificname, parms);
+	} else {
+	    run_warning("Don't know to identify \"%s\" features", classname);
 	}
     }
     finish_lengthy_process();
@@ -1098,11 +1135,12 @@ int calls, runs;
 /* Identify the highest high points as "peaks". */
 
 static void
-name_highest_peaks(parms)
+name_highest_peaks(name, parms)
+char *name;
 Obj *parms;
 {
-    int x, y, maxpeaks, numpeaks, *peakx, *peaky, i, lo;
-    char *name;
+    int density, x, y, maxpeaks, numpeaks, *peakx, *peaky, i, lo;
+    char *fname;
     Feature *mountain;
 
     /* If no possibility of peaks, return quietly.  (A warning here seemed like
@@ -1110,9 +1148,13 @@ Obj *parms;
        wasn't actually anything wrong with those.) */
     if (!elevations_defined() || world_is_flat())
       return;
-    /* This gives a peak for about every 20x10 group of cells, or about 10 for a
-       default-sized game, which seems right. */
-    maxpeaks = max(1, area.numcells / 200);
+    /* Default to a peak for about every 20x10 group of cells, or about 10 for a
+       default-sized game. */
+    density = 50;
+    if (consp(parms) && numberp(car(parms))) {
+	density = c_number(car(parms));
+    }
+    maxpeaks = max(1, (area.numcells * density) / 10000);
     peakx = (int *) xmalloc(maxpeaks * sizeof(int));
     peaky = (int *) xmalloc(maxpeaks * sizeof(int));
     numpeaks = 0;
@@ -1139,12 +1181,12 @@ Obj *parms;
 	}
     }
     for (i = 0; i < numpeaks; ++i) {
-	name = name_feature_at(x, y, "peak");
-	if (name == NULL) {
+	fname = name_feature_at(x, y, "peak");
+	if (fname == NULL) {
 	    sprintf(tmpbuf, "Pk %d", elev_at(peakx[i], peaky[i]));
-	    name = copy_string(tmpbuf);
+	    fname = copy_string(tmpbuf);
 	}
-	mountain = create_feature("peak", name);
+	mountain = create_feature("peak", fname);
 	mountain->size = 1;
 	set_raw_feature_at(peakx[i], peaky[i], mountain->id);
     }
@@ -1168,25 +1210,125 @@ int x, y;
 }
 
 static void
-name_lakes(parms)
+name_islands(name, parms)
+char *name;
+Obj *parms;
+{
+    int sizemax, x, y, found;
+    TRegion *region;
+    TRegionLink *neighbor;
+    char *fname;
+    Feature *island;
+    extern TRegion *region_list;
+
+    if (area.regions == NULL) {
+	area.regions = malloc_area_layer(TRegion *);
+	divide_into_regions(area.terrain, area.regions);
+    }
+    sizemax = 30;
+    if (consp(parms) && numberp(car(parms))) {
+	sizemax = c_number(car(parms));
+    }
+    for (region = region_list; region != NULL; region = region->next) {
+	if (region->size > 0
+	    && region->size <= sizemax
+	    && region->ttype == 1) {
+	    found = TRUE;
+	    for (neighbor = region->neighbors; neighbor != NULL; neighbor = neighbor->next) {
+		if (neighbor->neighbor->ttype == 1) {
+		    found = FALSE;
+		    break;
+		}
+	    }
+	    if (found) {
+		fname = name_feature_at(area.width / 2, area.height / 2, name);
+		if (fname != NULL) {
+		    island = create_feature("island", fname);
+		    island->size = 0;
+		    for (x = region->xmin; x <= region->xmax; ++x) {
+			for (y = region->ymin; y <= region->ymax; ++y) {
+			    if (aref(area.regions, x, y) == region) {
+				set_raw_feature_at(x, y, island->id);
+				++(island->size);
+			    }
+			}
+		    }
+		}
+	    }
+	}
+    }
+}
+
+/* Identify lakes by looking for small areas of liquid terrain surrounded by
+   non-liquid terrain. */
+
+static void
+name_lakes(name, parms)
+char *name;
+Obj *parms;
+{
+    int sizemax, found, x, y;
+    TRegion *region;
+    TRegionLink *neighbor;
+    char *fname;
+    Feature *lake;
+    extern TRegion *region_list;
+
+    if (area.regions == NULL) {
+	area.regions = malloc_area_layer(TRegion *);
+	divide_into_regions(area.terrain, area.regions);
+    }
+    sizemax = 30;
+    if (consp(parms) && numberp(car(parms))) {
+	sizemax = c_number(car(parms));
+    }
+    for (region = region_list; region != NULL; region = region->next) {
+	if (region->size > 0
+	    && region->size <= sizemax
+	    && region->ttype == 0) {
+	    found = TRUE;
+	    for (neighbor = region->neighbors; neighbor != NULL; neighbor = neighbor->next) {
+		if (neighbor->neighbor->ttype == 0) {
+		    found = FALSE;
+		    break;
+		}
+	    }
+	    if (found) {
+		fname = name_feature_at(area.width / 2, area.height / 2, name);
+		if (fname != NULL) {
+		    lake = create_feature("lake", fname);
+		    lake->size = 0;
+		    for (x = region->xmin; x <= region->xmax; ++x) {
+			for (y = region->ymin; y <= region->ymax; ++y) {
+			    if (aref(area.regions, x, y) == region) {
+				set_raw_feature_at(x, y, lake->id);
+				++(lake->size);
+			    }
+			}
+		    }
+		}
+	    }
+	}
+    }
+}
+
+/* Identify bays in the world.  Bays are single points where liquid terrain is
+   mostly surrounded by non-liquid terrain. */
+
+static void
+name_bays(name, parms)
+char *name;
 Obj *parms;
 {
     int x, y;
-    char *name;
-    Feature *lake, *bay;
+    char *fname;
+    Feature *bay;
 
     for_all_interior_cells(x, y) {
-	if (water_point(x, y)) {
-	    name = name_feature_at(x, y, "lake");
-	    if (name != NULL) {
-		lake = create_feature("lake", name);
-		lake->size = 1;
-		set_raw_feature_at(x, y, lake->id);
-	    }
-	} else if (bay_point(x, y)) {
-	    name = name_feature_at(x, y, "bay");
-	    if (name != NULL) {
-		bay = create_feature("bay", name);
+	if (bay_point(x, y)) {
+	    fname = name_feature_at(x, y, name);
+	    if (fname != NULL) {
+		bay = create_feature("bay", fname);
 		bay->size = 1;
 		set_raw_feature_at(x, y, bay->id);
 	    }
@@ -1194,43 +1336,28 @@ Obj *parms;
     }
 }
 
-/* True if xy is isolated water. */
-
-static int
-water_point(x, y)
-int x, y;
-{
-    int dir, nx, ny;
-
-    if (strcmp(t_type_name(terrain_at(x, y)), "sea") != 0
-        && strcmp(t_type_name(terrain_at(x, y)), "shallows") != 0)
-      return FALSE;
-    for_all_directions(dir) {
-	point_in_dir(x, y, dir, &nx, &ny);
-	if (terrain_at(x, y) == terrain_at(nx, ny) || t_liquid(terrain_at(nx, ny))) {
-	    return FALSE;
-	}
-    }
-    return TRUE;
-}
-
 static int
 bay_point(x, y)
 int x, y;
 {
-    int dir, nx, ny;
-    int seacount = 0, landcount = 0;
+    int dir, nx, ny, seacount, dir1, dir2;
 
-    if (strcmp(t_type_name(terrain_at(x, y)), "shallows") != 0)
+    if (!t_liquid(terrain_at(x, y)))
       return FALSE;
+    seacount = 0;
     for_all_directions(dir) {
 	point_in_dir(x, y, dir, &nx, &ny);
-	if (strcmp(t_type_name(terrain_at(x, y)), "sea") == 0)
-	  ++seacount;
-	else
-	  ++landcount;
+	if (t_liquid(terrain_at(nx, ny))) {
+	    ++seacount;
+	    if (seacount == 1)
+	      dir1 = dir;
+	    if (seacount == 2)
+	      dir2 = dir;
+	}
     }
-    return (seacount > 0 && landcount > 2);
+    return (seacount == 1
+	    || (seacount == 2
+		&& (dir2 == left_dir(dir1) || dir2 == right_dir(dir1))));
 }
 
 static char *
@@ -1268,16 +1395,17 @@ Obj *namerlist;
 int x, y;
 char *typename;
 {
-    Obj *rest, *namerspec;
+    Obj *rest, *head, *namerspec;
 
     /* If no namers found, get out of here. */
     if (namerlist == lispnil)
       return NULL;
-    for (rest = namerlist; rest != lispnil; rest = cdr(rest)) {
-	if (consp(car(rest))
-	    && stringp(car(car(rest)))
-	    && strcmp(c_string(car(car(rest))), typename) == 0) {
-	    namerspec = cadr(car(rest));
+    for_all_list(namerlist, rest) {
+	head = car(rest);
+	if (consp(head)
+	    && (stringp(car(head)) || symbolp(car(head)))
+	    && strcmp(c_string(car(head)), typename) == 0) {
+	    namerspec = cadr(head);
 	    /* This might be a string naming a namer, try making it into
 	       a symbol. */
 	    if (stringp(namerspec))
@@ -1308,8 +1436,9 @@ fix_adjacent_terrain()
 	for_all_terrain_types(t2) {
 	    t3 = tt_adj_terr_effect(t1, t2);
 	    if (t3 >= 0) {
-		/* Table definitions have no provision for dynamically-determined
-		   bounds, so we have to check it here. */
+		/* Table definitions have no provision for
+		   dynamically-determined bounds, so we have to check
+		   it here. */
 		if (!is_terrain_type(t3)) {
 		    run_warning("Bad value %d in adjacent-terrain-effect table, ignoring", t3);
 		    break;
@@ -1363,13 +1492,21 @@ fix_adjacent_terrain()
    area generation method. */
 
 void
-add_edge_terrain()
+add_edge_terrain(void)
+{
+    set_edge_terrain(TRUE);
+}
+
+void
+set_edge_terrain(int doterr)
 {
     int x, y, t = g_edge_terrain(), halfheight = area.halfheight;
 
     /* Use ttype 0 if edge terrain is nonsensical. */
     if (!between(0, t, numttypes-1))
       t = 0;
+    if (!doterr)
+      t = NONUTYPE;
     /* Right/left upper/lower sides of a hexagon. */
     if (!area.xwrap) {
 	for (y = 0; y < halfheight; ++y) {
@@ -1391,47 +1528,53 @@ add_edge_terrain()
 }
 
 static void
-set_edge_values(x, y, t)
-int x, y, t;
+set_edge_values(int x, int y, int t)
+{
+    if (t != NONUTYPE)
+      set_terrain_at(x, y, t);
+    /* Give it an average elevation. */
+    if (elevations_defined())
+      set_edge_elevation(x, y);
+}
+
+static void
+set_edge_elevation(int x, int y)
 {
     int sumelev, numadj, dir, x1, y1, elev;
 
-    set_terrain_at(x, y, t);
-    /* Give it an average elevation. */
-    if (elevations_defined()) {
-	sumelev = 0;
-	numadj = 0;
-	for_all_directions(dir) {
-	    if (interior_point_in_dir(x, y, dir, &x1, &y1)) {
-		sumelev += elev_at(x1, y1);
-		++numadj;
-	    }
-	}
-	/* It's possible that the caller is being sloppy and setting
-	   edge cells not actually in the area, so be cool about it. */
-	if (numadj > 0) {
-	    elev = sumelev / numadj;
-	    /* Note that we clip only to overall bounds for the layer, not
-	       to the specific terrain type's bounds.  This is so that the
-	       edge terrain can be, say, ice, whose normal elevation range
-	       is high altitude, which would make a perspective view look
-	       weird (like peering into a sink). */
-	    if (elev < minelev)
-	      elev = minelev;
-	    if (elev > maxelev)
-	      elev = maxelev;
-	    set_elev_at(x, y, elev);
+    sumelev = 0;
+    numadj = 0;
+    for_all_directions(dir) {
+	if (interior_point_in_dir(x, y, dir, &x1, &y1)) {
+	    sumelev += elev_at(x1, y1);
+	    ++numadj;
 	}
     }
+    /* It's possible that the caller is being sloppy and setting edge
+       cells not actually in the area, so be cool about it. */
+    if (numadj > 0) {
+	elev = sumelev / numadj;
+	/* Note that we clip only to overall bounds for the layer, not
+	   to the specific terrain type's bounds.  This is so that the
+	   edge terrain can be, say, ice, whose normal elevation range
+	   is high altitude, which would make a perspective view look
+	   weird (like peering into a sink). */
+	if (elev < minelev)
+	  elev = minelev;
+	if (elev > maxelev)
+	  elev = maxelev;
+	set_elev_at(x, y, elev);
+    }
 }
 
-/* This makes area of liquid terrain types all have the same elevation, and an
-   elevation less than that of any adjacent non-liquid terrain.  In simple terms,
-   this makes ponds and such look right on contour maps (more importantly, LOS
-   visibility will work as expected around water features). */
+/* This makes area of liquid terrain types all have the same
+   elevation, and an elevation less than that of any adjacent
+   non-liquid terrain.  In simple terms, this makes ponds and such
+   look right on contour maps (more importantly, LOS visibility will
+   work as expected around water features). */
 
 static void
-flatten_liquid_terrain()
+flatten_liquid_terrain(void)
 {
     int t1, t2, anyliquideffects, affected[MAXTTYPES];
     int maxpasses, anychanges, x, y, dir, x1, y1, highest, lowest;
