@@ -163,7 +163,7 @@ make_countries(int calls, int runs)
 	announce_progress(sideprogress);
 	if (!country_is_complete(side, runs)) {
 	    sideadvantage = max(1, side->advantage);
-	    advantage = (side->player ? side->player->advantage : sideadvantage);
+	    advantage = actual_advantage(side);
 	    tmpradius = baseradius;
 	    if (advantage > 1)
 	      tmpradius = (baseradius * isqrt(advantage * 100) + 9) / 10;
@@ -174,62 +174,70 @@ make_countries(int calls, int runs)
 	    Dprintf("%s starts around %d,%d\n", side_desig(side), x0, y0);
 	    announce_progress(sideprogress + sidedeltahalf);
 	    totnumtodo = totnumdone = 0;
-	    /* Calculate how many units of each type we will get, both the ones
-	       that belong to the side, and the independents nearby. */
+	    /* Calculate how many units of each type we will get, both
+	       the ones that belong to the side, and any independent
+	       units that should start out nearby. */
 	    for_all_unit_types(u) {
 		numtodo[u] = numindeptodo[u] = 0;
-		   /* Seems logical that side does not get units at start it can't build. */
-		if (type_allowed_on_side(u, side)
-		    && has_advance_to_build(side, u)) {
+		/* Note that the numbers to create are primarily
+		   governed by unit type properties and the sides'
+		   advantages, but we also need not to put units on
+		   disallowed sides. */
+		if (type_allowed_on_side(u, side)) {
 		    if (runs > 0) {
-			/* If the side is just coming into the game,
-			   give it an average number of each type that
-			   the existing sides started out with, so
-			   that the new side has some parity. */
+			/* If the side is just coming into an ongoing
+			   game, give it an average number of each
+			   type that the existing sides started with,
+			   so that the new side has some parity. */
 			if (u_start_with(u) > 0 || u_indep_near_start(u) > 0)
-			  numtodo[u] = (average_numunits(u) * advantage) / sideadvantage;
+			  numtodo[u] =
+			    (average_numunits(u) * advantage) / sideadvantage;
 		    } else {
-			numtodo[u] = (u_start_with(u) * advantage) / sideadvantage;
+			numtodo[u] =
+			  (u_start_with(u) * advantage) / sideadvantage;
 		    }
 		    totnumtodo += numtodo[u];
 		}
-		/* Seems logical that null side does not get units at start it can't build. */
-		if (type_allowed_on_side(u, indepside)
-		    && has_advance_to_build(indepside, u)) {
-			numindeptodo[u] = (u_indep_near_start(u) * advantage) / sideadvantage;
-			totnumtodo += numindeptodo[u];
+		if (type_allowed_on_side(u, indepside)) {
+		    numindeptodo[u] =
+		      (u_indep_near_start(u) * advantage) / sideadvantage;
+		    totnumtodo += numindeptodo[u];
 		}
 		numleft[u] = numindepleft[u] = 0;
 	    }
-	    /* First place units actually belonging to the side initially. */
+	    /* First place the units belonging to the side initially. */
 	    for_all_unit_types(u) {
 		if (canbeinopen[u]) {
 		    for (i = 0; i < numtodo[u]; ++i) {
 			if (find_unit_place(u, x0, y0, &x, &y)) {
 			    place_unit(u, side, x, y, NULL);
 			} else {
-			    /* If can't find places for this type, give up. */
+			    /* If can't find places for this type,
+			       give up and go on to the next type. */
 		            numleft[u] = numtodo[u] - i;
 			    break;
 			}
 		    }
 		}
 	    }
-	    /* Now do independents in the initial country area. */
+	    /* Now do independent units in the initial country area. */
 	    for_all_unit_types(u) {
 		for (i = 0; i < numindeptodo[u]; ++i) {
 		    if (find_unit_place(u, x0, y0, &x, &y)) {
 			place_unit(u, indepside, x, y, NULL);
 		    } else {
-			/* If can't find places for this type, give up. */
+			/* If can't find places for this type, give up
+			   and go on to the next type. */
 			numindepleft[u] = numindeptodo[u] - i;
 			break;
 		    }
 		}
 	    }
-	    /* Now do units that have to be occupants.  Note that if occupants
-	       are allowed in independent units, then they might be
-	       placed in an independent rather than an owned unit. */
+	    /* Now do units that have to be occupants.  Note that if
+	       occupants are allowed in independent units, then they
+	       might be placed in an independent rather than an owned
+	       unit.  We have to trust that the game designer intended
+	       this to happen... */
 	    for_all_unit_types(u) {
 		if (!canbeinopen[u]) {
 		    for (i = 0; i < numtodo[u]; ++i) {
@@ -294,7 +302,7 @@ country_is_complete(Side *side, int runs)
     int u, totunits, numunits[MAXUTYPES];
     Unit *unit;
     int sideadvantage = max(1, side->advantage);
-    int advantage = (side->player ? side->player->advantage : sideadvantage);
+    int advantage = actual_advantage(side);
 
     /* Don't make any more independent units than preassigned. This prevents 
     the formation of a third side in games like flattop.g and duel.g. */
@@ -336,7 +344,7 @@ add_preexisting_units(Side *side, int runs)
     if (any_side_possible_units < 0) {
 	any_side_possible_units = FALSE;
 	for_all_units(unit) {
-	    lis = get_x_property(unit, K_SIDES);
+	    lis = unit_sides(unit);
 	    if (lis != lispnil) {
 		any_side_possible_units = TRUE;
 	    }
@@ -349,7 +357,7 @@ add_preexisting_units(Side *side, int runs)
 	    possunits = origpossunits = side->possible_units;
 	    if (any_side_possible_units) {
 		for_all_units(unit) {
-		    lis = get_x_property(unit, K_SIDES);
+		    lis = unit_sides(unit);
 		    if ((stringp(lis) || symbolp(lis))
 			&& side == find_side_by_name(c_string(lis))) {
 			possunits = cons(new_number(unit->id), possunits);
@@ -627,12 +635,14 @@ mung_terrain(int x, int y, int u)
     }
 }
 
+/* Find a unit for an occupant of the given type. */
+
 static Unit *
 find_occupant_place(Side *side, int u, int x0, int y0)
 {
     Unit *transport;
 
-    /* (should cast about randomly first, then do full search) */
+    /* (should cast about randomly first, then do the full search) */
     for_all_side_units(side, transport) {
 	if (is_unit(transport)
 	    && inside_area(transport->x, transport->y)
@@ -640,36 +650,40 @@ find_occupant_place(Side *side, int u, int x0, int y0)
 	    return transport;
 	}
     }
-    /* (should be able to look at indep units in country) */
+    /* (should be able to place inside indep units in country) */
     return NULL;
 }
 
+/* Given a type, position, and optional transport, put down a
+   newly-created unit.  We assume the caller has already checked
+   that the type is allowed to be owned by the given side. */
+
 static void
 place_unit(int u, Side *side, int x, int y, Unit *transport)
 {
     Unit *unit;
 
-    /* Check for this BEFORE creating the unit, or it will end up as an independent ghost! */
-    /* has_advance_to_build no longer included by type_allowed_on_side. */
-    /* Reasonable to limit start units to those we can build ourselves. */
-    if (!new_unit_allowed_on_side(u, side)
-       || !has_advance_to_build(side, u))
-	    return;
     unit = create_unit(u, TRUE);
-    if (unit != NULL) {
-	if (side != NULL) {
-	    set_unit_side(unit, side);
-	    set_unit_origside(unit, side);
-	}
-	init_supply(unit);
-	if (transport != NULL)
-	  enter_transport(unit, transport);
-	else
-	  enter_cell(unit, x, y);
-	announce_progress(sideprogress + sidedeltahalf + (sidedeltahalf * (++totnumdone)) / totnumtodo);
-    }
+    /* If we failed to create the unit, but didn't die due to memory
+       exhaustion, we must have gone past a designer-specified limit
+       on the numbers of units; so just return quietly. */
+    if (unit == NULL)
+      return;
+    set_unit_side(unit, side);
+    set_unit_origside(unit, side);
+    init_supply(unit);
+    if (transport != NULL)
+      enter_transport(unit, transport);
+    else
+      enter_cell(unit, x, y);
+    announce_progress(sideprogress
+		      + sidedeltahalf
+		      + (sidedeltahalf * (++totnumdone)) / totnumtodo);
 }
 
+/* Country expansion works by iterating over increasing distances from
+   the initial location, adding compatible cells to the country. */
+
 static void
 expand_countries(void)
 {
@@ -750,85 +764,86 @@ expand_country(int x, int y)
     } else {
 	prob = t_country_takeover(t);
     }
-    if (probability(prob)) {
-	/* Claim the cell, attempt to own it. */
-	ours = theirs = FALSE;
-	if (unit_at(x, y) == NULL) {
-	    /* If completely empty, we claim it. */
-	    ours = TRUE;
-	    for_all_unit_types(u) {
-	    	tmputype = u;
-		sideadvantage = max(1, tmpside->advantage);
-		advantage = (tmpside->player ? tmpside->player->advantage : sideadvantage);
-		/* Note that while theoretically this multiplication won't work
-		   if the product is > 100%, in practice the growth probabilities are
-		   going to be well under 10%. */
-		prob2 = advantage * u_unit_growth(u);
-		if (xrandom(10000) < prob2
-		    && !at_country_units_max(tmpside, u)
-		    && probability(ut_favored(u, t))
-		    && valid_unit_place(x, y)) {
-		    unit = create_unit(u, TRUE);
-		    if (unit != NULL) {
+    /* The cell should become known even if not occupied, but we can't
+       do that here because view structures don't exist yet, so just
+       leave for now. */
+    if (!probability(prob))
+      return;
+    /* Claim the cell, attempt to own it. */
+    ours = theirs = FALSE;
+    if (unit_at(x, y) == NULL) {
+	/* If completely empty, we claim it. */
+	ours = TRUE;
+	for_all_unit_types(u) {
+	    tmputype = u;
+	    sideadvantage = max(1, tmpside->advantage);
+	    advantage = actual_advantage(tmpside);
+	    /* Note that while theoretically this multiplication won't
+	       work if the product is > 100%, in practice the growth
+	       probs are going to be well under 10%. */
+	    prob2 = advantage * u_unit_growth(u);
+	    if (xrandom(10000) < prob2
+		&& !at_country_units_max(tmpside, u)
+		&& probability(ut_favored(u, t))
+		&& valid_unit_place(x, y)) {
+		unit = create_unit(u, TRUE);
+		if (unit != NULL) {
+		    set_unit_side(unit, tmpside);
+		    set_unit_origside(unit, tmpside);
+		    init_supply(unit);
+		    enter_cell(unit, x, y);
+		    break;
+		}
+	    }
+	    prob2 = advantage * u_indep_growth(u);
+	    if (xrandom(10000) < prob2
+		&& !at_country_units_max(tmpside, u)
+		&& probability(ut_favored(u, t))
+		&& valid_unit_place(x, y)) {
+		unit = create_unit(u, TRUE);
+		if (unit != NULL) {
+		    init_supply(unit);
+		    enter_cell(unit, x, y);
+		    break;
+		}
+	    }
+	}
+    } else {
+	/* Somebody is here already. */
+	for_all_stack(x, y, unit) {
+	    if (unit->side != tmpside) {
+		if (unit->side != NULL) {
+		    /* Maybe take over another side's unit. */
+		    if (xrandom(10000) < u_unit_takeover(unit->type)
+			&& !at_country_units_max(tmpside, unit->type)
+			) {
 			set_unit_side(unit, tmpside);
 			set_unit_origside(unit, tmpside);
-			init_supply(unit);
-			enter_cell(unit, x, y);
-		    	break;
-		    }
-		}
-		prob2 = advantage * u_indep_growth(u);
-		if (xrandom(10000) < prob2
-		    && !at_country_units_max(tmpside, u)
-		    && probability(ut_favored(u, t))
-		    && valid_unit_place(x, y)) {
-		    unit = create_unit(u, TRUE);
-		    if (unit != NULL) {
-			init_supply(unit);
-			enter_cell(unit, x, y);
-		    	break;
-		    }
-		}
-	    }
-	} else {
-	    /* Somebody is here already. */
-	    for_all_stack(x, y, unit) {
-		if (unit->side != tmpside) {
-		    if (unit->side != NULL) {
-			/* Maybe take over another side's unit. */
-			if (xrandom(10000) < u_unit_takeover(unit->type)
-			    && !at_country_units_max(tmpside, unit->type)
-			    ) {
-			    set_unit_side(unit, tmpside);
-			    set_unit_origside(unit, tmpside);
-			    ours = TRUE;
-			} else {
-			    /* Indicate that we don't have possession of every
-			       unit in this cell. */
-			    theirs = TRUE;
-			}
+			ours = TRUE;
 		    } else {
-			/* Maybe take over an independent unit. */
-			if (xrandom(10000) < u_indep_takeover(unit->type)
-			    && !at_country_units_max(tmpside, unit->type)
-			    ) {
-			    set_unit_side(unit, tmpside);
-			    set_unit_origside(unit, tmpside);
-			    ours = TRUE;
-			}
+			/* Indicate that we don't have possession of every
+			   unit in this cell. */
+			theirs = TRUE;
+		    }
+		} else {
+		    /* Maybe take over an independent unit. */
+		    if (xrandom(10000) < u_indep_takeover(unit->type)
+			&& !at_country_units_max(tmpside, unit->type)
+			) {
+			set_unit_side(unit, tmpside);
+			set_unit_origside(unit, tmpside);
+			ours = TRUE;
 		    }
 		}
 	    }
 	}
-	/* If cell is claimed, and has no opposition, then country gets it. */
-	if (ours && !theirs && probability(t_country_people(t))) {
-	    set_people_side_at(x, y, side_number(tmpside));
-	}
-	/* Count this cell as one we expanded into. */
-	++growth;
     }
-    /* The cell should become known even if not occupied, but we can't
-       do that here because view structures don't exist yet. */
+    /* If cell is claimed, and has no opposition, then country gets it. */
+    if (ours && !theirs && probability(t_country_people(t))) {
+	set_people_side_at(x, y, side_number(tmpside));
+    }
+    /* Count this cell as one we expanded into. */
+    ++growth;
 }
 
 static int
@@ -844,6 +859,10 @@ at_country_units_max(Side *side, int u)
 
 /* A method that scatters independent units over the world. */
 
+/* (should consider reviving the old rule that if indep density was
+   nonzero, at least one unit of that type was guaranteed to be
+   placed) */
+
 int
 make_independent_units(int calls, int runs)
 {
@@ -916,8 +935,8 @@ make_independent_units(int calls, int runs)
 }
 
 /* Set the people inside the country to be on the country's side.
-   Flip a coin along the edges, to make the border more interesting.
-   Also don't always overwrite existing people. */
+   Flip a coin along the edge of the country, to make the border more
+   interesting.  Also don't always overwrite existing people. */
 
 static void
 set_people_on_side(int x, int y)
