@@ -7,6 +7,7 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
+#include "kernel.h"
 
 extern char *readerrbuf;
 
@@ -44,7 +45,7 @@ create_game_module(char *name)
     /* The module's name must never be NULL. */
     if (name == NULL)
       name = "";
-    module->name = name;
+    module->name = copy_string(name);
     /* Add to front of module list. */
     module->next = modulelist;
     modulelist = module;
@@ -145,6 +146,9 @@ load_game_description(Module *module)
     Obj *form, *thecar;
     char *name;
 
+    /* Module is already open, don't mess with it. */
+    if (module->open)
+      return FALSE;
     if (open_module(module, FALSE)) {
 	if ((form = read_form(module->fp,
 			      &(module->startlineno),
@@ -160,6 +164,7 @@ load_game_description(Module *module)
 		}
 	    }
 	}
+	close_module(module);
     }
     return FALSE;
 }
@@ -171,7 +176,15 @@ void
 load_game_module(Module *module, int dowarn)
 {
     char ch;
+    Module *module2;
 
+    if (numutypes == 0) {
+	load_game_description(module);
+	if (!empty_string(module->defaultbasemodulename)) {
+	    module2 = get_game_module(module->defaultbasemodulename);
+	    load_game_module(module2, TRUE);
+	}
+    }
     if (open_module(module, dowarn)) {
 	if (module->fp) {
 	    /* Peek at the first character - was 'X' in old format files. */
@@ -350,7 +363,19 @@ read_forms(Module *module)
     Dprintf("... Done reading \"%s\".\n", module->name);
 }
 
-/* Interpret the given list of variants. */
+void
+set_variant_value(int which, int v1, int v2, int v3)
+{
+    Variant *var;
+
+    var = &(mainmodule->variants[which]);
+    var->newvalues[0] = v1;
+    var->newvalues[1] = v2;
+    var->newvalues[2] = v3;
+}
+
+/* Interpret the given list of variants, using the given list to
+   fill in values not already supplied in the variant. */
 
 void
 do_module_variants(Module *module, Obj *lis)
@@ -393,67 +418,73 @@ do_module_variants(Module *module, Obj *lis)
     }
 }
 
+/* Implement the effect of the single given variant. */
+
 static void
 do_one_variant(Module *module, Variant *var, Obj *varsetdata)
 {
     int val, caseval;
-    int width = 0, height = 0, circumference, latitude, longitude;
+    int width = 0, height = 0, circumference /*, latitude, longitude*/;
     int rtime, rtimeperturn, rtimeperside;
     char *vartypename = c_string(var->id);
-    Obj *restcases, *headcase, *rest, *filler, *rawval, *rawcaseval;
+    Obj *restcases, *headcase, *rest, *filler, *rawcaseval;
 
     if (Debug) {
 	if (readerrbuf == NULL)
 	  readerrbuf = (char *) xmalloc(BUFSIZE);
-    	sprintlisp(readerrbuf, varsetdata, BUFSIZE);
+	if (varsetdata != lispnil)
+	  sprintlisp(readerrbuf, varsetdata, BUFSIZE);
+	else
+	  sprintf(readerrbuf, "%d %d %d",
+		  var->newvalues[0], var->newvalues[1], var->newvalues[2]);
     	Dprintf("Module %s variant %s being set to `%s'\n",
 	    	module_desig(module), vartypename, readerrbuf);
     }
     switch (keyword_code(vartypename)) {
       case K_WORLD_SEEN:
-	val = (varsetdata == lispnil ?
-	       (var->dflt == lispnil ? FALSE : c_number(eval(var->dflt))) :
-	       c_number(eval(car(varsetdata))));
+	val = var->newvalues[0];
+	if (varsetdata != lispnil)
+	  val = c_number(eval(car(varsetdata)));
 	var->hasintvalue = TRUE;
 	var->intvalue = val;
 	set_g_terrain_seen(val);
 	break;
       case K_SEE_ALL:
-	val = (varsetdata == lispnil ?
-	       (var->dflt == lispnil ? FALSE : c_number(eval(var->dflt))) :
-	       c_number(eval(car(varsetdata))));
+	val = var->newvalues[0];
+	if (varsetdata != lispnil)
+	  val = c_number(eval(car(varsetdata)));
 	var->hasintvalue = TRUE;
 	var->intvalue = val;
 	set_g_see_all(val);
 	break;
       case K_SEQUENTIAL:
-	val = (varsetdata == lispnil ?
-	       (var->dflt == lispnil ? FALSE : c_number(eval(var->dflt))) :
-	       c_number(eval(car(varsetdata))));
+	val = var->newvalues[0];
+	if (varsetdata != lispnil)
+	  val = c_number(eval(car(varsetdata)));
 	var->hasintvalue = TRUE;
 	var->intvalue = val;
 	set_g_use_side_priority(val);
 	break;
       case K_PEOPLE:
-	val = (varsetdata == lispnil ?
-	       (var->dflt == lispnil ? FALSE : c_number(eval(var->dflt))) :
-	       c_number(eval(car(varsetdata))));
+	val = var->newvalues[0];
+	if (varsetdata != lispnil)
+	  val = c_number(eval(car(varsetdata)));
 	var->hasintvalue = TRUE;
 	var->intvalue = val;
 	set_g_people(val);
 	break;
       case K_ECONOMY:
-	val = (varsetdata == lispnil ?
-	       (var->dflt == lispnil ? FALSE : c_number(eval(var->dflt))) :
-	       c_number(eval(car(varsetdata))));
+	val = var->newvalues[0];
+	if (varsetdata != lispnil)
+	  val = c_number(eval(car(varsetdata)));
 	var->hasintvalue = TRUE;
 	var->intvalue = val;
 	set_g_economy(val);
 	break;
       case K_SUPPLY:
-	val = (varsetdata == lispnil ?
-	       (var->dflt == lispnil ? FALSE : c_number(eval(var->dflt))) :
-	       c_number(eval(car(varsetdata))));
+	val = var->newvalues[0];
+	if (varsetdata != lispnil)
+	  val = c_number(eval(car(varsetdata)));
 	var->hasintvalue = TRUE;
 	var->intvalue = val;
 	set_g_supply(val);
@@ -462,29 +493,32 @@ do_one_variant(Module *module, Variant *var, Obj *varsetdata)
       	filler = lispnil;
 	if (varsetdata != lispnil) {
 	    filler = varsetdata;
-	} else if (var->dflt != lispnil) {
-	    filler = var->dflt;
 	}
 	/* Pick the width and height out of the list. */
 	if (filler != lispnil) {
 	    width = c_number(eval(car(filler)));
 	    filler = cdr(filler);
+	} else {
+	    width = var->newvalues[0];
 	}
 	if (filler != lispnil) {
 	    height = c_number(eval(car(filler)));
 	    filler = cdr(filler);
 	} else {
-	    height = width;
+	    height = var->newvalues[1];
 	}
 	/* Pick up a circumference if given. */
 	if (filler != lispnil) {
 	    circumference = c_number(eval(car(filler)));
-	    set_world_circumference(circumference, TRUE);
 	    filler = cdr(filler);
+	} else {
+	    circumference = var->newvalues[2];
 	}
 	/* This is more useful after the circumference has been set. */
 	if (width > 0 && height > 0)
 	  set_area_shape(width, height, TRUE);
+	set_world_circumference(circumference, TRUE);
+#if 0 /* (redo as separate variant) */
 	/* Pick up latitude and longitude if given. */
 	if (filler != lispnil) {
 	    latitude = c_number(eval(car(filler)));
@@ -498,32 +532,31 @@ do_one_variant(Module *module, Variant *var, Obj *varsetdata)
 	    area.longitude = longitude;
 	    filler = cdr(filler);
 	}
+#endif
 	/* (should record settings somehow) */
 	break;
       case K_REAL_TIME:
       	filler = lispnil;
 	if (varsetdata != lispnil) {
 	    filler = varsetdata;
-	} else if (var->dflt != lispnil) {
-	    filler = var->dflt;
 	}
 	if (filler != lispnil) {
 	    rtime = c_number(eval(car(filler)));
 	    filler = cdr(filler);
 	} else {
-	    rtime = 0;
+	    rtime = var->newvalues[0];
 	}
 	if (filler != lispnil) {
 	    rtimeperside = c_number(eval(car(filler)));
 	    filler = cdr(filler);
 	} else {
-	    rtimeperside = 0;
+	    rtimeperside = var->newvalues[1];
 	}
 	if (filler != lispnil) {
 	    rtimeperturn = c_number(eval(car(filler)));
 	    filler = cdr(filler);
 	} else {
-	    rtimeperturn = 0;
+	    rtimeperturn = var->newvalues[2];
 	}
 	/* If the values were specified, tweak the official
 	   realtime globals. */
@@ -537,24 +570,11 @@ do_one_variant(Module *module, Variant *var, Obj *varsetdata)
 	break;
       default:
 	/* This is the general case. */
-	if (varsetdata != lispnil) {
-	    rawval = car(varsetdata);
-	} else if (var->dflt != lispnil) {
-	    rawval = var->dflt;
-	} else {
-	    rawval = lispnil;
-	}
-	/* Always evaluate the raw value of the variant. */
-	rawval = eval(rawval);
-	if (numberp(rawval)) {
-	    val = c_number(rawval);
-	    var->hasintvalue = TRUE;
-	    var->intvalue = val;
-	} else {
-	    val = FALSE;
-	    var->hasintvalue = TRUE;
-	    var->intvalue = val;
-	}
+	val = var->newvalues[0];
+	if (varsetdata != lispnil)
+	  val = c_number(eval(car(varsetdata)));
+	var->hasintvalue = TRUE;
+	var->intvalue = val;
 	for_all_list(var->cases, restcases) {
 	    headcase = car(restcases);
 	    rawcaseval = eval(car(headcase));
@@ -704,31 +724,9 @@ module_desig(Module *module)
 
 /* (random code below, should be sent to better places) */
 
-#ifdef DEBUGGING
-/* Use this to ensure that everything is cool. */
-
-void
-doublecheck_state(Side *side)
-{
-    Unit *unit;
-
-    for_all_units(unit) {
-	if (unit->x < 0 || unit->x >= area.width ||
-	    unit->y <= 0 || unit->y >= (area.height - 1) ||
-	    unit->hp <= 0) {
-	    Dprintf("%s off map hp %d", unit_desig(unit), unit->hp);
-	}
-    }
-}
-#endif /* DEBUGGING */
-
-void set_u_internal_name(int u, char *s) { utypes[u].iname = s; }
-void set_u_type_name(int u, char *s) { utypes[u].name = s; }
-void set_m_type_name(int m, char *s) { mtypes[m].name = s; }
-void set_t_type_name(int t, char *s) { ttypes[t].name = s; }
-void set_a_type_name(int a, char *s) { atypes[a].name = s; }
-
-/* If a special symbol, we might not have to fail. */
+/* If a special symbol, we might not have to fail.  Note that although
+   this looks like Lisp-level code, it knows about unit types and so
+   forth, so is higher-level. */
 
 int
 lazy_bind(Obj *sym)
@@ -765,33 +763,3 @@ lazy_bind(Obj *sym)
     setq(sym, value);
     return TRUE;
 }
-
-int
-coerce_to_side_id(Obj *x)
-{
-    if (numberp(x)) {
-	return c_number(x);
-    }
-    return 0;
-}
-
-Side *
-coerce_to_side(Obj *x)
-{
-    return side_n(coerce_to_side_id(x));
-}
-
-int
-coerce_to_unit_id(Obj *x)
-{
-    if (numberp(x)) {
-	return c_number(x);
-    }
-    return 0;
-}
-
-Unit *
-coerce_to_unit(Obj *x)
-{
-    return find_unit(coerce_to_unit_id(x));
-}
