@@ -7,15 +7,7 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern int carryable(int u);
-extern int accelerable(int u);
-extern int accelerator(int u1, int u2);
-extern int border_slide_possible(int u2, int ox, int oy, int nx, int ny); 
 #include "kernel.h"
-#include "kpublic.h"
-
-int unit_traverse_blockable_by(Unit *unit, Unit *unit2);
-int any_friendly_at(Unit *unit, int x, int y);
 
 enum {
     over_nothing = 0,
@@ -36,6 +28,24 @@ enum {
 extern int retreating;
 extern int retreating_from;
 
+static int border_slide_possible(int u2, int ox, int oy, int nx, int ny); 
+static int unit_traverse_blockable_by(Unit *unit, Unit *unit2);
+static int any_friendly_at(Unit *unit, int x, int y);
+static int unit_blockable_by(Unit *unit, Unit *unit2);
+static int total_entry_cost(int u1, int u3, int x1, int y1, int z1, int u2, int x2, int y2, int z2);
+static int number_member(int x, Obj *lis);
+static int wind_value(Unit *unit, int angle, int force, Obj *effect, int maxval);
+static void detonate_on_approach_around(Unit *unit);
+static int zoc_range(Unit *unit, int u2);
+static int damaged_value(Unit *unit, Obj *effect, int maxval);
+static void try_detonate_on_approach(int x, int y);
+static void test_blocking_zoc(int x, int y);
+
+static int maybe_react_to_move(Unit *unit, int ox, int oy);
+static void consume_move_supplies(Unit *unit);
+
+static int tmprslt;
+
 /* Movement actions. */
 
 /* Record a move action as the next to do. */
@@ -53,16 +63,6 @@ prep_move_action(Unit *unit, Unit *unit2, int x, int y, int z)
     return TRUE;
 }
 
-int
-net_prep_move_action(Unit *unit, Unit *unit2, int x, int y, int z)
-{
-    int rslt;
-
-    rslt = prep_move_action(unit, unit2, x, y, z);
-    broadcast_next_action(unit);
-    return rslt;
-}
-
 /* The basic act of moving.  This attempts to move and maybe fails,
    but takes no corrective action.  Note that this requires space in
    the destination cell, will not board, attack, etc - all that is
@@ -96,12 +96,19 @@ do_move_action(Unit *unit, Unit *unit2, int x, int y, int z)
 	       || (dist == 2 && border_slide_possible(u2, ox, oy, x, y))) {
 	/* Movement to an adjacent cell. */
 	if (!inside_area(x, y)) {
+	    /* (should notify all observers) */
+	    notify(unit2->side, "%s leaves the area!",
+		   unit_handle(unit2->side, unit2));
 	    kill_unit(unit2, H_UNIT_LEFT_WORLD);
 	    rslt = A_ANY_DONE;
 	} else if (ut_vanishes_on(u2, t) && !can_move_via_conn(unit2, x, y)) {
+	    notify(unit2->side, "%s vanishes into the %s!",
+		   unit_handle(unit2->side, unit2), t_type_name(t));
 	    kill_unit(unit2, H_UNIT_VANISHED);
 	    rslt = A_MOVE_UNIT_GONE;
 	} else if (ut_wrecks_on(u2, t) && !can_move_via_conn(unit2, x, y)) {
+	    notify(unit2->side, "%s wrecks in the %s!",
+		   unit_handle(unit2->side, unit2), t_type_name(t));
 	    if (u_wrecked_type(u2) == NONUTYPE) {
 		/* Occupants always die if the wrecked unit disappears. */
 		kill_unit(unit, H_UNIT_WRECKED);
@@ -129,12 +136,16 @@ do_move_action(Unit *unit, Unit *unit2, int x, int y, int z)
 	       hostile terrain and tries to move to hospitable terrain
 	       without using a bridge, or if it is in a transport that
 	       won't ferry across hostile terrain. */
+	    notify(unit2->side, "%s vanishes into the %s!",
+		   unit_handle(unit2->side, unit2), t_type_name(t));
 	    kill_unit(unit2, H_UNIT_VANISHED);
 	    rslt = A_MOVE_UNIT_GONE;
 	} else if (ut_wrecks_on(u2, ot)
 		   && (unit->transport == NULL
 		       || uu_ferry_on_leave(unit->transport->type, u2) < over_own)
 		   && !can_move_via_conn(unit2, x, y)) {
+	    notify(unit2->side, "%s wrecks in the %s!",
+		   unit_handle(unit2->side, unit2), t_type_name(t));
 	    if (u_wrecked_type(u2) == NONUTYPE) {
 		/* Occupants always die if the wrecked unit disappears. */
 		kill_unit(unit, H_UNIT_WRECKED);
@@ -373,7 +384,11 @@ unit_speed(Unit *unit, int nx, int ny)
     int dir, angle1, windval1, angle2, windval2;
     int occeff, totocceff, totoccdenom;
     Unit *occ;
-    
+
+    /* Dead units can sometimes end up here, but don't bother
+       complaining. */
+    if (unit->hp <= 0)
+      return 0;
     speed = u_speed(u);
     if (unit->hp < u_hp_max(u) && u_speed_damage_effect(u) != lispnil) {
 	speed = damaged_value(unit, u_speed_damage_effect(u), speed);
@@ -422,7 +437,7 @@ damaged_value(Unit *unit, Obj *effect, int maxval)
     err = interpolate_in_list_ext(unit->hp, effect, 0, 0, 0, 0, u_hp(u),
 				  maxval, &rslt);
     if (err != 0) {
-	run_warning("cannot get damaged speed for %s at hp %d, using 100",
+	run_warning("cannot get damaged value for %s at hp %d, using 100",
 		    u_type_name(u), unit->hp);
 	rslt = 100;
     }
@@ -542,8 +557,8 @@ move_unit(Unit *unit, int nx, int ny)
 void
 change_control_side_around(int x, int y, int u, Side *side)
 {
-    int con = control_side_at(x, y), s = side_number(side);
-    Side *oldside, *side2;
+    int con = control_side_at(x, y), s = side_number(side), pop;
+    Side *oldside, *peopside, *side2;
 
     /* Do nothing if the cell cannot be controlled by a side. */
     if (con == NOCONTROL)
@@ -555,6 +570,13 @@ change_control_side_around(int x, int y, int u, Side *side)
     /* Don't challenge control by our allies. */
     if (trusted_side(side, oldside))
       return;
+    /* Return an ally's area. */
+    if (people_sides_defined()
+	&& (pop = people_side_at(x, y)) != NOBODY) {
+	peopside = side_n(pop);
+	if (trusted_side(side, peopside))
+	  s = pop;
+    }
     /* Change the cell's control. */
     set_control_side_at(x, y, s);
     /* All of our buddies get to see what goes on here. */
@@ -584,10 +606,6 @@ can_move_at_all(Unit *unit)
 
 /* This is true if the given location is in a blocking zoc for the unit. */
 
-static int tmprslt;
-
-static void test_blocking_zoc(int x, int y);
-
 int
 in_blocking_zoc(Unit *unit, int x, int y, int z)
 {
@@ -977,8 +995,6 @@ maybe_react_to_move(Unit *unit, int ox, int oy)
     return 0;
 }
 
-static void try_detonate_on_approach(int x, int y);
-
 static void
 try_detonate_on_approach(int x, int y)
 {
@@ -992,19 +1008,25 @@ try_detonate_on_approach(int x, int y)
 	    /* (should make doctrine-based decision about whether to go off) */
 	    && !was_detonated(unit)
 	    ) {
+	    /* Found one, now set it off! */
 	    detonate_unit(unit, unit->x, unit->y, unit->z);
 	}
     }
 }
 
+/* For the given unit, search around the given area looking for other
+   units might detonate just because of the proximity. */
+
 void
 detonate_on_approach_around(Unit *unit)
 {
     int maxrange;
 
     tmpunit = unit;
-    apply_to_area(unit->x, unit->y, max_detonate_on_approach_range, try_detonate_on_approach);
-    maxrange = max(max_u_detonate_effect_range, max_t_detonate_effect_range) + max_detonate_on_approach_range;
+    apply_to_area(unit->x, unit->y, max_detonate_on_approach_range,
+		  try_detonate_on_approach);
+    maxrange = max(max_u_detonate_effect_range, max_t_detonate_effect_range)
+      + max_detonate_on_approach_range;
     reckon_damage_around(unit->x, unit->y, maxrange);
 }
 
@@ -1013,8 +1035,7 @@ detonate_on_approach_around(Unit *unit)
    so check for this case and maybe hit/kill the unit. */
 
 void  
-consume_move_supplies(unit)
-Unit *unit;
+consume_move_supplies(Unit *unit)
 {
     int u = unit->type, m, checkstarve = FALSE;
     
@@ -1046,8 +1067,7 @@ Unit *unit;
 /* Record an enter action as the next to do. */
 
 int
-prep_enter_action(unit, unit2, unit3)
-Unit *unit, *unit2, *unit3;
+prep_enter_action(Unit *unit, Unit *unit2, Unit *unit3)
 {
     if (unit == NULL || unit->act == NULL || unit2 == NULL || unit3 == NULL)
       return FALSE;
@@ -1058,19 +1078,7 @@ Unit *unit, *unit2, *unit3;
 }
 
 int
-net_prep_enter_action(unit, unit2, unit3)
-Unit *unit, *unit2, *unit3;
-{
-    int rslt;
-
-    rslt = prep_enter_action(unit, unit2, unit3);
-    broadcast_next_action(unit);
-    return rslt;
-}
-
-int
-do_enter_action(unit, unit2, unit3)
-Unit *unit, *unit2, *unit3;
+do_enter_action(Unit *unit, Unit *unit2, Unit *unit3)
 {
     int u2, u3, u4, ox, oy, oz, ot, nx, ny, nz, speed, acpcost, mpcost, rslt;
 
@@ -1081,18 +1089,25 @@ Unit *unit, *unit2, *unit3;
     nx = unit3->x;  ny = unit3->y;  nz = unit3->z;
     mpcost = 1;
     if (ut_vanishes_on(u2, ot)
+	&& uu_ferry_on_enter(u3, u2) < over_all
 	&& (unit->transport == NULL
 	    || uu_ferry_on_leave(unit->transport->type, u2) < over_own)
 	&& !can_move_via_conn(unit2, nx, ny)) {
-	/* This case is if the unit is already on a connection in hostile terrain
-	   and tries to enter without using a bridge, or if it is on a transport
-	   in hostile terrain that doesn't ferry. */
+	/* This case is if the unit is already on a connection in
+	   hostile terrain and tries to enter without using a bridge,
+	   or if it is on a transport in hostile terrain that doesn't
+	   ferry. */
+	notify(unit2->side, "%s vanishes into the %s!",
+	       unit_handle(unit2->side, unit2), t_type_name(ot));
 	kill_unit(unit2, H_UNIT_VANISHED);
 	rslt = A_MOVE_UNIT_GONE;
     } else if (ut_wrecks_on(u2, ot)
+	       && uu_ferry_on_enter(u3, u2) < over_all
 	       && (unit->transport == NULL
 		   || uu_ferry_on_leave(unit->transport->type, u2) < over_own)
 	       && !can_move_via_conn(unit2, nx, ny)) {
+	notify(unit2->side, "%s wrecks in the %s!",
+	       unit_handle(unit2->side, unit2), t_type_name(ot));
 	if (u_wrecked_type(u2) == NONUTYPE) {
 	    /* Occupants always die if the wrecked unit disappears. */
 	    kill_unit(unit, H_UNIT_WRECKED);
@@ -1129,8 +1144,7 @@ Unit *unit, *unit2, *unit3;
 }
 
 int
-check_enter_action(unit, unit2, unit3)
-Unit *unit, *unit2, *unit3;
+check_enter_action(Unit *unit, Unit *unit2, Unit *unit3)
 {
     int u, u2, u3, u4, u2x, u2y, u3x, u3y, totcost, speed, mpavail, m;
     int ox, oy, oz, nx, ny, nz;
@@ -1178,7 +1192,8 @@ Unit *unit, *unit2, *unit3;
 	/* Zero mp always disallows movement. */
 	if (mpavail <= 0)
 	  return A_MOVE_NO_MP;
-	/* The free mp might get us enough moves, so add it before comparing. */
+	/* The free mp might get us enough moves, so add it before
+           comparing. */
 	if (mpavail + u_free_mp(u2) < totcost)
 	  return A_MOVE_NO_MP;
     }
@@ -1194,8 +1209,7 @@ Unit *unit, *unit2, *unit3;
    and a prospective transport allows for entry of the unit. */
 
 int
-sides_allow_entry(unit, transport)
-Unit *unit, *transport;
+sides_allow_entry(Unit *unit, Unit *transport)
 {
     if (unit->side == indepside) {
     	if (transport->side == indepside) {
@@ -1219,8 +1233,8 @@ Unit *unit, *transport;
 /* This computes the total mp cost of entering a transport. */
 
 int
-total_entry_cost(u1, u3, x1, y1, z1, u2, x2, y2, z2)
-int u1, u3, x1, y1, z1, u2, x2, y2, z2;
+total_entry_cost(int u1, int u3, int x1, int y1, int z1,
+		 int u2, int x2, int y2, int z2)
 {
     int cost = 0, ferryout, ferryin, t1, t2, b, dir, conncost, travcost, c;
 
@@ -1282,72 +1296,8 @@ int u1, u3, x1, y1, z1, u2, x2, y2, z2;
     return cost;
 }
 
-/* Note the recursion - should precalc this property. */
-
 int
-carryable(u)
-int u;
-{
-    int u2;
-	
-    for_all_unit_types(u2) {
-	if (could_carry(u2, u)
-	    && (mobile(u2) /* || carryable(u2) */ ))
-	  return TRUE;
-    }
-    return FALSE;
-}
-
-static short *accelerables = NULL;
-
-int
-accelerable(u)
-int u;
-{
-    int u1, u2;
-
-    if (accelerables == NULL) {
-	accelerables = (short *) xmalloc(numutypes * sizeof(int));
-	for_all_unit_types(u1) {	
-	    for_all_unit_types(u2) {
-		if (accelerator(u1, u2)) {
-		    accelerables[u2] = TRUE;
-		    break;
-		}
-	    }
-	}
-    }
-    return accelerables[u];
-}
-
-/* True if u1 is a type that can move u2 faster by transporting it. */
-
-int
-accelerator(u1, u2)
-int u1, u2;
-{
-    int t;
-
-    if (could_carry(u1, u2)
-	&& mobile(u1)) {
-	if (u_acp(u1) * u_speed(u1) > u_acp(u2) * u_speed(u2)) {
-	    /* The transport must be able to move on any type of terrain
-	       accessible to the transportee. */
-	    for_all_terrain_types(t) {
-		if (terrain_always_impassable(u1, t)
-		    && !terrain_always_impassable(u2, t)) {
-		    return FALSE;
-		}
-	    }
-	}
-	return TRUE;
-    }
-    return FALSE;
-}
-
-int
-terrain_always_impassable(u, t)
-int u, t;
+terrain_always_impassable(int u, int t)
 {
     if (ut_vanishes_on(u, t))
       return TRUE;
