@@ -15,8 +15,6 @@ any later version.  See the file COPYING.  */
    different language.) */
 
 #include "conq.h"
-extern char *apparent_unit_handle(Side *side, Unit *unit, Side *side2);
-extern void size_desc(char *buf, Unit *unit, int label);
 #include "kernel.h"
 
 static void notify_combat(Unit *unit, Unit *atker, char *str);
@@ -30,6 +28,9 @@ static int gain_count(Side *side, int u, int r);
 static int loss_count(Side *side, int u, int r);
 static int atkstats(Side *side, int a, int d);
 static int hitstats(Side *side, int a, int d);
+static void pad_out(char *buf, int n);
+static char *past_unit_handle(Side *side, PastUnit *past_unit);
+static char *short_side_title_with_adjective(Side *side, char *adjective);
 
 static char *tmpnbuf;
 
@@ -55,7 +56,7 @@ static char *loss_reason_names[] = { "Cbt", "Cap", "Stv", "Acc", "Dis", "Oth" };
 
 typedef enum {
     cal_unknown,
-    cal_turn,
+    cal_number,
     cal_usual
 } CalendarType;
 
@@ -109,7 +110,7 @@ static UsualDate *usual_initial;
 
 char *datebuf;
 
-char *turnname;
+char *turn_name;
 
 char *featurebuf;
 
@@ -157,8 +158,7 @@ notify_all(char *fmt, ...)
 	    vsprintf(tmpnbuf, fmt, ap);
 	    va_end(ap);
 	    /* Always capitalize first char of notice. */
-	    if (islower(tmpnbuf[0]))
-	      tmpnbuf[0] = toupper(tmpnbuf[0]);
+	    capitalize(tmpnbuf);
 	    low_notify(side, tmpnbuf);
 	}
     }
@@ -176,8 +176,7 @@ notify(Side *side, char *fmt, ...)
     vsprintf(tmpnbuf, fmt, ap);
     va_end(ap);
     /* Always capitalize first char of notice. */
-    if (islower(tmpnbuf[0]))
-      tmpnbuf[0] = toupper(tmpnbuf[0]);
+    capitalize(tmpnbuf);
     low_notify(side, tmpnbuf);
 }
 
@@ -189,8 +188,7 @@ vnotify(Side *side, char *fmt, va_list ap)
     maybe_mention_date(side);
     vsprintf(tmpnbuf, fmt, ap);
     /* Always capitalize first char of notice. */
-    if (islower(tmpnbuf[0]))
-      tmpnbuf[0] = toupper(tmpnbuf[0]);
+    capitalize(tmpnbuf);
     low_notify(side, tmpnbuf);
 }
 
@@ -320,7 +318,7 @@ short_side_title_plural_p(Side *side)
     } else if (side->noun) {
 	return TRUE;
     } else if (side->adjective) {
-	sprintf(side_short_title, "The %s side", side->adjective);
+	sprintf(side_short_title, "the %s side", side->adjective);
 	return FALSE;
     } else {
 	return FALSE;
@@ -353,7 +351,7 @@ sidemask_desc(char *buf, SideMask sidemask)
     if (sidemask < 0 || sidemask >= ((1 << (numsides + 1)) - 2))
       return "all";
     buf[0] = '\0';
-    for_all_sides_plus_indep(side2) {
+    for_all_sides(side2) {
 	if (side_in_set(side2, sidemask)) {
 	    if (first)
 	      first = FALSE;
@@ -448,6 +446,8 @@ short_player_title(char *buf, Player *player, char *thisdisplayname)
     return buf;
 }
 
+#if 0 	/* Unused. */
+
 void
 side_and_type_name(char *buf, Side *side, int u, Side *side2)
 {
@@ -463,6 +463,8 @@ side_and_type_name(char *buf, Side *side, int u, Side *side2)
     strcat(buf, u_type_name(u));
 }
 
+#endif
+
 /* Build a short phrase describing a given unit to a given side,
    basically consisting of indication of unit's side, then of unit
    itself. */
@@ -488,6 +490,10 @@ apparent_unit_handle(Side *side, Unit *unit, Side *side2)
     Side *side3;
     Obj *frest, *fmt1;
 
+    /* This should be impossible, be really obvious if it happens. */
+    if (side2 == NULL)
+      return "null side2 in apparent_unit_handle?";
+    /* Make sure our working space exists. */
     if (unitbuf == NULL)
       unitbuf = xmalloc(BUFSIZE);
     /* Handle various weird situations. */
@@ -504,25 +510,33 @@ apparent_unit_handle(Side *side, Unit *unit, Side *side2)
     /* Decide how to identify the side.  If the unit's original side
        is not its current side, list both of them. */
     side3 = NULL;
-    if (unit->origside != NULL && side2 != unit->origside) {
-	side3 = unit->origside;
-    }
+    if (unit->origside != NULL && side2 != unit->origside)
+      side3 = unit->origside;
     sidebuf[0] = '\0';
-    if (side2 == NULL) {
-	strcat(sidebuf, "the");
-    } else if (side2 == side) {
+    if (side2 == side) {
 	strcat(sidebuf, "your");
     } else {
-	strcat(sidebuf, "the ");
+	/* If the side adjective is a genitive (ends in 's, s' or z')
+	   we should skip the definite article. */
+	int len = strlen(side_adjective(side2));
+	char *end = side_adjective(side2) + len - 2;
+
+	if (strcmp(end, "'s") != 0
+	    && strcmp(end, "s'") != 0
+	    && strcmp(end, "z'") != 0)
+	  strcat(sidebuf, "the ");
 	strcat(sidebuf, side_adjective(side2));
-	if (side3 != NULL)
-	  strcat(sidebuf, "-held");
     }
     if (side3 != NULL) {
 	if (side3 == side) {
-	    strcat(sidebuf, " (your)");
+	    strcat(sidebuf, " (formerly your)");
+	} else if (side3 == indepside) {
+	    /* Don't add anything for captured independents.  While
+	       technically there's no reason not to do this, in
+	       practice a side will often have many captured indeps,
+	       and this keeps the text from getting too cluttered. */
 	} else {
-	    strcat(sidebuf, " (");
+	    strcat(sidebuf, " (formerly ");
 	    strcat(sidebuf, side_adjective(side3));
 	    strcat(sidebuf, ")");
 	}
@@ -534,8 +548,11 @@ apparent_unit_handle(Side *side, Unit *unit, Side *side2)
 	for_all_list(u_desc_format(unit->type), frest) {
 	    fmt1 = car(frest);
 	    if (stringp(fmt1)) {
+		/* Append strings verbatim. */
 		strcat(unitbuf, c_string(fmt1));
 	    } else if (symbolp(fmt1)) {
+		/* Symbols indicate the types of data to format and
+                   output. */
 		fmtstr = c_string(fmt1);
 		if (strcmp(fmtstr, "name") == 0) {
 		    strcat(unitbuf, (unit->name ? unit->name : "anon"));
@@ -562,6 +579,7 @@ apparent_unit_handle(Side *side, Unit *unit, Side *side2)
 	strcat(unitbuf, sidebuf);
 	strcat(unitbuf, " ");
     }
+    /* Default formats for units. */
     if (unit->name) {
 	tprintf(unitbuf, "%s %s", utypename, unit->name);
     } else if (unit->number > 0) {
@@ -573,7 +591,10 @@ apparent_unit_handle(Side *side, Unit *unit, Side *side2)
     return unitbuf;
 }
 
-/* Shorter unit description omits side name, but uses same buffer. */
+/* Shorter unit description omits side name, but uses same buffer.
+   This is mainly useful for describing the transport of a unit and
+   suchlike, where the player will likely already know the side of the
+   unit. */
 
 char *
 short_unit_handle(Unit *unit)
@@ -592,14 +613,53 @@ short_unit_handle(Unit *unit)
     if (unit->side != NULL && unit == unit->side->self_unit)
       return "you";
     u = unit->type;
+    /* Use the name alone if the unit is named, or else use optional
+       ordinal and the shortest type name available. */
     if (!empty_string(unit->name)) {
 	strcpy(unitbuf, unit->name);
-    } else if (!empty_string(u_short_name(u))) {
-	sprintf(unitbuf, "%d%s %s",
-		unit->number, ordinal_suffix(unit->number), u_short_name(u));
     } else {
-	sprintf(unitbuf, "%d%s %s",
-		unit->number, ordinal_suffix(unit->number), u_type_name(u));
+	unitbuf[0] = '\0';
+	if (unit->number > 0) {
+	    sprintf(unitbuf, "%d%s ",
+		    unit->number, ordinal_suffix(unit->number));
+	}
+	if (!empty_string(u_short_name(u)))
+	  strcat(unitbuf, u_short_name(u));
+	else
+	  strcat(unitbuf, u_type_name(u));
+    }
+    return unitbuf;
+}
+
+/* This version lists the side but skips original side etc. */
+
+char *
+medium_long_unit_handle(Unit *unit)
+{
+    if (unitbuf == NULL)
+      unitbuf = xmalloc(BUFSIZE);
+    if (unit == NULL)
+      return "???";
+    if (!alive(unit)) {
+    	sprintf(unitbuf, "dead #%d", unit->id);
+        return unitbuf;
+    }
+    strcpy(unitbuf, side_adjective(unit->side));
+    /* If the unit has a name, write its type followed by the name. */
+    if (!empty_string(unit->name)) {
+	strcat(unitbuf, " ");
+	strcat(unitbuf, u_type_name(unit->type));
+	strcat(unitbuf, " ");
+	strcat(unitbuf, unit->name);
+    /* If the unit has a number, write it followed by the type. */
+    } else if (unit->number > 0) {
+	tprintf(unitbuf, " %d%s %s",
+		unit->number, ordinal_suffix(unit->number),
+		u_type_name(unit->type));
+    /* Else just write the unit type. */
+    } else {
+	strcat(unitbuf, " ");
+	strcat(unitbuf, u_type_name(unit->type));
     }
     return unitbuf;
 }
@@ -640,7 +700,16 @@ past_unit_handle(Side *side, PastUnit *past_unit)
     } else if (side2 == side) {
 	sprintf(past_unitbuf, "your ");
     } else {
-	sprintf(past_unitbuf, "the %s ", side_adjective(side2));
+	/* If the side adjective is a genitive (ends in 's, s' or z')
+	   we should skip the definite article. */
+	int len = strlen(side_adjective(side2));
+	char *end = side_adjective(side2) + len - 2;
+
+	if (strcmp(end, "'s") != 0
+	    && strcmp(end, "s'") != 0
+	    && strcmp(end, "z'") != 0)
+	  sprintf(past_unitbuf, "the ");
+	sprintf(past_unitbuf, side_adjective(side2));
     }
     /* Now add the past_unit's unique description. */
     utypename = u_type_name(past_unit->type);
@@ -656,32 +725,6 @@ past_unit_handle(Side *side, PastUnit *past_unit)
     return past_unitbuf;
 }
 
-/* Shorter past_unit description omits side name, but uses same buffer. */
-
-char *
-short_past_unit_handle(PastUnit *past_unit)
-{
-    int u;
-
-    if (past_unitbuf == NULL)
-      past_unitbuf = xmalloc(BUFSIZE);
-    if (past_unit == NULL)
-      return "???";
-    u = past_unit->type;
-    if (!empty_string(past_unit->name)) {
-	strcpy(past_unitbuf, past_unit->name);
-    } else if (!empty_string(u_short_name(u))) {
-	sprintf(past_unitbuf, "%d%s %s",
-		past_unit->number, ordinal_suffix(past_unit->number),
-		u_short_name(u));
-    } else {
-	sprintf(past_unitbuf, "%d%s %s",
-		past_unit->number, ordinal_suffix(past_unit->number),
-		u_type_name(u));
-    }
-    return past_unitbuf;
-}
-
 /* Given a unit and optional type u, summarize construction status
    and timing. */
 
@@ -736,7 +779,7 @@ research_desc(char *buf, Unit *unit, int a)
     char abuf[10], tmpbuf[100];
 
     if (a != NONATYPE) {
-	if (u_advanced(unit->type)) {
+	if (u_can_research(unit->type)) {
 	    sprintf(abuf, "[%2d] ", a_rp(a));
 	} else {
 	    strcpy(abuf, " --  ");
@@ -818,26 +861,7 @@ constructible_desc(char *buf, Side *side, int u, Unit *unit)
     }
 }
 
-int
-est_completion_time(Unit *unit, int u2)
-{
-    int u, tooluptime, tp;
-
-    u = unit->type;
-    if (uu_acp_to_create(u, u2) < 1)
-      return (-1);
-    tooluptime = 0;
-    tp = (unit->tooling ? unit->tooling[u2] : 0);
-    if (tp < uu_tp_to_build(u, u2)) {
-	if (uu_acp_to_toolup(u, u2) < 1
-	    || uu_tp_per_toolup(u, u2) <= 0
-	    || u_acp(u) <= 0)
-	  return (-1);
-	tooluptime = ((uu_tp_to_build(u, u2) - tp) * uu_acp_to_toolup(u, u2))
-	 / (uu_tp_per_toolup(u, u2) * u_acp(u));
-    }
-    return tooluptime + normal_completion_time(unit->type, u2);
-}
+#if 0		/* Unused. */
 
 void
 historical_event_date_desc(HistEvent *hevt, char *buf)
@@ -845,6 +869,8 @@ historical_event_date_desc(HistEvent *hevt, char *buf)
     sprintf(buf, "%d: ", hevt->startdate);
 }
 
+#endif
+
 int
 find_event_type(Obj *sym)
 {
@@ -857,9 +883,6 @@ find_event_type(Obj *sym)
     return -1;
 }
 
-int pattern_matches_event(Obj *pattern, HistEvent *hevt);
-void event_desc_from_list(Side *side, Obj *lis, HistEvent *hevt, char *buf);
-
 /* (should abstract out evt arg -> unit/pastunit description code) */
 
 void
@@ -1325,6 +1348,9 @@ action_result_desc(int rslt)
       case A_ANY_NO_MATERIAL:
 	str = "insufficient material";
 	break;
+      case A_ANY_NO_AMMO:
+	str = "insufficient ammo";
+	break;
       case A_ANY_TOO_FAR:
 	str = "too far";
 	break;
@@ -1349,6 +1375,9 @@ action_result_desc(int rslt)
       case A_OVERRUN_SUCCEEDED:
 	str = "overrun succeeded";
 	break;
+      case A_FIRE_BLOCKED:
+	str = "fire blocked by terrain";
+	break;
       case A_FIRE_INTO_OUTSIDE_WORLD:
 	str = "cannot fire outside world";
 	break;
@@ -1362,12 +1391,45 @@ action_result_desc(int rslt)
 	str = "misc error";
 	break;
       default:
-	str = "???";
+	if (between(0, rslt, NUMHEVTTYPES - 1))
+	  str = hevtdefns[rslt].name;
+	else
+	  /* This should never happen. */
+	  str = "???";
 	break;
     }
     return str;
 }
 
+/* Explain the reason for failure to move into a cell. */
+
+void
+advance_failure_desc(char *buf, Unit *unit, HistEventType reason)
+{
+    strcpy(buf, unit_handle(unit->side, unit));
+    if (reason == A_ANY_CANNOT_DO)
+      strcat(buf, " can never do this!");
+    else if (reason == A_ANY_NO_ACP)
+      strcat(buf, " does not have enough ACP!");
+    else if (reason == A_ANY_NO_MATERIAL)
+      strcat(buf, " is out of some material!");
+    else if (reason == A_ANY_NO_AMMO)
+      strcat(buf, " is out of ammo!");
+    else if (reason == A_MOVE_NO_MP)
+      /* Player doesn't see mp calcs, so explain as ACP */
+      strcat(buf, " does not have enough ACP!");
+    else if (reason == A_MOVE_CANNOT_LEAVE_WORLD)
+      strcat(buf, " can never leave the world!");
+    else if (reason == A_MOVE_BLOCKING_ZOC)
+      strcat(buf, " cannot enter a blocking ZOC!");
+    else if (reason == A_MOVE_DEST_FULL)
+      strcat(buf, " cannot fit into the destination!");
+    else if (reason != H_UNDEFINED)
+      sprintf(buf, " cannot act, reason is %s", action_result_desc(reason));
+    else
+      strcat(buf, " is unable to act, don't know why");
+}
+
 /* Generate a description of the borders and connections in and around
    a location. */
 
@@ -1382,13 +1444,17 @@ linear_desc(char *buf, int x, int y)
 	    if (t_is_border(t)
 		&& aux_terrain_defined(t)
 		&& any_borders_at(x, y, t)) {
+
+#if 0 /* Confusing. */
 		if (first) {
-		    strcat(buf, " +");
+		    strcat(buf, " at ");
 		    first = FALSE;
 		} else {
 		    strcat(buf, ",");
 		}
 		strcat(buf, t_type_name(t));
+#endif
+
 #if 0 /* takes up more space, but not very useful */
 		for_all_directions(dir) {
 		    if (border_at(x, y, dir, t)) {
@@ -1396,11 +1462,12 @@ linear_desc(char *buf, int x, int y)
 		    }
 		}
 #endif
+
 	    } else if (t_is_connection(t)
 		       && aux_terrain_defined(t)
 		       && any_connections_at(x, y, t)) {
 		if (first) {
-		    strcat(buf, " +");
+		    strcat(buf, " + ");
 		    first = FALSE;
 		} else {
 		    strcat(buf, ",");
@@ -1429,7 +1496,7 @@ elevation_desc(char *buf, int x, int y)
 char *
 feature_desc(Feature *feature, char *buf)
 {
-    int i, capitalize = FALSE;
+    int i, caps = FALSE;
     char *str;
 
     if (feature == NULL)
@@ -1443,13 +1510,13 @@ feature_desc(Feature *feature, char *buf)
 		    /* Interpret substitution directives. */
 		    switch (*(str + 1)) {
 		      case 'T':
-			capitalize = TRUE;
+			caps = TRUE;
 		      case 't':
 			if (feature->typename) {
 			    buf[i] = '\0';
 			    strcat(buf, feature->typename);
-			    if (capitalize && islower(buf[i]))
-			      buf[i] = toupper(buf[i]);
+			    if (caps)
+			      capitalize(buf);
 			    i = strlen(buf);
 			}
 			++str;
@@ -1684,6 +1751,37 @@ supply_desc(char *buf, Unit *unit, int mrow)
     return (strlen(buf) > 0);
 }
 
+/* Describe a builder's current tooling. */
+
+int
+tooling_desc(char *buf, Unit *unit)
+{
+    int u2, num;
+
+    buf[0] = '\0';
+    if (unit->tooling == NULL)
+      return FALSE;
+    /* Use the number of nonzero tooling to govern the use of
+       abbreviations. */
+    num = 0;
+    for_all_unit_types(u2)
+      if (unit->tooling[u2] > 0)
+	++num;
+    if (num == 0) {
+	strcat(buf, "No tooling");
+	return TRUE;
+    }
+    strcat(buf, "Tooling: ");
+    for_all_unit_types(u2) {
+	if (unit->tooling[u2] > 0) {
+	    tprintf(buf, "%s %d/%d  ",
+		    (num > 2 ? shortest_unique_name(u2) : u_type_name(u2)),
+		    unit->tooling[u2], uu_tp_to_build(unit->type, u2));
+	}
+    }
+    return TRUE;
+}
+
 void
 location_desc(char *buf, Side *side, Unit *unit, int u, int x, int y)
 {
@@ -1772,9 +1870,7 @@ destination_desc(char *buf, Side *side, Unit *unit, int x, int y, int z)
 }
 
 void
-latlong_desc(buf, x, y, xf, yf, which)
-char *buf;
-int x, y, xf, yf, which;
+latlong_desc(char *buf, int x, int y, int xf, int yf, int which)
 {
     char minbuf[10];
     int rawlat, latdeg, latmin, rawlon, londeg, lonmin;
@@ -1806,9 +1902,7 @@ int x, y, xf, yf, which;
 }
 
 void
-others_here_desc(buf, unit)
-char *buf;
-Unit *unit;
+others_here_desc(char *buf, Unit *unit)
 {
     int u2, first = TRUE, nums[MAXUTYPES], incomplete[MAXUTYPES];
     Unit *unit2, *top;
@@ -1847,9 +1941,7 @@ Unit *unit;
 }
 
 void
-occupants_desc(buf, unit)
-char *buf;
-Unit *unit;
+occupants_desc(char *buf, Unit *unit)
 {
     int u2, first = TRUE, nums[MAXUTYPES], incomplete[MAXUTYPES];
     Unit *occ;
@@ -1881,12 +1973,11 @@ Unit *unit;
     }
 }
 
-extern char *goal_desc(char *buf, Goal *goal);
+/* Fill the given buffer with a verbal description of the unit's
+   current plan. */
 
 void
-plan_desc(buf, unit)
-char *buf;
-Unit *unit;
+plan_desc(char *buf, Unit *unit)
 {
     char goalbuf[BUFSIZE];
     int i;
@@ -1897,7 +1988,7 @@ Unit *unit;
 	buf[0] = '\0';
     	return;
     }
-    sprintf(buf, "%s", plantypenames[plan->type]);
+    sprintf(buf, "Plan: %s", plantypenames[plan->type]);
     if (plan->waitingfortasks)
       strcat(buf, " Waiting");
     if (plan->asleep)
@@ -1915,28 +2006,32 @@ Unit *unit;
     if (plan->waitingfortransport)
       strcat(buf, " WaitingForTransport");
     if (plan->maingoal) {
-	strcat(buf, " goal ");
+	strcat(buf, ". Goal: ");
 	strcat(buf, goal_desc(goalbuf, plan->maingoal));
     }
     if (plan->formation) {
-	strcat(buf, " formation ");
+	strcat(buf, ". Formation: ");
 	strcat(buf, goal_desc(goalbuf, plan->formation));
     }
+    /* For tasks, just put out a count. */
     if (plan->tasks) {
 	i = 0;
 	for_all_tasks(plan, task)
 	  ++i;
-	tprintf(buf, " %d task%s", i, (i == 1 ? "" : "s"));
+	tprintf(buf, ". %d task%s", i, (i == 1 ? "" : "s"));
     } 
+    strcat(buf, ".");
 }
 
-/* Describe a task in a brief but intelligible way. */
+/* Describe a task in a brief but intelligible way.  Note that the
+   given side and/or unit may be NULL, such as when describing a task
+   not yet assigned to a unit (standing orders for instance). */
 
 void
 task_desc(char *buf, Side *side, Unit *unit, Task *task)
 {
     int i, slen, arg0, arg1, arg2, arg3;
-    char *argtypes;
+    char *argtypes, *sidedesc, *utypedesc;
     Unit *unit2;
 
     if (task == NULL) {
@@ -1947,82 +2042,105 @@ task_desc(char *buf, Side *side, Unit *unit, Task *task)
     arg1 = task->args[1];
     arg2 = task->args[2];
     arg3 = task->args[3];
+    sprintf(buf, "%s ", taskdefns[task->type].display_name);
     switch (task->type) {
       case TASK_BUILD:
-	sprintf(buf, "build %s, ", u_type_name(arg0));
+	tprintf(buf, "%s", u_type_name(arg0));
 	if (arg1 != 0) {
 	    unit2 = find_unit(arg1);
 	    if (unit2 != NULL) {
-		tprintf(buf, "%d/%d done with", unit2->cp, u_cp(unit2->type));
+		tprintf(buf, ", %d/%d done", unit2->cp, u_cp(unit2->type));
 	    }
-	} else {
-	    tprintf(buf, "to start on");
 	}
-	tprintf(buf, " %d%s of %d", arg2 + 1, ordinal_suffix(arg2 + 1), arg3);
+	if (arg3 > 1)
+	  tprintf(buf, " (%d%s of %d)",
+		  arg2 + 1, ordinal_suffix(arg2 + 1), arg3);
+	break;
+      case TASK_CAPTURE:
+	sidedesc = (arg3 == ALLSIDES ? "any" : side_adjective(side_n(arg3)));
+	utypedesc = (arg2 == NONUTYPE ? "unit" : u_type_name(arg2));
+	tprintf(buf, "%s %s ", sidedesc, utypedesc);
+	unit2 = unit_at(arg0, arg1);
+	/* (should scan stack, only list a visible unit) */
+	if (unit2 != NULL 
+	    && unit2->side == side_n(arg3)
+	    && !empty_string(unit2->name)) {
+	    tprintf(buf, "%s ", unit2->name);
+	    tprintf(buf, "(%d,%d)", arg0, arg1);
+	} else {
+	    tprintf(buf, "at %d,%d", arg0, arg1);
+	}
 	break;
       case TASK_COLLECT:
-	sprintf(buf, "collect %s around ", m_type_name(arg0));
-        destination_desc(buf+strlen(buf), side, unit, arg1, arg2, 0);
+	tprintf(buf, "%s around ", m_type_name(arg0));
+	tprintf(buf, "%d,%d", arg0, arg1);
 	break;
       case TASK_DEVELOP:
-	sprintf(buf, "develop %s to tech %d", u_type_name(arg0), arg1);
+	tprintf(buf, "%s to tech %d", u_type_name(arg0), arg1);
 	if (side != NULL) {
 	    tprintf(buf, " (currently at %d)", side->tech[arg0]);
 	}
 	break;
       case TASK_HIT_POSITION:
-	sprintf(buf, "hit position ");
-        destination_desc(buf+strlen(buf), side, unit, arg0, arg1, 0);
+	tprintf(buf, "at %d,%d", arg0, arg1);
 	break;
       case TASK_HIT_UNIT:
-	sprintf(buf, "hit unit at ");
-        destination_desc(buf+strlen(buf), side, unit, arg0, arg1, 0);
-	if (arg2 != NONUTYPE) {
-	    tprintf(buf, " (%s %s)",
-		    side_adjective(side_n(arg3)), shortest_unique_name(arg2));
-      	}
+	sidedesc = (arg3 == ALLSIDES ? "any" : side_adjective(side_n(arg3)));
+	utypedesc = (arg2 == NONUTYPE ? "unit" : u_type_name(arg2));
+	tprintf(buf, "%s %s ", sidedesc, utypedesc);
+	unit2 = unit_at(arg0, arg1);
+	if (unit2 != NULL 
+	    && unit2->side == side_n(arg3)
+	    && !empty_string(unit2->name)) {
+	    tprintf(buf, "%s ", unit2->name);
+	    tprintf(buf, "(%d,%d)", arg0, arg1);
+	} else {
+	    tprintf(buf, "at %d,%d", arg0, arg1);
+	}
 	break;
       case TASK_MOVE_DIR:
-	sprintf(buf, "move %s, %d times", dirnames[arg0], arg1);
+	tprintf(buf, "%s, %d times", dirnames[arg0], arg1);
 	break;
       case TASK_MOVE_TO:
-	sprintf(buf, "move to");
 	if (unit != NULL && unit->x == arg0 && unit->y == arg1) {
-	    tprintf(buf, " here");
-	    break;
-        } else if (arg3 == 0) {
-	    tprintf(buf, " ");
-        } else if (arg3 == 1) {
-	    tprintf(buf, " adj ");
-        } else {
-	    tprintf(buf, " within %d of ", arg3);
-        }
-        destination_desc(buf+strlen(buf), side, unit, arg0, arg1, arg2);
-	break;
+	    tprintf(buf, "here");
+	} else {
+	    if (arg3 == 0) {
+		/* do nothing */
+	    } else if (arg3 == 1) {
+		tprintf(buf, "adj ");
+	    } else {
+		tprintf(buf, "within %d of ", arg3);
+	    }
+	    tprintf(buf, "%d,%d", arg0, arg1);
+	}
+        break;
       case TASK_OCCUPY:
-	sprintf(buf, "occupy ");
 	unit2 = find_unit(arg0);
 	if (unit2 != NULL) {
-	    tprintf(buf, "%s", unit_handle(side, unit2));
+	    tprintf(buf, "%s ", medium_long_unit_handle(unit2));
+	    if (!empty_string(unit2->name)) {
+		tprintf(buf, "(%d,%d)", unit2->x, unit2->y);
+	    } else {
+	   	tprintf(buf, "at %d,%d", unit2->x, unit2->y);
+	    }
 	} else {
 	    tprintf(buf, "unknown unit #%d", arg0);
 	}
 	break;
       case TASK_PICKUP:
-	sprintf(buf, "pick up ");
 	unit2 = find_unit(arg0);
 	if (unit2 != NULL) {
-	    tprintf(buf, "%s", unit_handle(side, unit2));
+	    tprintf(buf, "%s", medium_long_unit_handle(unit2));
 	} else {
 	    tprintf(buf, "unknown unit #%d", arg0);
 	}
 	break;
       case TASK_RESUPPLY:
-	sprintf(buf, "%s", taskdefns[task->type].name);
 	if (arg0 != NONMTYPE)
-	  tprintf(buf, " %s", m_type_name(arg0));
+	  tprintf(buf, "%s", m_type_name(arg0));
 	else
-	  tprintf(buf, " all");
+	  tprintf(buf, "all");
 	if (arg1 != 0) {
 	    unit2 = find_unit(arg1);
 	    if (unit2 != NULL) {
@@ -2031,12 +2149,12 @@ task_desc(char *buf, Side *side, Unit *unit, Task *task)
 	}
 	break;
       case TASK_SENTRY:
-	sprintf(buf, "sentry, for %d turns", arg0);
+	/* (should display as calendar time) */
+	tprintf(buf, "for %d turns", arg0);
 	break;
       default:
 	/* Default is just to dump out the raw data about the task. */
-	sprintf(buf, "%s", taskdefns[task->type].name);
-	tprintf(buf, " raw");
+	tprintf(buf, "raw");
 	argtypes = taskdefns[task->type].argtypes;
 	slen = strlen(argtypes);
 	for (i = 0; i < slen; ++i) {
@@ -2056,13 +2174,12 @@ task_desc(char *buf, Side *side, Unit *unit, Task *task)
 /* Describe a goal in a human-intelligible way. */
 
 char *
-goal_desc(buf, goal)
-char *buf;
-Goal *goal;
+goal_desc(char *buf, Goal *goal)
 {
     int numargs, i, arg;
     char *argtypes;
-
+    Unit *unit;
+    
     if (goal == NULL)
       return "null goal";
     switch (goal->type) {
@@ -2073,13 +2190,22 @@ Goal *goal;
 		goal->args[3]);
 	break;
       case GOAL_VICINITY_HELD:
-	sprintf(buf, "hold %dx%d area around %d,%d",
+	sprintf(buf, "Hold %dx%d area around %d,%d",
 		goal->args[2], goal->args[3], goal->args[0], goal->args[1]);
 	break;
+      case GOAL_UNIT_OCCUPIED:
+	unit = find_unit(goal->args[0]);
+	sprintf(buf, "Occupy %s ", medium_long_unit_handle(unit));
+	    if (!empty_string(unit->name)) {
+		tprintf(buf, "(%d,%d)", unit->x, unit->y);
+	    } else {
+	   	tprintf(buf, "at %d,%d", unit->x, unit->y);
+	    }
+	break;
 	/* (should add more cases specific to common types of goals) */
       default:
 	sprintf(buf, "%s%s", (goal->tf ? "" : "not "),
-		goaldefns[goal->type].name);
+		goaldefns[goal->type].display_name);
 	argtypes = goaldefns[goal->type].argtypes;
 	numargs = strlen(argtypes);
 	for (i = 0; i < numargs; ++i) {
@@ -2125,13 +2251,11 @@ Goal *goal;
     return buf;
 }
 
-/* Format a clock time into a standard form.  This routine will omit the hours
-   part if it will be uninteresting. */
+/* Format a (real, not game) clock time into a standard form.  This
+   routine will omit the hours part if it will be uninteresting. */
 
 void
-time_desc(buf, time, maxtime)
-char *buf;
-int time, maxtime;
+time_desc(char *buf, int time, int maxtime)
 {
     int hour, minute, second;
 
@@ -2147,13 +2271,13 @@ int time, maxtime;
     }
 }
 
+#if 0		/* Unused. */
+
 /* General-purpose routine to take an array of anonymous unit types and
    summarize what's in it, using numbers and unit chars. */
 
 char *
-summarize_units(buf, ucnts)
-char *buf;
-int *ucnts;
+summarize_units(char *buf, int *ucnts)
 {
     char tmp[BUFSIZE];  /* should be bigger? */
     int u;
@@ -2168,13 +2292,12 @@ int *ucnts;
     return buf;
 }
 
-char *get_word(char *str);
-void notify_doctrine_1(Side *side, Doctrine *doctrine);
+#endif
+
+static void notify_doctrine_1(Side *side, Doctrine *doctrine);
 
 void
-notify_doctrine(side, spec)
-Side *side;
-char *spec;
+notify_doctrine(Side *side, char *spec)
 {
     int u;
     char *arg, *rest, substr[BUFSIZE], outbuf[BUFSIZE];
@@ -2186,7 +2309,7 @@ char *spec;
       arg = "";
     if ((doctrine = find_doctrine_by_name(arg)) != NULL) {
 	/* Found a specific named doctrine. */
-	/* (should say which unit types use it) */
+	/* (should say which of our own unit types use it) */
     } else if ((u = utype_from_name(arg)) != NONUTYPE) {
 	doctrine = side->udoctrine[u];
     } else if (strcmp(arg, "default") == 0) {
@@ -2194,12 +2317,17 @@ char *spec;
 	/* (should say which unit types use it) */
     } else {
 	if (!empty_string(arg))
-	  notify(side, "\"%s\" not recognized as doctrine name or unit type.");
-	/* Note that although we list all doctrines, including those belonging
-	   to other sides, we don't know if the other side is actually *using*
-	   any particular doctrine. */
+	  notify(side, "\"%s\" not recognized as doctrine name or unit type.",
+		 arg);
+	/* Note that although we list all doctrines, including those
+	   belonging to other sides, because we don't know if the
+	   other side is actually *using* any particular doctrine.
+	   While this may seem like an information leak, if a side has
+	   its own unique doctrine, curiously enough this corresponds
+	   well to the real world; for instance, present-day nations'
+	   militaries are all familiar with each other's doctrines. */
 	outbuf[0] = '\0';
-	for (doctrine = doctrine_list; doctrine != NULL; doctrine = doctrine->next) {
+	for_all_doctrines(doctrine) {
 	    if (!empty_string(doctrine->name))
 	      tprintf(outbuf, " %s", doctrine->name);
 	    else
@@ -2208,30 +2336,56 @@ char *spec;
 	      tprintf(outbuf, ",");
 	}
 	notify(side, "Doctrines available:%s", outbuf);
+	for_all_doctrines(doctrine) {
+	    notify_doctrine_1(side, doctrine);
+	}
 	return;
     }
     /* We now have a doctrine to display. */
     notify_doctrine_1(side, doctrine);
 }
 
-void
-notify_doctrine_1(side, doctrine)
-Side *side;
-Doctrine *doctrine;
+static void
+notify_doctrine_1(Side *side, Doctrine *doctrine)
 {
+    int u, i, num, dflt;
+    char abuf[BUFSIZE];
+
     if (!empty_string(doctrine->name))
-      notify(side, "Doctrine '%s':", doctrine->name);
+      notify(side, "Doctrine '%s' (%s):",
+	     doctrine->name, (doctrine->locked ? "fixed" : "adjustable"));
     else
-      notify(side, "Doctrine #%d:", doctrine->id);
+      notify(side, "Doctrine #%d (%s):",
+	     doctrine->id, (doctrine->locked ? "fixed" : "adjustable"));
     notify(side, "  Resupply at %d%% of storage", doctrine->resupply_percent);
     notify(side, "  Rearm at %d%% of storage", doctrine->rearm_percent);
     notify(side, "  Repair at %d%% of hp", doctrine->repair_percent);
-    /* (should display construction runs) */
+    abuf[0] = '\0';
+    i = 0;
+    num = 0;
+    dflt = doctrine->construction_run[0];
+#if 0 /* should use histo code in write_table */
+    for_all_unit_types(u) {
+	if (dflt == doctrine->construction_run[u]) {
+	    ++num;
+	}
+    }
+#endif
+    for_all_unit_types(u) {
+	tprintf(abuf, "  %s %d",
+		u_type_name(u), doctrine->construction_run[u]);
+	if (i > 0 && (i++) % 4 == 0 && !empty_string(abuf)) {
+	    notify(side, "  Construction run:%s", abuf);
+	    abuf[0] = '\0';
+	}
+    }
+    if (!empty_string(abuf))
+      notify(side, "  Construction run:%s", abuf);
 }
 
 #define first_person(unit) ((unit)->side && (unit)->side->self_unit == (unit))
 
-int report_combat_special(Unit *unit1, Unit *unit2, char *str);
+static int report_combat_special(Unit *unit1, Unit *unit2, char *str);
 int type_matches_symbol(Obj *sym, int u);
 
 void
@@ -2294,7 +2448,7 @@ notify_combat(Unit *unit1, Unit *unit2, char *str)
     }
 }
 
-int
+static int
 report_combat_special(Unit *unit1, Unit *unit2, char *str)
 {
     int found = FALSE;
@@ -2344,9 +2498,7 @@ report_combat_special(Unit *unit1, Unit *unit2, char *str)
 }
 
 static int
-pattern_matches_combat(parms, unit, unit2)
-Obj *parms;
-Unit *unit, *unit2;
+pattern_matches_combat(Obj *parms, Unit *unit, Unit *unit2)
 {
     Obj *head;
 
@@ -2361,9 +2513,7 @@ Unit *unit, *unit2;
 }
 
 int
-type_matches_symbol(sym, u)
-Obj *sym;
-int u;
+type_matches_symbol(Obj *sym, int u)
 {
     char *typename;
     Obj *val, *rest, *head;
@@ -2395,11 +2545,8 @@ int u;
 }
 
 static void
-combat_desc_from_list(side, lis, unit, unit2, str, buf)
-Side *side;
-Obj *lis;
-Unit *unit, *unit2;
-char *str, *buf;
+combat_desc_from_list(Side *side, Obj *lis, Unit *unit, Unit *unit2,
+		      char *str, char *buf)
 {
     int n;
     char *symname;
@@ -2432,11 +2579,29 @@ char *str, *buf;
 }
 
 void
-report_take(side, unit, needed, rslts)
-Side *side;
-Unit *unit;
-int needed;
-short *rslts;
+report_give(Side *side, Unit *unit, Unit *unit2, short *rslts)
+{
+    char buf[BUFSIZE];
+    int m, something = FALSE;
+
+    sprintf(buf, "%s gave", unit_handle(side, unit));
+    for_all_material_types(m) {
+	if (rslts[m] > 0) {
+	    tprintf(buf, " %d %s", rslts[m], m_type_name(m));
+	    something = TRUE;
+	}
+    }
+    if (!something) {
+	strcat(buf, " nothing");
+    }
+    if (unit2 != NULL) {
+	tprintf(buf, " to %s", unit_handle(side, unit2));
+    }
+    notify(side, "%s.", buf);
+}
+
+void
+report_take(Side *side, Unit *unit, int needed, short *rslts)
 {
     char buf[BUFSIZE];
     int m, something = FALSE;
@@ -2460,8 +2625,7 @@ short *rslts;
 }
 
 void
-notify_all_of_resignation(side, side2)
-Side *side, *side2;
+notify_all_of_resignation(Side *side, Side *side2)
 {
     Side *side3;
 
@@ -2488,8 +2652,7 @@ Side *side, *side2;
 /* Given a number, figure out what suffix should go with it. */
 
 char *
-ordinal_suffix(n)
-int n;
+ordinal_suffix(int n)
 {
     if (n % 100 == 11 || n % 100 == 12 || n % 100 == 13) {
 	return "th";
@@ -2503,14 +2666,14 @@ int n;
     }
 }
 
-/* Pluralize a word, attempting to be smart about various possibilities
-   that don't have a different plural form (such as "Chinese" and "Swiss"). */
+/* Pluralize a word, attempting to be smart about various
+   possibilities that don't have a different plural form (such as
+   "Chinese" and "Swiss"). */
 
 /* There should probably be a test for when to add "es" instead of "s". */
 
 char *
-plural_form(word)
-char *word;
+plural_form(char *word)
 {
     char endch = ' ', nextend = ' ';
     int len;
@@ -2533,20 +2696,13 @@ char *word;
     return pluralbuf;
 }
 
-/* General text generation. */
+/* Make the first letter of the buffer upper-case. */
 
 char *
-make_text(buf, maker, a1, a2, a3, a4)
-char *buf;
-Obj *maker;
-long a1, a2, a3, a4;
+capitalize(char *buf)
 {
-    if (buf == NULL)
-      buf = xmalloc(BUFSIZE);
-    if (maker != lispnil) {
-    } else {
-	sprintf(buf, "%ld %ld %ld %ld", a1, a2, a3, a4);
-    }
+    if (islower(buf[0]))
+      buf[0] = toupper(buf[0]);
     return buf;
 }
 
@@ -2559,8 +2715,8 @@ absolute_date_string(int date)
     if (calendar_type == cal_unknown)
       init_calendar();
     switch (calendar_type) {
-      case cal_turn:
-	sprintf(datebuf, "%s%4d", turnname, date);
+      case cal_number:
+	sprintf(datebuf, "%s%4d", turn_name, date);
 	return datebuf;
       case cal_usual:
 	return usual_date_string(date);
@@ -2575,18 +2731,20 @@ absolute_date_string(int date)
 static void
 init_calendar(void)
 {
-    int caltypeunknown = FALSE;
     Obj *cal, *caltype, *stepname, *steprest;
 
     cal = g_calendar();
-    /* Turn-based calendar is the default. */
-    calendar_type = cal_turn;
-    turnname = "Turn";
+    turn_name = "Turn";
     if (cal == lispnil) {
-	/* Default is fine. */
+	/* The default is to have numbered turns named "Turn". */
+	calendar_type = cal_number;
     } else if (consp(cal)) {
 	caltype = car(cal);
-	if (symbolp(caltype) && strcmp("usual", c_string(caltype)) == 0) {
+	if (match_keyword(caltype, K_NUMBER)) {
+	    calendar_type = cal_number;
+	    if (stringp(cadr(cal)))
+	      turn_name = c_string(cadr(cal));
+	} else if (match_keyword(caltype, K_USUAL)) {
 	    calendar_type = cal_usual;
 	    stepname = cadr(cal);
 	    if (symbolp(stepname)) {
@@ -2601,21 +2759,12 @@ init_calendar(void)
 		date_step_ranges[0].step_size = 1;
 		num_date_step_ranges = 1;
 	    }
-	    if (!empty_string(g_initial_date())) {
-		set_initial_date(g_initial_date());
-	    }
-	} else {
-	    caltypeunknown = TRUE;
 	}
-    } else if (stringp(cal)) {
-	turnname = c_string(cal);
-    } else {
-	caltypeunknown = TRUE;
     }
-    if (caltypeunknown)
-      init_warning("Unknown calendar type");
-    /* If calendar type is unknown, we drop back to using turns - not ideal,
-       could allow bogus game designs to proceed. */
+    if (calendar_type == cal_unknown)
+      init_error("Bad calendar type");
+    if (!empty_string(g_initial_date()))
+      set_initial_date(g_initial_date());
 }
 
 static void
@@ -2669,28 +2818,6 @@ parse_date_step_range(Obj *form)
     ++num_date_step_ranges;
 }
 
-/* Compose a date, omitting components supplied by the base date. */
-
-char *
-relative_date_string(int date, int base)
-{
-    if (calendar_type == cal_unknown)
-      init_calendar();
-    switch (calendar_type) {
-      case cal_turn:
-	sprintf(datebuf, "%d(%d)", date, base);
-	break;
-      case cal_usual:
-	/* (should do this for real eventually) */
-	sprintf(datebuf, "%d(%d)", date, base);
-	break;
-      default:
-	case_panic("calendar type", calendar_type);
-	break;
-    }
-    return datebuf;
-}
-
 /* Given two dates, figure out how many turns encompassed by them. */
 
 int
@@ -2702,7 +2829,7 @@ turns_between(char *datestr1, char *datestr2)
     if (calendar_type == cal_unknown)
       init_calendar();
     switch (calendar_type) {
-      case cal_turn:
+      case cal_number:
 	sscanf("%d", datestr1, &turn1);
 	sscanf("%d", datestr2, &turn2);
 	return (turn2 - turn1);
@@ -2742,13 +2869,15 @@ turns_between(char *datestr1, char *datestr2)
     return 1;  /* appease the compiler */
 }
 
+/* Given a date string, make it be the date of the first turn. */
+
 void
 set_initial_date(char *str)
 {
     if (calendar_type == cal_unknown)
       init_calendar();
     switch (calendar_type) {
-      case cal_turn:
+      case cal_number:
 	sscanf("%d", str, &turn_initial);
 	break;
       case cal_usual:
@@ -2762,8 +2891,8 @@ set_initial_date(char *str)
     }
 }
 
-/* Pick a date out of the given string. Note that this implementation does not
-   detect extra garbage in the string, should fix someday. */
+/* Pick a date out of the given string. Note that this implementation
+   does not detect extra garbage in the string, should fix someday. */
 
 /* (should use strtol etc instead of sscanf) */
 
