@@ -97,15 +97,6 @@ Summary:
 	  as you would with a stack.)
 */
 
-
-#ifndef PARAMS
-#ifdef ANSI_PROTOTYPES
-#define PARAMS(ARGS) ARGS
-#else
-#define PARAMS(ARGS) ()
-#endif /* ANSI_PROTOTYPES */
-#endif /* PARAMS */
-
 /* Don't do the contents of this file more than once.  */
 
 #ifndef __OBSTACK_H__
@@ -162,8 +153,8 @@ struct obstack		/* control current object in current chunk */
   char	*chunk_limit;		/* address of char after current chunk */
   PTR_INT_TYPE temp;		/* Temporary for some macros.  */
   int   alignment_mask;		/* Mask of alignment for each object. */
-  struct _obstack_chunk *(*chunkfun) PARAMS ((int)); /* User's fcn to allocate a chunk.  */
-  void (*freefun) PARAMS ((void *));		/* User's function to free a chunk.  */
+  struct _obstack_chunk *(*chunkfun) (int); /* User's fcn to allocate a chunk.  */
+  void (*freefun) (void *);		/* User's function to free a chunk.  */
   char *extra_arg;		/* first arg for chunk alloc/dealloc funcs */
   unsigned use_extra_arg:1;	/* chunk alloc/dealloc funcs take extra arg */
   unsigned maybe_empty_object:1;/* There is a possibility that the current
@@ -179,9 +170,9 @@ struct obstack		/* control current object in current chunk */
 extern void _obstack_newchunk (struct obstack *, int);
 extern void _obstack_free (struct obstack *, void *);
 extern int _obstack_begin (struct obstack *, int, int,
-			    void *(*) (), void (*) ());
+			    void *(*) (int), void (*) (void *));
 extern int _obstack_begin_1 (struct obstack *, int, int,
-			      void *(*) (), void (*) (), void *);
+			      void *(*) (int), void (*) (void *), void *);
 extern int _obstack_memory_used (struct obstack *);
 #else
 extern void _obstack_newchunk ();
@@ -259,7 +250,7 @@ int obstack_memory_used (struct obstack *obstack);
 
 #define obstack_begin(h, size) \
   _obstack_begin ((h), (size), 0, \
-		  (void *(*) ()) obstack_chunk_alloc, (void (*) ()) obstack_chunk_free)
+		  (void *(*) (int)) obstack_chunk_alloc, (void (*) (void *)) obstack_chunk_free)
 
 #define obstack_specify_allocation(h, size, alignment, chunkfun, freefun) \
   _obstack_begin ((h), (size), (alignment), \
