@@ -1,5 +1,5 @@
 /* Definitions for players in Xconq.
-   Copyright (C) 1992, 1993, 1994, 1996 Stanley T. Shebs.
+   Copyright (C) 1992, 1993, 1994, 1996, 1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@ extern Player *last_player;
 
 extern Assign *assignments;
 
-extern Player *add_player PARAMS ((void));
-extern Player *find_player PARAMS ((int n));
-extern void canonicalize_player PARAMS ((Player *player));
-extern char *player_desig PARAMS ((Player *player));
+extern Player *add_player(void);
+extern Player *find_player(int n);
+extern void canonicalize_player(Player *player);
+extern char *player_desig(Player *player);
