@@ -1,5 +1,5 @@
 /* Definitions for printing in Xconq.
-   Copyright (C) 1994, 1996 Stanley T. Shebs and Massimo Campostrini.
+   Copyright (C) 1994, 1996, 1999 Stanley T. Shebs and Massimo Campostrini.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -36,5 +36,5 @@ typedef struct a_print_parameters {
 
 /* Prototypes. */
 
-extern void init_ps_print PARAMS ((PrintParameters *pp));
-extern void dump_ps_view PARAMS ((Side *side, PrintParameters *pp, char *filename));
+extern void init_ps_print(PrintParameters *pp);
+extern void dump_ps_view(Side *side, PrintParameters *pp, char *filename);
