@@ -1,5 +1,5 @@
 /* PostScript(tm) printing for Xconq.
-   Copyright (C) 1994, 1995 Massimo Campostrini & Stanley T. Shebs.
+   Copyright (C) 1994, 1995, 1999 Massimo Campostrini & Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -56,7 +56,6 @@ Piazza Torricelli 2, I56126 Pisa, Italy  ||  Phone: (+39)(50)911272
 Internet: campo@sunthpi3.difi.unipi.it   ||  Fax:   (+39)(50)48277  */
 
 #include "conq.h"
-extern int num_features PARAMS ((void));
 #include "print.h"
 #include "imf.h"
 #include "ui.h"
@@ -247,23 +246,23 @@ static int conn_x[NUMDIRS], conn_y[NUMDIRS],
 
 /* Local functions. */
 
-static char *add_esc_string PARAMS ((char *str));
+static char *add_esc_string(char *str);
 
-static void ps_cook_imf PARAMS ((void));
-static void ps_initialize_imf PARAMS ((PsImage *psim));
-static char *ps_hex_dump PARAMS ((Obj *data, int w, int h));
-static char *img_untile PARAMS ((char *tiled, int w, int h, int istile,
-			 int dither, int scale, int gray));
-static void misc_init PARAMS ((void));
-static void ps_initialize PARAMS ((FILE *fp));
-static void page_init PARAMS ((FILE *fp, int page, int i, int j, char *stime));
-static int nearest_valid_x PARAMS ((int x, int y));
-static int sideno_of_seen_unit_at PARAMS ((int x, int y, Side *side));
-static char *name_of_seen_unit_at PARAMS ((int x, int y, Side *side));
-static char *summary_of_seen_units_at PARAMS ((int x, int y, Side *side));
-static int ps_string_width PARAMS ((char *str, int font));
-static int print_unit_legends PARAMS ((FILE *fp, char *name, char *summary, char *m,
-	int dir, int cx, int cy));
+static void ps_cook_imf(void);
+static void ps_initialize_imf(PsImage *psim);
+static char *ps_hex_dump(Obj *data, int w, int h);
+static char *img_untile(char *tiled, int w, int h, int istile,
+			int dither, int scale, int gray);
+static void misc_init(void);
+static void ps_initialize(FILE *fp);
+static void page_init(FILE *fp, int page, int i, int j, char *stime);
+static int nearest_valid_x(int x, int y);
+static int sideno_of_seen_unit_at(int x, int y, Side *side);
+static char *name_of_seen_unit_at(int x, int y, Side *side);
+static char *summary_of_seen_units_at(int x, int y, Side *side);
+static int ps_string_width(char *str, int font);
+static int print_unit_legends(FILE *fp, char *name, char *summary, char *m,
+			      int dir, int cx, int cy);
 
 static void
 ps_cook_imf()
@@ -302,7 +301,7 @@ ps_cook_imf()
 	    unit_icon[i].img = NULL;
 	} else {
 	    unit_icon[i].img = unit_icon[i].imf->images;
-	    for (img=unit_icon[i].imf->images->next; img; img=img->next) {
+	    for_all_images(unit_icon[i].imf, img) {
 		/* should we prefer the new image? */
 		if ((!unit_icon[i].img->monodata && img->monodata) ||
 		    (!unit_icon[i].img->maskdata && img->maskdata) ||
@@ -335,7 +334,7 @@ ps_cook_imf()
     }
     
     i = -1;
-    for_all_sides_plus_indep(s) {
+    for_all_sides(s) {
 	i++;
 	side_icon[i].imf = NULL;
 	/* first choice */
@@ -767,7 +766,7 @@ FILE *fp;
     
     /* draw an emblem */
     i = 0;
-    for_all_sides_plus_indep(s) {
+    for_all_sides(s) {
 	if (!side_icon[i].hexmonodata) {
 	    fprintf(fp, "%% icon for side %s not found\n", side_name(s));
 	    fprintf(fp, "/s%X { } def\n", side_number(s));
@@ -894,16 +893,16 @@ int x, y;
 Side *side;
 {
    Unit *unit;
-   short view;
+   UnitView *uview;
 
    if (!in_area(x, y))
      return indepside->id;
    unit = unit_seen_at(side, x, y);
    if (unit)
      return side_number(unit->side);
-   view = unit_view(side, x, y);
-   if (view != EMPTY)
-     return vside(view);
+   uview = unit_view_at(side, x, y);
+   if (uview != NULL)
+     return view_side(uview)->id;
    return indepside->id;
 }
 
@@ -972,10 +971,7 @@ Side *side;
 /* This routine could still use a cleanup */
 
 void
-dump_ps_view(side, ipp, filename)
-Side *side;
-PrintParameters *ipp;
-char *filename;
+dump_ps_view(Side *side, PrintParameters *ipp, char *filename)
 {
     int x, y, xx, i, j, t, nx, utype, page, d, cx, cy, skip, nsp;
     int f, nf, tw, th, font, named_unit_max=100, xmin, ymin, xmin1;
@@ -999,7 +995,7 @@ char *filename;
     
     /* Compute position of legends of geographical features. */
     if (pp->features) {
-	nf = num_features();
+	nf = numfeatures;
 	if (nf > 0) {
 	    legs = (Legend *) malloc((nf + 1) * sizeof(Legend));
 	    if (legs)
@@ -1018,7 +1014,7 @@ char *filename;
     }
     
     /*  open view file  */
-    fp = fopen(filename, "w");
+    fp = open_file(filename, "w");
     if (fp == NULL) {
 	notify(ps_side, "Cannot open \"%s\", dump skipped", filename);
 	return;
@@ -1108,12 +1104,12 @@ char *filename;
 		    if (!in_area(x, y))
 		      continue;
 
-		    for_all_terrain_types(t) {
-			if (t_is_border(t) && aux_terrain_defined(t)) {
+		    for_all_border_types(t) {
+			if (aux_terrain_defined(t)) {
 			    for_all_directions(d) {
 				if (border_at(x, y, d, t) &&
 				    (!side
-				     || all_see_all
+				     || g_see_all()
 				     || seen_border(side, x, y, d))) {
 				    fprintf(fp, "%d %g %d %g li\n",
 					    cx + hexc_x[d],
@@ -1144,11 +1140,11 @@ char *filename;
 		    x = wrapx(x);
 		    if (!in_area(x, y))
 		      continue;
-		    for_all_terrain_types(t) {
-			if (t_is_connection(t) && aux_terrain_defined(t)) {
+		    for_all_connection_types(t) {
+			if (aux_terrain_defined(t)) {
 			    for_all_directions(d) {
 				if (connection_at(x, y, d, t) &&
-				    (!side || all_see_all ||
+				    (!side || g_see_all() ||
 				     terrain_view(side, x, y) != UNSEEN)) {
 				    fprintf(fp, "%d %g %d %g li\n",
 					    cx,
