@@ -10,13 +10,12 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 #include "kernel.h"
-#include "ai.h"
 
 extern int stop_apply;
 
 static void vary_winds(void);
 
-/* Advanced unit support. */
+/* Functions in run.c only called from here. */
 extern void run_people(void);
 extern void run_advanced_units(void);
 extern void run_turn_start(void);
@@ -25,6 +24,7 @@ extern void run_turn_end(void);
 
 extern int gamestatesafe;
 
+static int visible_to(Unit *unit, Unit *unit2);
 static void add_new_sides_to_game(void);
 static void run_tech_leakage(void);
 static void cache_init_tech_levels(void);
@@ -70,7 +70,6 @@ static void damage_unit_with_temperature(Unit *unit, int n);
 static void run_people_side_changes(void);
 static void run_appearances(void);
 static void run_disappearances(void);
-static void run_side_research(void);
 static void run_hp_recovery(void);
 static void run_auto_repair(void);
 static void auto_repair_from_here(int x, int y);
@@ -81,6 +80,7 @@ static void run_unit_fates(void);
 static void run_detonation_accidents(void);
 static void run_people_limits(void);
 static void spy_on_location(int x, int y);
+static int survival_time(Unit *unit);
 
 /* The number of the current turn within a year. */
 
@@ -90,6 +90,8 @@ int curyearpart = -1;
 
 char *curseasonname;
 
+/* The full date/season/day-night string that interfaces should display. */
+
 char *curdatestr;
 
 /* How often to do saves while playing. */
@@ -106,7 +108,7 @@ char **players_requested;
 
 /* True whenever the game has both day and night. */
 
-short daynight = FALSE;
+int daynight = FALSE;
 
 /* The location of the sun, as a position relative to the area.  The
    actual values may be far outside the area. */
@@ -117,6 +119,10 @@ int sunx, suny;
 
 int lastsunx = -1, lastsuny = -1;
 
+/* If the area is uniformly lighted, this is that value. */
+
+int area_lighting = -1;
+
 /* Flags indicating whether various sets of calculations need to be done. */
 
 /* Typically we set to -1 to indicate that the value is uncomputed, then detect
@@ -243,7 +249,6 @@ run_turn_start(void)
       run_supply();
     /* Advanced unit support. */
     run_advanced_units();
-    run_side_research();
     run_hp_recovery();
     run_auto_repair();
     run_self_builds();
@@ -417,8 +422,6 @@ add_new_sides_to_game(void)
 	    /* Recalculate all view info for all sides; simpler than
 	       trying to figure out what's really changed. */
 	    reset_coverage();
-	    /* (should not use this, it will lose the results of
-	       spotting hard-to-see units) */
 	    reset_all_views();
 	    compute_all_feature_centroids();
 	    /* Redraw everything that's displayed. */
@@ -617,6 +620,8 @@ reset_all_reserves(void)
 	if (unit->plan != NULL) {
 	    unit->plan->reserve = FALSE;
 	}
+	unit->buildingdone = FALSE;
+	unit->researchdone = FALSE;
     }
 }
 
@@ -654,7 +659,7 @@ compute_moves(void)
     Unit *unit;
     Side *side;
 
-    for_all_sides_plus_indep(side) {
+    for_all_sides(side) {
 	side->numacting = 0;
 	side->numfinished = 0;
 	for_all_side_units(side, unit) {
@@ -666,25 +671,7 @@ compute_moves(void)
 		   zero.  (This could maybe be done better.) */
 		if (curturn == 1)
 		  unit->act->acp = 0;
-
-		/* Advanced unit support. */
-
-		/* This keeps the option of letting advanced units do
-		   the standard acp calculation followed by action(s)
-		   in the action loop, in addition to the building and
-		   research they do during
-		   run_advanced_units. However, disable_standard_acp
-		   should be set to TRUE for advanced units in order
-		   to avoid a number of complications. Setting it to
-		   FALSE will in effect let advanced units act twice
-		   each turn, once in advanced mode and once in the
-		   standard mode. */
-
-		if (u_disable_standard_acp(unit->type))
-		  unit->act->initacp = unit->act->acp = 0;
-		else
-		  compute_acp(unit);
-
+		compute_acp(unit);
 		update_unit_acp_display(side, unit, FALSE);
 	    }
 	}
@@ -820,6 +807,15 @@ compute_season(void)
 	strcat(curdatestr, curseasonname);
 	strcat(curdatestr, ")");
     }
+    /* Even though this has nothing to do with seasons, this is
+       when the curdatestr is being updated. */
+    if (area_lighting >= 0) {
+	strcat(curdatestr, " (");
+	strcat(curdatestr,
+	       (area_lighting == 2 ? "day" :
+		(area_lighting == 1 ? "twilight" : "night")));
+	strcat(curdatestr, ")");
+    }
 }
 
 static int
@@ -903,22 +899,58 @@ compute_sun(void)
 static void
 run_sun(void)
 {
-    int x, y;
+    int x, y, oldlighting, anychange = FALSE, anyvariation = FALSE;
     Side *side;
 
     compute_sun();
     if (world.daylength <= 1 || world.circumference < area.width)
       return;
+    oldlighting = area_lighting;
+    area_lighting = -1;
     /* Find a cell whose lighting has changed; if so, update the
        whole area at once. */
     for_all_cells(x, y) {
-	if (lighting(x, y, sunx, suny) != lighting(x, y, lastsunx, lastsuny)) {
+	if (area_lighting < 0)
+	  area_lighting = lighting(x, y, sunx, suny);
+	if (!anychange
+	    && (lighting(x, y, sunx, suny)
+		!= lighting(x, y, lastsunx, lastsuny))) {
+	    /* Night may impair units' vision, so redo coverage calcs. */
+	    reset_coverage();
 	    for_all_sides(side) {
 		update_area_display(side);
 	    }
-	    /* (Note that "break" won't work here, because for_all_cells
-	       is actually a pair of nested loops.) */
-	    return;
+	    /* Don't do this code again. */
+	    anychange = TRUE;
+	}
+	/* If not all cells have the same lighting this turn, reset
+	   the area_lighting value. */
+	if (!anyvariation
+	    && lighting(x, y, sunx, suny) != area_lighting) {
+	    area_lighting = -1;
+	    anyvariation = TRUE;
+	}
+    }
+    /* Ensure that any new overall lighting value gets reflected on
+       the screen. */
+    if (area_lighting >= 0) {
+	compute_season();
+	for_all_sides(side) {
+	    update_turn_display(side, TRUE);
+	}
+	/* Announce changes in area lighting. */
+	if (area_lighting != oldlighting) {
+	    switch (area_lighting) {
+	      case 0:
+		notify_all("Night has fallen.");
+		break;
+	      case 1:
+		notify_all("Twilight.");
+		break;
+	      case 2:
+		notify_all("The sun is up.");
+		break;
+	    }
 	}
     }
 }
@@ -1407,8 +1439,7 @@ all_see_cell_weather(int x, int y)
    location, figure out just what it is going to see. */
 
 static void
-spy_on_location(x, y)
-int x, y;
+spy_on_location(int x, int y)
 {
     int qual;
     Unit *unit2, *occ;
@@ -1436,7 +1467,7 @@ int x, y;
 /* Certain kinds of units can do spying for the side they're on. */
 
 static void
-run_spies()
+run_spies(void)
 {
     int u, chance;
     Unit *unit;
@@ -1475,7 +1506,7 @@ run_spies()
    just damage it. */
 
 static void
-run_accidents()
+run_accidents(void)
 {
     int u, t, chance, hit;
     Unit *unit;
@@ -1506,7 +1537,10 @@ run_accidents()
 	    	/* Kill the unit outright. */
 		kill_unit(unit, H_UNIT_VANISHED);
 		/* (should make a hevt) */
-	    } else if ((chance = ut_accident_hit(u, t)) > 0 && xrandom(10000) < chance) {
+		continue;
+	    }
+	    chance = ut_accident_hit(u, t);
+	    if (chance > 0 && xrandom(10000) < chance) {
 		/* Damage the unit. */
 		hit = roll_dice(ut_accident_damage(u, t));
 		if (hit > 0) {
@@ -1523,7 +1557,7 @@ run_accidents()
    for those. */
 
 static void
-run_attrition()
+run_attrition(void)
 {
     int u, t, dmg;
     Unit *unit;
@@ -1560,7 +1594,7 @@ run_attrition()
    overall state of the world into account. */
 
 static void
-run_revolts()
+run_revolts(void)
 {
     int u;
     Unit *unit;
@@ -1590,9 +1624,7 @@ run_revolts()
 /* Test for and run a single revolt. */
 
 static void
-unit_revolt(unit, force)
-Unit *unit;
-int force;
+unit_revolt(Unit *unit, int force)
 {
     int u = unit->type, ux = unit->x, uy = unit->y, chance, lo, opin;
     int sideweights[MAXSIDES+1], sn2, sum, n, i, psum;
@@ -1621,7 +1653,7 @@ int force;
 	    /* Find all the non-allied sides in the game and weight
 	       each according to the unit's opinion of the side. */
 	    sum = 0;
-	    for_all_sides_plus_indep(side2) {
+	    for_all_sides(side2) {
 		sn2 = side_number(side2);
 		sideweights[sn2] = 0;
 		if ((side2 == NULL || side2 == indepside || side2->ingame)
@@ -1630,10 +1662,7 @@ int force;
 		    && unit_allowed_on_side(unit, side2)
 		    /* Don't go over to a side that doesn't even know
                        we're there. */
-		    && (side2->see_all
-			|| side2 == indepside
-			|| (unit_view(side2, ux, uy) != EMPTY
-			    && vtype(unit_view(side2, ux, uy)) == unit->type))
+		    && side_sees_image(side2, unit)
 		    ) {
 		    sideweights[sn2] =
 		      unit_opinion(unit, side2) + u_opinion_min(u);
@@ -1657,7 +1686,7 @@ int force;
 	       each according to the number of units of the same type
 	       present. */
 	    sum = 0;
-	    for_all_sides_plus_indep(side2) {
+	    for_all_sides(side2) {
 		sn2 = side_number(side2);
 		sideweights[sn2] = 0;
 		if ((side2 == NULL || side2 == indepside || side2->ingame)
@@ -1666,10 +1695,7 @@ int force;
 		    && unit_allowed_on_side(unit, side2)
 		    /* Don't go over to a side that doesn't even know
                        we're there. */
-		    && (side2->see_all
-			|| side2 == indepside
-			|| (unit_view(side2, ux, uy) != EMPTY
-			    && vtype(unit_view(side2, ux, uy)) == unit->type))
+		    && side_sees_image(side2, unit)
 		    ) {
 		    sideweights[sn2] = 1;
 		    for_all_side_units(side2, unit2) {
@@ -1720,7 +1746,7 @@ int force;
 /* Test whether surrenders can happen in this game. */
 
 static void
-run_surrenders()
+run_surrenders(void)
 {
     int u1, u2, u3, range;
     Unit *unit;
@@ -1761,8 +1787,7 @@ run_surrenders()
    view to work from.  We sort of compute the view "on the fly". */
 
 static void
-unit_surrender(unit)
-Unit *unit;
+unit_surrender(Unit *unit)
 {
     int u = unit->type, dir, x1, y1, range /*, surrounded = TRUE */;
     Unit *unit2;
@@ -1802,18 +1827,14 @@ Unit *unit;
 
 /* Calculate whether one unit is visible to another, even if the other
    is independent. */
-
+/* (should rewrite this) */
 int
-visible_to(unit, unit2)
-Unit *unit, *unit2;
+visible_to(Unit *unit, Unit *unit2)
 {
-    int uview;
-
     if (g_see_all()) {
 	return TRUE;
-    } else if (unit->side != NULL) {
-    	uview = unit_view(unit->side, unit2->x, unit2->y);
-	return (uview != EMPTY && vtype(uview) == unit2->type);
+    } else if (side_sees_unit(unit->side, unit2)) {
+	return TRUE;
     } else {
 	/* (should be more careful to check see-chances) */
     	if (distance(unit->x, unit->y, unit2->x, unit2->y)
@@ -1825,8 +1846,7 @@ Unit *unit, *unit2;
 }
 
 static void
-maybe_surrender_to(unit, unit2)
-Unit *unit, *unit2;
+maybe_surrender_to(Unit *unit, Unit *unit2)
 {
     int chance;
 
@@ -1843,22 +1863,19 @@ int tmpneed;
    is gone. */
 
 static int
-excess_left(x, y)
-int x, y;
+excess_left(int x, int y)
 {
     return (tmpexcess > 0);
 }
 
 static int
-need_left(x, y)
-int x, y;
+need_left(int x, int y)
 {
     return (tmpneed > 0);
 }
 
 static void
-try_transfer_to_cell(x, y)
-int x, y;
+try_transfer_to_cell(int x, int y)
 {
     Unit *unit2, *occ;
 
@@ -1911,7 +1928,7 @@ try_sharing_with_cell(int x, int y)
 /* The main routine does production, distribution, and discarding in order. */
 
 static void
-run_economy()
+run_economy(void)
 {
     int u, m, t, amt, dist, x, y, x1, y1, m1, m2;
     int prod, ptivity, dir, ptivityadj, stor, oldamt, newamt;
@@ -2143,9 +2160,7 @@ run_economy()
 }
 
 static int
-base_production(unit, m)
-Unit *unit;
-int m;
+base_production(Unit *unit, int m)
 {
     int u = unit->type, occprod;
 
@@ -2160,9 +2175,7 @@ int m;
 /* Give away supplies, but save enough to stay alive for a couple turns. */
 
 static void
-try_transfer(from, to, m)
-Unit *from, *to;
-int m;
+try_transfer(Unit *from, Unit *to, int m)
 {
     int oldsupply = from->supply[m];
 
@@ -2171,9 +2184,7 @@ int m;
 }
 
 void
-try_sharing_need(from, to, m)
-Unit *from, *to;
-int m;
+try_sharing_need(Unit *from, Unit *to, int m)
 {
     int oldsupply = to->supply[m];
 
@@ -2185,9 +2196,7 @@ int m;
 /* Note that this may be called on newly-completed units during a turn. */
 
 void
-try_sharing(from, to, m)
-Unit *from, *to;
-int m;
+try_sharing(Unit *from, Unit *to, int m)
 {
     try_transfer_aux(from, to, m);
 }
@@ -2198,9 +2207,7 @@ int m;
    give away supplies than net consumers. */
 
 static void
-try_transfer_aux(from, to, m)
-Unit *from, *to;
-int m;
+try_transfer_aux(Unit *from, Unit *to, int m)
 {
     int nd, u = from->type, u2 = to->type, fromrate, torate;
 
@@ -2248,6 +2255,28 @@ int m;
     }
 }
 
+/* Check how long a unit can sit where it is */
+
+static int
+survival_time(Unit *unit)
+{
+    int u = unit->type, m, least = 99999, rate, tmp;
+    int t = terrain_at(unit->x, unit->y);
+
+    for_all_material_types(m) {
+	rate = (um_base_consumption(u, m)
+		- (um_base_production(u, m) * ut_productivity(u, t)) / 100);
+	if (rate > 0) {
+	    tmp = unit->supply[m];
+	    if (unit->act) {
+		tmp += unit->act->actualmoves * um_consumption_per_move(u, m);
+	    }
+	    least = min(least, tmp / rate);
+	}
+    }
+    return least;
+}
+
 /* This estimates if a need can be met.  Note that total transfer of
    requested amount is not a good idea, since the supplies might be
    essential to the unit that has them first.  If we're not worried
@@ -2256,9 +2285,7 @@ int m;
 /* (should replace with doctrine/plan controls, check underlying terrain) */
 
 static int
-can_satisfy_need(unit, m, need)
-Unit *unit;
-int m, need;
+can_satisfy_need(Unit *unit, int m, int need)
 {
     int supp = unit->supply[m];
     int stor = um_storage_x(unit->type, m);
@@ -2275,7 +2302,7 @@ int m, need;
    automatically when they get to a certain point. */
 
 static void
-run_self_builds()
+run_self_builds(void)
 {
     int u, cpper;
     Unit *unit;
@@ -2314,7 +2341,7 @@ run_self_builds()
 }
 
 static void
-run_environment_effects()
+run_environment_effects(void)
 {
     int err, dmg;
     Unit *unit;
@@ -2341,7 +2368,9 @@ run_environment_effects()
     for_all_units(unit) {
 	if (is_active(unit)) {
 	    attrition = u_temp_attrition(unit->type);
-	    if (attrition != lispnil) {
+	    if (attrition != lispnil
+		&& !(unit->transport != NULL
+		     && uu_temp_protect(unit->transport->type, unit->type))) {
 		err = interpolate_in_list(temperature_at(unit->x, unit->y), attrition, &dmg);
 		if (err != 0) {
 		    dmg = 0;
@@ -2354,9 +2383,7 @@ run_environment_effects()
 }
 
 static void
-damage_unit_with_temperature(unit, dmg)
-Unit *unit;
-int dmg;
+damage_unit_with_temperature(Unit *unit, int dmg)
 {
     int n;
 
@@ -2374,7 +2401,7 @@ int dmg;
 }
 
 static void
-run_people_side_changes()
+run_people_side_changes(void)
 {
     int x, y, u, t;
     Unit *unit;
@@ -2429,8 +2456,7 @@ run_people_side_changes()
 }
 
 int
-people_surrender_chance(u, x, y)
-int u, x, y;
+people_surrender_chance(int u, int x, int y)
 {
     int m, chance, peop;
 
@@ -2482,8 +2508,8 @@ change_people_side_around(int x, int y, int u, Side *side)
     /* (should add ability to change adjacent cells also) */
 }
 
-/* See if the numbers of individuals in a cell exceeds the max, and migrate or
-   remove so as to bring the numbers back in line. */
+/* See if the numbers of individuals in a cell exceeds the max, and
+   migrate or remove so as to bring the numbers back in line. */
 
 static void
 run_people_limits(void)
@@ -2676,55 +2702,6 @@ run_disappearances(void)
     }
 }
 
-/* Run any per-side research activities. */
-
-static void
-run_side_research(void)
-{
-    int a, m, consump, maxrp, rp, amt;
-    Side *side;
-
-    /* No research if there are no advances. */
-    if (numatypes == 0)
-      return;
-    if (!g_side_can_research())
-      return;
-    for_all_sides(side) {
-	a = side->research_topic;
-	/* Two different values that do nothing - one is player-specified,
-	   the other triggers a query to the player. */
-	if (a == NOADVANCE || a == NONATYPE)
-	  continue;
-	/* Compute the maximum number of rps that our material limits
-	   allow us to add. */
-	maxrp = 999999;
-	for_all_material_types(m) {
-	    consump = am_consumption_per_rp(a, m);
-	    if (consump > 0) {
-		rp = side->treasury[m] / consump;
-		maxrp = min(maxrp, rp);
-	    }
-	}
-	/* Add 1 rp per turn by default. */
-	if (maxrp == 999999)
-	  maxrp = 1;
-	side->advance[a] += maxrp;
-	if (side->advance[a] >= a_rp(a)) {
-	    side->advance[a] = DONE;
-	    notify(side, "Your wise men discover %s!", a_type_name(a));
-	    side->research_topic = NOADVANCE;
-	}
-	/* Consume the materials. */
-	for_all_material_types(m) {
-	    consump = am_consumption_per_rp(a, m);
-	    if (consump > 0) {
-		amt = maxrp * consump;
-		side->treasury[m] -= amt;
-	    }
-	}
-    }
-}
-
 /* Some types of units recover lost hp spontaneously. */
 
 static void
@@ -3056,8 +3033,7 @@ run_unit_fates(void)
 }
 
 static void
-lost_unit_surrender(unit)
-Unit *unit;
+lost_unit_surrender(Unit *unit)
 {
     int u = unit->type, dir, x1, y1;
     Unit *unit2;
@@ -3086,8 +3062,7 @@ Unit *unit;
 }
 
 static void
-maybe_surrender_lost_to(unit, unit2)
-Unit *unit, *unit2;
+maybe_surrender_lost_to(Unit *unit, Unit *unit2)
 {
     int chance;
 
@@ -3098,7 +3073,7 @@ Unit *unit, *unit2;
 }
 
 static void
-run_detonation_accidents()
+run_detonation_accidents(void)
 {
     int u, t, x, y, z, chance;
     Unit *unit;
@@ -3360,7 +3335,7 @@ run_unit_base_consumption(void)
    turns, and not just once at the point of supply exhaustion. */
 
 static void
-run_unit_starvation()
+run_unit_starvation(void)
 {
     int u, m;
     Unit *unit;
@@ -3395,9 +3370,7 @@ run_unit_starvation()
    how the dice roll... */
 
 void
-maybe_starve(unit, partial)
-Unit *unit;
-int partial;
+maybe_starve(Unit *unit, int partial)
 {
     int u = unit->type, m, starv, oneloss, hploss = 0;
 
@@ -3429,9 +3402,7 @@ int partial;
 /* (should be more sophisticated and account for supply lines etc) */
 
 static int
-in_supply(unit, m)
-Unit *unit;
-int m;
+in_supply(Unit *unit, int m)
 {
     if (unit->transport != NULL) {
     	if (unit->transport->supply[m] > 0)
