@@ -23,10 +23,6 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 #include "kernel.h"
 
-static int read_scorefile(void);
-static int interp_score_record(Obj *form);
-static char *basic_player_name(Player *player);
-
 typedef struct a_score_record {
     char *gamename;
     Obj *sides;
@@ -47,6 +43,9 @@ typedef struct a_score_record {
 #define for_all_score_records(sr)  \
   for ((sr) = records; (sr) != NULL; (sr) = (sr)->next)
 
+static int read_scorefile(void);
+static int interp_score_record(Obj *form);
+static char *basic_player_name(Player *player);
 static void eval_sk_last_side_wins(Scorekeeper *sk);
 static void eval_sk_last_alliance_wins(Scorekeeper *sk);
 static void score_variant_desc(ScoreRecord *sr, char *varbuf);
@@ -580,6 +579,8 @@ eval_sk_test(Side *side, Scorekeeper *sk, Obj *form)
     return FALSE;
 }
 
+/* Test whether a single side is still alive in the game. */
+
 static void
 eval_sk_last_side_wins(Scorekeeper *sk)
 {
@@ -587,37 +588,39 @@ eval_sk_last_side_wins(Scorekeeper *sk)
     int numleft = 0, points;
 
     /* This is only meaningful in games with at least two sides. */
-    if (num_sides_originally >= 2) {
-	for_all_sides(side2) {
-	    if (side2->ingame) {
-		points = side_point_value(side2);
-		Dprintf("%s has %d points worth of units\n",
-			side_desig(side2), points);
-		if (points == 0) {
-		  if (!only_checking)
-		    side_loses(side2, NULL, sk->id);
-		} else if (side2 == indepside
-			   && !side_has_ai(side2)
-			   && !side_has_display(side2)) {
-		    /* The independent side should not be counted
-		       among the sides left (unless it has somebody
-		       running it), otherwise the but-I-already-won!
-		       player has to scour the map looking for
-		       remaining inert independents. */
-		} else {
-		    ++numleft;
-		    winner = side2;
-		}
+    if (num_sides_originally < 2)
+      return;
+    for_all_sides(side2) {
+	/* The independent side should not be counted among the sides
+	   left (unless it has somebody running it), otherwise the
+	   but-I-already-won!  player has to scour the map looking for
+	   remaining inert independents. */
+	if (inactive_indepside(side2))
+	  continue;
+	if (side2->ingame) {
+	    points = side_point_value(side2);
+	    Dprintf("%s has %d points worth of units\n",
+		    side_desig(side2), points);
+	    if (points == 0) {
+		if (!only_checking)
+		  side_loses(side2, NULL, sk->id);
+	    } else {
+		++numleft;
+		/* Take note of the possible winner. */
+		winner = side2;
 	    }
 	}
-	if (numleft == 1) {
-	  we_have_a_winner = TRUE;
-	  if (!only_checking)
-	    side_wins(winner, sk->id);
-	}
+    }
+    if (numleft == 1) {
+	we_have_a_winner = TRUE;
+	if (!only_checking)
+	  side_wins(winner, sk->id);
     }
 }
 
+/* Test whether a single alliance (a group of trusting sides) is all
+   that remains in the game. */
+
 static void
 eval_sk_last_alliance_wins(Scorekeeper *sk)
 {
@@ -626,62 +629,70 @@ eval_sk_last_alliance_wins(Scorekeeper *sk)
     int alliancewins;
 
     /* This is only meaningful in games with at least two sides. */
-    if (num_sides_originally >= 2) {
-	for_all_sides(side2) {
-	    sidepoints[side2->id] = alliancepoints[side2->id] = 0;
-	    if (side2->ingame) {
-		sidepoints[side2->id] = side_point_value(side2);
-		Dprintf("%s has %d points worth of units\n",
-			side_desig(side2), sidepoints[side2->id]);
-	    }
+    if (num_sides_originally < 2)
+      return;
+    for_all_sides(side2) {
+	sidepoints[side2->id] = alliancepoints[side2->id] = 0;
+	if (side2->ingame) {
+	    sidepoints[side2->id] = side_point_value(side2);
+	    Dprintf("%s has %d points worth of units\n",
+		    side_desig(side2), sidepoints[side2->id]);
 	}
-	/* Sum up to get points for each alliance. */
-	for_all_sides(side2) {
-	    if (side2->ingame) {
-		for_all_sides(side3) {
-		    if (side2 == side3 || trusted_side(side2, side3)) {
-			alliancepoints[side2->id] += sidepoints[side3->id];
-		    }
+    }
+    /* Sum up to get points for each alliance. */
+    for_all_sides(side2) {
+	if (side2->ingame) {
+	    for_all_sides(side3) {
+		if (side2 == side3 || trusted_side(side2, side3)) {
+		    alliancepoints[side2->id] += sidepoints[side3->id];
 		}
 	    }
 	}
-	/* Make all the sides belonging to point-less alliances lose now,
-	   and look for a non-losing side. */
-	for_all_sides(side2) {
-	    if (side2->ingame) {
-		if (alliancepoints[side2->id] == 0) {
-		    if (!only_checking)
-		      side_loses(side2, NULL, sk->id);
-		} else if (side2 == indepside
-			   && !side_has_ai(side2)
-			   && !side_has_display(side2)) {
-		    /* See comment in eval_sk_last_side_wins. */
-		} else {
-		    ++numleft;
-		    winner = side2;
-		}
+    }
+    /* Make all the sides belonging to point-less alliances lose now,
+       and look for a non-losing side. */
+    for_all_sides(side2) {
+	if (inactive_indepside(side2))
+	  continue;
+	if (side2->ingame) {
+	    if (alliancepoints[side2->id] == 0) {
+		if (!only_checking)
+		  side_loses(side2, NULL, sk->id);
+	    } else {
+		++numleft;
+		/* We still need to pick a single side, we'll take
+		   care of all its buddies shortly. */
+		winner = side2;
 	    }
 	}
-	if (numleft > 0) {
-	    /* See if the non-losers all belong to a single alliance. */
-	    alliancewins = TRUE;
-	    for_all_sides(side2) {
-		if (side2->ingame && side2 != winner
-		    && !trusted_side(winner, side2)) {
-		    alliancewins = FALSE;
-		    break;
-		}
-	    }
-	    /* If so, everybody in the alliance wins equally. */
-	    if (alliancewins) {
-		we_have_a_winner = TRUE;
-		for_all_sides(side2) {
-		    if (side2->ingame
-			&& (side2 == winner || trusted_side(winner, side2))) {
-			if (!only_checking)
-			  side_wins(side2, sk->id);
-		    }
-		}
+    }
+    /* It may happen that all sides lose (for instance if the last two
+       units die in a duel). */
+    if (numleft == 0)
+      return;
+    /* See if the non-losers all belong to a single alliance. */
+    alliancewins = TRUE;
+    for_all_sides(side2) {
+	if (inactive_indepside(side2))
+	  continue;
+	if (side2->ingame
+	    && side2 != indepside
+	    && side2 != winner
+	    && !trusted_side(winner, side2)) {
+	    alliancewins = FALSE;
+	    break;
+	}
+    }
+    /* If so, everybody in the alliance wins equally. */
+    if (alliancewins) {
+	we_have_a_winner = TRUE;
+	for_all_sides(side2) {
+	    if (inactive_indepside(side2))
+	      continue;
+	    if (side2->ingame
+		&& (side2 == winner || trusted_side(winner, side2))) {
+		if (!only_checking)
+		  side_wins(side2, sk->id);
 	    }
 	}
     }
@@ -866,9 +877,7 @@ record_into_scorefile(void)
     /* (should make following code into a separate routine) */
     adv = -1;
     for_all_sides(side) {
-	adv2 = max(1, side->advantage);
-	if (side->player)
-	  adv2 = side->player->advantage;
+	adv2 = actual_advantage(side);
 	if (adv < 1)
 	  adv = adv2;
 	if (adv != adv2) {
@@ -973,10 +982,7 @@ record_into_scorefile(void)
 			    : "drew")));
 		/* (should write info about ai helping human players) */
 		if (any_advantage_variation) {
-		    /* (should have cached as "actual advantage"?) */
-		    advantage = max(1, side->advantage);
-		    if (side->player)
-		      advantage = side->player->advantage;
+		    advantage = actual_advantage(side);
 		    if (advantage > 1)
 		      fprintf(fp, " (a %d)", advantage); 
 		}
