@@ -1,5 +1,5 @@
 /* Definitions relating to scorekeepers in Xconq.
-   Copyright (C) 1987-1989, 1991-1997 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1997, 1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -45,24 +45,24 @@ extern int numscores;
 extern int any_post_action_scores;
 extern int any_post_event_scores;
 
-extern void init_scorekeepers PARAMS ((void));
-extern Scorekeeper *create_scorekeeper PARAMS ((void));
-extern Scorekeeper *find_scorekeeper PARAMS ((int id));
-extern void init_scores PARAMS ((void));
-extern void check_pre_turn_scores PARAMS ((void));
-extern void check_post_turn_scores PARAMS ((void));
-extern void check_post_action_scores PARAMS ((void));
-extern void check_post_event_scores PARAMS ((void));
-extern void run_scorekeeper PARAMS ((Side *side, Scorekeeper *sk));
-extern int eval_sk_form PARAMS ((Side *side, Scorekeeper *sk, Obj *form));
-extern int sum_property PARAMS ((Side *side, Obj *form));
-extern int point_value PARAMS ((Unit *unit));
-extern int side_point_value PARAMS ((Side *side));
-extern int eval_sk_test PARAMS ((Side *side, Scorekeeper *sk, Obj *form));
-extern void side_wins PARAMS ((Side *side, int why));
-extern void side_loses PARAMS ((Side *side, Side *side2, int why));
-extern void all_sides_draw PARAMS ((void));
-extern void record_into_scorefile PARAMS ((void));
-extern int should_try_to_win PARAMS ((Side *side));
-extern char *get_scores PARAMS ((Side *side));
+extern void init_scorekeepers(void);
+extern Scorekeeper *create_scorekeeper(void);
+extern Scorekeeper *find_scorekeeper(int id);
+extern void init_scores(void);
+extern void check_pre_turn_scores(void);
+extern void check_post_turn_scores(void);
+extern void check_post_action_scores(void);
+extern void check_post_event_scores(void);
+extern void run_scorekeeper(Side *side, Scorekeeper *sk);
+extern int eval_sk_form(Side *side, Scorekeeper *sk, Obj *form);
+extern int sum_property(Side *side, Obj *form);
+extern int point_value(Unit *unit);
+extern int side_point_value(Side *side);
+extern int eval_sk_test(Side *side, Scorekeeper *sk, Obj *form);
+extern void side_wins(Side *side, int why);
+extern void side_loses(Side *side, Side *side2, int why);
+extern void all_sides_draw(void);
+extern void record_into_scorefile(void);
+extern int should_try_to_win(Side *side);
+extern char *get_scores(Side *side);
 
