@@ -1,5 +1,5 @@
 /* Definitions for sides in Xconq.
-   Copyright (C) 1987-1989, 1991-1997, 1999 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1997, 1999-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -97,21 +97,19 @@ typedef struct a_side {
     char *uorders;		/* bit vector of types that have orders */
     Obj *possible_units;	/* list of units to give to side */
     short ai_may_resign;	/* true if AI may draw or resign on its own */
-    short numadvances;		/* Number of completed advances */
     short *advance;		/* State of research (accum pts) for each */ 
     				/* advance. Set to -1 when completed. */
-    short research_topic;	/* Advance that the side is working on. */
-    short autoresearch;		/* TRUE if next research topic is autoselected. */
-    short *treasury;		/* Globally available supply of each
-				   material type. */
-    short *c_rates;		/* Material conversion rates. */
+    short research_topic;	/* advance that the side is working on */
+    short autoresearch;		/* TRUE if next research topic autoselected */
+    short *treasury;		/* globally available supply of each
+				   material type */
+    short *c_rates;		/* material conversion rates */
     /* View layers. */
     char *terrview;		/* ptr to view of terrain */
     char **auxterrview;		/* ptr to view of aux terrain */
     short *terrviewdate;	/* ptr to dates of view of terrain */
     short **auxterrviewdate;	/* ptr to dates of view of aux terrain */
-    short *unitview;		/* ptr to view of units */          
-    short *unitviewdate;	/* ptr to dates of view of units */          
+    struct a_unit_view **unit_views;  /* ptr to table of views of units */
     short **materialview;	/* ptr to view of cell materials */
     short **materialviewdate;	/* ptr to dates of view of cell materials */
     short *tempview;		/* ptr to view of temperature */
@@ -127,7 +125,7 @@ typedef struct a_side {
     short everingame;		/* true if side ever participated in a turn */
     short priority;		/* overall action priority of this side */
     short status;		/* -1/0/1 for lost/draw/won */
-    short *scores;		/* an array of scores managed by scorekeepers */
+    short *scores;		/* array of scores managed by scorekeepers */
     Obj *rawscores;		/* score data as read from file */
     short willingtodraw;	/* will this side go along with quitters? */
     short autofinish;		/* turn done when all units acted */
@@ -162,27 +160,39 @@ typedef struct a_side {
     struct a_ai *ai;		/* pointer to the AI making decisions */
     Obj *aidata;		/* readable/writable form of AI state */
     struct a_rui *rui;		/* pointer to interface data if remote */
+    struct a_rai *rai;		/* pointer to AI data if remote */
     /* Misc working slots, never saved. */
     short startx, starty;	/* approx center of side's "country" */
     short busy;			/* true when side state not saveable */
     short finalradius;
     short willingtosave;	/* will this side go along with saving game? */
-    short see_all;		/* true if this side sees everything */
-    short show_all;		/* true if this side displays everything */
-    short may_set_show_all;	/* true if player can ask to disp everything */
+    /* True if the side sees everything.  Right now this is just a cache
+       for g_see_all().  Note in particular that becoming a designer
+       should not change this value. */
+    short see_all;
+    /* True if the side's interface should display everything.  This is
+       seeded from side->see_all, and also becomes true when designing
+       or when the game is over. */
+    short show_all;
+    /* True if the value of show_all can be changed by the player.  This
+       is normally off, but can be toggled while designing or after the
+       game is over. */
+    short may_set_show_all;
     short unit_view_restored;	/* true if unit view was read in */
 #ifdef DESIGNERS
     short designer;		/* true if side is doing scenario design */
 #endif /* DESIGNERS */
     /* Various caches, should be possible to recalc as needed. */
-    short *uavail;		/* vector of types allowed on this side */
+    short *uavail;			/* vector of types allowed on this side (as determined by 
+    					u_possible_sides)*/
+    short *canbuild;		/* vector of unit types that side has advances to build */
+    short *canresearch;		/* vector of advances that side has advances to research */
     struct a_unit *unithead;	/* points to list of all units on this side */
     struct a_unitvector *actionvector;  /* vector of acting units */
     short numwaiting;		/* number of units waiting to get orders */
     short *coverage;		/* indicates how many looking at this cell */
     short *alt_coverage;	/* indicates minimum altitude visible */
     short *numunits;		/* number of units the side has */
-    short *numlive;		/* number of live & in-play units the side has */
     short numacting;		/* number of units that can do things */
     short numfinished;		/* number of units that have finished acting */
     long turnstarttime;		/* real time at start of turn */
@@ -202,24 +212,17 @@ typedef struct a_side {
 
 /* Iteration over all sides. */
 
-/* This is the old for_all_sides which does not include indepside. It
-   now also avoids null pointer crashes by checking that the sidelist
-   exists. */
+/* This is the old for_all_sides which does not include indepside. */
 
 #define for_all_real_sides(v)  \
   for (v = (sidelist ? sidelist->next : NULL); v != NULL; v = v->next)
 
-/* This is the new indepside-including version of for_all_sides. */
+/* This is the new indepside-including version of for_all_sides. It has
+replaced for_all_side_plus_indep everywhere. */
 
 #define for_all_sides(v)  \
   for (v = sidelist; v != NULL; v = v->next)
 
-/* This is kept for compatibility with those places in the code where
-   there already was a specific need to include indepside. */
-
-#define for_all_sides_plus_indep(v)  \
-  for (v = sidelist; v != NULL; v = v->next)
-
 /* Macros for accessing and setting a side's view data.  In general,
    interfaces should go through these macros instead of accessing the
    world's state directly. */
@@ -254,37 +257,26 @@ typedef struct a_side {
 #define set_aux_terrain_view_date(s,x,y,t,v)  \
   (((s)->auxterrviewdate && (s)->auxterrviewdate[t]) ? aset((s)->auxterrviewdate[t], x, y, v) : 0)
 
-/* Basic view is encoded as hhhhsssssuuuuuuu + 1, where the u's are the unit
-   type, s's are the side number, and h's are approx hp/size.  0 == empty. */
-   
-#define unit_view(s,x,y)  \
-  ((s)->unitview ? aref((s)->unitview, x, y) : EMPTY)
+/* Iterate over all unit views at a given location.  Note that if
+   side->see_all, the view objects are built on-the-fly and all
+   actually point to the same bit of storage, so you can't expect to
+   keep them from one loop iteration to the next. */
 
-#define set_unit_view(s,x,y,v)  \
-  ((s)->unitview ? aset((s)->unitview, x, y, v) : 0)
+#define for_all_view_stack(s,x,y,uv) \
+  for ((uv) = unit_view_at(s,x,y); (uv) != NULL; (uv) = unit_view_next(s,x,y,uv))
 
-#define builduview(s,u) (((((s) & 0x1f) << 7) | ((u) & 0x7f)) + 1)
+/* Access to various properties of a unit view. */
 
-#define vside(v) ((((v) - 1) >> 7) & 0x1f)
+#define view_type(uv) ((uv)->type)
 
-#define vtype(v) (((v) - 1) & 0x7f)
+#define view_side(uv) (side_n((uv)->side_id))
 
-#define builduviewhp(s,u,hp)  \
-  (((((hp) & 0x0f) << 12) | (((s) & 0x1f) << 7) | ((u) & 0x7f)) + 1)
+#define view_size(uv) ((uv)->size)
 
-#define vhp(v) ((((v) - 1) >> 12) & 0x0f)
+/* Get a pointer to the actual unit being viewed if possible. */
 
-#define EMPTY (0)
-
-#define appears_empty(v) ((v) == EMPTY)
-
-#define vside_indep(v) (vside(v) == 0)
-
-#define unit_view_date(s,x,y)  \
-  ((s)->unitviewdate ? aref((s)->unitviewdate, x, y) : 0)
-
-#define set_unit_view_date(s,x,y,v)  \
-  ((s)->unitviewdate ? aset((s)->unitviewdate, x, y, v) : 0)
+#define view_unit(uv) \
+  (((uv)->unit != NULL && (uv)->id == (uv)->unit->id) ? (uv)->unit : NULL)
 
 /* Manipulation of view of cell materials. */
 
@@ -391,9 +383,13 @@ typedef struct a_side {
 
 #define side_has_display(s) (((s)->ui) != NULL || ((s)->rui) != NULL)
 
-#define side_has_local_display(side) ((side)->ui != NULL)
+#define side_has_local_display(s) ((s)->ui != NULL)
 
-#define side_has_ai(s) (((s)->ai) != NULL)
+#define side_has_ai(s) (((s)->ai) != NULL || ((s)->rai) != NULL)
+
+#define side_has_local_ai(s) ((s)->ai != NULL)
+
+/* Tests of side state. */
 
 #define side_lost(s) ((s) != NULL && !(s)->ingame && (s)->status < 0)
 
@@ -418,8 +414,14 @@ typedef struct a_side {
 #define units_visible(side, x, y)  \
   ((side)->see_all || (cover((side), wrapx(x), (y)) >= 1))
 
-/* Handle crashes due to null indep side. */
-#define has_advance(side,a) (side != NULL ? ((side)->advance[(a)] == -1) : FALSE)
+#define has_advance(side,a) ((side)->advance[(a)] == -1)
+
+#define side_can_build(side,a) ((side)->canbuild != NULL ? (side)->canbuild[(a)] : TRUE)
+
+#define side_can_research(side,a) ((side)->canresearch != NULL ? (side)->canresearch[(a)] : TRUE)
+
+#define inactive_indepside(side)  \
+  ((side) == indepside && !side_has_ai(side) && !side_has_display(side))
 
 #ifdef DESIGNERS
 #define is_designer(side) (side->designer)
@@ -427,6 +429,9 @@ typedef struct a_side {
 #define is_designer(side) (FALSE)
 #endif /* DESIGNERS */
 
+#define for_all_doctrines(d)  \
+  for ((d) = doctrine_list; (d) != NULL; (d) = (d)->next)
+
 /* Side-related variables. */
 
 extern Side *sidelist;
@@ -453,9 +458,9 @@ typedef struct a_agreement {
     SideMask proposers;       /* sides ready to propose the draft agreement */
     SideMask signers;         /* proposed list of signers */
     SideMask willing;         /* sides that have indicated agreement so far */
-    SideMask knownto;         /* sides that are aware of the signed agreement */
+    SideMask knownto;         /* sides that are aware of signed agreement */
     struct a_obj *terms;      /* list of specific terms */
-    short enforcement;        /* true if program should attempt to enforce terms */
+    short enforcement;        /* true if program should try to enforce terms */
     struct a_agreement *next;
 } Agreement;
 
@@ -528,6 +533,9 @@ extern int allied_side(Side *s1, Side *s2);
 extern int neutral_side(Side *s1, Side *s2);
 extern void reveal_side(Side *sender, Side *recipient, int *types);
 extern void receive_message(Side *side, Side *sender, char *str);
+extern struct a_unit_view *unit_view_at(Side *side, int x, int y);
+extern struct a_unit_view *unit_view_next(Side *side, int x, int y,
+					  struct a_unit_view *uview);
 extern void all_see_occupy(struct a_unit *unit, int x, int y, int inopen);
 extern void all_see_leave(struct a_unit *unit, int x, int y, int inopen);
 extern void cover_area(Side *side, struct a_unit *unit,
@@ -548,6 +556,8 @@ extern char *side_desig(Side *side);
 
 extern Side *parse_side_spec(char *str);
 
+extern int actual_advantage(Side *side);
+
 extern Doctrine *new_doctrine(int id);
 extern Doctrine *find_doctrine(int id);
 extern Doctrine *find_doctrine_by_name(char *name);
@@ -571,8 +581,6 @@ extern void fn_set_terrain_view(int x, int y, int val);
 extern void fn_set_aux_terrain_view(int x, int y, int val);
 extern void fn_set_terrain_view_date(int x, int y, int val);
 extern void fn_set_aux_terrain_view_date(int x, int y, int val);
-extern void fn_set_unit_view(int x, int y, int val);
-extern void fn_set_unit_view_date(int x, int y, int val);
 extern void fn_set_material_view(int x, int y, int val);
 extern void fn_set_material_view_date(int x, int y, int val);
 extern void fn_set_temp_view(int x, int y, int val);
@@ -585,3 +593,5 @@ extern void fn_set_wind_view(int x, int y, int val);
 extern void fn_set_wind_view_date(int x, int y, int val);
 
 extern int load_side_config(Side *side);
+
+extern void update_side_display_all_sides(Side *side, int rightnow);
