@@ -18,18 +18,6 @@ any later version.  See the file COPYING.  */
 #include "kpublic.h"
 #include "print.h"
 #include "cmdline.h"
-extern char *default_player_spec;
-extern void make_default_player_spec(void);
-
-extern void auto_pick_new_research(Unit *unit);
-extern int auto_pick_new_build_task(Unit *unit);
-extern void auto_pick_new_plan(Unit *unit);
-void unit_research_dialog(Unit *unit);
-void side_research_dialog(Side *side, Unit *unit, int s);
-void unit_advance_dialog(Unit *unit);
-void global_advance_dialog(Side *side, Unit *unit, int s);
-int unit_build_dialog(Unit *unit);
-void unit_plan_dialog(Unit *unit);
 
 static void init_displays(void);
 static void get_input(void);
@@ -391,9 +379,9 @@ list_actors(Side *side, Obj *cmd, Obj *parms)
 	       side_desig(side2),
 	       (side2->finishedturn ? "(finished)" : ""));
     }
-    for_all_sides_plus_indep(side2) {
+    for_all_sides(side2) {
 	for (i = 0; i < side2->actionvector->numunits; ++i) {
-	    unit = (side2->actionvector->units)[i].unit;
+	    unit = unit_in_vector(side2->actionvector, i);
 	    list_one_unit(unit);
 	}
     }
@@ -860,6 +848,13 @@ update_side_display(Side *side, Side *side2, int rightnow)
 	printf("Update %s: side %s%s\n",
 	       side_desig(side), side2desc, (rightnow ? " (now)" : ""));
     }
+    if (numatypes > 0
+	&& g_side_can_research()
+	&& side == side2
+	&& side->research_topic == NOADVANCE) {
+	/* Should eventually pop up a side research dialog instead. */
+	auto_pick_side_research(side);
+    }
 }
 
 /* This hook should update the side's view of the given unit, no matter
@@ -1127,45 +1122,10 @@ serial_port_dialog()
 
 #endif /* MAC */
 
-extern void auto_pick_new_research(Unit *unit);
-extern int auto_pick_new_build_task(Unit *unit);
-extern void auto_pick_new_plan(Unit *unit);
-
-void
-unit_advance_dialog(Unit *unit)
-{
-    auto_pick_new_research(unit);
-}
-
-void
-global_advance_dialog(Side *side, Unit *unit, int s)
-{
-    auto_pick_new_research(unit);
-}
-
 void
 unit_research_dialog(Unit *unit)
 {
-    auto_pick_new_research(unit);
-}
-
-void
-side_research_dialog(Side *side, Unit *unit, int s)
-{
-    auto_pick_new_research(unit);
-}
-
-int
-unit_build_dialog(Unit *unit)
-{
-    int u = auto_pick_new_build_task(unit);
-    return u;
-}
-
-void
-unit_plan_dialog(Unit *unit)
-{
-    auto_pick_new_plan(unit);
+    auto_pick_unit_research(unit);
 }
 
 void
