@@ -113,6 +113,7 @@ static void zone_sort(void);
 static void zone_shuffle(void);
 static int compare_zone_nodes(const void *a, const void *b);
 static void extend_workspace(int semantic_bytes);
+static void run_supply_side(Side *side);
 
 static int supply_initialized = FALSE;
 
@@ -199,6 +200,8 @@ supply_system_in_use(void)
     return mclass_count ? TRUE : FALSE;
 }
 
+#if 0		/* Unused. */
+
 void
 compute_supply_zones(side, m)
 Side *side;
@@ -234,6 +237,8 @@ int m;
     run_supply_lines(side, side_number(side), m);
 }
 
+#endif
+
 /* init_supply_system() determines if, and how, the supply algorithm will
  * be used. */
 static void
