@@ -1,5 +1,5 @@
 /* Definitions and functions for Xconq's tables.
-   Copyright (C) 1991, 1992, 1993, 1994, 1996 Stanley T. Shebs.
+   Copyright (C) 1991, 1992, 1993, 1994, 1996, 1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -33,6 +33,15 @@ TableDefn tabledefns[] = {
 #undef  DEF_MM_TABLE
 #define DEF_MM_TABLE(NAME,FNAME,DOC,TABLE,CNST,LO,DFLT,HI,VALTYPE)  \
     { NAME, FNAME, DOC, &TABLE, DFLT, LO, HI, MTYP, MTYP, VALTYPE },
+#undef  DEF_UA_TABLE
+#define DEF_UA_TABLE(NAME,FNAME,DOC,TABLE,CNST,LO,DFLT,HI,VALTYPE)  \
+    { NAME, FNAME, DOC, &TABLE, DFLT, LO, HI, UTYP, ATYP, VALTYPE },
+#undef  DEF_AM_TABLE
+#define DEF_AM_TABLE(NAME,FNAME,DOC,TABLE,CNST,LO,DFLT,HI,VALTYPE)  \
+    { NAME, FNAME, DOC, &TABLE, DFLT, LO, HI, ATYP, MTYP, VALTYPE },
+#undef  DEF_AA_TABLE
+#define DEF_AA_TABLE(NAME,FNAME,DOC,TABLE,CNST,LO,DFLT,HI,VALTYPE)  \
+    { NAME, FNAME, DOC, &TABLE, DFLT, LO, HI, ATYP, ATYP, VALTYPE },
 
 #include "table.def"
 
@@ -83,4 +92,25 @@ TableDefn tabledefns[] = {
     { checkm(m1);  checkm(m2);  \
       return (TABLE ? TABLE[nummtypes*(m1)+(m2)] : CNST); }
 
+#undef  DEF_UA_TABLE
+#define DEF_UA_TABLE(name,FNAME,doc,TABLE,CNST,lo,DFLT,hi,valtype)  \
+  short *TABLE = NULL, CNST = DFLT;  \
+  int FNAME(u,a) int u, a;  \
+    { checku(u);  checka(a);  \
+      return (TABLE ? TABLE[numatypes*(u)+(a)] : CNST); }
+
+#undef  DEF_AM_TABLE
+#define DEF_AM_TABLE(name,FNAME,doc,TABLE,CNST,lo,DFLT,hi,valtype)  \
+  short *TABLE = NULL, CNST = DFLT;  \
+  int FNAME(a,m) int a, m;  \
+    { checka(a);  checkm(m);  \
+      return (TABLE ? TABLE[nummtypes*(a)+(m)] : CNST); }
+
+#undef  DEF_AA_TABLE
+#define DEF_AA_TABLE(name,FNAME,doc,TABLE,CNST,lo,DFLT,hi,valtype)  \
+  short *TABLE = NULL, CNST = DFLT;  \
+  int FNAME(a1,a2) int a1, a2;  \
+    { checka(a1);  checka(a2);  \
+      return (TABLE ? TABLE[numatypes*(a1)+(a2)] : CNST); }
+
 #include "table.def"
