@@ -9,9 +9,6 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 #include "kernel.h"
 
-extern int need_ai_task_reaction;
-extern int target_visible(Unit *unit, Task *task);
-
 /* This is the number of tasks to allocate initially.  More will always be
    allocated as needed, so this should be a "reasonable" value. */
 
@@ -36,13 +33,19 @@ static int extractable_test(int x, int y);
 static int repair_test(int x, int y);
 static int resupply_test(int x, int y);
 static TaskOutcome execute_task_aux(Unit *unit, Task *task);
-static TaskOutcome do_approach_subtask(Unit *unit, Task *task, int tx, int ty,
-				       short *statep);
+static TaskOutcome do_approach_subtask(Unit *unit, Task *task, int tx, int ty, int *statep);
+static void allocate_task_block(void);
+static int fire_can_damage(Unit *unit, Unit *unit2);
+static Unit *repair_here(int x, int y);
+static Unit *aux_resupply_here(Unit *unit);
+static Unit *resupply_here(int x, int y);
+static int can_auto_resupply_self(Unit *unit, int *materials, int numtypes);
+
 
 /* Declare all the task functions. */
 
 #undef  DEF_TASK
-#define DEF_TASK(name,code,argtypes,DOFN,createfn,setfn,netsetfn,pushfn,netpushfn,argdecl)  \
+#define DEF_TASK(name,dname,code,argtypes,DOFN,createfn,setfn,netsetfn,pushfn,netpushfn,argdecl)  \
   static TaskOutcome DOFN(Unit *unit, Task *task);
 
 #include "task.def"
@@ -52,8 +55,8 @@ static TaskOutcome do_approach_subtask(Unit *unit, Task *task, int tx, int ty,
 TaskDefn taskdefns[] = {
 
 #undef  DEF_TASK
-#define DEF_TASK(NAME,code,ARGTYPES,DOFN,createfn,setfn,netsetfn,pushfn,netpushfn,argdecl)  \
-  { NAME, ARGTYPES, DOFN },
+#define DEF_TASK(NAME,DNAME,code,ARGTYPES,DOFN,createfn,setfn,netsetfn,pushfn,netpushfn,argdecl)  \
+  { NAME, DNAME, ARGTYPES, DOFN },
 
 #include "task.def"
 
@@ -75,7 +78,7 @@ static Unit *tmpbuilder, *tmpbuildunit;
 /* Allocate an initial collection of task objects. */
 
 void
-init_tasks()
+init_tasks(void)
 {
     allocate_task_block();
 }
@@ -83,7 +86,7 @@ init_tasks()
 /* Allocate a new block of tasks. */
 
 void
-allocate_task_block()
+allocate_task_block(void)
 {
     int i;
 
@@ -98,8 +101,7 @@ allocate_task_block()
 /* Create and return a new task. */
 
 Task *
-create_task(type)
-TaskType type;
+create_task(TaskType type)
 {
     int i;
     Task *task;
@@ -121,9 +123,10 @@ TaskType type;
     return task;
 }
 
+/* Make a copy of the given task. */
+
 Task *
-clone_task(oldtask)
-Task *oldtask;
+clone_task(Task *oldtask)
 {
     int i;
     Task *newtask;
@@ -376,9 +379,9 @@ do_capture_task(Unit *unit, Task *task)
       case 1:
 	for_all_stack(tx, ty, unit2) {
 	    if ((ts2 >= 0 ?
-		 (side_number(unit2->side) == ts2) :
-		 (unit2->side != us /* should be "not a friendly side" */))
-		&& (tu2 == NONUTYPE || tu2 == unit->type)) {
+		 (unit2->side->id == ts2) :
+		 enemy_side(us, unit2->side))
+		&& (tu2 == NONUTYPE || tu2 == unit2->type)) {
 		if (valid(check_capture_action(unit, unit, unit2))) {
 		    prep_capture_action(unit, unit, unit2);
 		    return TASK_PREPPED_ACTION;
@@ -402,7 +405,8 @@ do_capture_task(Unit *unit, Task *task)
 	    && flip_coin()) {
 	    return TASK_IS_INCOMPLETE;
 	}
-	/* If out of range and can move, push a task to get closer (usually). */
+	/* If out of range and can move, push a task to get closer
+           (usually). */
 	if (mobile(u) && probability(90)) {
 	    push_move_to_task(unit, tx, ty, 1);
 	    return TASK_IS_INCOMPLETE;
@@ -451,6 +455,7 @@ do_hit_unit_task(Unit *unit, Task *task)
 {
     int u = unit->type, tx, ty, dist, movedist, tu, ts;
     Unit *unit2;
+    UnitView *uview;
     Side *us = unit->side;
 
     /* This is to hit a (given type/side of) unit at a given place. */
@@ -460,12 +465,13 @@ do_hit_unit_task(Unit *unit, Task *task)
     dist = distance(tx, ty, unit->x, unit->y);
     if (dist <= 1 /* direct attack range */) {
 	if (can_attack(unit)) {
-	    for_all_stack(tx, ty, unit2) {
-		if (!trusted_side(us, unit2->side)
-		    && (tu == NONUTYPE || unit2->type == tu)
+	    for_all_view_stack(us, tx, ty, uview) {
+		if (!trusted_side(us, view_side(uview))
+		    && (tu == NONUTYPE || tu == view_type(uview))) {
 		    /* (should check designated side to hit, if defined) */
-		    ) {
-		    if (valid(check_attack_action(unit, unit, unit2, 100))
+		    unit2 = view_unit(uview);
+		    if (unit2 != NULL
+			&& valid(check_attack_action(unit, unit, unit2, 100))
 			&& attack_can_damage_or_capture(unit, unit2)) {
 			prep_attack_action(unit, unit, unit2, 100);
 			return TASK_PREPPED_ACTION;
@@ -483,12 +489,14 @@ do_hit_unit_task(Unit *unit, Task *task)
 	}
 	/* Might be able to fire at pointblank range. */
 	if (can_fire(unit) && dist >= u_range_min(u)) {
-	    for_all_stack(tx, ty, unit2) {
-		if (!trusted_side(us, unit2->side)
-		    && (tu == NONUTYPE || unit2->type == tu)
+	    for_all_view_stack(us, tx, ty, uview) {
+		if (!trusted_side(us, view_side(uview))
+		    && (tu == NONUTYPE || tu == view_type(uview))
 		    /* (should check designated side to hit, if defined) */
 		    ) {
-		    if (valid(check_fire_at_action(unit, unit, unit2, -1))
+		    unit2 = view_unit(uview);
+		    if (unit2 != NULL
+			&& valid(check_fire_at_action(unit, unit, unit2, -1))
 			/* Also count with capture when firing at
 			   pointblank range. */
 			&& fire_can_damage_or_capture(unit, unit2)) {
@@ -504,12 +512,16 @@ do_hit_unit_task(Unit *unit, Task *task)
 	/* should move further away */
 	return TASK_FAILED;
     }
-    /* This is why units failed to fire at max range. */
+    /* If we're within firing range, attempt to fire. */
     if (dist <= u_range(u)) {
-	if (units_visible(us, tx, ty)) {
-	    /* Check if we may safely move in and attempt to capture a unit. */
+	for_all_view_stack(us, tx, ty, uview) {
 	    int danger = FALSE;
 
+	    unit2 = view_unit(uview);
+	    if (unit2 == NULL)
+	      continue;
+	    /* Check if we may safely move in and attempt to capture a unit. */
+
 	    for_all_stack_with_occs(tx, ty, unit2) {
 		int u2 = unit2->type;
 		
@@ -517,9 +529,6 @@ do_hit_unit_task(Unit *unit, Task *task)
 		  continue;
 		if (!is_active(unit2))
 		  continue;
-		/* No cheating! */
-		if (!side_sees_image(unit->side, unit2))
-		  continue;
 		if (u_acp(u2) + u_acp_to_fire(u2) <= 0)
 		  continue;
 		if (uu_hit(u2, u) <= 0)
@@ -579,8 +588,7 @@ do_hit_unit_task(Unit *unit, Task *task)
 /* Return true if the unit can actually damage the other unit. */
 
 int
-attack_can_damage_or_capture(unit, unit2)
-Unit *unit, *unit2;
+attack_can_damage_or_capture(Unit *unit, Unit *unit2)
 {
     Unit *occ;
 
@@ -625,11 +633,11 @@ Unit *unit, *unit2;
     return FALSE;
 }
 
-/* Return true if the unit can actually damage the other unit. */
+/* Return true if the unit can actually damage the other unit by
+   firing at it. */
 
 int
-fire_can_damage(unit, unit2)
-Unit *unit, *unit2;
+fire_can_damage(Unit *unit, Unit *unit2)
 {
     Unit *occ;
 
@@ -779,29 +787,18 @@ real_capture_chance(Unit *unit, Unit *unit2)
 int
 target_visible(Unit *unit, Task *task)
 {
-    int tx, ty, uview, tu, ts;
-    Unit *unit2;
+    int tx, ty, tu, ts;
     Side *us = unit->side;
+    UnitView *uview;
 
     tx = task->args[0];  ty = task->args[1];
     tu = task->args[2];  ts = task->args[3];
 
-    if (units_visible(us, tx, ty)) {
-	for_all_stack(tx, ty, unit2) {
-	    if (!trusted_side(us, unit2->side)
-	    	&& (tu == NONUTYPE || unit2->type == tu)
-	    	/* (should check designated side to hit, if defined) */
-	    	) {
-	    	return TRUE;
-	    }
-	}
-    } else {
-	/* Assess old image or non-image. */
-	uview = unit_view(us, tx, ty);
-	if (uview != EMPTY
-	    && (tu == NONUTYPE || tu == vtype(uview))) {
-	    return TRUE;
-	}
+    for_all_view_stack(us, tx, ty, uview) {
+	if (!trusted_side(us, view_side(uview))
+	    && (tu == NONUTYPE || tu == view_type(uview)))
+	  /* (should test task's target side also?) */
+	  return TRUE;
     }
     return FALSE;
 }
@@ -863,16 +860,29 @@ do_move_to_task(Unit *unit, Task *task)
 	return TASK_IS_COMPLETE;
       case 1:
 	/* Adjacent cell, do a single move. */
-	if (unit_at(tx, ty)) {
-	    for_all_stack(tx, ty, unit2) {
-		if (can_occupy(unit, unit2)) {
-		    if (valid(check_enter_action(unit, unit, unit2))) {
-			prep_enter_action(unit, unit, unit2);
-			return TASK_PREPPED_ACTION;
-		    } else {
-			continue;
-		    }
-		} else if (!trusted_side(unit->side, unit2->side)) {
+	/* But first, if there are units here already, prefer to
+	   interact with them. */
+	for_all_stack(tx, ty, unit2) {
+	    /* If there's somebody that we can enter, prefer to do that. */
+	    if (can_occupy(unit, unit2)
+		&& valid(check_enter_action(unit, unit, unit2))) {
+		prep_enter_action(unit, unit, unit2);
+		return TASK_PREPPED_ACTION;
+	    }
+	    /* Perhaps an occupant... */
+	    /* (We could use the recursive occupant test, but if
+	       things are that complicated, the player should probably
+	       exercise manual control here.) */
+	    for_all_occupants(unit2, occ) {
+		if (can_occupy(unit, occ)
+		    && valid(check_enter_action(unit, unit, occ))) {
+		    prep_enter_action(unit, unit, occ);
+		    return TASK_PREPPED_ACTION;
+		}
+	    }
+	}
+#if 0 /* auto-attack on move should be player-controlled... */
+		if (!trusted_side(unit->side, unit2->side)) {
 		    /* This is probably not a good idea, combat odds not
 		       taken into account. */
 		    if (valid(check_attack_action(unit, unit, unit2, 100))) {
@@ -881,30 +891,20 @@ do_move_to_task(Unit *unit, Task *task)
 		    } else {
 			continue;
 		    }
-		} else {
-		    /* We could find room in an occupant... */
-		    for_all_occupants(unit2, occ) {
-			if (can_occupy(unit, occ)) {
-			    if (valid(check_enter_action(unit, unit, occ))) {
-				prep_enter_action(unit, unit, occ);
-				return TASK_PREPPED_ACTION;
-			    }
-			}
-		    }
 		}
-	    }
-	    return TASK_FAILED;
-	}
+#endif
+	/* Now try a basic move action. */
 	if (valid(check = check_move_action(unit, unit, tx, ty, unit->z))) {
 	    /* Moving into an empty cell. */
 	    prep_move_action(unit, unit, tx, ty, unit->z);
 	    return TASK_PREPPED_ACTION;
 	} else {
-	    /* If we're just short on mp, wait until the next turn to try to move. */
+	    /* If we're just short on mp, wait until the next turn to
+               try to move. */
 	    if (check == A_MOVE_NO_MP) {
-		if (unit->side)
-		  notify(unit->side, "%s has insufficient MP to move this turn; waiting until next",
-				     unit_handle(unit->side, unit));
+		notify(unit->side,
+		       "%s has insufficient ACP to move this turn; waiting until next",
+		       unit_handle(unit->side, unit));
 		set_unit_reserve(unit->side, unit, TRUE, FALSE);
 		return TASK_IS_INCOMPLETE;
 	    }
@@ -932,7 +932,7 @@ do_move_to_task(Unit *unit, Task *task)
 }
 
 static TaskOutcome
-do_approach_subtask(Unit *unit, Task *task, int tx, int ty, short *statep)
+do_approach_subtask(Unit *unit, Task *task, int tx, int ty, int *statep)
 {
     int nx, ny, dirs[NUMDIRS], numdirs, i, numdirs2, check;
     Unit *unit2;
@@ -1132,7 +1132,8 @@ do_occupy_task(Unit *unit, Task *task)
 	}
     } else {
 	/* Still some distance away, pick a way to go. */
-	return do_approach_subtask(unit, task, transport->x, transport->y, &(task->args[1]));
+	return do_approach_subtask(unit, task, transport->x, transport->y,
+				   &(task->args[1]));
     }
 }
 
@@ -1147,9 +1148,12 @@ do_pickup_task(Unit *unit, Task *task)
     if (!in_play(occupant))
       return TASK_FAILED;
     wake_unit(occupant->side, occupant, FALSE);
-    if (occupant->transport == unit) {
-	return TASK_IS_COMPLETE;
-    } else if (task->execnum > 10) {
+    if (occupant->transport == unit)
+      return TASK_IS_COMPLETE;
+    if (distance(unit->x, unit->y, occupant->x, occupant->y) > 1)
+      return do_approach_subtask(unit, task, occupant->x, occupant->y,
+				 &(task->args[1]));
+    if (task->execnum > 10) {
 	/* Waiting around isn't working for us, give up.  If the
 	   prospective occupant still needs us, we'll get another
 	   call. */
@@ -1400,13 +1404,15 @@ repair_test(int x, int y)
 
 static int *lowm = NULL, numlow; 
 
+/* Test the given unit to see if it or one of its occupants can help
+   the unit looking for supplies. */
+
 Unit *
 aux_resupply_here(Unit *unit)
 {
     int i, enough = TRUE;
     Unit *occ;
 
-    /* what about allies? */
     if (unit_trusts_unit(unit, tmpunit)
 	&& can_carry(unit, tmpunit)) {
 	for (i = 0; i < numlow; ++i) {
@@ -1576,7 +1582,14 @@ execute_task(Unit *unit)
     /* This should never happen. */
     if (unit->plan == NULL)
       run_error("???");
-    if (need_ai_task_reaction && unit->plan->last_task_outcome != TASK_UNKNOWN)
+    /* If the unit is AI-run, don't do more than task execution with
+       it during each run step. */
+    if (unit->plan->last_task_outcome != TASK_UNKNOWN
+	&& unit->plan->last_task_outcome != TASK_PREPPED_ACTION
+	&& side_has_ai(unit->side)
+	/* unless it is acp-independent ... */
+	&& !acp_indep(unit)
+	&& !unit->plan->aicontrol)
       return unit->plan->last_task_outcome;
     task = plan->tasks;
     if (task == NULL)
@@ -1587,9 +1600,6 @@ execute_task(Unit *unit)
     /* Record it. */
     memcpy(&(unit->plan->last_task), task, sizeof(Task));
     unit->plan->last_task_outcome = rslt;
-    if (unit->side != NULL && side_has_ai(unit->side)) {
-	need_ai_task_reaction = TRUE;
-    }
     /* Now look at what happened with task execution. */
     switch (rslt) {
       case TASK_UNKNOWN:
@@ -1802,6 +1812,7 @@ compare_directions(CONST void *a0, CONST void *a1)
     int u = tmputype, t0, t1;
     int ux = tmpunit->x, uy = tmpunit->y, u2 = NONUTYPE;
     int cost0 = 0, cost1 = 0, s, ps0, ps1, surr0, surr1, rslt;
+    int cs0, cs1;
     extern int *any_people_surrenders;
 
     i0 = *((int *) a0);  i1 = *((int *) a1);
@@ -1832,7 +1843,8 @@ compare_directions(CONST void *a0, CONST void *a1)
 	return rslt;
     }
     /* Prefer to go over cells that we can change to our side. */
-    /* (should also do for control-side?) */
+    /* (should control via doctrine, this will reveal movements
+       more often) */
     if (any_people_surrenders != NULL
 	&& any_people_surrenders[u]
 	&& people_sides_defined()) {
@@ -1847,13 +1859,22 @@ compare_directions(CONST void *a0, CONST void *a1)
 	    return surr1 - surr0;
     	}
     }
+    if (control_sides_defined()) {
+    	s = side_number(tmpunit->side);
+    	cs0 = control_side_at(xs[i0], ys[i0]);
+    	cs1 = control_side_at(xs[i1], ys[i1]);
+	/* (should test trusted side) */
+    	surr0 = ((ps0 != NOBODY && s != ps0) ? 1 : 0);
+    	surr1 = ((ps1 != NOBODY && s != ps1) ? 1 : 0);
+    	if (surr0 != surr1) {
+	    return surr1 - surr0;
+    	}
+    }
     return 0;
 }
 
 void
-sort_directions(unit, dirs, numdirs)
-Unit *unit;
-int *dirs, numdirs;
+sort_directions(Unit *unit, int *dirs, int numdirs)
 {
     int i, tmp, i0 = 0, i1 = 1, compar;
 
@@ -1879,18 +1900,14 @@ int *dirs, numdirs;
 /* Put the given task back onto the list of free tasks. */
 
 void 
-free_task(task)
-Task *task;
+free_task(Task *task)
 {
     task->next = freetasks;
     freetasks = task;
 }
 
 void
-add_task(unit, pos, task)
-Unit *unit;
-int pos;
-Task *task;
+add_task(Unit *unit, int pos, Task *task)
 {
     int numcleared;
 
@@ -2018,33 +2035,38 @@ push_build_task(Unit *unit, int u2, int run, int x, int y)
     add_task(unit, 0, create_build_task(unit, u2, run, x, y));
 }
 
+void
+resume_build_task(Unit *unit, Unit *unit2, int run, int x, int y)
+{
+	set_build_task(unit, unit2->type, run, x, y);
+	unit->plan->tasks->args[1] = unit2->id;		
+}
+
 Task *
-create_capture_task(Unit *unit, int x, int y)
+create_capture_task(Unit *unit, int x, int y, int u, int s)
 {
     Task *task = create_task(TASK_CAPTURE);
 
     task->args[0] = x;  task->args[1] = y;
-    task->args[2] = NONUTYPE;
-    task->args[3] = -1;
+    task->args[2] = u;
+    task->args[3] = s;
     return task;
 }
 
 void
-set_capture_task(Unit *unit, int x, int y)
+set_capture_task(Unit *unit, int x, int y, int u, int s)
 {
-    add_task(unit, CLEAR_AGENDA, create_capture_task(unit, x, y));
+    add_task(unit, CLEAR_AGENDA, create_capture_task(unit, x, y, u, s));
 }
 
 void
-push_capture_task(Unit *unit, int x, int y)
+push_capture_task(Unit *unit, int x, int y, int u, int s)
 {
-    add_task(unit, 0, create_capture_task(unit, x, y));
+    add_task(unit, 0, create_capture_task(unit, x, y, u, s));
 }
 
 Task *
-create_collect_task(unit, m, x, y)
-Unit *unit;
-int m, x, y;
+create_collect_task(Unit *unit, int m, int x, int y)
 {
     Task *task = create_task(TASK_COLLECT);
 
@@ -2054,31 +2076,25 @@ int m, x, y;
 }
 
 void
-set_collect_task(unit, m, x, y)
-Unit *unit;
-int m, x, y;
+set_collect_task(Unit *unit, int m, int x, int y)
 {
     add_task(unit, 0, create_collect_task(unit, m, x, y));
 }
 
 Task *
-create_disband_task(unit)
-Unit *unit;
+create_disband_task(Unit *unit)
 {
     return create_task(TASK_DISBAND);
 }
 
 void
-set_disband_task(unit)
-Unit *unit;
+set_disband_task(Unit *unit)
 {
     add_task(unit, CLEAR_AGENDA, create_disband_task(unit));
 }
 
 Task *
-create_hit_unit_task(unit, x, y, u, s)
-Unit *unit;
-int x, y, u, s;
+create_hit_unit_task(Unit *unit, int x, int y, int u, int s)
 {
     Task *task = create_task(TASK_HIT_UNIT);
 
@@ -2089,17 +2105,13 @@ int x, y, u, s;
 }
 
 void
-set_hit_unit_task(unit, x, y, u, s)
-Unit *unit;
-int x, y, u, s;
+set_hit_unit_task(Unit *unit, int x, int y, int u, int s)
 {
     add_task(unit, CLEAR_AGENDA, create_hit_unit_task(unit, x, y, u, s));
 }
 
 void
-push_hit_unit_task(unit, x, y, u, s)
-Unit *unit;
-int x, y, u, s;
+push_hit_unit_task(Unit *unit, int x, int y, int u, int s)
 {
     add_task(unit, 0, create_hit_unit_task(unit, x, y, u, s));
 }
@@ -2107,9 +2119,7 @@ int x, y, u, s;
 /* Create a task to move in a given direction for a given distance. */
 
 Task *
-create_move_dir_task(unit, dir, n)
-Unit *unit;
-int dir, n;
+create_move_dir_task(Unit *unit, int dir, int n)
 {
     Task *task = create_task(TASK_MOVE_DIR);
 
@@ -2121,17 +2131,13 @@ int dir, n;
 /* Fill in the given unit with direction-moving orders. */
 
 void
-set_move_dir_task(unit, dir, n)
-Unit *unit;
-int dir, n;
+set_move_dir_task(Unit *unit, int dir, int n)
 {
     add_task(unit, CLEAR_AGENDA, create_move_dir_task(unit, dir, n));
 }
 
 Task *
-create_move_to_task(unit, x, y, dist)
-Unit *unit;
-int x, y, dist;
+create_move_to_task(Unit *unit, int x, int y, int dist)
 {
     Task *task = create_task(TASK_MOVE_TO);
 
@@ -2149,25 +2155,19 @@ int x, y, dist;
 }
 
 void
-set_move_to_task(unit, x, y, dist)
-Unit *unit;
-int x, y, dist;
+set_move_to_task(Unit *unit, int x, int y, int dist)
 {
     add_task(unit, CLEAR_AGENDA, create_move_to_task(unit, x, y, dist));
 }
 
 void
-push_move_to_task(unit, x, y, dist)
-Unit *unit;
-int x, y, dist;
+push_move_to_task(Unit *unit, int x, int y, int dist)
 {
     add_task(unit, 0, create_move_to_task(unit, x, y, dist));
 }
 
 Task *
-create_occupy_task(unit, transport)
-Unit *unit;
-Unit *transport;
+create_occupy_task(Unit *unit, Unit *transport)
 {
     Task *task = create_task(TASK_OCCUPY);
 
@@ -2178,23 +2178,19 @@ Unit *transport;
 }
 
 void
-set_occupy_task(unit, transport)
-Unit *unit, *transport;
+set_occupy_task(Unit *unit, Unit *transport)
 {
     add_task(unit, CLEAR_AGENDA, create_occupy_task(unit, transport));
 }
 
 void
-push_occupy_task(unit, transport)
-Unit *unit, *transport;
+push_occupy_task(Unit *unit, Unit *transport)
 {
     add_task(unit, 0, create_occupy_task(unit, transport));
 }
 
 Task *
-create_pickup_task(unit, occ)
-Unit *unit;
-Unit *occ;
+create_pickup_task(Unit *unit, Unit *occ)
 {
     Task *task = create_task(TASK_PICKUP);
 
@@ -2204,16 +2200,13 @@ Unit *occ;
 }
 
 void
-push_pickup_task(unit, occ)
-Unit *unit, *occ;
+push_pickup_task(Unit *unit, Unit *occ)
 {
     add_task(unit, 0, create_pickup_task(unit, occ));
 }
 
 Task *
-create_produce_task(unit, m, n)
-Unit *unit;
-int m, n;
+create_produce_task(Unit *unit, int m, int n)
 {
     Task *task = create_task(TASK_PRODUCE);
 
@@ -2224,23 +2217,19 @@ int m, n;
 }
 
 void
-push_produce_task(unit, m, n)
-Unit *unit;
-int m, n;
+push_produce_task(Unit *unit, int m, int n)
 {
     add_task(unit, 0, create_produce_task(unit, m, n));
 }
 
 Task *
-create_repair_task(unit)
-Unit *unit;
+create_repair_task(Unit *unit)
 {
     return create_task(TASK_REPAIR);
 }
 
 void
-set_repair_task(unit)
-Unit *unit;
+set_repair_task(Unit *unit)
 {
     add_task(unit, CLEAR_AGENDA, create_repair_task(unit));
 }
@@ -2248,9 +2237,7 @@ Unit *unit;
 /* This routine sets up a task to develop a unit of the given type. */
 
 Task *
-create_develop_task(unit, u2, n)
-Unit *unit;
-int u2, n;
+create_develop_task(Unit *unit, int u2, int n)
 {
     Task *task = create_task(TASK_DEVELOP);
 
@@ -2260,17 +2247,13 @@ int u2, n;
 }
 
 void
-push_develop_task(unit, u2, n)
-Unit *unit;
-int u2, n;
+push_develop_task(Unit *unit, int u2, int n)
 {
     add_task(unit, 0, create_develop_task(unit, u2, n));
 }
 
 Task *
-create_resupply_task(unit, m)
-Unit *unit;
-int m;
+create_resupply_task(Unit *unit, int m)
 {
     Task *task = create_task(TASK_RESUPPLY);
 
@@ -2279,17 +2262,13 @@ int m;
 }
 
 void
-set_resupply_task(unit, m)
-Unit *unit;
-int m;
+set_resupply_task(Unit *unit, int m)
 {
     add_task(unit, CLEAR_AGENDA, create_resupply_task(unit, m));
 }
 
 Task *
-create_sentry_task(unit, n)
-Unit *unit;
-int n;
+create_sentry_task(Unit *unit, int n)
 {
     Task *task = create_task(TASK_SENTRY);
 
@@ -2298,17 +2277,13 @@ int n;
 }
 
 void
-set_sentry_task(unit, n)
-Unit *unit;
-int n;
+set_sentry_task(Unit *unit, int n)
 {
     add_task(unit, CLEAR_AGENDA, create_sentry_task(unit, n));
 }
 
 void
-push_sentry_task(unit, n)
-Unit *unit;
-int n;
+push_sentry_task(Unit *unit, int n)
 {
     add_task(unit, 0, create_sentry_task(unit, n));
 }
@@ -2320,9 +2295,7 @@ extern Unit *parse_unit(Side *side, char *arg);
 /* Find a unit with the given name, either alive or dead. */
 
 Unit *
-parse_unit(side, nm)
-Side *side;
-char *nm;
+parse_unit(Side *side, char *nm)
 {
     Unit *unit;
 
@@ -2338,8 +2311,7 @@ char *nm;
 	    && unit->side != side
 	    && unit->name != NULL
 	    && strcmp(unit->name, nm) == 0
-	    && (side->see_all
-		|| side_sees_image(side, unit)))
+	    && side_sees_image(side, unit))
 	  return unit;
     }
     return NULL;
@@ -2349,10 +2321,7 @@ char *nm;
    if possible. */
 
 int
-parse_location(side, arg, xp, yp)
-Side *side;
-char *arg;
-int *xp, *yp;
+parse_location(Side *side, char *arg, int *xp, int *yp)
 {
     char *arg2;
     Unit *unit;
@@ -2375,10 +2344,7 @@ int *xp, *yp;
    of the string, if NULL if failure. */
 
 char *
-parse_task(side, str, taskp)
-Side *side;
-char *str;
-Task **taskp;
+parse_task(Side *side, char *str, Task **taskp)
 {
     int tasktype, i, x, y, n, dir, u, taskargs[MAXTASKARGS], numargs;
     char *arg, *arg2, substr[BUFSIZE], *rest, *argtypes;
@@ -2478,23 +2444,10 @@ Task **taskp;
     }
 }
 
-int
-lookup_task_type(name)
-char *name;
-{
-    int i;
-
-    for (i = 0; taskdefns[i].name != NULL; ++i)
-      if (strcmp(name, taskdefns[i].name) == 0)
-	return i; /* should get real enum? */
-    return -1;
-}
-
 /* Describe a task succinctly - use for debugging only. */
 
 char *
-task_desig(task)
-Task *task;
+task_desig(Task *task)
 {
     int i, slen;
     char *argtypes;
