@@ -9,15 +9,8 @@ any later version.  See the file COPYING. */
 /* This is the implementation of the high-level protocol. */
 
 #include "conq.h"
-extern void send_variant_value(int i, int v1, int v2, int v3, int v4, int v5);
-extern void (*update_variant_callback)(int i, int v1, int v2, int v3, int v4, int v5);
-extern void (*update_assignment_callback)(int n);
-extern void add_to_packet(char *str);
-extern void flush_write(void);
-extern void send_game_checksum_error(int rid, int my_csum, int master_csum);
 #include "kernel.h"
 #include "kpublic.h"
-extern int game_checksum(void);
 
 extern long randstate;
 
@@ -54,7 +47,7 @@ static void broadcast_unit_str_property(Side *side, Unit *unit, char *prop,
 					char *val);
 static void broadcast_add_task(Unit *unit, int pos, Task *task);
 static void broadcast_layer_change(char *layername, Side *side, int x, int y,
-				   int a1, int a2, int a3);
+				   int a1, int a2, int a3, int a4);
 static void broadcast_packet(char *buf);
 static void save_outgoing_packet(int id, char *inbuf);
 static void save_incoming_packet(int id, char *inbuf);
@@ -70,7 +63,7 @@ static void receive_unit_prop(char *str);
 static void receive_world_prop(char *str);
 static void receive_run_game(char *str);
 static void receive_game_checksum(char *str);
-static void receive_error(char *str);
+static void receive_error(int id, char *str);
 static void receive_remote_program(char *str);
 static void receive_chat(char *str);
 static void receive_variant_setting(char *str);
@@ -78,6 +71,17 @@ static void receive_assignment_setting(char *str);
 static int tohex(int x);
 static int fromhex(int x);
 static void flush_incoming_queue(void);
+static void send_game_checksum_error(int rid, int my_csum, int master_csum);
+
+static int send_packet(int id, char *buf);
+static void broadcast_game_checksum(void);
+static void send_assignment(int id, Side *side, Player *player);
+static void send_randstate(int id);
+static void broadcast_next_action(Unit *unit);
+static void receive_packet(int id, char *buf);
+static void add_remote_program(int rid, char *name);
+static void download_game_module(int rid);
+static void send_remote_id(int rid);
 
 /* This is true if the program expects to host the game and also be the
    master when all the remote programs are hooked up. */
@@ -122,7 +126,7 @@ int dlbufend;
 
 long new_randstate;
 
-void (*update_variant_callback)(int i, int v1, int v2, int v3, int v4, int v5) = NULL;
+void (*update_variant_callback)(int which) = NULL;
 
 void (*update_assignment_callback)(int n) = NULL;
 
@@ -177,7 +181,17 @@ add_remote_players(void)
     Player *player;
 
     for_all_remotes(rid) {
-	player = add_player();
+	/* First look for an existing player object that's waiting to
+	   be associated with a remote program. */
+	player = NULL;
+	for_all_players(player) {
+	    if (!empty_string(player->name)
+		&& strcmp(player->name, "?") == 0)
+	      break;
+	}
+	/* Otherwise create a new player. */
+	if (player == NULL)
+	  player = add_player();
 	parse_player_spec(player, remote_player_specs[rid]);
 	canonicalize_player(player);
 	player->rid = rid;
@@ -236,11 +250,13 @@ send_version(int rid)
 }
 
 void
-send_variant_value(int i, int v1, int v2, int v3, int v4, int v5)
+net_set_variant_value(int which, int v1, int v2, int v3)
 {
-    /* Local master change done already. */
+    if (my_rid == master_rid) {
+	set_variant_value(which, v1, v2, v3);
+    }
     if (numremotes > 0) {
-	sprintf(spbuf, "v %d %d %d %d %d %d", i, v1, v2, v3, v4, v5);
+	sprintf(spbuf, "v %d %d %d %d", which, v1, v2, v3);
 	broadcast_packet(spbuf);
     }
 }
@@ -273,13 +289,13 @@ net_add_side_and_player(void)
 }
 
 void
-net_rename_side_for_player(int n)
+net_rename_side_for_player(int n, int which)
 {
     if (my_rid == master_rid) {
-	rename_side_for_player(n);
+	rename_side_for_player(n, which);
     }
     if (numremotes > 0) {
-	sprintf(spbuf, "a%d rename", n);
+	sprintf(spbuf, "a%d rename %d", n, which);
 	broadcast_packet(spbuf);
     }
 }
@@ -318,6 +334,7 @@ send_assignment(int rid, Side *side, Player *player)
     send_packet(rid, spbuf);
 }
 
+/* (this routine is almost certainly incorrect now - only used on Macs) */
 void
 download_to_player(Player *player)
 {
@@ -334,6 +351,13 @@ download_to_player(Player *player)
     /* If errors, should set player rid back to zero */
 }
 
+void
+net_update_player(Player *player)
+{
+    sprintf(spbuf, "a%d update %s", player->id, player_desig(player));
+    broadcast_packet(spbuf);
+}
+
 /* Given the id of a remote program that has just connected,
    download our own current state, as a module. */
 
@@ -343,6 +367,12 @@ download_game_module(int rid)
     int rslt;
     Module *module;
 
+    if (1 /* sending name only */) {
+	sprintf(spbuf, "g %s", mainmodule->name);
+	send_packet(rid, spbuf);
+	/* (should send some sort of state checksum also) */
+	return;
+    }
     /* The indepside might not have been filled in yet (such as when
        connecting before sides/players have been set up), but we need
        it to be correct before going into write_game_module. */
@@ -407,6 +437,8 @@ flush_write(void)
     }
 }
 
+/* Make sure every remote program has the same game that we do. */
+
 void
 broadcast_game_module(void)
 {
@@ -419,6 +451,13 @@ broadcast_game_module(void)
     }
 }
 
+void
+broadcast_start_variant_setup(void)
+{
+    sprintf(spbuf, "vSTART");
+    broadcast_packet(spbuf);
+}
+
 void
 broadcast_variants_chosen(void)
 {
@@ -431,6 +470,13 @@ broadcast_variants_chosen(void)
     broadcast_packet(spbuf);
 }
 
+void
+broadcast_start_player_setup(void)
+{
+    sprintf(spbuf, "aSTART");
+    broadcast_packet(spbuf);
+}
+
 void
 broadcast_players_assigned(void)
 {
@@ -445,13 +491,6 @@ send_randstate(int rid)
     send_packet(rid, spbuf); 
 }
 
-void
-send_game_checksum(int rid)
-{
-    sprintf(spbuf, "Z%d %d", my_rid, game_checksum());
-    send_packet(rid, spbuf); 
-}
-
 void
 broadcast_game_checksum(void)
 {
@@ -502,7 +541,7 @@ net_run_game(int maxactions)
 	    } else {
 		newcsum = game_checksum();
 		if (newcsum != oldcsum)
-		  run_warning("Checksum changed %d -> %d when numdone == 0",
+		  notify_all("Checksum changed %d -> %d when numdone == 0",
 			      oldcsum, newcsum);
 	    }
 	}
@@ -665,17 +704,6 @@ net_set_trust(Side *side, Side *side2, int val)
     }
 }
 
-void
-net_set_mutual_trust(Side *side, Side *side2, int val)
-{
-    if (my_rid == master_rid) {
-	set_mutual_trust(side, side2, val);
-    }
-    if (numremotes > 0)  {
-	broadcast_side_property_2(side, "mutualtrust", side_number(side2), val);
-    }
-}
-
 void
 net_set_controlled_by(Side *side, Side *side2, int val)
 {
@@ -945,6 +973,28 @@ net_set_unit_autobuild(Side *side, Unit *unit, int flag)
     }
 }
 
+void
+net_set_unit_buildingdone(Side *side, Unit *unit, int flag)
+{
+    if (my_rid == master_rid) {
+	set_unit_buildingdone(side, unit, flag);
+    }
+    if (numremotes > 0) {
+	broadcast_unit_property(side, unit, "buildingdone", flag);
+    }
+}
+
+void
+net_set_unit_researchdone(Side *side, Unit *unit, int flag)
+{
+    if (my_rid == master_rid) {
+	set_unit_researchdone(side, unit, flag);
+    }
+    if (numremotes > 0) {
+	broadcast_unit_property(side, unit, "researchdone", flag);
+    }
+}
+
 void
 net_set_unit_waiting_for_transport(Side *side, Unit *unit, int flag)
 {
@@ -1238,6 +1288,270 @@ broadcast_next_action(Unit *unit)
     }
 }
 
+/* From actions.c. */
+
+int
+net_prep_none_action(Unit *unit, Unit *unit2)
+{
+    int rslt;
+
+    rslt = prep_none_action(unit, unit2);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+int
+net_prep_produce_action(unit, unit2, m, n)
+Unit *unit, *unit2;
+int m, n;
+{
+    int rslt;
+
+    rslt = prep_produce_action(unit, unit2, m, n);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+int
+net_prep_extract_action(unit, unit2, x, y, m, n)
+Unit *unit, *unit2;
+int x, y, m, n;
+{
+    int rslt;
+
+    rslt = prep_extract_action(unit, unit2, x, y, m, n);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+int
+net_prep_develop_action(Unit *unit, Unit *unit2, int u3)
+{
+    int rslt;
+
+    rslt = prep_develop_action(unit, unit2, u3);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+int
+net_prep_toolup_action(Unit *unit, Unit *unit2, int u3)
+{
+    int rslt;
+
+    rslt = prep_toolup_action(unit, unit2, u3);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+int
+net_prep_create_in_action(Unit *unit, Unit *unit2, int u3, Unit *dest)
+{
+    int rslt;
+
+    rslt = prep_create_in_action(unit, unit2, u3, dest);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+int
+net_prep_create_at_action(Unit *unit, Unit *unit2, int u3, int x, int y, int z)
+{
+    int rslt;
+
+    rslt = prep_create_at_action(unit, unit2, u3, x, y, z);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+int
+net_prep_build_action(Unit *unit, Unit *unit2, Unit *newunit)
+{
+    int rslt;
+
+    rslt = prep_build_action(unit, unit2, newunit);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+int
+net_prep_repair_action(unit, unit2, unit3)
+Unit *unit, *unit2, *unit3;
+{
+    int rslt;
+
+    rslt = prep_repair_action(unit, unit2, unit3);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+int
+net_prep_disband_action(unit, unit2)
+Unit *unit, *unit2;
+{
+    int rslt;
+
+    rslt = prep_disband_action(unit, unit2);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+int
+net_prep_transfer_part_action(unit, unit2, parts, unit3)
+Unit *unit, *unit2, *unit3;
+int parts;
+{
+    int rslt;
+
+    rslt = prep_transfer_part_action(unit, unit2, parts, unit3);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+int
+net_prep_change_type_action(unit, unit2, u3)
+Unit *unit, *unit2;
+int u3;
+{
+    int rslt;
+
+    rslt = prep_change_type_action(unit, unit2, u3);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+int
+net_prep_change_side_action(unit, unit2, side)
+Unit *unit, *unit2;
+Side *side;
+{
+    int rslt;
+
+    rslt = prep_change_side_action(unit, unit2, side);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+int
+net_prep_alter_cell_action(unit, unit2, x, y, t)
+Unit *unit, *unit2;
+int x, y, t;
+{
+    int rslt;
+
+    rslt = prep_alter_cell_action(unit, unit2, x, y, t);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+int
+net_prep_add_terrain_action(unit, unit2, x, y, dir, t)
+Unit *unit, *unit2;
+int x, y, dir, t;
+{
+    int rslt;
+
+    rslt = prep_add_terrain_action(unit, unit2, x, y, dir, t);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+int
+net_prep_remove_terrain_action(unit, unit2, x, y, dir, t)
+Unit *unit, *unit2;
+int x, y, dir, t;
+{
+    int rslt;
+
+    rslt = prep_remove_terrain_action(unit, unit2, x, y, dir, t);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+/* From combat.c. */
+
+int
+net_prep_attack_action(Unit *unit, Unit *unit2, Unit *defender, int n)
+{
+    int rslt;
+
+    rslt = prep_attack_action(unit, unit2, defender, n);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+int
+net_prep_overrun_action(Unit *unit, Unit *unit2, int x, int y, int z, int n)
+{
+    int rslt;
+
+    rslt = prep_overrun_action(unit, unit2, x, y, z, n);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+int
+net_prep_fire_at_action(Unit *unit, Unit *unit2, Unit *unit3, int m)
+{
+    int rslt;
+
+    rslt = prep_fire_at_action(unit, unit2, unit3, m);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+int
+net_prep_fire_into_action(Unit *unit, Unit *unit2, int x, int y, int z, int m)
+{
+    int rslt;
+
+    rslt = prep_fire_into_action(unit, unit2, x, y, z, m);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+int
+net_prep_capture_action(Unit *unit, Unit *unit2, Unit *unit3)
+{
+    int rslt;
+
+    rslt = prep_capture_action(unit, unit2, unit3);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+int
+net_prep_detonate_action(Unit *unit, Unit *unit2, int x, int y, int z)
+{
+    int rslt;
+
+    rslt = prep_detonate_action(unit, unit2, x, y, z);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+/* From move.c. */
+
+int
+net_prep_move_action(Unit *unit, Unit *unit2, int x, int y, int z)
+{
+    int rslt;
+
+    rslt = prep_move_action(unit, unit2, x, y, z);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
+int
+net_prep_enter_action(Unit *unit, Unit *unit2, Unit *unit3)
+{
+    int rslt;
+
+    rslt = prep_enter_action(unit, unit2, unit3);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
 /* Task networking. */
 
 #define CLEAR_AGENDA 99
@@ -1260,13 +1574,13 @@ Task *task;
 }
 
 void
-net_set_capture_task(Unit *unit, int x, int y)
+net_set_capture_task(Unit *unit, int x, int y, int u, int s)
 {
     if (my_rid == master_rid) {
-	set_capture_task(unit, x, y);
+	set_capture_task(unit, x, y, u, s);
     }
     if (numremotes > 0) {
-	tmptask = create_capture_task(unit, x, y);
+	tmptask = create_capture_task(unit, x, y, u, s);
 	broadcast_add_task(unit, CLEAR_AGENDA, tmptask);
 	free_task(tmptask);
     }
@@ -1286,9 +1600,7 @@ net_set_occupy_task(Unit *unit, Unit *transport)
 }
 
 void
-net_set_collect_task(unit, m, x, y)
-Unit *unit;
-int m, x, y;
+net_set_collect_task(Unit *unit, int m, int x, int y)
 {
     if (my_rid == master_rid) {
 	set_collect_task(unit, m, x, y);
@@ -1301,9 +1613,7 @@ int m, x, y;
 }
 
 void
-net_set_move_to_task(unit, x, y, dist)
-Unit *unit;
-int x, y, dist;
+net_set_move_to_task(Unit *unit, int x, int y, int dist)
 {
     if (my_rid == master_rid) {
 	set_move_to_task(unit, x, y, dist);
@@ -1316,9 +1626,7 @@ int x, y, dist;
 }
 
 void
-net_push_move_to_task(unit, x, y, dist)
-Unit *unit;
-int x, y, dist;
+net_push_move_to_task(Unit *unit, int x, int y, int dist)
 {
     if (my_rid == master_rid) {
 	push_move_to_task(unit, x, y, dist);
@@ -1331,9 +1639,7 @@ int x, y, dist;
 }
 
 void
-net_set_move_dir_task(unit, dir, n)
-Unit *unit;
-int dir, n;
+net_set_move_dir_task(Unit *unit, int dir, int n)
 {
     if (my_rid == master_rid) {
 	set_move_dir_task(unit, dir, n);
@@ -1365,8 +1671,7 @@ void
 net_resume_build_task(Unit *unit, Unit *unit2, int run, int x, int y)
 {
     if (my_rid == master_rid) {
-	set_build_task(unit, unit2->type, run, x, y);
-	unit->plan->tasks->args[1] = unit2->id;		
+	resume_build_task(unit, unit2, run, x, y);
     }
     if (numremotes > 0) {
 	tmptask = create_build_task(unit, unit2->type, run, x, y);
@@ -1403,9 +1708,7 @@ net_push_develop_task(Unit *unit, int u2, int n)
 }
 
 void
-net_push_hit_unit_task(unit, x, y, u, s)
-Unit *unit;
-int x, y, u, s;
+net_push_hit_unit_task(Unit *unit, int x, int y, int u, int s)
 {
     if (my_rid == master_rid) {
 	push_hit_unit_task(unit, x, y, u, s);
@@ -1418,9 +1721,7 @@ int x, y, u, s;
 }
 
 void
-net_set_hit_unit_task(unit, x, y, u, s)
-Unit *unit;
-int x, y, u, s;
+net_set_hit_unit_task(Unit *unit, int x, int y, int u, int s)
 {
     if (my_rid == master_rid) {
 	set_hit_unit_task(unit, x, y, u, s);
@@ -1433,8 +1734,7 @@ int x, y, u, s;
 }
 
 void
-net_set_disband_task(unit)
-Unit *unit;
+net_set_disband_task(Unit *unit)
 {
     if (my_rid == master_rid) {
 	set_disband_task(unit);
@@ -1447,9 +1747,7 @@ Unit *unit;
 }
 
 void
-net_set_resupply_task(unit, m)
-Unit *unit;
-int m;
+net_set_resupply_task(Unit *unit, int m)
 {
     if (my_rid == master_rid) {
 	set_resupply_task(unit, m);
@@ -1462,22 +1760,20 @@ int m;
 }
 
 void
-net_push_occupy_task(unit, transp)
-Unit *unit, *transp;
+net_push_occupy_task(Unit *unit, Unit *transport)
 {
     if (my_rid == master_rid) {
-	push_occupy_task(unit, transp);
+	push_occupy_task(unit, transport);
     }
     if (numremotes > 0) {
-	tmptask = create_occupy_task(unit, transp);
+	tmptask = create_occupy_task(unit, transport);
 	broadcast_add_task(unit, 0, tmptask);
 	free_task(tmptask);
     }
 }
 
 void
-net_push_pickup_task(unit, occ)
-Unit *unit, *occ;
+net_push_pickup_task(Unit *unit, Unit *occ)
 {
     if (my_rid == master_rid) {
 	push_pickup_task(unit, occ);
@@ -1490,9 +1786,7 @@ Unit *unit, *occ;
 }
 
 void
-net_push_produce_task(unit, m, n)
-Unit *unit;
-int m, n;
+net_push_produce_task(Unit *unit, int m, int n)
 {
     if (my_rid == master_rid) {
 	push_produce_task(unit, m, n);
@@ -1517,6 +1811,19 @@ net_set_sentry_task(Unit *unit, int n)
     }
 }
 
+void
+net_push_sentry_task(Unit *unit, int n)
+{
+    if (my_rid == master_rid) {
+	push_sentry_task(unit, n);
+    }
+    if (numremotes > 0) {
+	tmptask = create_sentry_task(unit, n);
+	broadcast_add_task(unit, 0, tmptask);
+	free_task(tmptask);
+    }
+}
+
 static void
 broadcast_add_task(Unit *unit, int pos, Task *task)
 {
@@ -1537,171 +1844,149 @@ broadcast_add_task(Unit *unit, int pos, Task *task)
 #ifdef DESIGNERS
 
 void
-net_paint_cell(side, x, y, r, t)
-Side *side;
-int x, y, r, t;
+net_paint_cell(Side *side, int x, int y, int r, int t)
 {
     if (my_rid == master_rid) {
 	paint_cell(side, x, y, r, t);
     }
     if (numremotes > 0) {
-	broadcast_layer_change("cell", side, x, y, r, t, 0);
+	broadcast_layer_change("cell", side, x, y, r, t, 0, 0);
     }
 }
 
 void
-net_paint_border(side, x, y, dir, t, mode)
-Side *side;
-int x, y, dir, t, mode;
+net_paint_border(Side *side, int x, int y, int dir, int t, int mode)
 {
     if (my_rid == master_rid) {
 	paint_border(side, x, y, dir, t, mode);
     }
     if (numremotes > 0) {
-	broadcast_layer_change("bord", side, x, y, dir, t, mode);
+	broadcast_layer_change("bord", side, x, y, dir, t, mode, 0);
     }
 }
 
 void
-net_paint_connection(side, x, y, dir, t, mode)
-Side *side;
-int x, y, dir, t, mode;
+net_paint_connection(Side *side, int x, int y, int dir, int t, int mode)
 {
     if (my_rid == master_rid) {
 	paint_connection(side, x, y, dir, t, mode);
     }
     if (numremotes > 0) {
-	broadcast_layer_change("conn", side, x, y, dir, t, mode);
+	broadcast_layer_change("conn", side, x, y, dir, t, mode, 0);
     }
 }
 
 void
-net_paint_coating(side, x, y, r, t, depth)
-Side *side;
-int x, y, r, t, depth;
+net_paint_coating(Side *side, int x, int y, int r, int t, int depth)
 {
     if (my_rid == master_rid) {
 	paint_coating(side, x, y, r, t, depth);
     }
     if (numremotes > 0) {
-	broadcast_layer_change("coat", side, x, y, r, t, depth);
+	broadcast_layer_change("coat", side, x, y, r, t, depth, 0);
     }
 }
 
 void
-net_paint_people(side, x, y, r, s)
-Side *side;
-int x, y, r, s;
+net_paint_people(Side *side, int x, int y, int r, int s)
 {
     if (my_rid == master_rid) {
 	paint_people(side, x, y, r, s);
     }
     if (numremotes > 0) {
-	broadcast_layer_change("peop", side, x, y, r, s, 0);
+	broadcast_layer_change("peop", side, x, y, r, s, 0, 0);
     }
 }
 
 void
-net_paint_control(side, x, y, r, s)
-Side *side;
-int x, y, r, s;
+net_paint_control(Side *side, int x, int y, int r, int s)
 {
     if (my_rid == master_rid) {
 	paint_control(side, x, y, r, s);
     }
     if (numremotes > 0) {
-	broadcast_layer_change("ctrl", side, x, y, r, s, 0);
+	broadcast_layer_change("ctrl", side, x, y, r, s, 0, 0);
     }
 }
 
 void
-net_paint_feature(side, x, y, r, f)
-Side *side;
-int x, y, r, f;
+net_paint_feature(Side *side, int x, int y, int r, int f)
 {
     if (my_rid == master_rid) {
 	paint_feature(side, x, y, r, f);
     }
     if (numremotes > 0) {
-	broadcast_layer_change("feat", side, x, y, r, f, 0);
+	broadcast_layer_change("feat", side, x, y, r, f, 0, 0);
     }
 }
 
 void
-net_paint_elevation(side, x, y, r, elev)
-Side *side;
-int x, y, r, elev;
+net_paint_elevation(Side *side, int x, int y, int r, int code, int elev,
+		    int vary)
 {
     if (my_rid == master_rid) {
-	paint_elevation(side, x, y, r, elev);
+	paint_elevation(side, x, y, r, code, elev, vary);
     }
     if (numremotes > 0) {
-	broadcast_layer_change("elev", side, x, y, r, elev, 0);
+	broadcast_layer_change("elev", side, x, y, r, code, elev, vary);
     }
 }
 
 void
-net_paint_temperature(side, x, y, r, temp)
-Side *side;
-int x, y, r, temp;
+net_paint_temperature(Side *side, int x, int y, int r, int temp)
 {
     if (my_rid == master_rid) {
 	paint_temperature(side, x, y, r, temp);
     }
     if (numremotes > 0) {
-	broadcast_layer_change("temp", side, x, y, r, temp, 0);
+	broadcast_layer_change("temp", side, x, y, r, temp, 0, 0);
     }
 }
 
 void
-net_paint_material(side, x, y, r, m, amt)
-Side *side;
-int x, y, r, m, amt;
+net_paint_material(Side *side, int x, int y, int r, int m, int amt)
 {
     if (my_rid == master_rid) {
 	paint_material(side, x, y, r, m, amt);
     }
     if (numremotes > 0) {
-	broadcast_layer_change("m", side, x, y, r, m, amt);
+	broadcast_layer_change("m", side, x, y, r, m, amt, 0);
     }
 }
 
 void
-net_paint_clouds(side, x, y, r, cloudtype, bot, hgt)
-Side *side;
-int x, y, r, cloudtype, bot, hgt;
+net_paint_clouds(Side *side, int x, int y, int r, int cloudtype, int bot,
+		 int hgt)
 {
     if (my_rid == master_rid) {
 	paint_clouds(side, x, y, r, cloudtype, bot, hgt);
     }
     if (numremotes > 0) {
-	broadcast_layer_change("clouds", side, x, y, r, cloudtype, 0);
-	broadcast_layer_change("cloud-bottoms", side, x, y, r, bot, 0);
-	broadcast_layer_change("cloud-heights", side, x, y, r, hgt, 0);
+	broadcast_layer_change("clouds", side, x, y, r, cloudtype, 0, 0);
+	broadcast_layer_change("cloud-bottoms", side, x, y, r, bot, 0, 0);
+	broadcast_layer_change("cloud-heights", side, x, y, r, hgt, 0, 0);
     }
 }
 
 void
-net_paint_winds(side, x, y, r, dir, force)
-Side *side;
-int x, y, r, dir, force;
+net_paint_winds(Side *side, int x, int y, int r, int dir, int force)
 {
     if (my_rid == master_rid) {
 	paint_winds(side, x, y, r, dir, force);
     }
     if (numremotes > 0) {
-	broadcast_layer_change("wind", side, x, y, r, dir, force);
+	broadcast_layer_change("wind", side, x, y, r, dir, force, 0);
     }
 }
 
 static void
 broadcast_layer_change(char *layername, Side *side, int x, int y,
-		       int a1, int a2, int a3)
+		       int a1, int a2, int a3, int a4)
 {
     char buf[BUFSIZE];
 
-    sprintf(buf, "W%s %d %d %d %d %d %d",
-	    layername, side->id, x, y, a1, a2, a3);
+    sprintf(buf, "W%s %d %d %d %d %d %d %d",
+	    layername, side->id, x, y, a1, a2, a3, a4);
     broadcast_packet(buf);
 }
 
@@ -1727,7 +2012,7 @@ broadcast_packet(char *buf)
     extern int in_run_game;
  
     if (in_run_game) {
-	run_warning("Attempting to send \"%s\" while in run_game", buf);
+	notify_all("Attempting to send \"%s\" while in run_game", buf);
     }
     if (my_rid == master_rid) {
 	for_all_remotes(rid) {
@@ -1741,8 +2026,8 @@ broadcast_packet(char *buf)
     }
 }
 
-/* Given a packet to be sent, package it up (add header, embed escape chars,
-   add checksum), send, and maybe wait for an acknowledgement. */
+/* Given a packet to be sent, package it up (add header, embed escape
+   chars, add checksum), send, and maybe wait for acknowledgement. */
 
 int
 send_packet(int id, char *inbuf)
@@ -1791,7 +2076,9 @@ send_packet(int id, char *inbuf)
 	if (!expecting_ack)
 	  break;
 	if (timeout_warnings)
-	  run_warning("Timed out waiting for ack");
+	  notify_all("Timed out waiting for ack");
+	else
+	  notify_all("Timed out waiting for ack");
 	++numtimeouts;
 	if (hosting && my_rid == master_rid) {
 	    /* If we're the master, we MUST be assured that our
@@ -1885,6 +2172,8 @@ static int rsltid;
 
 #define PACKETBUFSIZE 1000
 
+static time_t nothing_start_time;
+
 static int nothing_count;
 
 static int nothing_timeout;
@@ -1898,6 +2187,8 @@ remove_chars(char *buf, int n)
       buf[i] = buf[i + n];
 }
 
+/* Handle the basic process of receiving packets. */
+
 void
 receive_data(int timeout)
 {
@@ -2001,7 +2292,13 @@ receive_data(int timeout)
 		}
 		Dprintf("Rcvd: %d \"%s\"\n", rsltid, (buf ? buf : "<null>"));
 	    } else {
-		if (nothing_count >= 1000 || timeout != nothing_timeout) {
+		time_t now;
+
+		if (nothing_count == 0)
+		  time(&nothing_start_time);
+		time(&now);
+		if (idifftime(now, nothing_start_time) > 0
+		    || timeout != nothing_timeout) {
 		    Dprintf("Rcvd nothing %d times (timeout %d secs)\n",
 			    nothing_count, nothing_timeout);
 		    nothing_count = 0;
@@ -2026,6 +2323,8 @@ receive_data(int timeout)
     }
 }
 
+/* Given a received packet, take it apart and do what it says. */
+
 void
 receive_packet(int id, char *buf)
 {
@@ -2045,7 +2344,7 @@ receive_packet(int id, char *buf)
 	    /* Make all the cross-references right. */
 	    patch_object_references();
 	    check_game_validity();
-	    current_stage = variant_setup_stage;
+	    start_variant_setup_stage();
 	} else {
 	    /* Make sure we have space with which to concatenate. */
 	    if (strlen(downloadbuf) + strlen(buf) >= 200000) {
@@ -2065,7 +2364,7 @@ receive_packet(int id, char *buf)
 	    receive_command(buf + 1);
 	    break;
 	  case 'E':
-	    receive_error(buf + 1);
+	    receive_error(id, buf + 1);
 	    should_rebroadcast = FALSE;
 	    break;
 	  case 'M':
@@ -2119,8 +2418,13 @@ receive_packet(int id, char *buf)
 	    should_rebroadcast = FALSE;
 	    break;
 	  case 'a':
-	    if (buf[1] == 'O' && buf[2] == 'K') {
-		current_stage = game_ready_stage;
+	    if (buf[1] == 'S' && buf[2] == 'T') {
+		start_player_setup_stage();
+		/* This is only issued by the master, no need to
+                   rebroadcast. */
+		should_rebroadcast = FALSE;
+	    } else if (buf[1] == 'O' && buf[2] == 'K') {
+		start_game_ready_stage();
 		/* This is only issued by the master, no need to
                    rebroadcast. */
 		should_rebroadcast = FALSE;
@@ -2132,11 +2436,17 @@ receive_packet(int id, char *buf)
 	    receive_chat(buf + 1);
 	    break;
 	  case 'g':
-	    downloading = TRUE;
-	    /* (should use obstack here) */
-	    if (downloadbuf == NULL)
-	      downloadbuf = xmalloc(200000);
-	    downloadbuf[0] = '\0';
+	    if (buf[1] == ' ') {
+		Module *module = get_game_module(buf + 2);
+		mainmodule = module;
+		load_game_module(mainmodule, TRUE);
+	    } else {
+		downloading = TRUE;
+		/* (should use obstack here) */
+		if (downloadbuf == NULL)
+		  downloadbuf = xmalloc(200000);
+		downloadbuf[0] = '\0';
+	    }
 	    should_rebroadcast = FALSE;
 	    break;
 	  case 'j':
@@ -2147,7 +2457,7 @@ receive_packet(int id, char *buf)
 	    break;
 	  case 'p':
 	    receive_remote_program(buf + 1);
-	    current_stage = game_load_stage;
+	    start_game_load_stage();  /* seems wrong? */
 	    /* This is only issued by the master, no need to rebroadcast. */
 	    should_rebroadcast = FALSE;
 	    break;
@@ -2157,8 +2467,13 @@ receive_packet(int id, char *buf)
 	    should_rebroadcast = FALSE;
 	    break;
 	  case 'v':
-	    if (buf[1] == 'O' && buf[2] == 'K') {
-		current_stage = player_setup_stage;
+	    if (buf[1] == 'S' && buf[2] == 'T') {
+		start_variant_setup_stage();
+		/* This is only issued by the master, no need to
+                   rebroadcast. */
+		should_rebroadcast = FALSE;
+	    } else if (buf[1] == 'O' && buf[2] == 'K') {
+		start_player_pre_setup_stage();
 		/* This is only issued by the master, no need to
                    rebroadcast. */
 		should_rebroadcast = FALSE;
@@ -2433,12 +2748,6 @@ receive_side_prop(char *str)
 	finish_turn(side);
     } else if (strncmp(str, "longname ", 9) == 0) {
 	set_side_longname(side, side, str + 9);
-    } else if (strncmp(str, "mutualtrust ", 12) == 0) {
-	str += 12;
-	sn = strtol(str, &nstr, 10);
-	str = nstr;
-	val = strtol(str, &nstr, 10);
-	set_mutual_trust(side, side_n(sn), val);
     } else if (strncmp(str, "name ", 5) == 0) {
 	set_side_name(side, side, str + 5);
     } else if (strncmp(str, "noun ", 5) == 0) {
@@ -2545,6 +2854,12 @@ receive_unit_prop(char *str)
     } else if (strncmp(str, "autobuild ", 10) == 0) {
 	val = strtol(str + 10, &nstr, 10);
 	set_unit_autobuild(side, unit, val);
+    } else if (strncmp(str, "buildingdone ", 13) == 0) {
+	val = strtol(str + 13, &nstr, 10);
+	set_unit_autobuild(side, unit, val);
+    } else if (strncmp(str, "researchdone ", 13) == 0) {
+	val = strtol(str + 13, &nstr, 10);
+	set_unit_autobuild(side, unit, val);
     } else if (strncmp(str, "name ", 5) == 0) {
 	set_unit_name(side, unit, str + 5);
     } else if (strncmp(str, "plan ", 5) == 0) {
@@ -2574,7 +2889,7 @@ receive_unit_prop(char *str)
 static void
 receive_world_prop(char *str)
 {
-    int id, x, y, a1, a2, a3;
+    int id, x, y, a1, a2, a3, a4;
     char *str2, *nstr;
     Side *side;
 
@@ -2595,6 +2910,8 @@ receive_world_prop(char *str)
     a2 = strtol(str2, &nstr, 10);
     str2 = nstr;
     a3 = strtol(str2, &nstr, 10);
+    str2 = nstr;
+    a4 = strtol(str2, &nstr, 10);
     side = side_n(id);
     if (strcmp(str, "cell") == 0) {
 	paint_cell(side, x, y, a1, a2);
@@ -2611,7 +2928,7 @@ receive_world_prop(char *str)
     } else if (strcmp(str, "feat") == 0) {
 	paint_feature(side, x, y, a1, a2);
     } else if (strcmp(str, "elev") == 0) {
-	paint_elevation(side, x, y, a1, a2);
+	paint_elevation(side, x, y, a1, a2, a3, a4);
     } else if (strcmp(str, "temp") == 0) {
 	paint_temperature(side, x, y, a1, a2);
     } else if (strcmp(str, "m") == 0) {
@@ -2661,7 +2978,9 @@ receive_game_checksum(char *str)
 	/* If we're about to lose, make sure the master knows about it. */
 	if (my_rid != master_rid)
 	  send_game_checksum_error(master_rid, our_csum, other_csum);
-	run_warning("Game is out of sync! (received %d from %d, computed %d%s)",
+	/* This would be a warning normally, but being out of sync
+	   isn't always a real problem apparently... */
+	notify_all("Game is out of sync! (received %d from %d, computed %d%s)",
 		    other_csum, other_rid, our_csum,
 		    (before_run ? ", before run_game" : ""));
     }
@@ -2681,7 +3000,7 @@ game_checksum(void)
     Task *task;
 
     csum = 0;
-    for_all_sides_plus_indep(side) {
+    for_all_sides(side) {
 	add_int_to_checksum(csum, side->id);
 	add_int_to_checksum(csum, side->self_unit_id);
 	add_int_to_checksum(csum, side->controlled_by_id);
@@ -2724,9 +3043,11 @@ game_checksum(void)
 }
 
 static void
-receive_error(char *str)
+receive_error(int id, char *str)
 {
-    Dprintf("Error reported, %s\n", str);
+    /* (should decode specific types of errors?) */
+    notify_all("Error from #%d, \"%s\"", id, str);
+    Dprintf("Error from #%d, \"%s\"\n", id, str);
 }
 
 static void
@@ -2759,23 +3080,20 @@ receive_chat(char *str)
 static void
 receive_variant_setting(char *str)
 {
-    int i, v1, v2, v3, v4, v5;
+    int which, v1, v2, v3;
     char *nstr;
 
     ++str;
-    i = strtol(str, &nstr, 10);
+    which = strtol(str, &nstr, 10);
     str = nstr + 1;
     v1 = strtol(str, &nstr, 10);
     str = nstr + 1;
     v2 = strtol(str, &nstr, 10);
     str = nstr + 1;
     v3 = strtol(str, &nstr, 10);
-    str = nstr + 1;
-    v4 = strtol(str, &nstr, 10);
-    str = nstr + 1;
-    v5 = strtol(str, &nstr, 10);
+    set_variant_value(which, v1, v2, v3);
     if (update_variant_callback)
-      (*update_variant_callback)(i, v1, v2, v3, v4, v5);
+      (*update_variant_callback)(which);
 }
 
 static void
@@ -2797,9 +3115,8 @@ receive_assignment_setting(char *str)
 	    player->advantage = val;
 	}
     } else if (strncmp(str, "ai ", 3) == 0) {
-	if (str[3] == '\0')
-	  aitype = NULL;
-	else
+	aitype = NULL;
+	if (str[3] != '\0')
 	  aitype = copy_string(str + 3);
 	set_ai_for_player(n, aitype);
     } else if (strncmp(str, "exchange ", 9) == 0) {
@@ -2807,10 +3124,18 @@ receive_assignment_setting(char *str)
 	n2 = exchange_players(n, n2);
 	if (update_assignment_callback)
 	  (*update_assignment_callback)(n2);
-    } else if (strncmp(str, "rename", 6) == 0) {
-	rename_side_for_player(n);
+    } else if (strncmp(str, "rename ", 7) == 0) {
+	n2 = strtol(str + 7, NULL, 10);
+	rename_side_for_player(n, n2);
+    } else if (strncmp(str, "update ", 7) == 0) {
+	nstr = str + 7;
+	player = find_player(n);
+	if (player == NULL)
+	  player = add_player();
+	parse_player_spec(player, nstr);
+	return; /* a hack, need to handle callback below when windows not up */
     } else {
-	/* (should complain) */
+	run_error("assignment packet \"%s\" not recognized", str);
     }
     if (n >= 0) {
 	/* Most settings affect the same assignment, so always update it
