@@ -1,5 +1,5 @@
 /* Data structures defining the properties of types.
-   Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
+   Copyright (C) 1991-1996, 1998, 1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -69,6 +69,25 @@ PropertyDefn ttypedefns[] = {
     { NULL }
 };
 
+PropertyDefn atypedefns[] = {
+
+#undef  DEF_APROP_I
+#define DEF_APROP_I(NAME,FNAME,DOC,SLOT,LO,DFLT,HI)  \
+    { NAME, FNAME, NULL, NULL, OFFS(Atype, SLOT), DOC, DFLT, NULL, LO, HI },
+#undef  DEF_APROP_S
+#define DEF_APROP_S(NAME,FNAME,DOC,SLOT,DFLT)  \
+    { NAME, NULL, FNAME, NULL, OFFS(Atype, SLOT), DOC,    0, DFLT,  0,  0 },
+#undef  DEF_APROP_L
+#define DEF_APROP_L(NAME,FNAME,DOC,SLOT)  \
+    { NAME, NULL, NULL, FNAME, OFFS(Atype, SLOT), DOC,    0, NULL,  0,  0 },
+
+#include "atype.def"
+
+    { NULL }
+};
+
+/* Define functions for all unit type properties. */
+
 #undef  DEF_UPROP_I
 #define DEF_UPROP_I(name,FNAME,doc,SLOT,lo,dflt,hi)  \
   int FNAME(u) int u; { checku(u); return utypes[u].SLOT; }
@@ -109,71 +128,16 @@ PropertyDefn ttypedefns[] = {
 
 #include "ttype.def"
 
-/* Acquire a unit type extension property by name. */
+/* Define functions for all advance type properties. */
 
-/* (At any given time, there may be no parts of Xconq that use extension
-   properties, since releases will usually integrate all properties into
-   the normal set of property definitions ([umt]type.def).  However, these
-   extension accessors should be kept around, so that they're available
-   when needed.) */
+#undef  DEF_APROP_I
+#define DEF_APROP_I(name,FNAME,doc,SLOT,lo,dflt,hi)  \
+  int FNAME(a) int a; { checka(a); return atypes[a].SLOT; }
+#undef  DEF_APROP_S
+#define DEF_APROP_S(name,FNAME,doc,SLOT,dflt)  \
+  char *FNAME(a) int a; { checka(a); return atypes[a].SLOT; }
+#undef  DEF_APROP_L
+#define DEF_APROP_L(name,FNAME,doc,SLOT)  \
+  Obj *FNAME(a) int a; { checka(a); return atypes[a].SLOT; }
 
-Obj *
-get_u_extension(u, name, dflt)
-int u;
-char *name;
-Obj *dflt;
-{
-    Obj *ext = u_extensions(u), *rest, *bdg;
-
-    for_all_list(ext, rest) {
-	bdg = car(rest);
-	if (consp(bdg)
-	    && symbolp(car(bdg))
-	    && strcmp(c_string(car(bdg)), name) == 0) {
-	    return cadr(bdg);
-	}
-    }
-    return dflt;
-}
-
-/* Acquire a material type extension property by name. */
-
-Obj *
-get_m_extension(m, name, dflt)
-int m;
-char *name;
-Obj *dflt;
-{
-    Obj *ext = m_extensions(m), *rest, *bdg;
-
-    for_all_list(ext, rest) {
-	bdg = car(rest);
-	if (consp(bdg)
-	    && symbolp(car(bdg))
-	    && strcmp(c_string(car(bdg)), name) == 0) {
-	    return cadr(bdg);
-	}
-    }
-    return dflt;
-}
-
-/* Acquire a terrain type extension property by name. */
-
-Obj *
-get_t_extension(t, name, dflt)
-int t;
-char *name;
-Obj *dflt;
-{
-    Obj *ext = t_extensions(t), *rest, *bdg;
-
-    for_all_list(ext, rest) {
-	bdg = car(rest);
-	if (consp(bdg)
-	    && symbolp(car(bdg))
-	    && strcmp(c_string(car(bdg)), name) == 0) {
-	    return cadr(bdg);
-	}
-    }
-    return dflt;
-}
+#include "atype.def"
