@@ -7,10 +7,7 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern int type_can_sit_on_conn(int u, int x, int y);
 #include "kernel.h"
-extern int num_sides_allowed(int u);
-extern int side_owns_occupant(Side *side, Unit *unit);
 
 /* This is not a limit, just sets initial allocation of unit objects
    and how many additional to get if more are needed.  This can be
@@ -24,10 +21,21 @@ extern int side_owns_occupant(Side *side, Unit *unit);
 static void allocate_unit_block(void);
 static int compare_units(Unit *unit1, Unit *unit2);
 static int compare_units_by_keys(const void *e1, const void *e2);
-
 static void leave_cell_aux(Unit *unit, Unit *oldtransport);
-
 static void change_cell_aux(Unit *unit, int x0, int y0, int x, int y);
+static void kill_unit_aux(Unit *unit, int reason);
+static void free_used_cells(Unit *unit);
+static void set_unit_position(Unit *unit, int x, int y, int z);
+static void enter_cell_aux(Unit *unit, int x, int y);
+static void eject_occupant(Unit *unit, Unit *occ);
+static void insert_unit(Unit *unithead, Unit *unit);
+static void delete_unit(Unit *unit);
+static void check_unit(Unit *unit);
+static int disband_unit_directly(Side *side, Unit *unit);
+static void add_unit_to_stack(Unit *unit, int x, int y);
+static void remove_unit_from_stack(Unit *unit);
+static void glimpse_adjacent_terrain(Unit *unit);
+static int test_class_membership(Obj *leaf);
 
 /* The list of available units. */
 
@@ -77,11 +85,6 @@ int *numlivebytype;
 
 /* The next number to use for a unit id. */
 
-/* Starting the unit->id list at 1 will cause crashes in
-   side_number(unit->side) when it is fed the ghost unit with id = 0,
-   which seems to exist even though it does not have a valid side. */
-/* However, this should be found and fixed - id == 0 means "invalid unit". */
-
 int nextid = 1;
 
 /* A cache of completenesses. */
@@ -109,7 +112,7 @@ extern void free_used_cells(Unit *unit);
 /* Grab a block of unit objects to work with. */
 
 static void
-allocate_unit_block()
+allocate_unit_block(void)
 {
     int i;
     Unit *unitblock = (Unit *) xmalloc(INITMAXUNITS * sizeof(Unit));
@@ -127,7 +130,7 @@ allocate_unit_block()
    since newly created units will appear on it. */
 
 void
-init_units()
+init_units(void)
 {
     unitlist = NULL;
     allocate_unit_block();
@@ -139,8 +142,7 @@ init_units()
    make the dummy units that sit at the heads of the per-side unit lists. */
 
 Unit *
-create_bare_unit(type)
-int type;
+create_bare_unit(int type)
 {
     Unit *newunit;
 
@@ -162,8 +164,7 @@ int type;
    reasonable in them, since individual units objects see a lot of reuse. */
 
 Unit *
-create_unit(type, makebrains)
-int type, makebrains;
+create_unit(int type, int makebrains)
 {
     int m;
     Unit *newunit;
@@ -188,6 +189,9 @@ int type, makebrains;
     newunit = create_bare_unit(type);
     /* Init all the slots to distinguishable values.  The unit is not
        necessarily newly allocated, so we have to hit all of its slots. */
+    /* Note that we don't check for overflow of unit ids, since it's
+       highly unlikely that a single game would ever create 2 billion
+       units. */
     newunit->id = nextid++;
     newunit->name = NULL;
     /* Number == 0 means unit is unnumbered. */
@@ -257,7 +261,9 @@ int type, makebrains;
     newunit->autoresearch = FALSE;
     newunit->autoplan = FALSE;
     newunit->cp_stash = 0;
-
+    newunit->buildingdone = FALSE;
+    newunit->researchdone = FALSE;
+    
     /* Add to the global unit counts. */
     ++numunits;
     ++numliveunits;
@@ -266,8 +272,7 @@ int type, makebrains;
 }
 
 void
-init_unit_tooling(unit)
-Unit *unit;
+init_unit_tooling(Unit *unit)
 {
     unit->tooling = (short *) xmalloc(numutypes * sizeof(short));
 }
@@ -275,9 +280,7 @@ Unit *unit;
 /* Set or resize the array of a unit's opinions about each side in a game. */
 
 void
-init_unit_opinions(unit, nsides)
-Unit *unit;
-int nsides;
+init_unit_opinions(Unit *unit, int nsides)
 {
     int i;
     short *temp;
@@ -307,11 +310,12 @@ int nsides;
 /* Alter the actorstate object to be correct for this unit. */
 
 void
-init_unit_actorstate(unit, flagacp)
-Unit *unit;
-int flagacp;
+init_unit_actorstate(Unit *unit, int flagacp)
 {
-    if (u_acp(unit->type) > 0 && unit->cp > 0) {
+    if ((u_acp(unit->type) > 0 
+	/* Acp-independent units still need a plan. */
+	   || acp_indep(unit))
+    	&& unit->cp >= 0) {
 	/* Might already have an actorstate, don't realloc if so;
 	   but do clear an existing actorstate, since might be reusing. */
 	if (unit->act == NULL)
@@ -330,14 +334,18 @@ int flagacp;
     }
 }
 
+/* Allocate and fill in the unit extras.  This is needed whenever any
+   one of the extra properties is going to get a non-default value.
+   (Normally, the accessors for this structure return a default value
+   if one is not present in a unit.) */
+
 void
 init_unit_extras(Unit *unit)
 {
-    if (unit->extras == NULL) {
-	unit->extras = (UnitExtras *) xmalloc(sizeof(UnitExtras));
-    }
+    if (unit->extras == NULL)
+      unit->extras = (UnitExtras *) xmalloc(sizeof(UnitExtras));
     /* Each slot must get the same value as the accessor would return
-       if the structure had not been allocated. */ 
+       if the structure had not been allocated. */
     unit->extras->point_value = -1;
     unit->extras->appear = -1;
     unit->extras->appear_var_x = -1;
@@ -345,7 +353,7 @@ init_unit_extras(Unit *unit)
     unit->extras->disappear = -1;
     unit->extras->priority = -1;
     unit->extras->sym = lispnil;
-    unit->extras->hook = lispnil;
+    unit->extras->sides = lispnil;
 }
 
 /* Changing a unit's type has many effects. */
@@ -431,17 +439,10 @@ change_unit_type(Unit *unit, int newtype, int reason)
     ++(numlivebytype[newtype]);
 }
 
-int
-max_builds(int u)
-{
-    int u2;
-
-    for_all_unit_types(u2) {
-	if (could_create(u, u2))
-	  return 1;
-    }
-    return 0;
-}
+/* Test if any occupant of a given unit belongs to the given side.
+   This is used to detect if a unit might be known about even if it's
+   normally not visible (we have a spy inside a secret base for
+   instance). */
 
 int
 side_owns_occupant(Side *side, Unit *unit)
@@ -760,15 +761,13 @@ enter_cell_aux(Unit *unit, int x, int y)
 /* (this still needs to account for multipart units) */
 
 int
-can_occupy(unit, transport)
-Unit *transport, *unit;
+can_occupy(Unit *unit, Unit *transport)
 {
     return can_carry(transport, unit);
 }
 
 int
-can_carry(transport, unit)
-Unit *transport, *unit;
+can_carry(Unit *transport, Unit *unit)
 {
     int u = unit->type, u2 = transport->type, u3, o;
     int numthistype = 0, numalltypes = 0, occvolume = 0;
@@ -837,9 +836,7 @@ Unit *transport, *unit;
 /* (should share with prev routine somehow) */
 
 int
-type_can_occupy(u, transport)
-int u;
-Unit *transport;
+type_can_occupy(int u, Unit *transport)
 {
     int u2 = transport->type, u3, o;
     int numthistype = 0, numalltypes = 0, occvolume = 0;
@@ -901,9 +898,7 @@ Unit *transport;
 }
 
 int
-can_occupy_type(unit, u2)
-Unit *unit;
-int u2;
+can_occupy_type(Unit *unit, int u2)
 {
     int u = unit->type;
 
@@ -915,9 +910,7 @@ int u2;
 }
 
 int
-can_carry_type(transport, u)
-Unit *transport;
-int u;
+can_carry_type(Unit *transport, int u)
 {
     return type_can_occupy(u, transport);
 }
@@ -925,8 +918,7 @@ int u;
 /* Units become occupants by linking into the transport's occupant list. */
 
 void
-enter_transport(unit, transport)
-Unit *unit, *transport;
+enter_transport(Unit *unit, Unit *transport)
 {
     int u = unit->type, ustack, u2stack;
     Unit *unit2, *topunit, *prevunit = NULL, *nextunit = NULL;
@@ -984,8 +976,7 @@ Unit *unit, *transport;
 /* Dead units (hp = 0) may be run through here, so don't error out. */
 
 void
-leave_cell(unit)
-Unit *unit;
+leave_cell(Unit *unit)
 {
     int ux = unit->x, uy = unit->y;
     Unit *transport = unit->transport;
@@ -1012,8 +1003,7 @@ Unit *unit;
    this for all the occupants as well. */
 
 static void
-leave_cell_aux(unit, oldtransport)
-Unit *unit, *oldtransport;
+leave_cell_aux(Unit *unit, Unit *oldtransport)
 {
     Unit *occ;
 
@@ -1035,8 +1025,7 @@ Unit *unit, *oldtransport;
    hanging in limbo, so should have it occupy something immediately. */
 
 void
-leave_transport(unit)
-Unit *unit;
+leave_transport(Unit *unit)
 {
     Unit *transport = unit->transport, *occ;
 
@@ -1058,9 +1047,7 @@ Unit *unit;
 }
 
 int
-change_cell(unit, x, y)
-Unit *unit;
-int x, y;
+change_cell(Unit *unit, int x, int y)
 {
     int ux = unit->x, uy = unit->y;
     Unit *transport = unit->transport;
@@ -1135,14 +1122,12 @@ change_cell_aux(Unit *unit, int x0, int y0, int x, int y)
     }
 }
 
-/* Put the given unit into the cell and/or unit stack at the
-   given location.  Do not modify the unit's actual xyz position,
-   just the unit layer and links to other units. */
+/* Put the given unit into the cell and/or unit stack at the given
+   location.  Do not modify the unit's xyz properties, just the unit
+   layer and links to other units. */
 
 void
-add_unit_to_stack(unit, x, y)
-Unit *unit;
-int x, y;
+add_unit_to_stack(Unit *unit, int x, int y)
 {
     int u = unit->type, ustack, u2stack;
     Unit *topunit, *unit2, *prevunit = NULL, *nextunit = NULL;
@@ -1171,12 +1156,10 @@ int x, y;
     set_unit_at(x, y, topunit);
 }
 
-/* Remove the given unit from the stack at its current
-   location. */
+/* Remove the given unit from the stack at its current location. */
 
 void
-remove_unit_from_stack(unit)
-Unit *unit;
+remove_unit_from_stack(Unit *unit)
 {
     int ux = unit->x, uy = unit->y;
     Unit *other;
@@ -1197,7 +1180,8 @@ Unit *unit;
 }
 
 /* Add only the terrain types of adjacent cells to the unit's side's
-   view, don't show units or anything else. */
+   view, don't show units or anything else.  This is a workaround
+   for units with a vision range of 0. */
 
 void
 glimpse_adjacent_terrain(Unit *unit)
@@ -1226,8 +1210,7 @@ glimpse_adjacent_terrain(Unit *unit)
 /* Given an overfull unit, spew out occupants until back within limits. */
 
 void
-eject_excess_occupants(unit)
-Unit *unit;
+eject_excess_occupants(Unit *unit)
 {
     int u, u2 = unit->type, overfull = TRUE, count;
     int numalltypes = 0, occvolume = 0;
@@ -1295,8 +1278,7 @@ Unit *unit;
 #define ut_dies_on(u, t) (ut_vanishes_on(u,t) || ut_wrecks_on(u, t))
 
 void
-eject_occupant(unit, occ)
-Unit *unit, *occ;
+eject_occupant(Unit *unit, Unit *occ)
 {
     if (!in_play(unit) || !in_play(occ))
       return;
@@ -1355,6 +1337,10 @@ change_unit_side(Unit *unit, Side *newside, int reason, Unit *agent)
 	/* A freebie for the unit's previous side. */
 	see_exact(oldside, ux, uy);
     }
+
+    /* The new side gets to decide the unit's new plans.  */
+    init_unit_plan (unit);
+
     /* Reflect the changeover in any appropriate displays. */
     if (oldside != NULL) {
 	/* Now we see the unit as belonging to someone else. */
@@ -1473,25 +1459,6 @@ type_ever_available(int u, Side *side)
     return FALSE;
 }
 
-/* TRUE if side has advance to build this unit. */
-
-int
-has_advance_to_build(Side *side, int u)
-{
-    int a;
-
-    for_all_advance_types(a) {
-	if (ua_needed_to_build(u, a) && !has_advance(side, a)) {
-	    return FALSE;
-	}
-    }
-    /* Account for an obsoleting advance. */
-    a = u_obsolete(u);
-    if (a != NONATYPE && has_advance(side, a))
-      return FALSE;
-    return TRUE;
-}
-
 int
 num_sides_allowed(int u)
 {
@@ -1499,7 +1466,7 @@ num_sides_allowed(int u)
     Side *side;
 
     rslt = 0;
-    for_all_sides_plus_indep(side) {
+    for_all_sides(side) {
 	if (type_allowed_on_side(u, side)) {
 	    ++rslt;
 	}
@@ -1514,13 +1481,14 @@ unit_trusts_unit(Unit *unit1, Unit *unit2)
 	    || trusted_side(unit1->side, unit2->side));
 }
 
-/* Put the given unit on the given side, without all the fancy effects.
-   Important to handle independents, because this gets called during init.
-   This is the only way that a unit's side may be altered. */
+/* Put the given unit on the given side, without all the fancy
+   effects.  Important to handle independents, because this gets
+   called during init.  This is the only way that a unit's side may be
+   altered. */
 
 /* Note that this may be run on dead units, as part of clearing out a
-   side's units, in which case we just want to relink, don't care about
-   testing whether the type is allowed or not. */
+   side's units, in which case we just want to relink, don't care
+   about testing whether the type is allowed or not. */
 
 void
 set_unit_side(Unit *unit, Side *side)
@@ -1535,8 +1503,6 @@ set_unit_side(Unit *unit, Side *side)
     oldside = unit->side;
     if (oldside->numunits)
       --(oldside->numunits[u]);
-    if (oldside->numlive && in_play(unit))
-      --(oldside->numlive[u]);
     /* Set the unit's slot. */
     /* Note that indep units have a NULL side, even though there
 	   is an actual side object for independents. */
@@ -1548,8 +1514,6 @@ set_unit_side(Unit *unit, Side *side)
     /* Add to counts for the side. */
     if (newside->numunits)
       ++(newside->numunits[u]);
-    if (newside->numlive && in_play(unit))
-      ++(newside->numlive[u]);
     /* Invalidate both sides' point value caches. */
     oldside->point_value_valid = FALSE;
     newside->point_value_valid = FALSE;
@@ -1559,15 +1523,6 @@ set_unit_side(Unit *unit, Side *side)
 	side->tech[u] = u_tech_from_ownership(u);
 	/* (should update any displays of tech - how to ensure?) */
     }
-    /* Turn on automation for units that switch to being independent
-	or under AI control. Note that the reverse is NOT
-	true. Switching from AI to human control will therefore leave
-	the unit on automatic. */
-    if (side == indepside || !side_has_display(side) || side_has_ai(side)) {
-	unit->autoplan = TRUE;
-	unit->autoresearch = TRUE;
-	unit->autobuild = TRUE;
-    }
 }
 
 /* The origside is more of a historical record or cache, doesn't need
@@ -1618,9 +1573,7 @@ add_to_unit_hp(Unit *unit, int hp)
 }
 
 void
-change_morale(unit, sign, morchange)
-Unit *unit;
-int morchange;
+change_morale(Unit *unit, int sign, int morchange)
 {
     int u = unit->type, oldmorale;
 
@@ -1639,9 +1592,7 @@ int morchange;
 }
 
 int
-disband_unit(side, unit)
-Side *side;
-Unit *unit;
+disband_unit(Side *side, Unit *unit)
 {
     int rslt;
 
@@ -1668,9 +1619,7 @@ Unit *unit;
 }
 
 int
-disband_unit_directly(side, unit)
-Side *side;
-Unit *unit;
+disband_unit_directly(Side *side, Unit *unit)
 {
     if (side_can_disband(side, unit)) {
 	if (!completed(unit)) {
@@ -1686,8 +1635,7 @@ Unit *unit;
 }
 
 void
-wreck_unit(unit)
-Unit *unit;
+wreck_unit(Unit *unit)
 {
     int u = unit->type, nu;
 
@@ -1700,16 +1648,15 @@ Unit *unit;
     eject_excess_occupants(unit);
 }
 
-/* Remove a unit from play.  This is different from making it available for
-   reallocation - only the unit flusher can do that.  We remove all the
-   passengers too, recursively.  Sometimes units are "killed twice", so
-   be sure not to run all this twice.  Also count up occupant deaths, being
-   sure not to count the unit itself as an occupant. */
+/* Remove a unit from play.  This is different from making it
+   available for reallocation - only the unit flusher can do that.  We
+   remove all the passengers too, recursively.  Sometimes units are
+   "killed twice", so be sure not to run all this twice.  Also count
+   up occupant deaths, being sure not to count the unit itself as an
+   occupant. */
 
 void
-kill_unit(unit, reason)
-Unit *unit;
-int reason;
+kill_unit(Unit *unit, int reason)
 {
     int u = unit->type, selfdied = FALSE;
     int ux = unit->x, uy = unit->y;
@@ -1736,14 +1683,12 @@ int reason;
     }
 }
 
-/* Trash it now - occupant doesn't need to leave_cell.  Also record the
-   event, and update the apropriate display.  The unit here should
+/* Trash it now - occupant doesn't need to leave_cell.  Also record
+   the event, and update the apropriate display.  The unit here should
    be known to be alive. */
 
 void
-kill_unit_aux(unit, reason)
-Unit *unit;
-int reason;
+kill_unit_aux(Unit *unit, int reason)
 {
     int u = unit->type;
     Unit *occ;
@@ -1753,7 +1698,12 @@ int reason;
     /* Get rid of the unit's plan/tasks.  This should be safe, because
        unit death should only happen during action execution and in
        between turns, and plans/tasks should not be in use at those times. */
-    dispose_of_plan(unit);
+
+    /* Might not be anything to dispose of. */
+    if (unit->plan != NULL) {
+	    free_plan(unit->plan);
+	    unit->plan = NULL;
+    }
     /* Maybe enter the loss into the historical record. */
     if (reason >= 0)
       record_unit_death(unit, reason);
@@ -1778,17 +1728,35 @@ int reason;
     --(numlivebytype[u]);
 }
 
+/* Free up all cells used by unit that passed away. */
+
+void
+free_used_cells(Unit *unit)
+{
+    int x, y;
+
+    for_all_cells_within_reach(unit, x, y) {
+	if (!inside_area(x, y))
+	  continue;
+	if (user_at(x, y) == unit->id) 
+	  set_user_at(x, y, NOUSER);
+    }
+    unit->usedcells = 0;
+}
+
 /* Get rid of all dead units at once.
-   (This routine is basically a garbage collector, and should not be called
-   during a unit list traversal.) The process starts by finding the first
-   live unit, making it the head, then linking around all in the middle.
-   Dead units stay on the dead unit list for each side until that side has had
-   a chance to move.  Then they are finally flushed in a permanent fashion. */ 
+
+   (This routine is basically a garbage collector, and should not be
+   called during a unit list traversal.) The process starts by finding
+   the first live unit, making it the head, then linking around all in
+   the middle.  Dead units stay on the dead unit list for each side
+   until that side has had a chance to move.  Then they are finally
+   flushed in a permanent fashion. */
 
 static void flush_one_unit(Unit *unit);
 
 void
-flush_dead_units()
+flush_dead_units(void)
 {
     Unit *unit, *prevunit, *nextunit;
 
@@ -1825,8 +1793,7 @@ flush_dead_units()
    strictly necessary, but this is not an area to take chances with. */
 
 static void
-flush_one_unit(unit)
-Unit *unit;
+flush_one_unit(Unit *unit)
 {
     unit->id = -1;
     unit->occupant = NULL;
@@ -1847,15 +1814,14 @@ Unit *unit;
    sure that average performance in real games is what's being improved. */
 
 void
-sort_units(byidonly)
-int byidonly;
+sort_units(int byidonly)
 {
     int flips;
     int passes = 0;
     register Unit *unit, *nextunit;
     Side *side;
 
-    for_all_sides_plus_indep(side) {
+    for_all_sides(side) {
 	passes = 0;
 	flips = TRUE;
 	while (flips) {
@@ -1882,8 +1848,7 @@ int byidonly;
 }
 
 static int
-compare_units(unit1, unit2)
-Unit *unit1, *unit2;
+compare_units(Unit *unit1, Unit *unit2)
 {
     if (unit1->type != unit2->type)
       return (unit1->type - unit2->type);
@@ -1899,13 +1864,14 @@ Unit *unit1, *unit2;
     return (unit1->id - unit2->id);
 }
 
+#if 0		/* Unused. */
+
 /* Useful for the machine player to know how long it can move this
    piece before it should go home.  Assumes can't replenish from
    terrain.  Result may be negative, in which case it's time to go! */
 
 int
-moves_till_low_supplies(unit)
-Unit *unit;
+moves_till_low_supplies(Unit *unit)
 {
     int u = unit->type, m, moves = 1234567, tmp;
 
@@ -1918,13 +1884,14 @@ Unit *unit;
     return moves;
 }
 
+#endif
+
 /* Short, unreadable, but greppable listing of unit.  Primarily useful
    for debugging and warnings.  We use several buffers and rotate between
    them so we can call this more than once in a single printf. */
 
 char *
-unit_desig(unit)
-Unit *unit;
+unit_desig(Unit *unit)
 {
     int i;
     char *shortbuf;
@@ -1962,8 +1929,7 @@ Unit *unit;
    that changes from turn to turn. */
 
 char *
-unit_desig_no_loc(unit)
-Unit *unit;
+unit_desig_no_loc(Unit *unit)
 {
     char *shortbuf;
 
@@ -1990,8 +1956,7 @@ Unit *unit;
 /* Come up with a unit type name that fits in the given space. */
 
 char *
-utype_name_n(u, n)
-int u, n;
+utype_name_n(int u, int n)
 {
     char *utypename, *shortname, *rawname;
 
@@ -2022,19 +1987,18 @@ int u, n;
 }
 
 char *
-shortest_unique_name(u)
-int u;
+shortest_unique_name(int u)
 {
     char namebuf[BUFSIZE], *name1;
     int u1, u2, i, len, allhavechars, firstuniq[MAXUTYPES], firstuniq1;
     int shortestdone[MAXUTYPES];
 
-	/* Don't try to allocate shortestnames before numutypes has been
-	 * defined. This will cause crashes later on as the program mistakenly
-	 * believes that all shortestnames[u] have been allocated just because 
-	 * shortestnames != NULL. */
-	if (numutypes == 0)
-		return NULL;
+    /* Don't try to allocate shortestnames before numutypes has been
+       defined. This will cause crashes later on as the program
+       mistakenly believes that all shortestnames[u] have been
+       allocated just because shortestnames != NULL. */
+    if (numutypes == 0)
+      return NULL;
     if (shortestnames == NULL) {
 	shortestnames = (char **) xmalloc(numutypes * sizeof(char *));
 	/* First use game definition's single chars if possible. */
@@ -2112,8 +2076,7 @@ int u;
    instead. */
 
 char *
-shortest_generic_name(u)
-int u;
+shortest_generic_name(int u)
 {
     char namebuf[BUFSIZE], *name1;
     int u1, u2, i, allhavechars, firstuniq[MAXUTYPES], firstuniq1;
@@ -2198,8 +2161,7 @@ int u;
 /* This formats an actorstate readably. */
 
 char *
-actorstate_desig(as)
-ActorState *as;
+actorstate_desig(ActorState *as)
 {
     if (actorstatebuf == NULL)
       actorstatebuf = xmalloc(BUFSIZE);
@@ -2217,8 +2179,7 @@ ActorState *as;
 /* This is used a lot, it should be sped up. */
 
 Unit *
-find_unit(n)
-int n;
+find_unit(int n)
 {
     Unit *unit;
 
@@ -2232,8 +2193,7 @@ int n;
 /* Same, but don't by picky about liveness. */
 
 Unit *
-find_unit_dead_or_alive(n)
-int n;
+find_unit_dead_or_alive(int n)
 {
     Unit *unit;
 
@@ -2247,8 +2207,7 @@ int n;
 /* Find a unit with the given name, either alive or dead. */
 
 Unit *
-find_unit_by_name(nm)
-char *nm;
+find_unit_by_name(char *nm)
 {
     Unit *unit;
 
@@ -2264,8 +2223,7 @@ char *nm;
 /* Find a unit with the given number, either alive or dead. */
 
 Unit *
-find_unit_by_number(nb)
-int nb;
+find_unit_by_number(int nb)
 {
     Unit *unit;
 
@@ -2279,8 +2237,7 @@ int nb;
 /* Find a unit with the given symbol, either alive or dead. */
 
 Unit *
-find_unit_by_symbol(sym)
-Obj *sym;
+find_unit_by_symbol(Obj *sym)
 {
     Unit *unit;
 
@@ -2293,26 +2250,10 @@ Obj *sym;
     return NULL;
 }
 
-/* Given a name, find the type. */
-
-int
-find_unit_name(str)
-char *str;
-{
-    int u;
-
-    for_all_unit_types(u)
-      if (strcmp(str, u_short_name(u)) == 0
-	  || strcmp(str, u_type_name(u)) == 0) 
-	return u;
-    return NONUTYPE;
-}
-
 /* Insert the given unit after the other given unit. */
 
 void
-insert_unit(unithead, unit)
-Unit *unithead, *unit;
+insert_unit(Unit *unithead, Unit *unit)
 {
     unit->next = unithead->next;
     unit->prev = unithead;
@@ -2323,16 +2264,15 @@ Unit *unithead, *unit;
 /* Delete the unit from its list. */
 
 void
-delete_unit(unit)
-Unit *unit;
+delete_unit(Unit *unit)
 {
     unit->next->prev = unit->prev;
     unit->prev->next = unit->next;
 }
 
+#if 0 /* not used, although they seem useful... */
 int
-num_occupants(unit)
-Unit *unit;
+num_occupants(Unit *unit)
 {
     int num = 0;
     Unit *occ;
@@ -2344,7 +2284,7 @@ Unit *unit;
 }
 
 int
-num_units_at(x, y)
+num_units_at(int x, int y)
 int x, y;
 {
     int num = 0;
@@ -2360,11 +2300,12 @@ int x, y;
     }
     return num;
 }
+#endif
 
 /* Call this to doublecheck invariants on units. */
 
 void
-check_all_units()
+check_all_units(void)
 {
     Unit *unit;
 
@@ -2374,8 +2315,7 @@ check_all_units()
 }
 
 void
-check_unit(unit)
-Unit *unit;
+check_unit(Unit *unit)
 {
     if (alive(unit) && unit->transport && !alive(unit->transport)) {
     	run_warning("%s is inside a dead transport", unit_desig(unit));
@@ -2384,8 +2324,7 @@ Unit *unit;
 }
 
 UnitVector *
-make_unit_vector(initsize)
-int initsize;
+make_unit_vector(int initsize)
 {
     UnitVector *vec;
 	
@@ -2397,17 +2336,13 @@ int initsize;
 }
 
 void
-clear_unit_vector(vec)
-UnitVector *vec;
+clear_unit_vector(UnitVector *vec)
 {
     vec->numunits = 0;
 }
 
 UnitVector *
-add_unit_to_vector(vec, unit, flag)
-UnitVector *vec;
-Unit *unit;
-int flag;
+add_unit_to_vector(UnitVector *vec, Unit *unit, int flag)
 {
     int i;
     UnitVector *newvec;
@@ -2422,6 +2357,7 @@ int flag;
 	for (i = 0; i < vec->numunits; ++i) {
 	    newvec->units[i] = vec->units[i];
 	}
+	free(vec);
 	vec = newvec;
     }
     ((vec->units)[vec->numunits]).unit = unit;
@@ -2431,10 +2367,7 @@ int flag;
 }
 
 void
-remove_unit_from_vector(vec, unit, pos)
-UnitVector *vec;
-Unit *unit;
-int pos;
+remove_unit_from_vector(UnitVector *vec, Unit *unit, int pos)
 {
     int j;
 
@@ -2464,8 +2397,7 @@ int pos;
 enum sortkeys tmpsortkeys[MAXSORTKEYS];
 
 static int
-compare_units_by_keys(e1, e2)
-CONST void *e1, *e2;
+compare_units_by_keys(CONST void *e1, CONST void *e2)
 {
     int i;
     Unit *unit1 = ((UnitVectorEntry *) e1)->unit;
@@ -2552,86 +2484,18 @@ CONST void *e1, *e2;
 }
 
 void
-sort_unit_vector(vec)
-UnitVector *vec;
+sort_unit_vector(UnitVector *vec)
 {
     qsort(vec->units, vec->numunits, sizeof(UnitVectorEntry),
 	  compare_units_by_keys);
 }
 
-Obj *
-get_x_property(unit, subkey)
-Unit *unit;
-int subkey;
-{
-    Obj *lis, *bdg;
-
-    for_all_list(unit_hook(unit), lis) {
-	bdg = car(lis);
-	if (symbolp(car(bdg)) && keyword_code(c_string(car(bdg))) == subkey)
-	  return cdr(bdg);
-    }
-    return lispnil;
-}
-
-Obj *
-get_x_property_by_name(unit, str)
-Unit *unit;
-char *str;
-{
-    Obj *lis, *bdg;
-
-    for_all_list(unit_hook(unit), lis) {
-	bdg = car(lis);
-	if (symbolp(car(bdg)) && strcmp(c_string(car(bdg)), str) == 0)
-	  return cdr(bdg);
-    }
-    return lispnil;
-}
-
-/* This isn't the best place, but read.c getting too large, no
-   overflow file. */
-/* Note that the array is not initialized, allows for multiple calls. */
-
-void
-interp_utype_list(arr, lis)
-short *arr;
-Obj *lis;
-{
-    int u = 0;
-    Obj *rest, *head;
-
-    /* Assume that if the destination array does not exist, there is
-       probably a reason, and it's not our concern. */
-    if (arr == NULL)
-      return;
-    lis = eval(lis);
-    if (!consp(lis))
-      lis = cons(lis, lispnil);
-    if (numberp(car(lis)) && length(lis) == numutypes) {
-	u = 0;
-	for_all_list(lis, rest)
-	  arr[u++] = c_number(car(rest));
-    } else {
-	for_all_list(lis, rest) {
-	    head = car(rest);
-	    if (utypep(head)) {
-		arr[c_number(head)] = TRUE;
-	    } else {
-		/* syntax error */
-	    }
-	}
-    }
-}
-
 #ifdef DESIGNERS
 
 /* A designer can call this to create an arbitrary unit during the game. */
 
 Unit *
-designer_create_unit(side, u, s, x, y)
-Side *side;
-int u, s, x, y;
+designer_create_unit(Side *side, int u, int s, int x, int y)
 {
     Unit *newunit;
     Side *side2;
@@ -2695,9 +2559,7 @@ designer_teleport(Unit *unit, int x, int y, Unit *other)
 }
 
 int
-designer_change_side(unit, side)
-Unit *unit;
-Side *side;
+designer_change_side(Unit *unit, Side *side)
 {
     Side *side2;
 
@@ -2711,11 +2573,466 @@ Side *side;
 }
 
 int
-designer_disband(unit)
-Unit *unit;
+designer_disband(Unit *unit)
 {
     kill_unit(unit, -1);
     return TRUE;
 }
 
 #endif /* DESIGNERS */
+
+/* Unit-related functions moved here from actions.c and plan.c. */
+
+/* Functions returning general abilities of a unit. */
+
+int
+can_develop(Unit *unit)
+{
+    return type_can_develop(unit->type);
+}
+
+int
+type_can_develop(int u)
+{
+    int u2;
+	
+    for_all_unit_types(u2) {
+	if (uu_acp_to_develop(u, u2) > 0)
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+int
+can_toolup(Unit *unit)
+{
+    return type_can_toolup(unit->type);
+}
+
+int
+type_can_toolup(int u)
+{
+    int u2;
+	
+    for_all_unit_types(u2) {
+	if (uu_acp_to_toolup(u, u2) > 0)
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+int
+can_create(Unit *unit)
+{
+    return type_can_create(unit->type);
+}
+
+int
+type_can_create(int u)
+{
+    int u2;
+	
+    for_all_unit_types(u2) {
+	if (uu_acp_to_create(u, u2) > 0
+	    && uu_tp_max(u, u2) >= uu_tp_to_build(u, u2))
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+int
+can_complete(Unit *unit)
+{
+    return type_can_complete(unit->type);
+}
+
+int
+type_can_complete(int u)
+{
+    int u2;
+	
+    for_all_unit_types(u2) {
+	if (uu_acp_to_build(u, u2) > 0
+	    && uu_tp_max(u, u2) >= uu_tp_to_build(u, u2))
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+/* This tests whether the given unit is capable of doing repair. */
+
+int
+can_repair(Unit *unit)
+{
+    return type_can_repair(unit->type);
+}
+
+int
+type_can_repair(int u)
+{
+    int u2;
+	
+    for_all_unit_types(u2) {
+	if (uu_acp_to_repair(u, u2) > 0)
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+/* This is true if the given location has some kind of material for the
+   unit to extract. */
+
+int
+can_extract_at(Unit *unit, int x, int y, int *mp)
+{
+    int m;
+    Unit *unit2;
+
+    /* Can't do anything with an unseen location. */
+    if (unit->side != NULL
+	&& terrain_view(unit->side, x, y) == UNSEEN)
+      return FALSE;
+    for_all_material_types(m) {
+	if (um_acp_to_extract(unit->type, m) > 0) {
+	    /* Look for case of extraction from terrain. */
+	    if (any_cell_materials_defined()
+		&& cell_material_defined(m)
+		&& material_at(x, y, m) > 0) {
+		*mp = m;
+		return TRUE;
+	    }
+	    /* Then look for extraction from independent unit. */
+	    for_all_stack(x, y, unit2) {
+		if (in_play(unit2)
+		    && indep(unit2)
+		    && unit2->supply[m] > 0) {
+		    *mp = m;
+		    return TRUE;
+		}
+	    }
+	}
+    }
+    return FALSE;
+}
+
+/* This is true if the given location has some kind of material for the
+   unit to transfer. */
+
+int
+can_load_at(Unit *unit, int x, int y, int *mp)
+{
+    int m;
+    Unit *unit2;
+
+    /* Can't do anything with an unseen location. */
+    if (unit->side != NULL
+	&& terrain_view(unit->side, x, y) == UNSEEN)
+      return FALSE;
+    for_all_material_types(m) {
+	if (um_acp_to_load(unit->type, m) > 0) {
+	    for_all_stack(x, y, unit2) {
+		if (in_play(unit2)
+		    && unit2->side == unit->side
+		    && u_acp(unit2->type) == 0
+		    && um_acp_to_unload(unit2->type, m) > 0) {
+		    *mp = m;
+		    return TRUE;
+		}
+	    }
+	}
+    }
+    return FALSE;
+}
+
+/* This tests whether the given unit is capable of doing repair. */
+
+int
+can_change_type(Unit *unit)
+{
+    return type_can_change_type(unit->type);
+}
+
+int
+type_can_change_type(int u)
+{
+    int u2;
+	
+    for_all_unit_types(u2) {
+	if (uu_acp_to_change_type(u, u2) > 0)
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+int
+can_disband(Unit *unit)
+{
+    return (type_can_disband(unit->type) || !completed(unit));
+}
+
+int
+type_can_disband(int u)
+{
+    return (u_acp_to_disband(u) > 0);
+}
+
+int
+side_can_disband(Side *side, Unit *unit)
+{
+    if (is_designer(side))
+      return TRUE;
+    return (side_controls_unit(side, unit)
+	    && can_disband(unit));
+}
+
+/* This tests whether the given unit is capable of adding terrain. */
+
+int
+can_add_terrain(Unit *unit)
+{
+    return type_can_add_terrain(unit->type);
+}
+
+int
+type_can_add_terrain(int u)
+{
+    int t;
+	
+    for_all_terrain_types(t) {
+	if (ut_acp_to_add_terrain(u, t) > 0)
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+/* This tests whether the given unit is capable of removing terrain. */
+
+int
+can_remove_terrain(Unit *unit)
+{
+    return type_can_remove_terrain(unit->type);
+}
+
+int
+type_can_remove_terrain(int u)
+{
+    int t;
+	
+    for_all_terrain_types(t) {
+	if (ut_acp_to_remove_terrain(u, t) > 0)
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+/* These functions test if the given utype belonging to the given side
+can build various classes of units. */
+
+int
+can_build_attackers(Side *side, int u)
+{
+    int u2;
+	
+    for_all_unit_types(u2) {
+	if (u_offensive_worth(u2) > 0
+	    && could_create(u, u2)
+	    && side_can_build(side, u2))
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+int
+can_build_defenders(Side *side, int u)
+{
+    int u2;
+	
+    for_all_unit_types(u2) {
+	if (u_defensive_worth(u2) > 0
+	    && could_create(u, u2)
+	    && side_can_build(side, u2))
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+int
+can_build_explorers(Side *side, int u)
+{
+    int u2;
+	
+    for_all_unit_types(u2) {
+	if (u_explorer_worth(u2) > 0
+	    && could_create(u, u2)
+	    && side_can_build(side, u2))
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+int
+can_build_colonizers(Side *side, int u)
+{
+    int u2;
+	
+    for_all_unit_types(u2) {
+	if (u_colonizer_worth(u2) > 0
+	    && could_create(u, u2)
+	    && side_can_build(side, u2))
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+int
+can_build_facilities(Side *side, int u)
+{
+    int u2;
+	
+    for_all_unit_types(u2) {
+	if (u_facility_worth(u2) > 0
+	    && could_create(u, u2)
+	    && side_can_build(side, u2))
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+/* True if the given unit is a sort that can build other units. */
+
+int
+can_build(Unit *unit)
+{
+    int u2;
+
+    if (indep(unit) 
+        && g_indepside_can_build() != TRUE)
+	  return FALSE;
+    for_all_unit_types(u2) {
+	if (could_create(unit->type, u2))
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+int
+can_build_or_help(Unit *unit)
+{
+    int u2;
+
+    if (indep(unit) 
+        && g_indepside_can_build() != TRUE)
+	  return FALSE;
+    for_all_unit_types(u2) {
+	if (could_create(unit->type, u2)
+	    || uu_acp_to_build(unit->type, u2) > 0
+	    || uu_acp_to_develop(unit->type, u2) > 0)
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+int
+can_research(Unit *unit)
+{
+    /* Kind of crude, but works for now. */
+    if (u_advanced(unit->type))
+      return TRUE;
+    return FALSE;
+}
+
+int
+can_produce(Unit *unit)
+{
+    int m;
+
+    for_all_material_types(m) {
+	if (um_acp_to_produce(unit->type, m))
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+
+/* Test if unit can move out into adjacent cells. */
+
+int
+can_move(Unit *unit)
+{
+    int d, x, y;
+
+    for_all_directions(d) {
+	if (interior_point_in_dir(unit->x, unit->y, d, &x, &y)) {
+	    /* (should account for world-leaving options?) */
+	    if (could_live_on(unit->type, terrain_at(x, y)))
+	      return TRUE;
+	}
+    }
+    return FALSE;
+}
+
+/* This is the maximum distance from "home" that a unit can expect to get,
+   travelling on its most hostile terrain type. */
+
+int
+operating_range_worst(int u)
+{
+    int m, t, prod, range, worstrange = area.maxdim;
+
+    for_all_material_types(m) {
+    	if (um_base_consumption(u, m) > 0) {
+	    for_all_terrain_types(t) {
+	    	if (!terrain_always_impassable(u, t)) {
+	    	    prod = (um_base_production(u, m) * ut_productivity(u, t)) / 100;
+	    	    if (prod < um_base_consumption(u, m)) {
+			range = um_storage_x(u, m) / (um_base_consumption(u, m) - prod);
+			if (range < worstrange)
+			  worstrange = range;
+		    }
+		}
+	    }
+	}
+    }
+    return worstrange;
+}
+
+/* Same, but for best terrain. */
+
+int
+operating_range_best(int u)
+{
+    int m, t, prod, range, tbestrange, tbest = 0, bestrange = 0;
+    int moves, consump;
+
+    for_all_terrain_types(t) {
+	if (!terrain_always_impassable(u, t)) {
+	    tbestrange = area.maxdim;
+	    for_all_material_types(m) {
+		consump = 0;
+		moves = (u_acp(u) * u_speed(u)) / 100;
+		if (um_consumption_per_move(u, m) > 0) {
+		    consump = moves * um_consumption_per_move(u, m);
+		}
+		if (moves <= 0)
+		  moves = 1;
+		if (um_base_consumption(u, m) > 0) {
+		    consump = max(consump, um_base_consumption(u, m));
+		}
+	    	prod = (um_base_production(u, m) * ut_productivity(u, t)) / 100;
+	    	if (prod > 0) {
+		    consump = max(0, consump - prod);
+		}
+		if (consump > 0) {
+		    range = (um_storage_x(u, m) * moves) / consump;
+		    if (range < tbestrange)
+		      tbestrange = range;
+		}
+	    }
+	    if (tbestrange > bestrange) {
+		bestrange = tbestrange;
+		tbest = t;
+	    }
+	}
+    }
+    return bestrange;
+}
