@@ -21,10 +21,12 @@ extern long default_population;
 
 typedef struct a_unit {
     short type;			/* type */
-    short id;		  	/* truly unique id number */
+    int id;		  	/* truly unique id number */
     char *name;			/* the name, if given */
     int number;			/* semi-unique number */
     short x, y, z;	     	/* position of unit in world */
+    /* NOTE: unit->side and unit->origside must always point to
+       valid side objects; dereferences are not checked! */
     struct a_side *side;	/* whose side this unit is on */
     struct a_side *origside;	/* the first side this unit was on */
     short hp;			/* how much more damage each part can take */
@@ -43,7 +45,6 @@ typedef struct a_unit {
     struct a_plan *plan;	/* the unit's current plan */
     struct a_unit_extras *extras;  /* pointer to optional stuff */
     char *aihook;		/* used by AI to keep info about this unit */
-    char *uihook;		/* used by interfaces for their own purposes */
     /* Following slots are never saved. */
     struct a_unit *occupant;	/* pointer to first unit being carried */
     struct a_unit *nexthere;	/* pointer to fellow occupant */
@@ -65,15 +66,18 @@ typedef struct a_unit {
     short autoresearch;		/* True if research on automatic. */
     short autoplan;		/* True if planning on automatic. */
     short cp_stash;		/* Stash of cps recovered from incomplete build tasks. */
+    short buildingdone;		/* We are done with construction for this turn. */
+    short researchdone;		/* We are done with research for this turn. */
+   
 } Unit;
 
 /* The unit extras structure stores properties that are (usually)
-   uncommon or special-purpose.  It is allocated as needed, access
-   is through macros, and this does not appear as a separate
-   structure when saving/restoring.  If a value is likely to be
-   filled in for most or all units in several different game designs,
-   then it should be in the main structure (even at the cost of extra
-   space), rather than here. */
+   uncommon or special-purpose.  It is allocated as needed, access is
+   through macros, and this does not appear as a separate structure
+   when saving/restoring.  If a value is likely to be filled in for
+   most or all units in several different game designs, then it should
+   be in the main structure (even at the cost of extra space), rather
+   than here. */
 
 typedef struct a_unit_extras {
     short point_value;		/* individual point value for this unit */
@@ -82,9 +86,24 @@ typedef struct a_unit_extras {
     short disappear;		/* turn of disappearance from game */
     short priority;		/* individual priority */
     Obj *sym;			/* symbol for xrefs */
-    Obj *hook;		 	/* placeholder for more optional stuff */
+    Obj *sides;			/* list of possible sides */
 } UnitExtras;
 
+/* The unit view is a partial mirror of an actual unit, including only
+   the data that might be known to another side seeing the unit from a
+   distance. */
+
+typedef struct a_unit_view {
+    short type;			/* type */
+    short side_id;		/* id of view's side */
+    short size;			/* abstract size */
+    short x, y;			/* location of view */
+    short date;			/* turn on which last updated */
+    int id;			/* id of actual unit if known */
+    struct a_unit *unit;	/* pointer to actual unit, if id valid */
+    struct a_unit_view *nexthere; /* pointer to next unit view */
+} UnitView;
+
 /* Some convenient macros. */
 
 /* Since it is possible for a unit to change sides and therefore
@@ -108,7 +127,8 @@ typedef struct a_unit_extras {
 #define for_all_occupants(u1,v) \
   for (v = (u1)->occupant; v != NULL; v = v->nexthere)
 
-/* Iterate through all occupants including occs within occs within occs within (u). */
+/* Iterate through all occupants including occs within occs within
+   occs within (u). */
 
 /* This nifty little macro will climb the occupant : nexthere tree and
    follow all branches three levels below (u) to find all the occs
@@ -210,15 +230,14 @@ typedef struct a_unit_extras {
 
 #define unit_symbol(unit) ((unit)->extras ? (unit)->extras->sym : lispnil)
 
-/* We test the hook for NULL, because lispnil is not NULL and we can't
-   count on the hook being initialized when extras are allocated. */
-
-#define unit_hook(unit)  \
-  (((unit)->extras && (unit)->extras->hook) ? (unit)->extras->hook : lispnil)
+#define unit_sides(unit) ((unit)->extras ? (unit)->extras->sides : lispnil)
 
 #define unit_doctrine(unit)  \
   (unit->side->udoctrine[unit->type])
 
+#define construction_run_doctrine(unit,u2) \
+  (unit_doctrine(unit)->construction_run[u2])
+
 /* A sortable vector of units, generally useful. */
 
 /* The kinds of sort keys available for list windows. */
@@ -249,6 +268,11 @@ typedef struct a_unitvector {
     UnitVectorEntry units[1];
 } UnitVector;
 
+/* Given an index into a unit vector, return the unit. */
+/* (should check args?) */
+
+#define unit_in_vector(uvec,ix) ((uvec)->units[ix].unit)
+
 /* Types of primitive unit actions. */
 
 typedef enum actiontype {
@@ -272,8 +296,8 @@ typedef struct a_actiondefn {
 
 typedef struct a_action {
     ActionType type;		/* the type of the action */
-    short args[MAXACTIONARGS];	/* assorted parameters */
-    short actee;		/* the unit being affected by action */
+    int args[MAXACTIONARGS];	/* assorted parameters */
+    int actee;		/* the unit being affected by action */
     struct a_action *next;	/* chain to next action */
 } Action;
 
@@ -289,12 +313,23 @@ typedef struct a_actorstate {
 #define has_pending_action(unit)  \
   ((unit)->act && (unit)->act->nextaction.type != ACTION_NONE)
 
+#define acp_indep(unit) u_acp_independent((unit)->type)
+
 /* This is true if the unit still has acp left to act with.  This used
    to test acp > acp-min, but that allows units to move around with
    negative acp, and doesn't work as well in game designs. */
 
-#define has_acp_left(unit)  \
-  ((unit)->act && (unit)->act->acp > 0)
+/* Now returns TRUE if an acp-independent unit that can build is either 
+waiting for tasks or not yet done with construction for this turn. */
+
+#define has_acp_left(unit) \
+  ((unit)->act \
+  && ((unit)->act->acp > 0 \
+        || (acp_indep((unit)) \
+            && can_build((unit)) \
+        	   && ((unit)->buildingdone != TRUE \
+        	   	|| ((unit)->plan \
+        	   	    && (unit)->plan->waitingfortasks)))))
 
 /* All the definitions that govern planning. */
 
@@ -307,7 +342,7 @@ typedef struct a_actorstate {
 typedef enum goaltype {
 
 #undef  DEF_GOAL
-#define DEF_GOAL(name,GOALTYPE,args) GOALTYPE,
+#define DEF_GOAL(name,dname,GOALTYPE,args) GOALTYPE,
 
 #include "goal.def"
 
@@ -316,6 +351,7 @@ typedef enum goaltype {
 
 typedef struct a_goaldefn {
     char *name;
+    char *display_name;
     char *argtypes;
 } GoalDefn;
 
@@ -330,20 +366,22 @@ typedef struct a_goal {
     short args[MAXGOALARGS];
 } Goal;
 
-extern Goal *create_goal(GoalType type, Side *side, int tf);
-extern int cell_unknown(int x, int y);
-extern int enemies_present(int x, int y);
-extern int goal_truth(Side *side, Goal *goal);
-extern char *goal_desig(Goal *goal);
+/* A task is a single executable element of a unit's plan.  Each task
+   type is something that has been found useful or convenient to
+   encapsulate as a step in a plan.  Tasks are also useful for human
+   interfaces to use instead of actions, since they have retry
+   capabilities that are not part of action execution. */
 
-/* A task is a single executable element of a unit's plan.  Each task type
-   is something that has been found useful or convenient to encapsulate as
-   a step in a plan. */
+/* Note that a task's arguments are not the same as the arguments
+   passed to the functions that create and push tasks.  For instance,
+   the 'c' task argument is actually a state variable recording a
+   direction choice; this is important for coherent routefinding, but
+   is not needed in order to create a movement task. */
 
 typedef enum a_tasktype {
 
 #undef  DEF_TASK
-#define DEF_TASK(name,CODE,argtypes,dofn,createfn,setfn,netsetfn,pushfn,netpushfn,argdecl) CODE,
+#define DEF_TASK(name,dname,CODE,argtypes,dofn,createfn,setfn,netsetfn,pushfn,netpushfn,argdecl) CODE,
 
 #include "task.def"
 
@@ -362,15 +400,16 @@ typedef enum a_taskoutcome {
 
 typedef struct a_task {
     TaskType type;		/* the kind of task we want to do */
-    short args[MAXTASKARGS];	/* arguments */
+    int args[MAXTASKARGS];	/* arguments */
     short execnum;		/* how many times this has been done */
     short retrynum;		/* number of immed failures so far */
     struct a_task *next;	/* the next task to undertake */
 } Task;
 
 typedef struct a_taskdefn {
-    char *name;
-    char *argtypes;
+    char *name;			/* name of the task type */
+    char *display_name;		/* name to display in interfaces */
+    char *argtypes;		/* string giving types of task's args */
     TaskOutcome (*exec)(Unit *unit, Task *task);
 } TaskDefn;
 
@@ -422,6 +461,9 @@ typedef struct a_plan {
 #define for_all_tasks(plan,task)  \
   for (task = (plan)->tasks; task != NULL; task = task->next)
 
+#define ai_controlled(unit)  \
+  ((unit)->plan && (unit)->plan->aicontrol && side_has_ai(unit->side))
+
 /* Global unit variables. */
 
 extern Unit *unitlist;
@@ -442,160 +484,98 @@ extern TaskDefn taskdefns[];
 
 extern char *plantypenames[];
 
-/* Declarations of unit-related functions. */
+/* Declarations of unit-related functions that do not change the kernel state. 
+Those that actually do something have been moved to kernel.h. */
 
-extern void init_units(void);
-extern Unit *create_bare_unit(int type);
-extern Unit *create_unit(int type, int makebrains);
-extern void init_unit_actorstate(Unit *unit, int flagacp);
-extern void init_unit_plan(Unit *unit);
-extern void init_unit_tooling(Unit *unit);
-extern void init_unit_opinions(Unit *unit, int nsides);
-extern void init_unit_extras(Unit *unit);
-extern void change_unit_type(Unit *unit, int newtype, int reason);
-extern int max_builds(int u);
-extern int enter_cell(Unit *unit, int x, int y);
+/* Testable conditions etc. */
+
+extern int side_owns_occupant(Side *side, Unit *unit);
 extern int can_occupy_cell(Unit *unit, int x, int y);
 extern int type_can_occupy_cell(int u, int x, int y);
 extern int can_occupy_cell_without(Unit *unit, int x, int y, Unit *unit3);
 extern int type_can_occupy_cell_without(int u, int x, int y, Unit *unit3);
-extern void enter_cell_aux(Unit *unit, int x, int y);
+extern int type_can_sit_on_conn(int u, int x, int y);
 extern int can_occupy(Unit *unit, Unit *transport);
 extern int can_carry(Unit *transport, Unit *unit);
 extern int type_can_occupy(int u, Unit *transport);
 extern int can_occupy_type(Unit *unit, int u2);
 extern int can_carry_type(Unit *transport, int u);
-extern void enter_transport(Unit *unit, Unit *transport);
-extern void leave_cell(Unit *unit);
-extern void leave_transport(Unit *unit);
-extern void eject_excess_occupants(Unit *unit);
-extern void eject_occupant(Unit *unit, Unit *occ);
 extern int new_unit_allowed_on_side(int u, Side *side);
 extern int unit_allowed_on_side(Unit *unit, Side *side);
-extern int test_class_membership(Obj *leaf);
+extern int num_sides_allowed(int u);
 extern int type_allowed_on_side(int u, Side *side);
 extern int type_ever_available(int u, Side *side);
 extern int unit_trusts_unit(Unit *unit1, Unit *unit2);
-extern void dispose_of_plan(Unit *unit);
-extern void flush_dead_units(void);
-extern void sort_units(int byidonly);
+extern int can_occupy_conn(Unit *unit, int nx, int ny, int nz);
+extern int can_build(Unit *unit);
+extern int can_move(Unit *unit);
+extern int can_extract_at(Unit *unit, int x, int y, int *mp);
+extern int can_load_at(Unit *unit, int x, int y, int *mp);
+extern int can_develop(Unit *unit);
+extern int type_can_develop(int u);
+extern int can_toolup(Unit *unit);
+extern int type_can_toolup(int u);
+extern int can_create(Unit *unit);
+extern int type_can_create(int u);
+extern int can_complete(Unit *unit);
+extern int type_can_complete(int u);
+extern int can_repair(Unit *unit);
+extern int type_can_repair(int u);
+extern int can_change_type(Unit *unit);
+extern int type_can_change_type(int u);
+extern int can_disband(Unit *unit);
+extern int type_can_disband(int u);
+extern int side_can_disband(Side *side, Unit *unit);
+extern int can_add_terrain(Unit *unit);
+extern int type_can_add_terrain(int u);
+extern int can_remove_terrain(Unit *unit);
+extern int type_can_remove_terrain(int u);
+extern int can_build_attackers(Side *side, int u);
+extern int can_build_defenders(Side *side, int u);
+extern int can_build_explorers(Side *side, int u);
+extern int can_build_colonizers(Side *side, int u);
+extern int can_build_facilities(Side *side, int u);
+extern int can_build_or_help(Unit *unit);
+extern int can_research(Unit *unit);
+extern int can_produce(Unit *unit);
+extern int operating_range_best(int u);
+extern int operating_range_worst(int u);
+
+#if 0
 extern int moves_till_low_supplies(Unit *unit);
+extern int num_occupants(Unit *unit);
+extern int num_units_at(int x, int y);
+#endif
+
+/* Unit names and designations. */
+
 extern char *unit_desig(Unit *unit);
 extern char *unit_desig_no_loc(Unit *unit);
 extern char *utype_name_n(int u, int n);
 extern char *shortest_unique_name(int u);
 extern char *shortest_generic_name(int u);
 extern char *actorstate_desig(struct a_actorstate *as);
+
+/* Unit locators. */
+
 extern Unit *find_unit(int n);
 extern Unit *find_unit_dead_or_alive(int n);
 extern Unit *find_unit_by_name(char *nm);
 extern Unit *find_unit_by_number(int nb);
 extern Unit *find_unit_by_symbol(Obj *sym);
-extern int find_unit_name(char *str);
-extern void insert_unit(Unit *unithead, Unit *unit);
-extern void delete_unit(Unit *unit);
-extern int num_occupants(Unit *unit);
-extern int num_units_at(int x, int y);
-extern void check_all_units(void);
-extern void check_unit(Unit *unit);
+
+/* Unit vector manipulating functions. Declared here rather than in kernel.h 
+since they also are used in the interfaces. */
+
 extern UnitVector *make_unit_vector(int initsize);
 extern void clear_unit_vector(UnitVector *vec);
 extern UnitVector *add_unit_to_vector(UnitVector *vec, Unit *unit, int flag);
 extern void remove_unit_from_vector(UnitVector *vec, Unit *unit, int pos);
 extern void sort_unit_vector(UnitVector *vec);
-extern Obj *get_x_property(Unit *unit, int subkey);
-extern Obj *get_x_property_by_name(Unit *unit, char *str);
 
-extern int disband_unit_directly(Side *side, Unit *unit);
+/* Turn init code - does not really belong here. */
 
-extern void add_unit_to_stack(Unit *unit, int x, int y);
-extern void remove_unit_from_stack(Unit *unit);
-extern void glimpse_adjacent_terrain(Unit *unit);
-extern void add_to_unit_hp(Unit *unit, int hp);
-
-extern void interp_utype_list(short *arr, Obj *lis);
-
-/* Declarations of plan-related functions. */
-
-extern int move_into_formation(Unit *unit);
-extern int victim_here(int x, int y);
-extern int worth_capturing(Side *side, int u2, Side *oside, int x, int y);
-extern int target_here(int x, int y);
-extern int supplies_here(Unit *unit, int x, int y, int m);
-extern int indep_captureable_here(int x, int y);
-extern int useful_captureable_here(int x, int y);
-extern int useful_type(Side *side, int u);
-extern int could_capture_any(int u);
-extern void make_plausible_random_args(char *argtypestr, int i, short *args, Unit *unit);
-extern int range_left(Unit *unit);
-extern int find_worths(int range);
-extern int attack_worth(Unit *unit, int e);
-extern int threat(Side *side, int u, int x0, int y0);
-extern int react_to_enemies(Unit *unit);
-extern int move_patrol(Unit *unit);
-extern int build_time(Unit *unit, int prod);
-extern Plan *create_plan(void);
-extern void free_plan(Plan *plan);
-extern char *plan_desig(Plan *plan);
-extern int might_be_captured(Unit *unit);
-extern int occupant_could_capture(Unit *unit, int etype);
-extern int can_capture_neighbor(Unit *unit);
-extern int occupant_can_capture_neighbor(Unit *unit);
-extern int find_closest_unit(Side *side, int x0, int y0, int maxdist, int (*pred)(int x, int y), int *rxp, int *ryp);
-extern int fullness(Unit *unit);
-extern int can_build(Unit *unit);
-extern int can_move(Unit *unit);
-extern int out_of_ammo(Unit *unit);
-extern int survival_time(Unit *unit);
-extern int usable_cell(Unit *unit, int x, int y);
-extern int explorable_cell(int x, int y);
-extern int reachable_unknown(int x, int y);
-extern int adj_known_ok_terrain(int x, int y, Side *side, int u);
-extern int should_capture_maker(Unit *unit);
-extern int no_possible_moves(Unit *unit);
-extern int normal_completion_time(int u, int u2);
-extern int adj_unit(int x, int y);
-extern int past_halfway_point(Unit *unit);
-
-extern int self_build_base_for_self(Unit *unit);
-
-extern int operating_range_worst(int u);
-extern int operating_range_best(int u);
-
-extern int terrain_always_impassable(int u, int t);
-
-extern int execute_standing_order(Unit *unit, int addtask);
-extern int can_build_or_help(Unit *unit);
-extern int can_produce(Unit *unit);
-extern int low_on_supplies_one(Unit *unit);
-extern int low_on_ammo_one(Unit *unit);
-
-extern void set_waiting_for_tasks(Unit *unit, int flag);
-
-extern int execute_standing_order(Unit *unit, int addtask);
-
-/* Declarations of task-related functions. */
-
-extern void init_tasks(void);
-extern void allocate_task_block(void);
-extern Task *create_task(TaskType type);
-extern Task *clone_task(Task *oldtask);
-extern int fire_can_damage(Unit *unit, Unit *unit2);
-extern Unit *repair_here(int x, int y);
-extern Unit *aux_resupply_here(Unit *unit);
-extern Unit *resupply_here(int x, int y);
-extern int can_auto_resupply_self(Unit *unit, int *materials, int numtypes);
-extern int choose_move_dirs(Unit *unit, int tx, int ty, int shortest, int (*dirtest)(Unit *, int), void (*dirsort)(Unit *, int *, int), int *dirs);
-extern int plausible_move_dir(Unit *unit, int dir);
-extern void sort_directions(Unit *unit, int *dirs, int numdirs);
-extern void free_task(Task *task);
-extern char *parse_task(Side *side, char *str, Task **taskp);
-extern char *task_desig(Task *task);
-extern Unit *find_unit_to_complete(Unit *unit, Task *task);
-
-extern int real_capture_chance(Unit *unit, Unit *unit2);
-extern int attack_can_damage_or_capture(Unit *unit, Unit *unit2);
-extern int fire_can_damage_or_capture(Unit *unit, Unit *unit2);
+extern void check_all_units(void);
 
 #undef  DEF_ACTION
 #define DEF_ACTION(name,code,args,prepfn,netprepfn,dofn,CHECKFN,ARGDECL,doc)  \
@@ -604,8 +584,7 @@ extern int fire_can_damage_or_capture(Unit *unit, Unit *unit2);
 #include "action.def"
 
 #undef  DEF_TASK
-#define DEF_TASK(name,code,argtypes,dofn,CREATEFN,setfn,netsetfn,pushfn,netpushfn,ARGDECL)  \
+#define DEF_TASK(name,dname,code,argtypes,dofn,CREATEFN,setfn,netsetfn,pushfn,netpushfn,ARGDECL)  \
   extern Task *CREATEFN ARGDECL;
 
 #include "task.def"
-
