@@ -8,8 +8,6 @@ any later version.  See the file COPYING.  */
 
 #include "config.h"
 #include "misc.h"
-extern int select_by_weight(int *arr, int numvals);
-extern void debugging_string(char *buf);
 #include "lisp.h"
 #include "dir.h"
 
@@ -71,11 +69,6 @@ int xmalloc_warnings;
 
 int memory_exhausted;
 
-/* If true, the run error is a bug in Xconq, so termination should do
-   more to help debugging (such as dumping core). */
-
-int error_is_bug;
-
 static char *padbuf;
 
 /* Random number handling is important to game but terrible/nonexistent
@@ -337,7 +330,7 @@ int amt;
     instead). */ 
     if (value == NULL && amt != 0) {
 #else
-    if (value == NULL) {
+//    if (value == NULL) {
 #endif
 
 #ifdef DEBUGGING
@@ -404,8 +397,7 @@ vtprintf(char *buf, char *str, va_list ap)
 }
 
 int
-select_by_weight(arr, numvals)
-int *arr, numvals;
+select_by_weight(int *arr, int numvals)
 {
     int sum = 0, i, n;
 
@@ -436,8 +428,7 @@ int *arr, numvals;
 /* Copy a string to newly-allocated space.  The new space is never freed. */
 
 char *
-copy_string(str)
-char *str;
+copy_string(char *str)
 {
     int len = strlen(str);
     char *rslt;
@@ -453,9 +444,7 @@ char *str;
 /* Insert the given number of blanks between each char of the string. */
 
 char *
-pad_blanks(str, n)
-char *str;
-int n;
+pad_blanks(char *str, int n)
 {
     char *pb;
     int i;
@@ -478,9 +467,7 @@ int n;
    Return -1 on failed search. */
 
 int
-iindex(ch, str)
-int ch;
-char *str;
+iindex(int ch, char *str)
 {
     int i;
 
@@ -495,8 +482,7 @@ char *str;
 /* Return a time difference as an long integer number of seconds. */
 
 long
-idifftime(t1, t0)
-time_t t1, t0;
+idifftime(time_t t1, time_t t0)
 {
 #ifdef MAC /* (should be anywhere that actually has a difftime function) */
     return ((long) difftime(t1, t0));
@@ -509,11 +495,8 @@ time_t t1, t0;
    data values. */
 
 void
-case_panic(str, var)
-char *str;
-int var;
+case_panic(char *str, int var)
 {
-    error_is_bug = TRUE;
     run_error("Panic! Unknown %s %d", str, var);
 }
 
@@ -522,8 +505,7 @@ int var;
 /* Improved version from Bruce Fast, via Scott Herod. */
 
 int
-isqrt(i)
-int i;
+isqrt(int i)
 {
     int j, k;
 
@@ -654,6 +636,8 @@ int *flagp;
     prealloc_debug();
 }
 
+#if 0		/* Unused. */
+
 void
 debugging_string(buf)
 char *buf;
@@ -669,6 +653,8 @@ char *buf;
       strcat(buf, "G");
 }
 
+#endif
+
 void
 debug_printf(char *str, ...)
 {
