@@ -7,7 +7,6 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern void divide_into_regions(char *tlayer, TRegion **rlayer);
 #include "kernel.h"
 
 #include <math.h>
@@ -36,6 +35,11 @@ int numconntypes = 0;
 
 int numcoattypes = 0;
 
+/* This is an array listing all the aux types in order, typically used
+   to construct menus in interfaces. */
+
+short *aux_terrain_type_index;
+
 /* This is true if any terrain type has nonzero thickness. */
 
 int any_thickness;
@@ -104,7 +108,7 @@ int numregions;
 /* Clean out all the world and area data. */
 
 void
-init_world()
+init_world(void)
 {
     memset(&world, 0, sizeof(World));
     /* These default values effectively disable day and year effects. */
@@ -120,8 +124,7 @@ init_world()
 }
 
 int
-set_world_circumference(circum, warn)
-int circum, warn;
+set_world_circumference(int circum, int warn)
 {
 
     /* All world circumferences are valid, no checks necessary. */
@@ -138,8 +141,7 @@ int circum, warn;
 }
 
 int
-set_area_shape(width, height, warn)
-int width, height, warn;
+set_area_shape(int width, int height, int warn)
 {
     if (!valid_area_shape(width, height, warn))
       return FALSE;
@@ -154,8 +156,7 @@ int width, height, warn;
 }
 
 int
-valid_area_shape(width, height, warn)	
-int width, height, warn;
+valid_area_shape(int width, int height, int warn)	
 {
     if (width < MINWIDTH || height < MINHEIGHT) {
 	if (warn)
@@ -172,7 +173,7 @@ int width, height, warn;
 }
 
 void
-check_area_shape()
+check_area_shape(void)
 {
     if (area.width == 0 || area.height == 0)
       run_error("0x0 area");
@@ -184,7 +185,7 @@ check_area_shape()
    classes of calculations need to be done, such as weather changes. */
 
 void
-calculate_world_globals()
+calculate_world_globals(void)
 {
     int t, m;
 
@@ -246,7 +247,7 @@ calculate_world_globals()
 void
 count_terrain_subtypes(void)
 {
-    int t, prevb = -1, prevc = -1;
+    int t, i = 0, prevb = -1, prevc = -1;
 
     /* Note that this function may be called several times. */
     first_bord_type = first_conn_type = -1;
@@ -255,7 +256,9 @@ count_terrain_subtypes(void)
       next_bord_type = (short *) xmalloc(MAXTTYPES * sizeof(short));
     if (next_conn_type == NULL)
       next_conn_type = (short *) xmalloc(MAXTTYPES * sizeof(short));
-    
+    if (aux_terrain_type_index == NULL)
+      aux_terrain_type_index = (short *) xmalloc(MAXTTYPES * sizeof(short));
+
     numcelltypes = numbordtypes = numconntypes = numcoattypes = 0;
     any_thickness = FALSE;
     for_all_terrain_types(t) {
@@ -271,6 +274,7 @@ count_terrain_subtypes(void)
 		next_bord_type[prevb] = t;
 	    }
 	    prevb = t;
+	    aux_terrain_type_index[i++] = t;
 	    break;
 	  case connectionsubtype:
 	    ++numconntypes;
@@ -280,9 +284,11 @@ count_terrain_subtypes(void)
 		next_conn_type[prevc] = t;
 	    }
 	    prevc = t;
+	    aux_terrain_type_index[i++] = t;
 	    break;
 	  case coatingsubtype:
 	    ++numcoattypes;
+	    aux_terrain_type_index[i++] = t;
 	    break;
 	}
 	if (t_thickness(t) > 0)
@@ -299,6 +305,14 @@ count_terrain_subtypes(void)
 
 void
 final_init_world(void)
+{
+    compute_elevation_bounds();
+    /* Calculate extent and position of geographical features. */
+    compute_all_feature_centroids();
+}
+
+void
+compute_elevation_bounds(void)
 {
     int x, y, el, count, sum, rowcount, rowsum, rowavg;
 
@@ -329,8 +343,6 @@ final_init_world(void)
 	Dprintf("Area elevations range %d to %d, averaging %d\n",
 		area.minelev, area.maxelev, area.avgelev);
     }
-    /* Calculate extent and position of geographical features. */
-    compute_all_feature_centroids();
 }
 
 /* Make space for an area's terrain, and for the unit cache. */
@@ -512,108 +524,94 @@ allocate_area_users(void)
 {
     if (!user_defined()) {
 	check_area_shape();
-	area.user = malloc_area_layer(short);
+	area.user = malloc_area_layer(int);
 	/* NOUSER != 0, so need to blast it over the layer. Note:
 	   sizeof(short) important, since memset by default assumes
 	   char, and only half the layer get blasted. */
-	memset(area.user, NOUSER, area.width * area.height * sizeof(short));
+	memset(area.user, NOUSER, area.width * area.height * sizeof(int));
     }
 }
 
 int
-fn_terrain_at(x, y)
-int x, y;
+fn_terrain_at(int x, int y)
 {
     return terrain_at(x, y);
 }
 
 int
-fn_aux_terrain_at(x, y)
-int x, y;
+fn_aux_terrain_at(int x, int y)
 {
     return aux_terrain_at(x, y, tmpttype);
 }
 
 int
-fn_feature_at(x, y)
-int x, y;
+fn_feature_at(int x, int y)
 {
     return raw_feature_at(x, y);
 }
 
 int
-fn_elevation_at(x, y)
-int x, y;
+fn_elevation_at(int x, int y)
 {
     return elev_at(x, y);
 }
 
 int
-fn_people_side_at(x, y)
-int x, y;
+fn_people_side_at(int x, int y)
 {
     return people_side_at(x, y);
 }
 
 int
-fn_control_side_at(x, y)
-int x, y;
+fn_control_side_at(int x, int y)
 {
     return control_side_at(x, y);
 }
 
 int
-fn_user_at(x, y)
-int x, y;
+fn_user_at(int x, int y)
 {
     return user_at(x, y);
 }
 
 int
-fn_material_at(x, y)
-int x, y;
+fn_material_at(int x, int y)
 {
     return material_at(x, y, tmpmtype);
 }
 
 int
-fn_temperature_at(x, y)
-int x, y;
+fn_temperature_at(int x, int y)
 {
     return temperature_at(x, y);
 }
 
 int
-fn_raw_cloud_at(x, y)
-int x, y;
+fn_raw_cloud_at(int x, int y)
 {
     return raw_cloud_at(x, y);
 }
 
 int
-fn_raw_cloud_bottom_at(x, y)
-int x, y;
+fn_raw_cloud_bottom_at(int x, int y)
 {
     return raw_cloud_bottom_at(x, y);
 }
 
 int
-fn_raw_cloud_height_at(x, y)
-int x, y;
+fn_raw_cloud_height_at(int x, int y)
 {
     return raw_cloud_height_at(x, y);
 }
 
 int
-fn_raw_wind_at(x, y)
-int x, y;
+fn_raw_wind_at(int x, int y)
 {
     return raw_wind_at(x, y);
 }
 
 void
-fn_set_terrain_at(x, y, val)
-int x, y, val;
+fn_set_terrain_at(int x, int y, int val)
 {
     extern int warnbadterrain, numbadterrain;
 
@@ -631,8 +629,7 @@ int x, y, val;
 }
 
 void
-fn_set_aux_terrain_at(x, y, val)
-int x, y, val;
+fn_set_aux_terrain_at(int x, int y, int val)
 {
     /* Filter anything but the basic six bits. */
     val &= 0x3f;
@@ -640,78 +637,67 @@ int x, y, val;
 }
 
 void
-fn_set_people_side_at(x, y, val)
-int x, y, val;
+fn_set_people_side_at(int x, int y, int val)
 {
     set_people_side_at(x, y, val);
 }
 
 void
-fn_set_control_side_at(x, y, val)
-int x, y, val;
+fn_set_control_side_at(int x, int y, int val)
 {
     set_control_side_at(x, y, val);
 }
 
 void
-fn_set_user_at(x, y, val)
-int x, y, val;
+fn_set_user_at(int x, int y, int val)
 {
     set_user_at(x, y, val);
 }
 
 void
-fn_set_raw_feature_at(x, y, val)
-int x, y, val;
+fn_set_raw_feature_at(int x, int y, int val)
 {
     set_raw_feature_at(x, y, val);
 }
 
 void
-fn_set_elevation_at(x, y, val)
-int x, y, val;
+fn_set_elevation_at(int x, int y, int val)
 {
     set_elev_at(x, y, val);
 }
 
 void
-fn_set_material_at(x, y, val)
-int x, y, val;
+fn_set_material_at(int x, int y, int val)
 {
     set_material_at(x, y, tmpmtype, val);
 }
 
 void
-fn_set_temperature_at(x, y, val)
-int x, y, val;
+fn_set_temperature_at(int x, int y, int val)
 {
     set_temperature_at(x, y, val);
 }
 
 void
-fn_set_raw_wind_at(x, y, val)
-int x, y, val;
+fn_set_raw_wind_at(int x, int y, int val)
 {
     set_raw_wind_at(x, y, val);
 }
 
 void
-fn_set_raw_cloud_at(x, y, val)
-int x, y, val;
+fn_set_raw_cloud_at(int x, int y, int val)
 {
     set_raw_cloud_at(x, y, val);
 }
 
 void
-fn_set_raw_cloud_bottom_at(x, y, val)
-int x, y, val;
+fn_set_raw_cloud_bottom_at(int x, int y, int val)
 {
     set_raw_cloud_bottom_at(x, y, val);
 }
 
 void
-fn_set_raw_cloud_height_at(x, y, val)
-int x, y, val;
+fn_set_raw_cloud_height_at(int x, int y, int val)
 {
     set_raw_cloud_height_at(x, y, val);
 }
@@ -720,8 +706,7 @@ int x, y, val;
    type of terrain, and update everything affected by the change. */
 
 void
-change_terrain_type(x, y, t2)
-int x, y, t2;
+change_terrain_type(int x, int y, int t2)
 {
     int t, m, curview, newview, update, u2, nu2;
     HistEventType hevttype;
@@ -820,8 +805,8 @@ int x, y, t2;
 /* This needs to be changed to understand different world shapes. */
 
 int
-search_around(x0, y0, maxdist, pred, rxp, ryp, incr)
-int x0, y0, maxdist, (*pred)(int, int), *rxp, *ryp, incr;
+search_around(int x0, int y0, int maxdist, int (*pred)(int, int),
+	      int *rxp, int *ryp, int incr)
 {
     int clockwise, dist, dd, d, dir, x1, y1, i, dir2, x, y, xw;
 
@@ -849,10 +834,8 @@ int x0, y0, maxdist, (*pred)(int, int), *rxp, *ryp, incr;
 }
 
 int
-search_and_apply(x0, y0, maxdist, pred, rxp, ryp, incr, fn, num)
-int x0, y0, maxdist, *rxp, *ryp, incr, num;
-int (*pred)(int, int);
-void (*fn)(int, int);
+search_and_apply(int x0, int y0, int maxdist, int (*pred)(int, int),
+		 int *rxp, int *ryp, int incr, void (*fn)(int, int), int num)
 {
     int clockwise, dist, x0w, dd, d, dir, x1, y1, i, dir2, x, y, xw;
 
@@ -896,18 +879,17 @@ void (*fn)(int, int);
     return 0;
 }
 
-/* Apply a function to every cell within the given radius, being careful (for
-   both safety and efficiency reasons) not to go past edges.  Note that the
-   distance is inclusive, and that distance of 0 means x0,y0 only.  Also,
-   if the distance is greater than either map dimension, this routine still
-   operates on a correct intersection with the area.  */
+/* Apply a function to every cell within the given radius, being
+   careful (for both safety and efficiency reasons) not to go past
+   edges.  Note that the distance is inclusive, and that distance of 0
+   means x0,y0 only.  Also, if the distance is greater than either map
+   dimension, this routine still operates on a correct intersection
+   with the area.  */
 
 int stop_apply;
 
 void
-apply_to_area(x0, y0, dist, fn)
-int x0, y0, dist;
-void (*fn)(int, int);
+apply_to_area(int x0, int y0, int dist, void (*fn)(int, int))
 {
     int x, y, x1, y1, x2, y2;
 
@@ -933,10 +915,10 @@ void (*fn)(int, int);
     }
 }
 
+/* Similarly, but operating on edge cells as well. */
+
 void
-apply_to_area_plus_edge(x0, y0, dist, fn)
-int x0, y0, dist;
-void (*fn)(int, int);
+apply_to_area_plus_edge(int x0, int y0, int dist, void (*fn)(int, int))
 {
     int x, y, x1, y1, x2, y2;
 
@@ -959,9 +941,7 @@ void (*fn)(int, int);
 }
 
 void
-apply_to_ring(x0, y0, distmin, distmax, fn)
-int x0, y0, distmin, distmax;
-void (*fn)(int, int);
+apply_to_ring(int x0, int y0, int distmin, int distmax, void (*fn)(int, int))
 {
     int dist, x, y, x1, y1, x2, y2;
 
@@ -986,9 +966,7 @@ void (*fn)(int, int);
 /* Apply the function to the hexagon bounded by w,h. */
 
 void
-apply_to_hexagon(x0, y0, w2, h2, fn)
-int x0, y0, w2, h2;
-void (*fn)(int, int);
+apply_to_hexagon(int x0, int y0, int w2, int h2, void (*fn)(int, int))
 {
     int x, y, x1, y1, x2, y2;
 
@@ -1160,8 +1138,8 @@ Waypoint *waypoints;
 #endif
 
 int
-search_straight_line(x0, y0, x1, y1, pred, rxp, ryp)
-int x0, y0, x1, y1, (*pred)(int, int), *rxp, *ryp;
+search_straight_line(int x0, int y0, int x1, int y1, int (*pred)(int, int),
+		     int *rxp, int *ryp)
 {
     int i, dist, x, y, xw;
 
@@ -1499,8 +1477,7 @@ compute_feature_centroid(Feature *feature)
 }
 
 int
-num_people_at(x, y)
-int x, y;
+num_people_at(int x, int y)
 {
     int m, num;
 
@@ -1516,8 +1493,7 @@ int x, y;
 /* Compute the coords of a point in the given direction. */
 
 int
-point_in_dir(x, y, dir, xp, yp)
-int x, y, dir, *xp, *yp;
+point_in_dir(int x, int y, int dir, int *xp, int *yp)
 {
     *xp = wrapx(x + dirx[dir]);  *yp = y + diry[dir];
     return (in_area(*xp, *yp));
@@ -1527,8 +1503,7 @@ int x, y, dir, *xp, *yp;
    points not inside the area. */
 
 int
-interior_point_in_dir(x, y, dir, xp, yp)
-int x, y, dir, *xp, *yp;
+interior_point_in_dir(int x, int y, int dir, int *xp, int *yp)
 {
     *xp = wrapx(x + dirx[dir]);  *yp = y + diry[dir];
     return (inside_area(*xp, *yp));
@@ -1537,16 +1512,14 @@ int x, y, dir, *xp, *yp;
 /* Compute the point in a direction, n cells away. */
 
 int
-point_in_dir_n(x, y, dir, n, xp, yp)
-int x, y, dir, n, *xp, *yp;
+point_in_dir_n(int x, int y, int dir, int n, int *xp, int *yp)
 {
     *xp = wrapx(x + n * dirx[dir]);  *yp = y + n * diry[dir];
     return (in_area(*xp, *yp));
 }
 
 int
-interior_point_in_dir_n(x, y, dir, n, xp, yp)
-int x, y, dir, n, *xp, *yp;
+interior_point_in_dir_n(int x, int y, int dir, int n, int *xp, int *yp)
 {
     *xp = wrapx(x + n * dirx[dir]);  *yp = y + n * diry[dir];
     return (inside_area(*xp, *yp));
@@ -1900,16 +1873,13 @@ void add_neighbor(TRegion *reg1, TRegion *reg2);
 int region_value(int x, int y);
 
 int
-region_value(x, y)
-int x, y;
+region_value(int x, int y)
 {
     return (t_liquid(terrain_at(x, y)) ? 0 : 1);
 }
 
 void
-divide_into_regions(tlayer, rlayer)
-char *tlayer;
-TRegion **rlayer;
+divide_into_regions(char *tlayer, TRegion **rlayer)
 {
     int x, y, dir, x1, y1;
     TRegion *region, *reg1, *reg2;
@@ -1995,8 +1965,7 @@ TRegion **rlayer;
 }
 
 void
-add_neighbor(reg1, reg2)
-TRegion *reg1, *reg2;
+add_neighbor(TRegion *reg1, TRegion *reg2)
 {
     TRegionLink *link;
 
@@ -2026,6 +1995,8 @@ static void paint_material_1(int x, int y);
 static void paint_clouds_1(int x, int y);
 static void paint_winds_1(int x, int y);
 
+extern void fix_elevations(void);
+
 /* Cell painting. */
 
 void
@@ -2197,28 +2168,46 @@ paint_feature_1(int x, int y)
 /* Painting of terrain elevations. */
 
 void
-paint_elevation(Side *side, int x, int y, int r, int elev)
+paint_elevation(Side *side, int x, int y, int r, int code, int elev, int vary)
 {
     allocate_area_elevations();
     tmpside = side;
-    tmpint = elev;
+    tmpint = code;
+    tmpint2 = elev;
+    tmpint3 = vary;
     apply_to_area_plus_edge(x, y, r, paint_elevation_1);
 }
 
 static void
 paint_elevation_1(int x, int y)
 {
-    int n, t = terrain_at(x, y), oldelev = elev_at(x, y);
+    int oldelev = elev_at(x, y), newelev;
 
-    /* Clip desired elevation to what's allowed for the terrain here. */
-    n = max(t_elev_min(t), min(tmpint, t_elev_max(t)));
-    if (n != oldelev) {
-	set_elev_at(x, y, n);
-	if (n < area.minelev)
-	  area.minelev = n;
-	if (n > area.maxelev)
-	  area.maxelev = n;
-	/* (should force recomputation of contour intervals) */
+    /* Decide whether we're setting or adjusting the elevation. */
+    switch (tmpint) {
+      case 0:
+	newelev = tmpint2;
+	break;
+      case 1:
+	newelev = oldelev + tmpint2;
+	break;
+      default:
+	run_warning("bogus elevation paint code");
+	newelev = 0;
+	break;
+    }
+    /* Add an optional random variation if desired. */
+    if (tmpint3 > 0)
+      newelev += xrandom(tmpint3 * 2 + 1) - tmpint3;
+    if (newelev != oldelev) {
+	set_elev_at(x, y, newelev);
+	if (newelev < area.minelev)
+	  area.minelev = newelev;
+	if (newelev > area.maxelev)
+	  area.maxelev = newelev;
+	/* Note that the contour intervals may now no longer be
+	   reasonable, but since redoing them will probably require a
+	   full map update, don't adjust them automatically here. */
 	update_cell_display(tmpside, x, y, UPDATE_ALWAYS | UPDATE_ADJ);
     }
 }
@@ -2329,4 +2318,42 @@ paint_winds_1(int x, int y)
     }
 }
 
+/* Scan the elevation layer, putting all values into the ranges
+   required by terrain types. */
+
+void
+fix_elevations(void)
+{
+    int x, y, t, oldelev, newelev, numfixed;
+    Side *side;
+
+    if (!elevations_defined())
+      return;
+    numfixed = 0;
+    area.maxelev = area.minelev = elev_at(area.width / 2, area.height / 2);
+    for_all_interior_cells(x, y) {
+	t = terrain_at(x, y);
+	oldelev = elev_at(x, y);
+	/* Clip desired elevation to what's allowed for the terrain here. */
+	newelev = max(t_elev_min(t), min(oldelev, t_elev_max(t)));
+	set_elev_at(x, y, newelev);
+	/* (should try to share with final_init_world?) */
+	if (newelev < area.minelev)
+	  area.minelev = newelev;
+	if (newelev > area.maxelev)
+	  area.maxelev = newelev;
+	if (newelev != oldelev)
+	  ++numfixed;
+    }
+    /* Set edge elevations to be averages of adjacent interior
+       cells. */
+    set_edge_terrain(FALSE);
+    /* Let everybody know what we did. */
+    notify_all("Fixed %d elevations, range from %d to %d.",
+	       numfixed, area.minelev, area.maxelev);
+    if (numfixed > 0)
+      for_all_sides(side)
+	update_area_display(side);
+}
+
 #endif /* DESIGNERS */
