@@ -1,5 +1,5 @@
 /* Definitions relating to worlds and areas in Xconq.
-   Copyright (C) 1987-1989, 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -62,6 +62,8 @@ typedef struct a_area {
     short *tmp1;
     short *tmp2;
     short *tmp3;
+    struct a_region **regions;
+    int *user;			/* IDs of units using materials in each cell */
 } Area;
 
 /* Named geographical features. */
@@ -95,6 +97,23 @@ typedef struct a_path {
     Waypoint wps[100];
 } Path;
 
+/* Regions. */
+
+typedef struct a_region {
+    int ttype;
+    int size;
+    short xmin, ymin;
+    short xmax, ymax;
+    struct a_region *merge;
+    struct a_region_link *neighbors;
+    struct a_region *next;
+} TRegion;
+
+typedef struct a_region_link {
+    struct a_region *neighbor;
+    struct a_region_link *next;
+} TRegionLink;
+
 /* Use this macro to get a area-spanning layer of the given type. */
 
 #define malloc_area_layer(TYPE)  \
@@ -111,6 +130,34 @@ typedef struct a_path {
 
 #define aadd(m,x,y,v) ((m)[area.width * (y) + (x)] += (v))
 
+/* Advanced unit support. */
+
+#define user_defined() (area.user != NULL)
+
+#define user_at(x, y) aref(area.user, x, y)
+
+#define set_user_at(x, y, u) aset(area.user, x, y, u)
+
+/* Test if cell at (x1, y1) is within n steps from (x, y) */
+#define cell_is_within_range(x, y, n, x1, y1) \
+	( (x) - (x1) > (n) || (x1) - (x) > (n) || (y) - (y1) > (n) \
+	|| (y1) - (y) > (n) || (x) + (y) - (x1) - (y1) > (n) \
+	|| (x1) + (y1) - (x) - (y) > (n) ) ? FALSE : TRUE
+
+/* Test if cell at (x1, y1) is within reach of unit u */
+#define cell_is_within_reach(u, x1, y1) \
+	cell_is_within_range((u)->x, (u)->y, (u)->reach, (x1), (y1))
+
+/* Repeat for all cells at (x1, y1) within n steps from (x, y) */
+#define for_all_cells_within_range(x, y, n, x1, y1) \
+	for((x1) = (x) - (n); (x1) <= (x) + (n); (x1)++) \
+	for((y1) = max((y) - (n), (y) - (n) + (x) - (x1)); \
+	       (y1) <= min((y) + (n), (y) + (n) + (x) - (x1)); (y1)++)
+
+/* Repeat for all cells at (x1, y1) within reach of unit u */
+#define for_all_cells_within_reach(u, x1, y1) \
+	for_all_cells_within_range((u)->x, (u)->y, (u)->reach, (x1), (y1))
+
 /* The unit is a raw pointer - this macro is used a *lot*.  This could
    be space-optimized by using a 16-bit unit id. */
 
@@ -123,6 +170,33 @@ typedef struct a_path {
 #define for_all_stack(x,y,var)  \
   for ((var) = unit_at((x), (y)); (var) != NULL;  (var) = (var)->nexthere)
 
+/* Iterate through all units in this cell including occs within occs
+   within occs. */
+
+/* This nifty little macro will climb the occupant : nexthere tree and
+   follow all branches four levels down to find all the occs within
+   occs within occs within units in the stack. */
+
+#define for_all_stack_with_occs(x,y,var)  \
+  for ((var) = unit_at((x), (y)); \
+          (var) != NULL; \
+  	 (var) = ((var)->occupant != NULL ? \
+  	 	        (var)->occupant : \
+  		      ((var)->nexthere != NULL ? \
+  		        (var)->nexthere : \
+  		      ((var)->transport != NULL && \
+  		        (var)->transport->nexthere != NULL ? \
+  		        (var)->transport->nexthere : \
+  		      ((var)->transport != NULL && \
+  		        (var)->transport->transport != NULL && \
+  		        (var)->transport->transport->nexthere != NULL ? \
+  		        (var)->transport->transport->nexthere : \
+  		      ((var)->transport != NULL && \
+  		        (var)->transport->transport != NULL && \
+  		        (var)->transport->transport->transport != NULL && \
+  		        (var)->transport->transport->transport->nexthere != NULL ? \
+  		        (var)->transport->transport->transport->nexthere : NULL))))))
+
 /* Test if the terrain has been allocated yet. */
 
 #define terrain_defined() (area.terrain != NULL)
@@ -256,9 +330,13 @@ typedef struct a_path {
 
 #define set_raw_wind_at(x,y,v) aset(area.winds, x, y, v)
 
-#define wind_dir_at(x,y) (raw_wind_at(x, y) & 0x07)
+#define wind_dir(w) ((w) & 0x07)
 
-#define wind_force_at(x,y) (raw_wind_at(x, y) >> 3)
+#define wind_force(w) ((w) >> 3)
+
+#define wind_dir_at(x,y) (wind_dir(raw_wind_at(x, y)))
+
+#define wind_force_at(x,y) (wind_force(raw_wind_at(x, y)))
 
 #define set_wind_at(x,y,d,f) (set_raw_wind_at(x, y, ((f) << 3) | (d)))
 
@@ -365,6 +443,8 @@ extern int numbordtypes;
 extern int numconntypes;
 extern int numcoattypes;
 
+extern int numfeatures;
+
 extern int minelev;
 extern int maxelev;
 extern int mintemp;
@@ -373,6 +453,7 @@ extern int minwindforce;
 extern int maxwindforce;
 
 extern int any_materials_in_terrain;
+extern int any_elev_variation;
 extern int any_temp_variation;
 extern int any_temp_variation_in_layer;
 extern int any_wind_variation;
@@ -381,115 +462,115 @@ extern int any_clouds;
 
 /* World-related functions. */
 
-extern void init_world PARAMS ((void));
-extern int set_world_circumference PARAMS ((int circum, int warn));
-extern int set_area_shape PARAMS ((int width, int height, int warn));
-extern int valid_area_shape PARAMS ((int width, int height, int warn));
-extern void check_area_shape PARAMS ((void));
-extern void calculate_world_globals PARAMS ((void));
-extern void count_terrain_subtypes PARAMS ((void));
-extern void final_init_world PARAMS ((void));
+extern void init_world(void);
+extern int set_world_circumference(int circum, int warn);
+extern int set_area_shape(int width, int height, int warn);
+extern int valid_area_shape(int width, int height, int warn);
+extern void check_area_shape(void);
+extern void calculate_world_globals(void);
+extern void count_terrain_subtypes(void);
+extern void final_init_world(void);
+extern void compute_elevation_bounds(void);
 
-extern void allocate_area_terrain PARAMS ((void));
-extern void allocate_area_aux_terrain PARAMS ((int t));
-extern void allocate_area_scratch PARAMS ((int n));
-extern void allocate_area_elevations PARAMS ((void));
-extern void allocate_area_temperatures PARAMS ((void));
-extern void allocate_area_people_sides PARAMS ((void));
-extern void allocate_area_control_sides PARAMS ((void));
-extern void allocate_area_material PARAMS ((int m));
-extern void allocate_area_clouds PARAMS ((void));
-extern void allocate_area_cloud_altitudes PARAMS ((void));
-extern void allocate_area_cloud_bottoms PARAMS ((void));
-extern void allocate_area_cloud_heights PARAMS ((void));
-extern void allocate_area_winds PARAMS ((void));
+extern void allocate_area_terrain(void);
+extern void allocate_area_aux_terrain(int t);
+extern void allocate_area_scratch(int n);
+extern void allocate_area_elevations(void);
+extern void allocate_area_temperatures(void);
+extern void allocate_area_people_sides(void);
+extern void allocate_area_control_sides(void);
+extern void allocate_area_material(int m);
+extern void allocate_area_clouds(void);
+extern void allocate_area_cloud_altitudes(void);
+extern void allocate_area_cloud_bottoms(void);
+extern void allocate_area_cloud_heights(void);
+extern void allocate_area_winds(void);
 
-extern int fn_terrain_at PARAMS ((int x, int y));
-extern int fn_aux_terrain_at PARAMS ((int x, int y));
-extern int fn_feature_at PARAMS ((int x, int y));
-extern int fn_elevation_at PARAMS ((int x, int y));
-extern int fn_people_side_at PARAMS ((int x, int y));
-extern int fn_control_side_at PARAMS ((int x, int y));
-extern int fn_material_at PARAMS ((int x, int y));
-extern int fn_temperature_at PARAMS ((int x, int y));
-extern int fn_raw_cloud_at PARAMS ((int x, int y));
-extern int fn_raw_cloud_bottom_at PARAMS ((int x, int y));
-extern int fn_raw_cloud_height_at PARAMS ((int x, int y));
-extern int fn_raw_wind_at PARAMS ((int x, int y));
+extern int fn_terrain_at(int x, int y);
+extern int fn_aux_terrain_at(int x, int y);
+extern int fn_feature_at(int x, int y);
+extern int fn_elevation_at(int x, int y);
+extern int fn_people_side_at(int x, int y);
+extern int fn_control_side_at(int x, int y);
+extern int fn_material_at(int x, int y);
+extern int fn_temperature_at(int x, int y);
+extern int fn_raw_cloud_at(int x, int y);
+extern int fn_raw_cloud_bottom_at(int x, int y);
+extern int fn_raw_cloud_height_at(int x, int y);
+extern int fn_raw_wind_at(int x, int y);
+extern void allocate_area_users(void);
+extern int fn_user_at(int x, int y);
+extern void fn_set_user_at(int x, int y, int val);
+extern void fn_set_terrain_at(int x, int y, int val);
+extern void fn_set_aux_terrain_at(int x, int y, int val);
+extern void fn_set_people_side_at(int x, int y, int val);
+extern void fn_set_control_side_at(int x, int y, int val);
+extern void fn_set_raw_feature_at(int x, int y, int val);
+extern void fn_set_elevation_at(int x, int y, int val);
+extern void fn_set_material_at(int x, int y, int val);
+extern void fn_set_temperature_at(int x, int y, int val);
+extern void fn_set_raw_wind_at(int x, int y, int val);
+extern void fn_set_raw_cloud_at(int x, int y, int val);
+extern void fn_set_raw_cloud_bottom_at(int x, int y, int val);
+extern void fn_set_raw_cloud_height_at(int x, int y, int val);
 
-extern void fn_set_terrain_at PARAMS ((int x, int y, int val));
-extern void fn_set_aux_terrain_at PARAMS ((int x, int y, int val));
-extern void fn_set_people_side_at PARAMS ((int x, int y, int val));
-extern void fn_set_control_side_at PARAMS ((int x, int y, int val));
-extern void fn_set_raw_feature_at PARAMS ((int x, int y, int val));
-extern void fn_set_elevation_at PARAMS ((int x, int y, int val));
-extern void fn_set_material_at PARAMS ((int x, int y, int val));
-extern void fn_set_temperature_at PARAMS ((int x, int y, int val));
-extern void fn_set_raw_wind_at PARAMS ((int x, int y, int val));
-extern void fn_set_raw_cloud_at PARAMS ((int x, int y, int val));
-extern void fn_set_raw_cloud_bottom_at PARAMS ((int x, int y, int val));
-extern void fn_set_raw_cloud_height_at PARAMS ((int x, int y, int val));
+extern void change_terrain_type(int x, int y, int t2);
 
-extern void change_terrain_type PARAMS ((int x, int y, int t2));
-
-extern int search_around PARAMS ((int x0, int y0, int maxdist,
-				 int (*pred)(int, int),
-				 int *rxp, int *ryp, int incr));
-extern int search_and_apply PARAMS ((int x0, int y0, int maxdist,
-				    int (*pred)(int, int),
-				    int *rxp, int *ryp, int incr,
-				    void (*fn)(int, int), int num));
-extern void apply_to_area PARAMS ((int x0, int y0, int dist,
-				  void (*fn)(int, int)));
-extern void apply_to_area_plus_edge PARAMS ((int x0, int y0, int dist,
-					    void (*fn)(int, int)));
-extern void apply_to_ring PARAMS ((int x0, int y0, int distmin, int distmax,
-				  void (*fn)(int, int)));
-extern void apply_to_hexagon PARAMS ((int x0, int y0, int w2, int h2,
-				     void (*fn)(int, int)));
-extern void apply_to_path PARAMS ((int fx, int fy, int tx, int ty,
-				  int (*dirtest)(int x, int y, int dir),
-				  int (*dirsort)(int x, int y, int *dirchoices, int numchoices),
-				  int (*fn)(int x, int y, int dir, int j, int numchoices),
-				  int shortest));
+extern int search_around(int x0, int y0, int maxdist, int (*pred)(int, int),
+			 int *rxp, int *ryp, int incr);
+extern int search_and_apply(int x0, int y0, int maxdist, int (*pred)(int, int),
+			    int *rxp, int *ryp, int incr,
+			    void (*fn)(int, int), int num);
+extern void apply_to_area(int x0, int y0, int dist, void (*fn)(int, int));
+extern void apply_to_area_plus_edge(int x0, int y0, int dist,
+				    void (*fn)(int, int));
+extern void apply_to_ring(int x0, int y0, int distmin, int distmax,
+			  void (*fn)(int, int));
+extern void apply_to_hexagon(int x0, int y0, int w2, int h2,
+			     void (*fn)(int, int));
+extern void apply_to_path(int fx, int fy, int tx, int ty,
+			  int (*dirtest)(int x, int y, int dir),
+			  int (*dirsort)(int x, int y, int *dirchoices,
+					 int numchoices),
+			  int (*fn)(int x, int y, int dir, int j,
+				    int numchoices),
+			  int shortest);
 #if 0
-int find_path PARAMS ((int fx, int fy, int tx, int ty,
-		      int (*chooser)(int, int, int, int, int *),
-		      int maxwps, Waypoint *waypoints, int *numwpsp));
+int find_path(int fx, int fy, int tx, int ty,
+	      int (*chooser)(int, int, int, int, int *),
+	      int maxwps, Waypoint *waypoints, int *numwpsp);
 #endif
-extern int search_straight_line PARAMS ((int x0, int y0, int x1, int y1,
-					 int (*pred)(int, int),
-					 int *rxp, int *ryp));
+extern int search_straight_line(int x0, int y0, int x1, int y1,
+				int (*pred)(int, int), int *rxp, int *ryp);
 
-extern void set_border_at PARAMS ((int x, int y, int dir, int t, int onoff));
-extern void set_connection_at PARAMS ((int x, int y, int dir, int t, int onoff));
-extern void patch_linear_terrain PARAMS ((int t));
+extern void set_border_at(int x, int y, int dir, int t, int onoff);
+extern void set_connection_at(int x, int y, int dir, int t, int onoff);
+extern void patch_linear_terrain(int t);
 
-extern void init_features PARAMS ((void));
-extern Feature *find_feature PARAMS ((int fid));
-extern Feature *feature_at PARAMS ((int x, int y));
-extern void compute_all_feature_centroids PARAMS ((void));
-extern void compute_feature_centroid PARAMS ((Feature *feature));
+extern void init_features(void);
+extern Feature *find_feature(int fid);
+extern Feature *feature_at(int x, int y);
+extern void compute_all_feature_centroids(void);
+extern void compute_feature_centroid(Feature *feature);
 
-extern int point_in_dir PARAMS ((int x, int y, int dir, int *xp, int *yp));
-extern int interior_point_in_dir PARAMS ((int x, int y, int dir,
-					  int *xp, int *yp));
-extern int point_in_dir_n PARAMS ((int x, int y, int dir, int n,
-				   int *xp, int *yp));
-extern int interior_point_in_dir_n PARAMS ((int x, int y, int dir, int n,
-					    int *xp, int *yp));
-extern int random_point PARAMS ((int *xp, int *yp));
-extern int random_edge_point PARAMS ((int *xp, int *yp));
-extern int random_point_near PARAMS ((int cx, int cy, int radius,
-				      int *xp, int *yp));
-extern int random_point_in_area PARAMS ((int cx, int cy, int rx, int ry,
-					 int *xp, int *yp));
-extern void terrain_subtype_warning PARAMS ((char *context, int t));
-extern int approx_dir PARAMS ((int dx, int dy));
-extern int hextant PARAMS ((int dx, int dy));
-extern int distance PARAMS ((int x1, int y1, int x2, int y2));
-extern int world_distance PARAMS ((int x1, int y1, int x2, int y2));
-extern int closest_dir PARAMS ((int x, int y));
+extern int point_in_dir(int x, int y, int dir, int *xp, int *yp);
+extern int interior_point_in_dir(int x, int y, int dir, int *xp, int *yp);
+extern int point_in_dir_n(int x, int y, int dir, int n, int *xp, int *yp);
+extern int interior_point_in_dir_n(int x, int y, int dir, int n,
+				   int *xp, int *yp);
+extern int random_point(int *xp, int *yp);
+extern int random_edge_point(int *xp, int *yp);
+extern int random_point_near(int cx, int cy, int radius, int *xp, int *yp);
+extern int random_point_in_area(int cx, int cy, int rx, int ry,
+				int *xp, int *yp);
+extern void terrain_subtype_warning(char *context, int t);
+extern int approx_dir(int dx, int dy);
+extern int hextant(int dx, int dy);
+extern int distance(int x1, int y1, int x2, int y2);
+extern int world_distance(int x1, int y1, int x2, int y2);
+extern int closest_dir(int x, int y);
 
-extern void xy_to_latlong PARAMS ((int x, int y, int xf, int yf, int *latp, int *lonp));
-extern int latlong_to_xy PARAMS ((int lat, int lon, int *xp, int *yp, int *xfp, int *yfp));
+extern void xy_to_latlong(int x, int y, int xf, int yf, int *latp, int *lonp);
+extern int latlong_to_xy(int lat, int lon, int *xp, int *yp,
+			 int *xfp, int *yfp);
+extern int num_people_at(int x, int y);
