@@ -7,15 +7,11 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern char *escaped_symbol(char *str);
-extern char *escaped_string(char *str);
-extern void add_to_packet(char *str);
-extern void flush_write(void);
-#include "ai.h"
+#include "kernel.h"
 #include "imf.h"
+
 extern ImageFamily **recorded_imfs;
 extern int num_recorded_imfs;
-
 extern int debugging_state_sync;
 
 #define key(x) (keyword_name(x))
@@ -53,8 +49,6 @@ static int fn_terrain_view(int x, int y);
 static int fn_aux_terrain_view(int x, int y);
 static int fn_aux_terrain_view_date(int x, int y);
 static int fn_terrain_view_date(int x, int y);
-static int fn_unit_view(int x, int y);
-static int fn_unit_view_date(int x, int y);
 static int fn_material_view(int x, int y);
 static int fn_material_view_date(int x, int y);
 static int fn_temp_view(int x, int y);
@@ -120,6 +114,7 @@ static void write_side_value_list(char *name, short *arr,
 				  int dflt, int addnewline);
 static void write_utype_string_list(char *name, char **arr,
 				    char *dflt, int addnewline);
+static char *shortest_escaped_name(int u);
 
 /* The pointer to the file being written to. */
 
@@ -176,6 +171,7 @@ write_entire_game_state(char *fname)
     module->variants = NULL;
     /* We write all types etc, so suppress base module load attempts. */
     module->basemodulename = NULL;
+    module->defaultbasemodulename = NULL;
     module->compress_tables = TRUE;
     module->compress_layers = TRUE;
     module->def_all = TRUE;
@@ -229,8 +225,6 @@ write_game_module(Module *module)
 			   "", FALSE, TRUE);
 	    write_str_prop(key(K_VERSION), module->version,
 			   "", FALSE, TRUE);
-	    write_str_prop(key(K_PROGRAM_VERSION), version_string(),
-			   "", FALSE, TRUE);
 	    if (module->variants) {
 		space_form();
 		start_form(key(K_VARIANTS));
@@ -690,8 +684,9 @@ write_table(char *name, int (*getter)(int, int), int dflt, int typ1, int typ2,
 			qsort(mostcommon, next, sizeof(struct histo),
 			      histo_compare);
 			if (mostcommon[0].count >= (3 * dim2) / 4) {
-			    /* The most common value in this row(s) is not the only value,
-			       but it is worth writing into a separate clause. */
+			    /* The most common value in this row(s) is
+			       not the only value, but it is worth
+			       writing into a separate clause. */
 			    writeconst = TRUE;
 			    for (j = 0; j < dim2; ++j) {
 				/* Flag the other values as needing to be
@@ -709,9 +704,10 @@ write_table(char *name, int (*getter)(int, int), int dflt, int typ1, int typ2,
 			    }
 			}
 		    }
-		    /* Write out the most common value (if non-default) in the row(s),
-		       expressing it with a clause that applies the value
-		       to the entire row(s). */
+		    /* Write out the most common value (if
+		       non-default) in the row(s), expressing it with
+		       a clause that applies the value to the entire
+		       row(s). */
 		    if (writeconst && mostcommon[0].val != dflt) {
 			newline_form();
 			space_form();
@@ -1041,7 +1037,7 @@ write_area_elevations(int compress)
 	end_form();
     }
     newline_form();
-    write_rle(fn_elevation_at_offset, minelev - area.minelev, maxelev,
+    write_rle(fn_elevation_at_offset, 0, area.maxelev - area.minelev,
 	      NULL, compress);
     space_form();
     space_form();
@@ -1324,7 +1320,8 @@ write_doctrines(void)
 {
     Doctrine *doc;
 
-    for (doc = doctrine_list; doc != NULL; doc = doc->next) {
+    for_all_doctrines(doc) {
+	/* (should filter out doctrines with no non-default values) */
 	start_form(key(K_DOCTRINE));
 	if (doc->name)
 	  add_to_form(escaped_symbol(doc->name));
@@ -1333,6 +1330,7 @@ write_doctrines(void)
 	newline_form();
 	space_form();
 	write_num_prop(key(K_RESUPPLY_PERCENT), doc->resupply_percent,
+		       /* (should get these defaults from a common place) */
 		       50, FALSE, TRUE); 
 	write_num_prop(key(K_REARM_PERCENT), doc->rearm_percent,
 		       20, FALSE, TRUE); 
@@ -1428,17 +1426,16 @@ write_side_properties(Side *side)
     write_num_prop(key(K_PLAYER), (side->player ? side->player->id : 0), 0, FALSE, TRUE);
     write_num_prop(key(K_DEFAULT_DOCTRINE),
     		   (side->default_doctrine ? side->default_doctrine->id : 0), 0, FALSE, TRUE);
-    write_num_prop(key(K_NUMADVANCES), side->numadvances, 0, FALSE, TRUE);
     write_atype_value_list(key(K_ADVANCE), side->advance, 0, TRUE);
     write_mtype_value_list(key(K_TREASURY), side->treasury, 0, TRUE);
     anyudoctrines = FALSE;
     if (side->udoctrine != NULL) {
-      for_all_unit_types(u) {
-	if (side->udoctrine[u] != side->default_doctrine) {
-	    anyudoctrines = TRUE;
-	    break;
+	for_all_unit_types(u) {
+	    if (side->udoctrine[u] != side->default_doctrine) {
+		anyudoctrines = TRUE;
+		break;
+	    }
 	}
-      }
     }
     if (anyudoctrines) {
 	space_form();
@@ -1542,7 +1539,7 @@ write_side_properties(Side *side)
     }
     /* Have the AI paste its useful state into distinct element of
        side->aidata. */
-    if (side_has_ai(side)) {
+    if (side_has_local_ai(side)) {
 	ai_save_state(side);
     }
     write_lisp_prop(key(K_AI_DATA), side->aidata, lispnil, FALSE, TRUE, TRUE);
@@ -1641,18 +1638,6 @@ fn_aux_terrain_view_date(int x, int y)
     return aux_terrain_view_date(tmpside, x, y, tmpttype);
 }
 
-static int
-fn_unit_view(int x, int y)
-{
-    return unit_view(tmpside, x, y);
-}
-
-static int
-fn_unit_view_date(int x, int y)
-{
-    return unit_view_date(tmpside, x, y);
-}
-
 static int
 fn_material_view(int x, int y)
 {
@@ -1716,7 +1701,8 @@ fn_wind_view_date(int x, int y)
 static void
 write_side_view(Side *side, int compress)
 {
-    int t, m;
+    int t, m, x, y;
+    UnitView *uview;
 
     tmpside = side;
     tmpcompress = compress;
@@ -1752,10 +1738,33 @@ write_side_view(Side *side, int compress)
 	    }
 	}
     }
-    if (side->unitview) {
-	write_one_side_view_layer(K_UNIT_VIEW, fn_unit_view);
-	if (!debugging_state_sync)
-	  write_one_side_view_layer(K_UNIT_VIEW_DATES, fn_unit_view_date);
+    /* Write out unit view objects. */
+    if (!side->see_all && side->unit_views != NULL) {
+	newline_form();
+	start_form(key(K_SIDE));
+	add_num_to_form(side->id);
+	space_form();
+	start_form(key(K_UNIT_VIEWS));
+	newline_form();
+	for_all_cells(x, y) {
+	    for_all_view_stack(side, x, y, uview) {
+		/* (should filter out side's own units?) */
+		space_form();
+		space_form();
+		start_form(shortest_escaped_name(uview->type));
+		add_num_to_form(uview->side_id);
+		add_num_to_form(uview->x);
+		add_num_to_form(uview->y);
+		add_num_to_form(uview->id);
+		end_form();
+		newline_form();
+	    }
+	}
+	space_form();
+	space_form();
+	end_form();
+	end_form();
+	newline_form();
     }
     if (!g_see_weather_always()) {
 	if (side->tempview) {
@@ -1795,6 +1804,8 @@ write_one_side_view_layer(int propkey, int (*fn)(int x, int y))
     newline_form();
 }
 
+/* Dump all the players in the game. */
+
 static void
 write_players(void)
 {
@@ -1810,6 +1821,8 @@ write_players(void)
     Dprintf("... Done writing players.\n");
 }
 
+/* Dump everything about a single given player. */
+
 static void
 write_player(Player *player)
 {
@@ -1923,14 +1936,15 @@ write_units(Module *module)
 		  add_num_to_form(side_number(unit->side));
 		write_num_prop(key(K_Z), unit->z, 0, FALSE, FALSE);
 		write_str_prop(key(K_N), unit->name, NULL, FALSE, FALSE);
-		write_num_prop(key(K_OS), side_number(unit->origside), side_number(unit->side), FALSE, FALSE);
+		write_num_prop(key(K_OS), side_number(unit->origside),
+			       side_number(unit->side), FALSE, FALSE);
 		/* Maybe write the unit's id. */
 		if (module->def_all
 		    || module->def_unit_ids
 		    || (unit->occupant
 			&& unit->name == NULL
 			&& unit_symbol(unit) == lispnil))
-		  	write_num_prop(key(K_SHARP), unit->id, 0, FALSE, FALSE);
+		  write_num_prop(key(K_SHARP), unit->id, 0, FALSE, FALSE);
 		/* Need this to get back into the right transport. */
 		if (unit->transport) {
 		    if (unit_symbol(unit->transport) != lispnil
@@ -1988,7 +2002,7 @@ write_unit_properties(Unit *unit)
     write_num_prop(key(K_APPEAR), unit_appear_turn(unit), -1, FALSE, FALSE);
     /* (should do appear x,y also here?) */
     write_num_prop(key(K_DISAPPEAR), unit_disappear_turn(unit), -1, FALSE, FALSE);
-    write_lisp_prop(key(K_X), unit_hook(unit), lispnil, FALSE, TRUE, FALSE);
+    write_lisp_prop(key(K_SIDES), unit_sides(unit), lispnil, FALSE, TRUE, FALSE);
     /* Advanced unit support. */
     if (!u_advanced(unit->type))
       return;
@@ -2205,7 +2219,8 @@ write_historical_event(HistEvent *hevt)
 	    add_num_to_form(hevt->data[i]);
 	    break;
 	  default:
-	    run_warning("'%c' is not a recognized history data desc char", descs[i]);
+	    run_warning("'%c' is not a recognized history data desc char",
+			descs[i]);
 	    break;
 	}
     }
@@ -2271,14 +2286,15 @@ write_images(void)
     }
 }
 
-/* This is a generalized routine to do run-length-encoding of area layers.
-   It uses hook fns to acquire data at a point and an optional translator to
-   do any last-minute fixing.  It can use either a char or numeric encoding,
-   depending on the expected range of values. */
+/* This is a generalized routine to do run-length-encoding of area
+   layers.  It uses hook fns to acquire data at a point and an
+   optional translator to do any last-minute fixing.  It can use
+   either a char or numeric encoding, depending on the expected range
+   of values. */
 
 static void
-write_rle(datafn, lo, hi, translator, compress)
-int (*datafn)(int, int), lo, hi, (*translator)(int), compress;
+write_rle(int (*datafn)(int, int), int lo, int hi, int (*translator)(int),
+	  int compress)
 {
     int width, height, x, y, x0, y0, run, runval, val, trval;
     int numbad = 0;
@@ -2287,7 +2303,8 @@ int (*datafn)(int, int), lo, hi, (*translator)(int), compress;
     if (doreshape) {
 	width = reshaper->final_width;  height = reshaper->final_height;
     }
-    for (y = height-1; y >= 0; --y) {
+    /* Iterate through the rows from the top down. */
+    for (y = height - 1; y >= 0; --y) {
 	space_form();
 	space_form();
 	add_char_to_form('"');
