@@ -3,7 +3,7 @@
 
 // Resources shared by Think C and MPW C versions.
 
-include "XconqProj.rsrc" not 'ckid';
+#include "Xconq.r"
 
 // Need this for MPW only, since Think C synthesizes.
 
