@@ -1,5 +1,5 @@
 /* Program that translates and previews image families for Mac Xconq.
-   Copyright (C) 1992-1997 Stanley T. Shebs.
+   Copyright (C) 1992-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -11,41 +11,72 @@ any later version.  See the file COPYING.  */
 
 #include "config.h"
 #include "misc.h"
-void prealloc_debug(void);
 #include "lisp.h"
+void prealloc_debug(void);
 #include "imf.h"
-extern int emblem_position PARAMS ((Image *uimg, char *ename, Image *eimg, int sw, int sh,
-				    int *exxp, int *eyyp, int *ewp, int *ehp));
+
+#include <stddef.h>
+#include <stdarg.h>
+#include <stdlib.h>
 
 #ifdef THINK_C
 #include <MacHeaders>
-#else /* probably MPW */
+#include <Sound.h>
+#endif /* THINK_C */
+
+#if 1 /*def MPW*/
 #ifndef __MWERKS__ /* actually, "old headers" */
 #include <Values.h>
 #endif
+#ifdef NEW_HEADERS
+#include <MacTypes.h>
+#include <MacWindows.h>
+#else
 #include <Types.h>
-#include <Memory.h>
+#include <Windows.h>
+#endif /* NEW_HEADERS */
 #include <Resources.h>
 #include <QuickDraw.h>
 #include <Fonts.h>
 #include <Events.h>
-#include <Windows.h>
 #include <Menus.h>
+#include <TextEdit.h>
 #include <Dialogs.h>
+#ifndef NEW_HEADERS
 #include <Desk.h>
+#endif
+#include <Scrap.h>
 #include <ToolUtils.h>
+#ifdef NEW_HEADERS
+#include <MacMemory.h>
+#else
+#include <Memory.h>
+#endif
 #include <SegLoad.h>
 #include <Files.h>
 #include <Folders.h>
 #include <OSUtils.h>
+#ifndef __MWERKS__
 #include <OSEvents.h>
+#endif
 #include <DiskInit.h>
 #include <Packages.h>
 #include <Traps.h>
+#include <Lists.h>
 #include <StandardFile.h>
-#endif
+#include <Sound.h>
+#include <Devices.h>
+#endif /* MPW */
 
+#include <QDOffscreen.h>
+
+#ifdef NEW_HEADERS
+#include <Gestalt.h>
+#else
+#include <GestaltEqu.h>
+#endif
 #include <PPCToolbox.h>
+#include <AppleEvents.h>
 
 /* This is a way - perhaps not the best way - to distinguish "old headers"
    from "universal headers". */
@@ -65,7 +96,7 @@ extern int emblem_position PARAMS ((Image *uimg, char *ename, Image *eimg, int s
 
 #include "macimf.h"
 
-extern void syntax_error(Obj *x, char *msg);
+LibraryPath *xconq_libs;
 
 /* Semi-private routines from macimf.c. */
 
@@ -904,7 +935,7 @@ open_imf_dir_file()
 		SetVol(reply.fName, reply.vRefNum);
 		p2c(((char *) reply.fName), filename);
 		SetCursor(*watchcursor);
-		fp = fopen(filename, "r");
+		fp = open_file(filename, "r");
 		if (fp != NULL) {
 			load_image_families(fp, TRUE, imf_callback);
 			fclose(fp);
@@ -1084,7 +1115,7 @@ save_imf_file()
 		SetVol(reply.fName, reply.vRefNum);
 		p2c(((char *) reply.fName), filename);
 		SetCursor(*watchcursor);
-		fp = fopen(filename, "w");
+		fp = open_file(filename, "w");
 		if (fp != NULL) {
 			/* Write out the imf forms of all the image families. */
 			for (i = 0; i < numimages; ++i) {
@@ -1176,7 +1207,10 @@ ImageFamily *imf;
 		strcat(xcifbuf, ")");
 		makexcif = TRUE;
 	}
-    for (img = imf->images; img != NULL; img = img->next) {
+    for_all_images(imf, img) {
+		/* Skip over any synthesized images. */
+		if (img->synthetic)
+		  continue;
 		/* If we have multiple sizes, append the image size to some names. */
 		if (imf->numsizes > 1
 			&& (img->istile ? (img->w != 8 || img->h != 8) : (img->w != 32 || img->h != 32))) {
@@ -2055,4 +2089,3 @@ __ttyname(long fd)
 }
 
 #endif /* __MWERKS__ */
-
