@@ -1,5 +1,5 @@
 /* Commands for the Mac interface to Xconq.
-   Copyright (C) 1992-1999 Stanley T. Shebs.
+   Copyright (C) 1992-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -27,19 +27,10 @@ extern int _trace;	/* Not used by CodeWarrior. */
 #endif
 #endif
 
-extern void set_construction_run_length(int len);
-extern void do_idle_command(int n);
-extern int do_down_command(void);
-extern int do_up_command(void);
-
 extern WindowPtr playersetupwin;
 
 extern int forcedtoresign;
 
-extern int modal_construction;
-
-extern WindowPtr window_behind_construction;
-
 int position_set_modally;
 
 Point modal_point;
@@ -62,34 +53,27 @@ extern Side *tmpcmdside;
 
 /* Local function declarations. */
 
+static int get_a_position(Map **map, int *xp, int *yp, int *dirp);
+static int get_a_unit(Map **mapp, Unit **unitp);
+static int get_a_visible_unit(Map **mapp, Unit **unitp);
+static int add_unit_position(Unit *unit);
+static void message_dialog(int sidenum);
 static int do_one_add_terrain(Unit *unit);
-static int do_one_ai_control(Unit *unit);
 static int do_one_attack(Unit *unit);
 static void unit_do_build_2(Unit *unit);
-extern int do_one_clear_plan(Unit *unit);
-extern int do_one_collect(Unit *unit);
-static int do_one_delay(Unit *unit);
-extern int do_one_detach(Unit *unit);
-extern int do_one_detonate(Unit *unit);
-extern int do_one_dir_move(Unit *unit);
-extern int do_one_dir_multiple_move(Unit *unit);
-extern int do_one_disband(Unit *unit);
-extern int do_one_disembark(Unit *unit);
-extern int do_one_embark(Unit *unit);
+static int do_one_collect(Unit *unit);
 static int do_one_fire(Unit *unit);
 static int do_one_fire_into(Unit *unit);
-extern int do_one_give(Unit *unit);
-extern int do_one_give_unit(Unit *unit);
-extern int do_one_idle(Unit *unit);
 static int do_one_move_to(Unit *unit);
 static int do_one_occupant(Unit *unit);
 static int do_one_remove_terrain(Unit *unit);
-extern int do_one_repair(Unit *unit);
-static int do_one_reserve(Unit *unit);
-extern int do_one_return(Unit *unit);
+static int do_one_repair(Unit *unit);
 static int do_one_set_name(Unit *unit);
-static int do_one_asleep(Unit *unit);
-extern int do_one_take(Unit *unit);
+static int do_one_dir_move(Unit *unit);
+static int do_one_dir_multiple_move(Unit *unit);
+static void create_command_window(void);
+static void get_command_and_do(void);
+static int unit_rename_dialog(Unit *unit);
 
 void cmd_error(Side *side, char *fmt, ...);
 
@@ -102,11 +86,6 @@ static int tmpcmdx, tmpcmdy, tmpcmddir;
 
 static Unit *tmpcmdunit;
 
-/* This is the actual key typed, for use if several keyboard commands
-   share a single function. */
-
-extern char tmpkey;
-
 /* Prefixed numeric argument to commands. */
 
 static int tmpdir;
@@ -169,8 +148,6 @@ get_a_unit(Map **mapp, Unit **unitp)
 	return FALSE;
 }
 
-int get_a_visible_unit(Map **mapp, Unit **unitp);
-
 int
 get_a_visible_unit(Map **mapp, Unit **unitp)
 {
@@ -194,6 +171,52 @@ get_a_visible_unit(Map **mapp, Unit **unitp)
 	return rslt;
 }
 
+static UnitVector *selvec;
+
+UnitVector *
+get_selected_units(Side *side)
+{
+	int i, rslt, numcould = 0, numnot = 0;
+	Map *map;
+	List *list;  
+	UnitCloseup *closeup;
+	Unit *unit;
+
+	if (selvec == NULL)
+	  selvec = make_unit_vector(100);
+	clear_unit_vector(selvec);
+
+	if (frontmap) {
+		for (i = 0; i < frontmap->numselections; ++i) {
+			unit = frontmap->selections[i];
+			/* Don't apply to selected units that we don't control. */
+			if (in_play(unit) && side_controls_unit(dside, unit)) {
+			    selvec = add_unit_to_vector(selvec, unit, 0);
+			}
+		}
+	} else if ((list = list_from_window(topWindow)) != NULL) {
+		if ((unit = (Unit *) selected_unit_in_list(list)) != NULL) {
+		    selvec = add_unit_to_vector(selvec, unit, 0);
+		}
+	} else if ((closeup = unit_closeup_from_window(topFloat)) != NULL) {
+		unit = closeup->unit;
+		/* Don't apply to units that we don't control. */
+		if (side_controls_unit(dside, unit)) {
+		    selvec = add_unit_to_vector(selvec, unit, 0);
+		}
+	} else if (maplist != NULL) {
+		map = maplist;
+		for (i = 0; i < map->numselections; ++i) {
+			unit = map->selections[i];
+			/* Don't apply to selected units that we don't control. */
+			if (in_play(unit) && side_controls_unit(dside, unit)) {
+			    selvec = add_unit_to_vector(selvec, unit, 0);
+			}
+		}
+	}
+	return selvec;
+}
+
 /* Start of alphabetized commands. */
 
 void do_flash(Side *side) {}
@@ -305,29 +328,6 @@ do_one_attack(Unit *atker)
 	return FALSE;
 }
 
-void
-do_auto(Side *side)
-{
-	apply_to_all_selected(do_one_ai_control, TRUE);	
-}
-
-static int
-do_one_ai_control(Unit *unit)
-{
-    int newval;
-
-	if (unit->plan) {
-		if (dside->prefixarg < 0)
-		  newval = !unit->plan->aicontrol;
-		else if (dside->prefixarg == 0)
-		  newval = 0;
-		else
-		  newval = 1;
-		net_set_unit_ai_control(dside, unit, newval, FALSE);
-	}
-	return TRUE;
-}
-
 void
 do_build(Side *side)
 {
@@ -339,8 +339,10 @@ do_build(Side *side)
 	list = list_from_window(topWindow);
 	if (list != NULL) {
 		unit = selected_unit_in_list(list);
-		if (unit != NULL && can_build_or_help(unit)) {
-			unit_do_build_2(unit);
+		if (unit != NULL 
+		    && can_build_or_help(unit)
+		    && side_controls_unit(dside, unit)) {
+			show_unit_build_dialog(unit);
 			return;
 		}
 		return;
@@ -348,8 +350,10 @@ do_build(Side *side)
 	closeup = unit_closeup_from_window(topFloat);
 	if (closeup != NULL) {
 		unit = closeup->unit;
-		if (unit != NULL && can_build_or_help(unit)) {
-			unit_do_build_2(unit);
+		if (unit != NULL 
+		    && can_build_or_help(unit)
+		    && side_controls_unit(dside, unit)) {
+			show_unit_build_dialog(unit);
 			return;
 		}
 		return;
@@ -357,8 +361,10 @@ do_build(Side *side)
 	if (frontmap) {
 		for (i = 0; i < frontmap->numselections; ++i) {
 			unit = frontmap->selections[i];
-			if (unit != NULL && can_build_or_help(unit)) {
-				unit_do_build_2(unit);
+			if (unit != NULL 
+			    && can_build_or_help(unit)
+			    && side_controls_unit(dside, unit)) {
+				show_unit_build_dialog(unit);
 				return;
 			}
 		}
@@ -373,51 +379,17 @@ unit_do_build_2(Unit *unit)
 {
     extern int editedrunlength;
 
-	modal_construction = TRUE;
-	window_behind_construction = topWindow;
+	show_construction_window();
 	if (dside->prefixarg > 0)
 	  set_construction_run_length(dside->prefixarg);
 	if (dside->prefixarg > 0)
 	  editedrunlength = dside->prefixarg;
 	else
 	  editedrunlength = -1;
-	show_construction_window();
 	select_unit_in_construction_window(unit);
 	select_type_in_construction_window(favored_type(unit));
 }
 
-/* Create and/or bring up the construction planning window. */
-
-void
-show_construction_window()
-{
-	window_behind_construction = topWindow;
-	if (constructionwin == nil) {
-		create_construction_window();
-	} else {
-		reinit_construction_lists();
-	}
-	SelectTheWindow(constructionwin);
-	ActivateWindow(constructionwin, true);
-}
-
-void
-do_clear_plan(Side *side)
-{
-	apply_to_all_selected(do_one_clear_plan, TRUE);	
-}
-
-int
-do_one_clear_plan(Unit *unit)
-{
-	if (unit->plan) {
-		net_set_unit_plan_type(dside, unit, PLAN_NONE);
-		net_clear_task_agenda(dside, unit);
-		return TRUE;
-	}
-	return FALSE;
-}
-
 int mtocollect;
 
 void
@@ -460,69 +432,6 @@ do_copying(Side *side)
 	show_help_window(copying_help_node);
 }
 
-void
-do_delay(Side *side)
-{
-	apply_to_all_selected(do_one_delay, TRUE);
-}
-
-int
-do_one_delay(Unit *unit)
-{
-	if (unit->plan) {
-	    net_delay_unit(unit, TRUE);
-		return TRUE;
-	}
-	return FALSE;
-}
-
-void
-do_detach(Side *side)
-{
-	apply_to_all_selected(do_one_detach, TRUE);
-}
-
-int
-do_one_detach(Unit *unit)
-{
-	if (!completed(unit)) {
-		return FALSE;
-	} else if (valid(check_transfer_part_action(unit, unit, unit->hp / 2, NULL))) {
-		net_prep_transfer_part_action(unit, unit, unit->hp / 2, NULL);
-		return TRUE;
-	} else {
-		/* try to find a nearby unit to do it */
-	}
-	return FALSE;
-}
-
-/* This command samples the current mouse to get the desired detonate location. */
-
-void
-do_detonate(Side *side)
-{
-    int x, y, dir;
-	Map *map;
-	
-	if (get_a_position(&map, &x, &y, &dir)) {
-		tmpcmdx = x;  tmpcmdy = y;  tmpcmddir = dir;
-		apply_to_all_selected(do_one_detonate, TRUE);
-		return;
-	}
-	beep();
-}
-
-int
-do_one_detonate(Unit *unit)
-{
-	/* (should use tmpcmdx etc, but fall back to unit pos if x,y not valid?) */
-    if (valid(check_detonate_action(unit, unit, unit->x, unit->y, unit->z))) {
-		net_prep_detonate_action(unit, unit, unit->x, unit->y, unit->z);
-		return TRUE;
-	}
-	return FALSE;
-}
-
 /* Command all selected mobile units to move in a given direction. */
 
 /* The function that gets called on each selected unit. */
@@ -534,7 +443,7 @@ do_one_dir_move(Unit *unit)
 
 	if (mobile(unit->type)) {
 		if (point_in_dir(unit->x, unit->y, tmpdir, &nx, &ny)) {
-			return advance_into_cell(dside, unit, nx, ny, unit_at(nx, ny));
+			return advance_into_cell(dside, unit, nx, ny, unit_at(nx, ny), NULL);
 		}
 		return FALSE;
 	}
@@ -597,50 +506,6 @@ do_dir_multiple(Side *side)
 	}
 }
 
-void
-do_disband(Side *side)
-{
-	apply_to_all_selected(do_one_disband, TRUE);
-}
-
-int
-do_one_disband(Unit *unit)
-{
-    return net_disband_unit(dside, unit);
-}
-
-void
-do_disembark(Side *side)
-{
-	apply_to_all_selected(do_one_disembark, TRUE);
-}
-
-int
-do_one_disembark(Unit *unit)
-{
-	Unit *transport = unit->transport;
-
-	if (transport == NULL)
-	  return FALSE;
-	/* Try moving into the transport's transport, if there is one. */
-	if (transport->transport != NULL
-		&& can_occupy(unit, transport->transport)) {
-		net_prep_enter_action(unit, unit, transport->transport);
-		/* (should be able to set up task if can't do action immediately) */
-		return TRUE;
-	}
-	/* Try moving into the open in the cell. */
-	if (!inside_area(unit->x, unit->y))
-	  return FALSE;
-	if (valid(check_move_action(unit, unit, unit->x, unit->y, unit->z))) {
-		net_prep_move_action(unit, unit, unit->x, unit->y, unit->z);
-		return TRUE;
-	} else {
-		/* (should be able to set up task if can't do action immediately) */
-	}
-	return FALSE;
-}
-
 void
 do_distance(Side *side)
 {
@@ -648,70 +513,6 @@ do_distance(Side *side)
 	query_position_modally(DISTANCE_MODAL);
 }
 
-void
-do_embark(Side *side)
-{
-	apply_to_all_selected(do_one_embark, TRUE);
-}
-
-int
-do_one_embark(Unit *unit)
-{
-	int x = unit->x, y = unit->y;
-	Unit *transport = unit->transport, *unit2, *unit3;
-
-	/* Note that occupation tests check for unit == transport case,
-	   so not necessary in the code below. */
-	if (transport == NULL) {
-		/* Unit is in the open. */
-		for_all_stack(x, y, unit2) {
-			if (unit2 != transport && can_occupy(unit, unit2)) {
-				net_prep_enter_action(unit, unit, unit2);
-				return TRUE;
-			}
-		}
-		for_all_stack(x, y, unit2) {
-			for_all_occupants(unit2, unit3) {
-				if (unit3 != transport && can_occupy(unit, unit3)) {
-					net_prep_enter_action(unit, unit, unit3);
-					return TRUE;
-				}
-			}
-		}
-	} else {
-		for_all_occupants(transport, unit2) {
-			if (unit2 != transport && can_occupy(unit, unit2)) {
-				net_prep_enter_action(unit, unit, unit2);
-				return TRUE;
-			}
-		}
-		if (transport->transport == NULL) {
-			for_all_stack(x, y, unit2) {
-				if (unit2 != transport && can_occupy(unit, unit2)) {
-					net_prep_enter_action(unit, unit, unit2);
-					return TRUE;
-				}
-			}
-			for_all_stack(x, y, unit2) {
-				for_all_occupants(unit2, unit3) {
-					if (unit3 != transport && can_occupy(unit, unit3)) {
-						net_prep_enter_action(unit, unit, unit3);
-						return TRUE;
-					}
-				}
-			}
-		} else {
-			for_all_occupants(transport->transport, unit2) {
-				if (unit2 != transport && can_occupy(unit, unit2)) {
-					net_prep_enter_action(unit, unit, unit2);
-					return TRUE;
-				}
-			}
-		}
-	}
-	return FALSE;
-}
-
 #if 0 /* (should fix this) */
 void
 do_end_turn(Side *side)
@@ -804,20 +605,6 @@ do_follow_action(Side *side)
 	}
 }
 
-void
-do_give(Side *side)
-{
-	givetakeprefixarg = side->prefixarg;
-	apply_to_all_selected(do_one_give, TRUE);
-}
-
-int
-do_one_give(Unit *unit)
-{
-	give_supplies(unit, NULL, NULL);
-	return TRUE;
-}
-
 void
 do_give_unit(Side *side)
 {
@@ -862,26 +649,6 @@ do_help(Side *side)
 	show_help_window(NULL);
 }
 
-void
-do_idle(Side *side)
-{
-	do_idle_command(side->prefixarg);
-}
-
-void
-do_idle_command(int value)
-{
-	tmpcmdarg = value;
-	apply_to_all_selected(do_one_idle, TRUE);	
-}
-
-static int
-do_one_idle(Unit *unit)
-{
-	net_set_sentry_task(unit, tmpcmdarg);
-	return TRUE;
-}
-
 void
 do_message(Side *side)
 {
@@ -986,12 +753,64 @@ do_name(Side *side)
 	apply_to_all_selected(do_one_set_name, TRUE);
 }
 
+/* Unit naming/renaming. */
+
 int
 do_one_set_name(Unit *unit)
 {
 	return unit_rename_dialog(unit);
 }
 
+int
+unit_rename_dialog(Unit *unit)
+{
+	short done = FALSE, ditem;
+	char *newname;
+	char *namer = unit_namer(unit);
+	Str255 tmpstr;
+	DialogPtr win;
+	short itemtype;  Handle itemhandle;  Rect itemrect;
+
+	if (unit == NULL)
+	  return FALSE;
+	win = GetNewDialog(dRename, NULL, (DialogPtr) -1L);
+	/* Seed the text item with the original name. */
+	newname = unit->name;
+	if (newname == NULL)
+	  newname = "";
+	GetDItem(win, diRenameName, &itemtype, &itemhandle, &itemrect);
+	c2p(newname, tmpstr);
+	SetIText(itemhandle, tmpstr);
+	/* Gray out the random renaming button if no namers available. */
+	GetDItem(win, diRenameRandom, &itemtype, &itemhandle, &itemrect);
+	HiliteControl((ControlHandle) itemhandle, (!empty_string(namer) ? 0 : 255));
+	ShowWindow(win);
+	while (!done) {
+		draw_default_button(win, diRenameOK);
+		SetCursor(&QD(arrow));
+		ModalDialog(NULL, &ditem);
+		switch (ditem) {
+			case diRenameOK:
+				GetDItem(win, diRenameName, &itemtype, &itemhandle, &itemrect);
+				net_set_unit_name(dside, unit, get_string_from_item(itemhandle));
+				/* Fall into the next case. */
+			case diRenameCancel:
+				done = TRUE;
+				break;
+			case diRenameRandom:
+				newname = propose_unit_name(unit);
+				if (!empty_string(newname)) {
+					GetDItem(win, diRenameName, &itemtype, &itemhandle, &itemrect);
+				 	c2p(newname, tmpstr);
+					SetIText(itemhandle, tmpstr);
+				}
+				break;
+		}
+	}
+	DisposeDialog(win);
+	return TRUE;
+}
+
 int
 do_one_occupant(Unit *unit)
 {
@@ -1036,16 +855,10 @@ do_other(Side *side)
 
 WindowPtr commandwin;
 
-ControlHandle dobutton;
-
 TEHandle command_text = nil;
 
 Rect commandtextrect;
 
-void create_command_window(void);
-
-static void get_command_and_do(void);
-
 void
 show_command_window()
 {
@@ -1066,11 +879,10 @@ create_command_window()
 	} else {
 		commandwin = GetNewWindow(wCommand, NULL, NULL);
 	}
-	dobutton = GetNewControl(cCommandDoButton, commandwin);
 	SetPort(commandwin);
 	tmprect = commandwin->portRect;
 	commandtextrect = tmprect;
-	commandtextrect.bottom -= 25;
+	commandtextrect.bottom -= 14;
 	InsetRect(&commandtextrect, 5, 5);
 	/* Create and clear the TextEdit record. */
 	command_text = TENew(&commandtextrect, &commandtextrect);
@@ -1081,7 +893,8 @@ create_command_window()
 		sprintf(spbuf, "%d ", other_command_prefixarg);
 		TEInsert(spbuf, strlen(spbuf), command_text);
 	}
-	SelectTheWindow(commandwin);
+	/* Finally make it floating. */
+	MakeFloat(commandwin);
 }
 
 void
@@ -1089,7 +902,12 @@ draw_command()
 {
 	Rect tmprect;
 
+	MoveTo(5, 34);
+	TextFont(small_font_id);
+	TextSize(small_font_size);
+	DrawString("\pReturn executes the command. Escape just closes the window.");
 	TEUpdate(&(commandwin->portRect), command_text);
+	TESetSelect(0, 32767, command_text);
 	tmprect = commandtextrect;
 	InsetRect(&tmprect, -1, -1);
 	FrameRect(&tmprect);
@@ -1105,13 +923,22 @@ activate_command(int activate)
 }
 
 int
-do_key_down_command(key)
-int key;
+do_key_down_command(int keyCode, int charCode)
 {
-	if (key == 13 || key == 3) {
+	/* Return and Enter executes the command and closes
+	the window */
+	if (charCode == 13 || charCode == 3) {
 		get_command_and_do();
+		close_window(commandwin);
+	/* Escape key just closes the window. */
+	} else if (charCode == 27) {
+		close_window(commandwin);
+	/* Return FALSE if this is not a normal keypad key. */
+	}else if (keyCode > 0x3E) {
+	    return FALSE;
+	/* Else feed char to command text field. */
 	} else {
-		TEKey(key, command_text);
+		TEKey(charCode, command_text);
 	}
 	return TRUE;
 }
@@ -1119,16 +946,8 @@ int key;
 void
 do_mouse_down_command(Point mouse, int mods)
 {
-	short part;
-	ControlHandle control;
-
-	part = FindControl(mouse, commandwin, &control);
-	if (control == dobutton) {
-		TrackControl(control, mouse, NULL);
-		get_command_and_do();
-	} else if (PtInRect(mouse, &commandtextrect)) {
+	if (PtInRect(mouse, &commandtextrect))
 		TEClick(mouse, mods, command_text);
-	}
 }
 
 static void
@@ -1223,7 +1042,7 @@ do_quit(Side *side)
 
 static int allsumx, allsumy, unitcount;
 
-static int
+int
 add_unit_position(Unit *unit)
 {
 	allsumx += unit->x;  allsumy += unit->y;
@@ -1349,43 +1168,6 @@ do_one_repair(Unit *unit)
 	return FALSE;
 }
 
-static int
-do_one_reserve(Unit *unit)
-{
-	net_set_unit_reserve(dside, unit, tmpcmdarg, tmprecurse);
-	return TRUE;
-}
-
-void
-do_reserve_command(int value, int radius, int recurse)
-{
-	tmpcmdarg = value;
-	tmprecurse = recurse;
-	apply_to_all_selected(do_one_reserve, TRUE);	
-}
-
-void
-do_reserve(Side *side)
-{
-	do_reserve_command(TRUE, side->prefixarg, FALSE);
-}
-
-static int
-do_one_return(Unit *unit)
-{
-	if (1 /* has a place to return to */) {
-		net_set_resupply_task(unit, NONMTYPE);
-		return TRUE;
-	}
-	return FALSE;
-}
-
-void
-do_return(Side *side)
-{
-	apply_to_all_selected(do_one_return, TRUE);	
-}
-
 void
 do_resign(Side *side)
 {
@@ -1461,27 +1243,6 @@ do_set_view_angle(Side *side)
 	}
 }
 
-void
-do_sleep(Side *side)
-{
-	do_sleep_command(TRUE, side->prefixarg, FALSE);
-}
-
-void
-do_sleep_command(int value, int radius, int recurse)
-{
-	tmpcmdarg = value;
-	tmprecurse = recurse;
-	apply_to_all_selected(do_one_asleep, TRUE);	
-}
-
-static int
-do_one_asleep(Unit *unit)
-{
-	net_set_unit_asleep(dside, unit, tmpcmdarg, tmprecurse);
-	return TRUE;
-}
-
 void
 do_standing_orders(Side *side)
 {
@@ -1506,56 +1267,77 @@ do_survey(Side *side)
 	}
 }
 
-void
-do_take(Side *side)
-{
-	if (nummtypes == 0) {
-		beep();
-		return;
-	}
-	givetakeprefixarg = side->prefixarg;
-	apply_to_all_selected(do_one_take, TRUE);
-}
-
-int
-do_one_take(Unit *unit)
-{
-	short m, amts[MAXMTYPES], rslts[MAXMTYPES], needed;
-
-	for_all_material_types(m) {
-		amts[m] = givetakeprefixarg;
-	}
-	needed = take_supplies(unit, amts, rslts);
-	report_take(dside, unit, needed, rslts);
-	return TRUE;
-}
-
 void
 do_up(Side *side)
 {
-	do_up_command();
+	int i, numcould = 0, numnot = 0, rawamt = 1;
+	int amt, newalt;
+	Point target;
+	Unit *unit;
+
+	if (frontmap) {
+		for (i = 0; i < frontmap->numselections; ++i) {
+			if ((unit = frontmap->selections[i]) != NULL 
+			       && valid_selection(unit)) {
+				amt = rawamt;
+				newalt = (unit_alt(unit) + amt) << 1;
+				if (valid(check_move_action(unit, unit, unit->x, unit->y, newalt))) {
+					net_prep_move_action(unit, unit, unit->x, unit->y, newalt);
+					++numcould;
+				} else {
+					++numnot;
+				}
+			}
+		}
+	}
+	/* If nobody could do the action, beep once. */
+	if (numcould == 0 && numnot > 0) {
+		beep();
+	}
 }
 
+#if 0		/* Not supported in the kernel yet. */
+
+void
+do_down(Side *side)
+{
+	int i, numcould = 0, numnot = 0, rawamt = 0;
+	int amt, newalt;
+	Point target;
+	Unit *unit;
+
+	rawamt = 0;	
+	if (frontmap) {
+		for (i = 0; i < frontmap->numselections; ++i) {
+			if ((unit = frontmap->selections[i]) != NULL 
+			       && valid_selection(unit)) {
+				amt = rawamt;
+				if (amt == 0 && unit_alt(unit) > 0)
+				  amt = unit_alt(unit) / 2;
+				newalt = (unit_alt(unit) - amt) << 1;
+				if (valid(check_move_action(unit, unit, unit->x, unit->y, newalt))) {
+					net_prep_move_action(unit, unit, unit->x, unit->y, newalt);
+					++numcould;
+				} else {
+					++numnot;
+				}
+			}
+		}
+	}
+	/* If nobody could do the action, beep once. */
+	if (numcould == 0 && numnot > 0) {
+		beep();
+	}
+}
+
+#endif
+
 void
 do_version(Side *side)
 {
 	do_about_box();
 }
 
-void
-do_wake(Side *side)
-{
-	do_sleep_command(FALSE, side->prefixarg, FALSE);
-	do_reserve_command(FALSE, side->prefixarg, FALSE);
-}
-
-void
-do_wake_all(Side *side)
-{
-	do_sleep_command(FALSE, side->prefixarg, TRUE);
-	do_reserve_command(FALSE, side->prefixarg, TRUE);
-}
-
 void
 do_warranty(Side *side)
 {
