@@ -17,7 +17,6 @@ any later version.  See the file COPYING.  */
 
 /* --- FUNCTIONS PROTOTYPES -------------------------------------------------*/
 
-extern void mac_init_cicn(Image *img);
 static int append_image_names(MenuHandle menu);
 static int append_color_names(MenuHandle menu);
 static int append_terrain_names(MenuHandle menu);
@@ -31,23 +30,23 @@ extern Image **best_terrain_images;
 
 /* --- DEFAULTS FOR NEW MAP PREFERENCES ---------------------------------------*/
 
-int default_sidecolors = TRUE;		/* Use side colors in maps & lists */
-int default_iconmasks = TRUE;		/* Use masks for icons  */
+int default_sidecolors = FALSE;		/* Use side colors in maps & lists */
+int default_iconmasks = FALSE;		/* Use masks for icons  */
 int default_boxmasks = TRUE;		/* Use masks for grouping boxes */
 int default_textmasks = FALSE;		/* Use masks for text */
 int default_featureborders = TRUE;	/* Draw feature (province) borders */
 int default_featurenames = TRUE;		/* Draw feature (province) names */
 int default_simple_borders = TRUE;	/* Do not draw borders to water or to empty lands */
 int default_shorelines = TRUE;		/* Draw shorelines in 3D relief */
-int default_draw_ai = TRUE;		/* Draw AI regions (if present). */
-int alert_256_colors = TRUE;		/* Suggest switch to 256 colors at start. */
+int default_draw_ai = FALSE;		/* Draw AI regions (if present). */
+int alert_256_colors = FALSE;		/* Suggest switch to 256 colors at start. */
 int default_draw_scrollbars = FALSE;	/* Draw scrollbars in maps. */
 int default_draw_cpanel = TRUE;		/* Draw control panel in maps. */
 int default_draw_emblems = TRUE;	/* Draw unit emblems in maps and lists. */
 int default_draw_topline = TRUE;		/* Draw top line boxes in maps */
 int default_draw_topunit = TRUE;		/* Draw unit info boxes in maps */
 int default_drawothermaps = TRUE;	/* Draw other maps as boxes */
-int default_erase_names = FALSE;		/* Draw extra rows to erase name of moving unit */
+int default_erase_names = TRUE;		/* Draw extra rows to erase name of moving unit */
 int default_max_bufx = 600;		/* Horizontal scrolling buffers in offscreen graphics */
 int default_max_bufy = 400;		/* Vertical scrolling buffers in offscreen graphics */
 int default_optimize_fonts = TRUE;	/* Use optimized font sizes in map content */
@@ -61,7 +60,7 @@ int use_colornames = TRUE;			/* Save and read side colors as colorschemes */
 int use_RGB_values = FALSE;		/* Save and read RGB colors in XCol resource */
 int show_instructions = TRUE;		/* Show instructions window at start */
 int fullsize_map = TRUE;			/* Show full size map window at start */
-int unified_cicns = TRUE;			/* Use BitMap as mask for the PixMap */
+int unified_cicns = FALSE;			/* Use BitMap as mask for the PixMap */
 
 /* --- COLOR SCHEME DEFAULTS ---------------------------------------------------*/
 
@@ -73,7 +72,10 @@ int default_split_icon = TRUE;		/* Default use different colors for right and le
 /* --- COLORS, COLOR SCHEMES AND COLOR NAMES --------------------------------------*/
 
 RGBColor blackcolor = {0, 0, 0};
+RGBColor dkgraycolor = {24000, 24000, 24000};
 RGBColor graycolor = {32768, 32768, 32768};
+RGBColor ltgraycolor = {49000, 49000, 49000};
+RGBColor whitecolor = {0xFFFF, 0xFFFF, 0xFFFF};
 
 RGBColor *featColor;				/* Feature colors by RGB value */
 RGBColor *sideColor;				/* sideColor[0-15][1-3] are defined by default color schemes */
@@ -198,11 +200,12 @@ init_side_colors(void)
 			set_sideColorName(i, 2, "undefined");
 			set_sideColorName(i, 3, "undefined");
 			/* Use schemes defined in game file if present or else default schemes */
-			if (side_n(i) && side_n(i)->colorscheme)
-			  current_colorscheme[i] = copy_string(side_n(i)->colorscheme);
-			else
-			  current_colorscheme[i] = copy_string(default_colorscheme[i]);
-				
+			if (side_n(i) && side_n(i)->colorscheme) {
+				current_colorscheme[i] = copy_string(side_n(i)->colorscheme);
+			} else {
+				current_colorscheme[i] = copy_string(default_colorscheme[i]);
+			}
+			net_set_side_colorscheme(dside, side_n(i), current_colorscheme[i]);
 			dissect_colorscheme(i);
 		}
 
