@@ -35,34 +35,54 @@ extern short initialvrefnum;
 
 char *connection_method_name;
 
-extern void auto_pick_side_research(Side *side);
-extern void show_research_dialog();
-extern void draw_research_dialog(int force);
-extern int do_key_down_research(char key);
-extern void hit_research_dialog(int ditem);
-
-extern void recalc_spacing(void);
-extern void invert_map(Map *map, int x, int y, int r);
-
 /* Function prototypes. */
 
+static int splash_dialog(void);
 static void init_toolbox(void);
+static void recalc_depths(void);
 static void init_fonts(void);
 static void init_rects(void);
 static void init_ae(void);
+static void get_files(void);
+
+static void event_loop(void);
+static void handle_keystroke(EventRecord *event);
+static void set_end_of_game_interaction_modes(void);
+static void adjust_cursor(Point mouse, RgnHandle region);
 static void handle_event(EventRecord *event);
-static int handle_keystroke(EventRecord *event);
+
 static void drag_window(WindowPtr win, EventRecord *event);
+static void grow_window(WindowPtr win, Point where);
+static void zoom_window(WindowPtr win, Point where, int part);
+static void do_mouse_down(WindowPtr window, EventRecord *event);
+static void activate_window(WindowPtr win, int activate);
+
+static void maybe_select_next_unit(void);
+static int select_new_unit(Map *map, Unit *unit);
+
+static Boolean missed_any_parameters(AppleEvent *message);
 static pascal OSErr do_ae_open_application(AppleEvent *message, AppleEvent *reply, long refcon);
 static pascal OSErr do_ae_open_documents(AppleEvent *message, AppleEvent *reply, long refcon);
 static pascal OSErr do_ae_print_documents(AppleEvent *message, AppleEvent *reply, long refcon);
 static pascal OSErr do_ae_quit_application(AppleEvent *message, AppleEvent *reply, long refcon);
 static pascal OSErr do_ae_join_game(AppleEvent *message, AppleEvent *reply, long refcon);
 
-static Boolean missed_any_parameters(AppleEvent *message);
-
 static pascal Boolean filter_warning_alert(DialogPtr dialog, EventRecord *evt, short *itemhit);
 
+static int is_da_window(WindowPtr win);
+static int is_app_window(WindowPtr win);
+
+static void update_unit_in_maps(Unit *unit);
+
+static void won_game_dialog(void);
+static void lost_game_dialog(void);
+static void game_over_dialog(void);
+
+static void play_sound(char *soundname);
+static int position_on_screen(int h, int v);
+static int position_already_used(int h, int v);
+
+
 /* Global variables. */
 
 /* This is the id of any map modal tool currently in effect. */
@@ -178,6 +198,8 @@ static int told_outcome = FALSE;
 
 static int want_to_exit = FALSE;
 
+static int last_tick_count = 0;
+
 enum movie_type {
   movie_null,
   movie_miss,
@@ -497,8 +519,6 @@ get_files()
 	}
 	GetIndString(filename, imagesfilenameres, 2);
 	OpenResFile(filename);
-
-
 	GetIndString(filename, sFilenames, siSounds);
 	if (OpenResFile(filename) != -1) {
 		/* (need to do anything special if sounds not found??) */
@@ -528,8 +548,6 @@ event_loop()
 	cursorRgn = NewRgn();
 	/* Loop (almost) forever. */
 	while (!eventloopdone) {
-		/* Move here since we must call adjust_cursor at least once. Thus, we can 
-		avoid the second call below by always calling it here. */
 		get_global_mouse(&mouse);
 		adjust_cursor(mouse, cursorRgn);
 		/* Use WaitNextEvent if it is available, otherwise GetNextEvent. */
@@ -539,35 +557,7 @@ event_loop()
 			SystemTask();
 			gotevent = GetNextEvent(everyEvent, &event);
 		}
-		/* Deal with keystrokes first since their handling now is the same 
-		for modeless dialogs and normal windows. */
-		if (event.what == keyDown || event.what == autoKey) {
-			if (handle_keystroke(&event))
-			    continue;
-		}
-		/* Then handle non-keystroke modeless dialog events. */
-		if (IsDialogEvent(&event) && DialogSelect(&event, &dialog, &itemhit)) {
-				if (dialog == playersetupwin) {
-				hit_player_setup_dialog(itemhit, mouse);
-			} else if (dialog == researchwin) {
-				hit_research_dialog(itemhit);
-			} else if (topFloat && unit_closeup_from_window(topFloat)) {
-				hit_closeup_dialog(topFloat, itemhit, mouse);
-				}
-				/* If this was something like a key or click event, but wasn't handled
-			   by the dialog, we're done and can go wait for the next event. Otherwise, 
-			   for null events, fall through to the usual handling. */
-				if (gotevent)
-				  continue;
-			}
-		
 		if (gotevent) {
-
-#if 0			/* Redundant, adjust_cursor is now always called above. */
-		
-			/* Make sure we have the right cursor before handling the event. */
-			adjust_cursor(event.where, cursorRgn);
-#endif
 			handle_event(&event);
 		} else if (connection_method > 0 && numremotewaiting > 0) {
 			receive_data(0);
@@ -579,18 +569,13 @@ event_loop()
 #if __profile__
 	ProfilerSetStatus(true);
 #endif
-
 			run_local_ai(1, 20);
 			net_run_game(1);
-
 #if __profile__
 	ProfilerSetStatus(false);
 #endif
-
 			maybe_select_next_unit();
-#if 0
-			run_local_ai(2, 20);
-#endif
+
 			/* If the game ended, force various changes in interaction. */
 			if (endofgame)
 			  set_end_of_game_interaction_modes();
@@ -609,57 +594,58 @@ event_loop()
 	}
 }
 
-/* This function returns TRUE for keyboard events that get handled and
-FALSE for those who should fall through to the dialog manager. */
-
-int
+void
 handle_keystroke(EventRecord *event)
 {
 	char	charCode = event->message & charCodeMask;
 	char	keyCode = (event->message & keyCodeMask) >> 8;
+	int	mods = event->modifiers;
 	int	handled = FALSE;
 	UnitCloseup *closeup;
 	List	*list;
 	
 	/* Menu key commands. */
-	if (event->modifiers & cmdKey
+	if (mods & cmdKey
 	/* Don't interpret cmd + keypad as menu command. */
 	    && keyCode < 0x40) {
 		adjust_menus();
 		do_menu_command(MenuKey(charCode));
-		return TRUE;
+		return;
 	}
-	/* Alt-w closes topFloat. */
-	if (charCode == '') {
+	/* Alt-w and Escape closes topFloat. */
+	if (charCode == '' || charCode == 27) {
 		close_window(topFloat);
-		return TRUE;
+		return;
 	} 
 	/* Key commands for the player setup dialog. */ 
 	if (FrontWindow() == playersetupwin) {
-		handle_player_setup_event(event);
-		return TRUE;
+		do_key_down_setup(event);
+		return;
 	}		
-	/* Key commands for modeless dialogs etc. */	
+	/* Key commands for floating windows. */	
 	if (topFloat) {
 		closeup = unit_closeup_from_window(topFloat);
 		if (closeup) {
-			int result = do_key_down_closeup(closeup, keyCode);
-
-			/* We are done. Return TRUE to prevent further
-			handling by the dialog manager. */
-			if (result == DONE) {
-				return TRUE;
-			/* This was a key that should be handled by the
-			dialog manager. Return FALSE. */
-			} else if (result == TRUE) {
-				return FALSE;
-			/* The key returned FALSE and should therefore
-			be processed further below. */
-			}
+			handled = do_key_down_closeup(closeup, keyCode, charCode, mods);
 		} else if (topFloat == helpwin) {
 			handled = do_key_down_help(charCode);		
+		} else if (topFloat == instructionswin) {
+			handled = do_key_down_instructions(charCode);		
+		} else	if (topFloat == constructionwin) {
+			handled = do_key_down_construction(charCode);
 		} else if (topFloat == researchwin) {
 			handled = do_key_down_research(charCode);		
+		} else if (topFloat == commandwin) {
+			handled = do_key_down_command(keyCode, charCode);
+		} else if (topFloat == buildwin) {
+			handled = do_key_down_build(keyCode, charCode, mods);
+		}
+	}
+	/* Key commands for normal windows. */
+	if (!handled && topWindow) {
+		list = list_from_window(topWindow);
+		if (list) {
+			handled = do_key_down_list(list, charCode);
 		}
 	}
 	/* Special use of the numeric keypad and arrow keys for 
@@ -667,22 +653,11 @@ handle_keystroke(EventRecord *event)
 	if (!handled && (frontmap || worldmap)) {
 		handled = handle_numeric_keypad(event);
 	}
-	/* Key commands for normal windows. */
-	if (!handled && topWindow) {
-		list = list_from_window(topWindow);
-		if (list) {
-			handled = do_key_down_list(list, charCode);
-		} else	if (topWindow == constructionwin) {
-			handled = do_key_down_construction(charCode);
-		} else if (topWindow == commandwin) {
-			handled = do_key_down_command(charCode);
-		}
-	}
 	/* All other keys are interpreted as keyboard commands. */
 	if (!handled) {
 		do_keyboard_command(charCode);
 	}
-	return TRUE;
+	return;
 }
 
 void
@@ -724,15 +699,19 @@ adjust_cursor(Point mouse, RgnHandle region)
 	Unit *unit = NULL;
 	extern char *mouseover;
 	Map *map;
+	WindowPtr win;
 	GrafPtr oldport;
 
-	/* Work around of Appearance manager bug. Since clicks in the collapse box fails
+	/* Work around of Appearance manager bug. Since clicks in the collapse box fail
 	to generate an application level event, and since the system fails to update dialogs
 	that are uncollapsed, it is necessary to force dialog updates in some other way. */
-	if (topFloat && IsDialog(topFloat) 
-	    && PtInRgn(mouse, ((WindowPeek) topFloat)->strucRgn) == TRUE
-	    && PtInRgn(mouse, ((WindowPeek) topFloat)->contRgn) == FALSE) {
-		update_window(topFloat);
+	for_all_windows(win) {
+		if (IsDialog(win) 
+		    && PtInRgn(mouse, ((WindowPeek) win)->strucRgn) == TRUE
+		    && PtInRgn(mouse, ((WindowPeek) win)->contRgn) == FALSE) {
+			update_window(win);
+			return;
+		}
 	}
 	/* Check if we are inside the worldmap window. */
 	if (worldmap && PtInRgn(mouse, ((WindowPeek) worldmap->window)->contRgn))
@@ -755,25 +734,29 @@ adjust_cursor(Point mouse, RgnHandle region)
 		    && mouse.h < map->window->portRect.right - map->sbarwid
 		    && mouse.v < map->window->portRect.bottom - map->sbarwid) {
 
-			if (mouse.h < map->conw + 40) {
+			if (mouse.h < map->conw + 40
+			    && map->vp->sx > map->vp->sxmin) {
 				current_cursor = leftcursor;				
 				usual = FALSE;
 				/* Wait half second before autoscrolling. */
 				if (TickCount() > delay + 30)
 					scroll_map_window(map, -4, 0);
-			} else if (mouse.h > map->window->portRect.right - map->sbarwid - 40) {
+			} else if (mouse.h > map->window->portRect.right - map->sbarwid - 40
+			    && map->vp->sx < map->vp->sxmax) {
 				current_cursor = rightcursor;
 				usual = FALSE;
 				/* Wait half second before autoscrolling. */
 				if (TickCount() > delay + 30)
 					scroll_map_window(map, 4, 0);
-			} else if (mouse.v < map->toph + 40) {
+			} else if (mouse.v < map->toph + 40
+			    && map->vp->sy > map->vp->symin) {
 				current_cursor = upcursor;
 				usual = FALSE;
 				/* Wait half second before autoscrolling. */
 				if (TickCount() > delay + 30)
 					scroll_map_window(map, 0, -4);
-			} else if (mouse.v > map->window->portRect.bottom - map->sbarwid - 40) {
+			} else if (mouse.v > map->window->portRect.bottom - map->sbarwid - 40
+			    && map->vp->sy < map->vp->symax) {
 				current_cursor = downcursor;
 				usual = FALSE;
 				/* Wait half second before autoscrolling. */
@@ -901,8 +884,9 @@ handle_event(EventRecord *event)
 						This is not exactly in line with Apple's interface guidelines
 						but it does make things easier when more than one float is up
 						since you don't have to worry about which one is in front. */
-						if (IsFloating(win))
+						if (IsFloating(win) || IsDialog(win)) {
 						    do_mouse_down(win, event);
+						}
 						/* We always want to discard the event now, since clicks in a
 						   windows are often irreversible actions. */
 					} else {
@@ -933,12 +917,10 @@ handle_event(EventRecord *event)
 					break;					
 			}
 			break;
-#if 0					/* Now handled elsewhere. */
 		case keyDown:
 		case autoKey:
 			handle_keystroke(event);
 			break;
-#endif
 		case activateEvt:
 			activate_window((WindowPtr) event->message, event->modifiers & activeFlag);
 			break;
@@ -1090,8 +1072,9 @@ close_window(WindowPtr win)
 		HideTheWindow(win);
 		/* At least for now, don't actually dispose of the window. */
 	}
-	/* Necessary to redraw some modeless dialogs. */
-	update_all_visible_dialogs();
+	/* Necessary to redraw some modeless dialogs and to ensure that autoscrolling
+	does not create a white rectangle where a floating window used to be. */
+	update_all_visible_windows();
 }
 
 /* This just dispatches to the appropriate window handler. */
@@ -1115,12 +1098,18 @@ do_mouse_down(WindowPtr window, EventRecord *event)
 			do_mouse_down_list(list, mouse, event->modifiers); 
 		} else if ((unitcloseup = unit_closeup_from_window(window)) != NULL) {
 			do_mouse_down_unit_closeup(unitcloseup, mouse, event->modifiers); 
+		} else if (window == buildwin) {
+			do_mouse_down_build(mouse, event->modifiers); 
+		} else if (window == playersetupwin) {
+			do_mouse_down_setup(mouse, event->modifiers);
 		} else if (window == gamewin) {
 			do_mouse_down_game(mouse, event->modifiers);
 		} else if (window == historywin) {
 			do_mouse_down_history(mouse, event->modifiers);
 		} else if (window == constructionwin) {
 			do_mouse_down_construction(mouse, event->modifiers);
+		} else if (window == researchwin) {
+			do_mouse_down_research(mouse, event->modifiers);
 		} else if (window == helpwin) {
 			do_mouse_down_help(mouse, event->modifiers);
 		} else if (window == noticewin) {
@@ -1213,6 +1202,9 @@ update_window(WindowPtr win)
 		growbox = TRUE;
 	} else if ((unitcloseup = unit_closeup_from_window(win)) != NULL) {
 		draw_unit_closeup(unitcloseup);
+	} else if (win == buildwin) {
+		draw_unit_build_dialog(true);
+		controls = FALSE;
 	} else if (win == gamewin) {
 		draw_game();
 		controls = FALSE;
@@ -1227,13 +1219,11 @@ update_window(WindowPtr win)
 		growbox = TRUE;
 	} else if (win == instructionswin) {
 		draw_instructions();
-		growbox = FALSE;
 	} else if (win == noticewin) {
 		draw_notice();
 		growbox = TRUE;
 	} else if (win == commandwin) {
 		draw_command();
-		growbox = FALSE;
 	} else if (win == scoreswin) {
 		draw_scores();
 		growbox = TRUE;
@@ -1295,106 +1285,113 @@ draw_growicon_nolines(WindowPtr win)
 
 }
 
-static int last_tick_count = 0;
-
 void
 maybe_select_next_unit()
 {
-	Unit *unit;
-	Map *map;
-
-	if ((!beforestart && !endofgame)
-	    && (map = frontmap) != NULL
-	    && map->autoselect) {
-		    /* Hunt around for a reasonable "next unit" to select. */
-		    /* Scroll over to the current unit if appropriate. */
-		    if (map->curunit != NULL
-		    	&& in_play(map->curunit)
-			&& has_acp_left(map->curunit)
-		    	&& (map->curunit->plan
-					&& !map->curunit->plan->asleep
-					&& !map->curunit->plan->reserve
-					&& !map->curunit->plan->delayed)
-		    	) {
-			/* Make sure the current unit is selected if on autoselect. */
-		    	if (!unit_is_selected(map, map->curunit)) {
-		    		select_unit_on_map(map, map->curunit);
-		    	}
-		    	if (map->scrolltocurunit) {
-					scroll_to_unit(map, map->curunit);
-					map->scrolltocurunit = FALSE;
-				}
-		    	goto blink;
-		    }
-			unit = autonext_unit_inbox(dside, map->curunit, map->vp);
-			if (unit
-				&& unit->plan
-				&& !unit->plan->asleep
-				&& !unit->plan->reserve
-				&& !unit->plan->delayed
-				&& unit->plan->waitingfortasks
-				) {
-				map->curunit = unit;
-				select_exactly_one_unit(map, map->curunit);
-				goto blink;
-			}
-			/* Look for the next unit. */
-			unit = find_next_awake_mover(dside, map->curunit);
-			if (unit
-				&& unit->plan
-				&& !unit->plan->asleep
-				&& !unit->plan->reserve
-				&& !unit->plan->delayed
-				&& unit->plan->waitingfortasks
-				) {
-				map->curunit = unit;
-				select_exactly_one_unit(map, map->curunit);
-				goto blink;
-			}
-			/* Start over from beginning of list. */
-			unit = find_next_awake_mover(dside, NULL);
-			if (unit
-				&& unit->plan
-				&& !unit->plan->asleep
-				&& !unit->plan->reserve
-				&& !unit->plan->delayed
-				&& unit->plan->waitingfortasks
-				) {
-				map->curunit = unit;
-				select_exactly_one_unit(map, map->curunit);
-				goto blink;
-			}
-		blink:
-		if (map->curunit != NULL) {
-			int tick_count = TickCount();
-			    
-			/* Use blinking current unit if asked to. */
-			if (map->blinking_curunit == TRUE
-			      && tick_count - last_tick_count >= 20) {
-				int curcp = map->curunit->cp;
-
-				last_tick_count = tick_count;
-				/* Make sure we have a nonzero state that can switch sign. */
-		    		if (animation_pattern_state == 0)
-				    animation_pattern_state = -1;
-				/* Blink. */
-				animation_pattern_state = -animation_pattern_state;	    	    
-				/* Set cp to bogus zero in order to force drawing as a gray
-				incomplete unit. */
-				if (animation_pattern_state < 0)
-				    map->curunit->cp = 0;
-				update_cell(map, map->curunit->x, map->curunit->y);
-				/* Restore correct cp. */
-				map->curunit->cp = curcp;
-			/* Else use animated selection box. */
-			} else if (map->blinking_curunit != TRUE
-				      && tick_count - last_tick_count >= 10) {
-					last_tick_count = tick_count;
-				    animation_pattern_state = (animation_pattern_state + 1) % 8;
-				    draw_selection_animation(map, map->curunit);
-			}
-		}
+	int	ticks = TickCount();
+	Map	*map = frontmap;
+	Unit	*unit;
+	
+	if (beforestart 
+	    || endofgame
+	    || map == NULL
+	    || map->autoselect != TRUE) {
+		return;
 	}
+	
+	/* First check if we have a valid curunit. */
+	unit = map->curunit;
+	if (could_be_next_unit(unit)) {
+		/* If so, make sure it is selected. */
+		if (!unit_is_selected(map, unit)) {
+	    		select_unit_on_map(map, unit);
+		}		 	
+		goto blink;
+	}
+	/* Else look nearby for a possible new curunit. */	
+	unit = autonext_unit_inbox(dside, map->curunit, map->vp);
+	if (select_new_unit(map, unit)) {
+		goto blink;
+	}
+	/* Look for the next possible curunit in the list. */
+	unit = find_next_awake_mover(dside, map->curunit);
+	if (select_new_unit(map, unit)) {
+		goto blink;
+	}
+	/* Start over from beginning of list. */
+	unit = find_next_awake_mover(dside, NULL);
+	if (select_new_unit(map, unit)) {
+		goto blink;
+	}
+	/* We failed. Unselect any old non-valid curunit ... */
+	if (map->curunit) {
+		unselect_unit_on_map(map, map->curunit);
+		update_cell(map, map->curunit->x, map->curunit->y);
+		map->curunit = NULL;
+		/* Make sure buildwin is closed. */
+		close_window(buildwin);
+	}
+	/* ... and get out of here. */
+	return;
+
+	blink:
+	/* Scroll to curunit if asked to. */
+    	if (map->scrolltocurunit) {
+			scroll_to_unit(map, map->curunit);
+			map->scrolltocurunit = FALSE;
+		}
+	/* Use blinking current unit if asked to. */
+	if (map->blinking_curunit == TRUE
+	      && ticks - last_tick_count >= 20) {
+		int curcp = map->curunit->cp;
+
+		last_tick_count = ticks;
+		/* Make sure we have a nonzero state that can switch sign. */
+    		if (animation_pattern_state == 0)
+		    animation_pattern_state = -1;
+		/* Blink. */
+		animation_pattern_state = -animation_pattern_state;	    	    
+		/* Set cp to bogus zero in order to force drawing as a gray
+		incomplete unit. */
+		if (animation_pattern_state < 0)
+		    map->curunit->cp = 0;
+		update_cell(map, map->curunit->x, map->curunit->y);
+		/* Restore correct cp. */
+		map->curunit->cp = curcp;
+		/* Else use animated selection box. */
+	} else if (map->blinking_curunit != TRUE
+      	      && ticks - last_tick_count >= 10) {
+		last_tick_count = ticks;
+		animation_pattern_state = (animation_pattern_state + 1) % 8;
+		draw_selection_animation(map, map->curunit);
+	}
+}
+
+/* Try to select the next unit. Return true if successful. */
+
+int
+select_new_unit(Map *map, Unit *unit)
+{
+	int 		sx, sy;
+	Point		point;
+	
+	if (could_be_next_unit(unit)) {
+		map->curunit = unit;
+		select_exactly_one_unit(map, unit);
+		xform(map, unit->x, unit->y, &sx, &sy);
+		point.h = sx + map->vp->hw/2; 
+		point.v = sy + map->vp->hh/2; ;
+		/* Scroll to the new curunit if it is hidden from view. */
+	    	if (PtInRgn(point, map->window->visRgn) != TRUE)
+		    scroll_to_unit(map, map->curunit);
+		/* If something can build but cannot move, we assume
+		it is waiting for a build task (should be more refined
+		test). */
+		if (can_build(unit) && !mobile(unit->type))
+		    show_unit_build_dialog(unit);
+		return TRUE;
+	}
+	return FALSE;	
 }
 
 /* Used to check for any unread required parameters. Returns true if we
@@ -1838,13 +1835,19 @@ void
 update_side_display(Side *side, Side *side2, int rightnow)
 {
 	GrafPtr oldport;
+	int newnumsides;
 	extern int gamenumsides;
 
 	if (active_display(side)) {
 		if (gamewin != nil
-		    && side2 != indepside 
+		    &&  !inactive_indepside(side2) 
 		    && ((WindowPeek) gamewin)->visible) {
-			if (gamenumsides == numsides) {
+			if (side_has_ai(indepside) || side_has_display(indepside)) {
+				newnumsides = numtotsides;
+			} else {
+				newnumsides = numsides;
+			}
+			if (gamenumsides == newnumsides) {
 				GetPort(&oldport);
 				SetPort(gamewin);
 				draw_side_status(side2);
@@ -1858,21 +1861,19 @@ update_side_display(Side *side, Side *side2, int rightnow)
 			}
 		}
 		/* Handle the research window if necessary. */
-		if (numatypes > 0 && g_side_can_research()) {
+		if (numatypes > 0 
+		    && g_side_can_research()) {
 			/* Research is under human control and we need a new topic. */
-			if (!side_has_ai(dside) && dside->research_topic == NOADVANCE) {
-				 if (dside->autoresearch) {
-					auto_pick_side_research(dside);
-					if (researchwin)
-					    /* We call draw_research_dialog here passing false instead of 
-					    update_window, which would force redrawing. */
-					    draw_research_dialog(false);
-				/* Pick new topic manually. */
-				} else {
-					show_research_dialog();
-				}
-			/* Always update the window, but only if it exists. */
+			if (!side_has_ai(dside) 
+			    && !dside->autoresearch
+			    && dside->research_topic == NOADVANCE) {
+				show_research_dialog();
+			/* else update the window, but only if it exists. */
 			} else if (researchwin) {
+				/* We call draw_research_dialog here passing false instead of 
+				true, which would force redrawing. Now the dialog tests if the text
+				has changed before deciding on a redraw. Note: update_window does
+				not work with dialogs. */
 				draw_research_dialog(false);
 			}
 		}
@@ -1910,6 +1911,7 @@ update_turn_display(Side *side, int rightnow)
 		}
 		/* This routine might have been called because the game is over. */
 		if (endofgame && !told_outcome && side == dside) {
+		    draw_game();
 		    if (!side->see_all) {
 		    	/* Can't do update forcing here because modal dialogs will
 		    	   come up before windows actually get to update. */
@@ -1919,12 +1921,12 @@ update_turn_display(Side *side, int rightnow)
 					draw_map(map);
 				}
 		    }
-			if (side_won(dside)) {
+		    if (side_won(dside)) {
 		    	won_game_dialog();
 		    } else if (side_lost(dside)) {
 		    	lost_game_dialog();
 		    } else {
-				game_over_dialog();
+			game_over_dialog();
 			}
 			told_outcome = TRUE;
 		}
@@ -2142,6 +2144,15 @@ update_unit_acp_display(Side *side, Unit *unit, int rightnow)
 		    && ((WindowPeek) (unitcloseup->window))->visible) {
 			update_window(unitcloseup->window);
 		}
+		if (buildwin != nil
+		    && ((WindowPeek) buildwin)->visible
+		    && unit == buildwin_unit) {
+			/* We call draw_unit_build_dialog here passing false instead of 
+			true, which would force redrawing. Now the dialog tests if the text
+			has changed before deciding on a redraw. Note: update_window does
+			not work with dialogs. */
+			draw_unit_build_dialog(false);
+		}
 	}
 }
 
@@ -2232,7 +2243,7 @@ won_game_dialog()
 		}
 	}
 	DisposeDialog(win);
-	update_all_map_windows();
+	force_overall_update();
 }
 
 /* Bring up a modal dialog saying that the player has lost. */
@@ -2269,7 +2280,7 @@ lost_game_dialog()
 		}
 	}
 	DisposeDialog(win);
-	update_all_map_windows();
+	force_overall_update();
 }
 
 /* Bring up a modal dialog saying that the game has ended, with no
@@ -2299,7 +2310,7 @@ game_over_dialog()
 		}
 	}
 	DisposeDialog(win);
-	update_all_map_windows();
+	force_overall_update();
 }
 
 /* Update the displays to reflect the arrival of a message from another
@@ -2392,22 +2403,55 @@ play_movies(SideMask sidemask)
 	Unit *unit;
 	GrafPtr oldport, curport = NULL;
 	long startcount;
+	int play = FALSE;
 
     if (!side_in_set(dside, sidemask))
       return;
     if (movies == NULL)
       return;
-	if (1 /* anything visible */) {
+
+    /* First test if we really need to play any movies. */
+    for (j = 0; j < numscheduled; ++j) {
+	/* We just decided that we need to play at least one movie. */
+         if (play)
+             break;
+	switch (movies[j].itype) {
+		case movie_null:
+			break;
+		case movie_miss:
+		case movie_hit:
+		case movie_death:
+			unitid = movies[j].args[0];
+			unit = find_unit(unitid);
+			if (unit == NULL
+			    || !in_area(unit->x, unit->y)
+			    || !units_visible(dside, unit->x, unit->y))
+			  break;
+			for_all_maps(map) {
+				/* Only play movies that will be drawn. Skip the world map. */
+				if (map != worldmap
+				    && cell_is_in_gworld(map, unit->x, unit->y)) {
+					play = TRUE;
+					break;
+				}
+			}
+			break;
+		case movie_nuke:
+		case movie_flash:
+			play = TRUE;	/* Should test if visible on any map. */
+			break;
+		case movie_sound:
+			play = TRUE;	/* Should test that the units involved are visible. */
+			break;
+		default:
+			break;
+		}
+	}
+
+	/* Then proceed if necessary to waste those precious cycles. */
+	if (play) {
 		GetPort(&oldport);
 		startticks = TickCount();
-		/* Tweak the pen modes of all the maps. */
-		for_all_maps(map) {
-			SetPort(map->window);
-			if (map->vp->hw > 10)
-			  PenSize(2, 2);
-			else
-			  PenSize(1, 1);
-		}
 		while (TickCount() < startticks + 32) {
 			innerticks = TickCount();
 			for (j = 0; j < numscheduled; ++j) {
@@ -2419,19 +2463,19 @@ play_movies(SideMask sidemask)
 					case movie_death:
 						unitid = movies[j].args[0];
 						unit = find_unit(unitid);
-						if (unit == NULL || !in_area(unit->x, unit->y))
+						if (unit == NULL 
+						    || !in_area(unit->x, unit->y)
+						    || !units_visible(dside, unit->x, unit->y))
 						  continue;
 						blasttype = (movies[j].itype == movie_miss ? 0 :
 									 (movies[j].itype == movie_hit ? 1 : 2));
-						PenMode(patXor);
 						for_all_maps(map) {
-							if (curport != map->window) {
-								SetPort(map->window);
-								curport = map->window;
-							}
+							/* Only play movies that will be drawn. Skip the world map. */
+							if (map != worldmap
+							    && cell_is_in_gworld(map, unit->x, unit->y)) {
 							draw_unit_blast(map, unit, blasttype);
 						}
-						PenNormal();
+						}
 						startcount = TickCount();
 						play_sound("crunch");
 						if (blasttype > 0)
@@ -2440,15 +2484,9 @@ play_movies(SideMask sidemask)
 						while ((TickCount() - startcount) < 10)
 						  ;
 						break;
-					case movie_nuke:
-						/* ??? */
-						break;
+					case movie_nuke:	/* Temporary solution - use flash also for nukes. */
 					case movie_flash:
 						for_all_maps(map) {
-							if (curport != map->window) {
-								SetPort(map->window);
-								curport = map->window;
-							}
 							invert_map(map, movies[j].args[0], movies[j].args[1], area.maxdim);
 						}
 						startcount = TickCount();
@@ -2456,10 +2494,6 @@ play_movies(SideMask sidemask)
 						while ((TickCount() - startcount) < 10)
 						  ;
 						for_all_maps(map) {
-							if (curport != map->window) {
-								SetPort(map->window);
-								curport = map->window;
-							}
 							invert_map(map, movies[j].args[0], movies[j].args[1], area.maxdim);
 						}
 						break;
@@ -2487,22 +2521,22 @@ play_movies(SideMask sidemask)
 				case movie_death:
 					unitid = movies[j].args[0];
 					unit = find_unit(unitid);
-					if (unit == NULL || !in_area(unit->x, unit->y))
+					if (unit == NULL 
+					    || !in_area(unit->x, unit->y)
+					    || !units_visible(dside, unit->x, unit->y))
 					  continue;
-					blasttype = (movies[j].itype == movie_miss ? 0 :
-								 (movies[j].itype == movie_hit ? 1 : 2));
-					for_all_maps(map)
+					for_all_maps(map) {
+						/* Only erase blasts that really were drawn. */
+						if (map != worldmap
+						    && cell_is_in_gworld(map, unit->x, unit->y)) {
 					  update_cell(map, unit->x, unit->y);
+						}
+					}
 					break;
 				default:
 					break;
 			}
 		}
-		/* Restore the pen modes of all the maps. */
-		for_all_maps(map) {
-			SetPort(map->window);
-			PenNormal();
-		}
 		SetPort(oldport);
 	}
 	numscheduled = 0;
