@@ -263,6 +263,7 @@ typedef struct a_map {
 	short drawsizes;		/* Draw unit (city) size numbers */
 	short draw_emblems;
 	short draw_ai;
+	short draw_transitions;
 	short see_all;
 	short autoselect;
 	Unit *curunit;
@@ -439,12 +440,20 @@ extern int nummaps;
 extern ImageFamily **uimages;
 extern ImageFamily **timages;
 extern ImageFamily **eimages;
+extern ImageFamily **mimages;
 extern MacImage **tcolors;
 
+extern ImageFamily *generic_transition;
+extern ImageFamily *beach;
+
 extern WindowPtr helpwin;
 extern char *helpstring;
 extern TEHandle helptext;
+
 extern DialogPtr researchwin;
+extern DialogPtr buildwin;
+
+extern Unit *buildwin_unit;
 
 extern WindowPtr commandwin;
 extern WindowPtr instructionswin;
@@ -510,6 +519,8 @@ extern struct a_map *maplist;	   /* chain of maps that we're using */
 extern struct a_list *listlist;	 /* chain of lists */
 extern struct a_unit_closeup *unitcloseuplist;	 /* chain of unit closeups */
 
+struct cmdtab;	/* Shuldn't this be extern as well? */
+
 extern CursHandle paintcursors[];
 extern CursHandle bordpaintor;
 extern CursHandle connpaintor;
@@ -534,14 +545,16 @@ extern enum grays unseengray;
 extern enum grays bggray;
 
 extern RGBColor blackcolor;
+extern RGBColor dkgraycolor;
 extern RGBColor graycolor;
+extern RGBColor ltgraycolor;
+extern RGBColor whitecolor;
 
 extern int conwid;
 extern int tophgt;
 
 extern int first_windows;
 
-extern short daynight;
 extern int grid_matches_unseen;
 
 extern BitMap *bordbitmaps;
@@ -679,433 +692,132 @@ extern short prefs_refnum;		/* frefNum of Xconq Preferences file */
 /* Note that although the Xconq kernel must conditionalize its prototypes,
    all Mac C compilers are sufficiently standard C to handle them. */
 
+/* maccmd.c */
+
+extern void do_keyboard_command(int key);
+
+extern int do_one_give_unit(Unit *unit);
+extern int do_move_to_command(void);
+
+extern void show_command_window(void);
+extern void draw_command(void);
+extern void activate_command(int activate);
+extern int do_key_down_command(int keyCode, int charCode);
+extern void do_mouse_down_command(Point mouse, int mods);
+
+extern void toggle_profiling(void);
+
+/* maccolors.c */
+
+extern void set_default_colors(void);			/* Set default colors */
+extern void set_default_side_colors(int e);		/* Side colors Dialog called by Prefs popup */
+extern void set_default_feature_color(int e);	/* Feature color Dialog called by Prefs popup */
+
+extern void init_side_colors(void);			/* Grab memory, set colors to black, names to undefined */
+extern void dissect_colorscheme(int e);	 	/* Dissect a colorscheme into sideColorNames */
+
+extern void read_xcol(void);				/* Read RGB colors form XCol resource */
+extern void write_xcol(void);				/* Write RGB colors to XCol resource */
+
+extern void set_side_colors(int e);			/* Side colors Dialog called by Colors menu */
+extern void set_feature_color(int e);			/* Feature color Dialog called by Colors menu */
+extern void set_terrain_color(Str255 tname);	/* Feature color Dialog for terrain */
+
+extern void build_colors_menu(void);			/* Build the Colors menu */
+extern void build_sides_menu(void);			/* Build popup menu for prefs */
+extern void build_features_menu(void);		/* Build popup menu for prefs */
+
+/* macconq.c */
+
 extern int main(void);
-extern int splash_dialog(void);
-extern void connect_game_dialog(void);
-extern void recalc_depths(void);
-extern void get_files(void);
-extern int open_preferences(void);
-extern void close_preferences(void);
-extern void get_preferences(void);
-extern void save_preferences(void);
-extern void ui_update_state(void);
-extern void event_loop(void);
+extern void recalc_spacing(void);
 extern void get_global_mouse(Point *mouse);
-extern void adjust_cursor(Point mouse, RgnHandle region);
-extern void grow_window(WindowPtr win, Point where);
-extern void zoom_window(WindowPtr win, Point where, int part);
+
 extern void close_window(WindowPtr win);
-extern void do_mouse_down(WindowPtr window, EventRecord *event);
-extern void activate_window(WindowPtr win, int activate);
 extern void update_window(WindowPtr win);
-extern void maybe_select_next_unit(void);
-
-extern int is_da_window(WindowPtr win);
-extern int is_app_window(WindowPtr win);
-
-extern void won_game_dialog(void);
-extern void lost_game_dialog(void);
-extern void game_over_dialog(void);
-
-extern int position_on_screen(int h, int v);
-extern int position_already_used(int h, int v);
-extern GDHandle best_zoom_screen(Rect *rectptr);
-
-extern void force_update(WindowPtr win);
-extern void force_overall_update(void);
-
-extern void set_standard_state(WindowPtr win, int fullw, int fullh);
-extern void get_main_screen_size(int *widp, int *hgtp);
-
-extern void draw_default_button(DialogPtr dialog, short ditem);
-
-extern void stagger_window(WindowPtr win, int *lasthp, int *lastvp);
-
-extern void set_end_of_game_interaction_modes(void);
-extern void draw_selection_animation(Map *map, Unit *unit);
-
-extern void exit_macconq(void);
-
-/* Map-handling prototypes. */
-
-extern int at_all_visible(Map *map, int x, int y);
-extern int in_middle(Map *map, int x, int y);
-
-extern void xform(Map *map, int x, int y, int *sxp, int *syp);
-extern void xform_flat(Map *map, int x, int y, int *sxp, int *syp);
-extern void xform_fractional(Map *map, int x, int y, int xf, int yf, int *sxp, int *syp);
-extern void xform_fractional_flat(Map *map, int x, int y, int xf, int yf, int *sxp, int *syp);
-extern void m_xform_unit(Map *map, Unit *unit, int *sxp, int *syp, int *swp, int *shp);
-extern void m_xform_unit_self(Map *map, Unit *unit, int *sxp, int *syp, int *swp, int *shp);
-extern void m_xform_occupant(Map *map, Unit *transport, Unit *unit, int sx, int sy, int sw, int sh, int *sxp, int *syp, int *swp, int *shp);
-
-extern Map *create_map(int power, int id);
-extern void set_content_rect(Map *map);
-extern void m_focus_on_center(Map *map);
-extern void m_center_on_focus(Map *map);
-extern void set_map_scrollbars(Map *map);
-extern void set_map_power(Map *map, int power);
-extern void make_cell_clip(int power);
-extern Map *map_from_window(WindowPtr window);
-
-extern void grow_map(Map *map, int w, int h);
-extern void zoom_map(Map *map, int part);
-
-extern void adjust_map_decor(Map *map);
-
-extern int m_nearest_cell(Map *map, int sx, int sy, int *xp, int *yp);
-extern int m_nearest_boundary(Map *map, int sx, int sy, int *xp, int *yp, int *dirp);
-extern int m_nearest_unit(Map *map, int sx, int sy, Unit **unitp);
-
-extern void draw_map(Map *map);
-extern void draw_window_background(Map *map);
-extern void draw_map_content(Map *map);
-extern void draw_area_background(Map *map);
-extern void draw_control_panel(Map *map);
-extern void draw_top_line(Map *map);
-extern void draw_other_maps(Map *map);
-extern void draw_related_maps(Map *map);
-extern void draw_other_map(Map *map, Map *map2);
-extern void draw_row(Map *map, int x0, int y0, int len, int clearit);
-
 extern void draw_growicon_nolines(WindowPtr win);
 
-extern void draw_selections(Map *map);
-extern void draw_selections_at(Map *map, int x, int y);
-extern void draw_selected_unit(Map *map, Unit *unit);
-extern void update_cell(Map *map, int x, int y);
-extern void force_map_update(Map *map);
-extern void destroy_map(Map *map);
-extern void activate_map(Map *map, int activate);
-extern void print_map(Map *map);
+extern void connect_game_dialog(void);
+extern void connection_method_dialog(void);
+
+extern void stagger_window(WindowPtr win, int *lasthp, int *lastvp);
+extern GDHandle best_zoom_screen(Rect *rectptr);
+extern void set_standard_state(WindowPtr win, int fullw, int fullh);
+extern void get_main_screen_size(int *widp, int *hgtp);
+extern void draw_default_button(DialogPtr dialog, short ditem);
+extern char *get_string_from_item(Handle itemhandle);
+extern void force_update(WindowPtr win);
+extern void force_overall_update(void);
+extern void beep(void);
+extern void set_game_file_type(char *name);
+extern void exit_macconq(void);
+
+/* macdesign.c */
+
+#ifdef DESIGNERS
+
+extern void enable_designing(int forsure);
+extern void disable_designing(void);
+extern CursHandle adjust_designer_cursor(Point mouse, RgnHandle region);
+extern void show_design_window(void);
+extern void draw_design_window(void);
+extern void do_mouse_down_design(Point mouse, int mods);
+extern void apply_designer_tool(Map *map, int h, int v, int mods);
+extern void designer_save_dialog(void);
+
+#endif /* DESIGNERS */
+
+/* macdraw.c */
 
 extern void draw_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, int u, int s, int sidecolors, int selected, int mod, int emblem);
 extern void draw_side_emblem(WindowPtr win, int ex, int ey, int ew, int eh, int e, int style);
+
 extern void draw_cell_block(int sx, int sy, int n, int power, int t, int over, int angle, int dosolid);
-extern void calc_best_terrain_images(void);
-extern void draw_hex_region(int sx, int sy, int power, int t, int over, int angle,
-							int dogrid, int dosolid, int var);
+extern void draw_hex_region(int sx, int sy, int power, int t, int over, int angle, int dogrid, int dosolid, int var);
+extern void make_cell_clip(int power);
+
+extern void draw_terrain_transitions(Map *map, int x0, int y0, int len);
+extern void draw_shore_images(Map *map, int x, int y);
+extern void draw_border_images(Map *map, int x, int y, int b);
 extern void draw_border_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power, int t, int angle, int over, int dosolid);
 extern void draw_connection_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power, int t, int angle, int over, int dosolid);
-extern void draw_terrain_sample(WindowPtr win, Rect tmprect, int t);
+extern void draw_terrain_sample(WindowPtr win, Rect tmprect, int t, int power);
 extern void draw_country_borders(WindowPtr win, int sx, int sy, int bitmask, int power, int shade, int angle);
 extern void draw_feature_borders(WindowPtr win, int sx, int sy, int bitmask, int power);
 extern void draw_ai_region_borders(WindowPtr win, int sx, int sy, int bitmask, int power);
+#if 0
 extern int draw_elevation_here(int x, int y);
 extern void draw_elevation(int sx, int sy, int power, int elev);
+#endif
 extern void draw_temperature(int sx, int sy, int power, int temp);
-extern void draw_winds(int sx, int sy, int power, int rawwind);
 extern int draw_clouds_here(int x, int y);
 extern void draw_cloud_block(int sx, int sy, int i, int power, int cloudtype, int angle);
 extern void draw_clouds(int sx, int sy, int power, int cloudtype);
+extern void draw_winds(int sx, int sy, int power, int rawwind);
 extern void draw_coverage(int sx, int sy, int power, int cov, int altcov);
+
 extern void draw_unit_name(Unit *unit, int sx, int sy, int sw, int sh, int masks, int optimize);
 extern void draw_legend_text(int sx, int sy, int sh, char *legend, int just, int masks, int optimize);
 extern void draw_blast_image(WindowPtr win, int sx, int sy, int sw, int sh, int blasttype);
+#if 0
 extern void clear_blast_image(WindowPtr win, int sx, int sy, int sw, int sh, int blasttype);
+#endif
 extern int picture_width(PicHandle pichandle);
 extern int picture_height(PicHandle pichandle);
 extern void plot_sicn(WindowPtr win, int sx, int sy, Handle sicnhandle, int n, int erase, int mode);
 extern void gray_out_rect(Rect *rectptr);
-extern void draw_unit_blast(Map *map, Unit *unit, int blast);
-extern void draw_unit_size(Unit *unit, WindowPtr win, int sx, int sy, int sw, int sh);
 
-extern void do_mouse_down_map(Map *map, Point mouse, int mods);
-extern void do_mouse_down_map_control_panel(Map *map, int h, int v, int mods);
-extern void toggle_survey(Map *map);
-extern void magnify_map(Map *map, int inout);
-extern void set_map_mag(Map *map, int newpower);
-extern void toggle_map_grid(Map *map);
-extern void toggle_map_topline(Map *map);
-extern void toggle_map_topunit(Map *map);
-extern void toggle_map_scrollbars(Map *map);
-extern void toggle_map_cpanel(Map *map);
-extern void toggle_map_other_maps(Map *map);
-extern void toggle_map_lighting(Map *map);
-extern void toggle_map_coverage(Map *map);
-extern void toggle_map_names(Map *map);
-extern void toggle_map_people(Map *map);
-extern void toggle_map_control(Map *map);
-extern void toggle_map_elevations(Map *map);
-extern void toggle_map_materials(Map *map, int m);
-extern void toggle_map_aux_terrain(Map *map, int t);
-extern void toggle_map_temperature(Map *map);
-extern void toggle_map_winds(Map *map);
-extern void toggle_map_clouds(Map *map);
-extern void toggle_map_storms(Map *map);
-extern void toggle_map_plans(Map *map);
-extern void toggle_map_ai(Map *map);
-extern void toggle_map_draw_emblems(Map *map);
-extern void toggle_blinking_curunit(Map *map);
-extern void toggle_selectionmasks(Map *map);
+/* macgworld.c */
 
-extern void do_mouse_down_map_content(Map *map, int h, int v, int mods);
-extern void select_all_dragged_over(Map *map, int h0, int v0, int mods);
-extern void select_area_and_zoom(Map *map, int h0, int v0, int mods);
-extern void move_on_drag(Map *map, Unit *unit, int mods);
-extern void unselect_all(Map *map);
-extern void select_unit_on_map(Map *map, Unit *unit);
-extern int unit_is_selected(Map *map, Unit *unit);
-extern void unselect_unit_on_map(Map *map, Unit *unit);
-extern void select_all_units_in_rect(Map *map, Rect *rectptr);
-extern int move_the_selected_unit(Map *map, Unit *unit, int h, int v);
-extern void fire_the_selected_unit(Map *map, Unit *unit, int h, int v);
-extern void select_exactly_one_unit(Map *map, Unit *unit);
-extern void select_next_unit(Map *map);
-extern void select_previous_unit(Map *map);
-extern void select_next_actor(Map *map);
-extern void select_previous_actor(Map *map);
-extern void select_next_mover(Map *map);
-extern void select_previous_mover(Map *map);
-extern void select_next_awake_mover(Map *map);
-extern void select_previous_awake_mover(Map *map);
-extern void select_another(Map *map, Unit *(*fn)(Side *side, Unit *unit));
-extern void scroll_best_map_to_unit(Unit *unit, int bringtofront);
-extern void scroll_to_unit(Map *map, Unit *unit);
-extern void magnify_to_fit(Map *map, int x1, int y1, int x2, int y2);
+extern void check_screen_depths(void);				/* Check for more than 256 colors */
 
-extern void create_list(void);
-extern void init_list_contents(List *list);
-extern void organize_list_contents(List *list);
-extern void sort_list_contents(List *list);
-extern void add_unit_to_list(List *list, Unit *unit);
-extern void set_list_scrollbars(List *list);
-extern List *list_from_window(WindowPtr window);
-extern void draw_list(List *list);
-extern void draw_list_contents(List *list);
-extern void draw_list_headings(List *list);
-extern void draw_unit_list_entry(List *list, int n, int clearfirst);
-extern void grow_list(List *list, int w, int h);
-extern void zoom_list(List *list, int part);
-extern void adjust_list_decor(List *list);
-extern void do_mouse_down_list(List *list, Point mouse, int mods);
-extern int do_key_down_list(List *list, char key);
-extern void set_list_sorting(List *list, enum sortkeys newkey, int mi);
-extern void toggle_list_completed(List *list);
-extern void toggle_list_incomplete(List *list);
-extern void toggle_list_large_icons(List *list);
-extern void update_unit_in_lists(Unit *unit);
-extern int unit_position_in_list(List *list, Unit *unit);
-extern void reorganize_list(List *list);
-extern void redraw_unit_list_entry(List *list, int n);
-extern void clear_selections(List *list);
-extern Unit *selected_unit_in_list(List *list);
-extern void scroll_to_selected_unit_in_list(List *list);
-extern void activate_list(List *list, int activate);
-extern void print_list(List *list);
-extern void destroy_list(List *list);
+extern void scroll_map_window(Map *map, int dx, int dy);	/* Scroll map window within its gworld */
+extern void copy_from_gworld(Map *map, Rect destRect);	/* Copy rect from gworld to screen */
+extern void update_resized_map(Map *map);			/* Update map and then redraw its gworld */
+extern void update_gworld(Map *map);					/* Update the gworld (and redraw the map)*/
+extern int cell_is_in_gworld(Map *map, int x, int y);		/* Test if cell is within current gworld */
 
-extern void init_patterns(void);
-extern void init_icons(void);
-extern void init_cursors(void);
-extern int do_splash_box(void);
-extern void new_game_dialog(void);
-extern int start_new_game(void);
-extern void update_new_game_list(void);
-extern int variants_dialog(void);
-extern int open_game_dialog(void);
-extern int open_game_from_fsspec(FSSpec *fsspec);
-extern int open_game_from_name_and_volume(Str255 name, int vrefnum);
-extern void handle_player_setup_event(EventRecord *event);
-extern void hit_player_setup_dialog(int itemhit, Point mouse);
-extern void hit_closeup_dialog(WindowPtr win, int itemhit, Point mouse);
-extern int do_key_down_closeup(UnitCloseup *closeup, char key);
-extern int launch_game(void);
-extern void launch_game_2(void);
-extern void check_for_missing_images(void);
-extern void open_progress_dialog(void);
-extern void close_progress_dialog(void);
-extern void init_display(void);
-extern void init_unit_images(void);
-extern void init_emblem_images(void);
-extern void init_terrain_images(void);
-extern void init_menus(void);
-extern void add_window_menu_item(char *name, WindowPtr win);
-extern void remove_window_menu_item(WindowPtr win);
-extern void build_side_menu(void);
-extern void update_side_menu(Side *side2);
-extern void build_unit_type_menu(void);
-extern void build_material_type_menu(void);
-extern void build_terrain_type_menu(void);
-extern void build_ai_type_menu(void);
-extern void build_feature_menu(void);
-extern void build_optional_terrain_type_menu(void);
-extern void sanitize_for_menu(char *str, Str255 outstr);
-extern void do_menu_command(long which);
-extern void do_about_box(void);
-extern void save_the_game(int askname, int quitting);
-extern void set_preferences(void);
-extern void maybe_init_print(void);
-extern void quit_the_game(void);
-extern void apply_to_all_selected(int (*fn)(), int beepfailure);
-extern void do_reserve_command(int value, int radius, int recurse);
-extern void do_sleep_command(int value, int radius, int recurse);
-extern void show_construction_window(void);
-extern int do_move_to_command(void);
-extern void adjust_menus(void);
-
-extern int add_unit_position(Unit *unit);
-extern void set_map_focus(Map *map, int x, int y);
-extern void set_map_angle(Map *map, int angle);
-extern void message_dialog(int sidenum);
-extern void do_keyboard_command(int key);
-
-struct cmdtab;
-
-extern void toggle_profiling(void);
-
-extern char *get_string_from_item(Handle itemhandle);
-
-extern void show_instructions_window(void);
-extern void create_instructions_window(void);
-extern void activate_instructions(int activate);
-extern void do_mouse_down_instructions(Point mouse, int mods);
-extern void draw_instructions(void);
-
-extern void show_game_window(void);
-extern void create_game_window(void);
-extern void draw_game(void);
-extern void draw_game_date(void);
-extern void draw_game_clocks(void);
-extern void draw_game_progress(void);
-extern void draw_game_side(Side *side2);
-extern int feeling_towards(Side *side, Side *side2);
-extern void draw_side_status(Side *side2);
-extern void do_mouse_down_game(Point mouse, int mods);
-
-extern void create_construction_window(void);
-extern void init_construction_lists(void);
-extern void reinit_construction_lists(void);
-extern void draw_construction(void);
-extern void draw_construction_default(void);
-extern void calc_construction_rects(void);
-extern void activate_construction(int activate);
-extern Unit *get_selected_construction_unit(void);
-extern int get_selected_construction_type(void);
-extern void scroll_to_selected_construction_unit(void);
-extern void do_mouse_down_construction(Point mouse, int mods);
-extern void select_unit_in_construction_window(Unit *unit);
-extern void select_type_in_construction_window(int u);
-extern void update_construction_unit_list(Unit *unit);
-extern void maybe_add_unit_to_construction_list(Unit *unit);
-extern void update_unit_list_for_type(int u);
-extern void update_construction_type_list(void);
-extern void update_type_list_for_unit(Unit *unit);
-extern void adjust_construction_controls(void);
-extern void grow_construction(int h, int v);
-extern void zoom_construction(int part);
-extern int do_key_down_construction(int key);
-
-extern void side_rename_dialog(Side *side);
-
-extern int unit_rename_dialog(Unit *unit);
-
-extern void show_unit_closeup(Unit *unit);
-extern UnitCloseup *find_unit_closeup(Unit *unit);
-extern UnitCloseup *create_unit_closeup(Unit *unit);
-extern void preferred_closeup_size(int u, int *widp, int *hgtp);
-extern void draw_unit_closeup(UnitCloseup *unitcloseup);
-extern UnitCloseup *unit_closeup_from_window(WindowPtr win);
-extern int do_mouse_down_unit_closeup(UnitCloseup *unitcloseup, Point mouse, int mods);
-extern void destroy_unit_closeup(UnitCloseup *unitcloseup);
-extern MenuHandle build_construction_menu(Unit *unit);
-extern MenuHandle build_research_menu(Side *side);
-
-extern void show_history_window(void);
-extern void create_history_window(void);
-extern void draw_history(void);
-extern void draw_historical(HistEvent *hevt, int y, int drawevt);
-extern void do_mouse_down_history(Point mouse, int mods);
-extern void move_history_scrollbar(int h, int v);
-extern void grow_history(int h, int v);
-extern void zoom_history(int part);
-extern void update_history_window(HistEvent *hevt);
-
-extern void show_notice_window(void);
-extern void create_notice_window(void);
-extern void append_notice(char *str);
-extern void draw_notice(void);
-extern void adjust_notice_scrollbar(void);
-extern void activate_notice(int activate);
-extern void do_mouse_down_notice(Point mouse, int mods);
-extern void grow_notice(int h, int v);
-extern void zoom_notice(int part);
-
-extern void show_scores_window(void);
-extern void create_scores_window(void);
-extern void append_scores(char *str);
-extern void draw_scores(void);
-extern void adjust_scores_scrollbar(void);
-extern void activate_scores(int activate);
-extern void do_mouse_down_scores(Point mouse, int mods);
-extern void grow_scores(int h, int v);
-extern void zoom_scores(int part);
-
-extern void show_research_dialog();
-extern void draw_research_dialog(int force);
-extern int do_key_down_research(char key);
-extern void hit_research_dialog(int ditem);
-
-extern void draw_command(void);
-extern void activate_command(int activate);
-extern int do_key_down_command(int key);
-extern void do_mouse_down_command(Point mouse, int mods);
-
-extern void show_help_window(HelpNode *helpnode);
-extern void describe_menus(int arg, char *key, TextBuffer *buf);
-extern void describe_mouse(int arg, char *key, TextBuffer *buf);
-extern void describe_help(int arg, char *key, TextBuffer *buf);
-extern void create_help_window(void);
-extern void set_help_content(HelpNode *helpnode);
-extern void draw_help(void);
-extern void adjust_help_scrollbar(void);
-extern void activate_help(int activate);
-extern int do_key_down_help(char key);
-extern void do_mouse_down_help(Point mouse, int mods);
-extern void grow_help(int h, int v);
-extern void zoom_help(int part);
-
-#ifdef DESIGNERS
-extern void enable_designing(int forsure);
-extern void disable_designing(void);
-extern void init_design_cursors(void);
-extern CursHandle adjust_designer_cursor(Point mouse, RgnHandle region);
-extern void show_design_window(void);
-extern void create_design_window(void);
-extern void draw_design_window(void);
-extern void draw_design_window_tool(enum tooltype tool);
-extern void do_mouse_down_design(Point mouse, int mods);
-extern void mark_allowed_unit_types(void);
-extern void mark_allowed_sides(void);
-extern void feature_rename_dialog(Feature *feature);
-extern void apply_designer_tool(Map *map, int h, int v, int mods);
-extern void paint_on_drag(Map *map, int h0, int v0, int mods);
-extern void border_on_drag(Map *map, int h0, int v0, int mods, int paintmode);
-extern void connect_on_drag(Map *map, int h0, int v0, int mods, int paintmode);
-extern void designer_save_dialog(void);
-extern void designer_reshape_dialog(Module *module);
-#endif /* DESIGNERS */
-
-extern void update_unit_in_maps(Unit *unit);
-extern void beep(void);
-extern void set_game_file_type(char *name);
-
-extern int get_a_position(Map **map, int *xp, int *yp, int *dirp);
-extern int get_a_unit(Map **mapp, Unit **unitp);
-
-extern void interp_mac_ui_data(Obj *uispec);
-
-extern void draw_unit_info(Map *map);
-extern void connection_method_dialog(void);
-extern void play_sound(char *soundname);
-
-extern void query_position_modally(int mode);
-extern void set_position_modally(void);
-
-extern void show_command_window(void);
-
-extern void invert_map(Map *map, int x, int y, int r);
-extern Map *front_map(void);
+extern int handle_numeric_keypad(EventRecord *event);	/* Enables special use of numeric keypad */
 
 extern void update_all_dialogs(void);
 extern void update_all_windows(void);
@@ -1114,45 +826,237 @@ extern void update_all_visible_dialogs(void);
 extern void update_all_visible_windows(void);
 extern void update_all_visible_map_windows(void);
 
-extern void scroll_map_window(Map *map, int dx, int dy);	/* Scroll map window within its gworld */
-extern void copy_from_gworld(Map *map, Rect destRect);	/* Copy rect from gworld to screen */
-extern void update_resized_map(Map *map);			/* Update map and then redraw its gworld */
-extern void update_gworld(Map *map);					/* Update the gworld (and redraw the map)*/
+/* machelp.c */
 
-void set_meridians(Map *map);
-extern void set_offscreen_buffers(Map *map);			/* Set size of offscreen scrolling buffers */
-extern int handle_numeric_keypad(EventRecord *event);	/* Enables special use of numeric keypad */
-extern int cell_is_in_gworld(Map *map, int x, int y);		/* Test if cell is within current gworld */
+extern void show_instructions_window(void);
+extern void activate_instructions(int activate);
+extern int do_key_down_instructions(char key);
+extern void do_mouse_down_instructions(Point mouse, int mods);
+extern void draw_instructions(void);
 
-extern void toggle_map_sidecolors(Map *map);		
-extern void toggle_iconmasks(Map *map);
-extern void toggle_boxmasks(Map *map);
-extern void toggle_textmasks(Map *map);
-extern void toggle_featureborders(Map *map);
-extern void toggle_featurenames(Map *map);
-extern void toggle_shorelines(Map *map);
-extern void toggle_simple_borders(Map *map);
-extern void toggle_optimize_fonts(Map *map);
-extern void toggle_erase_names(Map *map);
-extern void toggle_draw_meridians(Map *map);
-extern void toggle_solid_color_terrain(Map *map);
+extern void show_help_window(HelpNode *helpnode);
+extern void draw_help(void);
+extern void activate_help(int activate);
+extern int do_key_down_help(char key);
+extern void do_mouse_down_help(Point mouse, int mods);
+extern void grow_help(int h, int v);
+extern void zoom_help(int part);
+
+/* macinit.c */
+
+extern void init_patterns(void);
+extern void init_icons(void);
+extern void init_cursors(void);
+extern int do_splash_box(void);
+
+extern void new_game_dialog(void);
+extern void do_mouse_down_setup(Point mouse, int mods);
+extern void do_key_down_setup(EventRecord *event);
+extern void add_remote_player(char *name);
+extern int open_game_dialog(void);
+extern int open_game_from_fsspec(FSSpec *fsspec);
+
+extern void init_unit_images(void);
+extern void init_emblem_images(void);
+
+/* maclist.c */
+
+extern void create_list(void);
+extern List *list_from_window(WindowPtr window);
+extern void draw_list(List *list);
+extern void grow_list(List *list, int w, int h);
+extern void zoom_list(List *list, int part);
+extern int do_key_down_list(List *list, char key);
+extern void do_mouse_down_list(List *list, Point mouse, int mods);
+
+extern void set_list_sorting(List *list, enum sortkeys newkey, int mi);
+extern void toggle_list_completed(List *list);
+extern void toggle_list_incomplete(List *list);
+extern void toggle_list_large_icons(List *list);
 extern void toggle_list_sidecolors(List *list);
 extern void toggle_list_draw_emblems(List *list);
 
-extern void read_xcol(void);				/* Read RGB colors form XCol resource */
-extern void write_xcol(void);				/* Write RGB colors to XCol resource */
-extern void check_screen_depths(void);		/* Check for more than 256 colors */
-extern void set_default_colors(void);			/* Set default colors */
-extern void init_side_colors(void);			/* Grab memory, set colors to black, names to undefined */
-extern void set_side_colors(int e);			/* Side colors Dialog called by Colors menu */
-extern void set_default_side_colors(int e);		/* Side colors Dialog called by Prefs popup */
-extern void set_default_feature_color(int e);	/* Feature color Dialog called by Prefs popup */
-extern void set_feature_color(int e);			/* Feature color Dialog called by Colors menu */
-extern void set_terrain_color(Str255 tname);	/* Feature color Dialog for terrain */
-extern void build_colors_menu(void);			/* Build the Colors menu */
-extern void build_sides_menu(void);			/* Build popup menu for prefs */
-extern void build_features_menu(void);		/* Build popup menu for prefs */
+extern void update_unit_in_lists(Unit *unit);
+extern void reorganize_list(List *list);
+extern Unit *selected_unit_in_list(List *list);
+extern void scroll_to_selected_unit_in_list(List *list);
 
-extern void dissect_colorscheme(int e);	 	/* Dissect a colorscheme into sideColorNames */
+extern void activate_list(List *list, int activate);
+extern void print_list(List *list);
+extern void destroy_list(List *list);
 
-extern void add_remote_player(char *name);
+/* macmap.c */
+
+extern void xform(Map *map, int x, int y, int *sxp, int *syp);
+extern void m_xform_unit(Map *map, Unit *unit, int *sxp, int *syp, int *swp, int *shp);
+extern void m_xform_unit_self(Map *map, Unit *unit, int *sxp, int *syp, int *swp, int *shp);
+extern void m_xform_occupant(Map *map, Unit *transport, Unit *unit, int sx, int sy, int sw, int sh, int *sxp, int *syp, int *swp, int *shp);
+extern int m_nearest_cell(Map *map, int sx, int sy, int *xp, int *yp);
+extern int m_nearest_boundary(Map *map, int sx, int sy, int *xp, int *yp, int *dirp);
+extern int m_nearest_unit(Map *map, int sx, int sy, Unit **unitp);
+extern int in_middle(Map *map, int x, int y);
+extern void m_center_on_focus(Map *map);
+
+extern Map *create_map(int power, int id);
+extern void set_content_rect(Map *map);
+extern void set_map_scrollbars(Map *map);
+extern void set_map_power(Map *map, int power);
+extern void set_map_focus(Map *map, int x, int y);
+extern void set_map_angle(Map *map, int angle);
+extern void grow_map(Map *map, int w, int h);
+extern void zoom_map(Map *map, int part);
+extern void adjust_map_decor(Map *map);
+extern void invert_map(Map *map, int x, int y, int r);
+extern void activate_map(Map *map, int activate);
+extern Map *map_from_window(WindowPtr window);
+extern void force_map_update(Map *map);
+extern void print_map(Map *map);
+extern void destroy_map(Map *map);
+
+extern void draw_map(Map *map);
+extern void draw_control_panel(Map *map);
+extern void draw_top_line(Map *map);
+extern void draw_unit_info(Map *map);
+extern void draw_related_maps(Map *map);
+
+extern void draw_unit_blast(Map *map, Unit *unit, int blast);
+extern void draw_selections(Map *map);
+extern void draw_selections_at(Map *map, int x, int y);
+extern void draw_selection_animation(Map *map, Unit *unit);
+
+extern void update_cell(Map *map, int x, int y);
+
+/* macmenus.c */
+
+extern void query_position_modally(int mode);
+extern void set_position_modally(void);
+
+extern void init_menus(void);
+extern void add_window_menu_item(char *name, WindowPtr win);
+extern void remove_window_menu_item(WindowPtr win);
+
+extern void build_side_menu(void);
+extern void update_side_menu(Side *side2);
+extern void build_unit_type_menu(void);
+extern void build_material_type_menu(void);
+extern void build_terrain_type_menu(void);
+extern void build_feature_menu(void);
+extern void build_optional_terrain_type_menu(void);
+extern void sanitize_for_menu(char *str, Str255 outstr);
+
+extern void do_menu_command(long which);
+extern void do_about_box(void);
+
+extern void save_the_game(int askname, int quitting);
+extern void quit_the_game(void);
+
+extern void apply_to_all_selected(int (*fn)(), int beepfailure);
+extern void adjust_menus(void);
+
+extern void toggle_map_grid(Map *map);
+extern void toggle_map_control(Map *map);
+extern void toggle_map_people(Map *map);
+extern void toggle_map_plans(Map *map);
+extern void toggle_map_ai(Map *map);
+extern void toggle_map_sidecolors(Map *map);		
+
+/* macmouse.c */
+
+extern void do_mouse_down_map(Map *map, Point mouse, int mods);
+extern void toggle_survey(Map *map);
+extern void magnify_map(Map *map, int inout);
+extern void set_map_mag(Map *map, int newpower);
+extern void unselect_all(Map *map);
+extern void select_unit_on_map(Map *map, Unit *unit);
+extern int unit_is_selected(Map *map, Unit *unit);
+extern void unselect_unit_on_map(Map *map, Unit *unit);
+extern void select_exactly_one_unit(Map *map, Unit *unit);
+extern void select_previous_unit(Map *map);
+extern void select_next_mover(Map *map);
+extern void scroll_best_map_to_unit(Unit *unit, int bringtofront);
+extern void scroll_to_unit(Map *map, Unit *unit);
+
+/* macprefs.c */
+
+extern void get_preferences(void);
+extern void save_preferences(void);
+extern void interp_mac_ui_data(Obj *uispec);
+extern void ui_update_state(void);
+extern void set_preferences(void);
+
+/* macrow.c */
+
+extern void draw_row(Map *map, int x0, int y0, int len, int clearit);
+extern void draw_gridlines(Map *map, int x, int y);
+extern void draw_shores(Map *map, int x, int y);
+
+/* macunit.c */
+
+extern UnitCloseup *find_unit_closeup(Unit *unit);
+extern UnitCloseup *unit_closeup_from_window(WindowPtr win);
+extern void show_unit_closeup(Unit *unit);
+extern void draw_unit_closeup(UnitCloseup *unitcloseup);
+extern void draw_unit_size(Unit *unit, WindowPtr win, int sx, int sy, int sw, int sh);
+extern int do_mouse_down_unit_closeup(UnitCloseup *unitcloseup, Point mouse, int mods);
+extern int do_key_down_closeup(UnitCloseup *closeup, char key, char code, int mods);
+extern void destroy_unit_closeup(UnitCloseup *unitcloseup);
+
+extern void show_unit_build_dialog(Unit *unit);
+extern void draw_unit_build_dialog(int force);
+extern int do_key_down_build(char key, char code, int mods);
+extern int do_mouse_down_build(Point mouse, int mods);
+
+extern MenuHandle build_research_menu(Side *side);
+
+/* macwins.c */
+
+extern void show_game_window(void);
+extern void draw_game(void);
+extern void draw_game_date(void);
+extern void draw_game_clocks(void);
+extern void draw_game_progress(void);
+extern void draw_side_status(Side *side2);
+extern void do_mouse_down_game(Point mouse, int mods);
+
+extern void show_construction_window(void);
+extern void set_construction_run_length(int len);
+extern void draw_construction(void);
+extern void activate_construction(int activate);
+extern void scroll_to_selected_construction_unit(void);
+extern void do_mouse_down_construction(Point mouse, int mods);
+extern int do_key_down_construction(int key);
+extern void select_unit_in_construction_window(Unit *unit);
+extern void select_type_in_construction_window(int u);
+extern void update_construction_unit_list(Unit *unit);
+extern void update_construction_type_list(void);
+extern void grow_construction(int h, int v);
+extern void zoom_construction(int part);
+
+extern void show_research_dialog(void);
+extern void draw_research_dialog(int force);
+extern void do_mouse_down_research(Point mouse, int mods);
+extern int do_key_down_research(char key);
+
+extern void side_research_dialog(Side *side, Unit *unit, int advance);
+
+extern void show_history_window(void);
+extern void draw_history(void);
+extern void update_history_window(HistEvent *hevt);
+extern void do_mouse_down_history(Point mouse, int mods);
+extern void grow_history(int h, int v);
+extern void zoom_history(int part);
+
+extern void show_notice_window(void);
+extern void append_notice(char *str);
+extern void draw_notice(void);
+extern void activate_notice(int activate);
+extern void do_mouse_down_notice(Point mouse, int mods);
+extern void grow_notice(int h, int v);
+extern void zoom_notice(int part);
+
+extern void show_scores_window(void);
+extern void draw_scores(void);
+extern void activate_scores(int activate);
+extern void do_mouse_down_scores(Point mouse, int mods);
+extern void grow_scores(int h, int v);
+extern void zoom_scores(int part);
