@@ -776,9 +776,8 @@ any later version.  See the file COPYING.  */
 #define dCity 803
 
 #define diCityMatPanel 4
-#define diCityFacPanel 5
-#define diCityOccPanel 6
-
+#define diCityTransportText 5
+#define diCityTransportName 6
 #define diCityPlanPopup 7
 #define diCityAdvancePopup 8
 #define diCityResearchStatus 9
@@ -799,10 +798,18 @@ any later version.  See the file COPYING.  */
 #define diCityMatBase 16
 #define diCityOccBase 41
 
+#define diCityMaterial 89
+#define diCityProduction 90
+#define diCitySupply 91
+#define diCityTreasury 92
+
 #define diCityIcon 93
 #define diCityClickText 94
 #define diCityBuildTimes 97
 #define diCityMap 98
+#define diCityFacPanel 99
+#define diCityOccPanel 100
+#define diCityTransportIcon 101
 
 #define dAdvance 804
 
@@ -838,15 +845,13 @@ any later version.  See the file COPYING.  */
 
 #define dBuild 807
 
-#define diBuildMadScientist 3
-#define diBuildBusyScientist 4
-#define diBuildIdleScientist 5
-#define diBuildMainText 6
-#define diBuildPopup 7
-#define diBuildStatus 8
-#define diBuildEdit 9
-#define diBuildText 10
-#define diBuildCheck 11
+#define diBuildText 3
+#define diBuildPopup 4
+#define diBuildAuto 5
+#define diBuildRepeat 6
+#define diBuildEdit 7
+#define diBuildTimes 8
+#define diBuildWorker 9
 
 #define dPlan 808
 
@@ -971,8 +976,6 @@ any later version.  See the file COPYING.  */
 #define cFeatureRemoveButton 402
 #define cFeatureEditButton 403
 
-#define cCommandDoButton 501
-
 /* Cursors. */
 
 #define cOpenCross 154
