@@ -14,6 +14,17 @@ any later version.  See the file COPYING. */
 
 #define numtooltypes 12
 
+static void init_design_cursors(void);
+static void create_design_window(void);
+static void draw_design_window_tool(enum tooltype tool);
+static void mark_allowed_unit_types(void);
+static void mark_allowed_sides(void);
+static void feature_rename_dialog(Feature *feature);
+static void paint_on_drag(Map *map, int h0, int v0, int mods);
+static void border_on_drag(Map *map, int h0, int v0, int mods, int paintmode);
+static void connect_on_drag(Map *map, int h0, int v0, int mods, int paintmode);
+static void designer_reshape_dialog(Module *module);
+
 extern MenuHandle featuremenu;
 
 extern short *feature_menu_fids;
@@ -78,7 +89,7 @@ short paintpeop;
 short paintctrl;
 short paintfid;
 short painttview;
-short paintuview;
+short paintuview = FALSE;  /* need a checkbox for this eventually */
 
 short enabledtooltype[12];
 
@@ -126,7 +137,7 @@ disable_designing()
 	  return;
 	/* Hide (but don't destroy) the designer's palette. */
 	if (designwin != nil) {
-		HideTheWindow(designwin);
+		close_window(designwin);
 	}
 	/* Actually change designer status, this will call back to alter all displays. */
 	net_become_nondesigner(dside);
@@ -319,10 +330,10 @@ draw_design_window_tool(enum tooltype tool)
 				auxtoolname = t_type_name(curbgttype);
 				/* bg type is always a cell subtype. */
 				OffsetRect(&imagerect, 3, 3);
-				draw_terrain_sample(designwin, imagerect, curbgttype);
+				draw_terrain_sample(designwin, imagerect, curbgttype, 4);
 				OffsetRect(&imagerect, -6, -6);
 			}
-			draw_terrain_sample(designwin, imagerect, curttype);
+			draw_terrain_sample(designwin, imagerect, curttype, 4);
 			break;
 		case unittool:
 			toolname = u_type_name(curutype);
@@ -407,7 +418,7 @@ draw_design_window_tool(enum tooltype tool)
 			break;
 		case cloudstool:
 			if (enabledtooltype[cloudstool]) {
-				draw_terrain_sample(designwin, imagerect, curbgttype);
+				draw_terrain_sample(designwin, imagerect, curbgttype, 4);
 				if (curcloudtype > 0) {
 					InsetRect(&imagerect, (dtoolh - hws[4]) / 2, (dtoolh - hhs[4]) / 2);
 					draw_clouds(imagerect.left, imagerect.top, 4, curcloudtype);
@@ -876,7 +887,7 @@ apply_designer_tool(Map *map, int h, int v, int mods)
 			paint_on_drag(map, h, v, mods);
 			break;
 		case elevationtool:
-			net_paint_elevation(dside, x, y, curbrushradius, curelevation);
+			net_paint_elevation(dside, x, y, curbrushradius, 0, curelevation, 0);
 			paint_on_drag(map, h, v, mods);
 			break;
 		case temperaturetool:
@@ -892,12 +903,10 @@ apply_designer_tool(Map *map, int h, int v, int mods)
 			paint_on_drag(map, h, v, mods);
 			break;
 		case viewtool:
-			if (dside->terrview == NULL || dside->unitview == NULL)
+			if (dside->terrview == NULL || dside->unit_views == NULL)
 			  break;
 			oldtview = terrain_view(dside, x, y);
 			painttview = (oldtview == UNSEEN ? buildtview(terrain_at(x, y)) : UNSEEN);
-			olduview = unit_view(dside, x, y);
-			paintuview = (curuview == olduview ? EMPTY : curuview);
 			net_paint_view(dside, x, y, curbrushradius, painttview, paintuview);
 			/* (should paint other view layers also) */
 			paint_on_drag(map, h, v, mods);
@@ -949,7 +958,7 @@ paint_on_drag(Map *map, int h0, int v0, int mods)
 						net_paint_feature(dside, x, y, curbrushradius, paintfid);
 						break;
 					case elevationtool:
-						net_paint_elevation(dside, x, y, curbrushradius, curelevation);
+						net_paint_elevation(dside, x, y, curbrushradius, 0, curelevation, 0);
 						break;
 					case temperaturetool:
 						net_paint_temperature(dside, x, y, curbrushradius, curtemperature);
@@ -966,9 +975,6 @@ paint_on_drag(Map *map, int h0, int v0, int mods)
 						if (painttview != UNSEEN)
 						  painttview = buildtview(terrain_at(x, y));
 						Dprintf("painttview is %d\n", painttview);
-						if (paintuview != EMPTY)
-						  paintuview = unit_view(dside, x, y);
-						Dprintf("paintuview is %d\n", paintuview);
 						net_paint_view(dside, x, y, curbrushradius, painttview, paintuview);
 						/* (should paint other view layers also) */
 						break;
