@@ -16,18 +16,24 @@ any later version.  See the file COPYING.  */
    get pretty wedged. */
 
 #include "conq.h"
+#include "macconq.h"
+
 #define wind_dir(w) ((w) & 0x07)
 #define wind_force(w) ((w) >> 3)
-#include "macconq.h"
+
+#define cell_style(map, x, y, power)  \
+  ((map->see_all || terrain_visible(dside, x, y) || unseen_image != NULL || !in_area(x, y))  \
+   ? (power >= 3 ? usepolygons : useblocks)  \
+   : dontdraw);
 
 extern short any_los;
 
-short pref_color_unit_images;
+short pref_color_unit_images = TRUE;
 
 /* When true, this causes terrain to be drawn with solid colors if possible;
    otherwise, color patterns will be preferred.  */
 
-short pref_solid_color_terrain;
+short pref_solid_color_terrain = FALSE;
 
 PolyHandle *polygons;
 int *lastpolyx, *lastpolyy;
@@ -61,6 +67,12 @@ static Image **best_unseen_images = NULL;
 
 static void draw_unit_mask(BitMap mm, Rect srcrect, Rect destrect, int wid1, int wid2);
 static void draw_unit_mask_layer(BitMap mm, Rect srcrect, Rect imagerect, int d, int mode);
+static void draw_unit_mask_and_shadow(BitMap mm, Rect srcrect, Rect destrect, int wid1, int wid2, int wid3);
+static void draw_unit_shadow(BitMap mm, Rect srcrect, Rect destrect, int wid);
+static void draw_unit_shadow_layer(BitMap mm, Rect srcrect, Rect imagerect, int d, int mode);
+static void calc_best_terrain_images(void);
+static int mac_compute_transition(Side *side, Map *map, int x, int y, int dir,
+		   int *sxp, int *syp, int *swp, int *shp, int *offsetp);
 
 /* 	Draw an image of the given type of unit in the given rectangle, adding an emblem if requested. */
 
@@ -152,7 +164,7 @@ draw_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, int u, int e, int
 			if (pref_color_unit_images || mod) {
 				/* Draw selection mask if asked to. */
 				if (selected && curmap->selectionmasks)
-				      draw_unit_mask(mm, srcrect, imagerect, 1, 1);
+				      draw_unit_mask(mm, srcrect, imagerect, 1, 0);
 				/* Draw white mask under native color pixmap. */
 				if (!sidecolors)
 				    BackColor(whiteColor);	
@@ -163,29 +175,32 @@ draw_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, int u, int e, int
 				if (unified_cicns)
 					CopyBits(&bm, &portbits, &srcrect, &imagerect, srcBic, 0);
 				else	CopyBits(&mm, &portbits, &srcrect, &imagerect, srcBic, 0);
-				/* Then plot the PixMap itself. */
-				CopyBits((BitMap*) &pm, &portbits, &srcrect, &imagerect, srcOr, 0);
+				/* Use CopyMask since cicns made from gifs have their transparency
+				info in the mask map. */
+				CopyMask((BitMap*) &pm, &mm, &portbits, &srcrect, &srcrect, &imagerect);
 				/* Optionally use a second side color to plot the right half of the icon. */
 				if (sidecolors && half_icon_color[e] && split_icon[e] &! mod) {
 					RGBForeColor(&(get_sideColor(e, half_icon_color[e])));
 					/* Restrict plotting to right half of image. */
 					srcrect.left = srcrect.right / 2;
 					imagerect.left += sw/2;
-					CopyBits((BitMap*) &pm, &portbits, &srcrect, &imagerect, srcOr, 0);
+					CopyMask((BitMap*) &pm, &mm, &portbits, &srcrect, &srcrect, &imagerect);
 				}
 
 			/* Plot the BitMap in all other cases. */
 			} else {
 				if (selected && curmap->selectionmasks)
-				      draw_unit_mask(mm, srcrect, imagerect, 1, 1);
-				CopyBits(&bm, &portbits, &srcrect, &imagerect, srcOr, 0);
+				      draw_unit_mask(mm, srcrect, imagerect, 1, 0);
+				/* Use CopyMask since cicns made from gifs carry their transparency
+				info in the mask map. */
+				CopyMask(&bm, &mm, &portbits, &srcrect, &srcrect, &imagerect);
 				/* Optionally use a second side color to plot the right half of the icon. */
 				if (sidecolors && half_icon_color[e] && split_icon[e]) {
 					RGBForeColor(&(get_sideColor(e, half_icon_color[e])));
 					/* Restrict plotting to right half of image. */
 					srcrect.left = srcrect.right / 2;
 					imagerect.left += sw/2;
-					CopyBits(&bm, &portbits, &srcrect, &imagerect, srcOr, 0);
+					CopyMask(&bm, &mm, &portbits, &srcrect, &srcrect, &imagerect);
 				}
 			}
 
@@ -209,7 +224,7 @@ draw_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, int u, int e, int
 			/* Now plot the real ICON. */ 
 			bm.baseAddr = *(macuimg->monoicon);
 				if (selected && curmap->selectionmasks)
-				draw_unit_mask(bm, srcrect, imagerect, 1, 1);
+				draw_unit_mask(bm, srcrect, imagerect, 1, 0);
 			/* Plot incomplete units in gray. */
 			if (mod) {
 				RGBForeColor(&graycolor);
@@ -244,7 +259,7 @@ draw_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, int u, int e, int
 			/* Now plot the real SICN. */
 			bm.baseAddr = *(macuimg->monosicn);
 			if (selected && curmap->selectionmasks)
-				draw_unit_mask(bm, srcrect, imagerect, 1, 1);
+				draw_unit_mask(bm, srcrect, imagerect, 1, 0);
 			/* Plot incomplete units in gray. */
 			if (mod) {
 				RGBForeColor(&graycolor);
@@ -266,14 +281,49 @@ draw_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, int u, int e, int
 	} else usebox = TRUE;
 
 	if (usebox) {
+		int done = FALSE;
+
 		SetRect(&imagerect, sx, sy, sx + sw, sy + sh);
-		FillRect(&imagerect, QDPat(black));
-		/* Optionally use a second color to plot the right half of the square. */
-		if (sidecolors && half_icon_color[e] && split_icon[e]) {
-			RGBForeColor(&(get_sideColor(e, half_icon_color[e])));
-			/* Restrict plotting to right half of image. */
-			imagerect.left += sw/2;
+		/* Use the default emblem color if it is available and we are not
+		using side colors. */
+		if (between(0, e, numsides) 
+		    && sidecolors != TRUE) {
+			if (eimages[e]) {
+				Image *eimg = best_image(eimages[e], 1, 1);
+				MacImage *maceimg;
+						
+				if (eimg && eimg->hook) {
+					maceimg = (MacImage *) eimg->hook;
+					if (maceimg->colrpat != nil) {
+						/* Make a black shadow. */
+						ForeColor(blackColor);
+						FillRect(&imagerect, QDPat(black));
+						imagerect.bottom -= 1;
+						imagerect.right -= 1;
+		 			    	FillCRect(&imagerect, maceimg->colrpat);
+						done = TRUE;
+					}
+				}
+			}
+		}
+		/* Always use sidecolors as a fallback even if sidecolors are off
+		for this map. */
+		if (!done) {
+			/* Make a black shadow. */
+			ForeColor(blackColor);
 			FillRect(&imagerect, QDPat(black));
+			imagerect.bottom -= 1;
+			imagerect.right -= 1;
+			if (main_icon_color[e])
+			    RGBForeColor(&(get_sideColor(e, main_icon_color[e])));
+			FillRect(&imagerect, QDPat(black));
+			/* Optionally use a second color to plot the right half of the square. */
+			if (half_icon_color[e] && split_icon[e]) {
+				RGBForeColor(&(get_sideColor(e, half_icon_color[e])));
+				/* Restrict plotting to right half of image. */
+				imagerect.left += sw/2;
+				FillRect(&imagerect, QDPat(black));
+			}
 		}
 		/* Gray out units under construction. */
 		if (mod) {
@@ -298,6 +348,9 @@ draw_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, int u, int e, int
 	if (between(0, e, numsides) && emblem && sw > 4 ) {
 		ename = (side_n(e) ? side_n(e)->emblemname : NULL);
 		if (emblem_position(uimg, ename, eimages[e], sw, sh, &ex, &ey, &ew, &eh)) {
+			/* Make sure emblem stays within the cell. */
+			if (curmap && uimg->h > curmap->vp->uh)
+				sy += (curmap->vp->hh - curmap->vp->uh) / 2;
 			/* Do the drawing proper. */
 			draw_side_emblem(win, sx + ex, sy + ey, ew, eh, e, plain_emblem);
 		}
@@ -307,8 +360,14 @@ draw_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, int u, int e, int
 void
 draw_unit_mask(BitMap mm, Rect srcrect, Rect imagerect, int wid1, int wid2)
 {
+	RGBColor oldfore;
+	RGBColor oldback;
 	int i;
 	
+	/* Save current colors. */
+	GetForeColor(&oldfore);
+	GetBackColor(&oldback);
+
 	/* First draw the outer layer using the fore color. */ 
 	for (i = wid1 + 1; i <= wid1 + wid2; i++) {
 		draw_unit_mask_layer(mm, srcrect, imagerect, i, srcOr);
@@ -317,6 +376,9 @@ draw_unit_mask(BitMap mm, Rect srcrect, Rect imagerect, int wid1, int wid2)
 	for (i = 1; i <= wid1; i++) {
 		draw_unit_mask_layer(mm, srcrect, imagerect, i, srcBic);
 	}
+	/* Restore current colors. */
+	RGBForeColor(&oldfore);
+	RGBBackColor(&oldback);
 }
 
 void
@@ -343,6 +405,68 @@ draw_unit_mask_layer(BitMap mm, Rect srcrect, Rect imagerect, int d, int mode)
 	OffsetRect(&imagerect, -d, 0);
 }
 
+void
+draw_unit_mask_and_shadow(BitMap mm, Rect srcrect, Rect imagerect, int wid1, int wid2, int wid3)
+{
+	RGBColor oldfore;
+	RGBColor oldback;
+	int i;
+	
+	GetForeColor(&oldfore);
+	GetBackColor(&oldback);
+
+	/* First draw the shadow layer using dkgraycolor. */ 
+	RGBForeColor(&dkgraycolor);
+	for (i = wid1 + wid2 + 1; i <= wid1 + wid2 + wid3; i++) {
+		draw_unit_shadow_layer(mm, srcrect, imagerect, i, srcOr);
+	}
+	/* First draw the outer layer using the fore color. */ 
+	ForeColor(blackColor);
+	for (i = wid1 + 1; i <= wid1 + wid2; i++) {
+		draw_unit_mask_layer(mm, srcrect, imagerect, i, srcOr);
+	}
+	/* Then draw the inner layer using the mask color. */
+	BackColor(whiteColor);
+	for (i = 1; i <= wid1; i++) {
+		draw_unit_mask_layer(mm, srcrect, imagerect, i, srcBic);
+	}
+	RGBForeColor(&oldfore);
+	RGBBackColor(&oldback);
+}
+
+void
+draw_unit_shadow(BitMap mm, Rect srcrect, Rect imagerect, int wid)
+{
+	BitMap portbits = QD(thePort)->portBits;
+	RGBColor oldfore;
+	int i;
+	
+	GetForeColor(&oldfore);
+
+	/* Draw the shadow layer using dkgraycolor. */ 
+	RGBForeColor(&dkgraycolor);
+	for (i = 1; i <= wid; i++) {
+		OffsetRect(&imagerect,  i,   i);
+		CopyBits(&mm, &portbits, &srcrect, &imagerect, srcOr, 0);		/* SE */
+		OffsetRect(&imagerect,  -i,   -i);
+	}
+	RGBForeColor(&oldfore);
+}
+
+void
+draw_unit_shadow_layer(BitMap mm, Rect srcrect, Rect imagerect, int d, int mode)
+{
+	BitMap portbits = QD(thePort)->portBits;
+
+	OffsetRect(&imagerect,  0,   d);
+	CopyBits(&mm, &portbits, &srcrect, &imagerect, mode, 0);		/* E */
+	OffsetRect(&imagerect,  d,   0);
+	CopyBits(&mm, &portbits, &srcrect, &imagerect, mode, 0);		/* SE */
+	OffsetRect(&imagerect,  0, -d);
+	CopyBits(&mm, &portbits, &srcrect, &imagerect, mode, 0);		/* S	*/
+	OffsetRect(&imagerect, -d,  0);
+}
+
 /* Draw a given side id's emblem. Uses the current GrafPort.  No iconmasks or sidecolors,
    since emblems are very rarely colorizable silhouettes. */
 
@@ -365,6 +489,8 @@ draw_side_emblem(WindowPtr win, int ex, int ey, int ew, int eh, int e, int style
 	/* Get best emblem image from family. */
 	eimg = best_image(eimages[e], ew, eh);
 
+#if 0		/* This code forces default color emblems to be 1 x 1 pixels in size. */
+
 	if (eimg->w < ew || eimg->h < eh) {
 		ex2 = ex + (ew - eimg->w) / 2;  ey2 = ey + (eh - eimg->h) / 2;
 		/* Only change the size of the rectangle being drawn if it's
@@ -378,6 +504,9 @@ draw_side_emblem(WindowPtr win, int ex, int ey, int ew, int eh, int e, int style
 		    eh = eimg->h;
 		}
 	}
+	
+#endif
+
 	SetRect(&imagerect, ex, ey, ex + ew, ey + eh);
 
 	/* If an image is available, display it, otherwise do nothing. */
@@ -408,9 +537,7 @@ draw_side_emblem(WindowPtr win, int ex, int ey, int ew, int eh, int e, int style
 					bm.baseAddr = (char *) (*cicnhandle)->iconMaskData;
 				}
 				if (hasColorQD) {
-					RGBColor tmpcolor;
-					tmpcolor.red = tmpcolor.green = tmpcolor.blue = 49000;
-					RGBForeColor(&tmpcolor);
+					RGBForeColor(&ltgraycolor);
 					if (bm.rowBytes > 0) {
 						 CopyBits(&bm, &portbits, &srcrect, &shadowrect, srcCopy, nil);
 					} else {
@@ -439,12 +566,24 @@ draw_side_emblem(WindowPtr win, int ex, int ey, int ew, int eh, int e, int style
 			CopyBits(&mm, &portbits, &srcrect, &imagerect, srcBic, nil);
 
 			if (hasColorQD) {
-				/* Always use the PixMap if has Color QD */
-				CopyBits((BitMap*) &pm, &portbits, &srcrect, &imagerect, srcOr, nil);
+				/* Use CopyMask since cicns made from gifs have their transparency
+				info in the mask map. */
+				CopyMask((BitMap*) &pm, &mm, &portbits, &srcrect, &srcrect, &imagerect);
 			} else {
-				/* Use the BitMap only for B/W plotting */
-				CopyBits(&bm, &portbits, &srcrect, &imagerect, srcOr, nil);
+				CopyMask(&bm, &mm, &portbits, &srcrect, &srcrect, &imagerect);
 			}
+		/* This handles default emblems generated from side colorschemes. */
+		} else if (maceimg->colrpat != nil) {
+			imagerect.bottom = min(imagerect.top + 12, imagerect.bottom);
+			if (style == shadow_emblem) {
+				shadowrect = imagerect;
+				OffsetRect(&shadowrect, (ew < 16 ? 1 : 2), (ew < 16 ? 1 : 2));
+				RGBForeColor(&ltgraycolor);
+				PaintRect(&shadowrect);
+				RGBForeColor(&blackcolor);
+			}
+			FillCRect(&imagerect, maceimg->colrpat);
+			FrameRect(&imagerect);
 		} else if (maceimg->monoicon != nil) {
 			/* Use ICON if no pict or cicn exists (or when cicn does but drawing
 			   is in B/W) */
@@ -698,11 +837,17 @@ draw_hex_region(int sx, int sy, int power, int t, int over, int angle,
 			} else if (mactimg->colricon != nil && (minscreendepth > 1 || !mactimg->patdefined)) {
 				Rect srcrect, imagerect;
 				CIconHandle cicnhandle;
-				BitMap portbits;	/* Target BitMap for CopyBits */
+				BitMap portbits;			/* Target BitMap for CopyBits */
 				BitMap mm;			/* icon BitMap & MaskMap */
 				PixMap pm;			/* icon PixMap */
 
-				SetRect(&srcrect, 0, 0, hws[power], hhs[power]);
+				if (angle == 90) {
+					SetRect(&srcrect, 0, 0, hws[power], hhs[power]);
+				} else {
+					/* Offset to middle of hex if drawing at an angle so that this flatter cell image does 
+					include the pointed Northern tip. */  	
+					SetRect(&srcrect, 0, hhs[power]/4, hws[power], hhs[power] + hhs[power]/4);
+				}				
 				SetRect(&imagerect, sx, sy, sx + hws[power], sy + hhs[power]);
 				portbits = QD(thePort)->portBits;
 				cicnhandle = (CIconHandle) mactimg->colricon;
@@ -710,7 +855,12 @@ draw_hex_region(int sx, int sy, int power, int t, int over, int angle,
 				pm = (*(cicnhandle))->iconPMap;
 				mm = (*(cicnhandle))->iconMask;
 				/* Draw it, using the mask as the copy mask. */
-				CopyMask((BitMap *) &pm, &mm, &portbits, &srcrect, &srcrect, &imagerect);
+				if (angle == 90) {
+					CopyMask((BitMap *) &pm, &mm, &portbits, &srcrect, &srcrect, &imagerect);
+				/* But use the cellrgn as destination mask if drawing at an angle. */
+				} else {
+					CopyBits((BitMap *) &pm, &portbits, &srcrect, &imagerect, srcCopy, rgn);
+				}
 			} else if (mactimg->colrpat != nil && (minscreendepth > 1 || !mactimg->patdefined)) {
 				FillCRgn(rgn, mactimg->colrpat);
 			} else if (minscreendepth > 1) {
@@ -851,6 +1001,351 @@ make_cell_clip(int power)
 	SectRgn(gridcellrgns15[power], tmprgn, gridcellrgns15[power]);
 }
 
+/* Differs from the kernel version in that it takes the map as arg instead of 
+its vp and calls xform instead of xform_cell. Also disables transitions at the
+water edge if shorelines are enabled. */
+
+int
+mac_compute_transition(Side *side, Map *map, int x, int y, int dir,
+		   int *sxp, int *syp, int *swp, int *shp, int *offsetp)
+{
+    int t, x1, y1, t1, x2, y2, sx, sy, sw, sh, sx2, sy2;
+    int trite, tleft, overrite, overleft, offset;
+    VP *vp = map->vp;
+    int w = vp->hw, h = vp->hh, hch = vp->hch;
+
+    if (!point_in_dir(x, y, dir, &x1, &y1))
+      return FALSE;
+    if (!(vp->show_all || terrain_visible(side, x1, y1)))
+      return FALSE;
+    t = terrain_at(x, y);
+    t1 = terrain_at(x1, y1);
+    /* We want to overlap the adjacent terrain into this cell if the
+       terrain type is lower-numbered (meaning that sea usually
+       overlaps land for instance), but not if there is a border,
+       because the border itself provides the transition. */
+    if (t1 >= t || any_borders_in_dir(x, y, dir))
+      return FALSE;
+    /* We don't want to do overlap if there is a beach and 
+    shorelines are drawn. */
+    if ((t_liquid(t) != t_liquid(t1))
+        && map->shorelines)
+      return FALSE;
+    if (point_in_dir(x, y, right_dir(dir), &x2, &y2))
+      trite = terrain_at(x2, y2);
+    else
+      trite = t;
+    /* Overlap on the right-hand corner if the third cell's terrain is
+       the same as the adjacent cell's terrain. */
+    overrite = (trite == t1);
+    /* Border on one side suppresses overlap. */
+    if (any_borders_in_dir(x, y, right_dir(dir)))
+      overrite = FALSE;
+    if (point_in_dir(x, y, left_dir(dir), &x2, &y2))
+      tleft = terrain_at(x2, y2);
+    else
+      tleft = t;
+    /* Overlap on the left-hand corner if the third cell's terrain is
+       the same as the adjacent cell's terrain. */
+    overleft = (tleft == t1);
+    /* Border on one side suppresses overlap. */
+    if (any_borders_in_dir(x, y, left_dir(dir)))
+      overleft = FALSE;
+    xform(map, x, y, &sx, &sy);
+    /* Now, given the overlaps on each side, choose the right piece of
+       transition bitmap to use. */
+    switch (dir) {
+      case NORTHEAST:
+	sx2 = sx + w / 2;  sy2 = sy;
+	sw = w / 2;  sh = h - hch + 1;
+	if (overrite)
+	  offset = 1;
+	else if (overleft)
+	  offset = 2;
+	break;
+      case EAST:
+	sx2 = sx + w / 2;  sy2 = sy + (h - hch) + 1;
+	sw = w / 2;  sh = h - 2 * (h - hch) - 2;
+	if (overrite)
+	  offset = 2;
+	else if (overleft)
+	  offset = 1;
+	break;
+      case SOUTHEAST:
+	sx2 = sx + w / 2;  sy2 = sy + hch - 1;
+	sw = w / 2;  sh = h - hch + 1;
+	if (overrite)
+	  offset = 1;
+	else if (overleft)
+	  offset = 2;
+	break;
+      case SOUTHWEST:
+	sx2 = sx;  sy2 = sy + hch - 1;
+	sw = w / 2;  sh = h - hch + 1;
+	if (overrite)
+	  offset = 2;
+	else if (overleft)
+	  offset = 1;
+	break;
+      case WEST:
+	sx2 = sx;  sy2 = sy + (h - hch) + 1;
+	sw = w / 2;  sh = h - 2 * (h - hch) - 2;
+	if (overrite)
+	  offset = 1;
+	else if (overleft)
+	  offset = 2;
+	break;
+      case NORTHWEST:
+	sx2 = sx;  sy2 = sy;
+	sw = w / 2;  sh = h - hch + 1;
+	if (overrite)
+	  offset = 2;
+	else if (overleft)
+	  offset = 1;
+	break;
+    }
+    if (overrite && overleft)
+      offset = 0;
+    else if (!overrite && !overleft)
+      offset = 3;
+    *sxp = sx2;  *syp = sy2;
+    *swp = sw;  *shp = sh;
+    *offsetp = offset;
+    return TRUE;
+}
+
+void
+draw_terrain_transitions(Map *map, int x0, int y0, int len)
+{
+	int 			sx2, sy2, sw, sh, offset, t1, rslt, quasirand, quasirand2;
+	int 			x1, y1, x, xw, t, dir, sx, sy, segstyle;
+	Image 		*timg, *subimg, *trimg, *subtrimg;
+	Rect 			srcrect, maskrect, imagerect;
+	MacImage 		*mactrimg, *macimg;
+	BitMap 		portbits, mm;			
+	CIconHandle 	cicnhandle;
+	PixMap 		pm;			
+
+	trimg = best_image(generic_transition, map->vp->hw, map->vp->hh);
+	if (trimg == NULL)
+	    return;
+	for (x = x0; x < x0 + len + 1; ++x) {
+		xw = wrapx(x);
+		t = terrain_at(xw, y0);
+		segstyle = cell_style(map, xw, y0, map->vp->power);
+		if (x == x0 + len
+		|| x == area.width
+		|| segstyle == dontdraw)
+		    continue;
+		for_all_directions(dir) {
+			rslt = mac_compute_transition(dside, map, xw, y0, dir, 
+								      &sx2, &sy2, &sw, &sh, &offset);
+			if (!rslt)
+			    continue;
+			xform(map, xw, y0, &sx, &sy);
+			/* A negative quasirand can cause a null pointer crash below. */
+			quasirand = abs((11 * xw + y0 + 7) * 41);
+			subtrimg = trimg->subimages[(quasirand % 4) * 4 + offset];
+			mactrimg = (MacImage *) subtrimg->hook;
+			if (mactrimg && mactrimg->colricon != nil) {
+				SetRect(&maskrect, sx2 - sx, sy2 - sy, sx2 - sx + sw, sy2 - sy + sh);
+				cicnhandle = (CIconHandle) mactrimg->colricon;
+				mm = (*(cicnhandle))->iconMask;
+			}
+			point_in_dir(xw, y0, dir, &x1, &y1);
+			t1 = terrain_at(x1, y1);
+			timg = best_image(timages[t1], map->vp->hw, map->vp->hh);
+			subimg = timg;
+			quasirand2 = 0;
+			if (timg->numsubimages > 0 && timg->subimages)
+			    subimg = timg->subimages[quasirand2 % timg->numsubimages];
+			macimg = (MacImage *) subimg->hook;
+			if (macimg && macimg->colricon != nil) {
+				SetRect(&srcrect,  sx2 - sx, sy2 - sy, sx2 - sx + sw, sy2 - sy + sh);
+				SetRect(&imagerect, sx2, sy2, sx2 + sw, sy2 + sh);
+				cicnhandle = (CIconHandle) macimg->colricon;
+				pm = (*(cicnhandle))->iconPMap;
+				portbits = QD(thePort)->portBits;
+				CopyMask((BitMap *) &pm, &mm, &portbits, 
+								&srcrect, &maskrect, &imagerect);
+			}
+		}
+	}
+}
+
+
+void
+draw_shore_images(Map *map, int x, int y)
+{
+	int			xw, bitmask, sx, sy, x2, y2, x3, y3, subi, halfside, power;
+	Rect 			srcrect, imagerect;
+	Image		*timg, *subimg;
+	MacImage		*mactimg;
+	CIconHandle	cicnhandle;
+	BitMap		portbits, mm;
+	PixMap		pm;
+
+	power = map->vp->power;
+	timg = best_image(beach, hws[power], hhs[power]);
+         halfside = halfsides[power];
+	xw = wrapx(x);
+	/* Draw the junction at the top of the x,y hex. */
+	bitmask = 0;
+	if (interior_point_in_dir(xw, y, NORTHEAST, &x2, &y2)
+	    && interior_point_in_dir(xw, y, NORTHWEST, &x3, &y3)
+	    && (t_liquid(terrain_at(x2,y2)) != t_liquid(terrain_at(x3,y3)))
+	    && seen_border(dside, x2, y2, WEST))
+	  bitmask |= 1;
+	if (interior_point_in_dir(xw, y, NORTHEAST, &x2, &y2)
+	    && (t_liquid(terrain_at(x2,y2)) != t_liquid(terrain_at(xw,y)))
+	    && seen_border(dside, xw, y, NORTHEAST))
+	  bitmask |= 2;
+	if (interior_point_in_dir(xw, y, NORTHWEST, &x2, &y2)
+	    && (t_liquid(terrain_at(x2,y2)) != t_liquid(terrain_at(xw,y)))
+	    && seen_border(dside, xw, y, NORTHWEST))
+	  bitmask |= 4;
+	if (bitmask != 0) {
+		xform(map, xw, y, &sx, &sy);
+		subi = (bitmask & 3) + ((bitmask & 4) ? 12 : 4);
+		if (timg->numsubimages > 0 && timg->subimages)
+		    subimg = timg->subimages[subi];
+		if (subimg && subimg->hook)
+		  mactimg = (MacImage *) subimg->hook;
+		if (mactimg->colricon != nil) {
+			SetRect(&srcrect, 0, 0, hws[power], hcs[power]);
+			SetRect(&imagerect, sx, 
+						       sy - halfside, 
+						       sx + hws[power], 
+						       sy - halfside + hcs[power]);
+			portbits = QD(thePort)->portBits;
+			cicnhandle = (CIconHandle) mactimg->colricon;
+			pm = (*(cicnhandle))->iconPMap;
+			mm = (*(cicnhandle))->iconMask;
+			CopyMask((BitMap *) &pm, &mm, &portbits, 
+							 &srcrect, &srcrect, &imagerect);
+		}
+	}
+	/* Draw the junction at the top left corner of the x,y hex. */
+	bitmask = 0;
+	if (interior_point_in_dir(xw, y, WEST, &x2, &y2)
+	    && (t_liquid(terrain_at(x2,y2)) != t_liquid(terrain_at(xw,y)))
+	    && seen_border(dside, xw, y, WEST))
+	  bitmask |= 1;
+	if (interior_point_in_dir(xw, y, NORTHWEST, &x2, &y2)
+	    && (t_liquid(terrain_at(x2,y2)) != t_liquid(terrain_at(xw,y)))
+	    && seen_border(dside, xw, y, NORTHWEST))
+	  bitmask |= 2;
+	if (interior_point_in_dir(xw, y, WEST, &x2, &y2)
+	    && interior_point_in_dir(xw, y, NORTHWEST, &x3, &y3)
+	    && (t_liquid(terrain_at(x2,y2)) != t_liquid(terrain_at(x3,y3)))
+	    && seen_border(dside, x2, y2, NORTHEAST))
+	  bitmask |= 4;
+	if (bitmask != 0) {
+		xform(map, xw, y, &sx, &sy);
+		subi = (bitmask & 3) + ((bitmask & 4) ? 8 : 0);
+		subimg = timg->subimages[subi];
+		if (subimg && subimg->hook)
+		    mactimg = (MacImage *) subimg->hook;
+		if (mactimg->colricon != nil) {
+			SetRect(&srcrect, 0, 0, hws[power], hcs[power]);
+			SetRect(&imagerect, sx -hws[power]/2 , 
+						       sy - halfside, 
+						       sx + hws[power]/2, 
+						       sy - halfside + hcs[power]);
+			portbits = QD(thePort)->portBits;
+			cicnhandle = (CIconHandle) mactimg->colricon;
+			pm = (*(cicnhandle))->iconPMap;
+			mm = (*(cicnhandle))->iconMask;
+			CopyMask((BitMap *) &pm, &mm, &portbits, 
+							&srcrect, &srcrect, &imagerect);
+		}
+	}
+}
+
+/* Ported from the tcltk interface. */
+
+void
+draw_border_images(Map *map, int x, int y, int b)
+{
+	int			xw, bitmask, sx, sy, x3, y3, subi, halfside, power;
+	Rect 			srcrect, imagerect;
+	Image		*timg, *subimg;
+	MacImage		*mactimg;
+	CIconHandle 	cicnhandle;
+	BitMap 		portbits, mm;
+	PixMap 		pm;
+
+	power = map->vp->power;
+	timg = best_image(timages[b], hws[power], hhs[power]);
+         halfside = halfsides[power];
+	xw = wrapx(x);
+	/* Draw the junction at the top of the x,y hex. */
+	bitmask = 0;
+	if (point_in_dir(xw, y, NORTHEAST, &x3, &y3)
+	    && border_at(x3, y3, WEST, b)
+	    && seen_border(dside, x3, y3, WEST))
+	  bitmask |= 1;
+	if (border_at(xw, y, NORTHEAST, b)
+	    && seen_border(dside, xw, y, NORTHEAST))
+	  bitmask |= 2;
+	if (border_at(xw, y, NORTHWEST, b)
+	    && seen_border(dside, xw, y, NORTHWEST))
+	  bitmask |= 4;
+	if (bitmask != 0) {
+		xform(map, xw, y, &sx, &sy);
+		subi = (bitmask & 3) + ((bitmask & 4) ? 12 : 4);
+		if (timg->numsubimages > 0 && timg->subimages)
+		    subimg = timg->subimages[subi];
+		if (subimg && subimg->hook)
+		  mactimg = (MacImage *) subimg->hook;
+		if (mactimg->colricon != nil) {
+			SetRect(&srcrect, 0, 0, hws[power], hcs[power]);
+			SetRect(&imagerect, sx, 
+						       sy - halfside, 
+						       sx + hws[power], 
+						       sy - halfside + hcs[power]);
+			portbits = QD(thePort)->portBits;
+			cicnhandle = (CIconHandle) mactimg->colricon;
+			pm = (*(cicnhandle))->iconPMap;
+			mm = (*(cicnhandle))->iconMask;
+			CopyMask((BitMap *) &pm, &mm, &portbits, 
+							 &srcrect, &srcrect, &imagerect);
+		}
+	}
+	/* Draw the junction at the top left corner of the x,y hex. */
+	bitmask = 0;
+	if (border_at(xw, y, WEST, b)
+	    && seen_border(dside, xw, y, WEST))
+	  bitmask |= 1;
+	if (border_at(xw, y, NORTHWEST, b)
+	    && seen_border(dside, xw, y, NORTHWEST))
+	  bitmask |= 2;
+	if (point_in_dir(xw, y, WEST, &x3, &y3)
+	    && border_at(x3, y3, NORTHEAST, b)
+	    && seen_border(dside, x3, y3, NORTHEAST))
+	  bitmask |= 4;
+	if (bitmask != 0) {
+		xform(map, xw, y, &sx, &sy);
+		subi = (bitmask & 3) + ((bitmask & 4) ? 8 : 0);
+		subimg = timg->subimages[subi];
+		if (subimg && subimg->hook)
+		    mactimg = (MacImage *) subimg->hook;
+		if (mactimg->colricon != nil) {
+			SetRect(&srcrect, 0, 0, hws[power], hcs[power]);
+			SetRect(&imagerect, sx -hws[power]/2 , 
+						       sy - halfside, 
+						       sx + hws[power]/2, 
+						       sy - halfside + hcs[power]);
+			portbits = QD(thePort)->portBits;
+			cicnhandle = (CIconHandle) mactimg->colricon;
+			pm = (*(cicnhandle))->iconPMap;
+			mm = (*(cicnhandle))->iconMask;
+			CopyMask((BitMap *) &pm, &mm, &portbits, 
+							 &srcrect, &srcrect, &imagerect);
+		}
+	}
+}
+
 /* Draw a set of borders for the given position. */
 
 void
@@ -1022,7 +1517,7 @@ draw_connection_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int po
 			&& (minscreendepth > 1 || !mactimg->patdefined)) {
 			Rect srcrect, imagerect;
 			CIconHandle cicnhandle;
-			BitMap portbits;	/* Target BitMap for CopyBits */
+			BitMap portbits;			/* Target BitMap for CopyBits */
 			BitMap mm;			/* icon BitMap & MaskMap */
 			PixMap pm;			/* icon PixMap */
 
@@ -1129,7 +1624,7 @@ draw_connection_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int po
 /* This draws a type of terrain in a way that indicates its subtype. */
 
 void
-draw_terrain_sample(WindowPtr win, Rect tmprect, int t)
+draw_terrain_sample(WindowPtr win, Rect tmprect, int t, int power)
 {
 	int dosolid = FALSE;
 
@@ -1139,16 +1634,16 @@ draw_terrain_sample(WindowPtr win, Rect tmprect, int t)
 
 	switch (t_subtype(t)) {
 		case cellsubtype:
-			draw_hex_region(tmprect.left, tmprect.top, 4, t, 0, 90, FALSE, dosolid, 0);
+			draw_hex_region(tmprect.left, tmprect.top, power, t, 0, 90, FALSE, dosolid, 0);
 			break;
 		case bordersubtype:
-			draw_border_line_multiple(win, tmprect.left, tmprect.top, 63, 4, t, 90, 0, dosolid);
+			draw_border_line_multiple(win, tmprect.left, tmprect.top, 63, power, t, 90, 0, dosolid);
 			break;
 		case connectionsubtype:
-			draw_connection_line_multiple(win, tmprect.left, tmprect.top, 63, 4, t, 90, 0, dosolid);
+			draw_connection_line_multiple(win, tmprect.left, tmprect.top, 63, power, t, 90, 0, dosolid);
 			break;
 		case coatingsubtype:
-			draw_hex_region(tmprect.left, tmprect.top, 4, t, 0, 90, FALSE, dosolid, 0);
+			draw_hex_region(tmprect.left, tmprect.top, power, t, 0, 90, FALSE, dosolid, 0);
 			/* Make it a 50% pattern. (should make more obvious somehow?) */
 			gray_out_rect(&tmprect);
 			break;
@@ -1263,6 +1758,7 @@ draw_ai_region_borders(WindowPtr win, int sx, int sy, int bitmask, int power)
 	}
 }
 
+#if 0
 int
 draw_elevation_here(int x, int y)
 {
@@ -1282,6 +1778,7 @@ draw_elevation(int sx, int sy, int power, int elev)
 		ForeColor(blackColor);
 	}
 }
+#endif
 
 /* Indicate the temperature of the given location, textually for now. */
 
@@ -1404,7 +1901,7 @@ draw_unit_name(Unit *unit, int sx, int sy, int sw, int sh, int masks, int optimi
 {
 	if (unit->name && sh > 5) {
 		RGBForeColor(&textcolor);
-		draw_legend_text(sx + sw + 1, sy + sh/2, sh, unit->name, -1, masks, optimize);
+		draw_legend_text(sx + sw + 2, sy + sh/2, sh, unit->name, -1, masks, optimize);
 		ForeColor(blackColor);
 	}
 }
@@ -1425,37 +1922,25 @@ draw_legend_text(int sx, int sy, int sh, char *legend, int just, int masks, int
 
 	/* Use fixed optimized fonts if asked to do so. */
 	if (optimize) {
-		switch (sh) {
-			case 8:
+		if (sh < 16) {
 				TextSize(9);
 				TextFace(normal);
-				break;
-			case 16:
+		} else if (sh < 32) {
 				TextSize(10);
 				TextFace(normal);
-				break;
-			case 32:
+		} else if (sh < 64) {
 				TextSize(14);
 				TextFace(bold);
-				break;
-			case 64:
+		} else if (sh < 128) {
 				TextSize(18);
 				TextFace(bold);
-				break;
-			case 128:
+		} else if (sh < 256) {
 				TextSize(24);
 				TextFace(bold);
-				break;
-			case 256:
+		} else {
 				TextSize(36);
 				TextFace(bold);
-				break;
-			default:
-				TextSize(9);
-				TextFace(normal);
-				break;
 		}
-
 	} else {
 		/* Otherwise scale text sizes according to this formula. */
 		TextSize(min(max(9, sh/2), 24));
@@ -1518,7 +2003,7 @@ draw_blast_image(WindowPtr win, int sx, int sy, int sw, int sh, int blasttype)
 		}
 	}
 	SetRect(&tmprect, sx, sy, sx + sw, sy + sh);
-	if (sw >= 16) {
+	if (sw >= 8) {
 		/* Set CopyBits destination to current port, which may differ from the current window. */				
 		winbits = &QD(thePort)->portBits;
 		imagerect.top = sy;  imagerect.left = sx;
@@ -1551,6 +2036,7 @@ draw_blast_image(WindowPtr win, int sx, int sy, int sw, int sh, int blasttype)
 	}
 }
 
+#if 0
 void
 clear_blast_image(WindowPtr win, int sx, int sy, int sw, int sh, int blasttype)
 {
@@ -1563,6 +2049,7 @@ clear_blast_image(WindowPtr win, int sx, int sy, int sw, int sh, int blasttype)
 		InvertRect(&tmprect);
     }
 }
+#endif
 
 int
 picture_width(PicHandle pichandle)
