@@ -77,6 +77,7 @@ static Rect debugRect = {0};
 static int scrolling = FALSE;
 
 static void reduce_screen_depth(GDHandle gdev);
+static void copy_clipped_to_map(Map *map, Rect sourcerect, Rect destrect);
 static void auto_scroll_left(Map *map);
 static void auto_scroll_right(Map *map);
 static void auto_scroll_up(Map *map);
@@ -85,16 +86,12 @@ static void auto_scroll_down_left(Map *map);
 static void auto_scroll_down_right(Map *map);
 static void auto_scroll_up_left(Map *map);
 static void auto_scroll_up_right(Map *map);
-
-extern void scroll_list_using_key(List *list, char key);
-
 static void auto_scroll_left_arrow(Map *map);
 static void auto_scroll_right_arrow(Map *map);
 static void auto_scroll_up_arrow(Map *map);
 static void auto_scroll_down_arrow(Map *map);
 static void rotate_angle_up(Map *map);
 static void rotate_angle_down(Map *map);
-static void copy_clipped_to_map(Map *map, Rect sourcerect, Rect destrect);
 
 /* Check if any device has more than 256 colors */
 
@@ -1102,138 +1099,6 @@ rotate_angle_down(Map *map)
 	  set_map_angle(map, 90);
 }
 
-/* Dialog that sets the size of the offscreen scrolling buffers. */
-
-void
-set_offscreen_buffers(Map *map)
-{
-	short ditem, itemtype, done = FALSE;
-	long *itemnum = 0;
-	Handle itemhandle;  
-	Rect itemrect;
-	Str255 tmpstr;
-	DialogPtr win;
-	GrafPtr oldport;
-
-	win = GetNewDialog(dOffscreen, NULL, (DialogPtr) -1);
-	GetPort(&oldport);	
-	SetPort(win);
-
-	GetDItem(win, diOffscreenHorizBuff, &itemtype, &itemhandle, &itemrect);
-	NumToString(map->max_bufx, tmpstr);
-	SetIText(itemhandle, tmpstr);
-
-	GetDItem(win, diOffscreenVertiBuff, &itemtype, &itemhandle, &itemrect);
-	NumToString(map->max_bufy, tmpstr);
-	SetIText(itemhandle, tmpstr);
-		
-	ShowWindow(win);
-	while (!done) {
-
-		SetCursor(&QD(arrow));
-		draw_default_button(win, diOffscreenOK);
-		ModalDialog(NULL, &ditem);
-		switch (ditem) {
-
-			case diOffscreenOK:
-
-				GetDItem(win, diOffscreenHorizBuff, &itemtype, &itemhandle, &itemrect);
-	        			GetIText(itemhandle,tmpstr);
-				StringToNum(tmpstr, itemnum);
-				/* Dont allow negative values */
-				map->max_bufx = max(*itemnum, 0);
-
-				GetDItem(win, diOffscreenVertiBuff, &itemtype, &itemhandle, &itemrect);
-	         			GetIText(itemhandle,tmpstr);
-				StringToNum(tmpstr, itemnum);
-				/* Dont allow negative values */
-				map->max_bufy = max(*itemnum, 0);
-				
-				/* Redraw the gworld (and the map) */
-				force_map_update(map);
-				done = TRUE;
-				break;
-
-			case diOffscreenCancel:
-
-				done = TRUE;
-				break;
-
-			default:
-				break;
-		}
-	}
-	DisposeDialog(win);
-	SetPort(oldport);
-	update_all_map_windows();
-}
-
-/* Dialog that sets the size of the meridian interval for a map. */
-/* Clone of set_offscreen_buffers, should be moved elsewhere. */
-
-void
-set_meridians(Map *map)
-{
-	short ditem, itemtype, done = FALSE;
-	long *itemnum = 0;
-	Handle itemhandle;  
-	Rect itemrect;
-	Str255 tmpstr;
-	DialogPtr win;
-	GrafPtr oldport;
-
-	win = GetNewDialog(dMeridians, NULL, (DialogPtr) -1);
-	GetPort(&oldport);	
-	SetPort(win);
-
-	GetDItem(win, diMeridiansOnOff, &itemtype, &itemhandle, &itemrect);
-	SetCtlValue((ControlHandle) itemhandle, map->vp->draw_meridians);
-
-	GetDItem(win, diMeridiansInterval, &itemtype, &itemhandle, &itemrect);
-	NumToString(map->vp->meridian_interval, tmpstr);
-	SetIText(itemhandle, tmpstr);
-
-	ShowWindow(win);
-	while (!done) {
-
-		SetCursor(&QD(arrow));
-		draw_default_button(win, diMeridiansOK);
-		ModalDialog(NULL, &ditem);
-		switch (ditem) {
-
-			case diMeridiansOnOff:
-				GetDItem(win, ditem, &itemtype, &itemhandle, &itemrect);
-				SetCtlValue((ControlHandle) itemhandle, !GetCtlValue((ControlHandle) itemhandle));
-				break;
-
-			case diMeridiansOK:
-				GetDItem(win, diMeridiansOnOff, &itemtype, &itemhandle, &itemrect);
-				map->vp->draw_meridians = GetCtlValue((ControlHandle) itemhandle);
-				GetDItem(win, diMeridiansInterval, &itemtype, &itemhandle, &itemrect);
-	        			GetIText(itemhandle,tmpstr);
-				StringToNum(tmpstr, itemnum);
-				/* Dont allow negative values */
-				map->vp->meridian_interval = max(*itemnum, 0);
-
-				/* Redraw the gworld (and the map) */
-				force_map_update(map);
-				done = TRUE;
-				break;
-
-			case diMeridiansCancel:
-
-				done = TRUE;
-				break;
-
-			default:
-				break;
-		}
-	}
-	DisposeDialog(win);
-	SetPort(oldport);
-	update_all_map_windows();
-}
-
 void
 update_all_windows(void)
 {
