@@ -9,8 +9,15 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 #include "macconq.h"
 
-static pascal void help_vscroll_fn(ControlHandle control, short code);
 static void insert_newlines(TEHandle te, int n);
+static void create_instructions_window(void);
+static void describe_menus(int arg, char *key, TextBuffer *buf);
+static void describe_mouse(int arg, char *key, TextBuffer *buf);
+static void describe_help(int arg, char *key, TextBuffer *buf);
+static void create_help_window(void);
+static void set_help_content(HelpNode *curnode);
+static void adjust_help_scrollbar(void);
+static pascal void help_vscroll_fn(ControlHandle control, short code);
 
 /* Globals for the instructions window. */
 
@@ -109,6 +116,8 @@ create_instructions_window()
 	closewinbutton = GetNewControl(cCloseButton, instructionswin);
 	MoveControl(closewinbutton, 235, 200);
 	add_window_menu_item("Instructions", instructionswin);
+	/* Finally make it floating. */
+	MakeFloat(instructionswin);
 }
 
 /* New function, in part derived from old create_instructions_dialog. */
@@ -121,6 +130,7 @@ draw_instructions()
 	Scorekeeper *sk;
 	GrafPtr oldport;
 	Rect textrect;
+	Rect tmprect;
 	Str255 pstr;
 	char *str;
 
@@ -234,6 +244,13 @@ draw_instructions()
 			TEInsert(str, strlen(str), instructions_text);
 		}
 	}
+	/* Draw a default line around the close button. */
+	tmprect = (*closewinbutton)->contrlRect;
+	PenSize(3, 3);
+	InsetRect(&tmprect, -4, -4);
+	FrameRoundRect(&tmprect, 16, 16);
+	InsetRect(&tmprect,  4,  4);
+	PenNormal();
 	SetPort(oldport);
 }
 
@@ -246,6 +263,29 @@ activate_instructions(int activate)
 	HiliteControl(closewinbutton, (activate ? 0 : 255));
 }
 
+int 
+do_key_down_instructions(char key)
+{
+	int		handled = TRUE;
+	GrafPtr	oldport;
+
+	GetPort(&oldport);
+	SetPort(instructionswin);
+
+	switch (key) {
+		case 3:		/* Enter. */
+		case 13:		/* Return. */
+		case 27:		/* Escape. */
+			close_window(instructionswin);
+			break;
+		default:
+			handled = FALSE;
+			break;
+	}
+	SetPort(oldport);
+	return handled;
+}
+
 /* New function. Replaces hit_instructions_dialog. */
 
 void
@@ -383,12 +423,12 @@ create_help_window()
 	SetPort(helpwin);
 	TextFont(title_font_id);
 	TextSize(title_font_size);
-	SetRect(&helptopicrect, 45, 45, 220, 75); 
+	SetRect(&helptopicrect, 60, 45, 220, 84); 
 	helptopic = TENew(&helptopicrect, &helptopicrect);
 	TextFont(small_font_id);
 	TextSize(small_font_size);
 	h = window_width(helpwin);  v = window_height(helpwin);
-	SetRect(&viewrect, 5, 75, h - floatsbarwid, v); 
+	SetRect(&viewrect, 5, 84, h - floatsbarwid, v); 
 	destrect = viewrect;
 	helptext = TENew(&destrect, &destrect);
 	/* Set up a vertical scrollbar. */
@@ -471,7 +511,7 @@ draw_help()
 	/* Draw a horizontal line under the navigation buttons. */
 	MoveTo(0, 30);
 	LineTo(helpwin->portRect.right - floatsbarwid, 30);
-	SetRect(&tmprect, 5, 40, 5 + 32, 40 + 32);
+	SetRect(&tmprect, 5, 35, 5 + 48, 35 + 48);
 	/* Grow so as to erase extra junk that may have been drawn. */
 	InsetRect(&tmprect, -2, -2);
 	EraseRect(&tmprect);
@@ -479,10 +519,10 @@ draw_help()
 	InsetRect(&tmprect, 2, 2);
 	/* Draw a bit of graphic for some help nodes. */
 	if (cur_help_node->nclass == utypenode && is_unit_type(cur_help_node->arg)) {
-		draw_unit_image(helpwin, tmprect.left, tmprect.top, 32, 32,
+		draw_unit_image(helpwin, tmprect.left, tmprect.top, 44, 44,
 						cur_help_node->arg, 0, 0, 0, 0, FALSE);
 	} else if (cur_help_node->nclass == ttypenode && is_terrain_type(cur_help_node->arg)) {
-		draw_terrain_sample(helpwin, tmprect, cur_help_node->arg); 
+		draw_terrain_sample(helpwin, tmprect, cur_help_node->arg, 5); 
 	} else if (cur_help_node->nclass == mtypenode && is_material_type(cur_help_node->arg)) {
 		/* (should draw material type usefully) */ 
 	}
