@@ -36,14 +36,14 @@ any later version.  See the file COPYING.  */
 
 #define mac_computed_rowbytes(w, pixelsize) (((w * pixelsize + 15) / 16) * 2)
 
+/* These need to be extern since they are called by imfapp.c. */
+
 extern int hasColorQD;
-
-extern PixPatHandle mac_create_ppat(Image *img);
 extern CIconHandle mac_create_cicn(Image *img);
-extern void mac_init_cicn(Image *img);
 extern void mac_add_cicn(ImageFamily *imf, Handle handle);
-extern void mac_add_ppat(ImageFamily *imf, Handle handle);
 
+static PixPatHandle mac_create_ppat(Image *img);
+static void mac_add_ppat(ImageFamily *imf, Handle handle);
 static MacImage *init_mac_image(Image *img);
 static void mac_interp_image(ImageFamily *imf, Image *img, int force);
 static void mac_interp_image_1(ImageFamily *imf, Image *img, Image *subimg, int subi, int force);
