@@ -9,10 +9,6 @@ any later version.  See the file COPYING. */
 #include "conq.h"
 #include "macconq.h"
 
-extern void send_version PARAMS ((int rid));
-extern void debugging_string PARAMS ((char *buf));
-extern void mac_init_cicn(Image *img);
-
 extern char *connection_method_name;
 
 /* Limit on the number of check boxes and sliders in the variants dialog. */
@@ -38,8 +34,12 @@ static Module *module_from_cell(Cell cell);
 static pascal Boolean filter_new_game(DialogPtr dialog, EventRecord *evt, short *itemhit);
 static void display_selected_game(void);
 static void select_game(void);
+static int start_new_game(void);
+static void update_new_game_list(void);
+
 static pascal void draw_variant_slider(WindowPtr win, short ditem);
 static pascal void draw_variant_help(WindowPtr win, short ditem);
+static int variants_dialog(void);
 static void set_variant_item(int di, int flag, int flag2, int val);
 static void interpret_variants(void);
 static void implement_variants(void);
@@ -49,18 +49,33 @@ static pascal Boolean filter_world_setup(DialogPtr dialog, EventRecord *evt, sho
 static int world_shape_dialog(void);
 static int real_time_dialog(void);
 static int more_variants_dialog(void);
+
 static pascal void draw_player_setup_list(WindowPtr win, short ditem);
 static pascal void draw_player_setup_advantage(WindowPtr win, short ditem);
 static int adjust_advantage(Player *player, Side *side, int amt);
 static void select_player(int n);
 static int player_setup_dialog(void);
-static void configure_indepside_dialog(void);
 static void set_player_setup_button_states(void);
+static void hit_player_setup_dialog(int itemhit, Point mouse);
+static void configure_indepside_dialog(void);
+
+static int open_game_from_name_and_volume(Str255 name, int vrefnum);
+static int launch_game(void);
+static void launch_game_2(void);
+static void check_for_missing_images(void);
+
 static pascal void draw_progress(WindowPtr win, short ditem);
-static void add_default_emblem_image(ImageFamily *imf, int e, char *name);
-static void dispose_terrain_images(void);
-static void dispose_emblem_images(void);
+static void open_progress_dialog(void);
+static void close_progress_dialog(void);
+
+static void init_display(void);
+static void init_material_images(void);
+static void init_terrain_images(void);
+
+static void dispose_material_images(void);
 static void dispose_unit_images(void);
+static void dispose_emblem_images(void);
+static void dispose_terrain_images(void);
 
 /* Global variables. */
 
@@ -198,6 +213,10 @@ UserItemUPP draw_player_setup_advantage_proc;
 ImageFamily **uimages = NULL;
 ImageFamily **timages = NULL;
 ImageFamily **eimages = NULL;
+ImageFamily **mimages = NULL;
+
+ImageFamily *generic_transition = NULL;
+ImageFamily *beach = NULL;
 
 MacImage **tcolors = NULL;
 
@@ -322,43 +341,49 @@ init_cursors()
 static pascal Boolean
 filter_splash(DialogPtr dialog, EventRecord *evt, short *itemhit)
 {
+	int	update = FALSE;
 	char ch;
 
 	/* Look for the right kind of event. */
 	switch (evt->what) {
 		case keyDown:
 			ch = evt->message & charCodeMask;
-			if (ch == 3 || ch == 13) {
+			if (ch == 3 || ch == 13 || ch == 'n' || ch == 'N') {
 				*itemhit = diSplashNew;
 				return TRUE;
+			} else if (ch == 27 || ch == 'q' || ch == 'Q') {
+				*itemhit = diSplashQuit;
+				return TRUE;
+			} else if (ch == 'o' || ch == 'O') {
+				*itemhit = diSplashOpen;
+				return TRUE;
 			}
 #ifdef DEBUGGING
 			/* A secret way to get debugging at startup. */
-			if (ch == 'D') {
+			else if (ch == 'D') {
 				toggle_debugging(&Debug);
-			}
-			if (ch == 'M') {
+				update = TRUE;	
+			} else if (ch == 'M') {
 				toggle_debugging(&DebugM);
-			}
-			if (ch == 'G') {
+				update = TRUE;	
+			} else if (ch == 'G') {
 				toggle_debugging(&DebugG);
-			}
-			if (ch == 'P') {
+				update = TRUE;	
+			} else if (ch == 'P') {
 				toggle_profiling();
+				update = TRUE;	
 			}
-			{
+			/* We only want to update the string if D, M, G or T was hit. */
+			if (update){
 				char buf[BUFSIZE];
 				Str255 tmpstr;
 				short itemtype;  Handle itemhandle;  Rect itemrect;
 
 				GetDItem(dialog, diSplashVersion, &itemtype, &itemhandle, &itemrect);
-
 				/* (debugging_string doesn't handle profiling). */
-				if (Debug || DebugM || DebugG || Profile) {
-					sprintf(buf, "%c%c%c%c",
-							(Debug ? 'D' : ''), (DebugM ? 'M' : ''), (DebugG ? 'G' : ''),
-							(Profile ? 'P' : ''));
-				}
+				sprintf(buf, "%c%c%c%c",
+						(Debug ? 'D' : ''), (DebugM ? 'M' : ''), 
+						(DebugG ? 'G' : ''), (Profile ? 'P' : ''));
 				c2p(buf, tmpstr);
 				SetIText(itemhandle, tmpstr);
 				DrawControls(dialog);
@@ -399,8 +424,11 @@ do_splash_box()
 			SetDItem(win, diSplashPicture, itemtype, (Handle) pic, &itemrect);
 		}
 	}
+	/* Hide defunct connect button for now. */
+	HideDItem(win, diSplashConnect);
 	ShowWindow(win);
 	SelectWindow(win);
+	draw_default_button(win, diSplashNew);
 	ModalDialog(filter_splash_proc, &ditem);
 	/* We don't loop around here, just return the ditem and let caller decide what to do. */
 	DisposeDialog(win);
@@ -1789,8 +1817,8 @@ draw_player_setup_list(WindowPtr win, short ditem)
 	short itemtype;  Handle itemhandle;  Rect itemrect;
 
 	GetDItem(win, ditem, &itemtype, &itemhandle, &itemrect);
-	TextFont(newYork);
-	TextSize(12);
+	TextFont(large_font_id);
+	TextSize(large_font_size);
 	/* Default to normal size and spacing. */
 	playerh = 22;
 	playerbaseline = 14;
@@ -1801,7 +1829,8 @@ draw_player_setup_list(WindowPtr win, short ditem)
 		playerh = 16;
 		playerbaseline = 11;
 		numvisentries = (itemrect.bottom - itemrect.top) / playerh;
-		TextSize(10);
+		TextFont(small_font_id);
+		TextSize(small_font_size);
 	}
 	tmprgn = NewRgn();
 	GetClip(tmprgn);
@@ -1809,7 +1838,7 @@ draw_player_setup_list(WindowPtr win, short ditem)
 	entryrect = itemrect;
 	entryrect.bottom = entryrect.top + playerh;
 	InsetRect(&entryrect, 1, 1);
-	for (i = firstside; i <= g_sides_max(); ++i) {
+	for (i = firstside; i <= firstside + g_sides_max(); ++i) {
 		if (i >= numvisentries) {
 			/* Not visible, don't draw. */
 		} else if (i < numtotsides) {
@@ -1825,21 +1854,18 @@ draw_player_setup_list(WindowPtr win, short ditem)
 			sidetitle = short_side_title(side);
 			if (empty_string(sidetitle))
 			  sidetitle = "(unnamed)";
-			MoveTo(entryrect.left + 2 + 16 + 2, entryrect.top + playerbaseline);
-			TextFace(0);
+			MoveTo(entryrect.left + 2 + 16 + 8, entryrect.top + playerbaseline);
 			DrawText(sidetitle, 0, strlen(sidetitle));
 			/* Only draw other info for sides "in" this game. */
 			if (side->ingame) {
 				/* Describe the intended player for the side. */
 				player = assignments[i].player;
 				long_player_title(playertitle, player, "mac");
-				MoveTo(entryrect.left + 2 + 130, entryrect.top + playerbaseline);
-				TextFace(italic);
+				MoveTo(entryrect.left + 2 + 170, entryrect.top + playerbaseline);
 				DrawText(playertitle, 0, strlen(playertitle));
 				/* Indicate the player's initial advantage. */
 				sprintf(spbuf, "%d", player->advantage);
-				MoveTo(entryrect.left + 2 + 260, entryrect.top + playerbaseline);
-				TextFace(0);
+				MoveTo(entryrect.left + 2 + 284, entryrect.top + playerbaseline);
 				DrawText(spbuf, 0, strlen(spbuf));
 				if (player->remotewanted) {
 					MoveTo(entryrect.right - 15, entryrect.top + playerbaseline);
@@ -2013,8 +2039,8 @@ set_player_setup_button_states()
 
 	set_player_button(diPlayerSetupOK, numremotewaiting == 0);
 	set_player_button(diPlayerSetupConfigure, indepside->ingame);
-	set_player_button(diPlayerSetupAdd, !locked && (numtotsides < g_sides_max()));
-	set_player_button(diPlayerSetupRemove, player && !locked && (numtotsides > g_sides_min()));
+	set_player_button(diPlayerSetupAdd, !locked && (numsides < g_sides_max()));
+	set_player_button(diPlayerSetupRemove, player && !locked && (numsides > g_sides_min()));
 	set_player_button(diPlayerSetupRename, player && !(side->nameslocked));
 	set_player_button(diPlayerSetupAI, player);
 	set_player_button(diPlayerSetupRemote, player);
@@ -2030,10 +2056,24 @@ set_player_setup_button_states()
 	SetIText(itemhandle, tmpstr);
 }
 
-/* Handle any event that we don't want DialogSelect to get. */
+void
+do_mouse_down_setup(Point mouse, int mods)
+{
+	short ditem;
+
+	/* Check if a dialog item was hit and branch to dialog handling code in that case.
+	This makes it possible to avoid using the Dialog Manager. */
+	ditem = FindDItem(playersetupwin, mouse) + 1;	/* FindDItem returns (ditem -1)! */
+	if (ditem > 0) {
+		/* First undo previous conversion to locals by do_mouse_down. */
+		LocalToGlobal(&mouse);
+		hit_player_setup_dialog(ditem, mouse);
+		return;
+	}
+}
 
 void
-handle_player_setup_event(EventRecord *event)
+do_key_down_setup(EventRecord *event)
 {
 	char ch;
 	Point mouse;
@@ -2083,10 +2123,6 @@ handle_player_setup_event(EventRecord *event)
 				draw_default_button(playersetupwin, diPlayerSetupOK);
 			}
 			break;
-		case kHighLevelEvent:
-			/* Hear from anyone attempting to join via AppleEvent. */
-			AEProcessAppleEvent(event);
-			break;
 	}
 }
 
@@ -2100,6 +2136,7 @@ hit_player_setup_dialog(int ditem, Point mouse)
 	Rect itemrect;
 	Side *tmpside;
 	GrafPtr oldport;
+	char *aitypename;
 
 	tmpplayer = selected_player();
 	tmpside = selected_side();
@@ -2123,12 +2160,14 @@ hit_player_setup_dialog(int ditem, Point mouse)
 			done = TRUE;
 			break;
 		case diPlayerSetupAdd:
-			add_side_and_player();
+			net_add_side_and_player();
 			/* Select the newly-created side/player pair. */
 			select_player(numsides);
 			/* Put it under mplayer control by default. */
 			tmpplayer = selected_player();
-			tmpplayer->aitypename = "mplayer";
+			tmpplayer->aitypename = next_ai_type_name(NULL);
+			/* (Would be better to just init the new side's colors.) */
+			init_side_colors();
 			init_emblem_images();
 			changedall = TRUE;
 			break;
@@ -2140,24 +2179,16 @@ hit_player_setup_dialog(int ditem, Point mouse)
 			}
 			break;
 		case diPlayerSetupRename:
-			tmpside->name = NULL;
-			tmpside->noun = NULL;
-			/* always need to clear this cache before renaming... */
-			tmpside->pluralnoun = NULL;
-			tmpside->adjective = NULL;
-			make_up_side_name(tmpside);
+			net_rename_side_for_player(selectedplayer, -1);
 			init_emblem_images();
 			changedselected = TRUE;
 			break;
 		case diPlayerSetupAI:
-			/* Click between mplayer, iplayer and no AI. */
-			if (strcmp(tmpplayer->aitypename, "mplayer") == 0) {
-				tmpplayer->aitypename = "iplayer";
-			} else if (strcmp(tmpplayer->aitypename, "iplayer") == 0) {
-				tmpplayer->aitypename = NULL;
-			} else {
-				tmpplayer->aitypename = "mplayer";
-			}
+	    		aitypename = next_ai_type_name(NULL);
+			while (strcmp(tmpplayer->aitypename, aitypename) != 0) {
+	    			aitypename = next_ai_type_name(aitypename);
+			}				
+	    		tmpplayer->aitypename = next_ai_type_name(aitypename);
 			changedselected = TRUE;
 			break;
 		case diPlayerSetupRemote:
@@ -2405,13 +2436,14 @@ launch_game()
 		/* Cancelled from variants dialog, go back to wherever. */
 		return FALSE;
 	}
-	init_side_colors();
 	/* Trial side/player assignment, will create the default player if nececessary. */
 	make_trial_assignments();
+	init_side_colors();
 	/* Set up the images we'll be needing. */
 	init_terrain_images();
 	init_emblem_images();
 	init_unit_images();
+	init_material_images();
 	/* Ask for modifications to the trial assignment. */
 	return player_setup_dialog();
 }
@@ -2754,12 +2786,32 @@ init_display()
 #ifdef DESIGNERS
 	/* If we're designing, bring up the palette automatically. */
 	if (dside->designer) {
-		create_design_window();
+		show_design_window();
 	}
 #endif /* DESIGNERS */
 	Dprintf("Opened Mac display!\n");
 }
 
+void
+init_material_images()
+{
+	int m;
+	ImageFamily *imf;
+
+	/* First trash the old images if they exist. */
+	if (mimages) {
+		dispose_material_images();
+	}
+	mimages = (ImageFamily **) xmalloc(nummtypes * sizeof(ImageFamily *));
+
+	for_all_material_types(m) {
+		imf = get_material_type_images(dside, m);
+		if (DebugG)
+		  describe_imf(dside, "material type", m_type_name(m), imf);
+		mimages[m] = imf;
+	}
+}
+
 void
 init_unit_images()
 {
@@ -2794,7 +2846,7 @@ init_emblem_images()
        may be added later. */
 	eimages = (ImageFamily **) xmalloc((g_sides_max() + 1) * sizeof(ImageFamily *));
 
-	for_all_sides_plus_indep(side2) {
+	for_all_sides(side2) {
 		s2 = side_number(side2);
 		eimages[s2] = get_emblem_images(dside, side2);
 	}
@@ -2858,6 +2910,46 @@ init_terrain_images()
 		}
 	}
 	get_unseen_images(dside);
+
+    	/* Get terrain transition bitmaps. */
+	imf = get_generic_images("transition");
+	record_imf_get(imf);
+	if (DebugG)
+	  describe_imf(dside, "transition", side_desig(dside), imf);
+	generic_transition = imf;
+
+    	/* Get beach bitmaps. */
+	imf = get_generic_images("beach");
+	record_imf_get(imf);
+	if (DebugG)
+	  describe_imf(dside, "beach", side_desig(dside), imf);
+	beach = imf;
+}
+
+void
+dispose_material_images()
+{
+	int m;
+	CIconHandle cicnhandle;
+	MacImage *macimg;
+	Image *img;
+	
+	for_all_material_types(m) {
+		for_all_images(mimages[m], img) {
+			if (img && img->hook) {
+				macimg = (MacImage *) img->hook;
+				if (macimg->colricon != NULL) {
+					cicnhandle = (CIconHandle) macimg->colricon;
+					DisposHandle((Handle) (*cicnhandle)->iconPMap.pmTable);
+					DisposHandle((Handle) (*cicnhandle)->iconData);
+					DisposHandle((Handle) cicnhandle);
+					macimg->colricon = nil;
+					macimg->cicn_inited = FALSE;
+				}
+			}
+		}
+	}
+	free(mimages);
 }
 
 void
@@ -2894,7 +2986,7 @@ dispose_emblem_images()
 	MacImage *macimg;
 	Image *img;
 	return;
-	for_all_sides_plus_indep(side) {
+	for_all_sides(side) {
 		for (img = eimages[side_number(side)]->images; img != NULL; img = img->next) {
 			if (img && img->hook) {
 				macimg = (MacImage *) img->hook;
