@@ -1,5 +1,5 @@
 /* Unit lists for the Mac interface to Xconq.
-   Copyright (C) 1992, 1993, 1994, 1995, 1996, 1998 Stanley T. Shebs.
+   Copyright (C) 1992-1996, 1998, 2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -9,23 +9,53 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 #include "macconq.h"
 
+static void init_list_contents(List *list);
+static void organize_list_contents(List *list);
+static void sort_list_contents(List *list);
+static void add_unit_to_list(List *list, Unit *unit);
+
+static void set_list_scrollbars(List *list);
+static void draw_list_contents(List *list);
+static void draw_list_headings(List *list);
+static void draw_unit_list_entry(List *list, int n, int clearfirst);
+static void adjust_list_decor(List *list);
+
 static pascal void list_vscroll_fn(ControlHandle control, short code);
 static pascal void list_hscroll_fn(ControlHandle control, short code);
+static void auto_scroll_list_up_arrow();
+static void auto_scroll_list_down_arrow();
+static void auto_scroll_list_right_arrow();
+static void auto_scroll_list_left_arrow();
+
+static void insert_unit_into_list(List *list, Unit *unit);
+static void remove_unit_from_list(List *list, Unit *unit);
+static int unit_position_in_list(List *list, Unit *unit);
+static void redraw_unit_list_entry(List *list, int n);
+static void clear_selections(List *list);
+
 
 /* (should be adjusted for expected range of values) */
 
 int curactcolw = 10;
 int imagecolw = 20;
 int namecolw = 100;
-int typecolw = 70;
+int typecolw = 80;
 int sidecolw = 70;
 int hpcolw = 40;
-int acpcolw = 60;
-int poscolw = 60;
+int acpcolw = 50;
+int poscolw = 50;
+int taskcolw = 130;
 int supcolw = 40;
-int taskcolw = 100;
 int notecolw = 50;
 
+/* A notes column is needed only if units are scheduled to appear. */
+
+int notes_column_needed = -1;
+
+/* A supply column is needed only if any unit can store the mtype. */
+
+int *sup_column_needed = NULL;
+
 /* Sum of non-scrolling field widths. */
 
 int fixedfieldwidth;
@@ -42,8 +72,8 @@ int listtopbaseline = 13;
 
 /* The possible heights of each individual list entry. */
 
-int smallentryspacing = 18;
-int largeentryspacing = 34;
+int smallentryspacing = 24;
+int largeentryspacing = 46;
 
 int listnum = 1;
 
@@ -57,10 +87,30 @@ ControlActionUPP list_hscroll_proc;
 void
 create_list()
 {
-	int i;
+	int i, m;
 	List *list = (List *) xmalloc(sizeof(List));
+	Unit *unit;
 	Rect hscrollrect, vscrollrect;
 
+	/* If this is the first list, test if we need supply columns. */ 
+	if (sup_column_needed == NULL) {
+		sup_column_needed = (int *) xmalloc(sizeof(int) * nummtypes);
+		for_all_material_types(m) {
+			sup_column_needed[m] = storage_possible(m);
+		}		
+	}
+	/* If this is the first list, test if we need a notes column. */ 
+	if (notes_column_needed == -1) {
+		for_all_units(unit) {
+			if (side_sees_unit(dside, unit)
+				&& !inside_area(unit->x, unit->y)
+				&& unit_appear_turn(unit) >= 0) {
+				notes_column_needed = TRUE;
+				break;
+			}
+			notes_column_needed = FALSE;
+		}
+	}
 	maxlistwidth = 0;
 	maxlistwidth += curactcolw;
 	maxlistwidth += imagecolw;
@@ -71,10 +121,14 @@ create_list()
 	maxlistwidth += hpcolw;
 	maxlistwidth += acpcolw;
 	maxlistwidth += poscolw;
-	maxlistwidth += nummtypes * supcolw;
+	for_all_material_types(m) {
+		if (sup_column_needed[m])
+		    maxlistwidth += supcolw;
+	}
 	maxlistwidth += taskcolw;
-	maxlistwidth += notecolw;
-
+	if (notes_column_needed) {
+		maxlistwidth += notecolw;
+	}
 	DGprintf("Creating a list\n");
 	/* Default to listing every side and indeps too. */
 	list->sides = ALLSIDES;
@@ -108,7 +162,7 @@ create_list()
 	hscrollrect = list->window->portRect;
 	hscrollrect.top = hscrollrect.bottom - sbarwid;
 	hscrollrect.bottom += 1;
-	hscrollrect.left = fixedfieldwidth - 1;
+	hscrollrect.left = fixedfieldwidth - 5;
 	hscrollrect.right -= sbarwid - 1;
 	list->hscrollbar = NewControl(list->window, &hscrollrect, "\p", TRUE,
 			 					  0, 0, 100, scrollBarProc, 0L);
@@ -143,8 +197,21 @@ init_list_contents(List *list)
 void
 organize_list_contents(List *list)
 {
+	int numvisunits, n, first = FALSE;
 	Side *side2;
-	Unit *unit;
+	Unit *unit, *unit2 = NULL, *unit3 = NULL;
+	Rect winrect = list->window->portRect;
+
+	/* Find and save the first visible non-dead unit in the list. */
+	n = list->firstvisible;
+	while (list->numunits > 0 && !first) {
+		unit2 = unit_in_vector(list->contents, n);
+		if (in_play(unit2))
+		    first = TRUE;
+		n++;
+	}
+	/* Save the selected unit. */
+	unit3 = selected_unit_in_list(list);	
 
 	/* Build up the array of units for this list. */
 	list->numunits = 0;
@@ -162,13 +229,27 @@ organize_list_contents(List *list)
 			}
 		}
 	}
-	for_all_side_units(indepside, unit) {
-		if (side_sees_image(dside, unit)) {
-			add_unit_to_list(list, unit);
-		}
-	}
 	/* Now sort the list according to its keys. */
 	sort_list_contents(list);
+	
+	/* Restore the old selected unit. */
+	if (in_play(unit3)
+	    && (n = unit_position_in_list(list, unit3)) >= 0) {	
+	 	    list->contents->units[n].flag = TRUE;
+	}
+	/* Restore the old first visible unit if possible. */
+	if (first) {
+		list->firstvisible = unit_position_in_list(list, unit2);
+		numvisunits = (winrect.bottom - winrect.top - listtoph - sbarwid) / list->entryspacing;
+		numvisunits = min(numvisunits, list->numunits);
+		list->lastvisible = list->firstvisible + numvisunits - 1;
+		/* Check that lastvisible is less than numunits. */
+		list->lastvisible = min(list->lastvisible, list->numunits - 1);
+		/* Readjust firstvisible if necessary. */
+		list->firstvisible = max(0, min(list->firstvisible, list->lastvisible - numvisunits + 1));
+	}
+	/* The number of units may have changed, so we need to update the scrollbars. */
+	set_list_scrollbars(list);
 }
 
 void
@@ -205,22 +286,14 @@ set_list_scrollbars(List *list)
 	/* Set the vertical scrollbar correctly. */	
 	numvisunits = (winrect.bottom - winrect.top - sbarwid - listtoph) / list->entryspacing;
 	maxval = max(0, list->numunits - numvisunits);
-	val = GetCtlValue(list->vscrollbar);
 	SetCtlMax(list->vscrollbar, maxval);
-	if (val > maxval) {
-		SetCtlValue(list->vscrollbar, maxval);
-		list->firstvisible = maxval;
-	}
+	SetCtlValue(list->vscrollbar, min(list->firstvisible, maxval));
 	/* Set up the horizontal scrollbar. */
 	visfieldwidth = winrect.right - winrect.left - sbarwid - fixedfieldwidth;
-	val = GetCtlValue(list->hscrollbar);
 	maxval = (maxlistwidth - fixedfieldwidth) - visfieldwidth;
 	maxval = max(0, maxval);
 	SetCtlMax(list->hscrollbar, maxval);
-	if (val > maxval) {
-		SetCtlValue(list->hscrollbar, maxval);
-		list->firstvisfield = maxval;
-	}
+	SetCtlValue(list->hscrollbar, min(list->firstvisfield, maxval));
 }
 
 List *
@@ -239,21 +312,26 @@ void
 draw_list(List *list)
 {
 	WindowPtr listwin = list->window;
-	Rect tmprect, cliprect;
+	Rect tmprect;
 	RgnHandle tmprgn;
 
-	tmprgn = NewRgn();
-	GetClip(tmprgn);
+	/* Erase the content tect. */
 	tmprect = listwin->portRect;
 	tmprect.right -= sbarwid;
 	tmprect.bottom -= sbarwid;
 	BackPat(QDPat(white));
 	EraseRect(&tmprect);
-	/* Set up clipping for the contents of the list. */
-	cliprect = listwin->portRect;
-	cliprect.right -= sbarwid;
-	cliprect.bottom -= sbarwid;
-	ClipRect(&cliprect);
+	/* Also erase small rect to the left of the bottom
+	scrollbar. */
+	tmprect = listwin->portRect;
+	tmprect.right = tmprect.left + fixedfieldwidth - 5;
+	tmprect.top = tmprect.bottom - sbarwid;
+	BackPat(QDPat(white));
+	EraseRect(&tmprect);
+	/* Make sure the list is up to date before redrawing it. */
+	organize_list_contents(list);
+	tmprgn = NewRgn();
+	GetClip(tmprgn);
 	draw_list_contents(list);
 	SetClip(tmprgn);
 	DisposeRgn(tmprgn);
@@ -273,12 +351,13 @@ draw_list_contents(List *list)
 	numvisunits = (winrect.bottom - winrect.top - listtoph - sbarwid) / list->entryspacing;
 	numvisunits = min(numvisunits, list->numunits);
 	list->lastvisible = list->firstvisible + numvisunits - 1;
+	/* Check that lastvisible is less than numunits. */
+	list->lastvisible = min(list->lastvisible, list->numunits - 1);
+	/* Readjust firstvisible if necessary. */
+	list->firstvisible = max(0, min(list->firstvisible, list->lastvisible - numvisunits + 1));
 	for (line = list->firstvisible; line <= list->lastvisible; ++line) {
 		draw_unit_list_entry(list, line, FALSE);
 	}
-	/* The HiliteControl calls at this point caused a display bug. They were not
-	necessary since UpdateControls is smart enough to figure out if a scrollbar
-	should be displayed as active or not depending on the window size. */
 	viswidth = winrect.right - winrect.left - sbarwid;
 }
 
@@ -288,11 +367,8 @@ draw_list_headings(List *list)
 	int x = 0, m;
 	Str255 tmpstr;
 	Rect cliprect, tmprect, winrect = list->window->portRect;
-	RgnHandle tmprgn;
+	RGBColor tmpcolor;
 
-	/* Save the current clip region. */
-	tmprgn = NewRgn();
-	GetClip(tmprgn);
 	cliprect = winrect;
 	cliprect.right -= sbarwid;
 	ClipRect(&cliprect);
@@ -305,8 +381,8 @@ draw_list_headings(List *list)
 	MoveTo(0, listtoph - 1);
 	Line(winrect.right, 0);
 	/* Draw a dividing line between the fixed and mobile fields. */
-	MoveTo(fixedfieldwidth - 15, 0);
-	LineTo(fixedfieldwidth - 15, listtoph - 1);
+	MoveTo(fixedfieldwidth - 5, 0);
+	LineTo(fixedfieldwidth - 5, listtoph - 1);
 	/* (should underline sort keys with varying line heaviness) */
 	/* We have to do MoveTo everywhere because DrawString moves the pen. */
 	/* First draw headings for fields not affected by horizontal scrolling. */
@@ -335,30 +411,36 @@ draw_list_headings(List *list)
 	x += hpcolw;
 	MoveTo(x, listtopbaseline);
 	DrawString("\pAcp");
+	MoveTo(x + 20, listtopbaseline);
+	tmpcolor = (RGBColor) {0x8FFF, 0x8FFF, 0x8FFF};
+	RGBForeColor(&tmpcolor);
+	DrawString("\pCp");
+	ForeColor(blackColor);
 	x += acpcolw;
 	MoveTo(x, listtopbaseline);
-	DrawString("\pLoc");
+	DrawString("\pLocation");
 	x += poscolw;
-	for_all_material_types(m) {
-		MoveTo(x, listtopbaseline);
-		c2p(m_type_name(m), tmpstr);
-		DrawString(tmpstr);
-		x += supcolw;
-	}
 	MoveTo(x, listtopbaseline);
 	DrawString("\pTask");
 	x += taskcolw;
-	MoveTo(x, listtopbaseline);
-	DrawString("\pNotes");
-	x += notecolw;
+	for_all_material_types(m) {
+		if (sup_column_needed[m]) {
+			MoveTo(x, listtopbaseline);
+			c2p(m_type_name(m), tmpstr);
+			DrawString(tmpstr);
+			x += supcolw;
+		}
+	}
+	if (notes_column_needed) {
+		MoveTo(x, listtopbaseline);
+		DrawString("\pNotes");
+		x += notecolw;
+	}
 	/* Draw a gray line indicating the end of the data columns. */
 	PenPat(QDPat(gray));
 	MoveTo(x, 0);
 	Line(0, listtoph - 4);
 	PenNormal();
-	/* Restore the clipping. */
-	SetClip(tmprgn);
-	DisposeRgn(tmprgn);
 }
 
 /* This draws a one-line entry for the given unit. */
@@ -369,33 +451,44 @@ draw_list_headings(List *list)
 void
 draw_unit_list_entry(List *list, int n, int clearfirst)
 {
-	int u, m, x, y = (n - list->firstvisible) * list->entryspacing + listtoph;
-	int texty = y + 15;
-	char tmpnbuf[BUFSIZE];
-	Side *side2;
-	Rect entryrect, tmprect;
-	Unit *unit = list->contents->units[n].unit;
-	Rect cliprect;
+	Unit *unit = unit_in_vector(list->contents, n);
 	Rect bbox = (*(list->window->visRgn))->rgnBBox;
-
-	SetRect(&entryrect, 0, y, list->window->portRect.right, y + list->entryspacing);
-#if 0
-	/* (should use rect intersection?) */
-	if (!between(bbox.top, y, bbox.bottom)
-	    && !between(bbox.top, y+list->entryspacing, bbox.bottom)) return;
-#endif
-	cliprect = list->window->portRect;
-	cliprect.right -= sbarwid;
+	Rect winrect = list->window->portRect;
+	Rect entryrect, tmprect, cliprect;
+	char tmpnbuf[BUFSIZE];
+	int u, m, x, y, texty;
+	RGBColor tmpcolor;
+	Side *side2;
+	
+	y = (n - list->firstvisible) * list->entryspacing + listtoph;
+	texty = y + 15;
+	/* Clip to the entry rect. */
+	SetRect(&entryrect, 0, y, winrect.right - sbarwid, y + list->entryspacing);
+	cliprect = entryrect;
 	ClipRect(&cliprect);
-
-	if (clearfirst) {
+	/* Draw incomplete unit on a blue background. */
+	if (!completed(unit)) {
+		tmpcolor = (RGBColor) {0xCFFF, 0xFFFF, 0xFFFF};
+		RGBForeColor(&tmpcolor);
+		FillRect(&entryrect, QDPat(black));
+		ForeColor(blackColor);
+	/* Else erase the entry rect. */
+	} else if (clearfirst) {
 		EraseRect(&entryrect);
 	}
-	if (unit == NULL || !alive(unit)) {
-		/* We need to recalculate the list contents. */
-		list->shouldreorg = TRUE;
-		return;
+	/* Fixes right end of the red frame. */
+	if (GetCtlValue(list->hscrollbar) != GetCtlMax(list->hscrollbar))
+	    entryrect.right += 2;
+	/* Draw a red frame around the selected unit. */
+	if (list->contents->units[n].flag) {
+		tmpcolor = (RGBColor) {0xFFFF, 0, 0};
+		RGBForeColor(&tmpcolor);
+		PenSize(2, 2);
+		FrameRect(&entryrect);
+		ForeColor(blackColor);
+		PenNormal();
 	}
+	
 	u = unit->type;
 	/* Draw whether the unit is awake or asleep. */
 	if (unit->plan && completed(unit)) {
@@ -404,42 +497,53 @@ draw_unit_list_entry(List *list, int n, int clearfirst)
 		InsetRect(&tmprect, 2, 2);
 		/* (should draw the following analogously to map display) */
 		if (unit->plan->asleep) {
-			/* Leave rectangle blank. */
-		} else if (unit->plan->reserve) {
-			FillRect(&tmprect, QDPat(gray));
-		} else {
+			tmpcolor = (RGBColor) {0xFFFF, 0, 0};
+			RGBForeColor(&tmpcolor);
 			FillRect(&tmprect, QDPat(black));
+			ForeColor(blackColor);
+		} else if (unit->plan->reserve) {
+			tmpcolor = (RGBColor) {0xFFFF, 0, 0xFFFF};
+			RGBForeColor(&tmpcolor);
+			FillRect(&tmprect, QDPat(black));
+			ForeColor(blackColor);
+		} else {
+			tmpcolor = (RGBColor) {0, 0xFFFF, 0};
+			RGBForeColor(&tmpcolor);
+			FillRect(&tmprect, QDPat(black));
+			ForeColor(blackColor);
 		}
 		FrameRect(&tmprect);
 	}
 	/* Draw an icon with side emblem for this unit. */
 	draw_unit_image(list->window, curactcolw + 2, y + 1,
-					(list->largeicons ? 32 : 16), (list->largeicons ? 32 : 16), 
+					(list->largeicons ? 44 : 22), (list->largeicons ? 44 : 22), 
 					u, side_number(unit->side), list->sidecolors, 0, !completed(unit), list->draw_emblems);
 
 	/* Draw unit size on top of advanced unit. */
 	if (u_advanced(unit->type) && list->drawsizes) {
 		draw_unit_size(unit, list->window, 
-			curactcolw + 2, y + 1, (list->largeicons ? 32 : 16), (list->largeicons ? 32 : 16));
+			curactcolw + 2, y + 1, (list->largeicons ? 44 : 22), (list->largeicons ? 44 : 22));
+	}
+	/* Draw gray text for incomplete units, */
+	if (!completed(unit)) {
+		tmpcolor = (RGBColor) {0x8FFF, 0x8FFF, 0x8FFF};
+		RGBForeColor(&tmpcolor);
 	}
-
 	/* Write the name or ordinal number. */
 	name_or_number(unit, spbuf);
 	spbuf[15] = '\0';  /* (should be clipping by pixels) */
 	x = curactcolw + imagecolw;
 	MoveTo(x, texty);
 	DrawText(spbuf, 0, strlen(spbuf));
-	/* Adjust according to the horizontal scroll. */
+	/* Adjust clipping etc. according to the horizontal scroll. */
 	x -= list->firstvisfield;
-	cliprect = list->window->portRect;
-	cliprect.right -= sbarwid;
 	cliprect.left = fixedfieldwidth;
 	ClipRect(&cliprect);
 	/* Write the name of the unit's type. */
 	x += namecolw;
 	MoveTo(x, texty);
 	sprintf(spbuf, "%s", u_type_name(u));
-	spbuf[10] = '\0';
+	spbuf[15] = '\0';
 	DrawText(spbuf, 0, strlen(spbuf));
 	/* Write the side of the unit. */
 	x += typecolw;
@@ -458,7 +562,9 @@ draw_unit_list_entry(List *list, int n, int clearfirst)
 	x += hpcolw;
 	if (side_sees_unit(dside, unit) && inside_area(unit->x, unit->y)) {
 		MoveTo(x, texty);
-		acp_desc(spbuf, unit, FALSE);
+		if (!completed(unit)) {
+			sprintf(spbuf, "%d/%d", unit->cp, u_cp(u));
+		} else acp_desc(spbuf, unit, FALSE);
 		if (strlen(spbuf) == 0)
 		  strcpy(spbuf, "-");
 		DrawText(spbuf, 0, strlen(spbuf));
@@ -475,28 +581,31 @@ draw_unit_list_entry(List *list, int n, int clearfirst)
 		strcpy(spbuf, "  -  ");
 	}
 	DrawText(spbuf, 0, strlen(spbuf));
-	/* Draw the state of all the supplies. */
-	x += poscolw;
-	if (side_sees_unit(dside, unit)) {
-		for_all_material_types(m) {
-			MoveTo(x, texty);
-			sprintf(spbuf, "%d", unit->supply[m]);
-			DrawText(spbuf, 0, strlen(spbuf));
-			x += supcolw;
-		}
-	} else {
-		x += nummtypes * supcolw;
-	}
 	/* Draw the task. */
+	x += poscolw;
 	MoveTo(x, texty);
 	if (unit->plan && unit->plan->tasks) {
 		task_desc(spbuf, unit->side, unit, unit->plan->tasks);
 	} else {
 		strcpy(spbuf, "  -  ");
 	}
+	spbuf[25] = '\0';
 	DrawText(spbuf, 0, strlen(spbuf));
-	/* Draw extra notes. */
+	/* Draw the state of all the supplies. */
 	x += taskcolw;
+	if (side_sees_unit(dside, unit)) {
+		for_all_material_types(m) {
+			if (sup_column_needed[m]) {
+				MoveTo(x, texty);
+				sprintf(spbuf, "%d", unit->supply[m]);
+				DrawText(spbuf, 0, strlen(spbuf));
+				x += supcolw;
+			}
+		}
+	} else {
+		x += nummtypes * supcolw;
+	}
+	/* Draw extra notes. */
 	MoveTo(x, texty);
 	spbuf[0] = '\0';
 	if (side_sees_unit(dside, unit)
@@ -508,21 +617,17 @@ draw_unit_list_entry(List *list, int n, int clearfirst)
 		MoveTo(x, texty);
 		DrawText(spbuf, 0, strlen(spbuf));
 	}
-	/* Fix the clipping. */
-	cliprect = list->window->portRect;
-	cliprect.right -= sbarwid;
+	ForeColor(blackColor);
+	/* Adjust clipping for vertical line. */
+	cliprect = winrect;
+	cliprect.right = fixedfieldwidth;
+	cliprect.bottom -= sbarwid;
 	ClipRect(&cliprect);
-
 	/* Draw a dividing line between the fixed and mobile fields. */
 	/* It may seem excessive to draw the whole line for each entry but 
 	otherwise the line may end before the bottom of the window content. */
-	MoveTo(fixedfieldwidth - 15, listtoph);
-	LineTo(fixedfieldwidth - 15, cliprect.bottom - sbarwid);
-
-	/* Highlight this entry if it was selected. */
-	if (list->contents->units[n].flag) {
-		InvertRect(&entryrect);
-	}
+	MoveTo(fixedfieldwidth - 5, listtoph);
+	LineTo(fixedfieldwidth - 5, winrect.bottom - sbarwid);
 }
 
 void
@@ -531,8 +636,9 @@ grow_list(List *list, int w, int h)
 	WindowPtr listwin = list->window;
 
 	SizeWindow(listwin, w, h, 1);
+	list->firstvisfield = 0;
 	adjust_list_decor(list);
-	InvalRect(&listwin->portRect);
+	force_update(listwin);
 }
 
 /* Zooming works like a list view in the Finder - it calculates a "perfect" size,
@@ -546,11 +652,12 @@ zoom_list(List *list, int part)
 
 	if (part == inZoomOut) {
 		maxh = listtoph + list->numunits * list->entryspacing + sbarwid;
-		set_standard_state(listwin, maxlistwidth, maxh);
+		set_standard_state(listwin, maxlistwidth + sbarwid, maxh);
 	}
-	EraseRect(&listwin->portRect);
 	ZoomWindow(listwin, part, false);
+	list->firstvisfield = 0;
 	adjust_list_decor(list);
+	force_update(listwin);
 }
 
 /* Move and size the scrollbars to reflect the list's window. */
@@ -564,8 +671,8 @@ adjust_list_decor(List *list)
 	h = list->window->portRect.bottom - list->window->portRect.top;
 	MoveControl(list->vscrollbar, w - sbarwid, listtoph - 1);
 	SizeControl(list->vscrollbar, sbarwid + 1, h - listtoph - sbarwid + 2);
-	MoveControl(list->hscrollbar, fixedfieldwidth - 1, h - sbarwid);
-	SizeControl(list->hscrollbar, w - fixedfieldwidth - sbarwid + 2, sbarwid + 1);
+	MoveControl(list->hscrollbar, fixedfieldwidth - 5, h - sbarwid);
+	SizeControl(list->hscrollbar, w - fixedfieldwidth + 5 - sbarwid + 2, sbarwid + 1);
 	set_list_scrollbars(list);
 }
 
@@ -632,13 +739,13 @@ list_hscroll_fn(ControlHandle control, short code)
 	pagesize = curlist->lastvisfield - curlist->firstvisfield;
 	switch (code) {
 		case inPageDown:
-			jump = pagesize - 10;
+			jump = 50;
 			break;
 		case inDownButton:
 			jump = 10;
 			break;
 		case inPageUp:
-			jump = - (pagesize - 10);
+			jump = -50;
 			break;
 		case inUpButton:
 			jump = -10;
@@ -651,39 +758,79 @@ list_hscroll_fn(ControlHandle control, short code)
 	curlist->firstvisfield = curvalue;
 	curlist->lastvisfield = curlist->firstvisfield + pagesize;
 	SetCtlValue(control, curvalue);
-
-#if 1
-
-/* 	New code that fixes horisontal scrolling bug. */
-
-	/* Return to avoid flicker if stuck at the end of the scrollbar. */
-	if (curvalue == oldvalue) return;
-	/* Update everything but the fixed field. */
-	InvalRect(&listwin->portRect);
-	SetRect(&tmprect, 0, 0, fixedfieldwidth, listwin->portRect.bottom);
-	ValidRect(&tmprect);
-	BeginUpdate(listwin);		
-	draw_list(curlist);
-	EndUpdate(listwin);
-
-#else
-
 	/* Scroll the already-drawn bits. */
 	tmprgn = NewRgn();
-	SetRect(&tmprect, 0, listtoph, listwin->portRect.right - sbarwid, listwin->portRect.bottom - sbarwid);
-	ScrollRect(&tmprect, 0, (oldvalue - curvalue) * curlist->entryspacing, tmprgn);
+	SetRect(&tmprect, fixedfieldwidth, 
+				    0, 
+				    listwin->portRect.right - sbarwid, 
+				    listwin->portRect.bottom - sbarwid);
+	ScrollRect(&tmprect, (oldvalue - curvalue), 0, tmprgn);
 	InvalRgn(tmprgn);
-	/* We'll need to redraw the headings line. */
-	draw_list_headings(curlist);
-	/* Do the update now, because we won't get back to the main event loop
-	   until the mouse button is released. */
 	update_window(curlist->window);
 	DisposeRgn(tmprgn);
-
-#endif
-
 }
 
+/* Scroll up as long as up arrow key is pressed. */
+
+void
+auto_scroll_list_up_arrow()
+{
+	unsigned char keyMap[16];
+	short keyDown = TRUE;
+
+	while (keyDown) {
+		list_vscroll_fn(curlist->vscrollbar, inUpButton);
+		GetKeys((GETKEYS_ARG_TYPE) keyMap);
+		keyDown = keyMap[0x7E >> 3] >> (0x7E & 7) & 1;	
+	}
+}
+
+/* Scroll down as long as down key is pressed. */
+
+void
+auto_scroll_list_down_arrow()
+{
+	unsigned char keyMap[16];
+	short keyDown = TRUE;
+
+	while (keyDown) {
+		list_vscroll_fn(curlist->vscrollbar, inDownButton);
+		GetKeys((GETKEYS_ARG_TYPE) keyMap);
+		keyDown = keyMap[0x7D >> 3] >> (0x7D & 7) & 1;
+	}
+}
+
+/* Scroll left as long as left arrow key is pressed */
+
+void
+auto_scroll_list_left_arrow()
+{
+	unsigned char keyMap[16];
+	short keyDown = TRUE;
+
+	while (keyDown) {
+		list_hscroll_fn(curlist->hscrollbar, inUpButton);
+		GetKeys((GETKEYS_ARG_TYPE) keyMap);
+		keyDown = keyMap[0x7B >> 3] >> (0x7B & 7) & 1;
+	}
+}
+
+/* Scroll right as long as right arrow key is pressed. */
+
+void
+auto_scroll_list_right_arrow()
+{
+	unsigned char keyMap[16];
+	short keyDown = TRUE;
+
+	while (keyDown) {
+		list_hscroll_fn(curlist->hscrollbar, inDownButton);
+		GetKeys((GETKEYS_ARG_TYPE) keyMap);
+		keyDown = keyMap[0x7C >> 3] >> (0x7C & 7) & 1;
+	}
+}
+
+
 int
 do_key_down_list(List *list, char key)
 {
@@ -699,16 +846,16 @@ do_key_down_list(List *list, char key)
 			close_window(curlist->window);
 			break;
 		case 0x1F:		/* Down Arrow. */
-			list_vscroll_fn(curlist->vscrollbar, inDownButton);
+			auto_scroll_list_down_arrow();
 			break;
 		case 0x1E:		/* Up Arrow. */
-			list_vscroll_fn(curlist->vscrollbar, inUpButton);
+			auto_scroll_list_up_arrow();
 			break;
 		case 0x1D:		/* Right Arrow. */
-			list_hscroll_fn(curlist->hscrollbar, inDownButton);
+			auto_scroll_list_right_arrow();
 			break;
 		case 0x1C:		/* Left Arrow. */
-			list_hscroll_fn(curlist->hscrollbar, inUpButton);
+			auto_scroll_list_left_arrow();
 			break;
 		case 0x0C:		/* Page Down. */
 			list_vscroll_fn(curlist->vscrollbar, inPageDown);
@@ -725,7 +872,6 @@ do_key_down_list(List *list, char key)
 	return handled;
 }
 
-
 /* Handle a mouse down in the list.  Grafport already set, mouse coords are local. */
 
 /* (mouse downs should select/deselect list elements) */
@@ -737,6 +883,7 @@ do_mouse_down_list(List *list, Point mouse, int mods)
 	short part;
 	int n, tmp;
 	WindowPtr window = list->window;
+	Rect winrect = window->portRect;
 
 	if (list_vscroll_proc == NULL)
 	  list_vscroll_proc = NewControlActionProc(list_vscroll_fn);
@@ -779,14 +926,27 @@ do_mouse_down_list(List *list, Point mouse, int mods)
 			if (between(curactcolw, mouse.h, curactcolw + imagecolw)) {
 				toggle_list_large_icons(list);
 			}
-			/* do others eventually */
-		} else {
+		/* Ignore clicks in the empty box to the left of the bottom scrollbar. */
+		} else if (mouse.v < winrect.bottom - sbarwid) {
 			/* Figure out the selected unit. */
 			n = (mouse.v - listtoph) / list->entryspacing + list->firstvisible;
 			tmp = list->contents->units[n].flag;
 			clear_selections(list);
-			list->contents->units[n].flag = !tmp;
-			redraw_unit_list_entry(list, n);
+			/* Don't draw units below lastvisible. */
+			if (n <= list->lastvisible) {
+				list->contents->units[n].flag = !tmp;
+				redraw_unit_list_entry(list, n);
+				/* Select the unit on frontmap and make it the current unit if possible. */
+				if (frontmap) {
+					Unit *unit = selected_unit_in_list(list);
+
+					select_exactly_one_unit(frontmap, unit);
+					if (side_controls_unit(dside, unit)
+					    && could_be_next_unit(unit)) {
+						frontmap->curunit = unit;
+					}
+				}
+			}
 		}
 	}
 }
@@ -828,8 +988,12 @@ toggle_list_incomplete(List *list)
 void
 toggle_list_large_icons(List *list)
 {
+	int delta = largeentryspacing - smallentryspacing;
+
 	list->largeicons = !list->largeicons;
 	list->entryspacing = (list->largeicons ? largeentryspacing : smallentryspacing);
+	fixedfieldwidth += (list->largeicons ? delta : -delta); 
+	adjust_list_decor(list);
 	force_update(list->window);
 }
 
@@ -850,29 +1014,98 @@ toggle_list_draw_emblems(List *list)
 void
 update_unit_in_lists(Unit *unit)
 {
-	int line;
-	List *list;
-
+	List	*list;
+	int	i, line;
+	 
 	for_all_lists(list) {
-		if ((line = unit_position_in_list(list, unit)) >= 0) {
-			if (between(list->firstvisible, line, list->lastvisible)) {
-				redraw_unit_list_entry(list, line);
+		line = unit_position_in_list(list, unit);
+		/* The unit is not in the list. */
+		if (line < 0) {
+			/* Insert the unit into the list if it should be there. */
+			if (in_play(unit)
+			     && (completed(unit) ? list->completed_units : list->incomplete_units)) {
+				insert_unit_into_list(list, unit);		
 			}
-		} else {
-			/* (should attempt to insert at the correct location) */
-			add_unit_to_list(list, unit);
-			force_update(list->window);
-		}
+		/* The unit is in the list and should stay there. */
+		} else if (in_play(unit)
+			     && (completed(unit) ? list->completed_units : list->incomplete_units)) {
+			/* Check if the unit should be moved within the list. */
+			organize_list_contents(list);
+			/* The unit did not move. Just redraw the entry. */
+			if (unit_position_in_list(list, unit) == line) {
+				/* Only redraw the entry if it is visible. */
+				if (between(list->firstvisible, line, list->lastvisible)) {
+					redraw_unit_list_entry(list, line);
+				}
+			/* The unit moved within the list. Redraw the list. */
+			} else {
+
+#if 0			/* This code still misses some necessary updates ... */
+
+				/* Find the first line from which the list needs to be redrawn. */
+				line = min(line, unit_position_in_list(list, unit));
+				line = max(line, list->firstvisible);
+				/* Only redraw those entries that are visible. */
+				if (line <= list->lastvisible) {
+					for (i = line; i <= list->lastvisible; ++i)
+						redraw_unit_list_entry(list, i);
+				}
+
+#else		/* ... so we use brute force for the time being. */
+
+				force_update(list->window);
+#endif
+
+			}
+		/* Else remove the unit from the list. */
+		} else remove_unit_from_list(list, unit);
 	}
 }
 
+void
+insert_unit_into_list(List *list, Unit *unit)
+{
+	int line, i;
+
+	/* We assume that the given unit is the only one that is added to
+	or removed from the list. */
+	organize_list_contents(list);
+	/* Find the new unit's position. */
+	line = unit_position_in_list(list, unit);
+	/* Update all visible lines below the new unit's position. */
+	line = max(line, list->firstvisible);
+	if (line <= list->lastvisible) {
+		for (i = line; i <= list->lastvisible; ++i)
+			redraw_unit_list_entry(list, i);
+	} 
+}
+
+void
+remove_unit_from_list(List *list, Unit *unit)
+{
+	int line, i;
+
+	/* Save the unit's old position. */
+	line = unit_position_in_list(list, unit);
+	/* We assume that the given unit is the only one that is added to
+	or removed from the list. */
+	organize_list_contents(list);
+	/* Update all visible lines below the unit's old position. */
+	line = max(line, list->firstvisible);
+	if (line <= list->lastvisible) {
+		for (i = line; i <= list->lastvisible; ++i)
+			redraw_unit_list_entry(list, i);
+	} 
+}
+
 int
 unit_position_in_list(List *list, Unit *unit)
 {
 	int i;
 	
 	for (i = 0; i < list->numunits; ++i) {
-		if (unit == list->contents->units[i].unit) return i;
+		if (unit == unit_in_vector(list->contents, i))
+		  return i;
 	}
 	return (-1);
 }
@@ -888,21 +1121,17 @@ void
 redraw_unit_list_entry(List *list, int n)
 {
 	WindowPtr listwin;
-	Rect cliprect;
-	GrafPtr oldport;
 	RgnHandle tmprgn;
+	GrafPtr oldport;
+	Unit *unit;
 
-	if (!active_display(dside) || list == NULL) return;
-
+	if (!active_display(dside) || list == NULL)
+	    return;
 	listwin = list->window;
  	GetPort(&oldport);
 	SetPort(listwin);
 	tmprgn = NewRgn();
 	GetClip(tmprgn);
-	/* Set up clipping for the contents of the list. */
-	cliprect = listwin->portRect;
-	cliprect.right -= sbarwid;
-	ClipRect(&cliprect);
 	draw_unit_list_entry(list, n, TRUE);
 	SetClip(tmprgn);
 	DisposeRgn(tmprgn);
@@ -928,7 +1157,8 @@ selected_unit_in_list(List *list)
 	int i;
 	
 	for (i = 0; i < list->numunits; ++i) {
-		if (list->contents->units[i].flag) return list->contents->units[i].unit;
+		if (list->contents->units[i].flag)
+		  return unit_in_vector(list->contents, i);
 	}
 	return NULL;
 }
@@ -947,7 +1177,7 @@ scroll_to_selected_unit_in_list(List *list)
 	}
 	unit = selected_unit_in_list(list);
 	if (unit != NULL && inside_area(unit->x, unit->y))
-	  scroll_best_map_to_unit(unit, TRUE);
+	  scroll_best_map_to_unit(unit, FALSE);
 }
 
 void
