@@ -21,15 +21,26 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 #include "macconq.h"
 
-static void draw_feature_names(Map *map, Rect drawrect);
-static void update_grid_leakage(Map *map);
-static void adjust_meridian_interval(Map *map);
-static void draw_meridians(Map *map, Rect drawrect);
-static void draw_info_text(Map *map, int x, int y, int len, char *buf);
-static void draw_unit_names_only(Map *map, int x, int y);
+static void xform_flat(Map *map, int x, int y, int *sxp, int *syp);
+static void xform_fractional(Map *map, int x, int y, int xf, int yf, int *sxp, int *syp);
+static void xform_fractional_flat(Map *map, int x, int y, int xf, int yf, int *sxp, int *syp);
+static int at_all_visible(Map *map, int x, int y);
+static void m_focus_on_center(Map *map);
 
-extern void draw_gridlines(Map *map, int x, int y);
-extern void draw_shorelines(Map *map, int x, int y);
+static void adjust_meridian_interval(Map *map);
+static Map *front_map(void);
+static void update_grid_leakage(Map *map);
+
+static void draw_window_background(Map *map);
+static void draw_map_content(Map *map);
+static void draw_feature_names(Map *map, Rect drawrect);
+static void draw_meridians(Map *map, Rect drawrect);
+static void draw_area_background(Map *map);
+static void draw_info_text(Map *map, int x, int y, int len, char *buf);
+static void draw_other_maps(Map *map);
+static void draw_other_map(Map *map, Map *map2);
+static void draw_selected_unit(Map *map, Unit *unit);
+static void draw_unit_names_only(Map *map, int x, int y);
 
 Map *frontmap = NULL;
 
@@ -263,7 +274,7 @@ m_center_on_focus(Map *map)
 Map *
 create_map(int power, int id)
 {
-	int m, t, i, x, y, mainwidth, mainheight;
+	int m, t, x, y, mainwidth, mainheight;
 	Rect vscrollrect, hscrollrect;
 	WindowPtr win;
 	Map *map;
@@ -299,14 +310,8 @@ create_map(int power, int id)
 	map->vp->wide_viewport = TRUE;
 	map->vp->draw_terrain = TRUE;
 	/* Display all types of borders and connections normally. */
-	/* (aux_terrain_types[t] == 0 implies not optional type - i can't overflow
-	   drawauxterrain because at least one ttype must be cell terrain.) */
-	/* (Addressing of map->vp->aux_terrain_types is 1-based instead of 0-based
-	    because it corresponds to menu items.) */
-	i = 1;
 	for_all_terrain_types(t) {
 		if (!t_is_cell(t)) {
-			map->vp->aux_terrain_types[i++] = t;
 			map->vp->draw_aux_terrain[t] = TRUE;
 		}
 	}
@@ -330,10 +335,8 @@ create_map(int power, int id)
 	/* Display AI info if we have one and draw_ai is on by default. */
 	map->vp->draw_ai = default_draw_ai && side_has_ai(dside);
 	map->draw_emblems = default_draw_emblems;
-	/* Always start with side colors in the world map. */
-	if (map == worldmap)
-		map->sidecolors = TRUE;
-	else	map->sidecolors = default_sidecolors;
+	map->draw_transitions = !pref_solid_color_terrain;
+	map->sidecolors = default_sidecolors;
 	map->iconmasks = default_iconmasks;
 	map->boxmasks = default_boxmasks;
 	map->textmasks = default_textmasks;
@@ -583,7 +586,7 @@ adjust_meridian_interval(Map *map)
 	if (world.circumference <= 0)
 		return;
 	/* First set the interval so that it corresponds to 64 pixels. */
-	map->vp->meridian_interval = 64 * 21600 / (world.circumference * map->vp->hw);
+	map->vp->meridian_interval = 128 * 21600 / (world.circumference * map->vp->hw);
 	/* Then adjust it upwards to an even number of degrees or minutes. */
 	if (map->vp->meridian_interval <= 10)
 	      	map->vp->meridian_interval = 10;
@@ -637,9 +640,13 @@ set_map_angle(Map *map, int angle)
 	/* Erase other-map boxes in other windows. */
 	draw_related_maps(map);
 
+#if 0 	/* No longer works. Should figure out why. */
+ 
 	/* Compute a suitable vertscale value. */
 	map->vp->vertscale = max(500, area.maxelev) / 100 ;
 
+#endif
+
 	/* Change the view angle. */
 	set_view_angle(map->vp, angle);
 
@@ -1886,6 +1893,13 @@ draw_unit_blast(Map *map, Unit *unit, int blast)
 	/* Find offscreen coordinates of the unit rect */
 	m_xform_unit(map, unit, &sx, &sy, &sw, &sh);
 
+	/* Make sure the blast stays within the cell. */
+	if (sw > map->vp->uw) {
+		sw = map->vp->uw;
+		sh = map->vp->uh;
+		sx += (map->vp->hw - map->vp->uw) / 2;
+		sy += (map->vp->hw - map->vp->uw) / 2;
+	}
 	draw_blast_image(map->window, sx, sy, sw, sh, blast);
 
 	RETURN_ONSCREEN(map);
@@ -2011,9 +2025,10 @@ Unit *unit;
 
 	if (!in_play(unit))
 	  return; /* unselect it too? */
-	if (map->vp->uw >= 32) {
+	if (map->vp->uw > 16
+	    || g_bigicons() == TRUE) {
 
-		/* Make sure the image si drawn in smaller self box */
+		/* Make sure the image is drawn in smaller self box */
 		m_xform_unit_self(map, unit, &sx, &sy, &sw, &sh);
 
 		/* This is to make sure occs are drawn exactly as in draw_unit_occss where
@@ -2029,9 +2044,9 @@ Unit *unit;
 		/* Draw unit size on top of advanced unit. */
 		if (u_advanced(unit->type) && map->drawsizes) { 
 			UNWRAP_SX(map, sx, 
-				draw_unit_size(unit, map->window, sx, sy, sw, sh));
+				draw_unit_size(unit, map->window, sx, sy, sw, map->vp->uh));
 		}
-		/* Then use bigger box enclosing also occs as for selection box. */
+		/* Then use bigger box enclosing also occs as for size and selection box. */
 		m_xform_unit(map, unit, &sx, &sy, &sw, &sh);
 		
 		/* Also use the box enclosing transport + occs for selected occ, NOT the whole cell */
@@ -2373,14 +2388,15 @@ update_cell(Map *map, int x, int y)
 	/* Redraw partially erased shorelines for cells below the row. */				
 	if (map->shorelines) {
 		for (i = x; i < x + cells + 1; i++)
-			draw_shorelines(map, i, y - 1);
+			draw_shores(map, i, y - 1);
 	}
 	/* Redraw any partially erased names of units just W of (x, y). */
 	/* Note: this is done even if erase_names is not on! */
-	if (map->vp->draw_names && map->vp->power > 2) {
+	if (map->vp->draw_names 
+	      && map->vp->power > 2) {
 		for (i = x - namelength + 1; i <  x; i++) {
 			for_all_stack(wrapx(i), y, unit) {
-				if (unit->name)	{							
+				if (unit->name) {							
 					draw_unit_names_only(map, i, y);
 					/* Only do this once for each cell. */
 					break;
@@ -2468,51 +2484,26 @@ draw_unit_names_only(Map *map, int x, int y)
 	  return;
 	xform(map, x, y, &sx, &sy);
 
-	/* Draws unit names at small mags where contents of grouping boxes is not shown */
-	if (map->vp->uw <= 16) {
+	/* Draws unit names at small mags where contents of grouping boxes is not shown. */
+	if (map->vp->uw <= 16 
+	      && g_bigicons() != TRUE) {
 		/* Adjust to unit part of cell. */
-		sw = map->vp->uw;  sh = map->vp->uh;
-		sx += (map->vp->hw - sw) / 2; sy += (map->vp->hh - sw) / 2;
+		sw = map->vp->uw;  
+		sh = map->vp->uh;
+		sx += (map->vp->hw - sw) / 2; 
+		sy += (map->vp->hh - sw) / 2;
 		UNWRAP_SX(map, sx,
 			draw_unit_name(unit, sx, sy, sw, sh, 
 						 map->textmasks, map->optimize_fonts));
+	/* Draw unit names at medium and high mags. */
 	} else {
-		/* Draw unit names at medium and high mags. */
 		for_all_stack(xw, y, unit) {
-			m_xform_unit(map, unit, &sx, &sy, &sw, &sh);
-
-			/* Draws the name of a boxed carrier with visible occupants */
-			if (unit->occupant
-				&& sw > 8
-				&& (dside->see_all
-					|| unit->side == dside
-					|| u_see_occupants(unit->type)
-					|| side_owns_occupant(dside, unit))) {
-				/* Transform carrier to the UL quarter of the box */
-				m_xform_occupant(map, unit, unit, sx, sy, sw, sh, &sx2, &sy2, &sw2, &sh2);
-				/* Always draw city and town names at uniform size */
-				if (!mobile(unit->type)) {
-				  UNWRAP_SX(map, sx,
-					draw_unit_name(unit, sx, sy, map->vp->uw, map->vp->uh, 
-				  				 map->textmasks, map->optimize_fonts));  
-				} else {
-				  /* sw2 doubled moves the name out of the box so that it can be read more easily */
-				  UNWRAP_SX(map, sx2,
-				  	draw_unit_name(unit, sx2, sy2, 2 * sw2, sh2,
-				  				  map->textmasks, map->optimize_fonts)); 
-				}
-			} else {
-				/* Draw the names of all other units. */
-				/* Always draw city and town names at uniform size */
-				if (!mobile(unit->type)) {
-				  UNWRAP_SX(map, sx,
-				  	draw_unit_name(unit, sx, sy, map->vp->uw, map->vp->uh, 
-				  				 map->textmasks, map->optimize_fonts));  
-				} else {
-				  UNWRAP_SX(map, sx,
-				  	draw_unit_name(unit, sx, sy, sw, sh, 
-				  				 map->textmasks, map->optimize_fonts));
-				}
+			/* Don't draw names of occupants. */
+			if (!unit->transport) {
+				m_xform_unit(map, unit, &sx, &sy, &sw, &sh);
+				UNWRAP_SX(map, sx, 
+					draw_unit_name(unit, sx, sy, sw, sh, 
+								 map->textmasks, map->optimize_fonts));
 			}
 		}
 	}
