@@ -7,7 +7,7 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-#include "ai.h"
+extern short *aux_terrain_type_index;
 #include "print.h"
 #include "macconq.h"
 
@@ -27,25 +27,13 @@ any later version.  See the file COPYING.  */
 
 #define MAXWINDOWS 100
 
-extern void enable_commands_for_unit(MenuHandle menu, MenuHandle menu2, Unit *unit);
-extern void side_research_dialog(Side *side, Unit *unit, int advance);
-extern void do_closeup_mi(void);				/* Needed in macmouse.c. */			
-
-extern int do_down_command(void);
-extern int do_up_command(void);
-extern int num_sides_allowed PARAMS ((int u));
-extern int do_one_give(Unit *unit);
-extern int do_one_take(Unit *unit);
-extern int do_one_detonate(Unit *unit);
-extern int do_one_give_unit(Unit *unit);
-extern int do_one_dir_move(Unit *unit);
-extern int do_one_dir_multiple_move(Unit *unit);
-extern int do_one_set_name(Unit *unit);
-
+static void build_ai_type_menu(void);
+static void maybe_init_print(void);
 static void resign_the_game(int forced);
 static void te_cut(TEHandle hdl);
 static void te_copy(TEHandle hdl);
 static void te_paste(TEHandle hdl);
+
 static void do_page_setup_mi(void);
 static void do_print_mi(void);
 static void do_select_all_mi(void);
@@ -53,24 +41,59 @@ static void do_find_previous_mi(void);
 static void do_find_next_mi(void);
 static void do_find_location_mi(void);
 static void do_find_unit_by_name_mi(void);
-static void do_find_selected_mi(void);
-static void do_overrun_mi(void);
-static void do_take_mi(void);
-static void do_detonate_mi(void);
-static void do_give_unit_mi(int mi);
-static void do_plan_type_mi(void);
-static void moveonclick_mi(void);
-static void autoselect_mi(void);
-static void adjust_menu_item(MenuHandle menu, int item, int value);
-
-static int do_one_plan_type(Unit *unit);
-static int worldmap_dialog(void);
 static int find_location_dialog(int *xp, int *yp);
+static int worldmap_dialog(void);
 static int find_unit_dialog(Unit **unitp);
 static int find_something(Side *side, char *str, int findunit, int *xp, int *yp, Unit **unitp);
+static void do_find_selected_mi(void);
+static void do_closeup_mi(void);	
+static void do_overrun_mi(void);
+static void moveonclick_mi(void);
+static void do_plan_type_mi(void);
+static int do_one_plan_type(Unit *unit);
+static void do_take_mi(void);
+static void autoselect_mi(void);
+static void do_detonate_mi(void);
+static void do_give_unit_mi(int mi);
 
-/* This permits more AIs to be dynamically added to the menu. */
-extern AI_ops **all_ai_ops;
+static void adjust_menu_item(MenuHandle menu, int item, int value);
+static void enable_commands_for_unit(MenuHandle menu, MenuHandle menu2, Unit *unit);
+
+static void set_offscreen_buffers(Map *map);
+static void set_meridians(Map *map);
+static void seed_side_rename_string(DialogPtr win, short ditem, char *str);
+static char *get_side_rename_string(DialogPtr win, short ditem);
+static void side_rename_dialog(Side *side);
+
+static void toggle_map_topline(Map *map);
+static void toggle_map_topunit(Map *map);
+static void toggle_map_scrollbars(Map *map);
+static void toggle_map_cpanel(Map *map);
+static void toggle_map_other_maps(Map *map);
+static void toggle_map_lighting(Map *map);
+static void toggle_map_coverage(Map *map);
+static void toggle_map_names(Map *map);
+static void toggle_map_elevations(Map *map);
+static void toggle_map_materials(Map *map, int m);
+static void toggle_map_aux_terrain(Map *map, int t);
+static void toggle_map_temperature(Map *map);
+static void toggle_map_winds(Map *map);
+static void toggle_map_clouds(Map *map);
+static void toggle_map_storms(Map *map);
+static void toggle_map_draw_emblems(Map *map);
+static void toggle_blinking_curunit(Map *map);
+static void toggle_selectionmasks(Map *map);
+static void toggle_iconmasks(Map *map);
+static void toggle_boxmasks(Map *map);
+static void toggle_textmasks(Map *map);
+static void toggle_featureborders(Map *map);
+static void toggle_featurenames(Map *map);
+static void toggle_shorelines(Map *map);
+static void toggle_simple_borders(Map *map);
+static void toggle_optimize_fonts(Map *map);
+static void toggle_erase_names(Map *map);
+static void toggle_draw_meridians(Map *map);
+static void toggle_solid_color_terrain(Map *map);
 
 Side *tmpcmdside;
 
@@ -80,8 +103,6 @@ int tmp_plan_type;
 
 int forcedtoresign;
 
-int modal_construction = FALSE;
-
 int Profile = FALSE;
 
 int feature_menu_size;
@@ -105,8 +126,6 @@ MenuHandle optterrmenu = nil;
 /* This is the array of windows that the Windows menu references. */
 WindowPtr *winmenuwins;
 
-WindowPtr window_behind_construction;
-
 WindowPtr worldmapwin = nil;
 
 Map *worldmap = NULL;
@@ -380,6 +399,8 @@ build_terrain_type_menu()
 void
 build_ai_type_menu()
 {
+	char *aitypename;
+
 	/* Permit AIs to be dynamically added to the menu. */
 	if (aitypemenu != nil && CountMItems(aitypemenu) < 1) {
 
@@ -388,10 +409,14 @@ build_ai_type_menu()
 
 		AppendMenu(aitypemenu, "\pNone");
 		EnableItem(aitypemenu, 1);
-		for (i = 1; i < numaitypes; i++) {
-			c2p(all_ai_ops[i]->name, pname);
+		aitypename = next_ai_type_name(NULL);
+		i = 1;
+		while (aitypename != NULL) {
+			c2p(aitypename, pname);
 			AppendMenu(aitypemenu, pname);
 			EnableItem(aitypemenu, i + 1);
+			aitypename = next_ai_type_name(aitypename);
+			++i;
 		}
 	}
 }
@@ -1052,7 +1077,7 @@ do_menu_command(long which)
 					net_set_side_ai(dside, NULL);
 					break;
 				default:
-					net_set_side_ai(dside, all_ai_ops[menuitem - 1]->name);
+					net_set_side_ai(dside, ai_type_name(menuitem - 1));
 					break;
 			}
 			break;
@@ -1066,7 +1091,7 @@ do_menu_command(long which)
 			break;
 		case mOptTerrainTypes:
 			if (map == NULL) break;
-			toggle_map_aux_terrain(map, map->vp->aux_terrain_types[menuitem]);
+			toggle_map_aux_terrain(map, aux_terrain_type_index[menuitem - 1]);
 			break;
 		case mPlanTypes:
 			tmp_plan_type = PLAN_NONE;
@@ -1679,41 +1704,6 @@ do_closeup_mi()
 	}
 }
 
-int
-do_down_command()
-{
-	int i, numcould = 0, numnot = 0, rawamt = 0;
-	int amt, newalt;
-	Point target;
-	Unit *unit;
-
-	rawamt = 0;	
-	if (frontmap) {
-		for (i = 0; i < frontmap->numselections; ++i) {
-			if ((unit = frontmap->selections[i]) != NULL 
-			       && valid_selection(unit)) {
-				amt = rawamt;
-				if (amt == 0 && unit_alt(unit) > 0)
-				  amt = unit_alt(unit) / 2;
-				newalt = (unit_alt(unit) - amt) << 1;
-				if (valid(check_move_action(unit, unit, unit->x, unit->y, newalt))) {
-					net_prep_move_action(unit, unit, unit->x, unit->y, newalt);
-					++numcould;
-				} else {
-					++numnot;
-				}
-			}
-		}
-	}
-	/* If nobody could do the action, beep once. */
-	if (numcould == 0 && numnot > 0) {
-		beep();
-		return FALSE;
-	} else {
-		return TRUE;
-	}
-}
-
 static void
 do_overrun_mi()
 {
@@ -1724,7 +1714,7 @@ do_overrun_mi()
 static void
 do_detonate_mi()
 {
-	apply_to_all_selected(do_one_detonate, TRUE);
+	do_detonate(dside);
 }
 
 static void
@@ -1772,39 +1762,7 @@ do_one_plan_type(Unit *unit)
 static void
 do_take_mi()
 {
-	apply_to_all_selected(do_one_take, TRUE);
-}
-
-int
-do_up_command()
-{
-	int i, numcould = 0, numnot = 0, rawamt = 1;
-	int amt, newalt;
-	Point target;
-	Unit *unit;
-
-	if (frontmap) {
-		for (i = 0; i < frontmap->numselections; ++i) {
-			if ((unit = frontmap->selections[i]) != NULL 
-			       && valid_selection(unit)) {
-				amt = rawamt;
-				newalt = (unit_alt(unit) + amt) << 1;
-				if (valid(check_move_action(unit, unit, unit->x, unit->y, newalt))) {
-					net_prep_move_action(unit, unit, unit->x, unit->y, newalt);
-					++numcould;
-				} else {
-					++numnot;
-				}
-			}
-		}
-	}
-	/* If nobody could do the action, beep once. */
-	if (numcould == 0 && numnot > 0) {
-		beep();
-		return FALSE;
-	} else {
-		return TRUE;
-	}
+	do_take(dside);
 }
 
 static void
@@ -1869,13 +1827,6 @@ adjust_menus()
 		build_side_menu();
 		build_ai_type_menu();
 	}
-#if 0
-	if (is_da_window(topWindow)) {
-		if ((menu = GetMHandle(mEdit)) != nil) {
-		}
-		return;
-	}
-#endif
 	menu = GetMHandle(mApple);
 	if (menu != nil) {
 		adjust_menu_item(menu, miAppleInstructions, gamedefined);
@@ -1885,7 +1836,11 @@ adjust_menus()
 		/* Availability of file menu items depends on whether we're in a game. */
 		adjust_menu_item(menu, miFileNew, !gamedefined);
 		adjust_menu_item(menu, miFileOpen, !gamedefined);
-		adjust_menu_item(menu, miFileConnect, !gamedefined);
+
+		/* Always disable defunct connect item for now. */
+		adjust_menu_item(menu, miFileConnect, FALSE);
+/*		adjust_menu_item(menu, miFileConnect, !gamedefined);		*/
+
 		adjust_menu_item(menu, miFileClose, (win != NULL));
 		adjust_menu_item(menu, miFileSave, menus_useful);
 		adjust_menu_item(menu, miFileSaveAs, menus_useful);
@@ -2046,7 +2001,7 @@ adjust_menus()
 				if (g_side_can_research()) {
 					EnableItem(menu, miWindowsResearch);
 				} else for_all_side_units(dside, unit) {
-				if (u_advanced(unit->type)) {
+				if (u_can_research(unit->type)) {
 					EnableItem(menu, miWindowsResearch);
 					break;
 				}
@@ -2275,7 +2230,7 @@ adjust_menus()
 			if (map != NULL) {
 				numitems = CountMItems(menu);
 				for (i = 1; i <= numitems; ++i) {
-					t = map->vp->aux_terrain_types[i];
+					t = aux_terrain_type_index[i - 1];
 					if (any_aux_terrain_defined() && aux_terrain_defined(t)) {
 						EnableItem(menu, i);
 					    CheckItem(menu, i, map->vp->draw_aux_terrain[t]);
@@ -2294,8 +2249,9 @@ adjust_menus()
 		/* Always on. */
 		EnableItem(menu, 0);
 		CheckItem(menu, 1, (!side_has_ai(dside)));
-		for (i = 1; i < numaitypes; i++)
-			CheckItem(menu, i + 1, side_has_ai(dside) && dside->aitype == i);
+		numitems = CountMItems(menu);
+		for (i = 2; i <= numitems; i++)
+			CheckItem(menu, i, side_has_ai(dside) && dside->aitype == (i - 1));
 	}
 	menu = GetMHandle(mViewText);
 	if (menu != nil && map != NULL) {
@@ -2374,7 +2330,7 @@ adjust_menus()
 /* Tweak the command menu according to what the unit can do. */
 /* (some of these should have the menu text tweaked also, terms weird sometimes) */
 
-int any_transport_here PARAMS ((Side *side, Unit *unit, int x, int y));
+int any_transport_here(Side *side, Unit *unit, int x, int y);
 
 int
 any_transport_here(side, unit, x, y)
@@ -2455,8 +2411,7 @@ enable_commands_for_unit(MenuHandle menu, MenuHandle menu2, Unit *unit)
 	if ((can_create(unit) || can_complete(unit))
 		&& (unit->transport == NULL
 			|| !completed(unit->transport)
-			|| uu_occ_can_build(unit->transport->type, u))
-		|| can_develop(unit)) {
+			|| uu_occ_can_build(unit->transport->type, u))) {
 		EnableItem(menu, miPlayBuild);
 	}
 	if (can_repair(unit)) {
@@ -2514,7 +2469,7 @@ enable_commands_for_unit(MenuHandle menu, MenuHandle menu2, Unit *unit)
 	if (menu3 != nil) {
 		if (num_sides_allowed(u) > 1 || is_designer(dside)) {
 			EnableItem(menu3, 0);
-			for_all_sides_plus_indep(side2) {
+			for_all_sides(side2) {
 				if (unit_allowed_on_side(unit, side2)
 					&& u_acp_to_change_side(unit->type)
 					&& unit->side != side2) {
@@ -2530,3 +2485,579 @@ enable_commands_for_unit(MenuHandle menu, MenuHandle menu2, Unit *unit)
 	}
 }
 
+/* Dialog that sets the size of the offscreen scrolling buffers. */
+
+void
+set_offscreen_buffers(Map *map)
+{
+	short ditem, itemtype, done = FALSE;
+	long *itemnum = 0;
+	Handle itemhandle;  
+	Rect itemrect;
+	Str255 tmpstr;
+	DialogPtr win;
+	GrafPtr oldport;
+
+	win = GetNewDialog(dOffscreen, NULL, (DialogPtr) -1);
+	GetPort(&oldport);	
+	SetPort(win);
+
+	GetDItem(win, diOffscreenHorizBuff, &itemtype, &itemhandle, &itemrect);
+	NumToString(map->max_bufx, tmpstr);
+	SetIText(itemhandle, tmpstr);
+
+	GetDItem(win, diOffscreenVertiBuff, &itemtype, &itemhandle, &itemrect);
+	NumToString(map->max_bufy, tmpstr);
+	SetIText(itemhandle, tmpstr);
+		
+	ShowWindow(win);
+	while (!done) {
+
+		SetCursor(&QD(arrow));
+		draw_default_button(win, diOffscreenOK);
+		ModalDialog(NULL, &ditem);
+		switch (ditem) {
+
+			case diOffscreenOK:
+
+				GetDItem(win, diOffscreenHorizBuff, &itemtype, &itemhandle, &itemrect);
+	        			GetIText(itemhandle,tmpstr);
+				StringToNum(tmpstr, itemnum);
+				/* Dont allow negative values */
+				map->max_bufx = max(*itemnum, 0);
+
+				GetDItem(win, diOffscreenVertiBuff, &itemtype, &itemhandle, &itemrect);
+	         			GetIText(itemhandle,tmpstr);
+				StringToNum(tmpstr, itemnum);
+				/* Dont allow negative values */
+				map->max_bufy = max(*itemnum, 0);
+				
+				/* Redraw the gworld (and the map) */
+				force_map_update(map);
+				done = TRUE;
+				break;
+
+			case diOffscreenCancel:
+
+				done = TRUE;
+				break;
+
+			default:
+				break;
+		}
+	}
+	DisposeDialog(win);
+	SetPort(oldport);
+	update_all_map_windows();
+}
+
+/* Dialog that sets the size of the meridian interval for a map. */
+
+void
+set_meridians(Map *map)
+{
+	short ditem, itemtype, done = FALSE;
+	long *itemnum = 0;
+	Handle itemhandle;  
+	Rect itemrect;
+	Str255 tmpstr;
+	DialogPtr win;
+	GrafPtr oldport;
+
+	win = GetNewDialog(dMeridians, NULL, (DialogPtr) -1);
+	GetPort(&oldport);	
+	SetPort(win);
+
+	GetDItem(win, diMeridiansOnOff, &itemtype, &itemhandle, &itemrect);
+	SetCtlValue((ControlHandle) itemhandle, map->vp->draw_meridians);
+
+	GetDItem(win, diMeridiansInterval, &itemtype, &itemhandle, &itemrect);
+	NumToString(map->vp->meridian_interval, tmpstr);
+	SetIText(itemhandle, tmpstr);
+
+	ShowWindow(win);
+	while (!done) {
+
+		SetCursor(&QD(arrow));
+		draw_default_button(win, diMeridiansOK);
+		ModalDialog(NULL, &ditem);
+		switch (ditem) {
+
+			case diMeridiansOnOff:
+				GetDItem(win, ditem, &itemtype, &itemhandle, &itemrect);
+				SetCtlValue((ControlHandle) itemhandle, !GetCtlValue((ControlHandle) itemhandle));
+				break;
+
+			case diMeridiansOK:
+				GetDItem(win, diMeridiansOnOff, &itemtype, &itemhandle, &itemrect);
+				map->vp->draw_meridians = GetCtlValue((ControlHandle) itemhandle);
+				GetDItem(win, diMeridiansInterval, &itemtype, &itemhandle, &itemrect);
+	        			GetIText(itemhandle,tmpstr);
+				StringToNum(tmpstr, itemnum);
+				/* Dont allow negative values */
+				map->vp->meridian_interval = max(*itemnum, 0);
+
+				/* Redraw the gworld (and the map) */
+				force_map_update(map);
+				done = TRUE;
+				break;
+
+			case diMeridiansCancel:
+
+				done = TRUE;
+				break;
+
+			default:
+				break;
+		}
+	}
+	DisposeDialog(win);
+	SetPort(oldport);
+	update_all_map_windows();
+}
+
+/* The side renaming dialog includes places for all the different name-related
+   properties of a side. */
+
+static void
+seed_side_rename_string(DialogPtr win, short ditem, char *str)
+{
+	Str255 tmpstr;
+	short itemtype;  Handle itemhandle;  Rect itemrect;
+
+	GetDItem(win, ditem, &itemtype, &itemhandle, &itemrect);
+	c2p((str ? str : ""), tmpstr);
+	SetIText(itemhandle, tmpstr);
+}
+
+static char *
+get_side_rename_string(DialogPtr win, short ditem)
+{
+	char *str;
+	short itemtype;  Handle itemhandle;  Rect itemrect;
+
+	GetDItem(win, ditem, &itemtype, &itemhandle, &itemrect);
+	str = get_string_from_item(itemhandle);
+	if (empty_string(str))
+	  str = NULL;
+	return str;
+}
+
+void
+side_rename_dialog(Side *side)
+{
+	short done = FALSE, changed = TRUE, ditem;
+	char *str;
+	DialogPtr win;
+
+	win = GetNewDialog(dSideRename, NULL, (DialogPtr) -1L);
+	while (!done) {
+		if (changed) {
+			/* Seed the items with the current side names. */
+			seed_side_rename_string(win, diSideRenameName, side->name);
+			seed_side_rename_string(win, diSideRenameFullName, side->longname);
+			seed_side_rename_string(win, diSideRenameAcronym, side->shortname);
+			seed_side_rename_string(win, diSideRenameNoun, side->noun);
+			seed_side_rename_string(win, diSideRenamePluralNoun, side->pluralnoun);
+			seed_side_rename_string(win, diSideRenameAdjective, side->adjective);
+			seed_side_rename_string(win, diSideRenameEmblemName, side->emblemname);
+			seed_side_rename_string(win, diSideRenameColorScheme, side->colorscheme);
+			ShowWindow(win);
+			changed = FALSE;
+		}
+		draw_default_button(win, diSideRenameOK);
+		SetCursor(&QD(arrow));
+		ModalDialog(NULL, &ditem);
+		switch (ditem) {
+			case diSideRenameOK:
+				/* Actually change the side's slots. */
+				net_set_side_name(dside, side, get_side_rename_string(win, diSideRenameName));
+				net_set_side_longname(dside, side, get_side_rename_string(win, diSideRenameFullName));
+				net_set_side_shortname(dside, side, get_side_rename_string(win, diSideRenameAcronym));
+				net_set_side_noun(dside, side, get_side_rename_string(win, diSideRenameNoun));
+				net_set_side_pluralnoun(dside, side, get_side_rename_string(win, diSideRenamePluralNoun));
+				net_set_side_adjective(dside, side, get_side_rename_string(win, diSideRenameAdjective));
+				net_set_side_emblemname(dside, side, get_side_rename_string(win, diSideRenameEmblemName));
+				net_set_side_colorscheme(dside, side, get_side_rename_string(win, diSideRenameColorScheme));
+				current_colorscheme[side_number(side)] = copy_string(side->colorscheme);
+				dissect_colorscheme(side_number(side));
+				init_emblem_images();
+				ui_update_state();
+				save_preferences();
+				/* Tweak the side menu. */
+				update_side_menu(dside);
+				/* Force redisplay of everything that might use any side names. */
+				force_overall_update();
+				/* Fall into next case. */
+			case diSideRenameCancel:
+				done = TRUE;
+				break;
+			case diSideRenameRandom:
+				net_set_side_name(dside, side, NULL);
+				net_set_side_noun(dside, side, NULL);
+				/* Always need to clear the plural noun cache before renaming... */
+				net_set_side_pluralnoun(dside, side, NULL);
+				net_set_side_adjective(dside, side, NULL);
+				/* (should not be able to call this here!) */
+				make_up_side_name(side);
+				init_emblem_images();
+				changed = TRUE;
+				break;
+		}
+	}
+	DisposeDialog(win);
+	update_all_map_windows();
+}
+
+
+void
+toggle_map_grid(Map *map)
+{
+	map->vp->draw_grid = !map->vp->draw_grid;
+	/* (should not do a total redraw?) */
+	force_map_update(map);
+}
+
+void
+toggle_map_topline(Map *map)
+{
+	int oldtoph = map->toph;
+	GrafPtr oldport;
+
+	/* Setting the port fixed a display bug. */
+	GetPort(&oldport);
+	SetPort(map->window);
+	map->draw_topline = !map->draw_topline;
+	map->toplineh = (map->draw_topline ? tophgt : 0);
+	/* Erase other-map boxes in other windows. */
+	draw_related_maps(map);
+	map->toph = map->toplineh + map->topunith;
+
+#if 1
+
+	/* This is a temp fix. It should be possible to get the code below to work! */
+	force_map_update(map);
+
+#else
+
+	/* Update without scrolling if we are expanding and the buffer is too small. */ 
+	if (map->bufy - map->offsety < oldtoph - map->toph) {
+		set_content_rect(map);
+		update_gworld(map);
+		/* This is not done by update_gworld (unlike update_resized_gworld). */
+		if (map->toplineh > 0)
+		  draw_top_line(map);
+		if (map->topunith > 0)
+		  draw_unit_info(map);
+	/* Always scroll the map if we are shrinking, or if the buffer is big enough. */
+	} else update_resized_map(map);
+
+#endif
+
+	/* Redraw other-map boxes in other windows. */
+	draw_related_maps(map);
+	SetPort(oldport);
+}
+
+void
+toggle_map_topunit(Map *map)
+{
+	GrafPtr oldport;
+	int oldtoph;
+
+	/* Setting the port fixed a display bug. */
+	GetPort(&oldport);
+	SetPort(map->window);
+	oldtoph = map->toph;
+	map->draw_topunit = !map->draw_topunit;
+	map->topunith = (map->draw_topunit ? topunithgt : 0);
+	/* Erase other-map boxes in other windows. */
+	draw_related_maps(map);
+	map->toph = map->toplineh + map->topunith;
+	/* Update without scrolling if we are expanding and the buffer is too small. */ 
+	if (map->bufy - map->offsety < oldtoph - map->toph) {
+		set_content_rect(map);
+		update_gworld(map);
+		/* This is not done by update_gworld (unlike update_resized_gworld). */
+		if (map->toplineh > 0)
+		  draw_top_line(map);
+		if (map->topunith > 0)
+		  draw_unit_info(map);
+	/* Always scroll the map if we are shrinking, or if the buffer is big enough. */
+	} else update_resized_map(map);
+	/* Redraw other-map boxes in other windows. */
+	draw_related_maps(map);
+	SetPort(oldport);
+}
+
+void
+toggle_map_scrollbars(Map *map)
+{
+	/* Erase other-map boxes in other windows. */
+	draw_related_maps(map);
+
+	if (map->sbarwid)
+		map->sbarwid = 0;
+	else if (map == worldmap)
+		map->sbarwid = floatsbarwid;
+	else	map->sbarwid = sbarwid;
+	
+	adjust_map_decor(map);
+	force_map_update(map);
+
+	/* Redraw other-map boxes in other windows. */
+	draw_related_maps(map);
+}
+
+void
+toggle_map_cpanel(Map *map)
+{
+	/* Erase other-map boxes in other windows. */
+	draw_related_maps(map);
+
+	/* Toggle the panel. */
+	map->conw = (map->conw > 0 ? 0 : conwid);
+	adjust_map_decor(map);
+	force_map_update(map);
+
+	/* Redraw other-map boxes in other windows. */
+	draw_related_maps(map);
+}
+
+void
+toggle_map_other_maps(Map *map)
+{
+	map->vp->draw_other_maps = !map->vp->draw_other_maps;
+	/* (should not do a total redraw) */
+	force_map_update(map);
+}
+
+void
+toggle_map_lighting(Map *map)
+{
+	map->vp->draw_lighting = !map->vp->draw_lighting;
+	/* We have to do a total redraw. */
+	force_map_update(map);
+}
+
+void
+toggle_map_coverage(Map *map)
+{
+	map->vp->draw_cover = !map->vp->draw_cover;
+	/* (should only change newly dimmed/lightened cells) */
+	force_map_update(map);
+}
+
+void
+toggle_map_names(Map *map)
+{
+	map->vp->draw_names = !map->vp->draw_names;
+	/* (if now on, should draw names on top of everything, don't redraw everything) */
+	if (map->vp->hh > 5) {
+		force_map_update(map);
+	} else {
+		/* (should be a force update on control panel alone) */
+		draw_control_panel(map);
+	}
+}
+
+void
+toggle_map_people(Map *map)
+{
+    if (!people_sides_defined())
+      return;
+	map->vp->draw_people = !map->vp->draw_people;
+	if (bwid2[map->vp->power] > 0) {
+		force_map_update(map);
+	} else {
+		/* (should be a force update on control panel alone) */
+		draw_control_panel(map);
+	}
+}
+
+void
+toggle_map_control(Map *map)
+{
+    if (!control_sides_defined())
+      return;
+	map->vp->draw_control = !map->vp->draw_control;
+	if (bwid2[map->vp->power] > 0) {
+		force_map_update(map);
+	} else {
+		/* (should be a force update on control panel alone) */
+		draw_control_panel(map);
+	}
+}
+
+void
+toggle_map_elevations(Map *map)
+{
+	map->vp->draw_elevations = !map->vp->draw_elevations;
+	force_map_update(map);
+}
+
+void
+toggle_map_materials(Map *map, int m)
+{
+	map->vp->draw_materials[m] = !map->vp->draw_materials[m];
+	map->vp->num_materials_to_draw += (map->vp->draw_materials[m] ? 1 : -1);
+	force_map_update(map);
+}
+
+void
+toggle_map_aux_terrain(Map *map, int t)
+{
+	map->vp->draw_aux_terrain[t] = !map->vp->draw_aux_terrain[t];
+	force_map_update(map);
+}
+
+void
+toggle_map_temperature(Map *map)
+{
+	map->vp->draw_temperature = !map->vp->draw_temperature;
+	force_map_update(map);
+}
+
+void
+toggle_map_winds(Map *map)
+{
+	map->vp->draw_winds = !map->vp->draw_winds;
+	force_map_update(map);
+}
+
+void
+toggle_map_clouds(Map *map)
+{
+	map->vp->draw_clouds = !map->vp->draw_clouds;
+	force_map_update(map);
+}
+
+void
+toggle_map_storms(Map *map)
+{
+	map->vp->draw_storms = !map->vp->draw_storms;
+	force_map_update(map);
+}
+
+void
+toggle_map_plans(Map *map)
+{
+	map->vp->draw_plans = !map->vp->draw_plans;
+
+	force_map_update(map);
+}
+
+void
+toggle_map_ai(Map *map)
+{
+	if (!side_has_ai(dside))
+	  return;
+	map->vp->draw_ai = !map->vp->draw_ai;
+	force_map_update(map);
+}
+
+void
+toggle_map_sidecolors(Map *map)
+{
+	map->sidecolors = !map->sidecolors;
+	force_map_update(map);
+}
+
+void
+toggle_map_draw_emblems(Map *map)
+{
+	map->draw_emblems = !map->draw_emblems;
+	force_map_update(map);
+}
+
+void
+toggle_iconmasks(Map *map)
+{
+	map->iconmasks = !map->iconmasks;
+	force_map_update(map);
+}
+
+void
+toggle_boxmasks(Map *map)
+{
+	map->boxmasks = !map->boxmasks;
+	force_map_update(map);
+}
+
+void
+toggle_textmasks(Map *map)
+{
+	map->textmasks = !map->textmasks;
+	force_map_update(map);
+}
+
+void
+toggle_featureborders(Map *map)
+{
+	map->featureborders = !map->featureborders;
+	force_map_update(map);
+}
+
+void
+toggle_featurenames(Map *map)
+{
+	map->featurenames = !map->featurenames;
+	force_map_update(map);
+}
+
+void
+toggle_shorelines(Map *map)
+{
+	map->shorelines = !map->shorelines;
+	force_map_update(map);
+}
+
+void
+toggle_simple_borders(Map *map)
+{
+	map->simple_borders = !map->simple_borders;
+	force_map_update(map);
+}
+
+void
+toggle_optimize_fonts(Map *map)
+{
+	map->optimize_fonts = !map->optimize_fonts;
+	force_map_update(map);
+}
+
+void
+toggle_draw_meridians(Map *map)
+{
+	map->vp->draw_meridians = !map->vp->draw_meridians;
+	force_map_update(map);
+}
+
+void
+toggle_solid_color_terrain(Map *map)
+{
+	map->solid_color_terrain = !map->solid_color_terrain;
+	force_map_update(map);
+}
+
+void
+toggle_blinking_curunit(Map *map)
+{
+	map->blinking_curunit = !map->blinking_curunit;
+	force_map_update(map);
+}
+
+void
+toggle_selectionmasks(Map *map)
+{
+	map->selectionmasks = !map->selectionmasks;
+	force_map_update(map);
+}
+
+void
+toggle_erase_names(Map *map)
+{
+	map->erase_names = !map->erase_names;
+	force_map_update(map);
+}
+
