@@ -12,15 +12,28 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 #include "macconq.h"
 
-extern int side_owns_occupant PARAMS ((Side *side, Unit *unit));
-
-extern void set_position_modally(void);
-extern void do_closeup_mi(void);
-
 static int selrect;
 static int downx, downy, downdir;
+
 static void drag_for_distance(Map *map, int h0, int v0);
 
+static void do_mouse_down_map_control_panel(Map *map, int h, int v, int mods);
+static void do_mouse_down_map_content(Map *map, int h, int v, int mods);
+static void select_all_dragged_over(Map *map, int h0, int v0, int mods);
+static void select_area_and_zoom(Map *map, int h0, int v0, int mods);
+static void move_on_drag(Map *map, Unit *unit, int mods);
+static void select_all_units_in_rect(Map *map, Rect *rectptr);
+static int move_the_selected_unit(Map *map, Unit *unit, int h, int v);
+static void fire_the_selected_unit(Map *map, Unit *unit, int h, int v);
+static void select_next_unit(Map *map);
+static void select_next_actor(Map *map);
+static void select_previous_actor(Map *map);
+static void select_previous_mover(Map *map);
+static void select_next_awake_mover(Map *map);
+static void select_previous_awake_mover(Map *map);
+static void select_another(Map *map, Unit *(*fn)(Side *side, Unit *unit));
+static void magnify_to_fit(Map *map, int x1, int y1, int x2, int y2);
+
 int topunithgt;
 
 Map *curmap;
@@ -107,15 +120,20 @@ scroll_best_map_to_unit(Unit *unit, int bringtofront)
 void
 scroll_to_unit(Map *map, Unit *unit)
 {
-	int oldsx = map->vp->sx, oldsy = map->vp->sy;
-	int	newsx, newsy;
+	int		oldsx = map->vp->sx, oldsy = map->vp->sy;
+	int		usx, usy, newsx, newsy;
+	Point		point;
 	
 	/* Return if unit is not at all on the map */
 	if (!inside_area(unit->x, unit->y))
 	  return;
 
-	/* Dont scroll if the unit already is in the window */
-	if (in_middle(map, unit->x, unit->y))
+	xform(map, unit->x, unit->y, &usx, &usy);
+	point.h = usx + map->vp->hw/2; 
+	point.v = usy + map->vp->hh/2; ;
+	/* Dont scroll if the unit already is in the window and is visible. */
+	if (in_middle(map, unit->x, unit->y)
+	    && PtInRgn(point, map->window->visRgn))
 	  return;
 
 	/* Find new vp */
@@ -373,357 +391,6 @@ set_map_mag(Map *map, int newpower)
 	}
 }
 
-void
-toggle_map_grid(Map *map)
-{
-	map->vp->draw_grid = !map->vp->draw_grid;
-	/* (should not do a total redraw?) */
-	force_map_update(map);
-}
-
-void
-toggle_map_topline(Map *map)
-{
-	int oldtoph = map->toph;
-	GrafPtr oldport;
-
-	/* Setting the port fixed a display bug. */
-	GetPort(&oldport);
-	SetPort(map->window);
-	map->draw_topline = !map->draw_topline;
-	map->toplineh = (map->draw_topline ? tophgt : 0);
-	/* Erase other-map boxes in other windows. */
-	draw_related_maps(map);
-	map->toph = map->toplineh + map->topunith;
-
-#if 1
-
-	/* This is a temp fix. It should be possible to get the code below to work! */
-	force_map_update(map);
-
-#else
-
-	/* Update without scrolling if we are expanding and the buffer is too small. */ 
-	if (map->bufy - map->offsety < oldtoph - map->toph) {
-		set_content_rect(map);
-		update_gworld(map);
-		/* This is not done by update_gworld (unlike update_resized_gworld). */
-		if (map->toplineh > 0)
-		  draw_top_line(map);
-		if (map->topunith > 0)
-		  draw_unit_info(map);
-	/* Always scroll the map if we are shrinking, or if the buffer is big enough. */
-	} else update_resized_map(map);
-
-#endif
-
-	/* Redraw other-map boxes in other windows. */
-	draw_related_maps(map);
-	SetPort(oldport);
-}
-
-void
-toggle_map_topunit(Map *map)
-{
-	GrafPtr oldport;
-	int oldtoph;
-
-	/* Setting the port fixed a display bug. */
-	GetPort(&oldport);
-	SetPort(map->window);
-	oldtoph = map->toph;
-	map->draw_topunit = !map->draw_topunit;
-	map->topunith = (map->draw_topunit ? topunithgt : 0);
-	/* Erase other-map boxes in other windows. */
-	draw_related_maps(map);
-	map->toph = map->toplineh + map->topunith;
-	/* Update without scrolling if we are expanding and the buffer is too small. */ 
-	if (map->bufy - map->offsety < oldtoph - map->toph) {
-		set_content_rect(map);
-		update_gworld(map);
-		/* This is not done by update_gworld (unlike update_resized_gworld). */
-		if (map->toplineh > 0)
-		  draw_top_line(map);
-		if (map->topunith > 0)
-		  draw_unit_info(map);
-	/* Always scroll the map if we are shrinking, or if the buffer is big enough. */
-	} else update_resized_map(map);
-	/* Redraw other-map boxes in other windows. */
-	draw_related_maps(map);
-	SetPort(oldport);
-}
-
-void
-toggle_map_scrollbars(Map *map)
-{
-	/* Erase other-map boxes in other windows. */
-	draw_related_maps(map);
-
-	if (map->sbarwid)
-		map->sbarwid = 0;
-	else if (map == worldmap)
-		map->sbarwid = floatsbarwid;
-	else	map->sbarwid = sbarwid;
-	
-	adjust_map_decor(map);
-	force_map_update(map);
-
-	/* Redraw other-map boxes in other windows. */
-	draw_related_maps(map);
-}
-
-void
-toggle_map_cpanel(Map *map)
-{
-	/* Erase other-map boxes in other windows. */
-	draw_related_maps(map);
-
-	/* Toggle the panel. */
-	map->conw = (map->conw > 0 ? 0 : conwid);
-	adjust_map_decor(map);
-	force_map_update(map);
-
-	/* Redraw other-map boxes in other windows. */
-	draw_related_maps(map);
-}
-
-void
-toggle_map_other_maps(Map *map)
-{
-	map->vp->draw_other_maps = !map->vp->draw_other_maps;
-	/* (should not do a total redraw) */
-	force_map_update(map);
-}
-
-void
-toggle_map_lighting(Map *map)
-{
-	map->vp->draw_lighting = !map->vp->draw_lighting;
-	/* We have to do a total redraw. */
-	force_map_update(map);
-}
-
-void
-toggle_map_coverage(Map *map)
-{
-	map->vp->draw_cover = !map->vp->draw_cover;
-	/* (should only change newly dimmed/lightened cells) */
-	force_map_update(map);
-}
-
-void
-toggle_map_names(Map *map)
-{
-	map->vp->draw_names = !map->vp->draw_names;
-	/* (if now on, should draw names on top of everything, don't redraw everything) */
-	if (map->vp->hh > 5) {
-		force_map_update(map);
-	} else {
-		/* (should be a force update on control panel alone) */
-		draw_control_panel(map);
-	}
-}
-
-void
-toggle_map_people(Map *map)
-{
-    if (!people_sides_defined())
-      return;
-	map->vp->draw_people = !map->vp->draw_people;
-	if (bwid2[map->vp->power] > 0) {
-		force_map_update(map);
-	} else {
-		/* (should be a force update on control panel alone) */
-		draw_control_panel(map);
-	}
-}
-
-void
-toggle_map_control(Map *map)
-{
-    if (!control_sides_defined())
-      return;
-	map->vp->draw_control = !map->vp->draw_control;
-	if (bwid2[map->vp->power] > 0) {
-		force_map_update(map);
-	} else {
-		/* (should be a force update on control panel alone) */
-		draw_control_panel(map);
-	}
-}
-
-void
-toggle_map_elevations(Map *map)
-{
-	map->vp->draw_elevations = !map->vp->draw_elevations;
-	force_map_update(map);
-}
-
-void
-toggle_map_materials(Map *map, int m)
-{
-	map->vp->draw_materials[m] = !map->vp->draw_materials[m];
-	map->vp->num_materials_to_draw += (map->vp->draw_materials[m] ? 1 : -1);
-	force_map_update(map);
-}
-
-void
-toggle_map_aux_terrain(Map *map, int t)
-{
-	map->vp->draw_aux_terrain[t] = !map->vp->draw_aux_terrain[t];
-	force_map_update(map);
-}
-
-void
-toggle_map_temperature(Map *map)
-{
-	map->vp->draw_temperature = !map->vp->draw_temperature;
-	force_map_update(map);
-}
-
-void
-toggle_map_winds(Map *map)
-{
-	map->vp->draw_winds = !map->vp->draw_winds;
-	force_map_update(map);
-}
-
-void
-toggle_map_clouds(Map *map)
-{
-	map->vp->draw_clouds = !map->vp->draw_clouds;
-	force_map_update(map);
-}
-
-void
-toggle_map_storms(Map *map)
-{
-	map->vp->draw_storms = !map->vp->draw_storms;
-	force_map_update(map);
-}
-
-void
-toggle_map_plans(Map *map)
-{
-	map->vp->draw_plans = !map->vp->draw_plans;
-
-	force_map_update(map);
-}
-
-void
-toggle_map_ai(Map *map)
-{
-	if (!side_has_ai(dside))
-	  return;
-	map->vp->draw_ai = !map->vp->draw_ai;
-	force_map_update(map);
-}
-
-void
-toggle_map_sidecolors(Map *map)
-{
-	map->sidecolors = !map->sidecolors;
-	force_map_update(map);
-}
-
-void
-toggle_map_draw_emblems(Map *map)
-{
-	map->draw_emblems = !map->draw_emblems;
-	force_map_update(map);
-}
-
-void
-toggle_iconmasks(Map *map)
-{
-	map->iconmasks = !map->iconmasks;
-	force_map_update(map);
-}
-
-void
-toggle_boxmasks(Map *map)
-{
-	map->boxmasks = !map->boxmasks;
-	force_map_update(map);
-}
-
-void
-toggle_textmasks(Map *map)
-{
-	map->textmasks = !map->textmasks;
-	force_map_update(map);
-}
-
-void
-toggle_featureborders(Map *map)
-{
-	map->featureborders = !map->featureborders;
-	force_map_update(map);
-}
-
-void
-toggle_featurenames(Map *map)
-{
-	map->featurenames = !map->featurenames;
-	force_map_update(map);
-}
-
-void
-toggle_shorelines(Map *map)
-{
-	map->shorelines = !map->shorelines;
-	force_map_update(map);
-}
-
-void
-toggle_simple_borders(Map *map)
-{
-	map->simple_borders = !map->simple_borders;
-	force_map_update(map);
-}
-
-void
-toggle_optimize_fonts(Map *map)
-{
-	map->optimize_fonts = !map->optimize_fonts;
-	force_map_update(map);
-}
-
-void
-toggle_draw_meridians(Map *map)
-{
-	map->vp->draw_meridians = !map->vp->draw_meridians;
-	force_map_update(map);
-}
-
-void
-toggle_solid_color_terrain(Map *map)
-{
-	map->solid_color_terrain = !map->solid_color_terrain;
-	force_map_update(map);
-}
-
-void
-toggle_blinking_curunit(Map *map)
-{
-	map->blinking_curunit = !map->blinking_curunit;
-	force_map_update(map);
-}
-
-void
-toggle_selectionmasks(Map *map)
-{
-	map->selectionmasks = !map->selectionmasks;
-	force_map_update(map);
-}
-
-void
-toggle_erase_names(Map *map)
-{
-	map->erase_names = !map->erase_names;
-	force_map_update(map);
-}
-
 void
 do_mouse_down_map_content(Map *map, int h, int v, int mods)
 {
@@ -1203,7 +870,7 @@ move_the_selected_unit(Map *map, Unit *unit, int h, int v)
 	if (unit_at(x, y) != NULL) {
 		m_nearest_unit(map, h, v, &other);
 	}
-	rslt = advance_into_cell(dside, unit, x, y, other);
+	rslt = advance_into_cell(dside, unit, x, y, other, NULL);
 	return rslt;
 }
 
@@ -1235,10 +902,14 @@ select_exactly_one_unit(Map *map, Unit *unit)
 {
     Unit *thisunit;
 
+#if 0		/* This prevents unselection of other selected units, which is
+		what we want to do when selecting exactly one unit. */
 	if (map->numselections > 0) {		
 		thisunit = map->selections[0];
 		if (thisunit == unit) return;
 	}
+#endif
+
 	unselect_all(map);
 	select_unit_on_map(map, unit);
 	scroll_to_unit(map, unit);
