@@ -9,8 +9,6 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 extern int ok_to_exit;
 #include "macconq.h"
-extern void recalc_spacing(void);
-extern void invert_map(Map *map, int x, int y, int r);
 
 #include <errors.h>
 #include <time.h>
@@ -28,15 +26,17 @@ extern void invert_map(Map *map, int x, int y, int r);
 #define PREFSIZE 2048		/* Size of prefs buffer in lisp format. Must be bigger than the default */
 						/* BUFSIZE = 256 to make room for all the new preferences */
 
-static Boolean missed_any_parameters(AppleEvent *message);
-
-static pascal Boolean filter_warning_alert(DialogPtr dialog, EventRecord *evt, short *itemhit);
+static int open_preferences(void);
+static void close_preferences(void);
+static pascal void draw_font_sample(WindowPtr win, short ditem);
+static void force_sample_redraw(WindowPtr win, short ditem);
+static void add_preferences_topic_items(DialogPtr win);
 
 /* Global variables. */
 
 /* True if new maps should show the cell grid. */
 
-int default_draw_grid = TRUE;
+int default_draw_grid = FALSE;
 
 /* True if new maps should display names and numbers. */
 
@@ -54,6 +54,17 @@ int default_draw_meridians = TRUE;
 /* Spacing, in minutes, of latitude/longitude lines. */
 int default_meridian_interval = 300;
 
+/* Dialogue to set the preferences. */
+
+int cur_prefs_topic = 2;
+
+UserItemUPP draw_font_sample_proc;
+
+int tmp_small_font_id;
+int tmp_small_font_size;
+int tmp_large_font_id;
+int tmp_large_font_size;
+
 int
 open_preferences()
 {
@@ -428,21 +439,6 @@ ui_update_state()
 	dside->uidata = replace_at_key(dside->uidata, "mac", state);
 }
 
-/* Dialogue to set the preferences. */
-
-int cur_prefs_topic = 2;
-
-static pascal void draw_font_sample(WindowPtr win, short ditem);
-static void force_sample_redraw(WindowPtr win, short ditem);
-static void add_preferences_topic_items(DialogPtr win);
-
-UserItemUPP draw_font_sample_proc;
-
-int tmp_small_font_id;
-int tmp_small_font_size;
-int tmp_large_font_id;
-int tmp_large_font_size;
-
 /* Dialog app-defined item callback to display the chosen font. */
 
 static pascal void
