@@ -13,16 +13,13 @@ any later version.  See the file COPYING.  */
 
 extern int any_thickness;
 
-extern int ai_region_at(Side *side, int x, int y);
-
 int tmpdrawlighting;
 int tmpdrawcoverage;
 
 static void draw_terrain_row(Map *map, int x0, int y0, int len);
 static void draw_cliffs(Map *map, int x, int y);
 static void draw_contours(Map *map, int x, int y);
-extern void draw_gridlines(Map *map, int x, int y);
-extern void draw_shorelines(Map *map, int x, int y);
+static void draw_shorelines(Map *map, int x, int y);
 static void draw_borders(Map *map, int x, int y, int b);
 static void draw_connections(Map *map, int x, int y, int c);
 static void draw_feature_boundary_simple(Map *map, int x, int y);
@@ -113,8 +110,6 @@ static void draw_region(Map *map, int x, int y);
 
 /* x0 may be negative here. */
 
-#if 0		/* Single-loop version of draw_row. */
-
 void
 draw_row(Map *map, int x0, int y, int len, int clearit)
 {
@@ -141,143 +136,9 @@ draw_row(Map *map, int x0, int y, int len, int clearit)
 
 	/* First do run-length based drawing of the terrain. */
 	draw_terrain_row(map, x0, y, len);
-
-	/* Then do cell-based drawing of all the other stuff. */
-	for (x = x0; x < x0 + len; ++x) {
-		if (!in_area(x, y))
-			continue;				/* xform crash fix. */
-		xform(map, x, y, &sx, &sy);
-		xw = wrapx(x);
-		/* Draw either cliffs or contour lines, depending on the angle. */
-		if ((map->see_all || terrain_visible(dside, x, y)) ) {
-			if ((elevations_defined() || any_thickness) && ang != 90) {
-				draw_cliffs(map, x, y);
-			} else if (elevations_defined() && map->vp->draw_elevations && ang == 90) {
-				draw_contours(map, x, y);
-			}
-		}
-		/* Draw grid explicitly if asked to and the quick method using smaller cells doesn't work. */
-		if (map->vp->draw_grid && map->leaky_grid) {
-			draw_gridlines(map, x, y);
-		}
-		/* Draw shorelines if desired. */
-		if (map->shorelines) { 
-			draw_shorelines(map, x, y);
-		}
-		/* Draw connections and borders. */
-		if (any_aux_terrain_defined()) {
-			if (bords_to_draw(map)) {
-				for_all_border_types(t) {
-					if (map->vp->draw_aux_terrain[t] 
-						&& aux_terrain_defined(t)) {
-							draw_borders(map, x, y, t);
-					}
-				}
-			}
-			/* Connections should always be drawn on top of borders. */
-			if (conns_to_draw(map)) {
-				for_all_connection_types(t) {
-					if (map->vp->draw_aux_terrain[t] 
-						&& aux_terrain_defined(t)) {
-							draw_connections(map, x, y, t);
-					}
-				}
-			}
-		}
-		/* Draw clouds, winds and temperatures. */
-		if (clouds_defined() 
-			&& map->vp->draw_clouds 
-			&& draw_clouds_here(x, y)) {
-				if (map->vp->hh <= 16) {
-				    	UNWRAP_SX(map, sx, 
-				    		draw_cloud_block(sx, sy, 1, pwr, cloud_view(dside, xw, y), ang));
-				} else UNWRAP_SX(map, sx, 
-						draw_clouds(sx, sy, pwr, cloud_view(dside, xw, y)));
-		}
-		if (winds_defined() 
-			&& map->vp->draw_winds && map->vp->hh > 10 
-			&& draw_winds_here(dside, xw, y)) {
-				UNWRAP_SX(map, sx, 
-					draw_winds(sx, sy, pwr, wind_view(dside, xw, y)));
-		}
-		if (temperatures_defined() 
-			&& map->vp->draw_temperature && map->vp->hh > 10
-			&& draw_temperature_here(dside, xw, y)) {
-				UNWRAP_SX(map, sx, 
-					draw_temperature(sx, sy, pwr, temperature_view(dside, xw, y)));
-		}
-
-		/* We had to draw the terrain on the edge, but skip everything else. */
-		if (!inside_area(x, y))
-		    continue;
-
-		/* Draw feature boundaries. */
-		if (features_defined() && map->featureborders && map->vp->hw > 4) {
-			if (map->simple_borders) { 
-				draw_feature_boundary_simple(map, x, y);
-			} else {
-				draw_feature_boundary(map, x, y);
-			}
-		}
-
-		/* Draw country borders and people rows. */
-		if ((people_sides_defined() || control_sides_defined()) && bwid2[pwr] > 0) {
-			draw_people_row(map, x, y);
-		}
-		/* Draw cell materials. */
-		if (any_cell_materials_defined() && map->vp->num_materials_to_draw > 0 && map->vp->hh > 20) {
-			draw_materials(map, x, y);
-		}
-		/* Draw units. */
-		if (map->vp->draw_units && map->vp->hw > 2) {
-			draw_units(map, x, y);
-		}
-		/* Draw ai regions. */
-		if (map->vp->draw_ai) {
-			draw_ai_region(map, x, y);
-		}
-		if (DebugG) {
-			draw_region(map, x, y);
-		}
-		/* If debugging, draw coverage on top of everything else. */
-		if (DebugG && !dside->see_all && map->vp->hw >= 16) {
-			UNWRAP_SX(map, sx, 
-				draw_coverage(sx, sy, pwr, cover(dside, xw, y), alt_cover(dside, xw, y)));
-		}
-	}
-}
-
-/* Multi-loop version, slightly faster than single-loop version. */
-
-#else
-
-void
-draw_row(Map *map, int x0, int y, int len, int clearit)
-{
-	int	pwr = map->vp->power; 
-	int	ang = map->vp->angle; 
-	int	empty = FALSE;
-	int	x, xw, sx, sy, t;
-
-	if (!between(0, y, area.height - 1))
-	    return;
-	if (!map->see_all && !map->vp->draw_ai && unseen_image == NULL) {
-		empty = TRUE;
-		/* Examine row to see if we can skip it entirely. */
-		for (x = x0; x < x0 + len; ++x) {
-			if (map->see_all || terrain_visible(dside, x, y)
-				|| (numbordtypes > 0 && terrain_visible(dside, x, y + 1))) {
-					empty = FALSE;
-					break;
-			}
-		}
-	}
-	if (empty && !clearit)
-	    return;
-
-	/* First do run-length based drawing of the terrain. */
-	draw_terrain_row(map, x0, y, len);
-
+	if (between(4, map->vp->power, 6) 
+	    && !map->solid_color_terrain)
+		draw_terrain_transitions(map, x0, y, len);
 	/* Draw either cliffs or contour lines, depending on the angle. */
 	if ((elevations_defined() || any_thickness) && map->vp->angle != 90) {
 		for (x = x0; x < x0 + len; ++x) {
@@ -299,8 +160,12 @@ draw_row(Map *map, int x0, int y, int len, int clearit)
 	}
 	/* Draw shorelines if desired. */
 	if (map->shorelines) {
-		for (x = x0; x < x0 + len + 1; ++x)		/* Draw one extra cell here! */
-		  draw_shorelines(map, x, y);
+		for (x = x0; x < x0 + len + 1; ++x) {
+			/* (Should modify draw_borders so that all six
+			border are drawn for each cell instead). */
+			draw_shores(map, x, y);
+			draw_shores(map, x, y -1);
+		}
 	}
 	/* Draw connections and borders. */
 	if (any_aux_terrain_defined()) {
@@ -308,8 +173,12 @@ draw_row(Map *map, int x0, int y, int len, int clearit)
 			for_all_border_types(t) {
 				if (map->vp->draw_aux_terrain[t] 
 					&& aux_terrain_defined(t)) {
-						for (x = x0; x < x0 + len; ++x)
-							draw_borders(map, x, y, t);
+						/* (Should modify draw_borders so that all six
+						border are drawn for each cell instead). */
+						for (x = x0; x < x0 + len + 1; ++x) {
+  							draw_borders(map, x, y, t);
+  							draw_borders(map, x, y -1, t);
+						}
 				}
 			}
 		}
@@ -420,8 +289,6 @@ draw_row(Map *map, int x0, int y, int len, int clearit)
 	}
 }
 
-#endif
-
 /* Draw an entire row of terrain, possibly with a single rectangle fill. */
 /* x0 may be negative. */
 
@@ -477,7 +344,9 @@ draw_terrain_row(Map *map, int x0, int y, int len)
 					case usepictures:
 						break;
 					case usepolygons:
-						quasirand = (11 * x1 + (13 * y) + 7) * 41;
+						/* (quasirand % numsubimages) is used to pick a terrain subimage in 
+						draw_hex_region. This will cause a null pointer crash if quasirand < 0. */
+						quasirand = abs((11 * x1 + (13 * y) + 7) * 41);
 						UNWRAP_SX(map, sx, 
 							draw_hex_region(sx, sy, pwr, segterr, segover, ang, dogrid, dosolid, quasirand));
 				}
@@ -738,6 +607,31 @@ draw_gridlines(Map *map, int x, int y)
 	ForeColor(blackColor);
 }
 
+void 
+draw_shores(Map *map, int x, int y)
+{
+	int	sx, sy, power;
+	int	xw = wrapx(x);
+	Image	*timg;
+
+	power = map->vp->power;
+	if (!inside_area(xw, y))
+	  return;
+	/* Dont draw shores if (xw, y) is invisible */
+	if (!map->see_all && !terrain_visible(dside, xw, y))
+	  return;
+	  
+	/* Use border images if available and asked for. */
+	timg = best_image(beach, hws[power], hhs[power]);
+	if (between(4, power, 6)
+	      && timg->isborder
+	      && timg->subimages != NULL
+	      && map->vp->angle == 90				
+	      && !map->solid_color_terrain) {
+		 draw_shore_images(map, x, y);
+	} else draw_shorelines(map, x, y);
+}
+
 /* This function draws shorelines in 3D relief, using shorecolor. */
 
 void 
@@ -753,12 +647,6 @@ draw_shorelines(Map *map, int x, int y)
 
 	if (wid == 0)
 	  return;
-	/* Dont draw outside the area (causes crashes) */
-	if (!in_area(xw, y))
-	  return;
-	/* Dont draw shores if (xw, y) is invisible */
-	if (!terrain_visible(dside, xw, y))
-	  return;
 	/* Very important to use UNWRAPPED x here! */
 	xform(map, x, y, &sx, &sy);
 
@@ -873,10 +761,30 @@ draw_shorelines(Map *map, int x, int y)
 static void
 draw_borders(Map *map, int x, int y, int b)
 {
-	int 	xw = wrapx(x), dir, sx, sy, bitmask = 0, over;
+	int 		xw = wrapx(x), dir, sx, sy, bitmask = 0, over;
+	int		power = map->vp->power;	
+	Image	*timg;
 	
- 	if (!(map->see_all || terrain_visible(dside, xw, y)) || !any_borders_at(xw, y, b))
- 	  return;
+	if (!inside_area(xw, y))
+	  return;
+ 	if (!map->see_all && !terrain_visible(dside, xw, y))
+	    return;
+	/* Use border images if available and asked for. */
+	timg = best_image(timages[b], hws[power], hhs[power]);
+	if (between(4, power, 6)
+	      && timg->isborder
+	      && timg->subimages != NULL
+	      && map->vp->angle == 90				
+	      && !map->solid_color_terrain) {
+		draw_border_images(map, x, y, b);
+		return;
+	}
+	/* This check must be done after image drawing since it relies on cells
+	without any_borders_at for the endpoints. */ 
+	if (!any_borders_at(xw, y, b))
+	    return;
+
+	/* Else use line drawing. */
 	for_all_directions(dir) {
 		if (border_at(xw, y, dir, b) && borders_visible(dside, xw, y, dir)) {
 			bitmask |= 1 << dir;
@@ -890,8 +798,7 @@ draw_borders(Map *map, int x, int y, int b)
 		/* (should compute and pass in overlay) */
 		UNWRAP_SX(map, sx, 
 			draw_border_line_multiple(map->window, sx, sy, bitmask, map->vp->power, b, 
-								map->vp->angle, over, map->solid_color_terrain));
-		
+								map->vp->angle, over, map->solid_color_terrain));		
 	}
 }
 
@@ -1154,8 +1061,9 @@ draw_materials(Map *map, int x, int y)
 static void
 draw_units(Map *map, int x, int y)
 {
-	int xw = wrapx(x), sx, sy, sw, sh, uview, u, s;
+	int xw = wrapx(x), sx, sy, sw, sh, u, s;
 	Unit *unit;
+	UnitView *uview;
 	Rect tmprect;
 	extern PicHandle dotdotdotpicture;
 
@@ -1163,15 +1071,21 @@ draw_units(Map *map, int x, int y)
 		unit = unit_at(xw, y);
 		if (unit != NULL) {
 			xform(map, x, y, &sx, &sy);
-			if (map->vp->uw <= 16) {
+			if (map->vp->uw <= 16 
+			    && g_bigicons() != TRUE) {
 				/* Adjust to unit part of cell. */
-				sw = map->vp->uw;  sh = map->vp->uh;
-				sx += (map->vp->hw - sw) / 2;  sy += (map->vp->hh - sw) / 2;
+				sw = map->vp->uw;  
+				sh = map->vp->uh;
+				sx += (map->vp->hw - sw) / 2;  
+				sy += (map->vp->hh - sw) / 2;
 				if (unit->occupant != NULL
-					&& sw >= 8
-	    			&& (unit->side == dside || dside->see_all || u_see_occupants(unit->type))) {
+				      && sw >= 8
+	    			      && occupants_visible(dside, map->vp, unit)) {
 					/* Draw a "grouping box", in white, but with no occs drawn. */
 					SetRect(&tmprect, sx, sy, sx + sw, sy + sh);
+					/* Increase top and right margin by one pixel each. */
+					tmprect.top -= 1;
+					tmprect.right += 1;
 					s = side_number(unit->side);
 					if (map->boxmasks) {
 						if (map->sidecolors && icon_mask_color[s]) {
@@ -1222,9 +1136,9 @@ draw_units(Map *map, int x, int y)
 			}
 		}
 	} else {
-		uview = unit_view(dside, xw, y);
-		if (uview != EMPTY) {
-			u = vtype(uview);  s = vside(uview);
+		uview = unit_view_at(dside, xw, y);
+		if (uview != NULL) {
+			u = view_type(uview);  s = view_side(uview)->id;
 			xform(map, x, y, &sx, &sy);
 			sw = map->vp->uw;  sh = map->vp->uh;
 			/* Adjust to unit part of cell. */
@@ -1251,7 +1165,10 @@ draw_unit_and_occs(Map *map, Unit *unit, int sx, int sy, int sw, int sh)
 	   will also be visible. */
 	if (unit->transport && unit->side == unit->transport->side)
 	  emblem = FALSE;
-	if (sw > 8 && occupants_visible(dside, map->vp, unit)) {
+	/* Don't draw grouping box and occs with big icons. */
+	if (sw > 8 
+	    && occupants_visible(dside, map->vp, unit)
+	    && g_bigicons() != TRUE) {
 		/* Draw a sort of "grouping box", in white. */
 		SetRect(&tmprect, sx, sy, sx + sw, sy + sh);
 		 /*Make box 1 pixel less tall to avoid leakage of its lower left corner into the grid.
@@ -1280,23 +1197,17 @@ draw_unit_and_occs(Map *map, Unit *unit, int sx, int sy, int sw, int sh)
 												!completed(unit), emblem));
 
 		/* Draw unit size on top of advanced unit. */
-		if (u_advanced(unit->type) && map->drawsizes) 
+		if (u_advanced(unit->type) && map->drawsizes) { 
 			UNWRAP_SX(map, sx2, 
 				draw_unit_size(unit, map->window, sx2, sy2, sw2, sh2));
-
-		if (map->vp->draw_names) {
-			/* Draw city names in uniform size and position */
-			if (!mobile(unit->type)) {
-			    UNWRAP_SX(map, sx, 
-				draw_unit_name(unit, sx, sy, map->vp->uw, map->vp->uh, 
+		}
+		/* Draw the unit name, but not for an occ within a transport. */
+		if (map->vp->draw_names 
+		      && !unit->transport) {
+			UNWRAP_SX(map, sx, 
+				draw_unit_name(unit, sx, sy, sw, sh, 
 						 	 map->textmasks, map->optimize_fonts));  
-			/* sw2 doubled moves the name out of the box so that it can be read more easily */
-			} else {
-			    UNWRAP_SX(map, sx2, 
-				draw_unit_name(unit, sx2, sy2, 2 * sw2, sh2, 
-							 map->textmasks, map->optimize_fonts)); 
-			}
-		}				
+		}
 		/* Draw all the occupants, in the bottom half of the box. */
 		for_all_occupants(unit, occ) {
 			m_xform_occupant(map, unit, occ, sx, sy, sw, sh, &sx2, &sy2, &sw2, &sh2);
@@ -1314,20 +1225,14 @@ draw_unit_and_occs(Map *map, Unit *unit, int sx, int sy, int sw, int sh)
 		/* Draw unit size on top of advanced unit. */
 		if (u_advanced(unit->type) && map->drawsizes) { 
 	    		UNWRAP_SX(map, sx, 
-				draw_unit_size(unit, map->window, sx, sy, sw, sh));
+				draw_unit_size(unit, map->window, sx, sy, sw, map->vp->uh));
 		}
-
-		if (map->vp->draw_names) {
-			/* Draw city names in uniform size and position */
-			if (!mobile(unit->type)) {
-			      UNWRAP_SX(map, sx, 
-				draw_unit_name(unit, sx, sy, map->vp->uw, map->vp->uh, 
-							 map->textmasks, map->optimize_fonts));  
-			} else {
-			      UNWRAP_SX(map, sx, 
+		/* Draw the unit name, but not for an occ within a transport. */
+		if (map->vp->draw_names 
+		      && !unit->transport) {
+			UNWRAP_SX(map, sx, 
 				draw_unit_name(unit, sx, sy, sw, sh, 
-							 map->textmasks, map->optimize_fonts));
-			}
+						 	  map->textmasks, map->optimize_fonts));
 		}
 	}
 }
