@@ -9,7 +9,6 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 #include "kpublic.h"
 #include "tkconq.h"
-Unit *embarkation_unit(Unit *unit);
 #include "print.h"
 
 void really_do_design(Side *side);
@@ -43,21 +42,79 @@ static void aux_set_formation(Side *side, Map *map, int cancel);
 
 static void aux_design(Side *side, Map *map, int cancel);
 
-/* Use this macro in any command if it requires a current unit. */
+/* Use these functions to check commands' preconditions. */
 
-#define REQUIRE_UNIT(map)  \
-  if (!in_play((map)->curunit) || (map)->curunit_id != (map)->curunit->id) {  \
-    (map)->curunit = NULL;  \
-    (map)->curunit_id = 0;  \
-    cmd_error(dside, "No current unit to command!");  \
-    return;  \
-  }
+static int
+during_game_only(void)
+{
+    if (endofgame) {
+	cmd_error(dside, "Cannot do after game is over!");
+	return FALSE;
+    }
+    return TRUE;
+}
 
-#define DURING_GAME_ONLY()  \
-  if (endofgame) {  \
-    cmd_error(dside, "Cannot do after game is over!");  \
-    return;  \
-  }
+static int
+require_unit(Map *map)
+{
+    if (!in_play(map->curunit) || map->curunit_id != map->curunit->id) {
+	map->curunit = NULL;
+	map->curunit_id = 0;
+	cmd_error(dside, "No current unit to command!");
+	return FALSE;
+    }
+    return TRUE;
+}
+
+static int
+require_own_unit_during_game(Map *map)
+{
+    if (!during_game_only())
+      return FALSE;
+    if (!require_unit(map))
+      return FALSE;
+    if (!side_controls_unit(dside, map->curunit)) {
+	cmd_error(dside, "The unit is not yours to command!");
+	return FALSE;
+    }
+    return TRUE;
+}
+
+/* This function tests that a unit that was originally specified as
+   part of a command is still usable.  The unit may have died or
+   changed sides since the command was first given. */
+
+static int
+unit_still_ok(Unit *unit)
+{
+    if (!in_play(unit)) {
+	/* Note that this is not an error, because the player didn't
+	   do anything wrong. */
+	notify(dside, "Unit is gone!");
+	return FALSE;
+    }
+    if (!side_controls_unit(dside, unit)) {
+	notify(dside, "%s is no longer yours to command!",
+	       unit_handle(dside, unit));
+	return FALSE;
+    }
+    return TRUE;
+}
+
+static UnitVector *selvec;
+
+UnitVector *
+get_selected_units(Side *side)
+{
+    Map *map = side->ui->curmap;
+
+    if (selvec == NULL)
+      selvec = make_unit_vector(2);
+    clear_unit_vector(selvec);
+    if (in_play(map->curunit) && map->curunit_id == map->curunit->id)
+      selvec = add_unit_to_vector(selvec, map->curunit, 0);
+    return selvec;
+}
 
 int designed_on = FALSE;
 
@@ -70,8 +127,8 @@ do_add_terrain(Side *side)
     int u, t, numtypes, tfirst = NONTTYPE, possibles[MAXTTYPES];
     Unit *unit = map->curunit;
 
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
+    if (!require_own_unit_during_game(map))
+      return;
     u = unit->type;
     numtypes = 0;
     for_all_terrain_types(t) {
@@ -146,27 +203,27 @@ aux_add_terrain_2(Side *side, Map *map, int cancel)
     if (grok_position(side, map, &x, &y, NULL)) {
 	if (in_area(x, y)) {
 	    unit = find_unit(map->argunitid);
-	    if (in_play(unit) && side_controls_unit(side, unit)) {
-		switch (t_subtype(map->tmpt)) {
-		  case cellsubtype:
-		    net_prep_alter_cell_action(unit, unit, x, y, map->tmpt);
-		    break;
-		  case bordersubtype:
-		  case connectionsubtype:
-		    if (x == unit->x && y == unit->y) {
-			beep(side);
-			notify(side, "Pick an adjacent location.");
-			map->modalhandler = aux_add_terrain_2;
-		    }
-		    dir = closest_dir(x - unit->x, y - unit->y);
-		    net_prep_add_terrain_action(unit, unit, unit->x, unit->y,
-						dir, map->tmpt);
-		    break;
-		  case coatingsubtype:
-		    net_prep_add_terrain_action(unit, unit, unit->x, unit->y,
-						1, map->tmpt);
-		    break;
+	    if (!unit_still_ok(unit))
+	      return;
+	    switch (t_subtype(map->tmpt)) {
+	      case cellsubtype:
+		net_prep_alter_cell_action(unit, unit, x, y, map->tmpt);
+		break;
+	      case bordersubtype:
+	      case connectionsubtype:
+		if (x == unit->x && y == unit->y) {
+		    beep(side);
+		    notify(side, "Pick an adjacent location.");
+		    map->modalhandler = aux_add_terrain_2;
 		}
+		dir = closest_dir(x - unit->x, y - unit->y);
+		net_prep_add_terrain_action(unit, unit, unit->x, unit->y,
+					    dir, map->tmpt);
+		break;
+	      case coatingsubtype:
+		net_prep_add_terrain_action(unit, unit, unit->x, unit->y,
+					    1, map->tmpt);
+		break;
 	    }
 	}
     } else {
@@ -181,8 +238,8 @@ do_attack(Side *side)
     Map *map = side->ui->curmap;
     Unit *unit = map->curunit;
     
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
+    if (!require_own_unit_during_game(map))
+      return;
     map->argunitid = unit->id;
     ask_position(side, map, "Attack where?", aux_attack);
 }
@@ -200,14 +257,12 @@ aux_attack(Side *side, Map *map, int cancel)
     }
     if (grok_position(side, map, &x, &y, &unit2)) {
 	unit = find_unit(map->argunitid);
-	if (in_play(unit) && side_controls_unit(side, unit)) {
-	    if (unit2 != NULL) {
-		net_prep_attack_action(unit, unit, unit2, 100);
-	    } else {
-	      /* (should attack anything in cell) */
-	    }
+	if (!unit_still_ok(unit))
+	  return;
+	if (unit2 != NULL) {
+	    net_prep_attack_action(unit, unit, unit2, 100);
 	} else {
-	    cmd_error(side, "unit disappeared!?");
+	    cmd_error(side, "Nothing to attack!");
 	}
     } else {
         if (map->inpch != '?')
@@ -216,21 +271,6 @@ aux_attack(Side *side, Map *map, int cancel)
     }
 }
 
-void
-do_auto(Side *side)
-{
-    Map *map = side->ui->curmap;
-    Unit *unit = map->curunit;
-    
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
-    if (unit->plan) {
-	net_set_unit_ai_control(side, unit, !unit->plan->aicontrol, FALSE);
-	/* a hack */
-	unit->plan->waitingfortasks = !unit->plan->aicontrol;
-    }
-}
-
 void
 do_build(Side *side)
 {
@@ -238,8 +278,8 @@ do_build(Side *side)
     int u, u2, possibles[MAXUTYPES], numtypes, ufirst = NONUTYPE;
     Unit *unit = map->curunit;
 
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
+    if (!require_own_unit_during_game(map))
+      return;
     u = unit->type;
     if (!can_build(unit)) {
 	cmd_error(side, "%s can't build anything!", unit_handle(side, unit));
@@ -248,8 +288,7 @@ do_build(Side *side)
     numtypes = 0;
     for_all_unit_types(u2) {
 	if (uu_acp_to_create(u, u2) > 0
-	    && type_allowed_on_side(u2, side)
-	    && has_advance_to_build(side, u2)) {
+	    && side_can_build(side, u2)) {
 	    possibles[u2] = TRUE;
 	    ++numtypes;
 	    ufirst = u2;
@@ -303,20 +342,20 @@ aux_build(Side *side, Map *map, int cancel)
 	notify(side, "Type a key or click on a unit type to select build.");
     }
     if (grok_unit_type(side, map, &u2)) {
-	if (u2 != NONUTYPE) {
-	    unit = find_unit(map->argunitid);
-	    if (in_play(unit) && side_controls_unit(side, unit)) {
-		if (u_speed(unit->type) > 0
-		    || uu_create_range(unit->type, u2) > 0) {
-		    map->inptype = u2;
-		    ask_position(side, map, "Where to build?", aux_build_2);
-		} else {
-		    impl_build(side, unit, u2, unit->x, unit->y,
-			       map->prefixarg);
-		}
-	    } else {
-		cmd_error(side, "unit disappeared!");
-	    }
+	/* Escape silently if the player cancelled. */
+	if (u2 == NONUTYPE)
+	  return;
+	unit = find_unit(map->argunitid);
+	if (!unit_still_ok(unit))
+	  return;
+	if (mobile(unit->type)
+	    || uu_create_range(unit->type, u2) > 0) {
+	    /* For mobile and/or at-a-distance builders, also ask
+	       where to build. */
+	    map->inptype = u2;
+	    ask_position(side, map, "Where to build?", aux_build_2);
+	} else {
+	    impl_build(side, unit, u2, unit->x, unit->y, map->prefixarg);
 	}
     } else {
         if (map->inpch != '?')
@@ -340,9 +379,9 @@ aux_build_2(Side *side, Map *map, int cancel)
     }
     if (grok_position(side, map, &x, &y, NULL)) {
 	unit = find_unit(map->argunitid);
-	if (unit == NULL) {
-	    cmd_error(side, "Unit (#%d) disappeared!", map->argunitid);
-	} else if (!inside_area(x, y)) {
+	if (!unit_still_ok(unit))
+	  return;
+	if (!inside_area(x, y)) {
 	    cmd_error(side, "%s cannot build on the edge of the world.",
 		      unit_handle(side, unit));
 	} else {
@@ -359,29 +398,19 @@ static void
 impl_build(Side *side, Unit *unit, int u2, int x, int y, int n)
 {
     if (n < 0)
-      n = unit_doctrine(unit)->construction_run[u2];
+      n = construction_run_doctrine(unit, u2);
     if (n <= 0)
       n = 99;
     notify(side, "%s will build %d %s.",
 	   unit_handle(side, unit), n, u_type_name(u2));
     net_set_build_task(unit, u2, n, x, y);
-    /* We may need to move to get to where we're supposed to build. */
+    /* We may need to move to get to where we're supposed to build.
+       Just push a task to get within range. */
     if (distance(x, y, unit->x, unit->y) > uu_create_range(unit->type, u2)) {
 	net_push_move_to_task(unit, x, y, uu_create_range(unit->type, u2));
     }
 }
 
-void
-do_clear_plan(Side *side)
-{
-    Map *map = side->ui->curmap;
-    Unit *unit = map->curunit;
-
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
-    net_set_unit_plan_type(side, unit, PLAN_NONE);
-}
-
 void
 do_collect(Side *side)
 {
@@ -390,8 +419,8 @@ do_collect(Side *side)
     int mtocollect;
     char *arg, *rest;
 
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
+    if (!require_own_unit_during_game(map))
+      return;
     mtocollect = NONMTYPE;
     if (nummtypes == 0) {
 	cmd_error(side, "No materials to collect");
@@ -418,73 +447,13 @@ do_copying(Side *side)
     popup_help(side, copying_help_node);
 }
 
-void
-do_delay(Side *side)
-{
-    Map *map = side->ui->curmap;
-    Unit *unit = map->curunit;
-
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
-    if (unit != NULL) {
-	if (unit->plan)
-	  net_delay_unit(unit, TRUE);
-    } else {
-	unit = find_next_awake_mover(side, map->curunit);
-	if (unit != map->curunit) {
-	    set_current_unit(map, unit);
-	} else {
-	    cmd_error(side, "No next awake mover found.");
-	}
-    }
-}
-
-void
-do_detach(Side *side)
-{
-    Map *map = side->ui->curmap;
-    int rslt;
-    Unit *unit = map->curunit;
-
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
-    if (!completed(unit)) {
-	cmd_error(side, "%s is incomplete; cannot detach",
-		  unit_handle(side, unit));
-	return;
-    }
-    rslt = check_transfer_part_action(unit, unit, unit->hp / 2, NULL);
-    if (valid(rslt)) {
-	net_prep_transfer_part_action(unit, unit, unit->hp / 2, NULL);
-    } else {
-	notify(side, "can't detach for some reason?");
-    }
-}
-
-void
-do_detonate(Side *side)
-{
-    Map *map = side->ui->curmap;
-    int rslt;
-    Unit *unit = map->curunit;
-
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
-    rslt = check_detonate_action(unit, unit, unit->x, unit->y, unit->z);
-    if (valid(rslt)) {
-	net_prep_detonate_action(unit, unit, unit->x, unit->y, unit->z);
-    } else {
-	notify(side, "can't detonate for some reason?");
-    }
-}
-
 void
 do_dir(Side *side)
 {
     Map *map = side->ui->curmap;
 
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
+    if (!require_own_unit_during_game(map))
+      return;
     aux_move_dir(side, map, map->curunit);
 }
 
@@ -493,26 +462,29 @@ do_dir_multiple(Side *side)
 {
     Map *map = side->ui->curmap;
 
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
+    if (!require_own_unit_during_game(map))
+      return;
     map->prefixarg = 9999;
     aux_move_dir(side, map, map->curunit);
 }
 
+/* Handle both single and multiple moves in the direction given by
+   tmpkey. */
+
 static void
-aux_move_dir(side, map, unit)
-Side *side;
-Map *map;
-Unit *unit;
+aux_move_dir(Side *side, Map *map, Unit *unit)
 {
     int ndirs, dir, n = map->prefixarg, x, y;
-  
+    HistEventType reason;
+    char failbuf[BUFSIZE];
+
     if (!unit->act || !unit->plan) { /* use a more sophisticated test? */
 	/* ??? can't act ??? */
 	return;
     }
     ndirs = char_to_dir(tmpkey, &dir, NULL, NULL);
     if (ndirs < 1) {
+	/* (would never occur in real life though...) */
 	cmd_error(side, "what direction is that?!?");
 	return;
     }
@@ -521,68 +493,28 @@ Unit *unit;
 		 unit_desig(unit), n, dirnames[dir]);
 	net_set_move_dir_task(unit, dir, n);
     } else {
+	/* Moving directly into an adjacent cell, do now instead of
+	   making a task. */
 	if (!point_in_dir(unit->x, unit->y, dir, &x, &y)) {
+	    /* (but what if leaving the world is allowed?) */
+	    beep(side);
 	    return;
 	}
-	if (!advance_into_cell(side, unit, x, y, unit_at(x, y))) {
+	if (!advance_into_cell(side, unit, x, y, unit_at(x, y), &reason)) {
+	    advance_failure_desc(failbuf, unit, reason);
+	    notify(dside, "%s", failbuf);
 	    beep(side);
 	}
     }
 }
 
-/* Get rid of a unit. */
-
-void
-do_disband(Side *side)
-{
-    Map *map = side->ui->curmap;
-    Unit *unit = map->curunit;
-
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
-    net_disband_unit(side, unit);
-}
-
-void
-do_disembark(Side *side)
-{
-    Map *map = side->ui->curmap;
-    Unit *unit = map->curunit, *transport;
-
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
-    transport = unit->transport;
-    if (transport == NULL) {
-	cmd_error(side, "Not in a transport");
-	return;
-    }
-    if (!in_play(transport)) {
-	cmd_error(side, "Transport is nonsensical?");
-	return;
-    }
-    /* Try moving into the transport's transport, if there is one. */
-    if (transport->transport != NULL
-        && can_occupy(unit, transport->transport)) {
-	net_prep_enter_action(unit, unit, transport->transport);
-	/* (should be able to set up task if can't do action immedly) */
-	return;
-    }
-    /* Try moving into the open in the cell. */
-    if (can_occupy_cell(unit, unit->x, unit->y)
-	|| can_occupy_conn(unit, unit->x, unit->y, unit->z)) {
-	net_prep_move_action(unit, unit, unit->x, unit->y, unit->z);
-	/* (should be able to set up task if can't do action immedly) */
-	return;
-    }
-    cmd_error(side, "Can't disembark here!");
-}
-
 void
 do_distance(Side *side)
 {
     Map *map = side->ui->curmap;
 
-    REQUIRE_UNIT(map);
+    if (!require_unit(map))
+      return;
     map->argunitid = map->curunit->id;
     ask_position(side, map, "Distance to where?", aux_distance);
 }
@@ -602,6 +534,7 @@ aux_distance(Side *side, Map *map, int cancel)
     }
     if (grok_position(side, map, &x, &y, NULL)) {
 	unit = find_unit(map->argunitid);
+	/* We don't care about the unit's ownership. */
 	if (in_area(x, y) && unit != NULL) {
 	    dist = distance(unit->x, unit->y, x, y);
 	    notify(side, "Distance from the current unit is %d cells.", dist);
@@ -614,46 +547,6 @@ aux_distance(Side *side, Map *map, int cancel)
     }
 }
 
-void
-do_embark(Side *side)
-{
-    Map *map = side->ui->curmap;
-    Unit *unit = map->curunit, *transport;
-
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
-    transport = embarkation_unit(unit);
-    if (transport != NULL) {
-	net_prep_enter_action(unit, unit, transport);
-	return;
-    }
-    cmd_error(side, "Nothing for %s to enter!", unit_handle(dside, unit));
-}
-
-/* (should be generic) */
-Unit *
-embarkation_unit(Unit *unit)
-{
-    Unit *transport, *occ;
-
-    /* look for the first possible transport */
-    for_all_stack(unit->x, unit->y, transport) {
-	/* make sure its not the transport we're in and we can enter it */
-	if (transport != unit->transport &&
-	    valid(check_enter_action(unit, unit, transport))) {
-	    return transport;
-	}
-	/* check the occupants too */
-	for_all_occupants(transport, occ) {
-	    if (occ != unit->transport &&
-		valid(check_enter_action(unit, unit, occ))) {
-		return occ;
-	    }
-	}
-    }
-    return NULL;
-}
-
 void
 do_escape(Side *side)
 {
@@ -667,8 +560,8 @@ do_fire(Side *side)
     Unit *other;
     Unit *unit = map->curunit;
 
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
+    if (!require_own_unit_during_game(map))
+      return;
 
     if (map->inpch == '\0') {
       cmd_error(side, "must use keystroke");
@@ -720,19 +613,19 @@ aux_fire_at(Side *side, Map *map, int cancel)
       return;
     if (grok_position(side, map, &x, &y, NULL)) {
 	unit = find_unit(map->argunitid);
+	if (!unit_still_ok(unit))
+	  return;
 	if (x != unit->x || y != unit->y) {
-	    if (unit->act && unit->plan) { /* (should be more sophisticated?) */
-		other = unit_at(x, y);
-		if (other != NULL) {
-		    /* There's a unit to fire at. */
-		    if (other->side == unit->side) {
-			cmd_error(side, "You can't fire at one of your own units!");
-		    } else if (valid(check_fire_at_action(unit, unit, other, -1))) {
-			net_prep_fire_at_action(unit, unit, other, -1);
-		    }
-		} else {
-		    cmd_error(side, "Nothing there to fire at");
+	    other = unit_at(x, y);
+	    if (other != NULL) {
+		/* There's a unit to fire at. */
+		if (other->side == unit->side) {
+		    cmd_error(side, "You can't fire at one of your own units!");
+		} else if (valid(check_fire_at_action(unit, unit, other, -1))) {
+		    net_prep_fire_at_action(unit, unit, other, -1);
 		}
+	    } else {
+		cmd_error(side, "Nothing there to fire at");
 	    }
 	}
     } else {
@@ -747,8 +640,8 @@ do_fire_into(Side *side)
     int x = map->inpx, y = map->inpy, rslt;
     Unit *unit = map->curunit;
 
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
+    if (!require_own_unit_during_game(map))
+      return;
 
     if (inside_area(x, y)) {
 	if (x != unit->x || y != unit->y) {
@@ -777,7 +670,8 @@ do_follow_action(Side *side)
 {
     Map *map = side->ui->curmap;
     
-    DURING_GAME_ONLY();
+    if (!during_game_only())
+      return;
     map->follow_action = !map->follow_action;
     if (map->follow_action) {
 	notify(side, "Following the action on map.");
@@ -786,49 +680,6 @@ do_follow_action(Side *side)
     }
 }
 
-void
-do_give(Side *side)
-{
-    Map *map = side->ui->curmap;
-    int something = FALSE;
-    int n, u, m, r, gift, actual;
-    Unit *unit = map->curunit, *mainunit = NULL;
-    char buf[BUFSIZE];
-
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
-    if (nummtypes == 0) {
-	cmd_error(side, "No materials in this game!");
-	return;
-    }
-    u = unit->type;
-    mainunit = unit->transport;
-    if (mainunit == NULL) {
-	cmd_error(side, "Nothing to give to here!");
-	return;
-    }
-    m = mainunit->type;
-    n = (map->prefixarg < 0 ? 1 : map->prefixarg);
-
-    buf[0] = '\0';
-    for_all_material_types(r) {
-	gift = (n < 0 ? (um_storage_x(m, r) - mainunit->supply[r]) : n);
-	if (gift > 0) {
-	    something = TRUE;
-	    /* Be stingy if we're low */
-	    if (2 * unit->supply[r] < um_storage_x(u, r))
-	      gift = max(1, gift/2);
-	    actual = transfer_supply(unit, mainunit, r, gift);
-	    tprintf(buf, " %d %s", actual, m_type_name(r));
-	}
-    }
-    if (something) {
-	notify(side, "%s gave%s.", unit_handle(side, unit), buf);
-    } else {
-	notify(side, "%s gave nothing.", unit_handle(side, unit));
-    }
-}
-
 /* Give a unit to another side or make it independent. */
 
 /* (but giving to indep should be tested, otherwise might kill unit) */
@@ -841,9 +692,10 @@ do_give_unit(Side *side)
     Map *map = side->ui->curmap;
     Side *side2;
 
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
+    if (!require_own_unit_during_game(map))
+      return;
     if (between(0, map->prefixarg, numsides)) {
+	/* Interpret a prefix as the recipient side. */
 	side2 = side_n(map->prefixarg);
 	really_do_give_unit(map->curunit, side2);
     } else {
@@ -881,14 +733,15 @@ really_do_give_unit(Unit *unit, Side *side2)
     if (0 /* unit is a type that cannot act */) {
 	/* (should add case for nonacting units) */
     } else {
+	/* (should check validity!) */
 	if (side2 != NULL && side2 != indepside) {
-		notify(dside, "You give %s to %s.", unit_handle(dside, unit),
-				short_side_title(side2));
-		net_prep_change_side_action(unit, unit, side2);
+	    notify(dside, "You give %s to %s.",
+		   unit_handle(dside, unit), short_side_title(side2));
+	    net_prep_change_side_action(unit, unit, side2);
 	} else {
-		notify(dside, "You give %s its independence.",
-			unit_handle(dside, unit));
-		net_prep_change_side_action(unit, unit, indepside);
+	    notify(dside, "You give %s its independence.",
+		   unit_handle(dside, unit));
+	    net_prep_change_side_action(unit, unit, indepside);
 	}
     }
 }
@@ -899,20 +752,7 @@ do_help(Side *side)
     popup_help(side, NULL);
 }
 
-/* Tell the unit to sit around for a given number of turns. */
-
-void
-do_idle(Side *side)
-{
-    Map *map = side->ui->curmap;
-    Unit *unit = map->curunit;
-
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
-    if (map->prefixarg < 0)
-      map->prefixarg = 9999;
-    net_set_sentry_task(unit, map->prefixarg);
-}
+/* In general, the subcommands of "map" are for the GUI to use. */
 
 void
 do_map(Side *side)
@@ -925,46 +765,30 @@ do_map(Side *side)
     if (map == NULL)
       return;
     vp = widget_vp(map);
-    if (strcmp(cmdargstr, "iso") == 0) {
-	vp->isometric = TRUE;
-	if (map->prefixarg >= 0)
-	  vp->vertscale = map->prefixarg;
-	redraw_map(map);
+    if (strncmp(cmdargstr, "contour-interval=", 17) == 0) {
+	if (strcmp("?", cmdargstr+17) == 0) {
+	    notify(side, "Contour interval is %d.", vp->contour_interval);
+	} else {
+	    int n = strtol(cmdargstr+17, NULL, 10);
+	    set_contour_interval(vp, n);
+	    redraw_map(map);
+	}
+    } else if (strcmp(cmdargstr, "iso") == 0) {
+	eval_tcl_cmd("set_isometric %d %d %d",
+		     map->number, !vp->isometric,
+		     (map->prefixarg >= 0 ? map->prefixarg : vp->vertscale));
     } else if (strcmp(cmdargstr, "meridians") == 0) {
 	vp->draw_meridians = !vp->draw_meridians;
 	if (map->prefixarg > 0)
 	  set_meridian_interval(vp, map->prefixarg);
 	redraw_map(map);
-    } else if (strcmp(cmdargstr, "over") == 0) {
-	vp->isometric = FALSE;
-	redraw_map(map);
-    } else if (strcmp(cmdargstr, "plan-none") == 0) {
-	if (map->curunit)
-	  net_set_unit_plan_type(dside, map->curunit, PLAN_NONE);
-    } else if (strcmp(cmdargstr, "plan-passive") == 0) {
-	if (map->curunit)
-	  net_set_unit_plan_type(dside, map->curunit, PLAN_PASSIVE);
-    } else if (strcmp(cmdargstr, "plan-defensive") == 0) {
-	if (map->curunit)
-	  net_set_unit_plan_type(dside, map->curunit, PLAN_DEFENSIVE);
-    } else if (strcmp(cmdargstr, "plan-exploratory") == 0) {
-	if (map->curunit)
-	  net_set_unit_plan_type(dside, map->curunit, PLAN_EXPLORATORY);
-    } else if (strcmp(cmdargstr, "plan-offensive") == 0) {
-	if (map->curunit)
-	  net_set_unit_plan_type(dside, map->curunit, PLAN_OFFENSIVE);
-    } else if (strcmp(cmdargstr, "plan-random") == 0) {
-	if (map->curunit)
-	  net_set_unit_plan_type(dside, map->curunit, PLAN_RANDOM);
     } else if (strcmp(cmdargstr, "rotl") == 0) {
-	vp->isodir = left_dir(vp->isodir);
-	printf("Looking %s\n", dirnames[vp->isodir]);
-	redraw_map(map);
+	eval_tcl_cmd("set_iso_dir %d %d", map->number, left_dir(vp->isodir));
+	notify(side, "Now looking %s.", dirnames[vp->isodir]);
     } else if (strcmp(cmdargstr, "rotr") == 0) {
-	vp->isodir = right_dir(vp->isodir);
-	printf("Looking %s\n", dirnames[vp->isodir]);
-	redraw_map(map);
-    } else if (strcmp(cmdargstr, "show_all") == 0) {
+	notify(side, "Now looking %s.", dirnames[vp->isodir]);
+	eval_tcl_cmd("set_iso_dir %d %d", map->number, right_dir(vp->isodir));
+    } else if (strcmp(cmdargstr, "show-all") == 0) {
 	if (!side->may_set_show_all) {
 	    cmd_error(side, "You are not permitted to see everything!");
 	    return;
@@ -1039,8 +863,8 @@ do_move_to(Side *side)
     Map *map = side->ui->curmap;
     Unit *unit = map->curunit;
 
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
+    if (!require_own_unit_during_game(map))
+      return;
     map->argunitid = unit->id;
     ask_position(side, map, "Move to where?", aux_move_to);
 }
@@ -1055,16 +879,15 @@ aux_move_to(Side *side, Map *map, int cancel)
       return;
     if (grok_position(side, map, &x, &y, NULL)) {
 	unit = find_unit(map->argunitid);
+	if (!unit_still_ok(unit))
+	  return;
 #ifdef DESIGNERS
 	if (is_designer(side)) {
 	    net_designer_teleport(unit, x, y, NULL);
-	} else
-#endif /* DESIGNERS */
-	if (in_play(unit)) {
-	    net_set_move_to_task(unit, x, y, 0);
-	} else {
-	    beep(side);
+	    return;
 	}
+#endif /* DESIGNERS */
+	net_set_move_to_task(unit, x, y, 0);
     } else {
 	map->modalhandler = aux_move_to;
     }
@@ -1079,8 +902,8 @@ do_name(Side *side)
     char tmpbuf[BUFSIZE];
     Unit *unit = map->curunit;
 
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
+    if (!require_own_unit_during_game(map))
+      return;
     map->argunitid = unit->id;
     sprintf(tmpbuf, "New name for %s:", unit_handle(side, unit));
     ask_string(side, map, tmpbuf, unit->name, aux_name);
@@ -1096,11 +919,9 @@ aux_name(Side *side, Map *map, int cancel)
       return;
     if (grok_string(side, map, &name)) {
 	unit = find_unit(map->argunitid);
-	if (in_play(unit) && side_controls_unit(side, unit)) {
-	    net_set_unit_name(side, unit, name);
-	} else {
-	    cmd_error(side, "Nothing here that could be named!");
-	}
+	if (!unit_still_ok(unit))
+	  return;
+	net_set_unit_name(side, unit, name);
     } else {
 	map->modalhandler = aux_name;
     }
@@ -1126,14 +947,14 @@ do_occupant(Side *side)
 {
     Map *map = side->ui->curmap;
     Unit *nextocc;
-    Unit *unit = map->curunit;
 
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
-    nextocc = find_next_occupant(unit);
-    if (nextocc != unit) {
-	set_current_unit(map, nextocc);
-    }
+    if (!require_unit(map))
+      return;
+    nextocc = find_next_occupant(map->curunit);
+    if (nextocc != map->curunit)
+      set_current_unit(map, nextocc);
+    else
+      beep(side);
 }
 
 void
@@ -1330,7 +1151,8 @@ do_recenter(Side *side)
 {
     Map *map = side->ui->curmap;
 
-    REQUIRE_UNIT(map);
+    if (!require_unit(map))
+      return;
     recenter(map, map->curunit->x, map->curunit->y);
 }
 
@@ -1346,6 +1168,7 @@ do_refresh(Side *side)
     dside->ui->legends = NULL;
     compute_all_feature_centroids();
     place_legends(dside);
+    update_contour_intervals();
     for_all_maps(map) {
 	redraw_map(map);
     }
@@ -1358,8 +1181,8 @@ do_remove_terrain(Side *side)
     int u, t, numtypes, tfirst = NONTTYPE, possibles[MAXTTYPES];
     Unit *unit = map->curunit;
 
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
+    if (!require_own_unit_during_game(map))
+      return;
     u = unit->type;
     numtypes = 0;
     for_all_terrain_types(t) {
@@ -1408,10 +1231,7 @@ aux_remove_terrain(Side *side, Map *map, int cancel)
 /* This is like do_remove_terrain, but with a terrain type given. */
 
 static void
-do_remove_terrain_2(side, map, t)
-Side *side;
-Map *map;
-int t;
+do_remove_terrain_2(Side *side, Map *map, int t)
 {
     map->tmpt = t;
     ask_position(side, map, "Remove where?", aux_remove_terrain_2);
@@ -1433,20 +1253,20 @@ aux_remove_terrain_2(Side *side, Map *map, int cancel)
     if (grok_position(side, map, &x, &y, NULL)) {
 	if (in_area(x, y)) {
 	    unit = find_unit(map->argunitid);
-	    if (in_play(unit) && side_controls_unit(side, unit)) {
-		switch (t_subtype(map->tmpt)) {
-		  case cellsubtype:
-		    cmd_error(side, "can't remove cell terrain!");
-		    break;
-		  case bordersubtype:
-		  case connectionsubtype:
-		    dir = closest_dir(x - unit->x, y - unit->y);
-		    net_prep_remove_terrain_action(unit, unit, unit->x, unit->y, dir, map->tmpt);
-		    break;
-		  case coatingsubtype:
-		    net_prep_remove_terrain_action(unit, unit, unit->x, unit->y, 1, map->tmpt);
-		    break;
-		}
+	    if (!unit_still_ok(unit))
+	      return;
+	    switch (t_subtype(map->tmpt)) {
+	      case cellsubtype:
+		cmd_error(side, "can't remove cell terrain!");
+		break;
+	      case bordersubtype:
+	      case connectionsubtype:
+		dir = closest_dir(x - unit->x, y - unit->y);
+		net_prep_remove_terrain_action(unit, unit, unit->x, unit->y, dir, map->tmpt);
+		break;
+	      case coatingsubtype:
+		net_prep_remove_terrain_action(unit, unit, unit->x, unit->y, 1, map->tmpt);
+		break;
 	    }
 	}
     } else {
@@ -1461,8 +1281,8 @@ do_repair(Side *side)
     Map *map = side->ui->curmap;
     Unit *unit, *other;
 
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
+    if (!require_own_unit_during_game(map))
+      return;
     unit = map->curunit;
     if (valid(check_repair_action(unit, unit, unit))) {
 	net_prep_repair_action(unit, unit, unit);
@@ -1477,16 +1297,6 @@ do_repair(Side *side)
     cmd_error(side, "No repair possible right now");
 }
 
-void
-do_reserve(Side *side)
-{
-    Map *map = side->ui->curmap;
-
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
-    net_set_unit_reserve(side, map->curunit, TRUE, TRUE /* is this right??? */);
-}
-
 void
 do_resign(Side *side)
 {
@@ -1535,30 +1345,13 @@ aux_resign_to(Side *side, Map *map, int cancel)
     }
 }
 
-/* Set up a task to resupply the unit. */
-
-/* (should warn if task is likely to fail) */
-
-void
-do_return(Side *side)
-{
-    Map *map = side->ui->curmap;
-    Unit *unit = map->curunit;
-
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
-    /* (should doublecheck range and error out if no chance) */
-    if (in_play(unit)) {
-	net_set_resupply_task(unit, NONMTYPE);
-    }
-}
-
 /* Stuff game state into a file. */
 
 void
 do_save(Side *side)
 {
-    DURING_GAME_ONLY();
+    if (!during_game_only())
+      return;
     eval_tcl_cmd("popup_game_save {%s} {%s}",
 		 saved_game_filename(), (cmdargstr ? cmdargstr : "0"));
 }
@@ -1568,8 +1361,8 @@ do_set_formation(Side *side)
 {
     Map *map = side->ui->curmap;
 
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
+    if (!require_own_unit_during_game(map))
+      return;
     map->argunitid = map->curunit->id;
     ask_position(side, map, "Form up on who?", aux_set_formation);
 }
@@ -1590,7 +1383,9 @@ aux_set_formation(Side *side, Map *map, int cancel)
     }
     if (grok_position(side, map, &x, &y, NULL)) {
 	unit = find_unit(map->argunitid);
-	if (in_play(unit) && in_area(x, y)) {
+	if (!unit_still_ok(unit))
+	  return;
+	if (in_area(x, y)) {
 	    strcpy(ubuf, unit_handle(side, unit));
 	    leader = unit_at(x, y);
 	    if (in_play(leader)
@@ -1622,10 +1417,9 @@ do_set_view_angle(Side *side)
     Map *map = side->ui->curmap;
     VP *vp = widget_vp(map);
 
-    vp->isometric = !vp->isometric;
-    if (map->prefixarg >= 0)
-      vp->vertscale = map->prefixarg;
-    redraw_map(map);
+    eval_tcl_cmd("set_isometric %d %d %d",
+		 map->number, !vp->isometric,
+		 (map->prefixarg >= 0 ? map->prefixarg : vp->vertscale));
 }
 
 void
@@ -1634,23 +1428,13 @@ do_side_closeup(Side *side)
     cmd_error(side, "Not implemented.");
 }
 
-void
-do_sleep(Side *side)
-{
-    Map *map = side->ui->curmap;
-    Unit *unit = map->curunit;
-
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
-    net_set_unit_asleep(side, unit, TRUE, TRUE /* is this right??? */);
-}
-
 void
 do_standing_orders(Side *side)
 {
     int rslt;
 
-    DURING_GAME_ONLY();
+    if (!during_game_only())
+      return;
     if (cmdargstr) {
 	rslt = parse_standing_order(side, cmdargstr);
 	if (rslt < 0)
@@ -1664,7 +1448,8 @@ do_survey(Side *side)
 {
     Map *map = side->ui->curmap;
 
-    DURING_GAME_ONLY();
+    if (endofgame && map->mode == survey_mode)
+      return;
     if (map->mode == survey_mode && !endofgame) {
 	map->mode = move_mode;
 	if (is_designer(side))
@@ -1690,28 +1475,6 @@ do_survey(Side *side)
       eval_tcl_cmd("update_mode %d survey", map->number);
 }
 
-/* Take supplies from transport. */
-
-void
-do_take(Side *side)
-{
-    Map *map = side->ui->curmap;
-    short m, amts[MAXMTYPES], rslts[MAXMTYPES], needed;
-    Unit *unit = map->curunit;
-
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
-    if (nummtypes == 0) {
-	cmd_error(side, "No materials in this game!");
-	return;
-    }
-    for_all_material_types(m) {
-	amts[m] = map->prefixarg;
-    }
-    needed = take_supplies(unit, amts, rslts);
-    report_take(side, unit, needed, rslts);
-}
-
 void
 do_unit_closeup(Side *side)
 {
@@ -1733,42 +1496,6 @@ do_version(Side *side)
     notify(side, "(c) %s", copyright_string());
 }
 
-void
-do_wake(Side *side)
-{
-    Map *map = side->ui->curmap;
-
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
-    /* Do the curunit explicitly, might only be an occupant. */
-    if (in_play(map->curunit)) {
-	net_wake_unit(side, map->curunit, FALSE);
-    }
-    /* If an argument was given, apply to all "top-level" units within
-       the radius specified by the argument. */
-    if (map->prefixarg >= 0)
-      net_wake_area(side, map->curunit->x, map->curunit->y, map->prefixarg,
-		    FALSE);
-}
-
-void
-do_wake_all(Side *side)
-{
-    Map *map = side->ui->curmap;
-
-    DURING_GAME_ONLY();
-    REQUIRE_UNIT(map);
-    /* Do the curunit explicitly, might only be an occupant. */
-    if (in_play(map->curunit)) {
-	net_wake_unit(side, map->curunit, TRUE);
-    }
-    /* If an argument was given, apply to all "top-level" units within
-       the radius specified by the argument. */
-    if (map->prefixarg >= 0)
-      net_wake_area(side, map->curunit->x, map->curunit->y, map->prefixarg,
-		    TRUE);
-}
-
 void
 do_warranty(Side *side)
 {
