@@ -8,7 +8,6 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 #include "tkconq.h"
-Unit *embarkation_unit(Unit *unit);
 
 void init_other_images(void);
 void really_do_design(Side *side);
@@ -25,6 +24,10 @@ Pixmap antpic;
 
 ImageFamily *generic_transition;
 
+int any_resources;
+
+int grid_matches_unseen;
+
 short utype_indexes[MAXUTYPES];
 short mtype_indexes[MAXMTYPES];
 
@@ -164,6 +167,9 @@ int min_h_for_unit_image = 16;
 void
 init_ui(Side *side)
 {
+    /* Might be called a second time, deal with it. */
+    if (dside != NULL)
+      return;
     if (side_wants_display(side)) {
 	if (dside == NULL) {
 	    dside = side;
@@ -263,6 +269,9 @@ init_display(void)
 	int m, i = 0;
 
 	for_all_material_types(m) {
+	    /* Test for the presence of "resources" to draw on maps. */
+	    if (m_resource_icon(m) > 0)
+	      any_resources = TRUE;
 	    if (m_treasury(m))
 	      mtype_indexes[m] = i++;
 	    else
@@ -317,6 +326,8 @@ set_colors(void)
 	for (i = 0; i < 5; ++i)
 	  dside->ui->cellshades[t][i] = dside->ui->graycolor;
     }
+    if (strcmp(g_grid_color(), g_unseen_color()) == 0)
+      grid_matches_unseen = TRUE;
 }
 
 /* This will set up colors that are not crucial to the game.  By doing
@@ -328,33 +339,32 @@ set_optional_colors(void)
 {
     XColor *color;
 
-    if (!dside->ui->monochrome) {
-	if ((color = request_color("maroon")) != NULL)
-	  dside->ui->diffcolor = color;
-	if ((color = request_color("light gray")) != NULL)
-	  dside->ui->graycolor = color;
-	if ((color = request_color(g_contour_color())) != NULL)
-	  dside->ui->contour_color = color;
-	if ((color = request_color(g_country_border_color())) != NULL)
-	  dside->ui->country_border_color = color;
-#if 0 /* need to work with a list here */
-	dside->ui->feature_color = request_color(g_feature_color());
-#endif
-	if ((color = request_color(g_frontline_color())) != NULL)
-	  dside->ui->frontline_color = color;
-	if ((color = request_color(g_meridian_color())) != NULL)
-	  dside->ui->meridian_color = color;
-	if ((color = request_color(g_shoreline_color())) != NULL)
-	  dside->ui->shoreline_color = color;
-	if ((color = request_color(g_unit_name_color())) != NULL)
-	  dside->ui->unit_name_color = color;
-	if ((color = request_color("red")) != NULL)
-	  dside->ui->badcolor = color;
-	if ((color = request_color("yellow")) != NULL)
-	  dside->ui->warncolor = color;
-	if ((color = request_color("green")) != NULL)
-	  dside->ui->goodcolor = color;
-    }
+    if ((color = request_color("maroon")) != NULL)
+      dside->ui->diffcolor = color;
+    if ((color = request_color("light gray")) != NULL)
+      dside->ui->graycolor = color;
+    if ((color = request_color(g_grid_color())) != NULL)
+      dside->ui->grid_color = color;
+    if ((color = request_color(g_unseen_color())) != NULL)
+      dside->ui->unseen_color = color;
+    if ((color = request_color(g_contour_color())) != NULL)
+      dside->ui->contour_color = color;
+    if ((color = request_color(g_country_border_color())) != NULL)
+      dside->ui->country_border_color = color;
+    if ((color = request_color(g_frontline_color())) != NULL)
+      dside->ui->frontline_color = color;
+    if ((color = request_color(g_meridian_color())) != NULL)
+      dside->ui->meridian_color = color;
+    if ((color = request_color(g_shoreline_color())) != NULL)
+      dside->ui->shoreline_color = color;
+    if ((color = request_color(g_unit_name_color())) != NULL)
+      dside->ui->unit_name_color = color;
+    if ((color = request_color("red")) != NULL)
+      dside->ui->badcolor = color;
+    if ((color = request_color("yellow")) != NULL)
+      dside->ui->warncolor = color;
+    if ((color = request_color("green")) != NULL)
+      dside->ui->goodcolor = color;
 }
 
 /* Collect all the names of types etc for which no images could be found,
@@ -505,17 +515,6 @@ request_color(char *name)
           "Requesting anonymous color on display \"%s\", substituting white",
 		     dside->player->displayname);
 	rslt = Tk_GetColor(interp, tkwin, Tk_GetUid("white"));
-    } else if (dside->ui->monochrome) {
-	if (strcmp("white", name) == 0) {
-	    rslt = Tk_GetColor(interp, tkwin, Tk_GetUid("white"));
-	} else if (strcmp("black", name) == 0) {
-	    rslt = Tk_GetColor(interp, tkwin, Tk_GetUid("black"));
-	} else {
-	    init_warning(
-              "No color \"%s\" on the mono display \"%s\", substituting white",
-			 name, dside->player->displayname);
-	    rslt = Tk_GetColor(interp, tkwin, Tk_GetUid("white"));
-	}
     } else {
 	/* Try the generic Tk mechanism first. */
 	rslt = Tk_GetColor(interp, tkwin, Tk_GetUid(name));
@@ -776,31 +775,6 @@ init_unit_images(void)
     }
 }
 
-extern ImageFamily *get_material_type_images(Side *side, int m);
-
-/* (should be in ui.c) */
-
-ImageFamily *
-get_material_type_images(Side *side, int m)
-{
-    char *name;
-    ImageFamily *imf;
-
-    if (!empty_string(m_image_name(m)))
-      name = m_image_name(m);
-    else
-      name = m_type_name(m);
-    imf = get_generic_images(name);
-#if 0
-    if (imf != NULL && imf->numsizes == 0) {
-	imf->ersatz = TRUE;
-	imf = add_default_material_image(imf, m);
-    }
-#endif
-    record_imf_get(imf);
-    return imf;
-}
-
 static void
 init_material_images(void)
 {
@@ -871,7 +845,7 @@ init_terrain_images(void)
 		  subimg = timg->subimages[0];
 		tkimg = (TkImage *) subimg->hook;
 		if (tkimg != NULL) {
-		    if (!dside->ui->monochrome && tkimg->colr != None)
+		    if (tkimg->colr != None)
 		      dside->ui->terrpics[p][t] = tkimg->colr;
 		    else
 		      dside->ui->terrpics[p][t] = tkimg->mono;
@@ -943,7 +917,7 @@ init_emblem_images(void)
       (short *) xmalloc((g_sides_max() + 1) * sizeof(short));
     /* Independent units may have a distinguishing emblem, so the
        indepside is included here. */
-    for_all_sides_plus_indep(side2) {
+    for_all_sides(side2) {
 	init_emblem(side2);
     }
 }
@@ -965,7 +939,7 @@ init_emblem(Side *side2)
 
     /* Collect the color names of the other side and try to request
        them for our own display. */
-    if (!dside->ui->monochrome && !empty_string(side2->colorscheme)) {
+    if (!empty_string(side2->colorscheme)) {
 	/* take spec apart and iterate for multiple colors */
         /* (should be generic code) */
 	for (s = side2->colorscheme, c = cbuf, i = 0; i < 3; ++s) {
