@@ -57,6 +57,7 @@ typedef struct {
     Tcl_Command widgetCmd;	/* Token for imfsample's widget command. */
     int border_width;		/* Width of 3-D border around whole widget. */
     Tk_3DBorder bg_border;	/* Used for drawing background. */
+    Tk_3DBorder fg_border;	/* Used for drawing foreground. */
     int relief;			/* Indicates whether window as a whole is
 				 * raised, sunken, or flat. */
     GC copygc;			/* Graphics context for copying from
@@ -75,6 +76,7 @@ typedef struct {
     int show_masks;
     int show_bbox;
     int show_solid;
+    int show_grid;
 
     int width, height;
 
@@ -109,6 +111,12 @@ static Tk_ConfigSpec config_specs[] = {
 	"0", Tk_Offset(Imfsample, border_width), 0},
     {TK_CONFIG_INT, "-dbl", "doubleBuffer", "DoubleBuffer",
 	"1", Tk_Offset(Imfsample, double_buffer), 0},
+    {TK_CONFIG_SYNONYM, "-fg", "foreground", (char *) NULL,
+	(char *) NULL, 0, 0},
+    {TK_CONFIG_BORDER, "-foreground", "foreground", "Foreground",
+	"white", Tk_Offset(Imfsample, fg_border), TK_CONFIG_COLOR_ONLY},
+    {TK_CONFIG_BORDER, "-foreground", "foreground", "Foreground",
+	"white", Tk_Offset(Imfsample, fg_border), TK_CONFIG_MONO_ONLY},
     {TK_CONFIG_PIXELS, "-height", "height", "Height",
 	"32", Tk_Offset(Imfsample, height), 0},
 #if 0 /* disabled until we debug */
@@ -127,6 +135,8 @@ static Tk_ConfigSpec config_specs[] = {
 	"0", Tk_Offset(Imfsample, show_bbox), 0},
     {TK_CONFIG_INT, "-showcolor", "showColor", "ShowColor",
 	"1", Tk_Offset(Imfsample, show_color), 0},
+    {TK_CONFIG_INT, "-showgrid", "showGrid", "ShowGrid",
+	"0", Tk_Offset(Imfsample, show_grid), 0},
     {TK_CONFIG_INT, "-shownames", "showNames", "ShowNames",
 	"0", Tk_Offset(Imfsample, show_names), 0},
     {TK_CONFIG_INT, "-showmasks", "showMasks", "ShowMasks",
@@ -192,6 +202,7 @@ imfsample_cmd(ClientData cldata, Tcl_Interp *interp, int argc, char **argv)
 			(ClientData) imfsample, imfsample_cmd_deleted_proc);
     imfsample->border_width = 0;
     imfsample->bg_border = NULL;
+    imfsample->fg_border = NULL;
     imfsample->relief = TK_RELIEF_FLAT;
     imfsample->copygc = None;
     imfsample->gc = None;
@@ -200,6 +211,7 @@ imfsample_cmd(ClientData cldata, Tcl_Interp *interp, int argc, char **argv)
     imfsample->show_color = 1;
     imfsample->show_names = 0;
     imfsample->show_masks = 0;
+    imfsample->show_grid = 0;
     imfsample->fill_color = NULL;
     imfsample->selected = -1;
     imfsample->with_terrain = -1;
@@ -428,6 +440,8 @@ imfsample_configure(Tcl_Interp *interp, Imfsample *imfsample,
 
     /* Set the background for the window and create a graphics context
        for use during redisplay.  */
+    Tk_SetWindowBackground(imfsample->tkwin,
+			   Tk_3DBorderColor(imfsample->fg_border)->pixel);
     Tk_SetWindowBackground(imfsample->tkwin,
 			   Tk_3DBorderColor(imfsample->bg_border)->pixel);
     if ((imfsample->copygc == None) && 1/*imfsample->double_buffer*/) {
@@ -535,11 +549,10 @@ imfsample_display(ClientData cldata)
     gc = imfsample->gc;
     XSetClipMask(dpy, gc, None);
     if (imfsample->show_names) {
-      tkfont = Tk_GetFont(interp, tkwin, "-size 10");
-      XSetFont(imfsample->display, gc, Tk_FontId(tkfont));
+	tkfont = Tk_GetFont(interp, tkwin, "-size 10");
+	XSetFont(imfsample->display, gc, Tk_FontId(tkfont));
     }
 
-
     /* Redraw the widget's background/border. */
     if (imfsample->with_terrain >= 0) {
 	ImageFamily *timf = imfsample->imf_list[imfsample->with_terrain];
@@ -579,13 +592,21 @@ imfsample_display(ClientData cldata)
 	if (imfsample->numimages == 1)
 	  imfsample->iwidth = imfsample->width;
     }
-    imfsample->eltw = imfsample->iwidth + 2 * imfsample->pad;
-    if (imfsample->show_names)
+    imfsample->eltw = imfsample->iwidth;
+    if (imfsample->show_grid)
+      imfsample->eltw += 2;
+    else
+      imfsample->eltw += 2 * imfsample->pad;
+    if (imfsample->show_names && !imfsample->show_grid)
       imfsample->eltw += 80;
-    imfsample->elth = imfsample->iheight + 2 * imfsample->pad;
+    imfsample->elth = imfsample->iheight;
+    if (imfsample->show_grid)
+      imfsample->elth += 2;
+    else
+      imfsample->elth += 2 * imfsample->pad;
     /* Fix a lower bound on the vertical spacing. */
     /* (should be determined by choice of app font) */
-    if (imfsample->elth < 10)
+    if (imfsample->elth < 10 && !imfsample->show_grid)
       imfsample->elth = 10;
     /* Compute and save the number of columns to use. */
     imfsample->cols = winwidth / imfsample->eltw;
@@ -593,7 +614,7 @@ imfsample_display(ClientData cldata)
       imfsample->cols = 1;
     /* We can get a little wider spacing by recalculating the element
        width. */
-    if (imfsample->show_names)
+    if (imfsample->show_names && !imfsample->show_grid)
       imfsample->eltw = (winwidth - 10) / imfsample->cols;
     imfsample->rows = imfsample->numimages / imfsample->cols;
     /* Account for a last partial row. */
@@ -609,17 +630,24 @@ imfsample_display(ClientData cldata)
     if (imfsample->firstvisrow + imfsample->numvisrows > imfsample->rows)
       imfsample->firstvisrow = imfsample->rows - imfsample->numvisrows;
     /* Now iterate through all the images we want to draw. */
-    for (row = imfsample->firstvisrow; row < (imfsample->firstvisrow + imfsample->numvisrows); ++row) {
+    for (row = imfsample->firstvisrow;
+	 row < (imfsample->firstvisrow + imfsample->numvisrows);
+	 ++row) {
 	for (col = 0; col < imfsample->cols; ++col) {
 	    n = row * imfsample->cols + col;
 	    if (n >= imfsample->numimages)
 	      break;
 	    sx = col * imfsample->eltw;
 	    sy = (row - imfsample->firstvisrow) * imfsample->elth;
+	    if (imfsample->show_grid)
+	      Tk_Fill3DRectangle(tkwin, d, imfsample->fg_border,
+				 sx + 2, sy + 2,
+				 imfsample->iwidth, imfsample->iheight,
+				 imfsample->border_width, imfsample->relief);
 	    draw_one_main_image(imfsample, d, gc, imfsample->imf_list[n],
 			   sx + imfsample->pad, sy + imfsample->pad,
 			   imfsample->iwidth, imfsample->iheight);
-	    if (imfsample->show_names) {
+	    if (imfsample->show_names && !imfsample->show_grid) {
 		namex = sx + imfsample->pad + imfsample->iwidth + 2;
 		namey = sy + (imfsample->elth / 2) + 5;
 		XSetClipMask(dpy, gc, None);
@@ -651,6 +679,16 @@ imfsample_display(ClientData cldata)
     }
     if (imfsample->show_names)
       Tk_FreeFont(tkfont);
+    /* In theory this shouldn't be necessary, but in practice the
+       interface widgets (esp. the progress bar fill color) are
+       affected by the last value of the foreground left over from
+       drawing, so set to a consistent value. */
+    /* (Note that as of 2000-09-16, some color errors persist, so
+       this might not really be necessary -sts) */
+    XSetForeground(imfsample->display, imfsample->gc, black_color->pixel);
+    XSetBackground(imfsample->display, imfsample->gc, white_color->pixel);
+    XSetForeground(imfsample->display, imfsample->copygc, black_color->pixel);
+    XSetBackground(imfsample->display, imfsample->copygc, white_color->pixel);
 }
 
 static void
@@ -734,23 +772,23 @@ draw_one_image(Imfsample *imfsample, Drawable d, GC gc, Image *img,
 	    XSetTile(dpy, gc, tkimg->colr);
 	    XFillRectangle(dpy, d, gc, sx, sy, sw, sh);
 	} else {
-	  if (use_clip_mask) {
-	    if (tkimg->mask != None && imfsample->show_masks) {
-	        XSetClipOrigin(dpy, gc, sx2, sy2);
-		/* Use the image's mask to do clipped drawing. */
-		XSetClipMask(dpy, gc, tkimg->mask);
+	    if (use_clip_mask) {
+		if (tkimg->mask != None && imfsample->show_masks) {
+		    XSetClipOrigin(dpy, gc, sx2, sy2);
+		    /* Use the image's mask to do clipped drawing. */
+		    XSetClipMask(dpy, gc, tkimg->mask);
+		}
+	    } else {
+		if (tkimg->mask != None && imfsample->show_masks) {
+		    XSetFunction(dpy, gc, GXand);
+		    XCopyArea(dpy, tkimg->mask, d, gc, 0, 0, sw, sh, sx, sy);
+		}
+		XSetFunction(dpy, gc, GXor);
 	    }
-	  } else {
-	    if (tkimg->mask != None && imfsample->show_masks) {
-	      XSetFunction(dpy, gc, GXand);
-	      XCopyArea(dpy, tkimg->mask, d, gc, 0, 0, sw, sh, sx, sy);
-	    }
-	    XSetFunction(dpy, gc, GXor);
-	  }
-	  /* Draw the color image. */
-	  XCopyArea(dpy, tkimg->colr, d, gc, 0, 0, sw, sh, sx, sy);
-	  if (!use_clip_mask)
-	    XSetFunction(dpy, gc, GXcopy);
+	    /* Draw the color image. */
+	    XCopyArea(dpy, tkimg->colr, d, gc, 0, 0, sw, sh, sx, sy);
+	    if (!use_clip_mask)
+	      XSetFunction(dpy, gc, GXcopy);
 	}
     } else if (tkimg->solid != NULL && imfsample->show_color) {
 	XSetFillStyle(dpy, gc, FillSolid);
@@ -758,35 +796,35 @@ draw_one_image(Imfsample *imfsample, Drawable d, GC gc, Image *img,
 	XFillRectangle(dpy, d, gc, sx, sy, sw, sh);
     } else if (tkimg->mono != None || tkimg->mask != None) {
 	if (use_clip_mask) {
-	  XSetFillStyle(dpy, gc, FillSolid);
-	  /* Set the color we're going to use for the mask; use the
-	     imagecolor if we'll be using the mask as the only image. */
-	  XSetForeground(dpy, gc,
-			 (tkimg->mono == None ? imagecolor : maskcolor)->pixel);
-	  XSetClipOrigin(dpy, gc, sx2, sy2);
-	  /* Set the clip mask to be explicit mask or unit's image. */
-	  if (tkimg->mask && imfsample->show_masks)
-	    XSetClipMask(dpy, gc, tkimg->mask);
-	  else
-	    XSetClipMask(dpy, gc, tkimg->mono);
-	  /* Draw the mask. */
-	  XFillRectangle(dpy, d, gc, sx, sy, sw, sh);
-	  /* Draw the image proper. */
-	  if (tkimg->mono != None) {
-	    XSetForeground(dpy, gc, imagecolor->pixel);
-	    XSetClipMask(dpy, gc, tkimg->mono);
+	    XSetFillStyle(dpy, gc, FillSolid);
+	    /* Set the color we're going to use for the mask; use the
+	       imagecolor if we'll be using the mask as the only image. */
+	    XSetForeground(dpy, gc,
+			   (tkimg->mono == None ? imagecolor : maskcolor)->pixel);
+	    XSetClipOrigin(dpy, gc, sx2, sy2);
+	    /* Set the clip mask to be explicit mask or unit's image. */
+	    if (tkimg->mask && imfsample->show_masks)
+	      XSetClipMask(dpy, gc, tkimg->mask);
+	    else
+	      XSetClipMask(dpy, gc, tkimg->mono);
+	    /* Draw the mask. */
 	    XFillRectangle(dpy, d, gc, sx, sy, sw, sh);
-	  }
+	    /* Draw the image proper. */
+	    if (tkimg->mono != None) {
+		XSetForeground(dpy, gc, imagecolor->pixel);
+		XSetClipMask(dpy, gc, tkimg->mono);
+		XFillRectangle(dpy, d, gc, sx, sy, sw, sh);
+	    }
 	} else {
-	  if (tkimg->mask != None && imfsample->show_masks) {
-	    XSetFunction(dpy, gc, GXand);
-	    XCopyArea(dpy, tkimg->mask, d, gc, 0, 0, sw, sh, sx, sy);
-	  }
-	  if (tkimg->mono != None) {
-	    XSetFunction(dpy, gc, GXor);
-	    XCopyArea(dpy, tkimg->mono, d, gc, 0, 0, sw, sh, sx, sy);
-	  }
-	  XSetFunction(dpy, gc, GXcopy);
+	    if (tkimg->mask != None && imfsample->show_masks) {
+		XSetFunction(dpy, gc, GXand);
+		XCopyArea(dpy, tkimg->mask, d, gc, 0, 0, sw, sh, sx, sy);
+	    }
+	    if (tkimg->mono != None) {
+		XSetFunction(dpy, gc, GXor);
+		XCopyArea(dpy, tkimg->mono, d, gc, 0, 0, sw, sh, sx, sy);
+	    }
+	    XSetFunction(dpy, gc, GXcopy);
 	}
     }
 }
