@@ -20,7 +20,7 @@ done
 
 # Count uses of function-based symbols.
 
-for file in utype.def mtype.def ttype.def table.def gvar.def ; do
+for file in utype.def mtype.def ttype.def atype.def table.def gvar.def ; do
 	for fun in `grep '^DEF_' kernel/${file} | sed -e 's/^[^,]*, \([a-zA-Z0-9_]*\),.*$/\1/'` ; do
 		rslt=`grep -c "${fun}[^A-Za-z_]" */*.[hc] | grep -v ':0$' | sed -e 's/:1$//' | tr '\012' ' '`
 		# Symbol name says what kind of thing it is.
