@@ -1,5 +1,5 @@
 /* Definitions of image families for the X11 interface to Xconq.
-   Copyright (C) 1992, 1993, 1994, 1995, 1998 Stanley T. Shebs.
+   Copyright (C) 1992-1995, 1998, 1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -26,9 +26,8 @@ typedef struct a_x11_image {
     Pixel *colpix;
 } X11Image;
 
-extern X11Image *init_x11_image PARAMS ((Image *img));
-extern X11Image *get_x11_image PARAMS ((Image *img));
-extern ImageFamily *x11_load_imf PARAMS ((ImageFamily *imf));
-extern ImageFamily *x11_interp_imf PARAMS ((ImageFamily *imf, Image *img,
-					    int force));
-extern void reverse_bit_endianness PARAMS ((char *rawdata, int numbytes));
+extern X11Image *init_x11_image(Image *img);
+extern X11Image *get_x11_image(Image *img);
+extern ImageFamily *x11_load_imf(ImageFamily *imf);
+extern ImageFamily *x11_interp_imf(ImageFamily *imf, Image *img, int force);
+extern void reverse_bit_endianness(char *rawdata, int numbytes);
