@@ -1,5 +1,5 @@
 /* A no-longer simple X program to display and edit xconq image families.
-   Copyright (C) 1994, 1995, 1996 Massimo Campostrini & Stanley T. Shebs.
+   Copyright (C) 1994, 1995, 1996, 1999 Massimo Campostrini & Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@ any later version.  See the file COPYING.  */
 #include "lisp.h"
 #include "imf.h"
 #include "ximf.h"
-#include "xutil.h"
+#include "xcutil.h"
 
 /* MAXROWS, MAXCOLS, MINROWS, and MINCOLS can be changed safely */
 #define MAXROWS 50
@@ -78,9 +78,6 @@ typedef struct a_family_stuff {
 
 FamilyStuff family[MAXFAMS];
 
-int numremotes;
-int my_rid;
-
 int basew = 32, baseh = 32;
 
 int cols, rows, color_comb;
@@ -179,9 +176,9 @@ selected by the \"-o\" option), and update.",
   NULL
 };
 
-void show_image   PARAMS ((Widget w, XEvent *event, String *params, Cardinal *num_params));
-void show_family  PARAMS ((Widget w, XEvent *event, String *params, Cardinal *num_params));
-void image_action PARAMS ((Widget w, XEvent *event, String *params, Cardinal *num_params));
+void show_image(Widget w, XEvent *event, String *params, Cardinal *num_params);
+void show_family(Widget w, XEvent *event, String *params, Cardinal *num_params);
+void image_action(Widget w, XEvent *event, String *params, Cardinal *num_params);
 
 static XtActionsRec  actions[] = {
   { "ShowImage",   show_image   },       /* set info line for this image */
@@ -195,52 +192,53 @@ static char Translations[] =
    <LeaveWindow>:  ShowFamily() \n\
    <BtnDown>:      ImageAction() ";
 
-void do_help PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
-void done_help PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
-void do_select PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
-void do_deselect PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
-void do_toggle PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
-void do_read PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
-void do_save PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
-void do_delete PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
-void do_quit PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
-void do_open_family PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
-void do_prep_delete PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
-void do_prep_update PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
-void do_prep_export PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
-void do_close_family PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
-void do_increase_magn PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
-void do_decrease_magn PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
+void do_help(Widget w, XtPointer cldata, XtPointer cadata);
+void done_help(Widget w, XtPointer cldata, XtPointer cadata);
+void do_select(Widget w, XtPointer cldata, XtPointer cadata);
+void do_deselect(Widget w, XtPointer cldata, XtPointer cadata);
+void do_toggle(Widget w, XtPointer cldata, XtPointer cadata);
+void do_read(Widget w, XtPointer cldata, XtPointer cadata);
+void do_save(Widget w, XtPointer cldata, XtPointer cadata);
+void do_delete(Widget w, XtPointer cldata, XtPointer cadata);
+void do_quit(Widget w, XtPointer cldata, XtPointer cadata);
+void do_open_family(Widget w, XtPointer cldata, XtPointer cadata);
+void do_prep_delete(Widget w, XtPointer cldata, XtPointer cadata);
+void do_prep_update(Widget w, XtPointer cldata, XtPointer cadata);
+void do_prep_export(Widget w, XtPointer cldata, XtPointer cadata);
+void do_close_family(Widget w, XtPointer cldata, XtPointer cadata);
+void do_increase_magn(Widget w, XtPointer cldata, XtPointer cadata);
+void do_decrease_magn(Widget w, XtPointer cldata, XtPointer cadata);
 
-void show_image_families PARAMS ((int first_time));
-void hide_image_families PARAMS ((void));
+void show_image_families(int first_time);
+void hide_image_families(void);
 
-void usage PARAMS ((void));
-void build_name PARAMS ((char *name, char *first, char *second));
-void display_family PARAMS ((int i, int imagn));
-void undisplay_family PARAMS ((int i));
-ImageStuff *init_ims PARAMS ((Image *img, Widget parent));
-ImageStuff *find_ims PARAMS ((FamilyStuff *fms, Widget w));
-int shell_index PARAMS ((Widget w));
-Pixmap magnify_bitmap PARAMS ((char *data, int h, int w, int s));
-Pixmap magnify_colrpix PARAMS ((Image *img, int s, Pixel bg_pix, Pixmap mask));
-void reset_prep PARAMS ((int i));
-void set_cursor PARAMS ((FamilyStuff *fms, Cursor cursor));
-int delete_image PARAMS ((Widget w));
-int update_or_export_image PARAMS ((Widget w, int flag));
-void destroy_family PARAMS ((int i));
-void delete_family PARAMS ((int i));
-int empty_family PARAMS ((int i)); 
-void mark_changed PARAMS ((ImageFamily *imf, int dummy));
+void usage(void);
+void build_name(char *name, char *first, char *second);
+void display_family(int i, int imagn);
+void undisplay_family(int i);
+ImageStuff *init_ims(Image *img, Widget parent);
+ImageStuff *find_ims(FamilyStuff *fms, Widget w);
+int shell_index(Widget w);
+Pixmap magnify_bitmap(char *data, int h, int w, int s);
+Pixmap magnify_colrpix(Image *img, int s, Pixel bg_pix, Pixmap mask);
+void reset_prep(int i);
+void set_cursor(FamilyStuff *fms, Cursor cursor);
+int delete_image(Widget w);
+int update_or_export_image(Widget w, int flag);
+void destroy_family(int i);
+void delete_family(int i);
+int empty_family(int i); 
+void mark_changed(ImageFamily *imf, int dummy);
 
 #ifdef HAVE_SELFILE
-extern FILE *XsraSelFile();
+extern FILE *XsraSelFile(Widget toplevel, char *prompt, char *ok,
+			 char *cancel, char *failed, char *init_path,
+			 char *mode, int (*show_entry)(void),
+			 char **name_return);
 #endif
 
 int
-main(argc, argv)
-int argc;
-char *argv[];
+main(int argc, char *argv[])
 {
     char *arg, *mask_color_name = NULL;
     char *stype;
